Package: HelpersMG
Type: Package
Title: Tools for Various R Functions Helpers
Version: 2026.8.24
Date: 2026-08-24
Authors@R: person(given = "Marc",
                    family = "Girondot",
                    role = c("aut", "cre"),
                    email = "marc.girondot@gmail.com", 
                    comment = c(ORCID = "0000-0001-6645-8530"))
Depends: R (>= 4.1), MASS, rlang, coda, Matrix, ggplot2
Suggests: lme4, RNetCDF, ncdf4, maps, fields, shiny, ppcor, pbmcapply,
        pbapply, parallel, visNetwork, igraph, shinyWidgets, cranlogs
Description: Contains miscellaneous functions useful for managing 'NetCDF' files (see <https://en.wikipedia.org/wiki/NetCDF>), get moon phase and time for sun rise and fall, tide level, analyse and reconstruct periodic time series of temperature with irregular sinusoidal pattern, show scales and wind rose in plot with change of color of text, Metropolis-Hastings algorithm for Bayesian MCMC analysis, plot graphs or boxplot with error bars, search files in disk by there names or their content, read the contents of all files from a folder at one time.
License: GPL-2
LazyLoad: yes
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-08-24 12:32:08 UTC; marcgirondot
Config/roxygen2/version: 8.1.0
Author: Marc Girondot [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6645-8530>)
Maintainer: Marc Girondot <marc.girondot@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-24 13:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method("+",PriorsmcmcComposite)
S3method(as.mcmc,mcmcComposite)
S3method(logLik,LD50)
S3method(logLik,compareAIC)
S3method(logLik,cutter)
S3method(merge,mcmcComposite)
S3method(plot,IconoCorel)
S3method(plot,LD50)
S3method(plot,PriorsmcmcComposite)
S3method(plot,cutter)
S3method(plot,mcmcComposite)
S3method(predict,LD50)
S3method(print,cutter)
S3method(summary,mcmcComposite)
export(ChangeCoordinate)
export(DIx)
export(ELPDweight)
export(ExtractAIC.glm)
export(FormatCompareAIC)
export(IC_clean_data)
export(IC_correlation_simplify)
export(IC_threshold_matrix)
export(LD50)
export(LD50_MHmcmc)
export(LD50_MHmcmc_p)
export(MHalgoGen)
export(MovingWindow)
export(NagelkerkeScaledR2)
export(RM_add)
export(RM_delete)
export(RM_duplicate)
export(RM_get)
export(RM_list)
export(RandomFromHessianOrMCMC)
export(RectangleRegression)
export(SEfromHessian)
export(ScalePreviousPlot)
export(addS3Class)
export(as.parameters)
export(as.quantiles)
export(asc)
export(barplot_errbar)
export(cArrows)
export(char)
export(compare)
export(compare_AIC)
export(compare_AICc)
export(compare_BIC)
export(contingencyTable.compare)
export(convert.tz)
export(cutter)
export(d)
export(dSnbinom)
export(dbeta_new)
export(dcutter)
export(dggamma)
export(dnbinom_new)
export(duplicated_packages)
export(ellipse)
export(fitdistrquantiles)
export(flexit)
export(format_ncdf)
export(from_min_max)
export(iCutter)
export(ind_long_lat)
export(index.periodic)
export(inside)
export(invlogit)
export(list.packages)
export(local.search)
export(logit)
export(minmax.periodic)
export(modeled.hist)
export(modifyVector)
export(moon.info)
export(newcompassRose)
export(newmap.scale)
export(openwd)
export(pSnbinom)
export(pggamma)
export(plot_add)
export(plot_errbar)
export(qSnbinom)
export(qggamma)
export(qvlmer)
export(r2norm)
export(rSnbinom)
export(rcutter)
export(read_folder)
export(rggamma)
export(rmnorm)
export(rnbinom_new)
export(series.compare)
export(setPriors)
export(setPriors1)
export(show_name)
export(similar)
export(specify_decimal)
export(sun.info)
export(symbol.Female)
export(symbol.Male)
export(symmetricize)
export(tide.info)
export(tnirp)
export(universalmclapply)
export(wget)
import(coda)
importFrom(rlang,.data)
importFrom("grDevices", "rgb",  "colorRampPalette")
importFrom("graphics", "arrows", "axis", "barplot", "hist", "legend",
             "lines", "mtext", "par", "plot", "polygon", "segments", "points", 
             "xspline", "xyinch", "strheight", "text", "locator")
importFrom("stats", "AIC", "aggregate", "as.formula", "binomial", "density", 
             "dgamma", "pgamma", "qgamma", "rgamma", "optimise", 
             "dlnorm", "qlnorm", "plnorm", "rlnorm", 
             "dnorm", "pnorm", "qnorm", "rnorm", 
             "pbinom", "qbinom", "qchisq", "uniroot", 
             "coef", "cor", "dbeta", "dbinom", "deriv", "df.residual", 
             "dmultinom", "dnbinom", "glm", "lm", "logLik", 
             "na.omit", "numericDeriv", "optim", "pchisq", "qf", 
             "qt", "quantile", "residuals", "rnbinom", 
             "runif", "sd", "vcov", "cov", "cov2cor", "pt", "var", 
             "model.matrix", "terms", "median", "qbeta", 
             "dweibull", "pweibull", "qweibull", "rweibull", 
             "p.adjust", "rexp")
importFrom("utils", "combn", "download.file", "getFromNamespace",
             "installed.packages", "modifyList", "read.delim",
             "setTxtProgressBar", "tail", "txtProgressBar", 
             "compareVersion", "timestamp", "capture.output")
importFrom("Matrix", "nearPD")
importFrom("MASS", "ginv")
importFrom("ggplot2", "ggplot", "geom_line", "aes")
importFrom("rlang", ".data")

# library(shiny); runApp("/Users/marcgirondot/Documents/Espace_de_travail_R/_shiny/BoneProfileR")
# library(shiny); runApp("http://134.158.74.46/BoneProfileR/")


library(shiny)
if (!requireNamespace("shinyWidgets", quietly = TRUE)) {
  progressbar <- FALSE
} else {
  progressbar <- TRUE
  library("shinyWidgets")
}

package.HelpersMG <- require('HelpersMG')
version <- "2026.3.31 build 1773"

splitLayout <- function (..., cellWidths = NULL, cellArgs = list()) 
{
  children <- list2(...)
  childIdx <- !nzchar(names(children) %||% character(length(children)))
  attribs <- children[!childIdx]
  children <- children[childIdx]
  count <- length(children)
  if (length(cellWidths) == 0 || any(is.na(cellWidths))) {
    cellWidths <- sprintf("%.3f%%", 100/count)
  }
  cellWidths <- rep(cellWidths, length.out = count)
  cellWidths <- sapply(cellWidths, validateCssUnit)
  do.call(tags$div, c(list(class = "shiny-split-layout"), attribs, 
                      mapply(children, cellWidths, FUN = function(x, w) {
                        do.call(tags$div, c(list(style = sprintf("width: %s;", 
                                                                 w)), cellArgs, list(x)))
                      }, SIMPLIFY = FALSE)))
}


# Define UI for application that draws a histogram
fluidPage(
  titlePanel(h1("Cutte",
                img(src="Rlogo.png", height=40, width=40), align = "center"), 
             windowTitle = "CutteR"), 
  p(HTML("<b><a href=\"https://hebergement.universite-paris-saclay.fr/marcgirondot/\">Marc Girondot</a></b> - Laboratoire Ecologie, Systématique, Evolution"), align = "center"),
  p(HTML("Université Paris-Saclay, CNRS, AgroParisTech, France."), align = "center"), 
  
  wellPanel(
    
    p(HTML("<strong>CutteR is a scientific method and a software used 
  to model left and right censored and truncated data.</strong>"), align = "left"),
    p("It used a Bayesian MCMC distribution fit using conditional gamma, lognormal, normal, Weibull, or generalized gamma distribution. The priors are based on Gaussian distribution of parameters obtained from maximum likelihood with large standard deviation. The posterior distribution is obtained with 5000 MCMC iterations."), 
    p(), 
    p(HTML("<strong>Copy data in the box below, select the required options and 
         click 'Run the analysis' button.</strong>"), align = "left")
  ), 
  
  # Show a plot of the generated distribution
  wellPanel(
    textAreaInput(inputId="Data", label="Copy the data here, separated by space, semicolon, tabulation, or return; unknown values must be LDL (Lower the Detection Limit) or UDL (Upper the Detection Limit). Do not use comma to separate values as commas are used as synonyme for decimal point.", 
                  value="",
                  rows = 10, 
                  width="100%", 
                  placeholder=""), 
    p("To try the method, copy these values and set a Lower limit of detection to 10:"), 
    p("89.77 12.75 44.18 60.66 32.68 34.57 28.09 91.11 13.98 56.95 59.64 76.22 84.64 61.45 27.50 17.27 16.39 22.10 15.81 62.86 86.39 50.86 26.33 29.43 53.95 40.94 64.13 19.49 40.26 44.03 66.70 22.10 21.07 36.57 70.29 50.20 54.13 13.11 22.76 40.04 28.49 LDL 13.98 LDL 62.24 55.50 15.30 11.29 14.39 40.88 26.31 33.04 19.02 47.35 56.16 24.00 18.66 20.52 61.26 LDL 15.45 41.65 77.76 71.66 22.13 LDL 46.61 39.65 58.96 LDL 28.01 83.33 64.28 24.31 22.52 LDL 64.47 62.77 11.45 32.91 39.11 71.29 56.73 67.44 32.42 45.63 32.60 60.96 16.38 LDL 12.26 143.26 18.80 13.28 29.26 50.78 15.98 45.35 46.43 89.13")
  ), 
  
  splitLayout(
    textInput(inputId="LDL", label="Lower limit of detection", value="")
    , textInput(inputId="UDL", label="Upper limit of detection", value="")
    , cellWidths=c("50%", "50%")
  ), 
  p("Let a box being empty if the option is not necessary."),
  splitLayout(
    selectInput("censored", label="Choose which cut will be fitted "
                , choices=list("Censored"="censored", 
                               "Truncated"= "truncated")
                , selected = "censored", multiple = FALSE,
                selectize = FALSE, size = NULL),
    selectInput("mixture", label="How many mixture should be used "
                , choices=list("1" = "1", 
                               "2" = "2", 
                               "3" = "3")
                , selected = "1", multiple = FALSE,
                selectize = FALSE, size = NULL),
    selectInput("distribution", label="Choose which distribution will be fitted "
                , choices=list("Auto" ="auto", 
                               "Gamma"="gamma", 
                               "Lognormal"= "lognormal", 
                               "Normal"= "normal", 
                               "Weibull" = "weibull", 
                               "Generalized gamma" = "generalized.gamma")
                , selected = "auto", multiple = FALSE,
                selectize = FALSE, size = NULL),
    cellWidths=c("33%", "33%", "34%")
  ), 
  
  actionButton(inputId="goButton", label="Run the analysis", width="30%", 
               icon("paper-plane"), 
               style="color: #fff; background-color: #337ab7; border-color: #2e6da4"),
  HTML("<i>Be patient, the analysis requires between 30 seconds to 5 minutes to be completed.</i>"), 
  p(""), 
  if (progressbar) {progressBar(
    id = "pb2",
    value = 0,
    total = 10000,
    title = "",
    display_pct = TRUE
  )},
  wellPanel(
    
    textOutput(outputId="Result"), 
    tags$style(type="text/css", "#Result {white-space: pre-wrap;}"), 
    plotOutput("Plot")
  ), 
  wellPanel(
    p(paste0("This web server version v. ",version, " is a simplified version of the complete tools available in the HelpersMG R package.")), 
    HTML('<img src="http://perso0.free.fr/cgi-bin/wwwcount.cgi?df=Cutter.dat&dd=C&ft=0">'), 
    p(HTML("<small><i><font color='#006699'>The Virtual Data initiative, run by LABEX P2IO and supported by Université Paris-Saclay, is thanked for providing computing resources on its cloud infrastructure.</font></i></small>"))
  )
  
)

# library(shiny); library("embryogrowth"); runApp("/Users/marcgirondot/Documents/Espace_de_travail_R/_shiny/compare")
# library(shiny); runApp("http://134.158.74.46/compare/")


library(shiny)
package.HelpersMG <- require('HelpersMG')


# Define UI for application that draws a histogram
fluidPage(
  
  # Application title
  
  wellPanel(
    wellPanel(
      HTML("<small><i><font color='#006699'>The Virtual Data initiative, run by LABEX P2IO and supported by Université Paris-Sud, is thanked for providing computing resources on its cloud infrastructure.</font></i></small>")
    )
    , 
    fluidRow(
    h1(HTML("The <em>w</em>-value: An alternative to <em>t</em>- and <em>&chi;<sup>2</sup></em>-tests"), align = "center")
    ), 
    fluidRow(h2(HTML("that does not use <em>p</em>-value"), align = "center")
    ), 
    fluidRow(
      p("This web server version v. 1.05 is a simplified version of the complete tools available ", 
        a("in the HelpersMG R package."
          , href="https://cran.r-project.org/package=HelpersMG"
          , target="_blank"))
    ), 
    fluidRow(p(HTML("The <em>w</em>-value methodology has been developped by 
             <a href=\"https://hebergement.universite-paris-saclay.fr/marcgirondot/\">Marc Girondot</a> and 
             Jean-Michel Guillon."))
    ), 
    fluidRow(
      p("Ecologie, Systématique, Evolution - CNRS, AgroParisTech, Université Paris Saclay, France.")
    ), 
    fluidRow(p("It has been published in: ")
    ), 
    fluidRow(
      p(HTML("<a href=\"https://www.researchgate.net/publication/323847382_The_w-value_An_Alternative_to_t-and_ch2_Tests\">Girondot, M., Guillon, J.-M., 2018. The <em>w</em>-value: An alternative to <em>t</em>- and &chi;<sup>2</sup> tests. Journal of Biostatistics & Biometrics 1(1): 1-4.</a>"))
    ), 
    fluidRow(
      p("")
    )
  ),
    
    # Show a plot of the generated distribution
    mainPanel(
      
      radioButtons("type", "What test should be done ?", 
                   list("Data series"=1, "Homogeneity contingency table"=2, "Conformity contingency table"=3), selected=1, inline = TRUE),
      # selectInput("type", "type"
      #            , choices=as.list(c("Contingency table", "Data"))
      #            , selected = "Data", multiple = FALSE,
      #            selectize = TRUE, width = NULL, size = NULL),
      # selectInput("var.equal", "var.equal"
      #             , choices=as.list(c(FALSE, TRUE))
      #             , selected = TRUE, multiple = FALSE,
      #             selectize = TRUE, width = NULL, size = NULL),
      uiOutput("varData"), 
      # radioButtons("var.equal", "Do the variances of the series should be supposed equal? (for data series test)", 
      #              list("Yes"=1, "No"=2), selected=1, inline = TRUE),
      # selectInput("Criterion", "Criterion"
      #             , choices=as.list(c("AIC", "AICc", "BIC"))
      #             , selected = "AIC", multiple = TRUE,
      #             selectize = TRUE, width = NULL, size = NULL),
      # textInput(inputId="data", label="Data to be analyzed (values separated by space and series by \";\")", 
      #           value="",
      #           width = "100%", 
      #           placeholder=c("12.4 13.3 17.2;12.9 19.20 32.9")),
      uiOutput("typeD"), 
      uiOutput("typeC"), 
      actionButton("goButton", "Compare"), 
      # submitButton("Compare"),
      h4("Results"),
      verbatimTextOutput(outputId="AW"), 
      p("The value \"w-value=BICw(identical)=\" represents the probability that, taking into the available information, a single model for all the series is sufficient 
        to modelize the data. The lower the value (closer to 0), the most likely the series are obtained from different distributions. 
        The higher the value (closer to 1) the most likely the series are obtained from a single distribution."),
      h4("Data table"), 
      tableOutput(outputId="DataOut")
    ), 
  HTML('<img src="http://perso0.free.fr/cgi-bin/wwwcount.cgi?df=Compare.dat&dd=C&ft=0">')
  )
  

library(shiny)

# require('HelpersMG')

# Define server logic required to draw a histogram
shinyServer(function(input, output) {
  
  # Expression that generates a histogram. The expression is
  # wrapped in a call to renderPlot to indicate that:
  #
  #  1) It is "reactive" and therefore should re-execute automatically
  #     when inputs change
  #  2) Its output type is a plot
  
  # output$resultsInfo <- renderPrint({print(input$MonthRef)})



  output$typeC <- renderUI({
    if (input$type == 3)
    textInput(inputId="probs", label="Probability for each category (values separated by space)", 
              value="",
              width = "100%", 
              placeholder=c("0.1 0.2 0.7"))
    
  })
  
  output$typeD <- renderUI({
    if (input$type == 1) {
  textInput(inputId="data", label="Data to be analyzed (values separated by space and series by \";\")", 
            value="",
            width = "100%", 
            placeholder=c("12.4 13.3 17.2;12.9 19.20 32.9"))
    } else {
      textInput(inputId="data", label="Data to be analyzed (values separated by space and series by \";\")", 
                value="",
                width = "100%", 
                placeholder=c("10 12 63;20 24 70"))
    }
  })
  
  output$varData <- renderUI({
    if (input$type == 1)
    radioButtons("var.equal", "Do the variances of the series should be supposed equal?",
                 list("Yes"=1, "No"=2), selected=1, inline = TRUE)

  })
  

  
  outtextfn <- eventReactive(input$goButton, {
      dataIn <- input$data
      if (input$type == 3) {
        probs <- input$probs
        if (probs == "") probs <- "0.1 0.2 0.7"
        probs <- gsub("  ", " ", probs)
        probs <- as.numeric(strsplit(probs, " ")[[1]])
        probs <- probs/sum(probs)
      }
      
      
      if (dataIn == "") if (input$type == 1) {
        dataIn <- "12.4 13.3 17.2;12.9 19.20 32.9"
      } else {
        dataIn <- "10 12 63;20 24 70"
      }
      
      
      dataIn <- gsub("  ", " ", dataIn)
      dataIn <- gsub("; ", ";", dataIn)
      dataIn <- gsub(" ;", ";", dataIn)
      
      l <- strsplit(dataIn, ";")[[1]]
      table.list <- list()
      elements <- NULL
      for (i in 1:length(l)) {
        k <- as.numeric(strsplit(l[i], " ")[[1]])
        table.list <- c(table.list, list(k))
        elements <- c(elements, length(k))
      }
      
      if (all(elements == elements[1])) {
        table <- t(as.data.frame(table.list))
      } else {
        lm <- max(elements)
        for (i in seq_along(elements)) {
          if (elements[i] != lm) {
            table.list[[i]] <- c(table.list[[i]], rep(NA, lm-elements[i]))
          }
        }
        table <- t(as.data.frame(table.list))
      }
      colnames(table) <- NULL
      rownames(table) <- paste("Series", 1:nrow(table))
      output$DataOut <- renderTable(table, rownames=TRUE, colnames = FALSE)
      
    if (input$type == 1) {
      outaw <- series.compare(table, 
                               var.equal=ifelse(input$var.equal==1, TRUE, FALSE), 
                               criterion = c("BIC"))
    } else {
      if (input$type == 2) {
        if (all(elements == elements[1])) {
        outaw <- contingencyTable.compare(table, criterion = c("BIC"))
        } else {
          outaw <- NULL
          outtext <- "The contingency table must have all series with the same number of columns."
        }
      } else {
        # je suis dans un test de conformité à probs
        if (all(elements == length(probs))) {
          outaw <- contingencyTable.compare(table, criterion = c("BIC"), 
                                            probs=probs)
        } else {
          outaw <- NULL
          outtext <- "The contingency table must have all series with the same number of columns than the probabilities to be tested."
        }
      }
    }
      if (!is.null(outaw)) {
        outtext <- "w-value="
        for (i in 1:length(outaw)) outtext <- paste0(outtext, names(outaw[i]), "=", outaw[i], "\n")
      }
      outtext

})

output$AW <- renderText({
  outtextfn()
})
  
})

library(shiny)


# Define server logic required to draw a histogram
shinyServer(function(input, output, session) {
  if (!requireNamespace("shinyWidgets", quietly = TRUE)) {
    progressbar <- FALSE
  } else {
    progressbar <- TRUE
    library("shinyWidgets")
  }
  
  outplotfn <- eventReactive(eventExpr=input$goButton, ignoreNULL = FALSE, valueExpr={
    
    data <- isolate(input$Data)
    
    if (data == "") {
      
      oldpar <- par(no.readonly = TRUE)    # code line i
      on.exit(par(oldpar))            # code line i + 1 
      
      
      par(mar=c(0, 0, 0, 0))
      plot(x=c(0, 1), y=c(0, 1), axes=FALSE,
           xaxt="n", yaxt="n", main="",
           xlab = "", ylab = "",
           xaxs="i", yaxs="i", type="n")
      text(x = 0.5, y=0.6, labels = "Load data to be analyzed",
           col="red", cex = 1.6)
    } else {
      
      LDL <- isolate(input$LDL)
      UDL <- isolate(input$UDL)
      method <- isolate(input$censored)
      distribution <- isolate(input$distribution)
      n.mixture <- as.numeric(isolate(input$mixture))
      
      if (LDL == "") LDL <- NA
      LDL <- as.numeric(LDL)
      if (UDL == "") UDL <- NA
      UDL <- as.numeric(UDL)
      
      # 89.7707600916124 12.7528137253536 44.1854453769165 60.669359673936 32.6843705727645 34.5730704570828 28.090749695105 91.1180106617791 13.9854115059374 56.9528385259751 59.6424331262563 76.2201823264057 84.6428469240878 61.4539947006212 27.5032991492022 17.2751640631116 16.3951178618038 22.1066596825853 15.8104734635694 62.860136780007 86.3920347016796 50.8633719498414 26.3350623257843 29.4309413858114 53.9577371460467 40.9435118742182 64.1395543447893 19.4952849783914 40.2695044921001 44.0330480738343 66.7024428576167 22.1085943874056 21.0741015828443 36.5769530586322 70.2957796454758 50.2016515754352 54.1383590002363 13.1126129957134 22.7606870305514 40.0486960995248 28.4901935386691 LDL 13.9830972917087 LDL 62.2462103726513 55.5096384295298 15.3024423712639 11.2900250294085 14.3952333513986 40.8815827926869 26.3100410588769 33.0489156162339 19.0232074637007 47.3519350475661 56.1662979035662 24.0095554995613 18.6680724684696 20.5267938077808 61.2603324259142 LDL 15.4503639027022 41.6557100140753 77.7671619325677 71.6647052427532 22.139625491507 LDL 46.6179606969968 39.6541156998326 58.9601278755938 LDL 28.0180573785966 83.3386058631985 64.2807584194209 24.31864943644 22.5232960339106 LDL 64.4776116851174 62.776603594233 11.4504640878683 32.9139695459943 39.1124720316336 71.295614787684 56.7337043921124 67.4445163896557 32.4214974228418 45.6384034482342 32.6078013843732 60.9605529048265 16.3864347564686 LDL 12.2659233909316 143.260422377747 18.8002300449792 13.2826042818602 29.2635178927549 50.7831016585992 15.9864114168415 45.3509802007882 46.4315969264461 89.133165449313
      
      
      # UDL <- NULL
      # LDL <- 10
      # 
      # DL <- 10
      # # Generate 100 random data from a gamma distribution
      # data_x <- rgamma(100, scale=20, shape=2)
      # # remove the data below the detection limit
      # data_x[data_x < DL] <- -Inf
      
      # data <- "10 20 30 40 LDL LDL 5"
      data <- gsub("\\n", " ", data)
      data <- gsub("\\r", " ", data)
      data <- gsub("\\t", " ", data)
      data <- gsub(";", " ", data)
      data <- gsub(",", ".", data)
      data <- gsub(" +", " ", data)
      data <- gsub(" +$", "", data)
      data <- gsub("^ +", "", data)
      
      data_x <- strsplit(data, " ")[[1]]
      data_x <- ifelse(data_x == "UDL", "Inf", data_x)
      data_x <- ifelse(data_x == "LDL", "-Inf", data_x)
      data_x <- as.numeric(data_x)
      
      if (distribution == "auto") {
        
        result_normal <- cutter(observations = data_x,
                                lower_detection_limit = LDL,
                                upper_detection_limit = UDL,
                                cut_method = method,
                                distribution = "normal", 
                                n.iter=NULL, n.mixture=n.mixture, 
                                progress.bar=FALSE)
        
        result_lognormal <- cutter(observations = data_x,
                                   lower_detection_limit = LDL,
                                   upper_detection_limit = UDL,
                                   cut_method = method,
                                   distribution = "lognormal", 
                                   n.iter=NULL, n.mixture=n.mixture, 
                                   progress.bar=FALSE)
        
        result_gamma <- cutter(observations = data_x,
                               lower_detection_limit = LDL,
                               upper_detection_limit = UDL,
                               cut_method = method,
                               distribution = "gamma", 
                               n.iter=NULL, n.mixture=n.mixture, 
                               progress.bar=FALSE)
        
        result_Weibull <- cutter(observations = data_x,
                                 lower_detection_limit = LDL,
                                 upper_detection_limit = UDL,
                                 cut_method = method,
                                 distribution = "weibull", 
                                 n.iter=NULL, n.mixture=n.mixture, 
                                 progress.bar=FALSE)
        
        result_generalized.gamma <- cutter(observations = data_x,
                                           lower_detection_limit = LDL,
                                           upper_detection_limit = UDL,
                                           cut_method = method,
                                           distribution = "generalized.gamma", 
                                           n.iter=NULL, n.mixture=n.mixture, 
                                           progress.bar=FALSE)
        
        C <- compare_AICc(normal=result_normal, 
                          lognormal=result_lognormal, 
                          gamma=result_gamma, 
                          weibull=result_Weibull, 
                          generalized.gamma=result_generalized.gamma, 
                          silent = TRUE)
        
        distribution <- rownames(C)[which.min(as.numeric(C[, "AICc"]))]
        selected <- paste0("The selected distribution model is ", distribution, " with an Akaike weight based on AICc of ", 
                           specify_decimal(C[distribution, "Akaike_weight"]), ".\n")
      } else {
        selected <- ""
      }
      
      if (progressbar) {
        result <- cutter(observations = data_x,
                         lower_detection_limit = LDL,
                         upper_detection_limit = UDL,
                         cut_method = method,
                         distribution = distribution, 
                         n.mixture=n.mixture, session=session)
      } else {
        result <- cutter(observations = data_x,
                         lower_detection_limit = LDL,
                         upper_detection_limit = UDL,
                         cut_method = method,
                         distribution = distribution, 
                         n.mixture=n.mixture, session=NULL)
      }
      
      textout <- print(result, silent=TRUE)
      
      textout <- paste0(selected, textout)
      
      output$Result <- renderText(textout)
      
      
      plot(result)
      
      # plot(1, 1)
    }
  }
  )
  
  
  
  
  output$Plot <- renderPlot({
    
    outplotfn()
    
  })
  
  
  
})

#' plot.PriorsmcmcComposite plot a prior
#' @title Plot a prior defined with setPriors function
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A ggplot object
#' @param x The priors to show
#' @param parameter The name or rank of prior to show
#' @param ... Not used
#' @description Create a ggplot graph with prior.\cr
#' The function makes minimal effort to decorate the plot.
#' @family mcmcComposite functions
#' @importFrom rlang .data
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' par <- c(a0=10, a1=2, b2=20, b1=-1)
#' rules <- rbind(data.frame(Name="^a", Min=0, Max="x*2"), 
#'               data.frame(Name="^b", Min=0, Max=100))
#' p <- setPriors(par=par, se=NULL, density="dgamma", rules=rules)
#' plot(p, parameter="a0")
#' q <- plot(p, parameter="b1")
#' q + geom_line(color = "red") + theme_bw() + 
#' theme(plot.margin=unit(c(2,1,1,1), 'cm'), 
#'       panel.border = element_blank(), 
#'       axis.line.x.bottom = element_line(colour = "black"), 
#'       axis.line.y.left = element_line(colour = "black")) + 
#' labs(title="Parameter: b1") + theme(plot.title = element_text(hjust = 0.5))
#' }
#' @method plot PriorsmcmcComposite
#' @export


plot.PriorsmcmcComposite <- function(x                                 , 
                                     parameter=1                      , 
                                     ...                                ) {
  
    x_axis <- seq(from=x[parameter, "Min"]-(x[parameter, "Max"]-x[parameter, "Min"])*0.01, 
                  to=x[parameter, "Max"]+(x[parameter, "Max"]-x[parameter, "Min"])*0.01, 
                  length.out=1000)
    y_axis <- do.call(what=x[parameter, "Density"], 
                 args=list(x=x_axis, x[parameter, "Prior1"], x[parameter, "Prior2"], log=FALSE))
    df <- data.frame(x=x_axis, Density=y_axis)
    pgg <- ggplot(data = df, 
                  aes(x = .data[["x"]], 
                      y = .data[["Density"]])) + geom_line()
  
  return(pgg)
}

#' from_min_max returns standard deviation and/or mean from minimum and maximum
#' @title Distribution from minimum and maximum
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return from_min_max returns a list with output, ML, Bayesian results
#' @param observed.Minimum The observed minimum
#' @param observed.Maximum The observed maximum
#' @param observed.Mean The observed mean (can be omitted)
#' @param observed.Median The observed median (can be omitted)
#' @param observed.Quantiles Observed quantiles with names being the values of quantiles with Q being first letter (can be omitted; see description)
#' @param priors The priors (see MHAlgogen()) or character "dnorm" or "dunif"
#' @param n Number of observations
#' @param fitted.parameters The initial value to fit
#' @param fixed.parameters The fixed parameters
#' @param D The distribution to fit as character. ex "norm" or "lnorm"
#' @param n.iter Number of iterations for each chain
#' @param n.chains Number of chains
#' @param n.adapt Number of iteration to stabilize likelihood
#' @param thin Interval for thinning likelihoods
#' @param adaptive Should an adaptive process for SDProp be used
#' @param trace Or FALSE or period to show progress
#' @param replicates Number of replicates to model D
#' @param silent If TRUE, do not print information
#' @description Bayesian estimate of distribution when minimum, maximum, median or mean are known.\cr
#' If D="norm**" or "lnorm**", it will use the approximation of Gumbel based on D.\cr
#' If D="norm*" or "lnorm*", it will generate D distribution using replicates number of random numbers, 
#' and estimate Gumbel parameters from the simulated D distribution. \cr
#' Otherwise it will estimate parameters of Gumbel distribution based on maximum likelihood.\cr
#' For D="pois", "beta" or "chisq" only second (D="pois*", "beta*" or "chisq*") and third solutions are available.\cr
#' The observed.Quantiles parameter must be a named value, for example observed.Quantiles = c(Q0.025=17, Q0.975=26)\cr
#' @examples
#' \dontrun{
#' minobs <- 5
#' maxobs <- 25
#' # These two values are not mandatory
#' meanobs <- 15
#' medianobs <- 16
#' n <- 10
#' # To estimate only the sd of the distribution; mean is fixed
#' # Note that there is an obligation to have mean even if it
#' # is in fixed.parameters
#' # By defaut a normal distribution is fitted
#' 
#' out_sd <- from_min_max(n=n                                , 
#'                        observed.Minimum=minobs                    , 
#'                        observed.Maximum=maxobs                    , 
#'                        fitted.parameters=c(sd=5)                  , 
#'                        fixed.parameters=c(mean=meanobs)           ,
#'                        n.iter=10000                               ,
#'                        trace=TRUE                                 )
#'                             
#' plot(out_sd, what="MarkovChain", parameters="sd")
#' plot(out_sd, what="posterior", parameters="sd")
#' as.parameters(out_sd, index="quantile")
#' 
#' # To estimate both the sd and mean of the distribution
#' 
#' out_sd_mean_norm <- from_min_max(n=n, 
#'                             observed.Minimum=minobs                    , 
#'                             observed.Maximum=maxobs                    , 
#'                             fitted.parameters=c(mean=15, sd=5)         , 
#'                             fixed.parameters=NULL                      ,
#'                             n.iter=10000                               ,
#'                             D="norm**"                                 ,
#'                             trace=FALSE                                )
#'                             
#' plot(out_sd_mean_norm, what="MarkovChain", parameters="sd", ylim=c(0, 10))
#' plot(out_sd_mean_norm, what="MarkovChain", parameters="mean", ylim=c(10, 20))
#' plot(out_sd_mean_norm, what="posterior", parameters="mean",  
#'      breaks=seq(from=0, to=100, by=1), xlim=c(10, 20))
#' as.parameters(out_sd_mean_norm, index="quantile")
#' 
#' # Let see what's happened for a lognormal distribution
#' 
#' out_sd_mean_lnorm <- from_min_max(n=n, 
#'                             observed.Minimum=minobs                    , 
#'                             observed.Maximum=maxobs                    , 
#'                             fitted.parameters=c(meanlog=15, sdlog=5)   , 
#'                             fixed.parameters=NULL                      ,
#'                             n.iter=10000                               ,
#'                             D="lnorm**"                                ,
#'                             trace=FALSE                                )
#'                             
#' plot(out_sd_mean_lnorm, what="MarkovChain", parameters="sdlog", ylim=c(0, 10))
#' plot(out_sd_mean_lnorm, what="MarkovChain", parameters="meanlog", ylim=c(10, 20))
#' plot(out_sd_mean_lnorm, what="posterior", parameters="meanlog", xlim=c(0, 20), 
#'      breaks=seq(from=0, to=100, by=1))
#' as.parameters(out_sd_mean_lnorm, index="quantile")
#' 
#' # To be compared with the rule of thumb:
#' print(paste0("mean = ", as.character((maxobs + minobs) / 2))) # Mean Not so bad
#' print(paste0("sd = ", as.character((maxobs - minobs) / 4))) # SD Clearly biased
#' 
#' # Covariation of sd and mean is nearly NULL
#' cor(x=as.parameters(out_sd_mean_norm, index="all")[, "mean"], 
#'     y=as.parameters(out_sd_mean_norm, index="all")[, "sd"])^2
#' plot(x=as.parameters(out_sd_mean_norm, index="all")[, "mean"], 
#'      y=as.parameters(out_sd_mean_norm, index="all")[, "sd"], 
#'      xlab="mean", ylab="sd")
#'      
#' # Example when minimum, maximum and mean are known
#' 
#' out_sd_mean2 <- from_min_max(n=n                                   , 
#'                              observed.Minimum=minobs               , 
#'                              observed.Maximum=maxobs               , 
#'                              observed.Mean=meanobs                 ,
#'                              fitted.parameters=c(mean=15, sd=5)    , 
#'                              fixed.parameters=NULL                 ,
#'                              n.iter=10000                          ,
#'                              trace=FALSE                            )
#'                             
#' # Example when minimum, maximum, mean and median are known
#' 
#' out_sd_mean3 <- from_min_max(n=n                                   , 
#'                              observed.Minimum=minobs               , 
#'                              observed.Maximum=maxobs               , 
#'                              observed.Mean=meanobs                 ,
#'                              observed.Median=medianobs             ,
#'                              fitted.parameters=c(mean=15, sd=5)    , 
#'                              fixed.parameters=NULL                 ,
#'                              n.iter=10000                          ,
#'                              trace=FALSE                            )
#'                              
#' plot(out_sd_mean2, what="MarkovChain", parameters="sd")
#' plot(out_sd_mean2, what="MarkovChain", parameters="mean")
#' plot(out_sd_mean2, what="posterior", parameters="mean", xlim=c(0, 100), 
#'      breaks=seq(from=0, to=100, by=5))
#' as.parameters(out_sd_mean2, index="quantile")
#' 
#' # Example of GEV density function  
#' # Parametrisation from https://en.wikipedia.org/wiki/Generalized_extreme_value_distribution
#' dGEV <- getFromNamespace(".dGEV", ns="HelpersMG")
#' x <- seq(from=-4, to=4, by=0.1)
#' plot(x, y=dGEV(x=x, 
#'                location=0, scale=1, shape=-1/2, log=FALSE, sum=FALSE), 
#'      type="l", col="green", xlab="x", ylab="Density")
#' lines(x, y=dGEV(x=x, 
#'                 location=0, scale=1, shape=0, log=FALSE, sum=FALSE), col="red")
#' lines(x, y=dGEV(x=x, 
#'                 location=0, scale=1, shape=1/2, log=FALSE, sum=FALSE), col="blue")
#' legend("topleft", legend=c("shape=-1/2", "shape=0", "shape=1/2"), 
#'        lty=1, col=c("green", "red", "blue"))
#'        
#' # Note the different parametrisation about shape
#' dGEV <- getFromNamespace("dgevd", ns="EnvStats")
#' x <- seq(from=-4, to=4, by=0.1)
#' plot(x, y=dGEV(x=x, 
#'                location=0, scale=1, shape=-1/2), 
#'      type="l", col="green", xlab="x", ylab="Density")
#' lines(x, y=dGEV(x=x, 
#'                 location=0, scale=1, shape=0), col="red")
#' lines(x, y=dGEV(x=x, 
#'                 location=0, scale=1, shape=1/2), col="blue")
#' legend("topleft", legend=c("shape=-1/2", "shape=0", "shape=1/2"), 
#'        lty=1, col=c("green", "red", "blue"))
#' 
#' # Compute dn using the approximation from Wan et al. (2014)
#'    get_dn <- function(n) {
#'      if (n < 2) {
#'        stop("Sample size n must be at least 2.")
#'      }
#'      qnorm((n - 0.375) / (n + 0.25)) * 2
#'    }
#'    
#'    # Estimate standard deviation from min and max
#'    estimate_sd_from_range <- function(min_val, max_val, n) {
#'      dn <- get_dn(n)
#'      range <- max_val - min_val
#'      sd_estimate <- range / dn
#'      return(sd_estimate)
#'    }
#'    
#'    # Example usage:
#'    n <- 10
#'    min_val <- 5
#'    max_val <- 25
#'    
#'    dn_value <- get_dn(n)
#'    sd_estimate <- estimate_sd_from_range(min_val, max_val, n)
#'    
#'    cat("dn =", dn_value, "\n")
#'    cat("Estimated SD =", sd_estimate, "\n")
#'    
#'    # To generate data from publication
#'    
#'    library(parallel)
#'    library(embryogrowth)
#'    
#'    # Values for the prior of SD
#'    outSD <- subset(DatabaseTSD, subset = (((!is.na(IP.SD)) | 
#'                    (!is.na(IP.SE))) & (!is.na(Hatched))), 
#'                    select=c("Hatched", "IP.SE", "IP.SD", "IP.mean"))
#'    outSD$IP.SD <- ifelse(is.na(outSD$IP.SD), outSD$IP.SE*sqrt(outSD$Hatched), outSD$IP.SD)
#'    
#'    # Model estimation
#'    
#'    Example <- subset(DatabaseTSD, subset = (!is.na(IP.min)) & 
#'                        ((is.na(IP.SE)) & (is.na(IP.SD)) & (!is.na(Hatched)) & 
#'                           (is.na(IP.mean))), select=c("Species", "Incubation.temperature.set", 
#'                                                       "Hatched", "IP.min", "IP.max", 
#'                                                       "Reference"))
#'    
#'    out <- universalmclapply(X=1:nrow(Example), FUN=function(i) {
#'      n <- Example[i, "Hatched"]
#'      
#'      priors <- structure(list(Density = c("dunif", "dlnorm"), 
#'                               Prior1 = c(30, log(mean(outSD$IP.SD))), 
#'                               Prior2 = c(120, log(sd(outSD$IP.SD))), 
#'                               SDProp = c(1, 1), 
#'                               Min = c(30, 0.1), 
#'                               Max = c(120, 6), 
#'                               Init = c((Example[i, "IP.min"]+ Example[i, "IP.max"])/2, log(2))), 
#'                          row.names = c("mean", "sd"), 
#'                          class = c("PriorsmcmcComposite", "data.frame"))
#'      
#'      
#'      out_sd_mean_mcmc <- from_min_max(n=n, observed.Minimum=Example[i, "IP.min"], 
#'                                       observed.Maximum=Example[i, "IP.max"], 
#'                                       fitted.parameters=c(mean=(Example[i, "IP.min"]+ 
#'                                                              Example[i, "IP.max"])/2, 
#'                                                           sd=log(2)), 
#'                                       priors = priors, 
#'                                       D="norm**", 
#'                                       n.iter = 10000, n.adapt=15000, thin=30, 
#'                                       trace=100, adaptive = TRUE)
#'      
#'      # plot(out_sd_mean_mcmc, what = "MarkovChain", parameters = "sd")
#'      
#'      assign(paste0("out_sd_mean_mcmc_", as.character(i)), out_sd_mean_mcmc)
#'      save(list = paste0("out_sd_mean_mcmc_", as.character(i)), 
#'           file = file.path("dataOut", paste0("out_sd_mean_mcmc_", as.character(i), ".Rdata")))
#'      # rm(list=paste0("out_sd_mean_mcmc_", as.character(i)))
#'    }, progressbar = TRUE)
#'    
#'    # Generate table with all results
#'    
#'    Example <- subset(DatabaseTSD, subset = (!is.na(IP.min)) & 
#'    ((is.na(IP.SE)) & (is.na(IP.SD)) & (!is.na(Hatched)) & 
#'      (is.na(IP.mean))), select=c("Species", "Incubation.temperature.set", 
#'                                  "Hatched", "IP.min", "IP.max", 
#'                                  "Reference"))
#'    
#'    Example <- cbind(Example, dn=NA)
#'    Example <- cbind(Example, "SD(Hozo 2005)"=NA)
#'    Example <- cbind(Example, "SD(Wan 2014)"=NA)
#'    Example <- cbind(Example, "mean(Wan 2014)"=NA)
#'    Example <- cbind(Example, "median(SD)"=NA)
#'    Example <- cbind(Example, "2.5%(SD)"=NA)
#'    Example <- cbind(Example, "97.5%(SD)"=NA)
#'    Example <- cbind(Example, "25%(SD)"=NA)
#'    Example <- cbind(Example, "75%(SD)"=NA)
#'    Example <- cbind(Example, "median(mean)"=NA)
#'    Example <- cbind(Example, "2.5%(mean)"=NA)
#'    Example <- cbind(Example, "97.5%(mean)"=NA)
#'    Example <- cbind(Example, "25%(mean)"=NA)
#'    Example <- cbind(Example, "75%(mean)"=NA)
#'    Example <- cbind(Example, "z(mean)"=NA)
#'    Example <- cbind(Example, "z(SD)"=NA)
#'    
#'    
#'    library(coda)
#'    
#'    for (i in 1:nrow(Example)) {
#'      
#'      n <- Example[i, "Hatched"]
#'      if (n<= 15) {
#'        Example[i, "SD(Hozo 2005)"] <- (1/sqrt(12))*sqrt(((Example[i, "IP.max"] - 
#'         Example[i, "IP.min"]))^2+((Example[i, "IP.max"] - Example[i, "IP.min"]))^2/4)
#'      } else {
#'        if (n<=70) {
#'          Example[i, "SD(Hozo 2005)"] <- (Example[i, "IP.max"] - Example[i, "IP.min"])/4
#'        } else {
#'          Example[i, "SD(Hozo 2005)"] <- (Example[i, "IP.max"] - Example[i, "IP.min"])/6
#'        }
#'      }
#'      
#'      Example[i, "dn"] <- get_dn(n)
#'      Example[i, "SD(Wan 2014)"] <- estimate_sd_from_range(Example[i, "IP.min"], 
#'             Example[i, "IP.max"], n)
#'      Example[i, "mean(Wan 2014)"] <- (Example[i, "IP.min"]+ Example[i, "IP.max"])/2
#'      load(file = file.path("dataOut", paste0("out_sd_mean_mcmc_", as.character(i), ".Rdata")))
#'      out_sd_mean_mcmc <- get(paste0("out_sd_mean_mcmc_", as.character(i)))
#'      k <- as.parameters(out_sd_mean_mcmc, index="quantile", probs=c(0.025, 0.25, 0.5, 0.75, 0.975))
#'      outgk <- geweke.diag(as.mcmc(out_sd_mean_mcmc))
#'      rm(out_sd_mean_mcmc)
#'      rm(list=paste0("out_sd_mean_mcmc_", as.character(i)))
#'      Example[i, "median(SD)"] <- k["50%", "sd"]
#'      Example[i, "2.5%(SD)"] <- k["2.5%", "sd"]
#'      Example[i, "97.5%(SD)"] <- k["97.5%", "sd"]
#'      Example[i, "25%(SD)"] <- k["25%", "sd"]
#'      Example[i, "75%(SD)"] <- k["75%", "sd"]
#'      Example[i, "median(mean)"] <- k["50%", "mean"]
#'      Example[i, "2.5%(mean)"] <- k["2.5%", "mean"]
#'      Example[i, "97.5%(mean)"] <- k["97.5%", "mean"]
#'      Example[i, "25%(mean)"] <- k["25%", "mean"]
#'      Example[i, "75%(mean)"] <- k["75%", "mean"]
#'      Example[i, "z(mean)"] <- outgk$`1`$z["mean"]
#'      Example[i, "z(SD)"] <- outgk$`1`$z["sd"]
#'    }
#'    
#'    rownames(Example) <- as.character(1:nrow(Example))
#'   
#'   # Figure 1
#'   layout(mat = matrix(1:4, nrow=2))
#'   par(mar=c(4, 4, 0, 0))
#'   plot(out_sd_mean_mcmc_11, what = "MarkovChain", parameters = "mean", ylim=c(70, 76))
#'   text(x=ScalePreviousPlot(x=0.05, y=0.95)$x, y=ScalePreviousPlot(x=0.85, y=0.95)$y, 
#'      labels = "A: mean", cex=1.5, pos=4)
#'   plot(out_sd_mean_mcmc_8, what = "MarkovChain", parameters = "mean", ylim=c(50, 52))
#'   text(x=ScalePreviousPlot(x=0.05, y=0.95)$x, y=ScalePreviousPlot(x=0.85, y=0.95)$y, 
#'       labels = "C: mean", cex=1.5, pos=4)
#'   plot(out_sd_mean_mcmc_11, what = "MarkovChain", parameters = "sd")
#'   text(x=ScalePreviousPlot(x=0.05, y=0.95)$x, y=ScalePreviousPlot(x=0.85, y=0.95)$y, 
#'       labels = "B: sd", cex=1.5, pos=4)
#'   plot(out_sd_mean_mcmc_8, what = "MarkovChain", parameters = "sd")
#'   text(x=ScalePreviousPlot(x=0.05, y=0.95)$x, y=ScalePreviousPlot(x=0.85, y=0.95)$y, 
#'       labels = "D: sd", cex=1.5, pos=4)
#'   
#'   # Figure 2
#'   dtafigure2 <- matrix(NA, nrow=nrow(as.parameters(out_sd_mean_mcmc, index = "all")), 
#'                        ncol=nrow(Example))
#'   
#'   for (i in 1:nrow(Example)) {
#'     # i <- 1
#'     load(file=file.path("dataOut", paste0("out_sd_mean_mcmc_", as.character(i), ".Rdata")))
#'     out_sd_mean_mcmc <- get(paste0("out_sd_mean_mcmc_", as.character(i)))
#'     PPM <- rnorm(nrow(as.parameters(out_sd_mean_mcmc, index = "all")), 
#'        mean = as.parameters(out_sd_mean_mcmc, index = "all")[, "mean"], 
#'        sd=as.parameters(out_sd_mean_mcmc, index = "all")[, "sd"])
#'     dtafigure2[, i] <- PPM
#'   }
#'   
#'   layout(mat = 1)
#'   par(mar=c(3, 4, 0, 0))
#'   boxplot(dtafigure2, outline=FALSE, las=1, bty="n", xaxt="n", frame=FALSE, ylim=c(40, 90), 
#'     col=sapply(as.character(Example$Species), 
#'       FUN=function(i) switch(i, "Caretta caretta"="white", 
#'                                  "Chelonia mydas"="green", 
#'                                  "Dermochelys coriacea"="lightblue")), 
#'     ylab="Incubation duration in days")
#'   axis(1, at=1:30, labels = rep(NA, 30))
#'   Cc <- sum(as.character(Example$Species) == "Caretta caretta")
#'   CcCm <- sum(as.character(Example$Species) == "Caretta caretta" | 
#'               as.character(Example$Species) == "Chelonia mydas")
#'   segments(x0= Cc + 0.5, 
#'            x1= Cc + 0.5, 
#'            y0=40, y1=90, lty = 2)
#'   
#'   segments(x0= CcCm + 0.5, 
#'            x1= CcCm + 0.5, 
#'            y0=40, y1=90, lty = 2)
#'   
#'   par(xpd=TRUE)
#'   text(x=Cc/2, y=33, labels = expression(italic("Caretta caretta")), cex=0.9)
#'   
#'   text(x=Cc+(CcCm-Cc)/2, y=32, labels = expression(italic("Chelonia\n  mydas")), cex=0.9)
#'   text(x=CcCm+(30-CcCm)/2, y=32, labels = expression(italic("Dermochelys\n  coriacea")), cex=0.9)
#'   
#'   # With Lognormal
#'    # To generate data from publication
#'    
#'    library(parallel)
#'    library(embryogrowth)
#'    
#'    # Values for the prior of SD
#'    outSD <- subset(DatabaseTSD, subset = (((!is.na(IP.SD)) | 
#'                    (!is.na(IP.SE))) & (!is.na(Hatched))), 
#'                    select=c("Hatched", "IP.SE", "IP.SD", "IP.mean"))
#'    outSD$IP.SD <- ifelse(is.na(outSD$IP.SD), outSD$IP.SE*sqrt(outSD$Hatched), outSD$IP.SD)
#'    
#'    # Model estimation
#'    
#'    Example <- subset(DatabaseTSD, subset = (!is.na(IP.min)) & 
#'                        ((is.na(IP.SE)) & (is.na(IP.SD)) & (!is.na(Hatched)) & 
#'                           (is.na(IP.mean))), select=c("Species", "Incubation.temperature.set", 
#'                                                       "Hatched", "IP.min", "IP.max", 
#'                                                       "Reference"))
#'    
#'    out <- universalmclapply(X=1:nrow(Example), FUN=function(i) {
#'      n <- Example[i, "Hatched"]
#'      
#'      priors <- structure(list(Density = c("dunif", "dlnorm"), 
#'                               Prior1 = c(30, log(mean(outSD$IP.SD))), 
#'                               Prior2 = c(120, log(sd(outSD$IP.SD))), 
#'                               SDProp = c(1, 1), 
#'                               Min = c(30, 0.1), 
#'                               Max = c(120, 6), 
#'                               Init = c((Example[i, "IP.min"]+ Example[i, "IP.max"])/2, log(2))), 
#'                          row.names = c("meanlog", "sdlog"), 
#'                          class = c("PriorsmcmcComposite", "data.frame"))
#'      
#'      
#'      out_sd_mean_mcmc_LN <- from_min_max(n=n, observed.Minimum=Example[i, "IP.min"], 
#'                                       observed.Maximum=Example[i, "IP.max"], 
#'                                       fitted.parameters=c(mean=(Example[i, "IP.min"]+ 
#'                                                              Example[i, "IP.max"])/2, 
#'                                                           sd=log(2)), 
#'                                       priors = priors, 
#'                                       D="lnorm**", 
#'                                       n.iter = 10000, n.adapt=15000, thin=30, 
#'                                       trace=100, adaptive = TRUE)
#'      
#'      # plot(out_sd_mean_mcmc, what = "MarkovChain", parameters = "sd")
#'      
#'      assign(paste0("out_sd_mean_mcmc_LN_", as.character(i)), out_sd_mean_mcmc_LN)
#'      save(list = paste0("out_sd_mean_mcmc_LN_", as.character(i)), 
#'           file = file.path("dataOut", paste0("out_sd_mean_mcmc_LN_", as.character(i), ".Rdata")))
#'      # rm(list=paste0("out_sd_mean_mcmc_LN_", as.character(i)))
#'    }, progressbar = TRUE)
#'    
#' }
#' @export

from_min_max <- function(n = stop("n must be known.")                                             , 
                         fitted.parameters=stop("At least one parameter must be supplied.")       , 
                         observed.Minimum                                                         , 
                         observed.Maximum                                                         , 
                         observed.Median=NULL                                                     ,
                         observed.Mean=NULL                                                       , 
                         observed.Quantiles=NULL                                                  , 
                         priors="dnorm"                                                           , 
                         fixed.parameters=NULL                                                    ,
                         D="norm**"                                                               ,
                         n.iter=10000                                                             ,
                         n.chains = 1                                                             ,
                         n.adapt = 100                                                            ,
                         thin=30                                                                  ,
                         adaptive = FALSE                                                         , 
                         trace = 100                                                              , 
                         replicates = 10000                                                       , 
                         silent = FALSE                                                           ) {
  
  
  # x sont les paramètres
  L_min_max <- function(x=NULL                               , 
                        fixed.parameters=NULL                , 
                        n=NULL                               , 
                        D="norm"                             ,
                        meanobs=NULL                         , 
                        medianobs=NULL                       ,
                        minobs=NULL                          , 
                        maxobs=NULL                          , 
                        quantilesobs=NULL                    ,
                        rep=10000                            ) {
    
    # J'envoie les paramètres de D (* ou **)
    # ainsi que les valeurs observées meanobs, medianobs, minobs, maxobs, quantilesobs
    # Et je retourne la vraisemblance
    
    # x = c(mean=100, sd=10); fixed.parameters=NULL; n=10; D="norm**"; meanobs=100; medianobs=100; minobs=90; maxobs=110; quantilesobs=c(Q0.95=106); rep=10000      
    
    
    # par <- c(mean=50, sd=3)
    # n <- 4; minobs <- 43; maxobs <- 57
    # print(fixed.parameters)
    x <- c(x, fixed.parameters)
    # print(d(x))
    
    # if (is.null(meanobs)) {
    #   meanobs <- (minobs + maxobs) / 2
    # }
    
    D_ec <- D
    if (D_ec == "norm**") D <- "norm"
    if (D_ec == "norm*") D <- "norm"
    if (D_ec == "lnorm**") D <- "lnorm"
    if (D_ec == "lnorm*") D <- "lnorm"
    
    Lmin <- Lmax <- NULL
    kkk <- NULL
    
    # Si norm** et lnorm**, je calcule Lmin et Lmax sur l'approximation
    
    if (D_ec == "norm**") {
      # x <- c(mean=50, sd=2)
      # hist(rnorm(1000, mean=x["mean"], sd=x["sd"]))
      # n <- 10
      # minobs <- 42
      # maxobs <- 56
      # D <- "norm"; rep <- 10000
      
      mean <- - x["mean"]
      sd <- x["sd"]
      scale <- sd/sqrt(2*log(n))
      location <- mean + sd*sqrt(2*log(n)) - ((log(log(n))+log(4*pi))*sd/(2*sqrt(2*log(n))))
      Lmin <- - getFromNamespace(".dGEV", ns="HelpersMG")(x = - minobs, location= location, scale= scale, shape=0, log=TRUE, sum=TRUE)
      
      mean <- x["mean"]
      # sd <- x["sd"] # J'ai le même SD
      # scale <- sd/sqrt(2*log(n)) # J'ai le même scale
      location <- mean + sd*sqrt(2*log(n)) - ((log(log(n))+log(4*pi))*sd/(2*sqrt(2*log(n))))
      Lmax <- - getFromNamespace(".dGEV", ns="HelpersMG")(x = maxobs, location= location, scale= scale, shape=0, log=TRUE, sum=TRUE)
    }
    
    if (D_ec == "lnorm**") {
      # x <- c(meanlog=log(50), sdlog=log(2))
      # hist(log(rlnorm(1000, meanlog=x["meanlog"], sdlog=x["sdlog"])))
      # hist(rnorm(1000, mean=x["meanlog"], sd=x["sdlog"]))
      # n <- 10
      # minobs <- 47
      # maxobs <- 54
      # D_ec <- "lnorm**"; D <- "lnorm"; rep <- 100000
      
      mean <- x["meanlog"]
      sd <- x["sdlog"]
      location <- mean - sd * sqrt(2*log(n))
      scale <- sd/sqrt(2*log(n))
      
      Lmin <- - getFromNamespace(".dGEV", ns="HelpersMG")(x= minobs, location=location, scale=scale, shape=0, log=TRUE, sum=TRUE)
      
      mean <- x["meanlog"]
      sd <- x["sdlog"]
      location <- mean + sd * sqrt(2*log(n))
      scale <- sd/sqrt(2*log(n))
      
      Lmax <- - getFromNamespace(".dGEV", ns="HelpersMG")(x=maxobs, location=location, scale=scale, shape=0, log=TRUE, sum=TRUE)
    }
    
    if ((D_ec == "norm*") | (D_ec == "lnorm*") | (D_ec == "pois*") | (D_ec == "beta*") | (D_ec == "chisq*")) {
      
      rD <- get(paste0("r", D))
      dsit <- sapply(1:rep, FUN=function(i) {k <- do.call(rD, args = as.list(c(n=n, x))); return(c(max(-k), max(k)))}) # mean(k), median(k), 
      kk <- matrix(dsit, ncol=2, byrow = TRUE)
      
      kkk <- apply(kk, MARGIN = 2, FUN=function(x) return(c(sd(x), median(x)))) # mean(x), 
      # colnames(kkk) <- c("min", "max")
      # rownames(kkk) <- c("sd", "median")
      
      # Je dois aussi sortir les quantiles si nécessaire
      
      
      # v <- kkk[2, 1]^2
      s <- kkk[1, 1] # sd de min
      # m <- kkk[1, 1]
      md <- kkk[2, 1] # median de min
      # print(c(m, md, v))
      # kpi <- 0.607927101854027 # (6/pi^2)
      sqkpi <- 0.779696801233676 # sqrt((6/pi^2))
      kll2 <- -0.366512920581664 # log(log(2))
      
      # sigma^2*(pi^2/6) = v
      # sigma^2 = v*(6/pi^2)
      scale <- s * sqkpi # sqrt(v*kpi)
      
      # md = mu - sigma * log(log(2))
      location <- md + scale * kll2
      
      shape <- 0
      # dGEV <- getFromNamespace(".dGEV", ns="HelpersMG")
      # dGEV(x=kk[, 1], par=c(location=location, scale=scale, shape=0), sum=TRUE, log=TRUE)
      
      # print(c(location=location, scale=scale, shape=0))
      # outGEV_min <- optim(par=c(location=location, scale=scale, shape=0), 
      #                     x=kk[, 1], 
      #                     sum=TRUE, log=TRUE, silent=TRUE, 
      #                     fn = getFromNamespace(".dGEV", ns="HelpersMG"), 
      #                     # lower = c(location=-Inf, scale=1E-6, shape=-Inf), 
      #                     # upper = c(location=Inf, scale=Inf, shape=Inf), 
      #                     method = "Nelder-Mead")$par
      # outGEV_min[2] <- abs(outGEV_min[2])
      Lmin <- - getFromNamespace(".dGEV", ns="HelpersMG")(x=-minobs, location=location, scale=scale, shape=0, log=TRUE, sum=TRUE)
      # print(outGEV_min)
      # print(Lmin)
      # v <- kkk[2, 4]^2
      s <- kkk[1, 2] # sd de max
      # m <- kkk[1, 4]
      md <- kkk[2, 2] # median de max
      
      # sigma^2*(pi^2/6) = v
      # sigma^2 = v*(6/pi^2)
      scale <- s * sqkpi # sqrt(v*kpi)
      
      # md = mu - sigma * log(log(2))
      location <- md + scale * kll2
      
      # outGEV_max <- optim(par=c(location=location, scale=scale, shape=0), 
      #                     x=kk[, 3], 
      #                     sum=TRUE, log=TRUE, silent=TRUE, 
      #                     fn = getFromNamespace(".dGEV", ns="HelpersMG"), 
      #                     # lower = c(location=-Inf, scale=1E-6, shape=-Inf), 
      #                     # upper = c(location=Inf, scale=Inf, shape=Inf), 
      #                     method = "Nelder-Mead")$par
      # outGEV_max[2] <- abs(outGEV_max[2])
      Lmax <- - getFromNamespace(".dGEV", ns="HelpersMG")(x=maxobs, location=location, scale=scale, shape=0, log=TRUE, sum=TRUE)
      
    }
    
    # Ca signifie que je n'ai rien trouvé
    if (is.null(Lmin)) {
      
      rD <- get(paste0("r", D))
      dsit <- sapply(1:rep, FUN=function(i) {k <- do.call(rD, args = as.list(c(n=n, x))); return(c(max(-k), max(k)))}) # mean(k), median(k), 
      kk <- matrix(dsit, ncol=2, byrow = TRUE)
      
      kkk <- apply(kk, MARGIN = 2, FUN=function(x) return(c(sd(x), median(x)))) # mean(x), 
      # colnames(kkk) <- c("min", "max")
      # rownames(kkk) <- c("sd", "median")
      
      # v <- kkk[2, 1]^2
      s <- kkk[1, 1] # sd de min
      # m <- kkk[1, 1]
      md <- kkk[2, 1] # median de min
      # print(c(m, md, v))
      # kpi <- 0.607927101854027 # (6/pi^2)
      sqkpi <- 0.779696801233676 # sqrt((6/pi^2))
      kll2 <- -0.366512920581664 # log(log(2))
      
      # sigma^2*(pi^2/6) = v
      # sigma^2 = v*(6/pi^2)
      scale <- s * sqkpi # sqrt(v*kpi)
      
      # md = mu - sigma * log(log(2))
      location <- md + scale * kll2
      
      shape <- 0
      # dGEV <- getFromNamespace(".dGEV", ns="HelpersMG")
      # dGEV(x=kk[, 1], par=c(location=location, scale=scale, shape=0), sum=TRUE, log=TRUE)
      
      # print(c(location=location, scale=scale, shape=0))
      outGEV_min <- optim(par=c(location=location, scale=scale, shape=0),
                          x=kk[, 1],
                          sum=TRUE, log=TRUE, silent=TRUE,
                          fn = getFromNamespace(".dGEV", ns="HelpersMG"),
                          # lower = c(location=-Inf, scale=1E-6, shape=-Inf),
                          # upper = c(location=Inf, scale=Inf, shape=Inf),
                          method = "Nelder-Mead")$par
      outGEV_min[2] <- abs(outGEV_min[2])
      Lmin <- - getFromNamespace(".dGEV", ns="HelpersMG")(x=-minobs, par=outGEV_min, log=TRUE, sum=TRUE)
      # print(outGEV_min)
      # print(Lmin)
      # v <- kkk[2, 1]^2
      s <- kkk[1, 2] # sd de max
      # m <- kkk[1, 4]
      md <- kkk[2, 2] # median de max
      # print(c(m, md, v))
      # kpi <- 0.607927101854027 # (6/pi^2)
      # sqkpi <- 0.779696801233676 # sqrt((6/pi^2))
      # kll2 <- -0.366512920581664 # log(log(2))
      
      # sigma^2*(pi^2/6) = v
      # sigma^2 = v*(6/pi^2)
      scale <- s * sqkpi # sqrt(v*kpi)
      
      # md = mu - sigma * log(log(2))
      location <- md + scale * kll2
      
      outGEV_max <- optim(par=c(location=location, scale=scale, shape=0),
                          x=kk[, 3],
                          sum=TRUE, log=TRUE, silent=TRUE,
                          fn = getFromNamespace(".dGEV", ns="HelpersMG"),
                          # lower = c(location=-Inf, scale=1E-6, shape=-Inf),
                          # upper = c(location=Inf, scale=Inf, shape=Inf),
                          method = "Nelder-Mead")$par
      outGEV_max[2] <- abs(outGEV_max[2])
      Lmax <- - getFromNamespace(".dGEV", ns="HelpersMG")(x=maxobs, par=outGEV_max, log=TRUE, sum=TRUE)
      
    }
    
    Lmean <- NULL
    Lmedian <- NULL
    Lquantiles <- NULL
    
    if (D_ec == "norm**") {
      # Je suis avec norm** 
      
      mean <- x["mean"]
      sd <- x["sd"]
      
      if (!is.null(meanobs)) {
        Lmean <- dnorm(x=meanobs, mean = mean, sd = sd/sqrt(n), log = TRUE)
      } else {
        Lmean <- 0
      }
      
      # Si median et norm
      if (!is.null(medianobs)) {
        
        likelihood_median_exact <- function(M, n, mean, sd) {
          if (n %% 2 == 0) {
            # --- Approximate likelihood (normal approximation) ---
            sd_median <- sd * sqrt(pi / (2 * n))
            logL <- dnorm(M, mean, sd_median, log = TRUE)
          } else {
            # --- Exact likelihood for odd n ---
            k <- (n - 1) / 2
            logf <- dnorm(M, mean, sd, log=TRUE)
            F <- pnorm(M, mean, sd)
            logcoef <- log(factorial(n)) - 2*log(factorial(k))
            logL <- logcoef + k*log(F) + k*log(1 - F) + logf
          }
          return(logL)
        }
        
        Lmedian <- likelihood_median_exact(M=medianobs, n, mean, sd)
      } else {
        Lmedian <- 0
      }
      
      if (!is.null(quantilesobs)) {
        
        likelihood_quantile <- function(Q, n, p = 0.5, mean = 0, sd = 1) {
          # choose k using the "floor((n+1)p)" convention
          k <- floor((n + 1) * p)
          k <- ifelse(k>1, 1, k)
          k <- ifelse(k>n, n, k)
          
          # Normal pdf and cdf at M
          logfM <- dnorm(Q, mean = mean, sd = sd, log = TRUE)
          FM <- pnorm(Q, mean = mean, sd = sd)
          
          # Exact PDF of the k-th order statistic
          logcoef <- lchoose(n - 1, k - 1)
          logL_exact <- logcoef +  (k - 1)*log(FM) + (n - k)*log((1 - FM)) + logfM
          
          # Asymptotic approx (large n): variance = p(1-p) / (n * f(q_p)^2)
          if (FALSE) {
            zp <- qnorm(p)
            q_p <- mean + sd * zp
            f_qp <- dnorm(q_p, mean = mean, sd = sd)
            var_qp <- (p * (1 - p)) / (n * (f_qp^2))
            sd_qp <- sqrt(var_qp)
            L_approx <- dnorm(Q, mean = q_p, sd = sd_qp)
          }
          return(logL_exact)
        }
        
        Lquantiles <- 0
        vp <- as.numeric(substr(names(quantilesobs), 2, nchar(names(quantilesobs))))
        vQ <- unname(quantilesobs)
        Lquantiles <- Lquantiles + sum(likelihood_quantile(Q=vQ, n, p = vp, mean = mean, sd = sd))
        
        
      } else {
        Lquantiles <- 0
      }
      
    }
    
    if (D_ec == "lnorm**") {
      
      meanlog <- x["meanlog"]
      sdlog <- x["sdlog"]
      
      if (!is.null(meanobs)) {
        Lmean <- dnorm(x=meanobs, mean = exp(meanlog+sdlog^2/2), sd = sqrt((exp(sdlog^2)-1)*exp(2*meanlog+sdlog^2)) / sqrt(n), log = TRUE)
      } else {
        Lmean <- 0
      }
      
      if (!is.null(medianobs) | !is.null(quantilesobs)) {
        lognormal_quantile_likelihood <- function(Q, n, p=0.5, meanlog, sdlog) {
          # Determine the order statistic index for the p-th quantile
          r <- ceiling(p * n)   # or floor(p * n) depending on convention
          
          # PDF and CDF of lognormal
          f_Q <- dlnorm(Q, meanlog = meanlog, sdlog = sdlog)
          F_Q <- plnorm(Q, meanlog = meanlog, sdlog = sdlog)
          # Numerical safety: avoid log(0)
          F_Q <- pmin(pmax(F_Q, 1e-12), 1 - 1e-12)
          # Order statistic likelihood
          logL <- lchoose(n - 1, r - 1) +
            log(f_Q) +
            (r - 1) * log(F_Q) +
            (n - r) * log(1 - F_Q)
          
          return(logL)
        }
      }
      
      if (!is.null(medianobs)) {
        Lmedian <- lognormal_quantile_likelihood(Q=medianobs, n, p=0.5, meanlog, sdlog)
      } else {
        Lmedian <- 0
      }
      
      if (!is.null(quantilesobs)) {
        Lquantiles <- 0
        vp <- as.numeric(substr(names(quantilesobs), 2, nchar(names(quantilesobs))))
        vQ <- unname(quantilesobs)
        Lquantiles <- Lquantiles + lognormal_quantile_likelihood(Q=vQ, n, p = vp, meanlog, sdlog)

      } else {
        Lquantiles <- 0
      }
      
    }
    
    if (is.null(Lmean) | is.null(Lmedian)) {
      if (((!is.null(meanobs)) | (!is.null(medianobs))) & (is.null(kkk))) {
        rD <- get(paste0("r", D))
        dsit <- sapply(1:rep, FUN=function(i) {k <- do.call(rD, args = as.list(c(n=n, x))); return(c(max(-k), mean(k), median(k), max(k)))})
        kk <- matrix(dsit, ncol=4, byrow = TRUE)
        kkk <- apply(kk, MARGIN = 2, FUN=function(x) return(c(mean(x), sd(x), median(x))))
      }
      
      if (!is.null(meanobs)) {
        Lmean <- dnorm(x=meanobs, mean = kkk[1, 2], sd = kkk[2, 2]/ sqrt(n), log = TRUE)
      } else {
        Lmean <- 0
      }
      if (!is.null(medianobs)) {
        Lmedian <- dnorm(x=medianobs, mean = kkk[1, 3], sd = kkk[2, 3]/ sqrt(n), log = TRUE)
      } else {
        Lmedian <- 0
      }
    }
    
    L <- - ( Lmin + Lmax + Lmean + Lmedian + Lquantiles)
    
    if (is.infinite(L)) {
      stop("Problem during likelihood estimation")
      print(d(x))
      print(n)
      print(D_ec)
      print(minobs)
      print(maxobs)
    }
    # print(L)
    return(L)
  }
  
  # https://en.wikipedia.org/wiki/Generalized_extreme_value_distribution
  # n <- 4; observed.Minimum <- 43; observed.Maximum <- 57
  # fitted.parameters <- c(mean=50, sd=4); fixed.parameters <- NULL
  
  if (!silent) cat(paste0("The data are supposed to be generated from a ", gsub("\\*", "", D), " distribution.\n"))
  
  # if (D == "lnorm**") {
  #   warning("The lnorm** model is not available; it has been changed to lnorm*.")
  #   D <- "lnorm*"
  # }
  
  if (is.null(priors) | is.character(priors)) {
    if ((D == "norm") | (D == "norm*") | (D == "norm**")) {
      if (priors == "dunif") {
        if (length(fitted.parameters) == 2) {
          priors <- setPriors(par=fitted.parameters, 
                              se=c(sd=1, mean=1), 
                              density = "dunif", 
                              rules = rbind(data.frame(Name="sd", Min=1E-6, Max=10*unname(fitted.parameters["sd"])), 
                                            data.frame(Name="mean", Min=-10*unname(fitted.parameters["mean"]), 
                                                       Max=10*unname(fitted.parameters["mean"]))))
          priors$SDProp <- unname(fitted.parameters)/10
        } else {
          priors <- setPriors(par=fitted.parameters, 
                              se=c(sd=1), 
                              density = "dunif", 
                              rules = data.frame(Name="sd", Min=1E-6, Max=10*unname(fitted.parameters)))
          priors$SDProp <- unname(fitted.parameters)/10
        }
      } else {
        if (length(fitted.parameters) == 2) {
          priors <- setPriors(par=fitted.parameters, 
                              se=c(sd=1, mean=1), 
                              density = "dnorm", 
                              rules = rbind(data.frame(Name="sd", Min=1E-6, Max=10*unname(fitted.parameters["sd"])), 
                                            data.frame(Name="mean", Min=-10*unname(fitted.parameters["mean"]), 
                                                       Max=10*unname(fitted.parameters["mean"]))))
          priors$SDProp <- unname(fitted.parameters)/10
        } else {
          priors <- setPriors(par=fitted.parameters, 
                              se=c(sd=1), 
                              density = "dnorm", 
                              rules = data.frame(Name="sd", Min=1E-6, Max=10*unname(fitted.parameters)))
          priors$SDProp <- unname(fitted.parameters)/10
        }
      }
    } else {
      if ((D == "lnorm") | (D == "lnorm*") | (D == "lnorm**")) {
        
        if (priors == "dunif") {
          if (length(fitted.parameters) == 2) {
            priors <- setPriors(par=fitted.parameters, 
                                se=c(sd=1, mean=1), 
                                density = "dunif", 
                                rules = rbind(data.frame(Name="sdlog", Min=1E-6, Max=10*unname(fitted.parameters["sdlog"])), 
                                              data.frame(Name="meanlog", Min=-10*unname(fitted.parameters["meanlog"]), 
                                                         Max=10*unname(fitted.parameters["meanlog"]))))
            priors$SDProp <- unname(fitted.parameters)/10
          } else {
            priors <- setPriors(par=fitted.parameters, 
                                se=c(sd=1), 
                                density = "dunif", 
                                rules = data.frame(Name="sdlog", Min=1E-6, Max=10*unname(fitted.parameters)))
            priors$SDProp <- unname(fitted.parameters)/10
          }
        } else {
          if (length(fitted.parameters) == 2) {
            priors <- setPriors(par=fitted.parameters, 
                                se=c(sd=1, mean=1), 
                                density = "dnorm", 
                                rules = rbind(data.frame(Name="sdlog", Min=1E-6, Max=10*unname(fitted.parameters["sdlog"])), 
                                              data.frame(Name="meanlog", Min=-10*unname(fitted.parameters["meanlog"]), 
                                                         Max=10*unname(fitted.parameters["meanlog"]))))
            priors$SDProp <- unname(fitted.parameters)/10
          } else {
            priors <- setPriors(par=fitted.parameters, 
                                se=c(sd=1), 
                                density = "dnorm", 
                                rules = data.frame(Name="sd", Min=1E-6, Max=10*unname(fitted.parameters)))
            priors$SDProp <- unname(fitted.parameters)/10
          }
        }
        
        
      } else {
        
        stop("Automatic priors is available only for lnorm and norm distributions.")
        
      }
      
    }
  }
  
  # try <- L_min_max(x=fitted.parameters                  , 
  #                  n=n                                  ,
  #                  fixed.parameters=fixed.parameters    , 
  #                  minobs=observed.Minimum              , 
  #                  maxobs=observed.Maximum              , 
  #                  medianobs=observed.Median            , 
  #                  meanobs=observed.Mean                , 
  #                  quantiles=observed.Quantiles         ,
  #                  rep=replicates                       , 
  #                  D=D                                  )
  
  out_mcmc <- MHalgoGen(likelihood = L_min_max                                    , 
                        parameters = priors                                       , 
                        fixed.parameters=fixed.parameters                         , 
                        meanobs=observed.Mean                                     , 
                        n=n                                                       , 
                        medianobs=observed.Median                                 ,
                        minobs=observed.Minimum                                   , 
                        maxobs=observed.Maximum                                   ,
                        quantiles=observed.Quantiles                              ,
                        D=D                                                       ,
                        n.iter=n.iter                                             , 
                        n.chains = n.chains                                       , 
                        n.adapt = n.adapt                                         , 
                        thin=thin                                                 , 
                        trace=trace                                               , 
                        rep=replicates                                            , 
                        adaptive=adaptive                                         )
  
  print(as.parameters(out_mcmc, index="quantile"))
  
  return(invisible(out_mcmc))
  
}

.dGEV <- function (x, par = NULL, location = 0, scale = 1, shape = 0, log=FALSE, sum=FALSE, silent=TRUE){
  if (!is.null(par)) {
    if (!is.na(par["location"])) location <- par["location"]
    if (!is.na(par["scale"])) scale <- par["scale"]
    if (!is.na(par["shape"])) shape <- par["shape"]
  }
  
  scale <- abs(scale)
  
  if (shape == 0) {
    # tx <- exp(-(x-location)/scale)
    # y <- (1/scale) * tx^(shape+1) * exp(-tx)
    # 
    logy <- - log(scale) - (x-location)/scale - exp(-(x-location)/scale)
    y <- exp(logy)
    if (any(is.infinite(logy))) {
      y <- ifelse(is.infinite(logy), 1E-99, y)
      logy <- log(y)
    }
  } else {
    tx <- (1+shape*(x-location)/scale)^(-1/shape)
    y <- (1/scale) * tx^(shape+1) * exp(-tx)
    logy <- log(y)
  }
  
  if (shape < 0) {
    y <- ifelse(x > (location + scale / abs(shape)), 1E-99, y)
    logy <- log(y)
  }
  if (shape > 0) {
    y <- ifelse(x < (location - scale/shape), 1E-99, y)
    logy <- log(y)
  }
  
  if (log) {
    y <- logy
    
    if (sum) {
      y <- unname(-sum(y))
      y <- ifelse(is.infinite(y), 1E99, y)
    }
  }
  y
  
}

# .dGEV_global <- function (x, par = NULL, location = 0, scale = 1, shape = 0, log=FALSE, sum=FALSE, silent=TRUE) 
# {
#   if (!is.null(par)) {
#     if (!is.na(par["location"])) location <- par["location"]
#     if (!is.na(par["scale"])) scale <- par["scale"]
#     if (!is.na(par["shape"])) shape <- par["shape"]
#   }
#   
#   scale <- abs(scale)
#   
#   if (!silent) {
#     print(c(location, scale, shape))
#   }
#   
#   # scale <- abs(scale)
#   
#   names.x <- names(x)
#   arg.mat <- suppressWarnings(cbind(x = as.vector(x), location = as.vector(location), 
#                                     scale = as.vector(scale), shape = as.vector(shape)))
#   na.index <- apply(arg.mat, MARGIN = 1, FUN = function(x) any(is.na(x)))
#   if (all(na.index)) {
#     y <- rep(NA, nrow(arg.mat))
#   } else {
#     y <- numeric(nrow(arg.mat))
#     y[na.index] <- NA
#     y.no.na <- y[!na.index]
#     for (i in c("x", "location", "scale", "shape")) assign(i, 
#                                                            arg.mat[!na.index, i])
#     if (any(scale < .Machine$double.eps)) 
#       stop("All values of 'scale' must be positive.")
#     z <- (x - location)/scale
#     shape.eq.0 <- shape == 0
#     if (any(shape.eq.0)) 
#       y.no.na[shape.eq.0] <- (exp(-exp(-z[shape.eq.0])) * 
#                                 exp(-z[shape.eq.0]))/scale[shape.eq.0]
#     shape.gt.0 <- shape > 0
#     if (any(shape.gt.0)) {
#       z.out <- z >= 1/shape
#       y.no.na[shape.gt.0 & z.out] <- 0
#       index <- shape.gt.0 & !z.out
#       if (any(index)) 
#         y.no.na[index] <- exp(-(1 - shape[index] * z[index])^(1/shape[index])) * 
#         (1/scale[index]) * ((1 - shape[index] * z[index])^((1/shape[index]) - 
#                                                              1))
#     }
#     shape.lt.0 <- shape < 0
#     if (any(shape.lt.0)) {
#       z.out <- z <= 1/shape
#       y.no.na[shape.lt.0 & z.out] <- 0
#       index <- shape.lt.0 & !z.out
#       if (any(index)) 
#         y.no.na[index] <- exp(-(1 - shape[index] * z[index])^(1/shape[index])) * 
#         (1/scale[index]) * ((1 - shape[index] * z[index])^((1/shape[index]) - 
#                                                              1))
#     }
#     y[!na.index] <- y.no.na
#   }
#   if (!is.null(names.x)) 
#     names(y) <- rep(names.x, length = length(x))
#   else names(y) <- NULL
#   
#   if (log) y <- log(y)
#   if (sum & log) {
#     y <- unname(-sum(y))
#     y <- ifelse(is.infinite(y), 1E9, y)
#   }
#   y
# }

#' d Write an ASCII Representation of a vector object
#' @title Write an ASCII Representation of a vector object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A string
#' @param x A named vector object
#' @param file either a character string naming a file or a connection. "" indicates output to the console.
#' @param control character vector indicating deparsing options. See .deparseOpts for their description.
#' @param collapse Characters used to separate values.
#' @description Writes an ASCII text representation of an R object.\cr
#' It can be used as a replacement of dput() for named vectors.\cr
#' The controls "keepNA", "keepInteger" and "showAttributes" are utilized for named vectors.
#' @family Characters
#' @examples
#' d(c(A=10, B=20))
#' dput(c(A=10, B=20))
#' @export

d <- function(x, file = "",
              control = c("keepNA", "keepInteger", "showAttributes"), 
              collapse=", \n  ") {
  
  opts <- intToBits(.deparseOpts(control))
  # 1 (1): keepInteger
  # 4 (3): showAttributes
  # 64 (7): keepNA
  
  if ((any(names(attributes(x)) != "names") & !is.null(attributes(x))) | 
    (opts[3] == 0) | 
       ((!inherits(x, "numeric")) & (!inherits(x, "character")) & (!inherits(x, "integer")))) {
    dput(x, file=file, control=control)
  } else {
    if (is.null(names(x))) {
      dput(x, file=file, control=control)
    } else {
      if (is.character(file)) 
        if (nzchar(file)) {
          file <- file(file, "wt")
          on.exit(close(file))
        }
      else file <- stdout()
      
      
      if (is.numeric(x)) {
        cat("c(", paste0(sapply(seq_along(x), FUN=function(i) {
          x1 <- x[i]
          ifelse(is.integer(x1) & (opts[1]==1),
                 if (is.na(x1) & (opts[7]==1)) {
                   paste0("'", gsub("'", "\\\\'", names(x1)), "' = NA_integer_")
                 } else {
                 paste0("'", gsub("'", "\\\\'", names(x1)), "' = ", x1, "L")
                   }, 
                 if (is.na(x1) & (opts[7]==1)) {
                   paste0("'", gsub("'", "\\\\'", names(x1)), "' = NA_real_")
                 } else {
                 paste0("'", gsub("'", "\\\\'", names(x1)), "' = ", format(x1, digits = 17, trim = TRUE))
                 }
          )
          }
        ), 
        collapse=collapse), ")\n", sep="", file = file)
      } else {
        cat("c(", paste0("'", gsub("'", "\\\\'", names(x)), "' = '", 
                         x, "'", collapse=collapse), ")\n", sep="", file = file)
      }
    }
  }
  return(invisible(x))
}

#' char returns the characters defined by the codes
#' @title Return the characters defined by the codes
#' @author Based on this blog: http://datadebrief.blogspot.com/2011/03/ascii-code-table-in-r.html
#' @return A string with characters defined by the codes
#' @param n The code to be used to return a character
#' @description Return a string with characters defined by the codes.
#' @family Characters
#' @examples
#' char(65:75)
#' char(unlist(tapply(144:175, 144:175, function(x) {c(208, x)})))
#' @export


char <- function(n) {rawToChar(as.raw(n))}

#' summary.mcmcComposite get info on the result of a mcmcComposite object
#' @title Summarize the result of a mcmcComposite object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A summary of the result
#' @param object A mcmcComposite object
#' @param ... Not used
#' @param chain The chain to use
#' @family mcmcComposite functions
#' @description Summary for the result of a mcmcComposite object.
#' @examples 
#' \dontrun{
#' library(HelpersMG)
#' require(coda)
#' x <- rnorm(30, 10, 2)
#' dnormx <- function(data, x) {
#'  data <- unlist(data)
#'  return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
#' }
#' parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
#' Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(1, 1), 
#' Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
#' row.names=c('mean', 'sd'))
#' mcmc_run <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
#' plot(mcmc_run, xlim=c(0, 20))
#' plot(mcmc_run, xlim=c(0, 10), parameters="sd")
#' mcmcforcoda <- as.mcmc(mcmc_run)
#' #' heidel.diag(mcmcforcoda)
#' raftery.diag(mcmcforcoda)
#' autocorr.diag(mcmcforcoda)
#' acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
#' acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
#' batchSE(mcmcforcoda, batchSize=100)
#' # The batch standard error procedure is usually thought to 
#' # be not as accurate as the time series methods used in summary
#' summary(mcmcforcoda)$statistics[,"Time-series SE"]
#' summary(mcmc_run)
#' as.parameters(mcmc_run)
#' lastp <- as.parameters(mcmc_run, index="last")
#' parameters_mcmc[,"Init"] <- lastp
#' # The n.adapt set to 1 is used to not record the first set of parameters
#' # then it is not duplicated (as it is also the last one for 
#' # the object mcmc_run)
#' mcmc_run2 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
#' likelihood=dnormx, n.chains=1, n.adapt=1, thin=1, trace=1)
#' mcmc_run3 <- merge(mcmc_run, mcmc_run2)
#' ####### no adaptation, n.adapt must be 0
#' parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
#' mcmc_run3 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
#' likelihood=dnormx, n.chains=1, n.adapt=0, thin=1, trace=1)
#' }
#' @method summary mcmcComposite
#' @export

summary.mcmcComposite <- function(object , chain=NULL, ...) {

  if (!requireNamespace("coda", quietly = TRUE)) {
    stop("coda package is necessary for this function")
  }
  
  
resultMCMC <- object

mcmc <- resultMCMC[["resultMCMC"]]

if (!is.null(chain)) {mcmc <- mcmc[[chain]]}
sm <- summary(mcmc)


if (inherits(sm$statistics, "numeric")) {
  sm$statistics <- t(as.matrix(sm$statistics))
  rownames(sm$statistics) <- rownames(resultMCMC$parametersMCMC$parameters)
  sm$quantiles <- t(as.matrix(sm$quantiles))
  rownames(sm$quantiles) <- rownames(resultMCMC$parametersMCMC$parameters)
}

return(sm)


}

#' newmap.scale Add Scale to Existing Unprojected Map
#' @title Add Scale to Existing Unprojected Map
#' @author See [maps::map.scale()].
#' @return The exact calculated scale is returned.
#' @param x Location of left end of distance gauge.
#' @param y Location of left end of distance gauge.
#' @param relwidth Proportion of width of display to be used for the scale. The default is 0.15.
#' @param metric If TRUE, the distance gauge will be in km, otherwise miles.
#' @param ratio If FALSE, the scale ratio of the map is not displayed.
#' @param col.line The color of lines for the gauge.
#' @param ... Further plotting parameters may be specified as for the command text().
#' @description Adds a scale to an existing map, both as a ratio and a distance gauge.
#' If x or y are not specified, this will be taken to be near the lower left corner of the map.
#' @examples
#' \dontrun{
#' library("maps")
#' library("HelpersMG")
#' map("world", "China")
#' newmap.scale(col.line = "red", col="blue")
#' }
#' @export


newmap.scale <- function (x, y, relwidth = 0.15, metric = TRUE, ratio = TRUE, 
                        col.line = "black", ...) 
{
  format.pretty <- function(x, digits = 2) {
    x = signif(x, 2)
    prettyNum(formatC(x, format = "fg", digits = digits), 
              big.mark = ",")
  }
  usr <- par("usr")
  if (missing(y)) 
    y <- (9 * usr[3] + usr[4])/10
  if (abs(y) >= 90) 
    warning("location of scale out of this world!")
  if (missing(x)) 
    x <- (9 * usr[1] + usr[2])/10
  cosy <- cos((2 * pi * y)/360)
  perdeg <- (2 * pi * (6356.78 + 21.38 * cosy) * cosy)/360
  scale <- (perdeg * 1e+05)/(2.54 * (par("pin")/diff(par("usr"))[-2])[1])
  if (metric) 
    unit <- "km"
  else {
    perdeg <- perdeg * 0.6213712
    unit <- "mi"
  }
  len <- perdeg * relwidth * (usr[2] - usr[1])
  ats <- pretty(c(0, len), n = 2)
  nats <- length(ats)
  labs <- as.character(ats)
  labs[nats] <- paste(labs[nats], unit)
  linexy <- matrix(NA, ncol = 2, nrow = 3 * nats)
  colnames(linexy) <- c("x", "y")
  cxy <- par("cxy")
  dy <- cxy[2] * par("tcl")
  dx <- ats[nats]/perdeg/(nats - 1)
  linexy[1, ] <- c(x, y)
  linexy[2, ] <- c(x, y + dy)
  for (i in 1:(nats - 1)) {
    linexy[3 * i, ] <- c(x + (i - 1) * dx, y)
    linexy[3 * i + 1, ] <- c(x + i * dx, y)
    linexy[3 * i + 2, ] <- c(x + i * dx, y + dy)
  }
  lines(linexy, col = col.line)
  text(x + ats/perdeg, y + dy - 0.5 * cxy[2], labs, adj = c(0.4, 
                                                            0.5), ...)
  if (ratio) 
    text(x, y + 0.5 * cxy[2], paste("scale approx 1:", format.pretty(scale), 
                                    sep = ""), adj = 0, ...)
  invisible(scale)
}

#' RM_duplicate duplicates a results managment within an object
#' @title Duplicate a results managment within an object.
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The original object with a duplicated results managment.
#' @param x The object to duplicate a results managment
#' @param RM The name of results managment stored
#' @param RMnamefrom The name of the results managment to be duplicated
#' @param RMnameto The new name of the results managment
#' @description RM_duplicate duplicates a results managment within an object.
#' @family Results Managment
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Let an object of class objclass being created
#' obj <- list(A=100, name="My object")
#' class(obj) <- "objclass"
#' # And now I create a RM to this object
#' obj <- RM_add(x=obj, RMname="NewAnalysis1")
#' RM_list(obj)
#' obj <- RM_duplicate(x=obj, RMnamefrom="NewAnalysis1", RMnameto="NewAnalysis2")
#' RM_list(obj)
#' }
#' @export

RM_duplicate <- function (x=stop("An object with results managment must be provided"), 
                     RM = "RM", RMnamefrom=1, RMnameto=2) {
  
  xa <- attributes(x)
  rx <- xa[[RM]][RMnamefrom]
  rxnew <- rx
  rxnew[[RMnamefrom]]$name <- as.character(RMnameto)
  names(rxnew) <- as.character(RMnameto)
  rxnew[[RMnameto]]$timestamp <- timestamp(quiet = TRUE)

  ix <- modifyList(xa[[RM]], rxnew)
  
  rx <- list(RM=ix)
  names(rx) <- RM
  
  attributes(x) <- modifyList(xa, rx)
  return(x)
}

#' ScalePreviousPlot returns the scale of the previous plot
#' @title Return the scale of the previous plot
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with xlim and ylim
#' @param x The position in x as relative position
#' @param y The position in y as relative position
#' @family plot and barplot functions
#' @description Return a list with the limits of the previous plot, the center, the range, and the position of label on this axe. 
#' @examples
#' \dontrun{
#' par(xaxs="i", yaxs="i")
#' plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="x", ylab="y")
#' xlim= ScalePreviousPlot()$xlim[1:2]
#' ylim= ScalePreviousPlot()$ylim[1:2]
#' par(xaxs="r", yaxs="i")
#' plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="x", ylab="y")
#' xlim= ScalePreviousPlot()$xlim[1:2]
#' ylim= ScalePreviousPlot()$ylim[1:2]
#' # Here is an example of the use of the label output
#' plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="", ylab="")
#' text(x=ScalePreviousPlot()$xlim["label"], y=ScalePreviousPlot()$ylim["center"], 
#'   xpd=TRUE, "Legend for Y axes", pos=3, srt=90)
#' text(x=ScalePreviousPlot()$xlim["center"], y=ScalePreviousPlot()$ylim["label"], 
#'   xpd=TRUE, "Legend for X axes", pos=1)
#' Example to plot legend always in the same place
#' layout(1:2)
#' plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="", ylab="")
#' text(x=ScalePreviousPlot(x=0.95, y=0.05)$x, 
#'      y=ScalePreviousPlot(x=0.95, y=0.05)$y, 
#'      labels="A", cex=2)
#' plot(x=0:1, y=0:1, type="p", bty="n")
#' text(x=ScalePreviousPlot(x=0.95, y=0.05)$x, 
#'      y=ScalePreviousPlot(x=0.95, y=0.05)$y, 
#'      labels="B", cex=2)
#' }
#' @export


ScalePreviousPlot <- function(x=NULL, y=NULL) {
  if (par("xaxs")=="i") {
    x1 <- par("usr")[1]
    x2 <- par("usr")[2]
  } else {
    x2 <- (par("usr")[1]+par("usr")[2]*26)/27
    x1 <- x2*26-par("usr")[2]/0.04
  }
    if (par("yaxs")=="i") {
    y1 <- par("usr")[3]
    y2 <- par("usr")[4]
  } else {
    y2 <- (par("usr")[3]+par("usr")[4]*26)/27
    y1 <- y2*26-par("usr")[4]/0.04
  }
  
  if (!is.null(x) & !is.null(y)) {
    return(list(x=x1+(x2-x1)*x, 
                y=y1+(y2-y1)*y))
    
  } else {
    return(list(xlim=c(begin=x1, end=x2, center=x1+(x2-x1)/2, range=x2-x1, label=x1-(x2-x1)/2.6), 
                ylim=c(begin=y1, end=y2, center=y1+(y2-y1)/2, range=y2-y1, label=y1-(y2-y1)/3)))
  }
}

#' setPriors1 is a general function to set priors for MHalgoGen()
#' @title Set priors for MHalgoGen()
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Return a data.frame with the formatted prior
#' @param Name Name of the parameter.
#' @param Prior1 The value for the first parameter of the prior distribution.
#' @param Prior2 The value for the second parameter of the prior distribution.
#' @param Prior3 The value for the third parameter of the prior distribution.
#' @param Parameters A vector with parameters of the prior distribution
#' @param Density The density function of the prior (ex. "dunif", "dnorm", "dbeta").
#' @param SDProp The SD of the propositions for the Markov chain.
#' @param Min The minimum value to truncate the prior distribution.
#' @param Max The maximum value to truncate the prior distribution.
#' @param Init The value at the first iteration.
#' @description Set priors for MHalgoGen()\cr
#' A check is done to verify that the Init value is well between the Min and Max range.\cr
#' The order of Prior1, 2, and 3 must be similar to the order in the Density function. 
#' For this reason, it is safer to use Parameters.\cr
#' As a rule of thumb, SDProp can be 1/10th of the range of posterior. For example if 
#' the posterior is thought to be from 0.1 to 0.2, you can use SDProp being (0.2-0.1)/10=0.01.
#' @family mcmcComposite functions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' Priors <- setPriors1(Name="a0", Prior1=0, Prior2=1, Density="dunif", 
#'                      SDProp=0.1, Min=0, Max=1, Init=0.5)
#' Priors <- Priors + setPriors1(Name="a1", Prior1=0.5, Prior2=0.1, Density="dnorm", 
#'                      SDProp=0.05, Min=0, Max=1, Init=0.5)
#' # Using Parameters
#' Priors <- setPriors1(Name="a0", Parameters=c(min=0, max=1), Density="dunif", 
#'                      SDProp=0.1, Min=0, Max=1, Init=0.5)
#' Priors <- Priors + setPriors1(Name="a1", Parameters=c(mean=0.5, sd=0.1), Density="dnorm", 
#'                      SDProp=0.05, Min=0, Max=1, Init=0.5)
#' }
#' @export


setPriors1 <- function(Name=stop("A name of parameter is required."), 
                       Prior1 = NULL, 
                       Prior2 = NULL,
                       Prior3 = NULL, 
                       Parameters = NULL, 
                       Density=stop("A density is required."), 
                       SDProp =stop("A SD of proposal is required."), 
                       Min=-Inf, 
                       Max=+Inf, 
                       Init=stop("An initial value is required.")) {
  
  if (Init < Min) stop("The Init value must not be lower than the Min value.")
  if (Init > Max) stop("The Init value must not be higher than the Max value.")
  
  if (Min > Max) {
    warning("I reverse Min and Max.")
    interm <- Min; Min <- Max; Max <- interm
  }
  
  if (is.null(c(Prior1, Prior2, Prior3, Parameters))) {
    stop("All Prior1, Prior2, Prior3, and Parameters cannot be NULL.")
  }
  
  if (!is.null(Parameters)) {
    np <- getFromNamespace("formalArgs", ns="methods")(Density)
    for (i in 1:length(Parameters)) {
      pos <- which(names(Parameters[i]) == np)
      if (identical(pos, integer(0))) stop(paste0("I don't understand the name of Parameters ", names(Parameters[i])))
      assign(paste0("Prior", as.character(pos-1)), unname(Parameters[i]))
    }
  }
  
  if (is.null(Prior3)) Prior3 <- NA
  
  df_priors <- data.frame(Density=Density, 
                          Prior1=Prior1, Prior2=Prior2, Prior3=Prior3, 
                          SDProp=SDProp, 
                          Min=Min, Max=Max, 
                          Init=Init, row.names = Name)
  
  df_priors <- addS3Class(df_priors, c("PriorsmcmcComposite", "data.frame"))
  return(df_priors)
}

#' FormatCompareAIC formats data to be used with compare_AIC()
#' @title Format data to be used with compare_AIC()
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return An object to be used with compare_AIC()
#' @param logLik The log likelihood
#' @param nobs Number of observations
#' @param df Number of parameters
#' @description Format data to be used with compare_AIC(), compare_AICc() and compare_BIC().\cr
#' Note that logLik is supposed to not be -logLik.
#' @family AIC
#' @examples
#' \dontrun{
#' ED <- FormatCompareAIC(logLik=-140, nobs=100, df=3)
#' L <- FormatCompareAIC(logLik=-145, nobs=100, df=4)
#' compare_AIC(L=L, ED=ED)
#' compare_AICc(L=L, ED=ED)
#' compare_BIC(L=L, ED=ED)
#' }
#' @export


FormatCompareAIC <- function(logLik, nobs, df) {
  l <- list(logLik=logLik, 
            AIC=-2*logLik+2*df, 
            AICc=(-2*logLik+2*df)+(2*df*(df+1))/(nobs-df-1), 
            BIC=-2*logLik+log(nobs)*df)
  attributes(l) <- modifyList(attributes(l), list(nall=nobs , nobs=nobs, 
                        df=df, class="compareAIC"))
  return(l)
}

#' Distribution of the Sum of Independent Negative Binomial Random Variables.
#' @title Distribution of the sum independent negative binomial random variables. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com} and Jon Barry \email{jon.barry@@cefas.gov.uk}
#' @references Furman, E., 2007. On the convolution of the negative binomial random variables. Statistics & Probability Letters 77, 169-172.
#' @references Vellaisamy, P. & Upadhye, N.S. 2009. On the sums of compound negative binomial and gamma random variables. Journal of Applied Probability, 46, 272-283.
#' @references Girondot M, Barry J. 2023. Computation of the distribution of the sum of independent negative binomial random variables. Mathematical and Computational Applications 2023, 28, 63, doi:10.3390/mca28030063 
#' @return \code{dSnbinom} gives the density, \code{pSnbinom} gives the distribution function, 
#' \code{qSnbinom} gives the quantile function, and \code{rSnbinom} generates random deviates.
#' @param x vector of (non-negative integer) quantiles.
#' @param n number of observations.
#' @param q vector of quantiles.
#' @param p	vector of probabilities.
#' @param size target for number of successful trials, or dispersion parameter (the shape parameter of the gamma mixing distribution). Must be strictly positive, need not be integer.
#' @param prob probability of success in each trial. 0 < prob <= 1.
#' @param mu alternative parametrization via mean.
#' @param log,log.p logical; if TRUE, probabilities \emph{p} are given as \emph{log(p)}.
#' @param tol Tolerance for recurrence for Furman (2007) method. If NULL, will use a saddlepoint estimation.
#' @param method Can be Furman (default), Vellaisamy&Upadhye or exact, approximate.normal, approximate.negativebinomial, approximate.RandomObservations, or saddlepoint.
#' @param normalize If TRUE (default) will normalize the saddlepoint approximation estimate.
#' @param max.iter Number of maximum iterations for Furman method. Can be NULL.
#' @param mean Mean of the distribution for approximate.normal method. If NULL, the theoretical mean will be used.
#' @param sd Standard deviation of the distribution for approximate.normal method. If NULL, the theoretical sd will be used.
#' @param n.random Number of random numbers used to estimate parameters of distribution for approximate.RandomObservations method.
#' @param lower.tail logical; if TRUE (default), probabilities are P\[X <= x\], otherwise, P\[X > x\].
#' @param parallel logical; if FALSE (default), parallel computing is not used for Vellaisamy&Upadhye methods.
#' @param verbose Give more information on the method.
#' @description Distribution of the sum of random variable with negative binomial distributions.\cr
#' Technically the sum of random variable with negative binomial distributions is a convolution of negative 
#' binomial random variables.\cr
#' \code{dSnbinom} returns the density for the sum of random variable with negative binomial distributions.\cr
#' \code{pSnbinom} returns the distribution function for the sum of random variable with negative binomial distributions.\cr
#' \code{qSnbinom} returns the quantile function for the sum of random variable with negative binomial distributions.\cr
#' \code{rSnbinom} returns random numbers for the sum of random variable with negative binomial distributions.\cr
#' If all prob values are the same, exact probabilities are estimated.\cr
#' Estimate using \code{Vellaisamy&Upadhye} method uses parallel computing 
#' depending on value of \code{parallel}. The number of cores in usage can be 
#' defined using \code{options(mc.cores = c)} with \code{c} being the number of cores to be used. By default it will 
#' use all the available cores. Forking will be used in Unix system and no forking on Windows systems.\cr
#' When \code{Furman} method is in use, it will return the progress of Pr(S = x) during recursion 
#' in an attribute if verbose is TRUE (see examples).
#' @family Distributions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' alpha <- c(1, 2, 5, 1, 2)
#' p <- c(0.1, 0.12, 0.13, 0.14, 0.14)
#' # By default, the Furman method with tol=1E-40 is used
#' dSnbinom(20, size=alpha, prob=p)
#' # Note the attribute is the dynamics of convergence of Pr(X=x)
#' attributes(dSnbinom(20, size=alpha, prob=p, verbose=TRUE))$Pk[, 1]
#' 
#' mutest <- c(0.01, 0.02, 0.03)
#' sizetest <- 2
#' x <- 20
#' # Exact probability
#' dSnbinom(x, size=sizetest, mu=mutest, method="vellaisamy&upadhye")
#' dSnbinom(x, size=sizetest, mu=mutest, method="vellaisamy&upadhye", log=TRUE)
#' # With Furman method and tol=1E-12, when probability 
#' # is very low, it will be biased
#' dSnbinom(x, size=sizetest, mu=mutest, method="Furman", tol=1E-12)
#' # The solution is to use a tolerance lower than the estimate
#' dSnbinom(x, size=sizetest, mu=mutest, method="Furman", tol=1E-45)
#' # Here the estimate used a first estimation by saddlepoint approximation
#' dSnbinom(x, size=sizetest, mu=mutest, method="Furman", tol=NULL)
#' # Or a huge number of iterations; but it is not the best solution
#' dSnbinom(x, size=sizetest, mu=mutest, method="Furman", 
#'          tol=1E-12, max.iter=10000)
#' # With the saddle point approximation method
#' dSnbinom(x, size=sizetest, mu=mutest, method="saddlepoint", log=FALSE)
#' dSnbinom(x, size=sizetest, mu=mutest, method="saddlepoint", log=TRUE)
#' 
#' # Another example
#' sizetest <- c(1, 1, 0.1)
#' mutest <- c(2, 1, 10)
#' x <- 5
#' (exact <- dSnbinom(x=x, size=sizetest, mu=mutest, method="Vellaisamy&Upadhye"))
#' (sp <- dSnbinom(x=x, size=sizetest, mu=mutest, method="saddlepoint"))
#' paste0("Saddlepoint approximation: Error of ", specify_decimal(100*abs(sp-exact)/exact, 2), "%")
#' (furman <- dSnbinom(x=x, size=sizetest, mu=mutest, method="Furman"))
#' paste0("Inversion of mgf: Error of ", specify_decimal(100*abs(furman-exact)/exact, 2), "%")
#' (na <- dSnbinom(x=x, size=sizetest, mu=mutest, method="approximate.normal")) 
#' paste0("Gaussian approximation: Error of ", specify_decimal(100*abs(na-exact)/exact, 2), "%")
#' (nb <- dSnbinom(x=x, size=sizetest, mu=mutest, method="approximate.negativebinomial"))
#' paste0("NB approximation: Error of ", specify_decimal(100*abs(nb-exact)/exact, 2), "%")
#' 
#' plot(0:20, dSnbinom(0:20, size=sizetest, mu=mutest, method="furman"), bty="n", type="h", 
#'      xlab="x", ylab="Density", ylim=c(0, 0.2), las=1)
#' points(x=0:20, y=dSnbinom(0:20, size=sizetest, mu=mutest, 
#'                            method="saddlepoint"), pch=1, col="blue")
#' points(x=0:20, y=dSnbinom(0:20, size=sizetest, mu=mutest, 
#'                            method="approximate.negativebinomial"), 
#'                            col="red")
#' points(x=0:20, y=dSnbinom(0:20, size=sizetest, mu=mutest, 
#'                            method="approximate.normal"), 
#'                            col="green")
#' 
#' # Test with a single distribution
#' dSnbinom(20, size=1, mu=20)
#' # when only one distribution is available, it is the same as dnbinom()
#' dnbinom(20, size=1, mu=20)
#' 
#' # If a parameter is supplied as only one value, it is supposed to be constant
#' dSnbinom(20, size=1, mu=c(14, 15, 10))
#' dSnbinom(20, size=c(1, 1, 1), mu=c(14, 15, 10))
#' 
#' # The functions are vectorized:
#' plot(0:200, dSnbinom(0:200, size=alpha, prob=p, method="furman"), bty="n", type="h", 
#'      xlab="x", ylab="Density")
#' points(0:200, dSnbinom(0:200, size=alpha, prob=p, method="saddlepoint"), 
#'      col="red", pch=3)
#'      
#' # Comparison with simulated distribution using rep replicates
#' alpha <- c(2.1, 2.05, 2)
#' mu <- c(10, 30, 20)
#' rep <- 100000
#' distEmpirique <- rSnbinom(rep, size=alpha, mu=mu)
#' tabledistEmpirique <- rep(0, 301)
#' names(tabledistEmpirique) <- as.character(0:300)
#' tabledistEmpirique[names(table(distEmpirique))] <- table(distEmpirique)/rep
#' 
#' plot(0:300, dSnbinom(0:300, size=alpha, mu=mu, method="furman"), type="h", bty="n", 
#'    xlab="x", ylab="Density", ylim=c(0,0.02))
#' plot_add(0:(length(tabledistEmpirique)-1), tabledistEmpirique, type="l", col="red")
#' legend(x=200, y=0.02, legend=c("Empirical", "Theoretical"), 
#'    text.col=c("red", "black"), bty="n")
#' 
#' 
#' # Example from Vellaisamy, P. & Upadhye, N.S. (2009) - Table 1
#' # Note that computing time for k = 7 using exact method is very long
#' k <- 2:7
#' x <- c(3, 5, 8, 10, 15)
#' table1_Vellaisamy <- matrix(NA, ncol=length(x), nrow=length(k))
#' rownames(table1_Vellaisamy) <- paste0("n = ", as.character(k))
#' colnames(table1_Vellaisamy) <- paste0("x = ", as.character(x))
#' table1_approximateObservations <- table1_Vellaisamy
#' table1_Furman3 <- table1_Vellaisamy
#' table1_Furman6 <- table1_Vellaisamy
#' table1_Furman9 <- table1_Vellaisamy
#' table1_Furman12 <- table1_Vellaisamy
#' table1_Furman40 <- table1_Vellaisamy
#' table1_Furman40 <- table1_Vellaisamy
#' table1_FurmanAuto <- table1_Vellaisamy
#' table1_FurmanAuto_iter <- table1_Vellaisamy
#' table1_Vellaisamy_parallel <- table1_Vellaisamy
#' table1_Approximate_Normal <- table1_Vellaisamy
#' table1_saddlepoint <- table1_Vellaisamy
#' 
#' st_Furman3 <- rep(NA, length(k))
#' st_Furman6 <- rep(NA, length(k))
#' st_Furman9 <- rep(NA, length(k))
#' st_Furman12 <- rep(NA, length(k))
#' st_Furman40 <- rep(NA, length(k))
#' st_FurmanAuto <- rep(NA, length(k))
#' st_approximateObservations <- rep(NA, length(k))
#' st_Vellaisamy <- rep(NA, length(k))
#' st_Vellaisamy_parallel <- rep(NA, length(k))
#' st_Approximate_Normal <- rep(NA, length(k))
#' st_saddlepoint <- rep(NA, length(k))
#' 
#' for (n in k) {
#'     print(n)
#'     alpha <- 1:n
#'     p <- (1:n)/10
#'     st_Vellaisamy[which(n == k)] <- 
#'         system.time({
#'         table1_Vellaisamy[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="Vellaisamy&Upadhye", log=FALSE, verbose=FALSE)
#'         })[1]
#'     st_Vellaisamy_parallel[which(n == k)] <- 
#'         system.time({
#'         table1_Vellaisamy_parallel[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      parallel=TRUE, 
#'                                      method="Vellaisamy&Upadhye", log=FALSE, verbose=FALSE)
#'         })[1]
#'     st_approximateObservations[which(n == k)] <- 
#'         system.time({
#'         table1_approximateObservations[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="approximate.RandomObservations", log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#'     st_Furman3[which(n == k)] <- 
#'         system.time({
#'             table1_Furman3[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="Furman", tol=1E-3, log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#'     st_Furman6[which(n == k)] <- 
#'         system.time({
#'             table1_Furman6[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="Furman", tol=1E-6, log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#'     st_Furman9[which(n == k)] <- 
#'         system.time({
#'             table1_Furman9[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="Furman", tol=1E-9, log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#'     st_Furman12[which(n == k)] <- 
#'         system.time({
#'             table1_Furman12[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="Furman", tol=1E-12, log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#'     st_Furman40[which(n == k)] <- 
#'         system.time({
#'             table1_Furman40[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="Furman", tol=1E-40, log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#' 
#'     st_FurmanAuto[which(n == k)] <- 
#'         system.time({
#'             table1_FurmanAuto[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="Furman", tol=NULL, log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#'
#'     st_Approximate_Normal[which(n == k)] <- 
#'         system.time({
#'             table1_Approximate_Normal[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="approximate.normal", tol=1E-12, log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#'         st_saddlepoint[which(n == k)] <- 
#'         system.time({
#'             table1_saddlepoint[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
#'                                      method="saddlepoint", tol=1E-12, log=FALSE, 
#'                                      verbose=FALSE)
#'         })[1]
#'
#' for (xc in x) {
#'  essai <- dSnbinom(x=xc, prob=p, size=alpha, method="Furman", tol=NULL, log=FALSE, verbose=TRUE)
#'  table1_FurmanAuto_iter[which(n == k), which(xc == x)] <- nrow(attributes(essai)[[1]])
#' }
#' }
#' 
#' cbind(table1_Vellaisamy, st_Vellaisamy)
#' cbind(table1_Vellaisamy_parallel, st_Vellaisamy_parallel)
#' cbind(table1_Furman3, st_Furman3)
#' cbind(table1_Furman6, st_Furman6)
#' cbind(table1_Furman9, st_Furman9)
#' cbind(table1_Furman12, st_Furman12)
#' cbind(table1_Furman40, st_Furman40)
#' cbind(table1_FurmanAuto, st_FurmanAuto)
#' cbind(table1_approximateObservations, st_approximateObservations)
#' cbind(table1_Approximate_Normal, st_Approximate_Normal)
#' cbind(table1_saddlepoint, st_saddlepoint)
#' 
#' 
#' # Test of different methods
#' n <- 9
#' x <- 17
#' alpha <- 1:n
#' p <- (1:n)/10
#' 
#' # Parallel computing is not always performant
#' # Here it is very performant
#' system.time({print(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
#'          verbose=TRUE, parallel=TRUE))})
#' system.time({print(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
#'          verbose=TRUE, parallel=FALSE))})
#'          
#' # Test of different methods
#' n <- 7
#' x <- 8
#' alpha <- 1:n
#' p <- (1:n)/10
#' 
#' # Parallel computing is not always performant
#' # Here it is approximately the same time of execution
#' system.time({print(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
#'          verbose=TRUE, parallel=TRUE))})
#' system.time({print(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
#'          verbose=TRUE, parallel=FALSE))})
#'          
#' # Test of different methods
#' n <- 7
#' x <- 15
#' alpha <- 1:n
#' p <- (1:n)/10
#' 
#' # Parallel computing is sometimes very performant
#' # Here parallel computing is 7 times faster (with a 8 cores computer) 
#' #             for vellaisamy&upadhye method
#' system.time(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
#'          verbose=TRUE, parallel=TRUE))
#' system.time(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
#'          verbose=TRUE, parallel=FALSE))
#'          
#' # Test of different methods
#' n <- 2
#' x <- 3
#' alpha <- 1:n
#' p <- (1:n)/10
#' 
#' # Parallel computing is sometimes very performant
#' # Here parallel computing is 7 times faster (with a 8 cores computer) 
#' #             for vellaisamy&upadhye method
#' system.time(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
#'          verbose=TRUE, parallel=TRUE))
#' system.time(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
#'          verbose=TRUE, parallel=FALSE))
#'          
#' # Test for different tolerant values
#' n <- 7
#' x <- 8
#' alpha <- 1:n
#' p <- (1:n)/10
#' dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, verbose=TRUE)
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-3, verbose=TRUE))
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-6, verbose=TRUE))
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-9, verbose=TRUE))
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, verbose=TRUE))
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Saddlepoint", log=FALSE, verbose=TRUE))
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="approximate.RandomObservations", 
#'          log=FALSE, verbose=TRUE))
#' 
#' # Test for criteria of convergence
#' Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
#'                    log=FALSE, verbose=TRUE)
#' Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
#'                  verbose=TRUE)
#' Pr_exact;as.numeric(Pr_Furman)
#' plot(1:length(attributes(Pr_Furman)$Pk), 
#'      log10(abs(attributes(Pr_Furman)$Pk-Pr_exact)), type="l", xlab="Iterations", 
#'      ylab="Abs log10", bty="n")
#' lines(1:(length(attributes(Pr_Furman)$Pk)-1), 
#'       log10(abs(diff(attributes(Pr_Furman)$Pk))), col="red")
#' legend("bottomleft", legend=c("Log10 Convergence to true value", "Log10 Rate of change"), 
#'        col=c("black", "red"), 
#'        lty=1)
#'        
#' n <- 7
#' x <- 6
#' alpha <- 1:n
#' p <- (1:n)/10
#' Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
#'                    log=FALSE, verbose=TRUE)
#' Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
#'                  verbose=TRUE)
#' Pr_saddlepoint <- dSnbinom(x=x, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=TRUE)                  
#'                  
#' # pdf("figure.pdf", width=7, height=7, pointsize=14)                 
#' ylab <- as.expression(bquote(.("P(S=6) x 10")^"6"))
#' layout(1:2)
#' par(mar=c(3, 4, 1, 1))
#' plot(1:length(attributes(Pr_Furman)$Pk), 
#'      attributes(Pr_Furman)$Pk*1E6, type="l", xlab="", 
#'      ylab="", bty="n", las=1, xlim=c(0, 80))
#' mtext(text=ylab, side=2, line=2.5)
#' segments(x0=25, x1=80, y0=Pr_exact*1E6, y1=Pr_exact*1E6, lty=3)
#' par(xpd=TRUE)
#' text(x=0, y=6, labels="Exact probability", pos=4)
#' txt <- "        Approximate probability\n    based on Furman (2007)\nrecursive iterations"
#' text(x=20, y=2, labels=txt, pos=4)
#' text(x=75, y=5, labels="A", cex=2)
#' par(mar=c(4, 4, 1, 1))
#' ylab <- as.expression(bquote("log"["10"]*""*"(P"["k+1"]*""*" - P"["k"]*""*")"))
#' plot(1:(length(attributes(Pr_Furman)$Pk)-1), 
#'       log10(diff(attributes(Pr_Furman)$Pk)), col="black", xlim=c(0, 80), type ="l", 
#'       bty="n", las=1, xlab="Iterations", ylab="")
#' mtext(text=ylab, side=2, line=2.5)
#'  peak <- (1:(length(attributes(Pr_Furman)$Pk)-1))[which.max(
#'             log10(abs(diff(attributes(Pr_Furman)$Pk))))]
#' segments(x0=peak, x1=peak, y0=-12, y1=-5, lty=2)
#' text(x=0, y=-6, labels="Positive trend", pos=4)
#' text(x=30, y=-6, labels="Negative trend", pos=4)
#' segments(x0=0, x1=43, y0=-12, y1=-12, lty=4)
#' segments(x0=62, x1=80, y0=-12, y1=-12, lty=4)
#' text(x=45, y=-12, labels="Tolerance", pos=4)
#' text(x=75, y=-7, labels="B", cex=2)
#' # dev.off()
#' 
#' # pdf("figure 2.pdf", width=7, height=7, pointsize=14)                 
#' ylab <- as.expression(bquote(.("P(S=6) x 10")^"6"))
#' layout(1:2)
#' par(mar=c(3, 4, 1, 1))
#' plot(1:length(attributes(Pr_Furman)$Pk), 
#'      attributes(Pr_Furman)$Pk*1E6, type="l", xlab="", 
#'      ylab="", bty="n", las=1, xlim=c(0, 80))
#' mtext(text=ylab, side=2, line=2.5)
#' segments(x0=25, x1=80, y0=Pr_exact*1E6, y1=Pr_exact*1E6, lty=3)
#' par(xpd=TRUE)
#' text(x=0, y=6, labels="Exact probability", pos=4)
#' txt <- "        Approximate probability\n    based on Furman (2007)\nrecursive iterations"
#' text(x=20, y=2, labels=txt, pos=4)
#' text(x=75, y=5, labels="A", cex=2)
#' par(mar=c(4, 4, 1, 1))
#' ylab <- as.expression(bquote("(P"["k+1"]*""*" - P"["k"]*""*") x 10"^"7"))
#' plot(1:(length(attributes(Pr_Furman)$Pk)-1), 
#'       diff(attributes(Pr_Furman)$Pk)*1E7, 
#'       col="black", xlim=c(0, 80), type ="l", 
#'       bty="n", las=1, xlab="Iterations", ylab="")
#' mtext(text=ylab, side=2, line=2.5)
#'  peak <- (1:(length(attributes(Pr_Furman)$Pk)-1))[which.max(diff(attributes(Pr_Furman)$Pk))]
#' segments(x0=peak, x1=peak, y0=0, y1=3.5, lty=2)
#' text(x=-2, y=3.5, labels="Positive trend", pos=4)
#' text(x=30, y=3.5, labels="Negative trend", pos=4)
#' segments(x0=0, x1=22, y0=1E-12, y1=1E-12, lty=4)
#' segments(x0=40, x1=80, y0=1E-12, y1=1E-12, lty=4)
#' text(x=22, y=1E-12+0.2, labels="Tolerance", pos=4)
#' text(x=75, y=3, labels="B", cex=2)
#' # dev.off()
#' 
#' # Test of different methods
#' n <- 2
#' x <- 15
#' alpha <- 1:n
#' p <- (1:n)/10
#' dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, verbose=TRUE)
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-3, verbose=TRUE))
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-6, verbose=TRUE))
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-9, verbose=TRUE))
#' as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, verbose=TRUE))
#' dSnbinom(x=x, prob=p, size=alpha, method="approximate.RandomObservations", 
#'          log=FALSE, verbose=TRUE)
#'
#'
#' n <- 50
#' x <- 300
#' alpha <- (1:n)/100
#' p <- (1:n)/1000
#' # Produce an error
#' Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
#'                    log=FALSE, verbose=TRUE)
#' Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
#'                  verbose=FALSE)
#' Pr_ApproximateNormal <- dSnbinom(x=x, prob=p, size=alpha, method="approximate.normal", 
#'                         log=FALSE, 
#'                         verbose=TRUE)
#' Pr_ApproximateRandom <- dSnbinom(x=x, prob=p, size=alpha, method="approximate.RandomObservations", 
#'                         log=FALSE, n.random=1E6, 
#'                         verbose=TRUE)
#'                         
#' n <- 500
#' x <- 3000
#' alpha <- (1:n)/100
#' p <- (1:n)/1000
#' # Produce an error
#' Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
#'                    log=FALSE, verbose=TRUE)
#' Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
#'                  verbose=FALSE)
#' Pr_ApproximateNormal <- dSnbinom(x=x, prob=p, size=alpha, method="approximate.normal", 
#'                         log=FALSE, 
#'                         verbose=TRUE)
#' Pr_ApproximateNegativeBinomial <- dSnbinom(x=x, prob=p, size=alpha, 
#'                         method="approximate.negativebinomial", 
#'                         log=FALSE, 
#'                         verbose=TRUE)
#' Pr_ApproximateRandom <- dSnbinom(x=x, prob=p, size=alpha, 
#'                         method="approximate.RandomObservations", 
#'                         log=FALSE, n.random=1E6, 
#'                         verbose=TRUE)
#' Pr_ApproximateSaddlepoint <- dSnbinom(x=x, prob=p, size=alpha, 
#'                         method="saddlepoint", 
#'                         log=FALSE, 
#'                         verbose=TRUE)
#'              
#'              
#'              
#' layout(matrix(1:4, ncol=2, byrow=TRUE))
#' par(mar=c(3, 4.5, 1, 1))
#' alpha <- seq(from=10, to=100, length.out=3)
#' p <- seq(from=0.5, to=0.9, length.out=3)
#' 
#' p_nb <- dSnbinom(0:100, prob=p, size=alpha, method="vellaisamy&upadhye", verbose=TRUE)
#' p_Furman <- dSnbinom(0:100, prob=p, size=alpha, method="Furman", verbose=FALSE)
#' p_normal <- dSnbinom(0:100, prob=p, size=alpha, method="approximate.normal", verbose=TRUE)
#' p_aNB <- dSnbinom(0:100, prob=p, size=alpha, method="approximate.negativebinomial", verbose=TRUE)
#' p_SA <- dSnbinom(0:100, prob=p, size=alpha, method="saddlepoint", verbose=TRUE)
#' 
#' lab_PSnx <- bquote(italic("P(S"* "" [n] * "=x)"))
#' 
#' plot(1, 1, las=1, bty="n", col="grey", xlab="", 
#'        xlim=c(10, 70), ylim=c(0, 0.05), 
#'        ylab=lab_PSnx, type="n")
#' par(xpd=FALSE)
#' segments(x0=(0:100), x1=(0:100), 
#'          y0=0, y1=as.numeric(p_nb), col="black")
#'          
#' plot(x=p_nb, y=p_normal, pch=4, cex=0.5, las=1, bty="n", 
#'      xlab=bquote(italic("P" * "" [exact] * "(S"* "" [n] * "=x)")), 
#'      ylab=bquote(italic("P" * "" [approximate] * "(S"* "" [n] * "=x)")),  
#'      xlim=c(0, 0.05), ylim=c(0, 0.05))
#' points(x=p_nb, y=p_aNB, pch=5, cex=0.5)
#' points(x=p_nb, y=p_SA, pch=6, cex=0.5)
#' points(x=p_Furman, y=p_SA, pch=19, cex=0.5)
#' 
#' n <- 2
#' x <- 15
#' alpha <- 1:n
#' p <- (1:n)/10
#' 
#' p_nb <- dSnbinom(0:80, prob=p, size=alpha, method="vellaisamy&upadhye", verbose=TRUE)
#' p_Furman <- dSnbinom(0:80, prob=p, size=alpha, method="Furman", verbose=FALSE)
#' p_normal <- dSnbinom(0:80, prob=p, size=alpha, method="approximate.normal", verbose=TRUE)
#' p_aNB <- dSnbinom(0:80, prob=p, size=alpha, method="approximate.negativebinomial", verbose=TRUE)
#' p_SA <- dSnbinom(0:80, prob=p, size=alpha, method="saddlepoint", verbose=TRUE)
#' 
#' 
#' par(mar=c(4, 4.5, 1, 1))
#' plot(1, 1, las=1, bty="n", col="grey", xlab="x", 
#'        xlim=c(0, 60), ylim=c(0, 0.05), 
#'        ylab=lab_PSnx, type="n")
#' par(xpd=FALSE)
#' segments(x0=(0:80), x1=(0:80), 
#'          y0=0, y1=as.numeric(p_nb), col="black")
#'          
#' plot(x=p_nb, y=p_normal, pch=4, cex=0.5, las=1, bty="n", 
#'      xlab=bquote(italic("P" * "" [exact] * "(S"* "" [n] * "=x)")), 
#'      ylab=bquote(italic("P" * "" [approximate] * "(S"* "" [n] * "=x)")), 
#'      xlim=c(0, 0.05), ylim=c(0, 0.05))
#' points(x=p_nb, y=p_aNB, pch=5, cex=0.5)
#' points(x=p_nb, y=p_SA, pch=6, cex=0.5)
#' points(x=p_Furman, y=p_SA, pch=19, cex=0.5)
#'
#' # pdf("figure 1.pdf", width=7, height=7, pointsize=14)    
#' 
#' layout(1:2)
#' par(mar=c(3, 4, 1, 1))
#' alpha <- seq(from=10, to=100, length.out=3)
#' p <- seq(from=0.5, to=0.9, length.out=3)
#' 
#' p_nb <- dSnbinom(0:100, prob=p, size=alpha, method="vellaisamy&upadhye", verbose=TRUE)
#' p_Furman <- dSnbinom(0:100, prob=p, size=alpha, method="Furman", verbose=FALSE)
#' p_normal <- dSnbinom(0:100, prob=p, size=alpha, method="approximate.normal", verbose=TRUE)
#' p_aNB <- dSnbinom(0:100, prob=p, size=alpha, method="approximate.negativebinomial", verbose=TRUE)
#' p_SA <- dSnbinom(0:100, prob=p, size=alpha, method="saddlepoint", verbose=TRUE)
#' 
#' lab_PSnx <- bquote(italic("P(S"* "" [n] * "=x)"))
#' 
#' plot(1, 1, las=1, bty="n", col="grey", xlab="", 
#'        xlim=c(10, 70), ylim=c(0, 0.09), 
#'        ylab="", type="n", yaxt="n")
#' axis(2, at=seq(from=0, to=0.05, by=0.01), las=1)
#' mtext(lab_PSnx, side = 2, adj=0.3, line=3)
#' par(xpd=FALSE)
#' segments(x0=(0:100), x1=(0:100), 
#'          y0=0, y1=as.numeric(p_nb), col="black")
#' errr <- (abs((100*(p_Furman-p_nb)/p_nb)))/1000+0.055
#' errr <- ifelse(is.infinite(errr), NA, errr)
#' lines(x=(0:100), y=errr, lty=5, col="red", lwd=2)
#' errr <- (abs((100*(p_normal-p_nb)/p_nb)))/1000+0.055
#' lines(x=(0:100), y=errr, lty=2, col="blue", lwd=2)
#' errr <- (abs((100*(p_aNB-p_nb)/p_nb)))/1000+0.055
#' lines(x=(0:100), y=errr, lty=3, col="purple", lwd=2)
#' errr <- (abs((100*(p_SA-p_nb)/p_nb)))/1000+0.055
#' lines(x=(0:100), y=errr, lty=4, col="green", lwd=2)
#' axis(2, at=seq(from=0, to=40, by=10)/1000+0.055, las=1, 
#'      labels=as.character(seq(from=0, to=40, by=10)))
#' mtext("|% error|", side = 2, adj=0.9, line=3)
#' par(xpd=TRUE)
#' legend(x=30, y=0.1, legend=c("Inversion of mgf", "Saddlepoint", "Normal", "Negative binomial"), 
#'        lty=c(5, 4, 2, 3), bty="n", cex=0.8, col=c("red", "green", "blue", "purple"), lwd=2)
#' legend(x=10, y=0.05, legend=c("Exact"), lty=c(1), bty="n", cex=0.8)
#' par(xpd=TRUE)
#' text(x=ScalePreviousPlot(x = 0.95, y = 0.1)$x, 
#'      y=ScalePreviousPlot(x = 0.95, y = 0.1)$y, labels="A", cex=2)
#'      
#'      
#' # When normal approximation will fail
#' n <- 2
#' x <- 15
#' alpha <- 1:n
#' p <- (1:n)/10
#' 
#' p_nb <- dSnbinom(0:80, prob=p, size=alpha, method="vellaisamy&upadhye", verbose=TRUE)
#' p_Furman <- dSnbinom(0:80, prob=p, size=alpha, method="Furman", verbose=FALSE)
#' p_normal <- dSnbinom(0:80, prob=p, size=alpha, method="approximate.normal", verbose=TRUE)
#' p_aNB <- dSnbinom(0:80, prob=p, size=alpha, method="approximate.negativebinomial", verbose=TRUE)
#' p_SA <- dSnbinom(0:80, prob=p, size=alpha, method="saddlepoint", verbose=TRUE)
#' 
#' par(mar=c(4, 4, 1, 1))
#' plot(1, 1, las=1, bty="n", col="grey", xlab="x", 
#'        xlim=c(0, 60), ylim=c(0, 0.09), 
#'        ylab="", type="n", yaxt="n")
#' axis(2, at=seq(from=0, to=0.05, by=0.01), las=1)
#' mtext(lab_PSnx, side = 2, adj=0.3, line=3)
#' par(xpd=FALSE)
#' segments(x0=(0:80), x1=(0:80), 
#'          y0=0, y1=as.numeric(p_nb), col="black")
#' errr <- (abs((100*(p_Furman-p_nb)/p_nb)))/1000+0.055
#' errr <- ifelse(is.infinite(errr), NA, errr)
#' lines(x=(0:80), y=errr, lty=5, col="red", lwd=2)
#' errr <- (abs((100*(p_normal-p_nb)/p_nb)))/1000+0.055
#' lines(x=(0:80), y=errr, lty=2, col="blue", lwd=2)
#' errr <- (abs((100*(p_aNB-p_nb)/p_nb)))/1000+0.055
#' lines(x=(0:80), y=errr, lty=3, col="purple", lwd=2)
#' errr <- (abs((100*(p_SA-p_nb)/p_nb)))/1000+0.055
#' lines(x=(0:80), y=errr, lty=4, col="green", lwd=2)
#' axis(2, at=seq(from=0, to=40, by=10)/1000+0.055, las=1, 
#'      labels=as.character(seq(from=0, to=40, by=10)))
#' mtext("|% error|", side = 2, adj=0.9, line=3)
#' legend(x=30, y=0.055, 
#'        legend=c("Exact", "Inversion of mgf", "Saddlepoint", "Normal", "Negative binomial"), 
#'        lty=c(1, 5, 4, 2, 3), bty="n", cex=0.8, col=c("black", "red", "green", "blue", "purple"), 
#'        lwd=c(1, 2, 2, 2, 2))
#' par(xpd=TRUE)
#' text(x=ScalePreviousPlot(x = 0.95, y = 0.1)$x, 
#'      y=ScalePreviousPlot(x = 0.95, y = 0.1)$y, labels="B", cex=2)
#'      
#' 
#' # dev.off()
#' 
#' 
#' # Test for criteria of convergence
#' Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
#'                    log=FALSE, verbose=TRUE)
#' Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
#'                  verbose=TRUE)
#' Pr_exact;as.numeric(Pr_Furman)
#' plot(1:length(attributes(Pr_Furman)$Pk), 
#'      log10(abs(attributes(Pr_Furman)$Pk-Pr_exact)), type="l", xlab="Iterations", 
#'      ylab="Abs log10", bty="n")
#' lines(1:(length(attributes(Pr_Furman)$Pk)-1), 
#'       log10(abs(diff(attributes(Pr_Furman)$Pk))), col="red")
#' legend("bottomleft", legend=c("Log10 Convergence to true value", "Log10 Rate of change"), 
#'        col=c("black", "red"), 
#'        lty=1)
#' 
#' 
#' # Test of different methods
#' alpha <- c(2.05, 2)
#' mu <- c(10, 30)
#' test <- rSnbinom(n=100000, size=alpha, mu=mu)
#' plot(0:200, table(test)[as.character(0:200)]/sum(table(test), na.rm=TRUE), 
#'      bty="n", type="h", xlab="x", ylab="Density")
#' lines(x=0:200, dSnbinom(0:200, size=alpha, mu=mu, log=FALSE, method="Furman"), col="blue")
#' lines(x=0:200, y=dSnbinom(0:200, size=alpha, mu=mu, log=FALSE, 
#'                           method="vellaisamy&upadhye"), col="red")
#' lines(x=0:200, y=dSnbinom(0:200, size=alpha, mu=mu, log=FALSE, 
#'                           method="approximate.randomobservations"), col="green")
#' 
#' # Test for criteria of convergence for x = 50
#' x <- 50
#' # Test for criteria of convergence
#' Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
#'                    log=FALSE, verbose=TRUE)
#' Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-12, 
#'                  verbose=TRUE)
#' Pr_exact;as.numeric(Pr_Furman)
#' plot(1:length(attributes(Pr_Furman)$Pk), 
#'      log10(abs(attributes(Pr_Furman)$Pk-Pr_exact)), type="l", xlab="Iterations", 
#'      ylab="Abs log10", bty="n")
#' lines(1:(length(attributes(Pr_Furman)$Pk)-1), 
#'       log10(abs(diff(attributes(Pr_Furman)$Pk))), col="red")
#' legend("bottomleft", legend=c("Log10 Convergence to true value", "Log10 Rate of change"), 
#'        col=c("black", "red"), 
#'        lty=1)
#' 
#' # Another example more complicated
#' set.seed(2)
#' mutest <- c(56, 6.75, 1)
#' ktest <- c(50, 50, 50)
#' nr <- 100000
#' test <- rSnbinom(nr, size=ktest, mu=mutest)
#' system.time({pr_vellaisamy <- dSnbinom(x=0:150, size=ktest, mu=mutest, 
#'                           method = "vellaisamy&upadhye", verbose=FALSE, parallel=FALSE)})
#' # Parallel computing is not efficient
#' system.time({pr_vellaisamy <- dSnbinom(x=0:150, size=ktest, mu=mutest, 
#'                           method = "vellaisamy&upadhye", verbose=FALSE, parallel=TRUE)})
#' system.time({pr_furman <- dSnbinom(x=0:150, size=ktest, mu=mutest, prob=NULL, 
#'                       method = "furman", verbose=FALSE, log=FALSE)})
#' pr_approximateObservations <- dSnbinom(0:150, size=ktest, mu=mutest, 
#'                                        method = "approximate.randomobservations")
#' 
#' plot(table(test), xlab="N", ylab="Density", las=1, bty="n", ylim=c(0, 4000), xlim=c(0, 150))
#' lines(0:150, pr_vellaisamy*nr, col="red")
#' lines(0:150, pr_furman*nr, col="blue")
#' lines(0:150, pr_approximateObservations*nr, col="green")
#' 
#' dSnbinom(x=42, size=ktest, mu=mutest, prob=NULL, 
#'          method = "vellaisamy&upadhye", verbose=TRUE)
#' as.numeric(dSnbinom(x=42, size=ktest, mu=mutest, prob=NULL, 
#'            method = "Furman", verbose=TRUE))
#' dSnbinom(x=42, size=ktest, mu=mutest, prob=NULL, 
#'          method = "approximate.randomobservations", verbose=TRUE)
#' 
#' x <- 100
#' # Test for criteria of convergence
#' Pr_exact <- dSnbinom(x=x, size=ktest, mu=mutest, method="vellaisamy&upadhye", 
#'                    log=FALSE, verbose=TRUE)
#' Pr_Furman <- dSnbinom(x=x, size=ktest, mu=mutest, method="Furman", log=FALSE, 
#'                  verbose=TRUE)
#' Pr_exact;as.numeric(Pr_Furman)
#' plot(1:length(attributes(Pr_Furman)$Pk), 
#'      log10(abs(attributes(Pr_Furman)$Pk-Pr_exact)), type="l", xlab="Iterations", 
#'      ylab="Abs log10", bty="n", ylim=c(-100, 0))
#' lines(1:(length(attributes(Pr_Furman)$Pk)-1), 
#'       log10(abs(diff(attributes(Pr_Furman)$Pk))), col="red")
#' legend("bottomright", legend=c("Log10 Convergence to true value", "Log10 Rate of change"), 
#'        col=c("black", "red"), 
#'        lty=1)
#'        
#'      
#' # example to fit a distribution
#' data <- rnbinom(1000, size=1, mu=10)
#' hist(data)
#' ag <- rep(1:100, 10)
#' r <- aggregate(data, by=list(ag), FUN=sum)
#' hist(r[,2])
#' 
#' parx <- c(size=1, mu=10)
#' 
#' dSnbinomx <- function(x, par) {
#'   -sum(dSnbinom(x=x[,2], mu=rep(par["mu"], 10), size=par["size"], log=TRUE))
#' }
#' 
#' fit_mu_size <- optim(par = parx, fn=dSnbinomx, x=r, method="BFGS", control=c(trace=TRUE))
#' fit_mu_size$par
#' 
#' alpha <- c(2.1, 2.05, 2)
#' mu <- c(10, 30, 20)
#' p <- pSnbinom(q=10, size=alpha, mu=mu, lower.tail = TRUE)
#' 
#' alpha <- c(2.1, 2.05, 2)
#' mu <- c(10, 30, 20)
#' q <- qSnbinom(p=0.1, size=alpha, mu=mu, lower.tail = TRUE)
#' 
#' alpha <- c(2.1, 2.05, 2)
#' mu <- c(10, 30, 20)
#' rep <- 100000
#' distEmpirique <- rSnbinom(n=rep, size=alpha, mu=mu)
#' tabledistEmpirique <- rep(0, 301)
#' names(tabledistEmpirique) <- as.character(0:300)
#' tabledistEmpirique[names(table(distEmpirique))] <- table(distEmpirique)/rep
#' 
#' plot(0:300, dSnbinom(0:300, size=alpha, mu=mu), type="h", bty="n", 
#'    xlab="x", ylab="Density", ylim=c(0,0.02))
#' plot_add(0:300, tabledistEmpirique, type="l", col="red")
#' legend(x=200, y=0.02, legend=c("Empirical", "Theoretical"), 
#'    text.col=c("red", "black"), bty="n")
#'    
#'    
#' # Test if saddlepoint approximation must be normalized
#' # Yes, it must be
#' n <- 7
#' alpha <- 1:n
#' p <- (1:n)/10
#' dSnbinom(x=10, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=TRUE)
#' dSnbinom(x=10, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=TRUE, normalize=FALSE)
#'                  
#' # Test for saddlepoint when x=0
#' n <- 7
#' alpha <- 1:n
#' p <- (1:n)/10
#' dSnbinom(x=0, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=TRUE)
#' dSnbinom(x=1, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=TRUE)
#' dSnbinom(x=c(0, 1), prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=TRUE)
#' dSnbinom(x=c(0, 1), prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=FALSE)
#'                  
#' # Test when prob are all the same
#' p <- rep(0.2, 7)
#' n <- 7
#' alpha <- 1:n
#' dSnbinom(x=0:10, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=TRUE)
#' dSnbinom(x=0:10, prob=p, size=alpha, method="furman", log=FALSE,  
#'                  verbose=TRUE)
#' dSnbinom(x=0:10, prob=p, size=alpha, method="exact", log=FALSE,  
#'                  verbose=TRUE)
#'                  
#' # Test when n=1
#' p <- 0.2
#' n <- 1
#' alpha <- 1:n
#' dSnbinom(x=0:10, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
#'                  verbose=TRUE)
#' dSnbinom(x=0:10, prob=p, size=alpha, method="furman", log=FALSE,  
#'                  verbose=TRUE)
#' dSnbinom(x=0:10, prob=p, size=alpha, method="exact", log=FALSE,  
#'                  verbose=TRUE)
#'
#' # Poisson distribution
#' dpois(20, lambda =sum(c(14, 10)))
#' dnbinom(20, size=+Inf, mu=sum(c(14, 10)))
#' dSnbinom(20, size=+Inf, mu=c(14, 10))
#' }
#' @export

#' @describeIn Snbinom Density for the sum of random variable with negative binomial distributions.
# #' @section Another section after function section:


dSnbinom <- function(x = stop("You must provide at least one x value")       , 
                     size = NULL                                             , 
                     prob = NULL                                             , 
                     mu = NULL                                               , 
                     log = FALSE                                             ,  
                     tol = NULL                                              , 
                     method="Furman"                                         ,
                     normalize=TRUE                                          ,
                     max.iter=NULL                                           ,
                     mean=NULL                                               ,
                     sd=NULL                                                 ,
                     n.random = 1E6                                          , 
                     parallel = FALSE                                        ,
                     verbose = FALSE                                         ) {
  
  method <- tolower(method)
  method <- match.arg(arg=method, choices = c("furman", 
                                              "vellaisamy&upadhye", "exact", 
                                              "approximate.randomobservations", 
                                              "approximate.normal", 
                                              "approximate.negativebinomial", 
                                              "saddlepoint"))
  
  # if (method == "convolution") method <- "furman"
  if (method == "exact") method <- "vellaisamy&upadhye"
  
  # prob=NULL; mu=NULL; log = FALSE
  if (is.null(mu) + is.null(size) + is.null(prob) != 1) stop("Two values exactly among mu, size and prob must be provided")
  
  m <- max(c(length(size), length(prob), length(mu)))
  if (!is.null(mu)) mu <- rep(mu, m)[1:m]
  if (!is.null(size)) size <- rep(size, m)[1:m]
  if (!is.null(prob)) prob <- rep(prob, m)[1:m]
  
  if (is.null(prob)) {
    prob <- size/(size+mu)
    prob <- ifelse(prob>1-(1e-9), 1-1e-6, prob)
  }
  
  if (is.null(mu)) mu <- size/prob - size
  if (is.null(size))  size  <- (prob * mu) / (1 - prob)
  
  if (all(size == +Inf)) {
    return(dpois(x, lambda =sum(mu), log=log))
  }
  
  if (all(prob == prob[1])) {
    if (method == "vellaisamy&upadhye") {
      return(dnbinom(x, size=sum(size), prob=prob[1], log=log))
    } else {
      if (verbose) message("Exact method could be used because all prob parameters are the same.")
    }
  }
  
  #SaddlePoint approximation####
  
  if (method == "saddlepoint") {
    
    K = function(t, phi, mu) {
      core = phi*(log(phi) - log(phi+mu*(1-exp(t))))
      Kt = sum(core)
      Kt
    }
    
    Kdash1 = function(t, phi, mu) {
      ## 1st derivative
      num = phi*mu*exp(t)
      den = phi + mu*(1-exp(t))
      core = num / den
      Kdash1t = sum(core)
      Kdash1t
    }
    
    
    Kdash2 = function(t, phi, mu) {
      # 2nd derivative
      num = phi*mu*(phi+mu)*exp(t)
      den = (phi + mu*(1-exp(t)))^2
      core = num / den
      Kdash2t = sum(core)
      Kdash2t
    }
    
    saddlefun = function(t, phis, mus, x) {
      ## Used by optimise
      f = abs(Kdash1(t, phis, mus) - x)^2
      f
    }
    
    
    dsaddle = function(x, size, mu, tol, verbose=FALSE) {
      ## Saddle point approximation to density
      if (x == 0) {
        if (verbose) message("Exact method is used for x = 0.")
        p <- sum(dnbinom(x=0, size=size, mu=mu, log=TRUE))
        p <- exp(p)
        return(p)
      }
      
      upper.bound = min(log(size/mu + 1))
      lower.bound <- -100
      repp <- 0
      repeat {
        ## I'm not sure how to set the lower bound in the general case
        arse = optimise(saddlefun, phis=size, mus=mu, x=x, 
                        lower=lower.bound, upper=upper.bound, 
                        tol=tol)
        repp <- repp + 1
        if ((arse$minimum == lower.bound) & repp < 10)
          lowerbound <- lowerbound / 10
        else break
      }
      
      if (verbose & (repp == 10)) warning("The lower bound reached its minimum; use results with caution.")
      
      sx = arse$minimum
      
      ## Generate the density now that we have xs
      numfx = exp(K(sx, size, mu) - sx*x)
      denfx = sqrt(2*pi*Kdash2(sx, size, mu))
      fx = numfx / denfx
      fx
    }
    
    if (verbose) message("Saddlepoint approximation method")
    if (is.null(tol)) {
      tol <- 1E-10
    }
    
    if (normalize & any(x != 0)) {
      if (verbose) message(paste0("Tolerance for normalization=", as.character(tol)))
      # Prepare normalization
      mean <- sum(size*(1-prob)/prob)
      sd <- sqrt(sum(size*(1-prob)/prob^2))
      Max <- max(c(floor(mean+20*sd), x))+1
      dstot <- sapply(X = 0:Max, FUN=function(y) dsaddle(x = y, 
                                                         size = size, 
                                                         mu = mu, tol=tol, verbose=FALSE))
      Max <- Max + 1
      repeat {
        ds <- dsaddle(x = Max, size = size, mu = mu, tol=tol, verbose=FALSE)
        dstot <- c(dstot, ds)
        if (ds < tol) break
        Max <- Max + 1
        if (verbose) message(paste0("Normalization: P(Sn=", as.character(Max), ")=", as.character(ds)))
      }
      
      if (verbose) message(paste0("Sum for normalization = ", as.character(sum(dstot))))
      ds <- dstot[x+1]/sum(dstot)
      
    } else {
      ds <- sapply(X = x, FUN=function(y) dsaddle(x = y, 
                                                  size = size, 
                                                  mu = mu, tol=tol, verbose=verbose))
      if (verbose & any(x != 0)) warning("Saddlepoint approximation was not normalized. Use this output with caution.")
    }
    if (!log) return(ds) else return(log(ds))
  }
  
  #Furman - Convolution####
  
  if (method == "furman") {
    if (verbose) message("Furman (2007) method by inversion of moment generating function")
    if (is.null(tol)) {
      tol <- min(dSnbinom(x=x, size = size, mu=mu, log=FALSE, 
                          normalize=FALSE, 
                          tol=1E-10, method="saddlepoint"))*1E-10
    }
    
    alpha <- size
    p <- prob
    
    q <- 1-p
    p1 <- max(p)
    q1 <- 1-p1
    
    # R <- sum(log(((q*p1)/(q1*p))^(-alpha)))
    R <- sum((-alpha)*log(((q*p1)/(q1*p))))
    
    delta <- 1
    xi <- NULL
    
    prePr <- rep(-Inf, length(x))
    # preDlt <- rep(-Inf, length(x))
    pr_tot <- NULL
    
    
    found <- rep(FALSE, length(x))
    Salpha <- sum(alpha)
    Prx_S <- dnbinom(x, size=Salpha, prob=p1)
    
    i <- 1
    repeat {
      # if (i == 545) stop()
      # xi <- c(xi, sum((alpha*(1-((q1*p)/(q*p1)))^i)/i))
      # xi <- c(xi, sum((alpha/i*((1-((q1*p)/(q*p1)))^i))/i))
      # 
      # (alpha*(1-((q1*p)/(q*p1)))^i)/i
      # exp(log((alpha*(1-((q1*p)/(q*p1)))^i)/i))
      # exp(log((alpha*(1-((q1*p)/(q*p1)))^i))-log(i))
      # 
      # exp((log(alpha)+i*log((1-((q1*p)/(q*p1)))))-log(i))
      # exp((log(alpha)+i*log((q*p1-q1*p)/(q*p1)))-log(i))
      # exp(log(alpha)+i*(log(q*p1-q1*p)-log(q*p1))-log(i))
      # New expression to prevent the use of ^
      xi <- c(xi, sum(exp(log(alpha)+i*(log(q*p1-q1*p)-log(q)-log(p1))-log(i))))
      
      Ks <- 1:i
      newdelta <- (1/i)*sum(Ks*xi[Ks]*delta[i-Ks+1])
      delta <- c(delta, newdelta)
      
      xf <- x[!found]
      
      Prx <- sapply(xf, function(S) {
        return(newdelta*dnbinom(S, size=Salpha+length(delta)-1, prob=p1))
      })
      Prx_S[!found] <- Prx_S[!found] + Prx
      Pr <- Prx_S
      
      if (log) {
        Pr <- R + log(Pr)
      } else {
        Pr <- exp(R) * Pr
      }
      
      pr_tot <- rbind(pr_tot, matrix(Pr, nrow=1))
      
      if (i>3) {
        if (is.null(max.iter)) {
          found <- found | ((abs(prePr - Pr) <= rate) & (all(abs(Pr - prePr) <= tol)))
          if (any(is.na(found))) break
          if (all(found)) break
        } else {
          if (i>=max.iter) break
        }
        if (verbose) {
          message(paste0("Iteration ", as.character(i), "; Max difference=", 
                         as.character(max(abs(Pr - prePr))), "; Trend of rate change ", 
                         ifelse((abs(prePr - Pr) <= rate)[which.max(abs(Pr - prePr))], "negative", "positive")))
        }
      }
      
      
      rate <- abs(prePr - Pr)
      prePr <- Pr
      # preDlt <- delta[i+1]
      
      i <- i + 1
    }
    
    if (any(is.na(found))) {
      warning("Error during recursions.")
      return() 
    }
    
    if (verbose) message("Loop has been stopped after ", as.character(i), " iterations.")
    
    # attributes(Pr) <- modifyList(as.list(attributes(Pr)), list(delta=delta, xi=xi, Pr=pr_tot))
    if (verbose) attributes(Pr) <- modifyList(as.list(attributes(Pr)), list("Pk"=pr_tot))
    return(Pr)
  } 
  
  #approximate.randomobservations####
  
  if (method == "approximate.randomobservations") {
    if (verbose) message("Approximate method with probabilities of observations.")
    test <- rSnbinom(n=n.random, size = size, mu=mu)
    return(sapply(X = x, FUN=function(y) sum(test==y)/n.random))
  }
  
  #approximate.negativebinomial####
  
  if (method == "approximate.negativebinomial") {
    if (verbose) message("Approximate method with negative binomial distribution.")
    
    if (is.null(mean)) {
      mean <- sum(size*(1-prob)/prob)
    }
    
    if (is.null(sd)) {
      sd <- sqrt(sum(size*(1-prob)/prob^2))
    }
    
    size <- mean^2 / (sd^2 - mean)
    
    if (verbose) message(paste0("Mean=", specify_decimal(mean)))
    if (verbose) message(paste0("Standard deviation=", specify_decimal(sd)))
    
    p <- dnbinom(x = x, mu=mean, size=size, log=log)
    
    if (verbose) attributes(p) <- modifyList(as.list(attributes(p)), list(mean=mean, sd=sd))
    return(p)
  }
  
  #approximate.normal####
  
  if (method == "approximate.normal") {
    if (verbose) message("Approximate method with normal distribution.")
    
    if (is.null(mean)) {
      mean <- sum(size*(1-prob)/prob)
    }
    if (is.null(sd)) {
      sd <- sqrt(sum(size*(1-prob)/prob^2))
    }
    
    if (verbose) message(paste0("Mean=", specify_decimal(mean)))
    if (verbose) message(paste0("Standard deviation=", specify_decimal(sd)))
    # if (verbose) message(paste0("Index of quality of approximation=", specify_decimal(pnorm(q=0, mean=mean, sd=sd, lower.tail = FALSE, log.p = FALSE))))
    
    p <- pnorm(x+0.5, mean=mean, sd=sd, log.p=FALSE) - pnorm(x-0.5,  mean=mean, sd=sd, log.p=FALSE)
    if (log) p <- log(p)
    
    if (verbose) attributes(p) <- modifyList(as.list(attributes(p)), list(mean=mean, sd=sd))
    return(p)
  }
  
  #Exact####
  
  if (verbose) message("Exact Vellaisamy & Upadhye (2009) method")
  if ((m > 7) & verbose) message("The Vellaisamy method with more than 7 summed distributions can be slow and produces out of memory error.")
  
  
  TS <- NULL
  
  for (xec in x) {
    
    # créer df avec xec balls dans m boxes
    
    Nballs <- xec
    # Number of boxes
    Nboxes <- m
    
    # The number of different ways to distribute n indistinguishable balls into
    # k distinguishable boxes is C(n+k-1,k-1).
    # nb<-choose(N+nbjour-1,nbjour-1)=dim(tb)[1]
    # divers<-matrix(rep(0, nbjour*nb), ncol=nbjour)
    
    # generate all possible positions of the boundaries
    xx <- combn(Nballs+Nboxes-1, Nboxes-1)
    # compute the number of balls in each box
    a <- cbind(0, diag(Nboxes)) - cbind(diag(Nboxes), 0)
    df <- t(a %*% rbind(0, xx, Nballs+Nboxes) - 1)
    
    if (verbose) {
      message(paste0(as.character(nrow(df)), " combinations of ", as.character(xec), " objects in ", as.character(m), " categories. The number of required iterations will be ", as.character(nrow(df)*m), "."))
    }
    
    if (parallel) {
      if (verbose) {
        message(paste0("Parallel computing with ", as.character(getOption("mc.cores", parallel::detectCores())), " cores."))
      }
      suppressMessages(L <- universalmclapply(1:nrow(df), FUN = function(rw) {
        n <- df[rw, , drop=TRUE]
        sum(sapply(1:m, FUN = function(j) dnbinom(x=n[j], size = size[j], mu=mu[j], log=TRUE)))
      }, mc.cores = getOption("mc.cores", parallel::detectCores()), 
      clusterExport=list(varlist=c("df", "size", "mu"), envir=environment())))
      L <- unlist(L)
    } else {
      L <- sapply(1:nrow(df), FUN = function(rw) {
        n <- df[rw, , drop=TRUE]
        sum(sapply(1:m, FUN = function(j) dnbinom(x=n[j], size = size[j], mu=mu[j], log=TRUE)))
      }
      )
    }
    
    
    
    TS <- c(TS, sum(exp(L)))
  }
  TS <- unname(TS)
  if (log) TS <- log(TS)
  return(TS)
  
}

#' @export
#' @describeIn Snbinom Distribution function for the sum of random variable with negative binomial distributions.

pSnbinom <- function(q=stop("At least one quantile must be provided"), 
                     size=NULL, 
                     prob=NULL, mu=NULL, lower.tail = TRUE, log.p = FALSE, tol=NULL, 
                     method="Furman", normalize=TRUE) {
  
  method <- tolower(method)
  method <- match.arg(arg=method, choices = c("furman", 
                                              "vellaisamy&upadhye", "exact", 
                                              "approximate.randomobservations", 
                                              "approximate.normal", 
                                              "approximate.negativebinomial", 
                                              "saddlepoint"))
  
  # if (method == "convolution") method <- "furman"
  if (method == "exact") method <- "vellaisamy&upadhye"
  
  # prob=NULL; mu=NULL; log = FALSE; infinite=10
  
  if (is.null(mu) + is.null(size) + is.null(prob) != 1) stop("Two values among mu, size and prob must be provided")
  
  m <- max(c(length(size), length(prob), length(mu)))
  if (!is.null(mu)) mu <- rep(mu, m)[1:m]
  if (!is.null(size)) size <- rep(size, m)[1:m]
  if (!is.null(prob)) prob <- rep(prob, m)[1:m]
  
  if (is.null(prob)) prob <- size/(size+mu)
  if (is.null(mu)) mu <- size/prob - size
  if (is.null(size))  size  <- (prob * mu) / (1 - prob)
  
  #  if (length(prob)<length(size)) prob <- rep(prob, length(size))[1:length(size)]
  #  if (length(size)<length(prob)) size <- rep(size, length(prob))[1:length(prob)]
  
  pp <- vapply(q, FUN=function(qq) {
    
    l <- dSnbinom(0:qq, prob=prob, size=size, mu=NULL, log=FALSE, 
                  tol=tol, method = method, normalize=normalize)
    p <- sum(l)
    if (!lower.tail) p <- 1-p
    if (log.p) p <- log(p)
    
    return(p)
  }, FUN.VALUE = 10.1)
  
  return(pp)
}

#' @export
#' @describeIn Snbinom Quantile function for the sum of random variable with negative binomial distributions.

qSnbinom <- function(p=stop("At least one probability must be provided"), 
                     size=stop("size parameter is mandatory"), 
                     prob=NULL, mu=NULL, lower.tail = TRUE, log.p = FALSE, 
                     tol=NULL, method="Furman") {
  
  # prob=NULL; mu=NULL; log = FALSE; infinite=10
  
  method <- tolower(method)
  method <- match.arg(arg=method, choices = c("furman", 
                                              "vellaisamy&upadhye", "exact", 
                                              "approximate.randomobservations", 
                                              "approximate.normal", 
                                              "approximate.negativebinomial", 
                                              "saddlepoint"))
  
 # if (method == "convolution") method <- "furman"
  if (method == "exact") method <- "vellaisamy&upadhye"
  
  if (is.null(mu) + is.null(size) + is.null(prob) != 1) stop("Two values among mu, size and prob must be provided")
  
  m <- max(c(length(size), length(prob), length(mu)))
  if (!is.null(mu)) mu <- rep(mu, m)[1:m]
  if (!is.null(size)) size <- rep(size, m)[1:m]
  if (!is.null(prob)) prob <- rep(prob, m)[1:m]
  
  if (is.null(prob)) prob <- size/(size+mu)
  if (is.null(mu)) mu <- size/prob - size
  if (is.null(size))  size  <- (prob * mu) / (1 - prob)  
  #  if (length(prob)<length(size)) prob <- rep(prob, length(size))[1:length(size)]
  #  if (length(size)<length(prob)) size <- rep(size, length(prob))[1:length(prob)]
  
  if (log.p) p <- exp(p)
  if (!lower.tail) p <- 1-p
  
  
  
  
  p1 <- max(p)
  
  rankl <- 10
  limit <- 10
  repeat {
    if (pSnbinom(limit, prob=prob, size=size, mu=NULL, log.p=FALSE, tol=tol, method = method)>=p1) break
    limit <- limit + rankl
    rankl <- rankl + 5
  }
  
  seqp <- pSnbinom(0:limit, prob=prob, size=size, mu=NULL, log.p=FALSE, tol=tol, method = method)
  
  qq <- vapply(p, FUN=function(pp) {
    
    return(which(seqp>pp)[1]-1)
  }, FUN.VALUE = 1.1)
  
  return(qq)
}

#' @export
#' @describeIn Snbinom Random numbers for the sum of random variable with negative binomial distributions.

rSnbinom <- function(n=1, 
                     size=NULL, 
                     prob=NULL, 
                     mu=NULL) {
  
  # prob=NULL; mu=NULL; log = FALSE; infinite=10
  
  if (is.null(mu) + is.null(size) + is.null(prob) != 1) stop("Two values among mu, size and prob must be provided")
  
  m <- max(c(length(size), length(prob), length(mu)))
  if (!is.null(mu)) mu <- rep(mu, m)[1:m]
  if (!is.null(size)) size <- rep(size, m)[1:m]
  if (!is.null(prob)) prob <- rep(prob, m)[1:m]
  
  if (is.null(prob)) prob <- size/(size+mu)
  if (is.null(mu)) mu <- size/prob - size
  if (is.null(size))  size  <- (prob * mu) / (1 - prob)  
  #  if (length(prob)<length(size)) prob <- rep(prob, length(size))[1:length(size)]
  #  if (length(size)<length(prob)) size <- rep(size, length(prob))[1:length(prob)]
  
  m <- matrix(1:m, nrow=1)
  
  rl <- apply(m, MARGIN=2, function(i) rnbinom(n, size=size[i], prob=prob[i]))
  
  if (inherits(rl, "integer")) rl <- as.data.frame(matrix(rl, nrow=1))
  
  return(apply(rl, MARGIN=1, sum))
}

#' +.PriorsmcmcComposite is a general function to set priors for MHalgoGen()
#' @title Set priors for MHalgoGen()
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Return a data.frame with the formatted priors
#' @param prior1 The first prior.
#' @param prior2 The second prior.
#' @description Construct priors for MHalgoGen()\cr
#' A check is done to verify that parameter names are not duplicated.
#' @family mcmcComposite functions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' Priors <- setPriors1(Name="a0", Prior1=0, Prior2=1, Density="dunif", 
#'                      SDProp=0.1, Min=0, Max=1, Init=0.5)
#' Priors <- Priors + setPriors1(Name="a1", Prior1=0.5, Prior2=0.1, Density="dnorm", 
#'                      SDProp=0.05, Min=0, Max=1, Init=0.5)
#' # Using Parameters
#' Priors <- setPriors1(Name="a0", Parameters=c(min=0, max=1), Density="dunif", 
#'                      SDProp=0.1, Min=0, Max=1, Init=0.5)
#' Priors <- Priors + setPriors1(Name="a1", Parameters=c(mean=0.5, sd=0.1), Density="dnorm", 
#'                      SDProp=0.05, Min=0, Max=1, Init=0.5)
#' }
#' @export

`+.PriorsmcmcComposite` <- function(prior1, prior2) {
  
  if (!inherits(prior2, "PriorsmcmcComposite")) {
    stop("Both objects must be of class 'PriorsmcmcComposite' generated using setPrior1().")
  }
  
  # Optional: check same columns
   # if (!all(names(prior1) == names(prior2))) {
   #   if (warn) warning("Priors of same names are replaced.")
   # }
  
  # if (any(duplicated(c(rownames(prior1), rownames(prior2))))) {
  #   if (warn) warning("Priors of same names are replaced.")
  # }
  
  if (all(colnames(prior2) != "Prior3")) {
    prior2 <- cbind(prior2[, c("Density", "Prior1", "Prior2")], Prior3=NA, prior2[, c("SDProp", "Min", "Max", "Init")])
  }
  
  if (all(colnames(prior1) != "Prior3")) {
    prior1 <- cbind(prior1[, c("Density", "Prior1", "Prior2")], Prior3=NA, prior1[, c("SDProp", "Min", "Max", "Init")])
  }
  
  prior1 <- prior1[!(rownames(prior1) %in% rownames(prior2)), ]
  
  res <- rbind(prior1, prior2)
  class(res) <- c("PriorsmcmcComposite", "data.frame")  # keep class
  
  return(res)
}

#' ind_long_lat is used to manage ncdf information
#' @title Return or the index in ncdf object from lat/longitude or inverse
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Or the index in ncdf object from lat/longitude or inverse
#' @param ncdf An object read from package ncdf4, ncdf or RNetCDF
#' @param long Longitude in decimal format
#' @param lat	Latitude in decimal format
#' @param indice.long	Index of longitude
#' @param indice.lat	Index of latitude
#' @param label.longitude Name of argument for longitude, default is lon
#' @param label.latitude Name of argument for latitude, default is lat
#' @description Return or the index in ncdf object from lat/longitude or reverse.
#' @family ncdf
#' @examples
#' \dontrun{
#' url <- "https://downloads.psl.noaa.gov/Datasets/noaa.oisst.v2.highres/"
#' url <- paste0(url, "sst.day.mean.2012.v2.nc")
#' dest <- paste(Sys.getenv("HOME"), "/sst.day.mean.2012.v2.nc", sep="")
#' download.file(url, dest)
#' library("ncdf4")
#' dta2012 <- nc_open(dest)
#' indices <- ind_long_lat(ncdf=dta2012, lat=5.89, long=-20.56)
#' coordinates <- ind_long_lat(ncdf=dta2012, indice.lat=20, indice.long=30)
#' # library("RNetCDF")
#' # dta2012 <- open.nc(dest)
#' # indices <- ind_long_lat(ncdf=dta2012, lat=5.89, long=-20.56)
#' # coordinates <- ind_long_lat(ncdf=dta2012, indice.lat=20, indice.long=30)
#' # ncdf library is depreciated in CRAN
#' # library("ncdf")
#' # dta2012 <- open.ncdf(dest)
#' # indices <- ind_long_lat(ncdf=dta2012, lat=5.89, long=-20.56)
#' # coordinates <- ind_long_lat(ncdf=dta2012, indice.lat=20, indice.long=30)
#' }
#' @export


ind_long_lat <- function (ncdf = stop("The ncdf data must be supplied") , 
                          long = NULL                                   , 
                          lat = NULL                                    , 
                          indice.long = NULL                            , 
                          indice.lat = NULL                             , 
                          label.longitude = "lon"                       , 
                          label.latitude = "lat"                        ) 
{
  maxindicelt <- NULL
  maxindicelg <- NULL
  # if (inherits(ncdf, "ncdf4")) {
  #   if (!requireNamespace("ncdf", quietly = TRUE)) {
  #     stop("ncdf package is necessary for this function")
  #   }
  #   maxindicelt <- ncdf$dim[[label.latitude]]$len
  #   lt <- ncdf$dim[[label.latitude]]$vals
  #   lg <- ncdf$dim[[label.longitude]]$vals
  #   maxlt <- lt[maxindicelt]
  #   minlt <- lt[1]
  #   maxindicelg <- ncdf$dim[[label.longitude]]$len
  #   maxlg <- lg[maxindicelg]
  #   minlg <- lg[1]
  # }
  if (inherits(ncdf, "ncdf4")) {
    if (!requireNamespace("ncdf4", quietly = TRUE)) {
      stop("ncdf4 package is necessary for this function")
    }
    maxindicelt <- ncdf$dim[[label.latitude]]$len
    lt <- ncdf$dim[[label.latitude]]$vals
    maxlt <- lt[maxindicelt]
    minlt <- lt[1]
    maxindicelg <- ncdf$dim[[label.longitude]]$len
    lg <- ncdf$dim[[label.longitude]]$vals
    maxlg <- lg[maxindicelg]
    minlg <- lg[1]
  }
  if (inherits(ncdf, "NetCDF")) {
    if (!requireNamespace("RNetCDF", quietly = TRUE)) {
      stop("RNetCDF package is necessary for this function")
    }
    maxindicelt <- getFromNamespace("dim.inq.nc", ns = "RNetCDF")(ncfile = ncdf, 
                                                                  label.latitude)$length
    lt <- getFromNamespace("var.get.nc", ns = "RNetCDF")(ncfile = ncdf, variable = label.latitude)
    maxlt <- lt[maxindicelt]
    minlt <- lt[1]
    maxindicelg <- getFromNamespace("dim.inq.nc", ns = "RNetCDF")(ncfile = ncdf, 
                                                                  label.longitude)$length
    lg <- getFromNamespace("var.get.nc", ns = "RNetCDF")(ncfile = ncdf, 
                                                         variable = label.longitude)
    maxlg <- lg[maxindicelg]
    minlg <- lg[1]
  }
  if (is.null(maxindicelt) | is.null(maxindicelg)) {
    stop("Check the ncdf data; it is not recognized")
  }
  if (!is.null(long) & !is.null(lat)) {
    # which.min(abs(lt - lat))
    if ((long > maxlg) | (long < minlg)) long <- long%%360
    if ((lat > maxlt) | (lat < minlt)) lat <- ((lat + 90)%%180) - 90
    return(c(indice.long = which.min(abs(lg - long)), indice.lat = which.min(abs(lt - lat))))
  } else {
    if (!is.null(indice.long) & !is.null(indice.lat)) {
      return(c(long = lg[indice.long], lat = lt[indice.lat]))
    }
    else {
      stop("Check the parameters")
    }
  }
}

#' moon.info calculates the moon phase based on a date.
#' @title Moon phase based on a date
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Return a value describing the moon phase:\cr
#' 0 and 100 are full moon, 50 is new moon, 25 last quarter and 75 first quater
#' @param date A date in class Date. By default, it will use today date
#' @param phase If TRUE, a vector of characters with NM, FQ, FL LQ will be returned
#' @family Periodic patterns of indices
#' @description The script gives an index (base 100) that represents moon phase.\cr
#' If the return value (from 0 to 100) is between:\cr
#' 0 and 1.6931595 or 98.3068405 and 100, it is full moon,\cr
#' 23.3068405 and 26.6931595, last quarter,\cr
#' 48.3068405 and 51.6931595, new moon,\cr	
#' 73.3068405 and 76.6931595, first quarter\cr
#' When phase is set to TRUE, a character representing the moon phase is returned.
#' @keywords Moon Lunar Lune
#' @examples 
#' \dontrun{
#' library("HelpersMG")
#' moon.info(as.Date("2001-12-31"))
#' moon.info(as.Date("14/04/2010", "%d/%m/%Y"))
#' moon.info(as.Date("22/06/07", "%d/%m/%y"))
#' moon.info(seq(from=as.Date("2012-03-01"), 
#'		to=as.Date("2012-04-15"), by="days"))
#' moon.info(seq(from=as.Date("2012-03-01"), 
#' 		to=as.Date("2012-04-15"), by="days"), phase=TRUE)
#' }
#' @export

moon.info<- function(date=Sys.Date(), phase=FALSE) {

XRef<-as.Date("2012-03-07")

nb<-as.numeric(date-XRef)

moon<-nb%%29.530589

moon<-100*(moon/29.530589)

if (phase) {

moonT<-rep(NA, length(moon))
moonT[which((moon>98.3068405) | (moon<1.6931595))]<-"FM"
moonT[which((moon>48.3068405) & (moon<51.6931595))]<-"NM"
moonT[which((moon>23.3068405) & (moon<26.6931595))]<-"LQ"
moonT[which((moon>73.3068405) & (moon<76.6931595))]<-"FQ"

return(moonT)

} else {

return(moon)

}
}

#' RM_add adds a results managment or a value in results managment to an object
#' @title Create a results managment or add a value in a results managment to an object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The original object with a new value in a results managment object or a new results managment
#' @param x The object to add a results managment or a result in a results managment
#' @param RM The name of results managment stored
#' @param RMname The name of the results managment to be modified or created
#' @param valuename The name of the new value to be added
#' @param value The value to be added
#' @description Return original object with a new value or a new results managment.
#' @family Results Managment
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Let an object of class objclass being created
#' obj <- list(A=100, name="My object")
#' class(obj) <- "objclass"
#' # And now I create a RM to this object
#' obj <- RM_add(x=obj, RMname="NewAnalysis1")
#' RM_list(obj)
#' obj <- RM_add(x=obj, RMname="NewAnalysis2")
#' RM_list(obj)
#' obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V1", value=100)
#' RM_get(x=obj, RMname="NewAnalysis2", valuename="V1")
#' obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V1", value=200)
#' RM_get(x=obj, RMname="NewAnalysis2", valuename="V1")
#' obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V2", value=300)
#' RM_get(x=obj, RMname="NewAnalysis2", valuename="V2")
#' RM_list(obj)
#' }
#' @export

RM_add <- function(x=stop("An object with results managment must be provided"), 
                   RM="RM", 
                   RMname=stop("A results managment name must be provided"), 
                   valuename=NULL, 
                   value=NULL) {
  xa <- attributes(x)
  # dans rx j'ai le RM
  rx <- xa[[RM]]
  ix <- rx
  
  if (is.null(rx)) {
    # Il n'y avait pas d'object RM
    ix <- list(list(name=RMname, timestamp=timestamp(quiet = TRUE)))
    names(ix) <- RMname
  } else {
    # il y en avait un
    # Je le crée s'il n'existait pas
    if (!any(names(rx) %in% RMname)) {
      ix <- list(list(name=RMname, timestamp=timestamp(quiet = TRUE)))
      names(ix) <- RMname
      ix <- c(rx, ix)
    }
  }
  
  # Est ce que je dois ajouter une valeur
  if ((!is.null(valuename)) & !is.null(value)) {
    if (!any(names(ix[[RMname]]) %in% valuename)) {
      # Le nom n'existait pas
      newvalue <- list(value)
      names(newvalue) <- valuename
      ix[RMname] <- list(modifyList(ix[[RMname]], newvalue))
    } else {
      ix[[RMname]][[valuename]] <- value  
    }
  }
  
  rx <- list(RM=ix)
  names(rx) <- RM
  
  xa[RM] <- rx
  
  attributes(x) <- xa
  return(x)
}

.getint <- function (cf, parm, ses, level = 0.95, df = NULL, ...) 
{
  a <- (1 - level)/2
  a <- c(a, 1 - a)
  if (is.null(df)) 
  fac <- qnorm(a)
  else fac <- qt(a, df)
  fac <- c(0, fac)
  fff = format(100 * a, 3)
  pct = paste(fff, "%")
  pct = c("value", pct)
  ci <- array(NA, dim = c(length(parm), 3L), dimnames = list(parm, 
  pct))
  ci[] <- cf + ses %o% fac
  ci
}

# .smartsub <- function (pat, repl, x) 
# {
#   args <- lapply(as.list(match.call())[-1L], eval, parent.frame())
#   names <- if (is.null(names(args))) 
#   character(length(args))
#   else names(args)
#   dovec <- names %in% vectorize.args
#   do.call("mapply", c(FUN = FUN, args[dovec], MoreArgs = list(args[!dovec]), 
#   SIMPLIFY = SIMPLIFY, USE.NAMES = USE.NAMES))
# }


.nlConfint <- function (obj = NULL, texts, level = 0.95, coeff = NULL, Vcov = NULL, 
          df2 = NULL, x = NULL, silent=FALSE) 
{
  
  # obj = NULL; texts = ""; level = 0.95; coeff = NULL; Vcov = NULL; df2 = NULL; x = NULL; silent=FALSE
  if (!is.null(obj)) {
    co = try(coef(obj), silent = TRUE)
    cond = attr(co, "condition")
    if (is.null(coeff) && (is.null(cond))) 
      coeff = co
    vc = try(vcov(obj), silent = TRUE)
    cond2 = attr(vc, "condition")
    if (is.null(Vcov) && (is.null(cond2))) 
      Vcov = vc
  }
  if (is.null(coeff)) {
    if (is.null(obj)) 
      mess = "Both  'obj' and 'coeff' are missing"
    else {
      clm = class(obj)
      part1 = "There are no coef() methods for model objects of class \""
      mess = paste0(part1, clm, "\".\nInput the 'coeff' parameter.")
    }
    stop(mess)
  }
  if (is.null(Vcov)) {
    if (is.null(obj)) 
      mess = "Both  'obj' and 'Vcov' are missing"
    else {
      clm = class(obj)
      part1 = "There are no vcov() methods for model objects of class \""
      mess = paste0(part1, clm, "\".\nInput the 'Vcov' parameter.")
    }
    stop(mess)
  }
  if (length(texts) > 1) kkk = texts[1] else kkk = strsplit(texts[1], ";")[[1]]
  kkkfl = as.formula(paste("~", kkk[1]))
  vvss = setdiff(all.vars(kkkfl), "x")
  # texts = getFromNamespace(".smartsub", ns="nlWaldTest")(vvss, "b", texts)
  if (length(texts) > 1) ltext0 = texts else ltext0 = strsplit(texts, ";")[[1]]
  texts1 = gsub("[", "", texts, fixed = TRUE)
  texts1 = gsub("]", "", texts1, fixed = TRUE)
  if (length(texts1) > 1) ltext = texts1 else ltext = strsplit(texts1, ";")[[1]]
  r = length(ltext)
  n = length(coeff)
  namess = paste0("b", 1:n)
  for (j in 1L:n) assign(namess[j], coeff[j])
  if (!is.null(x)) {
    nx = length(x)
    namesx = paste0("x", 1:nx)
    for (j in 1L:nx) assign(namesx[j], x[j])
  }
  grad = c()
  hess = c()
  for (i in 1L:r) {
    fli <- as.formula(paste("~", ltext[i]))
    z = try(deriv(as.formula(fli), namess), silent = T)
    if (inherits(z, "try-error")) {
      tei = as.character(i)
      tri2 = ", numerical derivatives were used in delta-method"
      wate = paste0("Note: For function ", i, tri2)
      if (!silent) message(wate)
      ez <- numericDeriv(expr=quote(eval(parse(text = ltext[i]))), 
                        theta=namess)
    }
    else ez = eval(z)
    hessj = attr(ez, "gradient")
    grad = rbind(grad, ez[1])
    hess = rbind(hess, hessj)
  }
  Rb = grad
  ddd = hess %*% Vcov %*% t(hess)
  matr = chol2inv(chol(ddd))
  ses = sqrt(diag(ddd))
  trydf = identical(df2, TRUE)
  if (trydf) {
    isdf = try(df.residual(obj), silent = TRUE)
    df2 = isdf
    if (is.null(df2)) {
      wn = "Note: Failed to extract df for denominator; z-intervals applied"
      if (!silent) message(wn)
    }
  }
  getFromNamespace(".getint", ns="HelpersMG")(as.numeric(Rb), ltext0, ses, level, df = df2)
}

#' logLik.LD50 Return Log Likelihood of a fit for LD50
#' @title Return Log Likelihood of a fit generated by LD50
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The Log Likelihood value for the fitted model with data
#' @param object A result file generated by fitRMU
#' @param ... Not used
#' @description Return Log Likelihood of a fit generated by LD50
#' @family LD50 functions
#' @examples
#' \dontrun{
#' data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
#' Alive=c(10, 12, 8, 6, 2, 1),
#' Dead=c(0, 1, 5, 6, 9, 15))
#' LD50_logistic <- LD50(data, equation="logistic")
#' logLik(LD50_logistic)
#' AIC(LD50_logistic)
#' }
#' @method logLik LD50
#' @export


logLik.LD50 <- function(object, ...) {
  l <- -object$value
  n <- sum(!is.na(as.numeric(as.matrix(object$data))))
  attributes(l) <- list(nall=n , nobs=n , df=length(object$par) , class="logLik")
  return(l)
}

#' duplicated_packages lists the duplicated packages with their locations
#' @title List the duplicated packages with their locations
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A data.frame with 4 elements for each duplicated packages:\cr
#' - versions: the version of the packages\cr
#' - libraries: the locations\cr
#' @description A data.frame with the duplicated packages and their locations and version.\cr
#' The columns Lib1 and Version1 should have the oldest version of the packages.
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' duplicated_packages()
#' # To remove the oldest versions of the installed packages, use
#' li <- duplicated_packages()
#' if (nrow(li) != 0)
#'     for (i in 1:nrow(li))
#'         remove.packages(rownames(li)[i], lib=li[i, "Lib1"])
#' }
#' @export


duplicated_packages <- function() {
  li <- matrix(character(0), ncol=4, 
               dimnames = list(NULL, 
                               c("Lib1", "Version1", "Lib2", "Version2")))
  lp <- list.packages()
  nli <- names(lp)
  if (length(nli) > 1) {
    for (i in 1:(length(nli)-1)) {
      for (j in (i+1):length(nli)) {
        
        d1 <- na.omit(match(rownames(lp[[j]]), rownames(lp[[i]])))
        # print(length(d1))
        if (length(d1) != 0) {
          for (k in seq_along(d1)) {
          m <- rownames(lp[[i]])[d1[k]]
          vi <- lp[[i]][d1[k], "Version"]
          vj <- lp[[j]][m, "Version"]
          if (compareVersion(vi, vj)==-1) {
          li <- rbind(li, matrix(c(nli[i], vi, nli[j], vj), ncol=4, 
                                 dimnames = list(m, 
                                                 c("Lib1", "Version1", "Lib2", "Version2"))))
          } else {
          li <- rbind(li, matrix(c(nli[j], vj, nli[i], vi), ncol=4, 
                                   dimnames = list(m, 
                                                   c("Lib1", "Version1", "Lib2", "Version2"))))
            
          }
          }
        }
      }
    }
  }
  return(li)
}

#' logit returns the logit
#' @title Return the logit
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A value
#' @param p The probability
#' @description Return the logit.
#' @family logit
#' @examples
#' n <- logit(0.5)
#' invlogit(n)
#' @export

logit <- function(p) {log(p/(1-p))}

#' NagelkerkeScaledR2 returns the scaled R2 defined by Nagelkerke (1991)
#' @title Return the scaled R2 defined by Nagelkerke (1991)
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The scaled R2 value
#' @param x The number of observations
#' @param size Number of trials
#' @param prediction Prediction of x/size
#' @param scaled If TRUE, return the scaled R2
#' @family Statistics
#' @description Return the scaled R2 of a binomial model based on:\cr
#' Nagelkerke NJD (1991) A note on a general definition of the coefficient 
#' of determination. Biometrika 78:691-192.\cr
#' This definition of scaled R2 by Nagelkerke (1991) has the following properties:\cr
#' (i) It is consistent with classical R2, that is the general definition applied to e.g. linear regression yields the classical R2.\cr
#' (ii) It is consistent with maximum likelihood as an estimation method, i.e. the maximum likelihood estimates of the model parameters maximize R2.\cr
#' (iii) It is asymptotically independent of the sample size n.\cr
#' (iv) 1-R2 has the interpretation of the proportion of unexplained 'variation'.\cr
#' (v) It is dimensionless, i.e. it does not depend on the units used.\cr
#' The reported value is similar to the value estimated with nagelkerke() function from rcompanion package but not
#' from the NagelkerkeR2() function from fmsb package. I don't know why.
#' @examples
#' x <- c(10, 9, 6, 4, 3, 1, 0)
#' size <- c(10, 10, 10, 10, 10, 10, 10)
#' prediction <- c(0.9, 0.8, 0.7, 0.5, 0.4, 0.3, 0.2)
#' NagelkerkeScaledR2(x, size, prediction)
#' 
#' # Using the example in fmsb::NagelkerkeR2
#' res <- glm(cbind(ncases,ncontrols) ~ agegp+alcgp+tobgp, data=esoph, family=binomial())
#' NagelkerkeScaledR2(x=esoph$ncases, size = esoph$ncases+esoph$ncontrols, 
#'                    prediction = res$fitted.values)
#' @export


NagelkerkeScaledR2 <- function(x, size, prediction, scaled=TRUE) {
  if (!((length(x) == length(size)) & (length(size) == length(prediction)))) 
    stop("Vectors x, size and prediction must be of same size")
  if (any(prediction > 1) | any(prediction < 0)) 
    stop("Predictions for a binomial distribution must be between 0 and 1.")
  L0 <- sum(dbinom(x, size, rep(sum(x)/sum(size), length(prediction)), 
                   log = TRUE))
  Lpar <- sum(dbinom(x, size, prediction, log = TRUE))
  n <- length(size)
  # r2 <- 1 - (exp(L0 - Lpar))^(2/n)
  # r2 <- 1 - exp(log((exp(L0 - Lpar))^(2/n)))
  # r2 <- 1 - exp((2/n)*log((exp(L0 - Lpar))))
  r2 <- 1 - exp((2/n)*(L0 - Lpar))
  
  # r2max <- 1 - (exp(L0))^(2/n)
  # r2max <- 1 - exp(log((exp(L0))^(2/n)))
  # r2max <- 1 - exp((2/n)*log((exp(L0))))
  r2max <- 1 - exp((2/n)*L0)
  
  if (!scaled) 
    r2max <- 1
  return(r2/r2max)
}

#' addS3Class add a S3 class to an object
#' @title Add a S3 class to an object. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The same object with the new class as first class
#' @param x The object to add class.
#' @param class The class to add.
#' @description Add a S3 class as first class to an object.
#' @examples
#' print.CF <- function(x) {cat("print.CF ", x)}
#' result <- "Je suis donc je pense"
#' result <- addS3Class(result, class="CF")
#' class(result)
#' print(result)
#' result <- addS3Class(result, class=c("ECF", "OCF"))
#' class(result)
#' print(result)
#' @export

addS3Class <- function(x, class=NULL) {
  class(x) <- c(class, class(x)[!(class(x) %in% class)])
  return(x)
}

#' @title Test if two vectors contains the same elements independently of their order
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A logical TRUE or FALSE
#' @param x A vector with numeric or character elements 
#' @param y A vector with numeric or character elements
#' @param test.names Logical. If TRUE, the names of the vector elements must be also identical and unique
#' @description Return TRUE only if all elements of x are present and only once in y.\cr
#' @examples
#' \dontrun{
#' A <- c("A", "B", "C", "D")
#' B <- c("A", "B", "C", "D")
#' similar(A, B)
#' similar(B, A)
#' A <- c(x="A", y="B", z="C", k="D")
#' B <- c(x="A", y="B", z="C", l="D")
#' similar(B, A)
#' similar(A, B, test.names=TRUE)
#' A <- c(x="A", y="B", z="C", k="D")
#' B <- c(x="A", z="C", k="D", y="B")
#' similar(B, A)
#' similar(A, B, test.names=TRUE)
#' }
#' @export


similar <- function(x, y, test.names=FALSE) {
	v1 <- (length(x)==length(y))
	v1 <- v1 & sum(match(x, y), na.rm = TRUE)==length(x)*(length(x)+1)/2
	if (test.names) 
	  v1 <- v1 & (
	    identical(match(x, y), match(names(x), names(y)))
	  )
	return(v1)
}

#' plot.IconoCorel checks and corrects the dataframe to be used with IC_threshold_matrix
#' @title Clean the dataframe before to be used with IC_threshold_matrix
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A igraph object
#' @param x The correlation matrix to show
#' @param show.legend.direction the position of the legend of direction; FALSE to not show it
#' @param show.legend.strength the position of the legend with intensity of correlation; FALSE to not show it
#' @param vertex.label.color a vector with the colors of labels
#' @param vertex.label a vector with the labels
#' @param vertex.color a vector of colors
#' @param vertex.label.cex a vector of cex
#' @param title the title of the plot
#' @param plot if TRUE, the plot is shown
#' @param ... other options of plot.igraph()
#' @description This function plots the data as a network. It returns an invisible object that can be used with visIgraph from package visNetwork.
#' \code{https://fr.wikipedia.org/wiki/Iconographie_des_corrélations}
#' @family Iconography of correlations
#' @references Lesty, M., 1999. Une nouvelle approche dans le choix des régresseurs de la régression multiple en présence d’interactions et de colinéarités. Revue de Modulad 22, 41-77.
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' es <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
#'                  Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
#'                  Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
#'                  Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
#'                  Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
#'                  row.names = c(NA, -8L), class = "data.frame")
#' 
#' es
#' 
#' df <- IC_clean_data(es, debug = TRUE)
#' cor_matrix <- IC_threshold_matrix(data=df, threshold = NULL, progress=FALSE)
#' cor_threshold <- IC_threshold_matrix(data=df, threshold = 0.3)
#' par(mar=c(1,1,1,1))
#' set.seed(4)
#' library("igraph")
#' library("visNetwork")
#' kk <- plot(cor_threshold, vertex.color="red")
#' # it can be shown also with the visNetwork package
#' visIgraph(kk)
#' cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
#' plot(cor_threshold_Note)
#' 
#' # You can record the position of elements and use them later
#' ly <- layout_nicely(kk)
#' plot(cor_threshold, vertex.color="red", layout=ly)
#' }
#' @method plot IconoCorel
#' @export


plot.IconoCorel <- function(x                                          , 
                            ...                                        , 
                            show.legend.direction="bottomright"        , 
                            show.legend.strength="topleft"             , 
                            title="Correlation iconography"            , 
                            vertex.label.color="black"                 , 
                            vertex.label=NULL                          , 
                            vertex.color="white"                       ,
                            vertex.label.cex=1                         ,
                            plot=TRUE                                  ) {
  
  if ((!is.element("igraph", installed.packages()[,1]))) {
    warning("Packages igraph is absent; the network cannot be shown.")
    plot <- FALSE
  }
  
  
  if (identical(show.legend.direction, TRUE)) show.legend.direction="bottomright"
  if (identical(show.legend.strength, TRUE)) show.legend.strength="topleft"
  
  threshold <- x$threshold
  matrix <- x$thresholded_correlation
  if (is.null(matrix)) {
    matrix <- x$correlation
    threshold <- "No threshold"
  }
  
  if (is.null(nrow(matrix)) | (nrow(matrix)==0)) {
    stop("No plot can be shown")
  }
  
  
  graph <- getFromNamespace("graph.adjacency", ns="igraph")(abs(matrix), weighted=TRUE, mode="lower")
  E <- getFromNamespace("E", ns="igraph")
  V <- getFromNamespace("V", ns="igraph")
  
  if (is.null(vertex.label)) {
    vertex.label <- colnames(matrix)
  }
  
  w <- getFromNamespace("edge_attr", ns="igraph")(graph, "weight", E(graph))
  if (length(w) <= 1) {
    ww <- 1
  } else {
    ww <- 1 + 4*(w-min(w))/(max(w)-min(w))
  }
  # E(g)[idx]$attr <- value is equivalent to g <- set_edge_attr(g, attr, E(g)[idx], value).
    graph <- getFromNamespace("set_edge_attr", ns="igraph")(graph, "width", E(graph), ww)
  graph <- getFromNamespace("set_edge_attr", ns="igraph")(graph, "color", E(graph), sapply(strsplit(attributes(E(graph))$vnames, "\\|"), 
                                                                                           FUN=function(k) ifelse(matrix[k[1], k[2]]>0, "blue", "red")))
  graph <- getFromNamespace("set_edge_attr", ns="igraph")(graph, "lty", E(graph), sapply(strsplit(attributes(E(graph))$vnames, "\\|"), 
                                                                                         FUN=function(k) ifelse(matrix[k[1], k[2]]>0, 1, 2)))
  
  # The assignment form of $ is a shortcut for set_vertex_attr, e.g. V(g)[idx]$attr <- value is equivalent to g <- set_vertex_attr(g, attr, V(g)[idx], value).
  
  graph <- getFromNamespace("set_vertex_attr", ns="igraph")(graph, "label", V(graph), vertex.label)
  graph <- getFromNamespace("set_vertex_attr", ns="igraph")(graph, "color", V(graph), vertex.color)
  graph <- getFromNamespace("set_vertex_attr", ns="igraph")(graph, "label.color", V(graph), vertex.label.color)
  graph <- getFromNamespace("set_vertex_attr", ns="igraph")(graph, "label.cex", V(graph), vertex.label.cex)
  
  
  
  # par(mar=c(2, 2, 4, 2))
  # set.seed(4)
  if (plot) {
    
    getFromNamespace("plot.igraph", ns="igraph")(graph, 
                                                 edge.width = ww, 
                                                 edge.color = sapply(strsplit(attributes(E(graph))$vnames, "\\|"), 
                                                                     FUN=function(k) ifelse(matrix[k[1], k[2]]>0, "blue", "red")), 
                                                 vertex.label = vertex.label, 
                                                 vertex.color = vertex.color, 
                                                 vertex.size = 1,
                                                 edge.lty = sapply(strsplit(attributes(E(graph))$vnames, "\\|"), 
                                                                   FUN=function(k) ifelse(matrix[k[1], k[2]]>0, 1, 2)), 
                                                 vertex.frame.color = NA, 
                                                 vertex.label.color = vertex.label.color, 
                                                 margin=c(0, 0, 0, 0)
                                                 , ...)
    
    title(main = title)
    if (!identical(show.legend.direction, FALSE)) {
      legend(show.legend.direction, legend=c("Positive relationship", "Negative relationship"), 
             col=c("blue", "red"), lty=c(1, 2))
    }
    
    if (!identical(show.legend.strength, FALSE)) {
      if (length(w) <= 1) {
        legend(show.legend.strength, title="Correlation coefficient",
               legend=specify_decimal(w), 
               lty=1, 
               lwd=1)
      } else {
        ml <- round(max(w)+0.1, 1)
        if (ml>1) ml <- 1
        legend(show.legend.strength, title="Correlation coefficient",
               legend=as.character(seq(from=round(threshold, 1), to=ml, by=0.1)), 
               lty=1, 
               lwd=1+4*(seq(from=round(threshold, 1), to=ml, by=0.1)-min(w))/
                 (max(w)-min(w)))
      }
    }
  }
  
  return(invisible(graph))
}

#' plot.LD50 plot result of LD50() that best describe LD50
#' @title Plot results of LD50() that best describe LD50
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @param x A result file generated by LD50()
#' @param ... Parameters for plot()
#' @param las.x las parameter for x axis
#' @param las.y las parameter for y axis
#' @param at Position of ticks in x-axis
#' @param lab.PT Label to describe pivotal dose
#' @param lab.TRD Label to describe transitional range of dose
#' @param col.TRD The color of TRD
#' @param col.TRD.CI The color of CI of TRD based on range.CI
#' @param col.PT.CI The color of CI of PT based on range.CI
#' @param show.CI Do the CI for the curve should be shown
#' @description Plot the estimates that best describe lethality of doses.
#' @family LD50 functions
#' @examples
#' \dontrun{
#' data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
#' Alive=c(10, 12, 8, 6, 2, 1),
#' Dead=c(0, 1, 5, 6, 9, 15))
#' LD50_logistic <- LD50(data, equation="logistic")
#' predict(LD50_logistic, doses=c(140, 170))
#' plot(LD50_logistic, xlim=c(0, 300))
#' }
#' @method plot LD50
#' @export



plot.LD50 <- function(x, ...,  
	las.x=1, las.y=1, lab.PT="LD50", at=NULL, 
	lab.TRD=paste0("Transitional range of doses l=",l*100,"%"), 
	col.TRD="gray", col.TRD.CI=rgb(0.8, 0.8, 0.8, 0.5), 
  col.PT.CI=rgb(0.8, 0.8, 0.8, 0.5), show.CI=TRUE) {


  range.CI.qnorm <- qnorm(1-((1-x$range.CI)/2))
  l <- x$l
  alive <- x$alive
  dead <- x$dead
  N <- x$N
  doses <- x$doses
  equation <- x$equation  
  par <- x$par
  res <- x$SE
  parP <- x$LD50
  resP <- x$SE_LD50
  
  result <- x
 
 
   L <- list(...)

		L1 <- modifyList(list(x=doses, y=alive/N, bty="n", type="n", xlab="Doses", 
		                      ylab="Alive proportion"), L)
  L1 <- modifyList(L1, list(ylim=c(0,1), xaxt="n", las=las.y))
  
  if (is.null(L$xlim)) {
    L1 <- modifyList(L1, list(xlim=c(floor(min(doses)), floor(1+max(doses)))))
  }
  
  a <- do.call(plot, L1) 
  
  x2 <- (par("usr")[1]+par("usr")[2]*26)/27
  x1 <- x2*26-par("usr")[2]/0.04
  
  axis(1, at=at, las=las.x)
  
  # je trace la TRD centree sur P
  
  
  polygon(c(result$TRD_limits[1], result$TRD_limits[1], result$TRD_limits[2], result$TRD_limits[2]), c(0,1,1,0), border=NA, col=col.TRD)  
  # limites de la limite basse de la TRD
  polygon(c(result$TRD_limits[1]+range.CI.qnorm*result$SE_TRD_limits[1], result$TRD_limits[1]+range.CI.qnorm*result$SE_TRD_limits[1], result$TRD_limits[1]-range.CI.qnorm*result$SE_TRD_limits[1], result$TRD_limits[1]-range.CI.qnorm*result$SE_TRD_limits[1]), c(0,1,1,0), border=NA, col=col.TRD.CI)
  # limites de la limite haute de la TRD
  polygon(c(result$TRD_limits[2]+range.CI.qnorm*result$SE_TRD_limits[2], result$TRD_limits[2]+range.CI.qnorm*result$SE_TRD_limits[2], result$TRD_limits[2]-range.CI.qnorm*result$SE_TRD_limits[2], result$TRD_limits[2]-range.CI.qnorm*result$SE_TRD_limits[2]), c(0,1,1,0), border=NA, col=col.TRD.CI)
  # limites de la PT
  polygon(c(parP-range.CI.qnorm*resP, parP-range.CI.qnorm*resP, parP+range.CI.qnorm*resP, parP+range.CI.qnorm*resP), c(0,1,1,0), border=NA, col=col.PT.CI)
  par(xpd=TRUE)
  segments(parP, 0, parP, 1.05, lty=4)
  segments(result$TRD_limits[1], 0, result$TRD_limits[1], 1.15, lty=3)
  segments(result$TRD_limits[2], 0, result$TRD_limits[2], 1.15, lty=3)
  text(x=parP, y=1.1, lab.PT)
  text(x=parP, y=1.2, lab.TRD)
  
  	  b <- getFromNamespace(".BinomialConfidence", ns="HelpersMG")(alive,N)
  	  L1 <- modifyList(list(x=doses, y=alive/N, bty="n", type="p", ylim=c(0,1), y.plus = b[,3], y.minus = b[,2]), L)
  L1 <- modifyList(L1, list(ylim=c(0,1), xlab="", ylab="", main="", axes=FALSE, xlim=c(x1, x2)))
  
  par(xpd=FALSE)

  par(new=TRUE)
  
  a <- do.call(plot_errbar, L1) 
  
  if (!is.null(result$CI)) {

x <- result$CI["doses", ]
p <- result$CI["mean", ]
   
  par(new=TRUE)
  L1 <- modifyList(list(x=x, y=p, bty="n"), L)
  L1 <- modifyList(L1, list(ylim=c(0,1), axes=FALSE, xlab="", ylab="", type="l", main="", xlim=c(x1, x2)))

  a <- do.call(plot, L1)

if (show.CI) {
  
  pm <- result$CI["2.5%", ]
  pp <- result$CI["97.5%", ]
  
	par(new=TRUE)
    L1 <- modifyList(list(x=x, y=pm, bty="n"), L)
  L1 <- modifyList(L1, list(ylim=c(0,1), axes=FALSE, xlab="", ylab="", type="l", main="", lty=2, xlim=c(x1, x2)))
  a <- do.call(plot, L1) 
  
  par(new=TRUE)
    L1 <- modifyList(list(x=x, y=pp, bty="n"), L)
  L1 <- modifyList(L1, list(ylim=c(0,1), axes=FALSE, xlab="", ylab="", type="l", main="", lty=2, xlim=c(x1, x2)))
  a <- do.call(plot, L1) 
}
}
}

#' sun.info estimate the time of sunrise and sunset according to longitude, latitude and date
#' @title Estimate the time of sunrise and sunset according to longitude, latitude and date
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A data.frame with information about daily sun
#' @param latitude The latitude at which estimate the sun fates
#' @param longitude The longitude at which estimate the sun fates
#' @param date  A vector with the time at which sun fates are needed
#' @family Periodic patterns of indices
#' @description Estimate the sun fates according to latitude and date.\cr
#' Can be compared with the function \code{sunrise.set()} of package \code{StreamMetabolism}.
#' @references Teets, D.A. 2003. Predicting sunrise and sunset times. The College Mathematics Journal 34(4):317-321.
#' @examples
#' \dontrun{
#' # Generate a timeserie of time
#' date <- seq(from=as.Date("2000-01-01"), to=as.Date("2000-12-31"), by="1 day")
#' plot(date, sun.info(date, latitude=23, longitude=0)$day.length, bty="n", 
#'  las=1, type="l", xlab="Ordinal days", ylab="Day length in hours")
#' plot(date, sun.info(date, latitude=23, longitude=0)$sunrise, bty="n", 
#'  las=1, type="l", xlab="Ordinal days", ylab="Sun rise in hours")
#' }
#' @export

sun.info <- function(date, latitude, longitude){
  
  d <- as.numeric(as.POSIXlt(date)$yday)+1
  Lat <- latitude
  Long <- longitude
  
  ## d is the day of year
  ## Lat is latitude in decimal degrees
  ## Long is longitude in decimal degrees (negative == West)
  
  ##This method is copied from:
  ##Teets, D.A. 2003. Predicting sunrise and sunset times.
  ##  The College Mathematics Journal 34(4):317-321.
  
  ## At the default location the estimates of sunrise and sunset are within
  ## seven minutes of the correct times (http://aa.usno.navy.mil/data/docs/RS_OneYear.php)
  ## with a mean of 2.4 minutes error.
  
  ## Function to convert degrees to radians
  rad <- function(x) pi*x/180
  
  ##Radius of the earth (km)
  R=6378
  
  ##Radians between the xy-plane and the ecliptic plane
  epsilon=rad(23.45)
  
  ##Convert observer's latitude to radians
  L=rad(Lat)
  
  ## Calculate offset of sunrise based on longitude (min)
  ## If Long is negative, then the mod represents degrees West of
  ## a standard time meridian, so timing of sunrise and sunset should
  ## be made later.
  timezone = -4*(abs(Long)%%15)*sign(Long)
  
  ## The earth's mean distance from the sun (km)
  r = 149598000
  
  theta = 2*pi/365.25*(d-80)
  
  z.s = r*sin(theta)*sin(epsilon)
  r.p = sqrt(r^2-z.s^2)
  
  t0 = 1440/(2*pi)*acos((R-z.s*sin(L))/(r.p*cos(L)))
  
  ##a kludge adjustment for the radius of the sun
  that = t0+5 
  
  ## Adjust "noon" for the fact that the earth's orbit is not circular:
  n = 720-10*sin(4*pi*(d-80)/365.25)+8*sin(2*pi*d/365.25)
  
  ## now sunrise and sunset are:
  sunrise = (n-that+timezone)/60
  sunset = (n+that+timezone)/60
  
  UTC <- (((7.5+Long)%%360)) %/% 15
  if (UTC>12) {UTC <- 12-UTC;tz <- "Etc/GMT"} else {tz <- "Etc/GMT+"}
  tz <- paste0(tz, UTC)
  
  df <- data.frame(sunrise = sunrise, sunset = sunset, day.length=sunset- sunrise, 
                   date.time.sunrise=as.POSIXlt(format(date, "%Y-%m-%d"), tz=tz)+sunrise*60*60, 
                   date.time.sunset=as.POSIXlt(format(date, "%Y-%m-%d"), tz=tz)+sunset*60*60)
  
  sunrise.UTC <- as.POSIXlt(format(df$date.time.sunrise, format="%Y-%m-%d %H:%M:%S"), 
                           tz="UTC", use.tz=TRUE)
  sunrise.UTC.dec <- sunrise.UTC$hour + sunrise.UTC$min/60 + sunrise.UTC$sec/3600
  sunset.UTC <- as.POSIXlt(format(df$date.time.sunset, format="%Y-%m-%d %H:%M:%S"), 
                            tz="UTC", use.tz=TRUE)
  sunset.UTC.dec <- sunset.UTC$hour + sunset.UTC$min/60 + sunset.UTC$sec/3600
  df <- cbind(df, date.time.sunrise.UTC=sunrise.UTC, date.time.sunset.UTC=sunset.UTC, 
              time.sunrise.UTC=sunrise.UTC.dec, time.sunset.UTC=sunset.UTC.dec)
  
  return(df)
}

#' MovingWindow returns a moving average of a vector.
#' @title Return a moving average of a vector.
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A vector
#' @param x The vector to analyze
#' @param window The window size
#' @param fill TRUE or FALSE, should the vector return NA
#' @param hole Should the returned vector have the same length than x
#' @param FUN Function to apply to the window
#' @description Return a moving average of a vector.\cr
#' hole parameter can be "none", "bothL", "bothR", "both", "begin", or "end".
#' @examples
#' MovingWindow(1:10, window = 4, fill = TRUE, hole="bothL")
#' MovingWindow(1:10, window = 4, fill = TRUE, hole="bothR")
#' MovingWindow(1:10, window = 4, fill = TRUE, hole="both")
#' MovingWindow(1:10, window = 4, fill = TRUE, hole="none")
#' MovingWindow(1:10, window = 4, fill = TRUE, hole="begin")
#' MovingWindow(1:10, window = 4, fill = TRUE, hole="end")
#' MovingWindow(1:10, window = 4, fill = TRUE, hole="end", FUN=sd)
#' @export

MovingWindow <- function(x, window, hole="begin", fill=TRUE, FUN=mean) {
  fill <- tolower(fill)
  if (!is.na(fill))  hole <- match.arg(hole, choices = c("begin", "end", "bothL", "bothR", "both", "none"))
  y <- rep(NA, length(x))
  for (i in window:length(x)) {
    y[i] <- FUN(x[(i-window+1):i])
  }
  if (hole=="none") y <- y[is.na(y)]
  if (hole=="begin") {
    if (fill) {
      y[1:(window-1)] <- y[window]
    }
  }
  if (hole=="end") {
    if (fill) {
      y <- c(y[window:length(y)], rep(y[length(y)], window-1))
    } else {
      y <- c(y[window:length(y)], rep(NA, window-1))
    }
  }
  if (hole=="bothL") {
    w_l <- floor((window-1)/2)
    w_r <- window-1-w_l
  }
  if (hole=="bothR") {
    w_l <- floor((window-1)/2+0.5)
    w_r <- window-1-w_l
  }
  if (hole=="both") {
    w_l <- floor((window-1)/2+sample(c(0, 0.5), 1))
    w_r <- window-1-w_l
  }
  if (substr(hole, 1, 4)=="both") {
    if (fill) {
      y <- c(rep(y[window], w_l), y[window:length(y)], rep(y[length(y)], w_r))
    } else {
      y <- c(rep(NA, w_l), y[window:length(y)], rep(NA, w_r))
    }
  }
  return(y)
}

#' compare_AIC compares the AIC of several outputs obtained with the same data.
#' @title Compares the AIC of several outputs
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with DeltaAIC and Akaike weight for the models.
#' @param ... Successive results to be compared as lists.
#' @param factor.value The $value of the list object is multiplied by factor.value to calculate AIC.
#' @param silent If TRUE, nothing is displayed.
#' @param FUN Function used to show values
#' @description This function is used to compare the AIC of several outputs obtained with the same data but with different set of parameters.\cr
#' The parameters must be lists with $aic or $AIC or $value and $par elements or if AIC(element) is defined.\cr
#' if \code{$value} and \code{$par} are present in the object, the AIC is calculated as \code{2*factor.value*value+2*length(par)}. If \code{$value} is -log(likeihood), then factor.value must be 1 and if \code{$value} is log(likeihood), then factor.value must be -1.\cr
#' If several objects are within the same list, their AIC are summed.\cr
#' For example, compare_AIC(g1=list(group), g2=list(separe1, separe2)) can be used to compare a single model onto two different sets of data against each set of data fited with its own set of parameters.\cr
#' Take a look at \code{ICtab} in package \code{bbmle} which is similar.
#' @family AIC
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Here two different models are fitted
#' x <- 1:30
#' y <- rnorm(30, 10, 2)+log(x)
#' plot(x, y)
#' d <- data.frame(x=x, y=y)
#' m1 <- lm(y ~ x, data=d)
#' m2 <- lm(y ~ log(x), data=d)
#' compare_AIC(linear=m1, log=m2)
#' # Here test if two datasets can be modeled with a single model
#' x2 <- 1:30
#' y2 <- rnorm(30, 15, 2)+log(x2)
#' plot(x, y, ylim=c(5, 25))
#' plot_add(x2, y2, col="red")
#' d2 <- data.frame(x=x2, y=y2)
#' m1_2 <- lm(y ~ x, data=d2)
#' x_grouped <- c(x, x2)
#' y_grouped <- c(y, y2)
#' d_grouped <- data.frame(x=x_grouped, y=y_grouped)
#' m1_grouped <- lm(y ~ x, data=d_grouped)
#' compare_AIC(separate=list(m1, m1_2), grouped=m1_grouped)
#' }
#' @export


compare_AIC <- function(..., factor.value=1, silent=FALSE, FUN=function(x) specify_decimal(x, decimals=2)) {
result <- list(...)

if (is.list(result) & length(result)==1) result <- unlist(result, recursive=FALSE)

if (!is.null(result)) {
	if (!is.list(result) || (is.null(names(result))) || (any(names(result)==""))) {
		stop("The results must be included within a list with names; see example.")
	} else {
	out<-NULL
	l<-length(result)
		if (l<2) {
			stop("A least two results must be provided.")
		} else {
			aic<-NULL
			name<-names(result)
			for (i in 1:l) {

        encours <- result[i]
        aaec <- try(AIC(encours[[1]]), silent=TRUE)
        t <- (inherits(aaec, "try-error"))
        if (t & is.null(encours[[1]]$aic) & is.null(encours[[1]]$AIC) & is.null(encours[[1]]$value))
          encours <- encours[[1]]
        
 
# je n'ai pas de AIC ou rien pour le calculer
          sumAIC <- 0
            AICencours <- NULL
            for (j in 1:length(encours)) {
              encours2 <- encours[[j]]
              aaec <- try(AIC(encours2), silent=TRUE)
              t <- (inherits(aaec, "try-error"))
              if (!t) AICencours <- AIC(encours2)
              if (!is.null(encours2$AIC)) AICencours <- encours2$AIC
              if (!is.null(encours2$aic)) AICencours <- encours2$aic
              if (!is.null(encours2$value) & !is.null(encours2$par))
                AICencours <- 2*factor.value*encours2$value+2*(length(encours2$par))
            if (is.null(AICencours))
              {
                stop(paste("Object", name[i], "has not the required format"))
              }
            sumAIC <- sumAIC + AICencours
            }
			aic <- c(aic, sumAIC)	
			
			}
			
			bestaic<-min(aic)
			ser<-which.min(aic)
			deltaaic<-aic-bestaic
			aw<-exp(-0.5*deltaaic)
			saw=sum(aw)
			aw<-aw/saw
			
			out<-data.frame(cbind(AIC=FUN(aic), DeltaAIC=FUN(deltaaic), Akaike_weight=FUN(aw)), row.names=name)
			if (!silent) print(paste("The lowest AIC (",sprintf("%.3f", bestaic) ,") is for series ", name[ser], " with Akaike weight=", sprintf("%.3f", aw[ser]), sep=""))
			
			return(out)
		}
	}
}
}

#' inside Search a string within files of a folder
#' @title Search a string within files of a folder
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Return an invisible vector with filenames in which the pattern occurs
#' @param text Text to search in files
#' @param path Path of the folder to search in
#' @param pattern Pattern for file names to search in
#' @param showallfilenames logical. Show all the filenames search for in
#' @param fixed logical. If TRUE, pattern is a string to be matched as is. Overrides all conflicting arguments (see gsub)
#' @param ignore.case logical. if FALSE, the pattern matching for text is case sensitive and if TRUE, case is ignored during matching.
#' @param ... Options for readLines(), example warn = FALSE
#' @description Search for a string inside the files of a folder and return where the string is found.\cr
#' The pattern for files that must be included uses regex for filtering.\cr
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' # Search for files in path with names based on pattern that have the string search inside.
#' inside("embryogrowth", path=".", pattern="*\\.R$")
#' }
#' @export


inside <- function(text=stop("A text to be searched for is necessary"), 
                   path=".", pattern="*\\.R$", showallfilenames=FALSE, 
                   ..., 
                   fixed=TRUE, ignore.case = FALSE) {
  
  ls <- list.files(path=path, pattern=pattern)
  if (identical(ls, character(0))) {
    warning("No file match this pattern at this path")
    return(invisible(NULL))
  }
  
  filesok <- NULL
  
  for (f in ls) {
    
    fc <- readLines(con = file.path(path, f), ...)
    x <- gsub(pattern=text, replacement="", x=fc, fixed=fixed, ignore.case = ignore.case)
    if (any(x != fc)) {
      cat("file ", f, "\n")
      ll <- c(1:length(fc))[x != fc]
      if (length(ll)>1) {
        cat("lines ", ll, "\n")
      } else {
        cat("line ", ll, "\n")
      }
      filesok <- c(filesok, f)
    } else {
      if (showallfilenames) {
        cat("file ", f, "\n")
        cat("none", "\n")
      }
    }
  }
  return(invisible(filesok))
}

#' modifyVector modifies elements of a vector
#' @title Modifies Elements of a Vector
#' @author Marc Girondot
#' @return A modified version of x, with the elements of val replacing the elements of x
#' @param x A named vector.
#' @param val A named vector with components to replace corresponding components in x.
#' @param add If FALSE, only existing elements of x are returned.
#' @description Modifies a vector by changing a subset of elements to match a second vector.
#' @examples
#' library("HelpersMG")
#' e <- c(M=10, L=20, J=30)
#' modifyVector(e, c(U=10, M=30))
#' modifyVector(e, c(U=10, M=30), add=FALSE)
#' @export

modifyVector <- function(x, val, add=TRUE) {
  r <- unlist(modifyList(as.list(x), as.list(val)))
  if (!add) {
    r <- r[names(x)]
  }
  return(r)
}

#' rcutter returns random values based on fitted distribution with cut.
#' @title Random values of unobserved values of cut distribution. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A vector with the random numbers.
#' @param cutter The fitted model obtained with cutter()
#' @param lower_detection_limit The lower detection limit
#' @param upper_detection_limit The upper detection limit
#' @param method_cut What method is used to cut the distribution: "censored", "truncated"?
#' @param n number of random numbers
#' @param observed_detection_limit If TRUE, will use the pattern of detection limit as in observations 
#' @param random_method How to get parameters; it can be "ML", "medianMCMC", or "MCMC"
#' @param index_mcmc For MCMC random_method, the index of data to be used.
#' @description Return n random numbers.\cr
#' It can be used to get the posterior predictive distribution; see example.\cr
#' If random_method is "ML", the parameter values obtained using maximum likelihood are used.\cr
#' If random_method is "medianMCMC", the parameter values obtained using median of posterior distribution are used.\cr
#' If random_method is "MCMC", the parameter values are one sample of the MCMC posterior distribution.\cr
#' if observed_detection_limit is set to TRUE, the number of random number is equal to the number of observations; n is not used.\cr
#' rcutter is the abbreviation for random-cutter.
#' @family Distributions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' # _______________________________________________________________
#' # right censored distribution with gamma distribution
#' # _______________________________________________________________
#' # Detection limit
#' DL <- 100
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc>DL] <- +Inf
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, upper_detection_limit=DL, 
#'                            cut_method="censored")
#' result
#' # Posterior predictive distribution
#' r <- rcutter(cutter=result, upper_detection_limit=DL, n=100)
#' hist(r)
#' # _______________________________________________________________
#' # left censored distribution with gamma distribution
#' # _______________________________________________________________
#' # Detection limit
#' DL <- 10
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc<DL] <- -Inf
#' # search for the parameters the best fit these truncated data
#' result <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored")
#' result
#' plot(result, breaks=seq(from=0, to=200, by=10))
#' r <- rcutter(cutter=result, n=100)
#' hist(r, breaks=seq(from=0, to=200, by=10))
#' r <- rcutter(cutter=result, lower_detection_limit=DL, n=100)
#' hist(r, breaks=seq(from=0, to=250, by=10))
#' # With censored method, some values are replaced with +Inf or -Inf
#' any(is.infinite(r))
#' r <- rcutter(cutter=result, upper_detection_limit=DL, n=100, 
#'              method_cut="truncated")
#' # With truncated method, the values below LDL or upper UDL are not present
#' any(is.infinite(r))
#' hist(r, breaks=seq(from=0, to=10, by=0.25))
#' r <- rcutter(cutter=result, observed_detection_limit=TRUE)
#' hist(r, breaks=seq(from=0, to=300, by=10))
#' }
#' @export

rcutter <- function(cutter=stop("A result of cutter() must be provided"), 
                    n=1, 
                    lower_detection_limit = NULL, 
                    upper_detection_limit = NULL, 
                    method_cut = c("censored", "truncated"), 
                    observed_detection_limit = FALSE, 
                    random_method = c("medianMCMC", "MCMC", "ML"), 
                    index_mcmc=NULL) {
  
  # cutter=result
  # n=1
  # lower_detection_limit = NULL
  # upper_detection_limit = NULL
  # method_cut = c("censored", "truncated")
  # observed_detection_limit = FALSE
  # random_method = c("ML", "medianMCMC", "MCMC")
  # index_mcmc=NULL
  
  n.mixture <- cutter$n.mixture
  
  getparcutter <- getFromNamespace(".getparcutter", ns="HelpersMG")
  
  random_method <- tolower(random_method)[1]
  random_method <- match.arg(random_method, choices = c("ml", "medianmcmc", "mcmc"))
  method_cut <- tolower(method_cut)[1]
  method_cut  <- match.arg(method_cut, choices = c("truncated", "censored"))
  distribution <- cutter$distribution
  
  if (random_method == "ml") {
    par <- cutter$par
  } else {
    if (random_method == "medianmcmc") {
      par <- cutter$par_mcmc[2, ]
    } else {
      if (is.null(index_mcmc)) {
        index_mcmc <- sample(x = 1:nrow(cutter$mcmc$resultMCMC[[1]]), size=1)
      }
      par <- cutter$mcmc$resultMCMC[[1]][index_mcmc,]
    }
  }
  
  obc <- cutter$observations[!is.na(cutter$observations[, "Observations"]), ]
  
  if (observed_detection_limit) {
    n <- nrow(obc)
    lower_detection_limit <- cutter$observations[!is.na(cutter$observations[, "Observations"]), "LDL"]
    upper_detection_limit <- cutter$observations[!is.na(cutter$observations[, "Observations"]), "UDL"]
    method_cut <- cutter$observations[!is.na(cutter$observations[, "Observations"]), "Cut"]
  } else {
    if (is.null(lower_detection_limit)) {
      lower_detection_limit <- rep(x=NA, n)
    } else {
      lower_detection_limit <- rep(x=lower_detection_limit, n)[1:n]
    }
    if (is.null(upper_detection_limit)) {
      upper_detection_limit <- rep(x=NA, n)
    } else {
      upper_detection_limit <- rep(x=upper_detection_limit, n)[1:n]
    }
    method_cut <- rep(x=method_cut, n)[1:n]
  }
  
  pparX <- getparcutter(par, set=NULL)
  
  rdistr <- switch(distribution, 
                   gamma = rgamma, 
                   lognormal = rlnorm, 
                   normal = rnorm, 
                   weibull = rweibull, 
                   generalized.gamma=rggamma)
  
  x2 <- NULL
  cpt <- 0
  # Je dois continuer si length(x2) < n et si cpt != 10*n
  while((length(x2) < n) & (cpt != (100*n))) {
    x1 <- NULL

    while((length(x1) < 1) & (cpt != (100*n))) {
      
      x1 <- NULL
      for (p in 1:n.mixture)
      x1 <- c(x1, do.call(rdistr, args = c(list(n = 1), as.list(getparcutter(par, set=p)))))
      
      
      x1 <- x1[which(runif(n=1) < cumsum(pparX))[1]]
      
      if (!is.na(lower_detection_limit[length(x2)+1])) {
        if (x1 < lower_detection_limit[length(x2)+1]) {
          if (method_cut[length(x2)+1] == "censored") {
            x1 <- -Inf
          } else {
            x1 <- NULL
          }
        }
      }
      if (!is.null(x1))
        if (!is.na(upper_detection_limit[length(x2)+1]) & is.finite(x1)) {
          if (x1 > upper_detection_limit[length(x2)+1]) {
            if (method_cut[length(x2)+1] == "censored") {
              x1 <- +Inf
            } else {
              x1 <- NULL
            }
          }
        }
      cpt <- cpt + 1
    }
    
    x2 <- c(x2, x1)
    
    if (cpt == n*100) {
      if (length(x2) == 0) {
        x2 <- NA
      }
      
        x2 <- rep(x2, n)[1:n]
      
    }
  }
  

  
  attributes(x2) <- c(attributes(x2), list(par=par))  
  
  return(x2)
}

#' RandomFromHessianOrMCMC returns random numbers based on Hessian matrix or MCMC
#' @title Random numbers based on Hessian matrix or MCMC
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Returns a list with three data.frames named random, fn, and quantiles
#' @param Hessian An Hessian matrix
#' @param se A named vector with SE of parameters
#' @param mcmc A result from MHalgogen()
#' @param chain MCMC chain to be used or "all"
#' @param regularThin If TRUE, use regular thin for MCMC or use a number
#' @param MinMax A data.frame with at least two columns: Min and Max and rownames being the variable names
#' @param fitted.parameters The fitted parameters
#' @param fixed.parameters The fixed parameters
#' @param replicates Number of replicates to generate the randoms
#' @param method Can be NULL, "SE", "Hessian", "MCMC", or "PseudoHessianFromMCMC"
#' @param fn The function to apply to each replicate
#' @param probs Probability for quantiles
#' @param silent Should the function display some information
#' @param ParTofn Name of the parameter to send random values to fn
#' @param ... Parameters send to fn function
#' @description If it is very long, use silent parameter to check if something goes wrong.\cr
#' If replicates is NULL or is 0, or if method is NULL, parameters are just copied into data.frame.\cr
#' If method is NULL, replicate.CI is set to 0.\cr
#' If method is hessian, it will generate replicate.CI random numbers using 
#' Hessian matrix with covariance.\cr
#' If method is se, it will generate replicate.CI random numbers using 
#' SE values then without covariance.\cr
#' If method is mcmc, it will generate replicate.CI random numbers using 
#' random samples of the MCMC or the regularThin number if it is a number.\cr
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' val <- rnorm(100, mean=20, sd=5)+(1:100)/10
#' # Return -ln L of values in val in Gaussian distribution with mean and sd in par
#' fitnorm <- function(par, data) {
#'   -sum(dnorm(data, par["mean"], abs(par["sd"]), log = TRUE))
#' }
#' # Initial values for search
#' p<-c(mean=20, sd=5)
#' # fit the model
#' result <- optim(par=p, fn=fitnorm, data=val, method="BFGS", hessian=TRUE)
#' # Using Hessian
#' df <- RandomFromHessianOrMCMC(Hessian=result$hessian, 
#'                               fitted.parameters=result$par, 
#'                               method="Hessian")$random
#' hist(df[, 1], main="mean")
#' hist(df[, 2], main="sd")
#' plot(df[, 1], df[, 2], xlab="mean", ylab="sd", las=1, bty="n")
#' 
#' # Using MCMC
#' parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
#' Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(0.35, 0.2), 
#' Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
#' row.names=c('mean', 'sd'))
#' # Use of trace and traceML parameters
#' # trace=1 : Only one likelihood is printed
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
#' parameters_name = "par", 
#' likelihood=fitnorm, n.chains=1, n.adapt=100, thin=1, trace=1)
#' df <- RandomFromHessianOrMCMC(mcmc=mcmc_run, fitted.parameters=NULL, 
#'                               method="MCMC")$random
#' hist(df[, 1], main="mean")
#' hist(df[, 2], main="sd")
#' plot(df[, 1], df[, 2], xlab="mean", ylab="sd", las=1, bty="n")
#' 
#' # Return the two first elements of the MCMC
#' df <- RandomFromHessianOrMCMC(mcmc=mcmc_run, fitted.parameters=NULL, 
#'                method="MCMC", replicates = 2, regularThin = c(1, 2))$random
#' 
#' # Using a function fn
#' fitnorm <- function(par, data, x) { 
#'   y=par["a"]*(x)+par["b"]
#'   -sum(dnorm(data, y, abs(par["sd"]), log = TRUE))
#' }
#' p<-c(a=0.1, b=20, sd=5)
#' # fit the model
#' x <- 1:100
#' result <- optim(par=p, fn=fitnorm, data=val, x=x, method="BFGS", hessian=TRUE)
#' # Using Hessian
#' df <- RandomFromHessianOrMCMC(Hessian=result$hessian, fitted.parameters=result$par, 
#'                               method="Hessian", 
#'                               fn=function(par) (par["a"]*(x)+par["b"]))
#' plot(1:100, val)
#' lines(1:100, df$quantiles["50%", ])
#' lines(1:100, df$quantiles["2.5%", ], lty=2)
#' lines(1:100, df$quantiles["97.5%", ], lty=2)
#' }
#' @export

RandomFromHessianOrMCMC <- function(se=NULL                          , 
                                    Hessian=NULL                     , 
                                    mcmc=NULL                        , 
                                    chain="all"                      , 
                                    regularThin=TRUE                 , 
                                    MinMax=NULL                      , 
                                    fitted.parameters=NULL           , 
                                    fixed.parameters=NULL            , 
                                    method=NULL                      , 
                                    probs=c(0.025, 0.5, 0.975)       , 
                                    replicates=10000                 , 
                                    fn=NULL                          , 
                                    silent=FALSE                      , 
                                    ParTofn="par"                    , 
                                    ...) {
  
  # se=NULL; Hessian=NULL; mcmc=NULL; chain=1
  # method=NULL
  # regularThin=TRUE
  # MinMax=NULL
  # fitted.parameters=NULL
  # fixed.parameters=NULL
  # probs=c(0.025, 0.5, 0.975)
  # replicates=10000; fn=NULL; silent=FALSE; ParTofn="par"
  
  if (is.null(replicates)) replicates <- 0
  if (is.null(method)) method <- "null"
  method <- tolower(method)
  
  method <- match.arg(method, choices=c("null", "se", "hessian", "mcmc", "pseudohessianfrommcmc"))
  
  df_random <- NULL
  
  if (method == "null" | replicates == 0)  {
    if (!is.null(fitted.parameters)) {
      # Je retourne un dataframe avec simplement des recopies
      df_random <- as.data.frame(matrix(data = rep(fitted.parameters, ifelse(replicates==0, 1, replicates)), 
                                        nrow = ifelse(replicates==0, 1, replicates), 
                                        byrow = TRUE))
      colnames(df_random) <- names(fitted.parameters)
    }
  }
  
  if (!is.null(Hessian) & (method == "hessian") & (is.null(fitted.parameters))) stop("Fitted.parameters or Hessian cannot be NULL with Hessian method.")
  if (!is.null(se) & (method == "se") & (is.null(fitted.parameters))) stop("Fitted.parameters or se cannot be NULL with se method.")
  
  if (method == "mcmc") {
    
    if (chain[1] == "all") chain <- seq_along(mcmc$resultMCMC)
    df_random <- NULL
    if (length(chain) >= 1) {
      for (i in chain) {
        df_random <- rbind(df_random, mcmc$resultMCMC[[i]])
      }
    } else {
      stop("No data have been selected.")
    }
    
    if (isTRUE(regularThin)) {
      if (replicates <= nrow(df_random)) {
        df_random <- df_random[seq(from=1, to=nrow(df_random), length.out=replicates), , drop=FALSE]
      } else {
        stop("When regularThin is TRUE, replicates must be lower or equal to number of MCMC iterations.")
      }
    } else {
      if (isFALSE(regularThin)) {
      if (replicates < nrow(df_random)) {
        df_random <- df_random[sample(x=1:nrow(df_random), size=replicates), , drop=FALSE]
      } else {
        df_random <- df_random[sample(x=1:nrow(df_random), size=replicates, replace = TRUE), , drop=FALSE]
      }
      } else {
        if (is.numeric(regularThin)) {
          regularThin <- rep(regularThin, replicates)[1:replicates]
          df_random <- df_random[regularThin, , drop=FALSE]
        } else {
          stop("I don't understand the format of regularThin.")
        }
      }
    }
  }
  
  if (method == "pseudohessianfrommcmc") {
    Hessian <- solve(cov(mcmc$resultMCMC[[chain]]))
    fitted.parameters <- as.parameters(mcmc)
    method <- "hessian"
  }
  
  if (method == "hessian") {
    
    sigma  <- try(solve(Hessian), silent=TRUE)
    # Add all: 22/4/2020; any !
    if (inherits(sigma, "try-error")) {
      if (!silent) warning("Error in Hessian matrix inversion")
      Hessianx <- try(as.matrix(getFromNamespace("nearPD", ns="Matrix")(Hessian)$mat), silent=TRUE)
      # 29/1/2021
      if (inherits(Hessianx, "try-error")) {
        if (!silent) warning("Error in estimation of the Nearest Positive Definite Matrix. Calculates the Moore-Penrose generalized inverse. Use result with caution.")
        sigma  <- try(ginv(Hessian), silent=TRUE)
      } else {
        if (!silent) warning("Calculates the Nearest Positive Definite Matrix. Use result with caution.")
        sigma  <- try(solve(Hessianx), silent=TRUE)
      }
    } 
    
    if (!inherits(sigma, "try-error")) {
      if (!silent) cat("Estimation using variance-covariance matrix\n")
      df_random <- matrix(data=NA, nrow=1, ncol=nrow(Hessian))
      df_random <- as.data.frame(df_random[-1, ])
      colnames(df_random) <- rownames(Hessian)
      
      reste.replicates <- replicates - nrow(df_random)
      repeat {
        if (!silent) cat(paste0("Still ", as.character(reste.replicates), " replicates\n"))
        s. <- svd(sigma)
        R <- t(s.$v %*% (t(s.$u) * sqrt(pmax(s.$d, 0))))
        df_random_int <- matrix(rnorm(reste.replicates * ncol(sigma)), 
                                nrow = reste.replicates, byrow = TRUE) %*% R
        df_random_int <- sweep(df_random_int, 2, fitted.parameters[rownames(Hessian)], "+")
        colnames(df_random_int) <- rownames(Hessian)
        if (!is.null(MinMax)) {
          L <- rep(TRUE, nrow(df_random_int))
          for (i in colnames(df_random_int)) {
            if ((!is.na(MinMax[i, "Min"])) & (!is.na(MinMax[i, "Max"]))) {
              # Je veux TRUE si c'est bon
              L <- L & (MinMax[i, "Min"] <= df_random_int[, i]) & 
                (MinMax[i, "Max"] >= df_random_int[, i])
            }
          }
          df_random_int <- df_random_int[L, , drop=FALSE]
        }
        df_random <- rbind(df_random, df_random_int)
        reste.replicates <- replicates - nrow(df_random)
        if (reste.replicates == 0) break
      }
      
    } else {
      if (!silent) cat("Error using Hessian; Estimation using standard errors estimated from Hessian")
      se <- SEfromHessian(Hessian)
      method <- "se"
    }
  }
  
  if ((method == "se") & (is.null(se)) & (!is.null(Hessian))) {
    if (!silent) cat("Estimation using standard errors estimated from Hessian")
    se <- SEfromHessian(Hessian)
  }
  
  if (method == "se") {
    # J'ai une erreur sur l'inversion de la matrice hessienne; ou bien j'ai des se
    # Je prends les SE
    df_random <- matrix(data=NA, nrow=1, ncol=length(se))
    df_random <- as.data.frame(df_random[-1, ])
    colnames(df_random) <- names(se)
    reste.replicates <- replicates - nrow(df_random)
    repeat {
      df_random_int <- matrix(data = NA, ncol=length(se), nrow=reste.replicates)
      colnames(df_random_int) <- names(se)
      for (i in colnames(df_random_int)) {
        df_random_int[, i] <- rnorm(reste.replicates, fitted.parameters[i], se[i])
      }
      if (!is.null(MinMax)) {
        L <- rep(TRUE, nrow(df_random_int))
        for (i in colnames(df_random_int)) {
          # Je veux TRUE si c'est bon
          L <- L & (MinMax[i, "Min"] <= df_random_int[, i]) & 
            (MinMax[i, "Max"] >= df_random_int[, i])
        }
        df_random_int <- df_random_int[L, ]
      }
      df_random <- rbind(df_random, df_random_int)
      reste.replicates <- replicates - nrow(df_random)
      if (reste.replicates == 0) break
    } 
    
  }
  
  
  if (!is.null(fixed.parameters)) {
    ajouter <- matrix(rep(fixed.parameters, ifelse(replicates==0, 1, replicates)), 
                      nrow=ifelse(replicates==0, 1, replicates), byrow=TRUE,
                      dimnames = list(c(NULL), names(fixed.parameters)))
    df_random <- cbind(df_random, ajouter)
  }
  
  if (is.null(df_random)) {
    stop("Impossible to generate random numbers")
  }
  
  if (!is.null(fn)) {
    parx <- list(...)
    # print(str(parx))
    df_fn <- apply(df_random, MARGIN=1, 
                   FUN=function(par_dfr) {
                     lst <- list(par=unlist(par_dfr))
                     names(lst) <- ParTofn
                     do.call(fn, modifyList(lst, parx))
                   })
    # df_fn peut être un vecteur ou une liste
    if (is.list(df_fn)) {
      df_fn <- unlist(df_fn)
      df_fn <- matrix(df_fn, nrow=ifelse(replicates==0, 1, replicates), byrow = TRUE)
    } else {
      
      if (is.null(dim(df_fn))) {
        df_fn <- matrix(df_fn, ncol=1)
      } else {
        df_fn <- t(df_fn)
      }
    }
    
    q <- apply(df_fn, MARGIN=2, FUN=function(x) quantile(x, probs = probs, na.rm=TRUE))
    if (any(is.na(df_fn))) warning("Some NA are produced; take care because outputs are biased")
  } else {
    df_fn <- NULL
    q <- NULL
  }
  
  
  
  
  return(list(random=df_random, fn=df_fn, quantiles=q))
}

#' IC_correlation_simplify simplifies the correlation matrix
#' @title Simplify the correlation matrix
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list
#' @param matrix The correlation matrix to simplify
#' @param variable a vector with the name of columns to keep
#' @description This function can be used to simplify the network of correlations.\cr
#' If no vector of variables is given, the variables not linked to any other variable are removed. 
#' If a vector of variables is given, only link to these variables are retained.
#' \code{https://fr.wikipedia.org/wiki/Iconographie_des_corrélations}
#' @family Iconography of correlations
#' @references Lesty, M., 1999. Une nouvelle approche dans le choix des régresseurs de la régression multiple en présence d’interactions et de colinéarités. Revue de Modulad 22, 41-77.
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' es <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
#'                  Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
#'                  Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
#'                  Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
#'                  Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
#'                  row.names = c(NA, -8L), class = "data.frame")
#' 
#' es
#' 
#' df <- IC_clean_data(es, debug = TRUE)
#' cor_matrix <- IC_threshold_matrix(data=df, threshold = NULL, progress=FALSE)
#' cor_threshold <- IC_threshold_matrix(data=df, threshold = 0.3)
#' par(mar=c(1,1,1,1))
#' set.seed(4)
#' plot(cor_threshold)
#' cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
#' plot(cor_threshold_Note)
#' }
#' @export


IC_correlation_simplify <- function(matrix, variable=NULL) {
  if (!inherits(matrix, "IconoCorel")) {
    stop("Only an object obtained using IC_threshold_matrix() can be used.")
  }
  m <- matrix$thresholded_correlation
  if (is.null(variable)) {
    garde <- !apply(X = m, MARGIN = 1, FUN=function(x) all(abs(x)<1E-5))
  } else {
    garde <- m[, variable, drop=FALSE] != 0
    garde[colnames(x = m) %in% variable] <- TRUE
    garde <- apply(garde, MARGIN = 1, FUN=any)
  }
  matrix$thresholded_correlation <- m[garde, garde]
  return(matrix)
}

#' Helpers functions for several packages
#'
#' \tabular{ll}{
#'  Package: \tab HelpersMG\cr
#'  Type: \tab Package\cr
#'  Version: \tab 2026.8.24 build 1817\cr
#'  Date: \tab 2026-08-24\cr
#'  License: \tab GPL (>= 2)\cr
#'  LazyLoad: \tab yes\cr
#'  }
#' @title Tools for Environmental Analyses, Ecotoxicology and Various R Functions
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @name HelpersMG-package
#' @description Contains miscellaneous functions useful for managing \cr
#' 'NetCDF' files (see http://en.wikipedia.org/wiki/NetCDF), \cr
#' get tide levels on any point of the globe, \cr
#' get moon phase and time for sun rise and fall, \cr
#' analyse and reconstruct daily time series of temperature \cr
#' with irregular sinusoidal pattern, \cr
#' show scales and wind rose in plot with change of color of text, \cr 
#' Metropolis-Hastings algorithm for Bayesian MCMC analysis, \cr
#' plot graphs or boxplot with error bars, \cr
#' search files in disk by there names or their content, \cr
#' read the contents of all files from a folder at one time, \cr
#' calculate IC50 for ecotoxicological studies, \cr
#' calculate the probability mass function of the sum of negative binomial \cr
#' distributions, calculate distribution of unobserved values in censored or 
#' truncated distributions.\cr
#' The latest version of this package can always be installed using:\cr
#' install.packages("http://marc.girondot.free.fr/CRAN/HelpersMG.tar.gz", 
#'                  repos=NULL, type="source")\cr
#' \if{html}{\figure{TB.png}{options: alt="HelpersMG logo"}}
#' \if{latex}{\figure{TB.png}}
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' print('----------------------------------')
#' print('Examples for mcmcComposite objects')
#' print('----------------------------------')
#' require(coda)
#' x <- rnorm(30, 10, 2)
#' dnormx <- function(x, par) return(-sum(dnorm(x, mean=par['mean'], sd=par['sd'], log=TRUE)))
#' parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
#' Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(0.35, 0.2), 
#' Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
#' row.names=c('mean', 'sd'))
#' mcmc_run <- MHalgoGen(n.iter=100000, parameters=parameters_mcmc, data=x, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
#' plot(mcmc_run, xlim=c(0, 20))
#' plot(mcmc_run, xlim=c(0, 10), parameters="sd")
#' mcmcforcoda <- as.mcmc(mcmc_run)
#' # Optimal rejection rate should be 0.234
#' rejectionRate(mcmcforcoda)
#' heidel.diag(mcmcforcoda)
#' raftery.diag(mcmcforcoda)
#' autocorr.diag(mcmcforcoda)
#' acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
#' acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
#' batchSE(mcmcforcoda, batchSize=100)
#' # The batch standard error procedure is usually thought to 
#' # be not as accurate as the time series methods used in summary
#' summary(mcmcforcoda)$statistics[,"Time-series SE"]
#' summary(mcmc_run)
#' as.parameters(mcmc_run)
#' lastp <- as.parameters(mcmc_run, index="last")
#' parameters_mcmc[,"Init"] <- lastp
#' # The n.adapt set to 1 is used to not record the first set of parameters
#' # then it is not duplicated (as it is also the last one for 
#' # the object mcmc_run)
#' mcmc_run2 <- MHalgoGen(n.iter=10000, parameters=parameters_mcmc, data=x, 
#' likelihood=dnormx, n.chains=1, n.adapt=1, thin=1, trace=1)
#' mcmc_run3 <- merge(mcmc_run, mcmc_run2)
#' ####### no adaptation, n.adapt must be 0
#' parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
#' mcmc_run3 <- MHalgoGen(n.iter=10000, parameters=parameters_mcmc, data=x, 
#' likelihood=dnormx, n.chains=1, n.adapt=0, thin=1, trace=1)
#' print('------------------------------------------')
#' print('Examples for Daily patterns of temperature')
#' print('------------------------------------------')
#' # Generate a timeserie of time
#' time.obs <- NULL
#' for (i in 0:9) time.obs <- c(time.obs, c(0, 6, 12, 18)+i*24)
#' # For these time, generate a timeseries of temperatures
#' temp.obs <- rep(NA, length(time.obs))
#' temp.obs[3+(0:9)*4] <- rnorm(10, 25, 3)
#' temp.obs[1+(0:9)*4] <- rnorm(10, 10, 3)
#' for (i in 1:(length(time.obs)-1)) 
#'   if (is.na(temp.obs[i])) 
#'   temp.obs[i] <- mean(c(temp.obs[i-1], temp.obs[i+1]))
#'   if (is.na(temp.obs[length(time.obs)])) 
#'   temp.obs[length(time.obs)] <- temp.obs[length(time.obs)-1]/2
#' observed <- data.frame(time=time.obs, temperature=temp.obs)
#' # Search for the minimum and maximum values
#' r <- minmax.periodic(time.minmax.daily=c(Min=2, Max=15), 
#' observed=observed, period=24)
#' 
#' # Estimate all the temperatures for these values
#' t <- temperature.periodic(minmax=r)
#' 
#' plot_errbar(x=t[,"time"], y=t[,"temperature"],
#' errbar.y=ifelse(is.na(t[,"sd"]), 0, 2*t[,"sd"]),
#' type="l", las=1, bty="n", errbar.y.polygon = TRUE, 
#' xlab="hours", ylab="Temperatures", ylim=c(0, 35), 
#' errbar.y.polygon.list = list(col="grey"))
#' 
#' plot_add(x=t[,"time"], y=t[,"temperature"], type="l")
#' 
#' # How many times this package has been download
#' library(cranlogs)
#' HelpersMG <- cran_downloads("HelpersMG", from = "2015-04-07", 
#'                             to = Sys.Date() - 1) 
#' sum(HelpersMG$count)
#' plot(HelpersMG$date, HelpersMG$count, type="l", bty="n")
#' }

NULL

#' SEfromHessian returns standard error of parameters based on Hessian matrix
#' @title Standard error of parameters based on Hessian matrix
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return SEfromHessian returns a vector with standard errors
#' @param a An Hessian matrix
#' @param hessian If TRUE, return a list with the hessian and SE
#' @param silent If TRUE, report some problems
#' @description Standard error of parameters based on Hessian matrix.\cr
#' The strategy is as follow:\cr
#' First it tries to inverse the Hessian matrix. If it fails, it uses the near 
#' positive definite matrix of the Hessian.\cr
#' So now the inverse of the Hessian matrix can be computed.\cr
#' The diagonal of the inverse of the Hessian matrix is calculated. If all values are positive, 
#' the SEs are the square root of the inverse of the Hessian.\cr
#' If not all values are positive, it will estimate the pseudo-variance 
#' matrix based on Gill & King (2004). It necessitates a Cholesky matrix.\cr
#' If from some reason it fails (for example all SE are 0 in output), then the strategy of 
#' Rebonato and Jackel (2000) will be used to generate the Cholesky matrix.
#' @references Gill J. and G. King 2004. What to do when your Hessian is not invertible: 
#' Alternatives to model respecification in nonlinear estimation. Sociological Methods 
#' & Research 33: 54-87.
#' @references Rebonato and Jackel, “The most general methodology for creating a valid 
#' correlation matrix for risk management and option pricing purposes”, Journal of Risk, 
#' Vol 2, No 2, 2000.
#' @examples
#' \dontrun{
#' val=rnorm(100, mean=20, sd=5)
#' # Return -ln L of values in val in Gaussian distribution with mean and sd in par
#' fitnorm<-function(par, val) {
#'   -sum(dnorm(val, par["mean"], par["sd"], log = TRUE))
#' }
#' # Initial values for search
#' p<-c(mean=20, sd=5)
#' # fit the model
#' result <- optim(par=p, fn=fitnorm, val=val, method="BFGS", hessian=TRUE)
#' SE <- SEfromHessian(result$hessian)
#' library(MASS)
#' fitdistr(val, densfun = "normal")
#' }
#' @export

SEfromHessian <- function(a, hessian=FALSE, silent=FALSE) {
  namesp <- colnames(a)
  mathessian <- a
  
  mathessian <- ifelse(mathessian==-Inf, -1E9, mathessian)
  mathessian <- ifelse(mathessian==+Inf, 1E9, mathessian)
  
  sigma  <- try(solve(mathessian), silent=TRUE)
  # Add all: 22/4/2020; any !
  if (inherits(sigma, "try-error")) {
    if (!silent) warning("Error in Hessian matrix inversion")
    mathessianx <- try(as.matrix(getFromNamespace("nearPD", ns="Matrix")(mathessian)$mat), silent=TRUE)
    # 29/1/2021
    if (inherits(mathessianx, "try-error")) {
      if (!silent) warning("Error in estimation of the Nearest Positive Definite Matrix. Calculates the Moore-Penrose generalized inverse. Use result with caution.")
      sigma  <- try(ginv(mathessian), silent=TRUE)
      if (is.null(colnames(sigma)) | is.null(rownames(sigma))) {
        colnames(sigma) <- rownames(sigma) <- colnames(mathessian)
      }
    } else {
      if (!silent) warning("Calculates the Nearest Positive Definite Matrix. Use result with caution.")
      sigma  <- try(solve(mathessianx), silent=TRUE)
    }
  } 
  
  # Add all: 22/4/2020
  if (!inherits(sigma, "try-error")) {
    
    if (all(diag(sigma)>=0)) {
      # méthode classique
      res <- sqrt(diag(sigma))
      
    } else {
      # Autre essai
      s. <- svd(sigma)
      R <- t(s.$v %*% (t(s.$u) * sqrt(pmax(s.$d, 0))))
      res <- structure((matrix(rep(1, nrow(R)), nrow = 1, byrow = TRUE) %*% R)[1, ], names=colnames(mathessian))
      
      # Si j'ai des SE négatif... pas bon signe
      if (any(res<0)) {
        d <- diag(as.matrix(getFromNamespace("nearPD", ns="Matrix")(sigma)$mat))
        names(d) <- colnames(mathessian)
        res <- ifelse(d<0, NA, sqrt(d))
      }
      if (any(is.na(res))) {
        
        
        a <- sigma
        
        n = dim(a)[1];
        root = matrix(0,n,n);
        
        for (i in 1:n){
          sum = 0;
          if (i>1){
            sum = sum(root[i,1:(i-1)]^2);
          }
          
          x = a[i,i] - sum;
          
          if (x<0){
            x = 0;
          }
          
          root[i,i] = sqrt(x);
          
          if (i < n){
            for (j in (i+1):n){
              
              if (root[i,i] == 0){
                x=0;
              }
              else{
                sum = 0;
                if (i>1) {
                  sum = root[i,1:(i-1)] %*% t(t(root[j,1:(i-1)]))
                }
                x = (a[i,j] - sum)/root[i,i];
              }
              
              root[j,i] = x;
            }
          }
        }
        colnames(root) <- rownames(root) <- colnames(mathessian)
        
        pseudoV <- root %*% t(root)
        
        d <- diag(pseudoV)
        
        if (any(d != 0) & all(d >= 0)) {
          res <- sqrt(d)
          
          if (!silent) warning("Estimates using pseudo-variance based on Gill & King (2004)")
          
        } else {
          if (!silent) warning("Approximation of Cholesky matrix based on Rebonato and Jackel (2000)")
          if (!silent) warning("Estimates using pseudo-variance based on Gill & King (2004)")
          
          # The paper by Rebonato and Jackel, “The most general methodology for creating a valid correlation matrix for risk management and option pricing purposes”, Journal of Risk, Vol 2, No 2, 2000, presents a methodology to create a positive definite matrix out of a non-positive definite matrix.
          # FP Brissette, M Khalili, R Leconte, Journal of Hydrology, 2007, “Efficient stochastic generation of multi-site synthetic precipitation data”
          # GA Baigorria, JW Jones, Journal of Climate, 2010, “GiST: A stochastic model for generating spatially and temporally correlated daily rainfall data”
          # M Mhanna and W Bauwens, International Journal of Climatology, 2012, “A stochastic space-time model for the generation of daily rainfall in the Gaza Strip”
          # fix the correl matrix
          newMat <- a
          cholError <- TRUE
          iter <- 0
          while (cholError) {
            
            iter <- iter + 1
            # cat("iteration ", iter, "\n")
            
            # replace -ve eigen values with small +ve number
            newEig <- eigen(newMat)
            newEig2 <- ifelse(newEig$values < 0, 0, newEig$values)
            
            # create modified matrix eqn 5 from Brissette et al 2007, inv = transp for
            # eig vectors
            newMat <- newEig$vectors %*% diag(newEig2) %*% t(newEig$vectors)
            
            # normalize modified matrix eqn 6 from Brissette et al 2007
            newMat <- newMat/sqrt(diag(newMat) %*% t(diag(newMat)))
            
            # try chol again
            cholStatus <- try(u <- chol(newMat), silent = TRUE)
            cholError <- ifelse(inherits(cholStatus, "try-error"), TRUE, FALSE)
          }
          root <- cholStatus
          
          colnames(root) <- rownames(root) <- colnames(mathessian)
          
          pseudoV <- root %*% t(root)
          
          res <- sqrt(diag(pseudoV))
        }
      }
    }
  }
  
  
  SEInf <- namesp[!namesp %in% names(res)]
  
  res <- c(res, structure(rep(+Inf, length(SEInf)), names=SEInf))
  
  if (hessian) {
    return(list(SE=res, hessian=mathessian))
  } else {
    return(res)
  }
  
}

#' RectangleRegression performs rectangle regression
#' @title Return parameters of rectangle regression
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with parameters of rectangle regression
#' @param x1 The first series of data
#' @param x2 The second series of data
#' @param replicate Number of replicates for bootstrap
#' @param x1new Values for x1 to generate x2
#' @description Fit a line using least rectangle method.
#' @family Rectangle Regression
#' @examples
#' x1 <- runif(100, min=10, max=20)
#' x2 <- runif(100, min=10, max=20)+x1
#' 
#' rectreg <- RectangleRegression(x1, x2)
#' 
#' plot(x=x1, y=x2, bty="n", las=1, xlim=c(10, 20), ylim=c(20, 40))
#' abline(a=rectreg$par["Intercept"], b=rectreg$par["Slope"], lwd=2)
#' par(xpd=FALSE)
#' lines(rectreg$x2new["x1new", ], rectreg$x2new["50%", ])
#' lines(rectreg$x2new["x1new", ], rectreg$x2new["2.5%", ], lty=2)
#' lines(rectreg$x2new["x1new", ], rectreg$x2new["97.5%", ], lty=2)
#' 
#' abline(a=rectreg$Intercept[1], b=rectreg$Slope[3], col="red")
#' abline(a=rectreg$Intercept[3], b=rectreg$Slope[1], col="red")
#' 
#' @export


RectangleRegression <- function(x1, x2, replicate=1000, 
                                x1new=seq(from=min(x1), to=max(x1), length.out = 100)) {
  
  frect <- function(x,y){ # droite des moindres rectangles 
    b <- sqrt(var(y)/var(x))*sign(cor(x,y)) 
    r <- c(b, mean(y)-b*mean(x)) 
    names(r) <- c("Slope", "Intercept")
    return(r)
  }

  
  m <- matrix(data=NA, nrow=replicate, ncol=length(x1new))
  c <- NULL
  d <- NULL
  rrx <- frect(x1, x2)
  for (r in 1:replicate) {
    s <- sample(1:length(x1), length(x1), replace = TRUE)
    rr <- frect(x1[s], x2[s])
    c <- c(c, rr[1])
    d <- c(d, rr[2])
    # Prediction
    x2new <- cbind(x1new, 1) %*% rr
    m[r, ] <- x2new
  }
  q <- apply(m, MARGIN=2, FUN=quantile, probs=c(0.025, 0.5, 0.975))
  q <- rbind(q, x1new=x1new)
  qb <- quantile(c, probs=c(0.025, 0.5, 0.975))
  qa <- quantile(d, probs=c(0.025, 0.5, 0.975))
  
  return(list(par=rrx,
              x2new=q,
              Intercept=qa,
              Slope=qb))
}

#' plot_add adds a plot to a previous one
#' @title Add a plot to a previous one
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @param ... Parameters for plot()
#' @family plot and barplot functions
#' @description To plot data, just add use it as a normal plot. It will plot 
#' the new data without axes, or labels for axes.\cr
#' This function is complementary to matlines() and matpoints() from package graphics.
#' @examples
#' \dontrun{
#' plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="x", ylab="y")
#' plot_add(x=1:200, y=cos(1:200), type="l", bty="n", col="red")
#' }
#' @export


plot_add <- function(...) 
  {
  	par(new=TRUE)
	par.plot <- list(...)

	par.plot <- modifyList(par.plot, list(xlab="", ylab="", main="", axes=FALSE, 
	xlim= ScalePreviousPlot()$xlim[1:2], ylim=ScalePreviousPlot()$ylim[1:2]))
	do.call(plot, par.plot) 
  }

#' dbeta_new returns the density for the Beta distributions
#' @title Density for the Beta distributions. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return dbeta_new gives the density for the Beta distributions
#' @param x vector of quantiles.
#' @param mu mean of the Beta distribution.
#' @param v variance of the Beta distribution.
#' @param shape1 non-negative parameters of the Beta distribution.
#' @param shape2 non-negative parameters of the Beta distribution.
#' @param ncp non-centrality parameter.
#' @param log logical; if TRUE, probabilities p are given as log(p).
#' @param silent If FALSE, show the shape1 and shape 2 values.
#' @description Density for the Beta distribution with parameters mu and v or shape1 and 
#' shape2 (and optional non-centrality parameter ncp).\cr
#' The returned object has three attributes:\cr
#' shape1, shape2, and ncp\cr
#' Note that if x has other attributes, they are preserved.
#' @details The Beta distribution with parameters shape1 = a and shape2 = b has density \cr
#' gamma(a+b)/(gamma(a)gamma(b))x^(a-1)(1-x)^(b-1)\cr
#' for a > 0, b > 0 and 0 <= x <= 1 where the boundary values at x=0 or x=1 are defined as by 
#' continuity (as limits).\cr
#' The mean is a/(a+b) and the variance is ab/((a+b)^2 (a+b+1)). These moments and all 
#' distributional properties can be defined as limits.
#' @family Distributions
#' @examples
#' pi <- rbeta(100, shape1=0.48, shape2=0.12)
#' hist(pi, freq=FALSE, breaks=seq(from=0, to=1, by=0.1), ylim=c(0, 8), las=1)
#' library("HelpersMG")
#' mx <- ScalePreviousPlot()$ylim["end"]/
#'       max(dbeta_new(seq(from=0.01, to=0.99, by=0.01), mu = 0.8, v=0.1))
#' curve(dbeta_new(x, mu = 0.8, v=0.1)*mx, add=TRUE, col="red")
#' out <- dbeta_new(0.1, mu = 0.8, v=0.1)
#' out
#' attributes(out)$shape1; attributes(out)$shape2; attributes(out)$ncp
#' dbeta(0.1, shape1=attributes(out)$shape1, shape2=attributes(out)$shape2, 
#'       ncp=attributes(out)$ncp)
#' 
#' # It can be used to generate random numbers using mu and v
#' out <- dbeta_new(0.1, mu = 0.8, v=0.1, silent=TRUE)
#' pi <- rbeta(100, shape1=attributes(out)$shape1, shape2=attributes(out)$shape2, 
#'       ncp=attributes(out)$ncp)
#' hist(pi, freq=FALSE, breaks=seq(from=0, to=1, by=0.1), ylim=c(0, 8), las=1)
#' @export

dbeta_new <- function(x, mu=NULL, v=NULL, shape1, shape2, ncp = 0, log = FALSE, silent=FALSE) {
  if (!is.null(mu) & !is.null(v)) {
    shape1 <- mu*(((mu*(1-mu))/v)-1)
    shape2 <- shape1*(1-mu)/mu
    if (!silent) message(paste0("Equivalent to shape1=", as.character(shape1), " and shape2=", as.character(shape2)))
  }
  out <- dbeta(x=x, shape1=shape1, shape2 = shape2, ncp=ncp, log=log)
  
  if (is.null(attributes(out))) {
    attributes(out) <- list(shape1=shape1, shape2 = shape2, ncp=ncp)
  } else {
    attributes(out) <- modifyList(attributes(out), list(shape1=shape1, shape2 = shape2, ncp=ncp))
  }
  return(out)
}

#' Generalized gamma distribution
#' 
#' \code{pggamma}, \code{qggamma}, \code{dggamma}, and \code{rggamma} are used to model the 
#' generalized gamma distribution.
#' 
#' The code is modified from \url{https://rpubs.com/FJRubio/GG}.
#' 
# #' @name ggamma
#' @title Generalized gamma distribution. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @param x,q vector of quantiles.
#' @param p vector of probabilities.
#' @param n number of observations.
#' @param lower.tail logical; if TRUE (default), probabilities are \emph{P\[X <= x\]} otherwise, \emph{P\[X > x\]}.
#' @param theta scale parameter.
#' @param kappa shape parameter.
#' @param delta shape parameter.
#' @param log,log.p logical; if TRUE, probabilities \emph{p} are given as \emph{log(p)}.
#' @return \code{dggamma} gives the density, \code{pggamma} gives the distribution function, 
#' \code{qggamma} gives the quantile function, and \code{rggamma} generates random deviates.
#' @family Distributions
#' @examples
#' # To reproduce the wikipedia page graphic
#' x <- seq(from=0, to=8, by=0.1)
#' plot(x, dggamma(x, theta=2, kappa=0.5, delta=0.5), lty=1, col="blue", 
#'      type="l", lwd=2, xlab="x", ylab="PDF")
#' lines(x, dggamma(x, theta=1, kappa=1, delta=0.5), lty=1, col="green", lwd=2)
#' lines(x, dggamma(x, theta=2, kappa=1, delta=2), lty=1, col="red", lwd=2)
#' lines(x, dggamma(x, theta=5, kappa=1, delta=5), lty=1, col="yellow", lwd=2)
#' lines(x, dggamma(x, theta=7, kappa=1, delta=7), lty=1, col="grey", lwd=2)
#' legend("topright", legend=c("a=2, d=0.5, p=0.5", "a=1, d=1, p=0.5", 
#'                             "a=2, d=1, p=2", "a=5, d=1, p=5", "a=7, d=1, p=7"), 
#'                             col=c("blue", "green", "red", "yellow", "grey"), 
#'                             lty=1, lwd=2, bty="n")
#' par <- c(theta=2, kappa=0.5, delta=0.5)
#' # Mean, var and sd
#' mean.ggamma <- function(theta, kappa, delta) 
#'        return(theta*(gamma((kappa+1)/delta))/gamma(kappa/delta))
#' var.ggamma <- function(theta, kappa, delta) 
#'        return(theta^2* ( ( (gamma((kappa+2)/delta))/gamma(kappa/delta) ) - 
#'                 ( (gamma((kappa+1)/delta))/gamma(kappa/delta) )^2 ) )
#' sd.ggamma <- function(theta, kappa, delta) 
#'        return(sqrt(theta^2* ( ( (gamma((kappa+2)/delta))/gamma(kappa/delta) ) - 
#'                 ( (gamma((kappa+1)/delta))/gamma(kappa/delta) )^2 ) ))
#' @export


#' @section More details here:
#' The generalized gamma is described here \url{https://en.wikipedia.org/wiki/Generalized_gamma_distribution}.\cr
#' With \code{a} being \code{theta}, \code{b} being \code{kappa}, and \code{p} being \code{delta}.\cr
#' \code{theta}, \code{kappa} and \code{delta} must be all > 0.

#' @describeIn ggamma Density of the generalized gamma.
# #' @section Another section after function section:
dggamma <- function(x, theta, kappa, delta, log = FALSE){
  val <- log(delta) - kappa*log(theta) - lgamma(kappa/delta) + (kappa - 1)*log(x) -
    (x/theta)^delta
  if(log) return(val) else return(exp(val))
}
#' @export
#' @describeIn ggamma Distribution function of the generalized gamma.
# #' @section Another section after function section:
pggamma <- function(q, theta, kappa, delta, lower.tail = TRUE, log.p = FALSE){
  val <- pgamma( q^delta, shape = kappa/delta, scale = theta^delta, log.p = TRUE) 
  if (!lower.tail) val <- log(1 - exp(val))
  if(log.p) return(val) else return(exp(val))
}
#' @export

#' @describeIn ggamma Quantile of the generalized gamma.
# #' @section Another section after function section:
qggamma <- function(p, theta, kappa, delta, lower.tail = TRUE,
                    log.p = FALSE){
  if (log.p) p <- exp(p)
  if (!lower.tail) p <- 1 - p
  out <- qgamma(p, shape = kappa/delta, scale = theta^delta)^(1/delta)
  return(out)
}
#' @export

#' @describeIn ggamma Random of the generalized gamma.
# #' @section Another section after function section:
rggamma <- function(n, theta, kappa, delta){
  p <- runif(n)
  out <- qgamma(p, shape = kappa/delta, scale = theta^delta)^(1/delta)
  return(as.vector(out))
}
#' @export

#' asc returns the codes (in UTF-8) of a string
#' @title Return the codes (in UTF-8) of a string
#' @author Based on this blog: http://datadebrief.blogspot.com/2011/03/ascii-code-table-in-r.html
#' @return A vector with ITF-8 codes of a string
#' @param x The string to be analyzed
#' @description Return the codes (in UTF-8) of a string.
#' @family Characters
#' @examples
#' asc("abcd")
#' asc("ABCD")
#' @export

asc <- function(x) {strtoi(charToRaw(x),16L) }

#' series.compare compares series of data using Akaike weight.
#' @title Data series comparison using Akaike weight
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The probability that a single proportion model is sufficient to explain the data
#' @param ... Series of data (at least two or data are in a table with series in different rows)
#' @param criterion Which criterion is used for model selection. can be AIC, AICc or BIC
#' @param var.equal Should the variances of all series being equal? Default TRUE
#' @description This function is used as a replacement of t.test() to not use p-value. 
#' @references Girondot, M., Guillon, J.-M., 2018. The w-value: An alternative to t- and X2 tests. Journal of Biostatistics & Biometrics 1, 1-4.
#' @family w-value functions
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' A <- rnorm(100, 10, 2)
#' B <- rnorm(100, 11.1, 2)
#' series.compare(A, B, criterion = "BIC", var.equal=TRUE)
#' B <- B[1:10]
#' series.compare(A, B, criterion = "BIC", var.equal=TRUE)
#' A <- rnorm(100, 10, 2)
#' B <- rnorm(100, 10.1, 2)
#' C <- rnorm(100, 10.5, 2)
#' series.compare(A, B, C, criterion = "BIC", var.equal=TRUE)
#' B <- B[1:10]
#' series.compare(A, B, criterion = "BIC", var.equal=TRUE)
#' t.test(A, B, var.equal=TRUE)
#' # Example with a data.frame
#' series.compare(t(data.frame(A=c(10, 27, 19, 20, NA), B=c(10, 20, NA, NA, NA))))
#' # Test in the context of big data
#' A <- rnorm(10000, 10, 2)
#' B <- rnorm(10000, 10.1, 2)
#' series.compare(A, B, criterion = "BIC", var.equal=TRUE)
#' t.test(A, B, var.equal=TRUE)
#' ###########################
#' w <- NULL
#' p <- NULL
#' 
#' for (i in 1:1000) {
#'   
#'   A <- rnorm(50000, 10, 2)
#'   B <- rnorm(50000, 10.01, 2)
#'   w <- c(w, unname(series.compare(A, B, criterion = "BIC", var.equal=TRUE)[1]))
#'   p <- c(p, t.test(A, B, var.equal=TRUE)$p.value)
#' 
#' }
#' 
#' layout(mat = 1:2)
#' par(mar=c(4, 4, 1, 1)+0.4)
#' hist(p, main="", xlim=c(0, 1), las=1, breaks = (0:20)/20, 
#'      freq=FALSE, xlab = expression(italic("p")*"-value"))
#' hist(w, main="", xlim=c(0, 1), las=1, breaks = (0:20)/20, 
#'     freq=FALSE, xlab = expression(italic("w")*"-value"))
#' ###########################
#' 
#' x <- seq(from=8, to=13, by=0.1)
#' 
#' pv <- NULL
#' aw <- NULL
#' A <- rnorm(100, mean=10, sd=2)
#' B <- A-2
#' 
#' for (meanB in x) {
#'   pv <- c(pv, t.test(A, B, var.equal = FALSE)$p.value)
#'   aw <- c(aw, series.compare(A, B, criterion="BIC", var.equal = FALSE)[1])
#'   B <- B + 0.1
#' }
#' 
#' par(mar=c(4, 4, 2, 1)+0.4)
#' y <- pv
#' plot(x=x, y=y, type="l", lwd=2,
#'      bty="n", las=1, xlab="Mean B value (SD = 4)", ylab="Probability", ylim=c(0,1), 
#'      main="")
#' y2 <- aw
#' lines(x=x, y=y2, type="l", col="red", lwd=2)
#' 
# w-value
#' l1 <- which(aw>0.05)[1]
#' l2 <- max(which(aw>0.05))
#' 
#' aw[l1]
#' pv[l1]
#' 
#' aw[l2]
#' pv[l2]
#' 
# p-value
#' l1 <- which(pv>0.05)[1]
#' l2 <- max(which(pv>0.05))
#' 
#' aw[l1]
#' pv[l1]
#' 
#' aw[l2]
#' pv[l2]
#' 
#' par(xpd=TRUE)
#' segments(x0=10-1.96*2/10, x1=10+1.96*2/10, y0=1.1, y1=1.1, lwd=2)
#' segments(x0=10, x1=10, y0=1.15, y1=1.05, lwd=2)
#' par(xpd=TRUE)
#' text(x=10.5, y=1.1, labels = "Mean A = 10, SD = 2", pos=4)
#' 
#' v1 <- c(expression(italic("p")*"-value"), expression("based on "*italic("t")*"-test"))
#' v2 <- c(expression(italic("w")*"-value for A"), expression("and B identical models"))
#' legend("topright", legend=c(v1, v2), 
#'        y.intersp = 1, 
#'        col=c("black", "black", "red", "red"), bty="n", lty=c(1, 0, 1, 0))
#' 
#' segments(x0=min(x), x1=max(x), y0=0.05, y1=0.05, lty=2)
#' par(xpd = TRUE)
#' text(x=13.05, y=0.05, labels = "0.05", pos=4)
#' }
#' @export


series.compare <- function(..., criterion = c("BIC", "AIC", "AICc"), var.equal = TRUE) {
  
  # data <- list(A, B); criterion = "BIC"; var.equal=TRUE
  
  data <- list(...)
  if (length(data) == 1) {
    datax <- data[[1]]
    data <- list()
    for (i in 1:nrow(datax)) data <- c(data, list(datax[i, !is.na(datax[i, ])]))
  }
  
  if (length(data) == 1) stop("At least two series must be used for comparison")
  
  # data <- list(A, B)
  n <- length(data)
  dtot <- unlist(data)
  sdtot <- sd(dtot)
  n_tot <- length(dtot)
  
  AIC_s <- 0
  BIC_s <- 0
  AICc_s <- 0
  BIC <- 0
  AIC <- 0
  AICc <-  0
  
  nk_AIC <- 1+ifelse(var.equal, 1, n)
  nk_AIC_s <- n+ifelse(var.equal, 1, n)
  nk_BIC <- 1+ifelse(var.equal, 1, n)
  nk_BIC_s <- n+ifelse(var.equal, 1, n)
  nk_AICc <- 1+ifelse(var.equal, 1, n)
  nk_AICc_s <- n+ifelse(var.equal, 1, n)
  
  
  for (nec in 1:n) {
    logL <- sum(dnorm(data[[nec]], mean=mean(data[[nec]]), sd=ifelse(var.equal, sdtot, sd(data[[nec]])), log=TRUE))
    BIC_s <- BIC_s -2*logL
    AIC_s <- AIC_s -2*logL
    AICc_s <- AICc_s -2*logL
    logL <- sum(dnorm(data[[nec]], mean=mean(dtot), sd=ifelse(var.equal, sdtot, sd(data[[nec]])), log=TRUE))
    BIC <- BIC -2*logL
    AIC <- AIC -2*logL
    AICc <-AICc -2*logL
  }
  
  AIC_s <- AIC_s + 2*nk_AIC_s
  AIC <- AIC + 2*nk_AIC
  BIC_s <- BIC_s + nk_BIC_s*log(n_tot)
  BIC <- BIC + nk_BIC*log(n_tot)
  AICc <- AICc + 2*nk_AICc + (2*nk_AICc*(nk_AICc+1))/(n_tot-nk_AICc-1)
  AICc_s <- AICc_s + 2*nk_AICc_s + (2*nk_AICc_s*(nk_AICc_s+1))/(n_tot-nk_AICc_s-1)
  
  
  crit <- NULL
  AICmin <- min(c(AIC, AIC_s))
  AICcmin <- min(c(AICc, AICc_s))
  BICmin <- min(c(BIC, BIC_s))
  
  if (any(criterion=="AIC")) crit <- c("AICw(identical)"=exp(-0.5*(AIC-AICmin))/(exp(-0.5*(AIC-AICmin))+exp(-0.5*(AIC_s-AICmin))), 
                                       "AICw(different)"=1-(exp(-0.5*(AIC-AICmin))/(exp(-0.5*(AIC-AICmin))+exp(-0.5*(AIC_s-AICmin))))
                                       )
  if (any(criterion=="BIC")) crit <- c(crit, "BICw(identical)"=exp(-0.5*(BIC-BICmin))/(exp(-0.5*(BIC-BICmin))+exp(-0.5*(BIC_s-BICmin))), 
                                                    "BICw(different)"=1-(exp(-0.5*(BIC-BICmin))/(exp(-0.5*(BIC-BICmin))+exp(-0.5*(BIC_s-BICmin))))                  
                                       )
  if (any(criterion=="AICc")) crit <- c(crit, "AICcw(identical)"=exp(-0.5*(AICc-AICcmin))/(exp(-0.5*(AICc-AICcmin))+exp(-0.5*(AICc_s-AICcmin))), 
                                        "AICcw(different)"=1-(exp(-0.5*(AICc-AICcmin))/(exp(-0.5*(AICc-AICcmin))+exp(-0.5*(AICc_s-AICcmin))))
                                        )
  
  return(crit)
}

# library(shiny); runApp("/Users/marcgirondot/Documents/Espace_de_travail_R/_shiny/BoneProfileR")
# library(shiny); runApp("http://134.158.74.46/BoneProfileR/")


library(shiny)
if (!requireNamespace("shinyWidgets", quietly = TRUE)) {
  progressbar <- FALSE
} else {
  progressbar <- TRUE
  library("shinyWidgets")
}

package.HelpersMG <- require('HelpersMG')
version <- "2026.8.24 build 1817"

splitLayout <- function (..., cellWidths = NULL, cellArgs = list()) 
{
  children <- list2(...)
  childIdx <- !nzchar(names(children) %||% character(length(children)))
  attribs <- children[!childIdx]
  children <- children[childIdx]
  count <- length(children)
  if (length(cellWidths) == 0 || any(is.na(cellWidths))) {
    cellWidths <- sprintf("%.3f%%", 100/count)
  }
  cellWidths <- rep(cellWidths, length.out = count)
  cellWidths <- sapply(cellWidths, validateCssUnit)
  do.call(tags$div, c(list(class = "shiny-split-layout"), attribs, 
                      mapply(children, cellWidths, FUN = function(x, w) {
                        do.call(tags$div, c(list(style = sprintf("width: %s;", 
                                                                 w)), cellArgs, list(x)))
                      }, SIMPLIFY = FALSE)))
}


# Define UI for application that draws a histogram
fluidPage(
  titlePanel(h1("Cutte",
                img(src="Rlogo.png", height=40, width=40), align = "center"), 
             windowTitle = "CutteR"), 
  p(HTML("<b><a href=\"https://hebergement.universite-paris-saclay.fr/marcgirondot/\">Marc Girondot</a></b> - Laboratoire Ecologie, Systématique, Evolution"), align = "center"),
  p(HTML("Université Paris-Saclay, CNRS, AgroParisTech, France."), align = "center"), 
  
  wellPanel(
    
    p(HTML("<strong>CutteR is a scientific method and a software used 
  to model left and right censored and truncated data.</strong>"), align = "left"),
    p("It used a Bayesian MCMC distribution fit using conditional gamma, lognormal, normal, Weibull, or generalized gamma distribution. The priors are based on Gaussian distribution of parameters obtained from maximum likelihood with large standard deviation. The posterior distribution is obtained with 5000 MCMC iterations."), 
    p(), 
    p(HTML("<strong>Copy data in the box below, select the required options and 
         click 'Run the analysis' button.</strong>"), align = "left")
  ), 
  
  # Show a plot of the generated distribution
  wellPanel(
    textAreaInput(inputId="Data", label="Copy the data here, separated by space, semicolon, tabulation, or return; unknown values must be LDL (Lower the Detection Limit) or UDL (Upper the Detection Limit). Do not use comma to separate values as commas are used as synonyme for decimal point.", 
                  value="",
                  rows = 10, 
                  width="100%", 
                  placeholder=""), 
    p("To try the method, copy these values and set a Lower limit of detection to 10:"), 
    p("89.77 12.75 44.18 60.66 32.68 34.57 28.09 91.11 13.98 56.95 59.64 76.22 84.64 61.45 27.50 17.27 16.39 22.10 15.81 62.86 86.39 50.86 26.33 29.43 53.95 40.94 64.13 19.49 40.26 44.03 66.70 22.10 21.07 36.57 70.29 50.20 54.13 13.11 22.76 40.04 28.49 LDL 13.98 LDL 62.24 55.50 15.30 11.29 14.39 40.88 26.31 33.04 19.02 47.35 56.16 24.00 18.66 20.52 61.26 LDL 15.45 41.65 77.76 71.66 22.13 LDL 46.61 39.65 58.96 LDL 28.01 83.33 64.28 24.31 22.52 LDL 64.47 62.77 11.45 32.91 39.11 71.29 56.73 67.44 32.42 45.63 32.60 60.96 16.38 LDL 12.26 143.26 18.80 13.28 29.26 50.78 15.98 45.35 46.43 89.13")
  ), 
  
  splitLayout(
    textInput(inputId="LDL", label="Lower limit of detection", value="")
    , textInput(inputId="UDL", label="Upper limit of detection", value="")
    , cellWidths=c("50%", "50%")
  ), 
  p("Let a box being empty if the option is not necessary."),
  splitLayout(
    selectInput("censored", label="Choose which cut will be fitted "
                , choices=list("Censored"="censored", 
                               "Truncated"= "truncated")
                , selected = "censored", multiple = FALSE,
                selectize = FALSE, size = NULL),
    selectInput("mixture", label="How many mixture should be used "
                , choices=list("1" = "1", 
                               "2" = "2", 
                               "3" = "3")
                , selected = "1", multiple = FALSE,
                selectize = FALSE, size = NULL),
    selectInput("distribution", label="Choose which distribution will be fitted "
                , choices=list("Auto" ="auto", 
                               "Gamma"="gamma", 
                               "Lognormal"= "lognormal", 
                               "Normal"= "normal", 
                               "Weibull" = "weibull", 
                               "Generalized gamma" = "generalized.gamma")
                , selected = "auto", multiple = FALSE,
                selectize = FALSE, size = NULL),
    cellWidths=c("33%", "33%", "34%")
  ), 
  
  actionButton(inputId="goButton", label="Run the analysis", width="30%", 
               icon("paper-plane"), 
               style="color: #fff; background-color: #337ab7; border-color: #2e6da4"),
  HTML("<i>Be patient, the analysis requires between 30 seconds to 5 minutes to be completed.</i>"), 
  p(""), 
  if (progressbar) {progressBar(
    id = "pb2",
    value = 0,
    total = 10000,
    title = "",
    display_pct = TRUE
  )},
  wellPanel(
    
    textOutput(outputId="Result"), 
    tags$style(type="text/css", "#Result {white-space: pre-wrap;}"), 
    plotOutput("Plot")
  ), 
  wellPanel(
    p(paste0("This web server version v. ",version, " is a simplified version of the complete tools available in the HelpersMG R package.")), 
    HTML('<img src="http://perso0.free.fr/cgi-bin/wwwcount.cgi?df=Cutter.dat&dd=C&ft=0">'), 
    p(HTML("<small><i><font color='#006699'>The Virtual Data initiative, run by LABEX P2IO and supported by Université Paris-Saclay, is thanked for providing computing resources on its cloud infrastructure.</font></i></small>"))
  )
  
)

#' @title Extract quantile distribution from mcmcComposite object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A data.frame with quantiles
#' @param x A mcmcComposite obtained as a result of \code{MHalgoGen()} function
#' @param chain The number of the chain in which to get parameters or "all"
#' @param fun The function to apply the parameters
#' @param probs The probability to get quantiles
#' @param xlim The values to apply in fun
#' @param nameparxlim The name of the parameter for xlim
#' @param namepar The name of parameters from mcmc object to be used in fun
#' @description Extract quantile distribution from mcmcComposite object
#' @family mcmcComposite functions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' require(coda)
#' x <- rnorm(30, 10, 2)
#' dnormx <- function(data, x) {
#'  data <- unlist(data)
#'  return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
#' }
#' parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
#' Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(1, 1), 
#' Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
#' row.names=c('mean', 'sd'))
#' mcmc_run <- MHalgoGen(n.iter=10000, parameters=parameters_mcmc, data=x, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
#' k <- as.quantiles(x=mcmc_run, namepar="mean")
#' k <- as.quantiles(x=mcmc_run, namepar="mean", 
#'                  xlim=c(1:5), nameparxlim="sd", 
#'                  fun=function(...) return(sum(as.numeric(list(...)))))
#' }
#' @export


as.quantiles <- function(x, chain="all", fun=function(...) return(as.numeric(list(...))), 
                            probs=c(0.025, 0.975), xlim=NULL, 
                            nameparxlim=NULL, namepar=NULL) {
  
  # chain=1; fun=function(x) {x}; probs=c(0.025, 0.975); xlim=NULL; nameparxlim=NULL; namepar=NULL
  if (chain[1] == "all") chain <- seq_along(x$resultMCMC)
  
  # if (total) p <- x$resultMCMC.total[[chain[1]]] else p <- x$resultMCMC[[chain[1]]]
  
  p <- NULL
  
  if (length(chain) >= 1) {
    for (i in chain) {
      p <- rbind(p, x$resultMCMC[[i]])
    }
  }
  
  if (is.null(p)) stop("No data are available.")
  
	# p <- x$resultMCMC[[chain]]
	df <- matrix(NA, ncol=max(1, length(xlim)))
	df <- df[-1, ]
	
	for (l in 1:nrow(p)) {
	  if (length(xlim) == 0) {
	    df <- rbind(df, rep(NA, 1))
	    par <- p[l, ]
	    if (!is.null(namepar)) par <- par[namepar]
	    parx <- as.list(par)
	    h <- do.call(fun, args=parx)
	    df[l, 1] <- h
	  } else {
	    par <- p[l, ]
	    if (!is.null(namepar)) par <- par[namepar]
	    df <- rbind(df, rep(NA, length(xlim)))
	    for (i in 1:length(xlim)) {
	      m <- xlim[i]
	      names(m) <- nameparxlim
	      parx <- as.list(c(m, par))
	      h <- do.call(fun, args=parx)
	      df[l, i] <- h
	    }
	  }
	}

	q <- apply(df, MARGIN = 2, FUN = function(x) quantile(x, probs))
	colnames(q) <- as.character(xlim)
	return(q)

}

#' Make a matrix symmetric
#' 
#' Make the matrix symmetric by making all "mirrored" positions consistent. A variety of methods are provided to make the matrix symmetrical.
#' @param matrix The matrix to make symmetric
#' @param method The method to use to make the matrix symmetric. Default is to take the maximum.
#' \describe{
#' 	\item{"max"}{For each position, \eqn{m_{i,j}}, use the maxiumum of \eqn{(m_{i,j}, m_{j,i})}}
#' 	\item{"min"}{For each position, \eqn{m_{i,j}}, use the minimum of \eqn{(m_{i,j}, m_{j,i})}}
#' 	\item{"avg"}{For each position, \eqn{m_{i,j}}, use the mean: \eqn{(m_{i,j} + m_{j,i})/2}}
#' 	\item{"ld"}{Copy the lower triangular portion of the matrix to the upper triangular portion.}
#' 	\item{"ud"}{Copy the upper triangular portion of the matrix to the lower triangular portion.}
#' }
#' @param adjacencyList Logical. If false, returns the symmetric matrix (the same format as the input). If true, returns an adjacency list representing the upper triangular portion of the adjacency matrix with addressing based on the row.names of the matrix provided.
#' @return The symmetric matrix
#' @description This function was part of the package ENA. This package is no more available
#' and it cannot be installed from archive because some dependencies are no more
#' available.\cr
#' @export
#' @author Jeffrey D. Allen \email{Jeffrey.Allen@@UTSouthwestern.edu}
#' @examples
#' #Create a sample 3x3 matrix
#' mat <- matrix(1:9, ncol=3)
#' 
#' #Copy the upper diagonal portion to the lower
#' symmetricize(mat, "ud")
#' 
#' #Take the average of each symmetric location
#' symmetricize(mat, "avg")
#' 
symmetricize <-
function(matrix, method=c("max", "min","avg", "ld", "ud"), adjacencyList=FALSE){
	method <- match.arg(method)
	
	if (missing(matrix)){
		stop("You must provide a matrix to symmetricize.")
	}
	
	x <- matrix
	if (method=="ld"){
		#solution from Michael Conklin, http://www.biostat.wustl.edu/archives/html/s-news/2000-03/msg00127.html
		x[matrix(c(col(x)[lower.tri(x)], row(x)[lower.tri(x)]), ncol = 2)] <-
			x[matrix(c(row(x)[lower.tri(x)], col(x)[lower.tri(x)]), ncol = 2)]
	}
	if (method=="ud"){				
		x[matrix(c(col(x)[upper.tri(x)], row(x)[upper.tri(x)]), ncol = 2)] <-
			x[matrix(c(row(x)[upper.tri(x)], col(x)[upper.tri(x)]), ncol = 2)]
	}
	if (method=="max" || method=="min" || method=="avg"){
		#retrieve the (row,col) indices for the lower-diagonal entries in the matrix
		ldi <- matrix(c(row(x)[lower.tri(x)], col(x)[lower.tri(x)]), ncol = 2)
			#already column-major, so we can leave this
		
		#retrieve the (row,col) indices for the upper-diagonal entries in the matrix
		udi <- matrix(c(row(x)[upper.tri(x)], col(x)[upper.tri(x)]), ncol = 2)
			#in order for these to be in the symmetrical order as ldi, we need to sort.
		udi <- udi[order(udi[,1], udi[,2]),]
		
		#extract the upper and lower diagonal elements in a way that's symmetrical in their indexces
		ud <- x[udi]
		ld <- x[ldi]
		
		#replace with either the min, max, or mean, depending on the selection
		if (method=="max"){			
			x[ldi] <- apply(rbind(ld,ud),2,max);
			x[udi] <- apply(rbind(ld,ud),2,max);
		}
		if (method=="min"){
			x[ldi] <- apply(rbind(ld,ud),2,min);
			x[udi] <- apply(rbind(ld,ud),2,min);
		}
		if (method=="avg"){
			x[ldi] <- apply(rbind(ld,ud),2,mean);
			x[udi] <- apply(rbind(ld,ud),2,mean);
		}
	}	
	
	if (!adjacencyList){
		#return the adjacency matrix
		return(x)
	}
	else{
		#convert to adjacency list and add addressing
		if (is.null(rownames(matrix))){
			stop("You requested adjacency list format, but the matrix provided has no (row) names.")
		}		
		names <- rownames(matrix)		
		add <- getFromNamespace(".getTableAddressing", ns="HelpersMG")(names)
		toReturn <- (cbind(add, x[upper.tri(x)]))		
		colnames(toReturn) <- c("Source", "Dest", "Symmetric")
		return(toReturn)
	}
}

#' predict.LD50 Estimate survival according to doses
#' @title Estimate survival according to doses
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A data.frame with informations about survival
#' @param object A result file generated by LD50
#' @param doses A vector of temperatures
#' @param SE The standard error for doses, optional
#' @param range.CI The range of confidence interval for estimation, default=0.95
#' @param replicates Number of replicates to estimate CI
#' @param progressbar Logical. Does a progression bar must be shown
#' @param ... Not used
#' @description Estimate survival according to doses.\cr
#' The returned data.frame has the following components:\cr
#' doses, SE, survival, CI.minus.sexratio, CI.plus.sexratio, range.CI\cr
#' @family LD50 functions
#' @examples
#' \dontrun{
#' #' data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
#' Alive=c(10, 12, 8, 6, 2, 1),
#' Dead=c(0, 1, 5, 6, 9, 15))
#' LD50_logistic <- LD50(data, equation="logistic")
#' predict(LD50_logistic, doses=c(140, 170))
#' plot(LD50_logistic
#' }
#' @method predict LD50
#' @export


predict.LD50 <-
function(object, doses=NULL, SE=NULL, 
	range.CI=0.95, replicates=1000, progressbar=FALSE, ...) {
  
  # durations=NULL; SE=NULL; range.CI=0.95; replicates=1000; progressbar=FALSE
  

  if (is.null(doses)) doses <- object$doses
  
  if (!is.null(SE) & length(SE) != length(doses)) {
   stop("SE must be of the same length than doses")
  }
   
  x <- object
	equation <- x$equation
	par <- c(x$par, x$fixed.parameters)
	res <- x$SE
	

if (is.null(range.CI) | is.infinite(range.CI) | range.CI>1 | range.CI<0) {
  warning("The parameter range.CI is changed to 0.95")
  range.CI <- 0.95
}

    if (progressbar) pb<-txtProgressBar(min=1, max=length(doses), style=3)

    
    rep <- replicates-1
    df_par <- data.frame(P=unname(c(par["P"], rnorm(rep, par["P"], ifelse(is.na(res["P"]), 0, res["P"])))), 
                         S=unname(c(par["S"], rnorm(rep, par["S"], ifelse(is.na(res["S"]), 0, res["S"])))),
                         K=unname(ifelse(is.na(par["K"]), rep(NA, replicates), c(par["K"], rnorm(rep, par["K"], ifelse(is.na(res["K"]), 0, res["K"]))))),
                         K1=unname(ifelse(is.na(par["K1"]), rep(NA, replicates), c(par["K1"], rnorm(rep, par["K1"], ifelse(is.na(res["K1"]), 0, res["K1"]))))),
                         K2=unname(ifelse(is.na(par["K2"]), rep(NA, replicates), c(par["K2"], rnorm(rep, par["K2"], ifelse(is.na(res["K2"]), 0, res["K2"]))))))

    if (is.null(SE)) {
      SE <- rep(0, length(doses))
      replicates <- 1
    }
    
    df.out <- data.frame(doses=numeric(), 
                        SE=numeric(),
                        sexratio=numeric(), 
                        CI.minus.sexratio=numeric(), 
                        CI.plus.sexratio=numeric(), 
                        range.CI=numeric())
    
    for(i in 1:length(doses)) {
      if (progressbar) setTxtProgressBar(pb, i)
        doses_ec <- c(doses[i], rnorm(replicates-1, doses[i], SE[i]))
        out_tsd_plot <- apply(df_par, 1, function(par) {
        p <- .modelLD50(par, doses_ec, equation)
        return(list(sr=p))
      }
      )
      
      out_tsd3_plot <- unlist(out_tsd_plot)
      outquant <- quantile(out_tsd3_plot, probs = c((1-range.CI)/2, range.CI+((1-range.CI)/2)))
 #     outquant <- rbind(outquant, mean=out_tsd3_plot[1,], doses=doses_ec)
      df.out <- rbind(df.out, data.frame(doses=unname(doses[i]), 
                                SE=unname(SE[i]),
                                survival=unname(out_tsd3_plot[1]),
                                CI.minus.survival=unname(outquant[1]),
                                CI.plus.survival=unname(outquant[2]),
                                range.CI=range.CI))
      
    }

return(df.out)
}

#' local.search() returns path of file serached in local disk based on its file name
#' @title Return path of file searched for in local disk based on its file name
#' @author Marc Girondot
#' @return A vector with paths
#' @param pattern The name of file to be searched for. Can use wildcards *
#' @param directory The path of directory to be explored in for Windows
#' @param folder The path of folder to be explored in for Unix based systems
#' @param intern A logical (not NA) which indicates whether to capture the output of the command as an R character vector (see system()).
#' @param ignore.stdout a logical (not NA) indicating whether messages written to 'stdout' should be ignored  (see system()).
#' @param ignore.stderr a logical (not NA) indicating whether messages written to 'stderr' should be ignored  (see system()).
#' @description Return path of file searched for in local disk based on its file name.\cr
#' It has been tested only with Windows XP and MacOSX. In MacOSX, you must have created the locate database first. Use OnyX utilities for this purpose.
#' @examples
#' \dontrun{
#' RnwFiles <- local.search("*.Rnw")
#' nc.files <- local.search("*.nc", folder=paste0("'",getwd(),"'"))
#' }
#' @export


local.search <- function(pattern, directory="", folder="$HOME", intern=TRUE, ignore.stdout=FALSE, ignore.stderr=TRUE) {
  if (.Platform$OS.type=="unix") {
    command <- paste0("find ", folder," -type f -name '", pattern, "'")
    dest <- suppressWarnings(system(command, intern=intern, ignore.stdout=ignore.stdout, ignore.stderr=ignore.stderr))
    } else {
    dest <- shell(paste0("dir ", directory, pattern, " /b/s "), 
                          intern=intern, ignore.stdout=ignore.stdout, ignore.stderr=ignore.stderr)
  }
  if (length(dest) == 0) {return(NULL)} else {return(dest)}
}

#' plot_errbar plot a xy graph with error bar on x and/or y
#' @title Plot a xy graph with error bar on x and/or y
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with x, y and names for points
#' @param ... Parameters for plot() such as main= or ylim=
#' @param errbar.x The length of error bars for x. Recycled if necessary.
#' @param errbar.x.plus The length of positive error bars for x. Recycled if necessary.
#' @param errbar.x.minus The length of negative error bars for x. Recycled if necessary.
#' @param errbar.y The length of error bars for y. Recycled if necessary.
#' @param errbar.y.plus The length of positive error bars for y. Recycled if necessary.
#' @param errbar.y.minus The length of negative error bars for y. Recycled if necessary.
#' @param x.plus The absolut position of the positive error bar for x. Recycled if necessary.
#' @param x.minus The absolut position of the negative error bar for x. Recycled if necessary.
#' @param y.plus The absolut position of the positive error bar for y. Recycled if necessary.
#' @param y.minus The absolut position of the nagative error bar for y. Recycled if necessary.
#' @param errbar.tick Size of small ticks at the end of error bars defined as a proportion of total width or height graph size.
#' @param errbar.lwd Error bar line width, see par("lwd")
#' @param errbar.lty Error bar line type, see par("lwd")
#' @param errbar.col Error bar line color, see par("col")
#' @param errbar.y.polygon If true, the errors are shown as a filed polygon.
#' @param errbar.y.polygon.list List of parameters to be used for polygon.
#' @param names The names of the points to be used with show_name().
#' @param add If true, add the graph to the previous one.
#' @family plot and barplot functions
#' @seealso \code{barplot_errorbar}
#' @description To plot data, just use it as a normal plot but add the errbar.x 
#' and errbar.y values or errbar.x.minus, errbar.x.plus if bars for x axis are 
#' asymetric and errbar.y.minus, errbar.y.plus if bars for y axis are 
#' asymetric. Use x.plus, x.minus, y.plus and y.minus to set absolut limits for
#' error bars. Note that x.plus and x.minus have priority over errbar.x, errbar.x.minus and
#' errbar.x.plus and that y.plus and y.minus have priority over errbar.y, errbar.y.minus and
#' errbar.y.plus.\cr
#' The parameter errbar.y.polygon=TRUE permits to define error as an envolop for y axis.
#' @examples
#' \dontrun{
#' plot_errbar(1:100, rnorm(100, 1, 2), 
#'		xlab="axe x", ylab="axe y", bty="n", xlim=c(1,100), 
#' 		errbar.x=2, errbar.y=rnorm(100, 1, 0.1))
#' x <- 1:100
#' plot_errbar(x=1:100, rnorm(100, 1, 2), 
#'                	xlab="axe x", ylab="axe y", bty="n", xlim=c(1,100), 
#'             		x.minus=x-2, x.plus=x+2)
#' x <- 1:100
#' plot_errbar(x=1:100, rnorm(100, 1, 2), 
#'                	xlab="axe x", ylab="axe y", bty="n", 
#'                	pch=21, bg="white", 
#'             		x.minus=x-10, x.plus=x+10)
#' x <- (1:200)/10
#' y <- sin(x)
#' plot_errbar(x=x, y=y, xlab="axe x", ylab="axe y", bty="n", xlim=c(1,20), 
#'      y.minus=y-1, y.plus=y+1, ylim=c(-3, 3), type="l",  
#' 		errbar.y.polygon=TRUE, 
#' 		errbar.y.polygon.list=list(border=NA, col=rgb(0, 0, 0, 0.5)))
#' 		}
#' @export


plot_errbar <- function(..., 
                        errbar.x=NULL, errbar.y=NULL, 
                        errbar.x.plus=NULL, errbar.x.minus=NULL, 
                        errbar.y.plus=NULL, errbar.y.minus=NULL,
                        x.plus=NULL, x.minus=NULL,
                        y.plus=NULL, y.minus=NULL,
                        errbar.tick=1/50, 
                        errbar.lwd=par("lwd"), 
                        errbar.lty=par("lty"), 
                        errbar.col=par("fg"), 
                        errbar.y.polygon=FALSE, 
                        errbar.y.polygon.list=list(NULL), 
                        names=NULL, 
                        add=FALSE) 
{
  
  # errbar.x=NULL; errbar.y=NULL; errbar.x.plus=NULL; errbar.x.minus=NULL; errbar.y.plus=NULL; errbar.y.minus=NULL; x.plus=NULL; x.minus=NULL; y.plus=NULL; y.minus=NULL; errbar.tick=1/50; errbar.lwd=par("lwd"); errbar.lty=par("lty"); errbar.col=par("fg"); errbar.y.polygon=FALSE; errbar.y.polygon.list=list(NULL); add=FALSE
  # par.plot <- list(x=x.axis, y=CTE, las=1, type="l", xlim=c(as.Date("1997-01-01"), as.Date("2014-01-01")), ylim=c(28, 32), bty="n", xlab="Year", ylab=expression("constant incubation temperature (" *degree*"C)"), xaxt="n")
  # y.plus=CTE.plus
  # y.minus=CTE.moins
  # errbar.y.polygon=TRUE
  # errbar.y.polygon.list=list(border=NA, col=rgb(0, 0, 0, 0.5))
  # par.plot <- list(1:100, rnorm(100, 1, 2),xlab="axe x", ylab="axe y", bty="n", xlim=c(1,100));errbar.x=2; errbar.y=rnorm(100, 1, 0.1)
  
  par.plot <- list(...)
  
  par(xpd=FALSE)
  
  x <- par.plot[["x"]]
  if (is.null(x)) {
    x <- par.plot[[1]]
    names(par.plot)[1] <- "x"
  }
  
  if (is.data.frame(x) | is.matrix(x)) {
    y <- x[,2]
    x <- x[,1]
  } else {
    y <- par.plot[["y"]]
  }
  if (is.null(y)) {
    y <- par.plot[[2]]
    names(par.plot)[2] <- "y"
  }
  
  if (!is.null(x.plus)) errbar.x.plus <- x.plus-x
  if (!is.null(x.minus)) errbar.x.minus <- x-x.minus
  if (!is.null(y.plus)) errbar.y.plus <- y.plus-y
  if (!is.null(y.minus)) errbar.y.minus <- y-y.minus
  
  if (is.null(errbar.x.minus) & !is.null(errbar.x)) {
    errbar.x.minus <- errbar.x
  }
  if (is.null(errbar.x.plus) & !is.null(errbar.x)) {
    errbar.x.plus <- errbar.x
  }
  if (is.null(errbar.y.minus) & !is.null(errbar.y)) {
    errbar.y.minus <- errbar.y
  }
  if (is.null(errbar.y.plus) & !is.null(errbar.y)) {
    errbar.y.plus <- errbar.y
  }
  
  if (add) {
    # Si je superpose le graphique à un précédent:
    s <- ScalePreviousPlot()
    par(new=TRUE)
    # Je retire les axes et je fixe xlim et ylim
    pp <- modifyList(par.plot, list(xlim=s$xlim[1:2], ylim=s$ylim[1:2], xlab="", ylab="", main="", axes=FALSE))
    do.call(plot, modifyList(pp, list(type="n")))
    
  } else {
    # Je fais un nouveau graphique
    # Là il n'y a pas de xlim, ylim; je le laisse calculer
    pp <- par.plot
    ebxm <- errbar.x.minus
    if (is.null(ebxm)) ebxm <- 0
    ebxm <- ifelse(is.na(ebxm), 0, ebxm)
    ebxp <- errbar.x.plus
    if (is.null(ebxp)) ebxp <- 0
    ebxp <- ifelse(is.na(ebxp), 0, ebxp)
    
    pp <- modifyList(pp, list(x=c(min(x-ebxm, na.rm = TRUE), 
                                  max(x+ebxp, na.rm = TRUE))
    ))
    
    ebym <- errbar.y.minus
    if (is.null(ebym)) ebym <- 0
    ebym <- ifelse(is.na(ebym), 0, ebym)
    ebyp <- errbar.y.plus
    if (is.null(ebyp)) ebyp <- 0
    ebyp <- ifelse(is.na(ebyp), 0, ebyp)
    
    pp <- modifyList(pp, list(y=c(min(y-ebym, na.rm = TRUE), 
                                  max(y+ebyp, na.rm = TRUE))
    ))
    
    do.call(plot, modifyList(pp, list(type="n")))
    
    # Je rajoute xlim et ylim sur par.plot: 2020-04-02
    
    s <- ScalePreviousPlot()
    par.plot <- modifyList(par.plot, list(xlim=s$xlim[1:2], ylim=s$ylim[1:2]))
  }
  
  
  
  
  if (errbar.y.polygon) {
    # je dois faire un polygon
    # Dans ce cas, c'est bon
    vx <- c(x, rev(x))
    vy <- c(y-errbar.y.minus, rev(y+errbar.y.plus))
    errbar.y.polygon.list <- modifyList(errbar.y.polygon.list, list(x=vx, y=vy))
    do.call(polygon, errbar.y.polygon.list)
    
    
  } else {
    
    sizebar <- (par("usr")[4]-par("usr")[3])*errbar.tick
    
    # Je fais les barres d'erreur
    if (!is.null(errbar.x.minus)) {
      segments(x-errbar.x.minus, y, x, y, 
               col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
      segments(x-errbar.x.minus, y-sizebar, x-errbar.x.minus, y+sizebar, 
               col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
    }
    if (!is.null(errbar.x.plus)) {
      segments(x+errbar.x.plus, y, x, y, 
               col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
      segments(x+errbar.x.plus, y-sizebar, x+errbar.x.plus, y+sizebar, 
               col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
    }
    
    sizebar <- (par("usr")[2]-par("usr")[1])*errbar.tick
    
    if (!is.null(errbar.y.minus)) {
      segments(x, y-errbar.y.minus, x, y, 
               col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
      segments(x-sizebar, y-errbar.y.minus, x+sizebar, y-errbar.y.minus, 
               col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
    }
    if (!is.null(errbar.y.plus)) {
      segments(x, y+errbar.y.plus, x, y, 
               col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
      segments(x-sizebar, y+errbar.y.plus, x+sizebar, y+errbar.y.plus, 
               col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
    }
  }
  
  # Et là je fais le plot
  do.call(plot_add, par.plot)
  
  if (is.null(names)) names <- paste0(as.character(x), ";", as.character(y))
  
  return(invisible(list(x=x, y=y, names=names)))
  
}

#' modeled.hist returns the theoretical value for the histogram bar based on a model of distribution.
#' @title Return the theoretical value for the histogram bar
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with x (the center of the bar) and y components
#' @param breaks Vector with the breaks; it can be obtained directly from hist()
#' @param FUN Function to be used to integrate the density, ex. pnorm
#' @param sum Total numbers in the histogram; 1 for emperical frequencies
#' @param ... Parameters to be used by FUN
#' @description Return the theoretical value for the histogram bar based on
#' a model of distribution.
#' @examples
#' \dontrun{
#' n <- rnorm(100, mean=10, sd=2)
#' breaks <- 0:20
#' hist(n, breaks=breaks)
#' 
#' s <- modeled.hist(breaks=breaks, FUN=pnorm, mean=10, sd=2, sum=100)
#' 
#' points(s$x, s$y, pch=19)
#' lines(s$x, s$y)
#' 
#' n <- rlnorm(100, meanlog=2, sdlog=0.4)
#' b <- hist(n, ylim=c(0, 70))
#' 
#' s <- modeled.hist(breaks=b$breaks, FUN=plnorm, meanlog=2, sdlog=0.4, sum=100)
#' 
#' points(s$x, s$y, pch=19)
#' lines(s$x, s$y)
#' }
#' @export

modeled.hist <- function(breaks, FUN, ..., sum=1) {
  # breaks is a vector with the breaks; it can be obtained directly from hist()
  # FUN is the function to integrate the density, ex. pnorm
  # ... are the parameters of FUN
  # sum is the total numbers in the histogram; 1 for emperical frequencies
  by <- breaks[2]-breaks[1]
  xp <- c(breaks, tail(breaks, 1)+by)
  par.fun <- list(...)
  par.fun <- modifyList(list(log.p=FALSE, q=xp, lower.tail=TRUE), 
                        par.fun)
  s <- do.call(FUN, par.fun)
  # s <- pfun(xp, par.fun[1], par.fun[2], log=FALSE)
  s <- utils::head(c(s[-1], 0)-s, length(breaks))
  return(list(x=breaks+by/2, y=s*sum))
}

#' tide.info gets the annual tide calendar for one particular location.
#' @title Annual tide calendar for one particular location
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Return a data.frame with annual tide calendar.
#' @param location Textual information about location name
#' @param year Year to get the calendar
#' @param longitude Longitude to search for
#' @param latitude Latitude to search for
#' @param force.tide.height If FALSE, can return a current speed rather than tide height
#' @family Periodic patterns of indices
#' @description Annual tide information.\cr
#' The columns are: Location, Longitude, Latitude, Phase, DateTime.local, DateTime.UTC, Tide.meter\cr
#' This function uses an API linking xtide software 
#' (https://flaterco.com/xtide/) with tide.info() function.\cr
#' You must have a working internet connection for this function.
#' @keywords Tide
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' Location <- "Les Hattes"
#' Year <- 2010
#' tide <- tide.info(Location, Year)
#' plot(tide[, "DateTime.local"], tide[, "Tide.meter"], 
#'      type="l", bty="n", las=1, 
#'      main=tide[1, "Location"], 
#'      xlab=as.character(Year), ylab="Tide level in meter")
#' 
#' Location <- "Hawaii"
#' Year <- 2010
#' tide <- tide.info(Location, Year)
#' 
#' Location <- "Hanamaulu Bay, Kauai Island, Hawaii"
#' Year <- 2010
#' tide <- tide.info(Location, Year)
#' plot(tide[, "DateTime.local"], tide[, "Tide.meter"], 
#'      type="l", bty="n", las=1, 
#'      main=tide[1, "Location"], 
#'      xlab=as.character(Year), ylab="Tide level in meter")
#'      
#' tide <- tide.info(year=2010, longitude=-32, latitude=-4)
#' library(maps)
#' map(database = "world", regions = "Brazil", asp=1, 
#'     xlim=c(-80, -30), ylim=c(-33, 5))
#' points(tide[1, "Longitude"], tide[1, "Latitude"], col="red", pch=19)
#' points(-32, -4, col="blue", pch=19)
#' axis(1)
#' axis(2, las=1)
#' 
#' # Show the locations with data    
#' library(maps)
#' map(xlim=c(-180, 180), ylim=c(-90, 90))
#' title("Locations with harmonics data")
#' axis(1, at=seq(from=-180, to=180, by=45))
#' axis(2, las=1, at=seq(from=-90, to=90, by=15))
#' points(getFromNamespace(x="tide_location", ns="HelpersMG")[, c("longitude")], 
#'        getFromNamespace(x="tide_location", ns="HelpersMG")[, c("latitude")], 
#'        pch=".", col="red", cex=2)
#' # Another example
#' tikei_lon  <- (-144.5465183)
#' tikei_lat <- -14.9505897
#' Year <- 2021
#' tikei_tide <- tide.info(year=Year, longitude=tikei_lon, latitude=tikei_lat)
#' plot(tikei_tide[, "DateTime.local"], tikei_tide[, "Tide.meter"], 
#'      type="l", bty="n", las=1, 
#'      main=tikei_tide[1, "Location"], 
#'      xlab=as.character(Year), ylab="Tide level in meter")
#' ## Another one
#' tikei_lon <- (-75.56861111)
#' tikei_lat <- 39.50083333
#' Year <- 2012
#' tikei_tide <- tide.info(year=Year, longitude=tikei_lon, latitude=tikei_lat)
#' 
#' library(mapdata)
#' map('worldHires', xlim=c(-77, -74), ylim=c(37, 40))
#' points(x=tikei_lon, y=tikei_lat, pch=19, col="red", cex=1)
#' points(x=tikei_tide$Longitude[1], y=tikei_tide$Latitude[2], 
#'        pch=19, col="blue", cex=1)
#' 
#' par(mar=c(4, 4, 2, 2))
#' plot(tikei_tide$DateTime.local, tikei_tide$Tide.meter, type="l")
#' }
#' @export

tide.info <- function(location=NULL, 
                      year=2021, 
                      longitude=NULL, latitude=NULL, 
                      force.tide.height=TRUE) {
  
  tide_location <- getFromNamespace(x="tide_location", ns="HelpersMG")
  
  if (is.null(location) & (is.null(longitude) | is.null(latitude))) {
    stop("Location or longitude/latitude must be provided.")
  }
  
  if (!is.null(location)) {
    
    pos <- which(grepl(location, tide_location$index))
    
    if (identical(pos, integer(0))) {
      stop("No location is found")
    }
    
    if (length(pos) != 1) {
      if (all(tide_location$name[pos[1]] == tide_location$name[pos[-1]])) {
        pos_time <- which.max(tide_location$date_imported[pos])
        pos <- pos[pos_time[1]]
      } else {
        print(tide_location$name[pos])
        stop("More that one location are found; be more precise.")
      }
    }
    dist <- NULL
    
  } else {
    indLl <- sqrt((tide_location$latitude-latitude)^2+(tide_location$longitude-longitude)^2)
    o_pos <- order(indLl)
    pos <- 1
    if (force.tide.height) {
      while (grepl("urrent", tide_location$name[o_pos[pos]])) {
        pos <- pos + 1
      }
    }
    pos <- o_pos[pos]
    toRad <- pi/180
    p1 <- matrix(c(latitude=latitude, longitude=longitude), ncol = 2)* toRad
    p2 <- as.matrix(tide_location[pos, c("latitude", "longitude")])* toRad
    r <- 6378137
      p = cbind(p1[, 1], p1[, 2], p2[, 1], p2[, 2], as.vector(r))
      dLat <- p[, 4] - p[, 2]
      dLon <- p[, 3] - p[, 1]
      a <- (sin(dLat/2))^2 + cos(p[, 2]) * cos(p[, 4]) * (sin(dLon/2))^2
      a <- pmin(a, 1)
      dist <- as.vector(2 * atan2(sqrt(a), sqrt(1 - a)) * p[, 5])
  }
  
  location <- tide_location$name[pos]
  message(paste0("The location ", location, " has been chosen."))
  if (!is.null(dist)) message(paste0("The distance between your setup and ", location, " is ", dist, " meters (Haversine distance)."))
  print(tide_location[pos, c("name", "longitude", "latitude", "timezone", "country")])
  
  # tz <- tide_location[pos, "timezone"]
  
  Begin <- paste0(as.character(year), "-01-01 00:00")
  End <- paste0(as.character(year), "-12-31 23:59")
  
  file <- tempfile(pattern = "file", tmpdir = tempdir(), fileext = ".txt")
  
  com <- paste0('curl --silent -o ', file, ' --data-urlencode "location=', location,'" --data-urlencode "begin=', Begin,'" --data-urlencode "end=', End,'" "http://157.136.255.42:20000/tide"')
  out <- system(command = com, intern = FALSE)
  out <- readChar(file, nchars=file.info(file)$size)
  
  out <- strsplit(out, split = "\",\"")[[1]]
  out <- gsub("\\[\"", "", out)
  out <- gsub("\"\\]", "", out)
  out <- strsplit(out, split = ",")
  
  out_df <- data.frame(Location=character(), 
                       Date=character(), Time=character(), 
                       Tide=character(), Phase=character())
  
  for (i in 1:length(out)) out_df <- rbind(out_df, as.data.frame(matrix(out[[i]], nrow=1)))
  out <- out_df
  rm(out_df)
  colnames(out) <- c("Location", "Date", "Time", "Tide", "Phase")
  
  out <- cbind(out, DateTime=paste(out$Date, out$Time))
  
  loc <- Sys.getlocale("LC_TIME")
  Sys.setlocale("LC_TIME", "C") 
  
  out <- cbind(out, DateTime.local=strptime(out$DateTime, format = "%Y-%m-%d %I:%M %p", tz=tide_location$timezone[pos]))
  Sys.setlocale("LC_TIME", loc) 
  
  # if (is.null(tz)) {
    out <- cbind(out, DateTime.UTC=convert.tz(out$DateTime.local, tz="UTC"))
  # } else {
  #   out <- cbind(out, DateTime.UTC=convert.tz(out$DateTime.local, tz=tz))
  # }
  
  out <- cbind(out, Longitude=tide_location$longitude[pos])
  out <- cbind(out, Latitude=tide_location$latitude[pos])
  
  unit <- NULL
  td <- NULL
  if (any(grepl("ft", out$Tide))) {
    td <- as.numeric(gsub(" ft", "", out$Tide))*0.3084
    unit <- "Tide.meter"
  }
  if (any(grepl("m", out$Tide[1]))) {
    td <- as.numeric(gsub(" m", "", out$Tide))
    unit <- "Tide.meter"
  }
  if (any(grepl("kt", out$Tide[1]))) {
    td <- as.numeric(gsub(" kt", "", out$Tide))
    unit <- "Tide.knot"
  }
  
  td <- data.frame(td=td)
  colnames(td) <- unit
  out <- cbind(out, td)[, c("Location", "Longitude", "Latitude", "Phase", "DateTime.local", "DateTime.UTC", unit)]
  out <- na.omit(out)
  
  return(out)
}

.BinomialConfidence <- 
function (x, n, conf.level = 0.95, sides = c("two.sided", "left", "right"),
          method = c("wilson", "wald", "waldcc", "agresti-coull", "jeffreys",
                     "modified wilson", "wilsoncc","modified jeffreys",
                     "clopper-pearson", "arcsine", "logit", "witting", "pratt", 
                     "midp", "lik", "blaker"),
          rand = 123, tol = 1e-05, std_est = TRUE, silent=TRUE)
#   Arguments
# x	number of successes.
# n	number of trials.
# conf.level  confidence level, defaults to 0.95.
# sides	 a character string specifying the side of the confidence interval, must be one of "two.sided" (default), "left" or "right". You can specify just the initial letter. "left" would be analogue to a hypothesis of "greater" in a t.test.
# method	character string specifing which method to use; this can be one out of: "wald", "wilson", "wilsoncc", "agresti-coull", "jeffreys", "modified wilson", "modified jeffreys", "clopper-pearson", "arcsine", "logit", "witting", "pratt", "midp", "lik" and "blaker". Defaults to "wilson". Abbreviation of method is accepted. See details.
# rand	seed for random number generator; see details.
# tol	tolerance for method "blaker".
# std_est	logical, specifying if the standard point estimator for the proportion value x/n should be returned (TRUE, default) or the method-specific internally used alternative point estimate (FALSE).

  {
    if (missing(method)) 
      method <- "wilsoncc"
    if (!silent) message(paste0("Method ", method))
    if (missing(sides)) 
      sides <- "two.sided"
    Recycle <- function (...) 
    {
      lst <- list(...)
      maxdim <- max(lengths(lst))
      res <- lapply(lst, rep, length.out = maxdim)
      attr(res, "maxdim") <- maxdim
      return(res)
    }
    
    iBinomCI <- function(x, n, conf.level = 0.95, sides = c("two.sided", 
                                                            "left", "right"), method = c("wilson", "wilsoncc", "wald", 
                                                                                         "waldcc", "agresti-coull", "jeffreys", "modified wilson", 
                                                                                         "modified jeffreys", "clopper-pearson", "arcsine", "logit", 
                                                                                         "witting", "pratt", "midp", "lik", "blaker"), rand = 123, 
                         tol = 1e-05, std_est = TRUE) {
      if (length(x) != 1) 
        stop("'x' has to be of length 1 (number of successes)")
      if (length(n) != 1) 
        stop("'n' has to be of length 1 (number of trials)")
      if (length(conf.level) != 1) 
        stop("'conf.level' has to be of length 1 (confidence level)")
      if (conf.level < 0.5 | conf.level > 1) 
        stop("'conf.level' has to be in [0.5, 1]")
      method <- match.arg(arg = method, choices = c("wilson", 
                                                    "wald", "waldcc", "wilsoncc", "agresti-coull", "jeffreys", 
                                                    "modified wilson", "modified jeffreys", "clopper-pearson", 
                                                    "arcsine", "logit", "witting", "pratt", "midp", "lik", 
                                                    "blaker"))
      sides <- match.arg(sides, choices = c("two.sided", "left", 
                                            "right"), several.ok = FALSE)
      if (sides != "two.sided") 
        conf.level <- 1 - 2 * (1 - conf.level)
      alpha <- 1 - conf.level
      kappa <- qnorm(1 - alpha/2)
      p.hat <- x/n
      q.hat <- 1 - p.hat
      est <- p.hat
      switch(method, wald = {
        term2 <- kappa * sqrt(p.hat * q.hat)/sqrt(n)
        CI.lower <- max(0, p.hat - term2)
        CI.upper <- min(1, p.hat + term2)
      }, waldcc = {
        term2 <- kappa * sqrt(p.hat * q.hat)/sqrt(n)
        term2 <- term2 + 1/(2 * n)
        CI.lower <- max(0, p.hat - term2)
        CI.upper <- min(1, p.hat + term2)
      }, wilson = {
        if (!std_est) {
          x.tilde <- x + kappa^2/2
          n.tilde <- n + kappa^2
          p.tilde <- x.tilde/n.tilde
          est <- p.tilde
        }
        term1 <- (x + kappa^2/2)/(n + kappa^2)
        term2 <- kappa * sqrt(n)/(n + kappa^2) * sqrt(p.hat * 
                                                        q.hat + kappa^2/(4 * n))
        CI.lower <- max(0, term1 - term2)
        CI.upper <- min(1, term1 + term2)
      }, wilsoncc = {
        if (!std_est) {
          x.tilde <- x + kappa^2/2
          n.tilde <- n + kappa^2
          p.tilde <- x.tilde/n.tilde
          est <- p.tilde
        }
        lci <- (2 * x + kappa^2 - 1 - kappa * sqrt(kappa^2 - 
                                                     2 - 1/n + 4 * p.hat * (n * q.hat + 1)))/(2 * 
                                                                                                (n + kappa^2))
        uci <- (2 * x + kappa^2 + 1 + kappa * sqrt(kappa^2 + 
                                                     2 - 1/n + 4 * p.hat * (n * q.hat - 1)))/(2 * 
                                                                                                (n + kappa^2))
        CI.lower <- max(0, ifelse(p.hat == 0, 0, lci))
        CI.upper <- min(1, ifelse(p.hat == 1, 1, uci))
      }, `agresti-coull` = {
        x.tilde <- x + kappa^2/2
        n.tilde <- n + kappa^2
        p.tilde <- x.tilde/n.tilde
        q.tilde <- 1 - p.tilde
        if (!std_est) est <- p.tilde
        term2 <- kappa * sqrt(p.tilde * q.tilde)/sqrt(n.tilde)
        CI.lower <- max(0, p.tilde - term2)
        CI.upper <- min(1, p.tilde + term2)
      }, jeffreys = {
        if (x == 0) CI.lower <- 0 else CI.lower <- qbeta(alpha/2, 
                                                         x + 0.5, n - x + 0.5)
        if (x == n) CI.upper <- 1 else CI.upper <- qbeta(1 - 
                                                           alpha/2, x + 0.5, n - x + 0.5)
      }, `modified wilson` = {
        if (!std_est) {
          x.tilde <- x + kappa^2/2
          n.tilde <- n + kappa^2
          p.tilde <- x.tilde/n.tilde
          est <- p.tilde
        }
        term1 <- (x + kappa^2/2)/(n + kappa^2)
        term2 <- kappa * sqrt(n)/(n + kappa^2) * sqrt(p.hat * 
                                                        q.hat + kappa^2/(4 * n))
        if ((n <= 50 & x %in% c(1, 2)) | (n >= 51 & x %in% 
                                          c(1:3))) CI.lower <- 0.5 * qchisq(alpha, 2 * 
                                                                              x)/n else CI.lower <- max(0, term1 - term2)
        if ((n <= 50 & x %in% c(n - 1, n - 2)) | (n >= 51 & 
                                                  x %in% c(n - (1:3)))) CI.upper <- 1 - 0.5 * qchisq(alpha, 
                                                                                                     2 * (n - x))/n else CI.upper <- min(1, term1 + 
                                                                                                                                           term2)
      }, `modified jeffreys` = {
        if (x == n) CI.lower <- (alpha/2)^(1/n) else {
          if (x <= 1) CI.lower <- 0 else CI.lower <- qbeta(alpha/2, 
                                                           x + 0.5, n - x + 0.5)
        }
        if (x == 0) CI.upper <- 1 - (alpha/2)^(1/n) else {
          if (x >= n - 1) CI.upper <- 1 else CI.upper <- qbeta(1 - 
                                                                 alpha/2, x + 0.5, n - x + 0.5)
        }
      }, `clopper-pearson` = {
        CI.lower <- qbeta(alpha/2, x, n - x + 1)
        CI.upper <- qbeta(1 - alpha/2, x + 1, n - x)
      }, arcsine = {
        p.tilde <- (x + 0.375)/(n + 0.75)
        if (!std_est) est <- p.tilde
        CI.lower <- sin(asin(sqrt(p.tilde)) - 0.5 * kappa/sqrt(n))^2
        CI.upper <- sin(asin(sqrt(p.tilde)) + 0.5 * kappa/sqrt(n))^2
      }, logit = {
        lambda.hat <- log(x/(n - x))
        V.hat <- n/(x * (n - x))
        lambda.lower <- lambda.hat - kappa * sqrt(V.hat)
        lambda.upper <- lambda.hat + kappa * sqrt(V.hat)
        CI.lower <- exp(lambda.lower)/(1 + exp(lambda.lower))
        CI.upper <- exp(lambda.upper)/(1 + exp(lambda.upper))
      }, witting = {
        set.seed(rand)
        x.tilde <- x + runif(1, min = 0, max = 1)
        pbinom.abscont <- function(q, size, prob) {
          v <- trunc(q)
          term1 <- pbinom(v - 1, size = size, prob = prob)
          term2 <- (q - v) * dbinom(v, size = size, prob = prob)
          return(term1 + term2)
        }
        qbinom.abscont <- function(p, size, x) {
          fun <- function(prob, size, x, p) {
            pbinom.abscont(x, size, prob) - p
          }
          uniroot(fun, interval = c(0, 1), size = size, 
                  x = x, p = p)$root
        }
        CI.lower <- qbinom.abscont(1 - alpha, size = n, x = x.tilde)
        CI.upper <- qbinom.abscont(alpha, size = n, x = x.tilde)
      }, pratt = {
        if (x == 0) {
          CI.lower <- 0
          CI.upper <- 1 - alpha^(1/n)
        } else if (x == 1) {
          CI.lower <- 1 - (1 - alpha/2)^(1/n)
          CI.upper <- 1 - (alpha/2)^(1/n)
        } else if (x == (n - 1)) {
          CI.lower <- (alpha/2)^(1/n)
          CI.upper <- (1 - alpha/2)^(1/n)
        } else if (x == n) {
          CI.lower <- alpha^(1/n)
          CI.upper <- 1
        } else {
          z <- qnorm(1 - alpha/2)
          A <- ((x + 1)/(n - x))^2
          B <- 81 * (x + 1) * (n - x) - 9 * n - 8
          C <- (0 - 3) * z * sqrt(9 * (x + 1) * (n - x) * 
                                    (9 * n + 5 - z^2) + n + 1)
          D <- 81 * (x + 1)^2 - 9 * (x + 1) * (2 + z^2) + 
            1
          E <- 1 + A * ((B + C)/D)^3
          CI.upper <- 1/E
          A <- (x/(n - x - 1))^2
          B <- 81 * x * (n - x - 1) - 9 * n - 8
          C <- 3 * z * sqrt(9 * x * (n - x - 1) * (9 * 
                                                     n + 5 - z^2) + n + 1)
          D <- 81 * x^2 - 9 * x * (2 + z^2) + 1
          E <- 1 + A * ((B + C)/D)^3
          CI.lower <- 1/E
        }
      }, midp = {
        f.low <- function(pi, x, n) {
          1/2 * dbinom(x, size = n, prob = pi) + pbinom(x, 
                                                        size = n, prob = pi, lower.tail = FALSE) - 
            (1 - conf.level)/2
        }
        f.up <- function(pi, x, n) {
          1/2 * dbinom(x, size = n, prob = pi) + pbinom(x - 
                                                          1, size = n, prob = pi) - (1 - conf.level)/2
        }
        CI.lower <- 0
        CI.upper <- 1
        if (x != 0) {
          CI.lower <- uniroot(f.low, interval = c(0, p.hat), 
                              x = x, n = n)$root
        }
        if (x != n) {
          CI.upper <- uniroot(f.up, interval = c(p.hat, 
                                                 1), x = x, n = n)$root
        }
      }, lik = {
        CI.lower <- 0
        CI.upper <- 1
        z <- qnorm(1 - alpha * 0.5)
        tol = .Machine$double.eps^0.5
        BinDev <- function(y, x, mu, wt, bound = 0, tol = .Machine$double.eps^0.5, 
                           ...) {
          ll.y <- ifelse(y %in% c(0, 1), 0, dbinom(x, wt, 
                                                   y, log = TRUE))
          ll.mu <- ifelse(mu %in% c(0, 1), 0, dbinom(x, 
                                                     wt, mu, log = TRUE))
          res <- ifelse(abs(y - mu) < tol, 0, sign(y - 
                                                     mu) * sqrt(-2 * (ll.y - ll.mu)))
          return(res - bound)
        }
        if (x != 0 && tol < p.hat) {
          CI.lower <- if (BinDev(tol, x, p.hat, n, -z, 
                                 tol) <= 0) {
            uniroot(f = BinDev, interval = c(tol, if (p.hat < 
                                                      tol || p.hat == 1) 1 - tol else p.hat), bound = -z, 
                    x = x, mu = p.hat, wt = n)$root
          }
        }
        if (x != n && p.hat < (1 - tol)) {
          CI.upper <- if (BinDev(y = 1 - tol, x = x, mu = ifelse(p.hat > 
                                                                 1 - tol, tol, p.hat), wt = n, bound = z, tol = tol) < 
                          0) {
            CI.lower <- if (BinDev(tol, x, if (p.hat < 
                                               tol || p.hat == 1) 1 - tol else p.hat, n, 
                                   -z, tol) <= 0) {
              uniroot(f = BinDev, interval = c(tol, p.hat), 
                      bound = -z, x = x, mu = p.hat, wt = n)$root
            }
          } else {
            uniroot(f = BinDev, interval = c(if (p.hat > 
                                                 1 - tol) tol else p.hat, 1 - tol), bound = z, 
                    x = x, mu = p.hat, wt = n)$root
          }
        }
      }, blaker = {
        acceptbin <- function(x, n, p) {
          p1 <- 1 - pbinom(x - 1, n, p)
          p2 <- pbinom(x, n, p)
          a1 <- p1 + pbinom(qbinom(p1, n, p) - 1, n, p)
          a2 <- p2 + 1 - pbinom(qbinom(1 - p2, n, p), n, 
                                p)
          return(min(a1, a2))
        }
        CI.lower <- 0
        CI.upper <- 1
        if (x != 0) {
          CI.lower <- qbeta((1 - conf.level)/2, x, n - 
                              x + 1)
          while (acceptbin(x, n, CI.lower + tol) < (1 - 
                                                    conf.level)) CI.lower = CI.lower + tol
        }
        if (x != n) {
          CI.upper <- qbeta(1 - (1 - conf.level)/2, x + 
                              1, n - x)
          while (acceptbin(x, n, CI.upper - tol) < (1 - 
                                                    conf.level)) CI.upper <- CI.upper - tol
        }
      })
      ci <- c(est = est, lwr.ci = max(0, CI.lower), upr.ci = min(1, 
                                                                 CI.upper))
      if (sides == "left") 
        ci[3] <- 1
      else if (sides == "right") 
        ci[2] <- 0
      return(ci)
    }
    lst <- list(x = x, n = n, conf.level = conf.level, sides = sides, 
                method = method, rand = rand, std_est = std_est)
    maxdim <- max(unlist(lapply(lst, length)))
    lgp <- lapply(lst, rep, length.out = maxdim)
    lgn <- Recycle(x = if (is.null(names(x))) 
      paste("x", seq_along(x), sep = ".")
      else names(x), n = if (is.null(names(n))) 
        paste("n", seq_along(n), sep = ".")
      else names(n), conf.level = conf.level, sides = sides, method = method, 
      std_est = std_est)
    xn <- apply(as.data.frame(lgn[sapply(lgn, function(x) length(unique(x)) != 
                                           1)]), 1, paste, collapse = ":")
    res <- t(sapply(1:maxdim, function(i) iBinomCI(x = lgp$x[i], 
                                                   n = lgp$n[i], conf.level = lgp$conf.level[i], sides = lgp$sides[i], 
                                                   method = lgp$method[i], rand = lgp$rand[i], std_est = lgp$std_est[i])))
    # colnames(res)[1] <- c("est")
    rownames(res) <- rep("", nrow(res))
    colnames(res) <- c("PointEst", "Lower", "Upper")
    return(res)
  }


.Arrows <- function (x0, y0, x1, y1, code = 2, arr.length = 0.4, arr.width = arr.length/2, 
    arr.adj = 1, arr.type = "curved", segment = TRUE, col = "black", 
    lcol = col, lty = 1, arr.col = lcol, lwd = 1, arr.lwd = lwd, 
    ...) 
{
    if (arr.type == "simple") {
        arrows(x0, y0, x1, y1, code = code, length = arr.length/2.54, 
            lty = lty, col = col, lwd = lwd, ...)
        return()
    }
    if (arr.type == "T") {
        arrows(x0, y0, x1, y1, code = code, length = arr.length/(2 * 
            2.54), lty = lty, angle = 90, col = col, lwd = lwd, 
            ...)
        return()
    }
    if (segment) 
        segments(x0, y0, x1, y1, col = lcol, lty = lty, lwd = lwd, 
            ...)
    user <- par("usr")
    pin <- par("pin")
    pin <- pin/max(pin)
    sy <- (user[4] - user[3])/pin[2]
    sx <- (user[2] - user[1])/pin[1]
    angle <- atan((y1 - y0)/(x1 - x0) * sx/sy)/pi * 180
    angle[is.nan(angle)] <- 0
    angle[x1 < x0] <- 180 + angle[x1 < x0]
    xx <- x1
    yy <- y1
    if (code == 3) 
      getFromNamespace(".Arrowhead", ns="HelpersMG")(x0 = xx, y0 = yy, angle = angle, lcol = lcol, 
            arr.col = arr.col, arr.adj = arr.adj, lty = lty, 
            arr.length = arr.length, arr.width = arr.width, arr.type = arr.type, 
            arr.lwd = arr.lwd)
    if (code != 2) {
        angle <- 180 + angle
        xx <- x0
        yy <- y0
    }
    getFromNamespace(".Arrowhead", ns="HelpersMG")(x0 = xx, y0 = yy, angle = angle, lcol = lcol, arr.col = arr.col, 
        arr.adj = arr.adj, lty = lty, arr.length = arr.length, 
        arr.width = arr.width, arr.type = arr.type, arr.lwd = arr.lwd)
}

.Arrowhead <- function (x0, y0, angle = 0, arr.length = 0.4, arr.width = arr.length/2, 
    arr.adj = 0.5, arr.type = "curved", lcol = "black", lty = 1, 
    arr.col = lcol, arr.lwd = 2, npoint = 5) 
{
  
    if (arr.type == "curved") {
        rad <- 0.7
        len <- 0.25 * pi
        mid <- c(0, rad)
        x <- seq(1.5 * pi + len, 1.5 * pi, length.out = npoint)
        rr <- cbind(mid[1] - rad * cos(x), mid[2] + rad * sin(x))
        mid <- c(0, -rad)
        x <- rev(x)
        rr <- rbind(rr, cbind(mid[1] - rad * cos(x), mid[2] - 
            rad * sin(x)))
        mid <- c(rr[nrow(rr), 1], 0)
        rd <- rr[1, 2]
        x <- seq(pi/2, 3 * pi/2, length.out = 3 * npoint)
        rr <- rbind(rr, cbind(mid[1] - rd * 0.25 * cos(x), mid[2] - 
            rd * sin(x)))
        rr[, 1] <- rr[, 1] * 2.6
        rr[, 2] <- rr[, 2] * 3.45
    }
    else if (arr.type == "triangle") {
        x <- c(-0.2, 0, -0.2)
        y <- c(-0.1, 0, 0.1)
        rr <- 6.22 * cbind(x, y)
    }
    else if (arr.type %in% c("circle", "ellipse")) {
        if (arr.type == "circle") 
            arr.width = arr.length
        rad <- 0.1
        mid <- c(-rad, 0)
        x <- seq(0, 2 * pi, length.out = 15 * npoint)
        rr <- 6.22 * cbind(mid[1] + rad * sin(x), mid[2] + rad * 
            cos(x))
    }
    if (arr.adj == 0.5) 
        rr[, 1] <- rr[, 1] - min(rr[, 1])/2
    if (arr.adj == 0) 
        rr[, 1] <- rr[, 1] - min(rr[, 1])
    user <- par("usr")
    pcm <- par("pin") * 2.54
    sy <- (user[4] - user[3])/pcm[2]
    sx <- (user[2] - user[1])/pcm[1]
    nr <- max(length(x0), length(y0), length(angle), length(arr.length), 
        length(arr.width), length(lcol), length(lty), length(arr.col))
    if (nr > 1) {
        x0 <- rep(x0, length.out = nr)
        y0 <- rep(y0, length.out = nr)
        angle <- rep(angle, length.out = nr)
        arr.length <- rep(arr.length, length.out = nr)
        arr.width <- rep(arr.width, length.out = nr)
        lcol <- rep(lcol, length.out = nr)
        lty <- rep(lty, length.out = nr)
        arr.col <- rep(arr.col, length.out = nr)
    }
    RR <- rr
    for (i in 1:nr) {
        dx <- rr[, 1] * arr.length[i]
        dy <- rr[, 2] * arr.width[i]
        angpi <- angle[i]/180 * pi
        cosa <- cos(angpi)
        sina <- sin(angpi)
        RR[, 1] <- cosa * dx - sina * dy
        RR[, 2] <- sina * dx + cosa * dy
        RR[, 1] <- x0[i] + RR[, 1] * sx
        RR[, 2] <- y0[i] + RR[, 2] * sy
        polygon(RR, col = arr.col[i], border = lcol[i], lty = lty[i], 
            lwd = arr.lwd)
    }
}

#' compare_BIC compares the BIC of several outputs obtained with the same data.
#' @title Compares the BIC of several outputs
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with DeltaBIC and Akaike weight for the models.
#' @param ... Successive results to be compared as lists.
#' @param factor.value The $value of the list object is multiplied by factor.value to calculate BIC.
#' @param silent If TRUE, nothing is displayed.
#' @param FUN Function used to show values
#' @description This function is used to compare the BIC of several outputs obtained with the same data but with different set of parameters.\cr
#' Each object must have associated \code{logLik()} method with df and nobs attributes.\cr
#' BIC for object x will be calculated as \code{2*factor.value*sum(logLik(x))+sum(attributes(logLik(x))$df)*log(attributes(logLik(x))$nobs))}.\cr
#' When several data (i..n) are included, the global BIC is calculated as:\cr
#' \code{2*factor.value*sum(logLik(x)) for i..n+sum(attributes(logLik(x))$df) for i..n*log(attributes(logLik(x))$nobs for i..n))}
#' @family AIC
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Here two different models are fitted
#' x <- 1:30
#' y <- rnorm(30, 10, 2)+log(x)
#' plot(x, y)
#' d <- data.frame(x=x, y=y)
#' m1 <- lm(y ~ x, data=d)
#' m2 <- lm(y ~ log(x), data=d)
#' compare_BIC(linear=m1, log=m2, factor.value=-1)
#' # Here test if two datasets can be modeled with a single model
#' x2 <- 1:30
#' y2 <- rnorm(30, 15, 2)+log(x2)
#' plot(x, y, ylim=c(5, 25))
#' plot_add(x2, y2, col="red")
#' d2 <- data.frame(x=x2, y=y2)
#' m1_2 <- lm(y ~ x, data=d2)
#' x_grouped <- c(x, x2)
#' y_grouped <- c(y, y2)
#' d_grouped <- data.frame(x=x_grouped, y=y_grouped)
#' m1_grouped <- lm(y ~ x, data=d_grouped)
#' compare_BIC(separate=list(m1, m1_2), grouped=m1_grouped, factor.value=-1)
#' }
#' @export


compare_BIC <- function(..., factor.value=-1, silent=FALSE, FUN=function(x) specify_decimal(x, decimals=2)) {

  result <- list(...)
  
  if (is.list(result) & length(result)==1) result <- unlist(result, recursive=FALSE)
  
  if (!is.null(result)) {
    if (!is.list(result) || (is.null(names(result))) || (any(names(result)==""))) {
      stop("The results must be included within a list with names; see example.")
    } else {
      out<-NULL
      
      l <- length(result)
      
      if (l<2) {
        stop("A least two results must be provided.")
      } else {
        bic <- NULL
        name <- names(result)
        for (i in 1:l) {
          
          encours <- result[i]
          # Je vais être un peu plus strict que pour l'AIC
          # Il faut que logLik(encours[[1]]) existe avec les attributs corrects
          # nall, nobs (nb d'observations) et df (nb de parametres)
          
          aaec <- try(logLik(encours[[1]]), silent=TRUE)
          t <- (inherits(aaec, "try-error"))
          if (t) encours <- encours[[1]]
          
          sumL <- 0
          sumdf <- 0
          sumnobs <- 0

          for (j in 1:length(encours)) {
            encours2 <- encours[[j]]
            aaec <- try(logLik(encours2), silent=TRUE)
            t <- (inherits(aaec, "try-error"))
            if (t) {
              stop(paste("Object", name[i], "has not the required format"))
            }
            sumL <- sumL + logLik(encours2)
            sumdf <- sumdf + attributes(logLik(encours2))$df
            sumnobs <- sumnobs + attributes(logLik(encours2))$nobs
          }
          bic <- c(bic, 2*factor.value*sumL+sumdf*log(sumnobs))	
        }
        
        bestbic<-min(bic)
        ser<-which.min(bic)
        deltabic<-bic-bestbic
        aw<-exp(-0.5*deltabic)
        saw=sum(aw)
        aw<-aw/saw
        
        out<-data.frame(cbind(BIC=FUN(bic), DeltaBIC=FUN(deltabic), Akaike_weight=FUN(aw)), row.names=name)
        if (!silent) print(paste("The lowest BIC (",sprintf("%.3f", bestbic) ,") is for series ", name[ser], " with Akaike weight=", sprintf("%.3f", aw[ser]), sep=""))
        
        return(out)
      }
    }
  }
}

#' compare runs a shiny application for basic functions of comparison
#' @title Run a shiny application for basic functions of comparison
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @description Run a shiny application for basic functions of comparison.
#' @references Girondot, M., Guillon, J.-M., 2018. The w-value: An alternative to t- and X2 tests. Journal of Biostatistics & Biometrics 1, 1-3.
#' @family w-value functions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' compare()
#' }
#' @export


compare <- function() {
  
  if (!requireNamespace("shiny", quietly = TRUE)) {
    stop("shiny package is absent; Please install it first")
  }
  
getFromNamespace("runApp", ns="shiny")(appDir = system.file(file.path("shiny", "compare"), package="HelpersMG"), 
                                       launch.browser =TRUE)

}

#' openwd will open a finder window with current working directory
#' @title Open a finder window with current working directory in MacOS X and windows
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A vector with the list of files.
#' @param directory The directory you want to open
#' @description This function opens a finder window with directory files 
#' in MacOS X. It has not been fully tested in Windows. In linux, it just returns the 
#' list of files in directory.\cr
#' By defaut, it uses the current working directory.
#' @examples 
#' \dontrun{
#' openwd()
#' }
#' @export


openwd <- function(directory=getwd()) {
  if (.Platform$OS.type == "unix") {
    if (Sys.info()["sysname"] == "Darwin") {
      system(command=sprintf('open %s', shQuote(directory)))
    }
  } else {
    if (.Platform$OS.type == "windows"){
      shell.exec(directory)
    }
  }
  return(list.files(directory))
}

#' setPriors is a general function to set priors for MHalgoGen()
#' @title Set priors for MHalgoGen()
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Return a data.frame with priors
#' @param par Named vector with init value of parameters
#' @param se Named vector with standard error of parameters
#' @param density Named vector with density or single value
#' @param rules List of rules to define priors
#' @param silent If TRUE, do not show warning.
#' @description Set priors for MHalgoGen()\cr
#' @family mcmcComposite functions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' rules <- rbind(data.frame(Name="^a", Min=0, Max="x*2"), 
#'               data.frame(Name="^b", Min=0, Max=100))
#' par <- c(a0=10, a1=2, b2=20)
#' (p <- setPriors(par=par, se=NULL, density="dgamma", rules=rules))
#' (p <- setPriors(par=par, se=NULL, density="dnorm", rules=rules))
#' (p <- setPriors(par=par, se=NULL, density="dunif", rules=rules))
#' par <- c(a0=10, a1=2, b2=20, b1=-1)
#' (p <- setPriors(par=par, se=NULL, density="dgamma", rules=rules))
#' }
#' @export


setPriors <- function(par=stop("A vector with init values is necessary."), 
                      se=NULL, 
                      density="dunif", 
                      rules=NULL, silent=FALSE) {
  
  # par=c(a0=10, b2=20); se=NULL; density="dunif"; rules=rbind(data.frame(Name="^a", Min=0, Max="x*2"), data.frame(Name="^b", Min=0, Max=100))
  
  if (is.null(se)) se <- abs(par/2)
  if (length(density) == 1) {
    density <- rep(density, length(par))
    names(density) <- names(par)
  }
  if ((length(density) != length(par)) | (length(se) != length(par))) 
    stop("Wrong number of elements in par, se or density.")
  
  df_priors <- data.frame(Density=character(), 
                          Prior1=numeric(), Prior2=numeric(), 
                          SDProp=numeric(), 
                          Min=numeric(), Max=numeric(), 
                          Init=numeric())
  names_rules <- rules[, "Name"]
  
  for (indice.par in seq_along(par)) {
    name_par <- names(par)[indice.par]
    # J'ai besoin de min, max pour chaque paramètre
    rank_rule <- sapply(names_rules, FUN=function(names_rules_ind) grepl(names_rules_ind, names(par)[indice.par]))
    if (any(rank_rule)) {
      vpar <- unname(par[indice.par])
      vse <- unname(se[indice.par])
      Min <- rules[which(rank_rule), "Min", drop=TRUE][1]
      Max <- rules[which(rank_rule), "Max", drop=TRUE][1]
      
      x <- vpar
      # if (is.na(suppressWarnings(as.numeric(Min)))) {
      Min <- eval(parse(text=Min), envir= environment())
      # } else {
      #   Min <- as.numeric(Min)
      # }
      # if (is.na(suppressWarnings(as.numeric(Max)))) {
      Max <- eval(parse(text=Max), envir= environment())
      # } else {
      #   Max <- as.numeric(Max)
      # }
      
      Min <- min(Min, vpar)
      Max <- max(Max, vpar)
      
      df_priors_ec <- NULL
      if (density[name_par]=="dnorm") {
        df_priors_ec <- data.frame(Density="dnorm", 
                                   Prior1=vpar, 
                                   Prior2=vse, 
                                   SDProp=abs(vpar), 
                                   Min=Min, Max=Max, 
                                   Init=vpar, 
                                   row.names = name_par)
      }
      if ((density[name_par]=="dgamma") & (Min < 0) & (!silent)) {
       warning(paste0("Parameter ", name_par, ": dgamma cannot be used with negative values; it is changed to dunif.")) 
      }
      if ((density[name_par]=="dgamma") & (Min >= 0)) {
        # Si Min < 0, je passe en dunif
        r <- vpar/(vse^2)
        a <- vpar*r
        df_priors_ec <- data.frame(Density="dgamma", 
                                   Prior1=a, Prior2=r, 
                                   SDProp=abs(vpar), 
                                   Min=Min, Max=Max, 
                                   Init=vpar, row.names = name_par)
      }
      if ((density[name_par]=="dunif") | is.null(df_priors_ec)) {
        df_priors_ec <- data.frame(Density="dunif", 
                                   Prior1=Min, Prior2=Max, 
                                   SDProp=abs(vpar), 
                                   Min=Min, Max=Max,  
                                   Init=vpar, row.names = name_par)
      }
      df_priors <- rbind(df_priors, df_priors_ec)
    }
  }
  
  df_priors <- addS3Class(df_priors, c("PriorsmcmcComposite", "data.frame"))
  return(df_priors)
}

#' invlogit returns the inverse logit
#' @title Return the inverse logit
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A value
#' @param n The value to inverse to get the probability
#' @description Return the inverse logit.
#' @family logit
#' @examples
#' n <- logit(0.5)
#' invlogit(n)
#' @export

invlogit <- function(n) {1/(1+exp(-n))}

#' rmnorm Generate random numbers from the multivariate normal distribution
#' @title Generate random numbers from the multivariate normal distribution
#' @author Based on lmf package
#' @return For n > 1 rmnorm returns a matrix of n rows of random vectors, while for n = 1 rmnorm returns a named random vector.
#' @param n the number of random vectors to be generated.
#' @param mean a vector with means of length d.
#' @param varcov a variance-covariance matrix with dimentions d * d.
#' @description rmnorm generate random numbers from a multivariate normal distribution.
#' @family Distributions
#' @examples 
#' \dontrun{
#' #Variance-covariance matrix
#' varcov <- matrix(c(2.047737e-03, 3.540039e-05, 0.0075178920, 3.540039e-05,
#' 6.122832e-07, 0.0001299661, 7.517892e-03, 1.299661e-04, 0.0276005740), ncol = 3)
#' #Set names
#' nam <- c("a", "b", "c")
#' dimnames(varcov) <- list(nam, nam)
#' #Check positive definiteness (all positive eigenvalues = positive definite)
#' eigen(varcov) $values
#' #Mean
#' mean <- c(1, 0.3, 0.5)
#' #Generate n = 1 random vector
#' rmnorm(n = 1, mean = mean, varcov = varcov)
#' #Generate n = 10 random vectors
#' rmnorm(n = 10, mean = mean, varcov = varcov)
#' #Generate n = 1 random vectors when varcov is non-positive definite
#' #Non-positive definite varcov matrix
#' varcov2 <- matrix(c(2.04e-03, 3.54e-05, 7.52e-03, 3.54e-05, 6.15e-07,
#'   1.30e-04, 7.52e-03, 1.30e-04, 2.76e-02), ncol = 3)
#'   dimnames(varcov2) <- dimnames(varcov)
#' eigen(varcov2)
#' #Random vector
#' rmnorm(n = 1, mean = mean, varcov = varcov2)
#' }
#' @export

rmnorm <- function (n = 1, mean = rep(0, d), varcov) {
  d <- if (is.matrix(varcov)) 
    ncol(varcov)
  else 1
  z <- try(matrix(rnorm(n * d), n, d) %*% chol(varcov), silent = TRUE)
  if (inherits(z, "try-error")) 
    z <- try(matrix(rnorm(n * d), n, d) %*% chol(nearPD(varcov)), 
             silent = TRUE)
  if (inherits(z, "try-error")) 
    return(rep(NA, d))
  y <- t(mean + t(z))
  if (n == 1 & !is.null(colnames(varcov))) {
    y <- c(y)
    names(y) <- colnames(varcov)
  }
  return(y)
  
}

.onAttach <- function(libname, pkgname) {
  actual <- utils::packageDescription(pkgname)[["Version"]]
  
  packageStartupMessage(paste("Welcome in package", pkgname, "version", actual))
  
  # conn <- url("http://marc.girondot.free.fr/CRAN/HelpersMG/version.txt")
  # 
  # version_get <- try(suppressWarnings(
  #   readLines(con=conn)), silent = TRUE
  # )
  # close(con=conn)
  # if (!(is.null(version_get)) & (!inherits(version_get, "try-error"))) {
  #   if (package_version(actual, strict = TRUE) < package_version(version_get, strict = TRUE)) {
  #     packageStartupMessage('An update is available; use:\ninstall.packages("http://marc.girondot.free.fr/CRAN/HelpersMG.tar.gz", repos=NULL, type="source")')
  #   } else {
  #     packageStartupMessage("No update is available")
  #   }
  # } else {
  #   packageStartupMessage("No internet connection is available to check for presence of update")
  # }
  
}

#' BP runs a shiny application to fit bone section
#' @title Run a shiny application to fit bone section
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @description Run a shiny application to fit bone section
#' @examples
#' \dontrun{
#' # Not run:
#' library(HelpersMG)
#' iCutter()
#' }
#' @export


iCutter <- function() {
  
  if (!requireNamespace("shiny", quietly = TRUE)) {
    stop("shiny package is absent; Please install it first")
  }
  
  if(interactive()){
    getFromNamespace("runApp", ns="shiny")(appDir = system.file(file.path("shiny", "cutter"), package="HelpersMG"), 
                                           launch.browser =TRUE)
  }
  
}

#' MHalgoGen is a function to use mcmc with Metropolis-Hastings algorithm
#' @title Monte-Carlo Markov-chain with Metropolis-Hastings algorithm
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A mcmcComposite object with all characteristics of the model and mcmc run
#' @param n.iter Number of iterations for each chain
#' @param parameters A data.frame with priors; see description and examples
#' @param likelihood The function that returns -ln likelihood using data and parameters
#' @param n.chains Number of chains
#' @param n.adapt Number of iteration to stabilize likelihood
#' @param thin Interval for thinning likelihoods
#' @param trace Or FALSE or period to show progress
#' @param traceML TRUE or FALSE to show ML
#' @param progress.bar.ini The command to initialize progress bar
#' @param progress.bar The command to run the progress bar
#' @param intermediate Or NULL of period to save intermediate result
#' @param filename Name of file in which intermediate results are saved
#' @param adaptive Should an adaptive process for SDProp be used
#' @param adaptive.lag  Lag to analyze the SDProp value in an adaptive context
#' @param adaptive.fun Function used to change the SDProp
#' @param previous The content of the file in which intermediate results are saved
#' @param session The shiny session
#' @param parameters_name The name of the parameters in the likelihood function, default is "x".
#' @param warn If TRUE, show information for debug.
#' @param WAIC.out If TRUE matrix or array are stored to be used with loo or waic.
#' @param n.datapoints Number of datapoints when WAIC.out is TRUE.
#' @param ... Parameters to be transmitted to likelihood function
#' @description The parameters must be stored in a data.frame with named rows for each parameter with the following columns:\cr
#' \itemize{
#'   \item Density. The density function name, example \code{dnorm}, \code{dlnorm}, \code{dunif}, \code{dbeta}
#'   \item Prior1. The first parameter to send to the \code{Density} function
#'   \item Prior2. The second parameter to send to the \code{Density} function
#'   \item Prior3. The third parameter to send to the \code{Density} function
#'   \item SDProp. The standard error from new proposition value of this parameter
#'   \item Min. The minimum value for this parameter
#'   \item Max. The maximum value for this parameter
#'   \item Init. The initial value for this parameter
#' }
#' This script has been deeply modified from a MCMC script provided by Olivier Martin (INRA, Paris-Grignon).\cr
#' The likelihood function must use a parameter named parameters_name for the named parameters.\cr
#' For adaptive mcmc, see:\cr
#' Rosenthal, J. S. 2011. Optimal Proposal Distributions and Adaptive MCMC. Pages 93-112 in S. Brooks, A. Gelman, 
#' G. Jones, and X.-L. Meng, editors. MCMC Handbook. Chapman and Hall/CRC.\cr
#' @family mcmcComposite functions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' require(coda)
#' val <- rnorm(30, 10, 2)
#' dnormx <- function(data, x) {
#'  data <- unlist(data)
#'  return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
#' }
#' parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
#' Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(0.35, 0.2), 
#' Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
#' row.names=c('mean', 'sd'))
#' # Or simpler
#' parameters_mcmc <- setPriors1(Name="mean", Density="dnorm", Parameters=c(mean=10, sd=2), 
#'                             SDProp=0.35, Min=-3, Max=100, Init=10)
#' parameters_mcmc <- parameters_mcmc + setPriors1(Name="sd", Density="dlnorm", 
#'                             Parameters=c(meanlog=0.5, sdlog=0.5), 
#'                             SDProp=0.2, Min=0, Max=10, Init=2)
#' # Use of trace and traceML parameters
#' # trace=1 : Only one likelihood is printed
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
#' # trace=10 : 10 likelihoods are printed
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=10)
#' # trace=TRUE : all likelihoods are printed
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=TRUE)
#' # traceML=TRUE : values when likelihood is better are shown
#' mcmc_run <- MHalgoGen(n.iter=100, parameters=parameters_mcmc, data=val, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=TRUE, traceML=TRUE)
#' mcmc_run <- MHalgoGen(n.iter=100, parameters=parameters_mcmc, data=val, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=FALSE, traceML=TRUE)
#' # trace=FALSE : No likelihood is printed
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=FALSE)
#' 
#' plot(mcmc_run, xlim=c(0, 20))
#' plot(mcmc_run, xlim=c(0, 10), parameters="sd")
#' library(graphics)
#' library(fields)
#' # show a scatter plot of the result
#' x <- mcmc_run$resultMCMC[[1]][, 1]
#' y <- mcmc_run$resultMCMC[[1]][, 2]
#' marpre <- par(mar=c(4, 4, 2, 6)+0.4)
#' smoothScatter(x, y)
#' # show a scale
#' n <- matrix(0, ncol=128, nrow=128)
#' xrange <- range(x)
#' yrange <- range(y)
#' for (i in 1:length(x)) {
#'   posx <- 1+floor(127*(x[i]-xrange[1])/(xrange[2]-xrange[1]))
#'   posy <- 1+floor(127*(y[i]-yrange[1])/(yrange[2]-yrange[1]))
#'   n[posx, posy] <- n[posx, posy]+1
#' }
#' image.plot(legend.only=TRUE, zlim= c(0, max(n)), nlevel=128, 
#'  col=colorRampPalette(c("white", blues9))(128))
#' # Compare with a heatmap
#' x <- seq(from=8, to=12, by=0.2)
#' y <- seq(from=1, to=4, by=0.2)
#' df <- expand.grid(mean=x, sd=y)
#' df <- cbind(df, L=rep(0, length(nrow(df))))
#' for (i in 1:nrow(df)) df[i, "L"] <- -sum(dnorm(val, df[i, 1], df[i, 2], log = TRUE))
#' hm <- matrix(df[, "L"], nrow=length(x))
#' par(mar = marpre)
#' image.plot(x=x, y=y, z=hm, las=1)
#' # Diagnostic function from coda library
#' mcmcforcoda <- as.mcmc(mcmc_run)
#' #' heidel.diag(mcmcforcoda)
#' raftery.diag(mcmcforcoda)
#' autocorr.diag(mcmcforcoda)
#' acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
#' acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
#' batchSE(mcmcforcoda, batchSize=100)
#' # The batch standard error procedure is usually thought to 
#' # be not as accurate as the time series methods used in summary
#' summary(mcmcforcoda)$statistics[,"Time-series SE"]
#' summary(mcmc_run)
#' as.parameters(mcmc_run)
#' lastp <- as.parameters(mcmc_run, index="last")
#' parameters_mcmc[,"Init"] <- lastp
#' # The n.adapt set to 1 is used to not record the first set of parameters
#' # then it is not duplicated (as it is also the last one for 
#' # the object mcmc_run)
#' mcmc_run2 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, x=x, data=val, 
#' likelihood=dnormx, n.chains=1, n.adapt=1, thin=1, trace=1)
#' mcmc_run3 <- merge(mcmc_run, mcmc_run2)
#' ####### no adaptation, n.adapt must be 0
#' parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
#' mcmc_run3 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, x=x, data=val, 
#' likelihood=dnormx, n.chains=1, n.adapt=0, thin=1, trace=1)
#' # Here is how to use adaptive mcmc
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, adaptive = FALSE, 
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
#' 1-rejectionRate(as.mcmc(mcmc_run))
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, adaptive = TRUE,  
#' likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
#' 1-rejectionRate(as.mcmc(mcmc_run))
#' # To see the dynamics :
#' var <- "mean"
#' par(mar=c(4, 4, 1, 1)+0.4)
#' plot(1:nrow(mcmc_run$resultMCMC[[1]]), mcmc_run$resultMCMC[[1]][, var], type="l", 
#'        xlab="Iterations", ylab=var, bty="n", las=1)
#' plot(mcmc_run, what="Posterior", parameters="mean")
#' plot(mcmc_run, what="MarkovChain", parameters="mean", ylim=c(0, 20), legend="topleft")
#' plot(mcmc_run, what="MarkovChain", parameters="mean", legend="topleft")
#' # Exemple with a progress bar
#' 
#' val <- rnorm(30, 10, 2)
#' dnormx <- function(data, x) {
#'  data <- unlist(data)
#'  return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
#' }
#' parameters_mcmc <- setPriors1(Name="mean", Density="dnorm", Parameters=c(mean=10, sd=2), 
#'                             SDProp=0.35, Min=-3, Max=100, Init=10)
#' parameters_mcmc <- parameters_mcmc + setPriors1(Name="sd", Density="dlnorm", 
#'                             Parameters=c(meanlog=0.5, sdlog=0.5), 
#'                             SDProp=0.2, Min=0, Max=10, Init=2)
#' # Set up the progress bar
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
#'                       likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=FALSE, 
#'                        progress.bar.ini=function(n.iter) {
#'                                  assign("pb", txtProgressBar(min=0, max=n.iter, style=3), 
#'                                         env = parent.frame())}, 
#'         progress.bar=function(iter) {setTxtProgressBar(get("pb", envir = parent.frame()), iter)})
#' }
#' @export

# Algo Metropolis-Hastings
# ------------------------

MHalgoGen<-function(likelihood=stop("A likelihood function must be supplied")  , 
                    parameters=stop("Priors must be supplied")                , 
                    ...                                                        , 
                    parameters_name="x"                                        ,
                    n.iter=10000                                               , 
                    n.chains = 1                                               , 
                    n.adapt = 100                                              , 
                    thin=30                                                    , 
                    trace=FALSE                                                , 
                    traceML=FALSE                                              , 
                    progress.bar.ini=NULL                                      , 
                    progress.bar=NULL                                          , 
                    adaptive = FALSE                                           , 
                    adaptive.lag = 500                                         , 
                    adaptive.fun = function(x) {ifelse(x>0.234, 1.3, 0.7)}     ,
                    intermediate=NULL                                          , 
                    filename="intermediate.Rdata"                              ,
                    previous=NULL                                              , 
                    session=NULL                                               , 
                    warn=FALSE                                                 , 
                    WAIC.out=FALSE                                             , 
                    n.datapoints=NULL                                          ) {
  
  if (n.iter + n.adapt < 2) stop("n.iter + n.adapt must be higher than 1.")
  
  if (!requireNamespace("coda", quietly = TRUE)) {
    stop("coda package is necessary for this function")
  }
  
  previousML <- -Inf
  
  # likelihood=NULL; parameters_name="x"; parameters=NULL; n.iter=10000; n.chains = 1
  # n.adapt = 100; thin=30; trace=FALSE; traceML=FALSE; intermediate=NULL; filename="intermediate.Rdata"
  # previous=NULL; adaptive = FALSE; adaptive.lag = 500; adaptive.fun = function(x) {ifelse(x>0.234, 1.3, 0.7)}
  # progress.bar.ini=NULL; progress.bar=NULL
  
  # datax <- list(temperatures=result$data, derivate=result$derivate, test=result$test, M0=result$M0, fixed.parameters=result$fixed.parameters, weight=result$weight, out="Likelihood",  progress=FALSE, warnings=FALSE, likelihood=getFromNamespace("info.nests", ns = "embryogrowth"))
  
  if (is.null(previous)) {
    nbvar <- dim(parameters)[1]
    deb_kk <- 1
    deb_i <- 2
    res <- as.list(NULL)
    restot <- as.list(NULL)
    deb_varp2 <- matrix(rep(NA, (nbvar+1)*(n.adapt+n.iter+2)), ncol=nbvar+1)
    deb_varp<-matrix(rep(NA, (nbvar+1)*(n.adapt+n.iter+2)), ncol=nbvar+1)
    colnames(deb_varp)<-c(rownames(parameters), "Ln L")
    colnames(deb_varp2)<-c(rownames(parameters), "Ln L")
    t <- as.character(trace)
    cpt_trace <- 0
    res<-as.list(NULL)
    restot <- as.list(NULL)
    resL<-as.list(NULL)
    resLtot<-as.list(NULL)
    # datax <- list(temperatures=result$data, derivate=result$derivate, test=result$test, M0=result$M0, fixed.parameters=result$fixed.parameters, weight=result$weight, out="Likelihood",  progress=FALSE, warnings=FALSE, likelihood=getFromNamespace("info.nests", ns = "embryogrowth"))
    # datax <- list(data)
    # datax <- list(pt)
    datax <- list(...)
    if (WAIC.out) {
      if (is.null(n.datapoints)) {
        stop("n.datapoints must be setup when WAIC.out is TRUE.")
      } else {
        WAIC.array <- array(NA, dim = c(n.iter+n.adapt, n.chains, n.datapoints))
      }
    } else {
      WAIC.array <- NULL
    }
  } else {
    n.iter <- previous$n.iter
    parameters <- previous$parameters
    # Initialisation
    nbvar<-dim(parameters)[1]
    datax <- previous$datax
    likelihood <- previous$likelihood
    parameters_name <- previous$parameters_name
    n.chains <- previous$n.chains
    n.adapt <- previous$n.adapt
    thin <- previous$thin
    trace <- previous$trace
    traceML <- previous$traceML
    intermediate <- previous$intermediate
    filename <- previous$filename
    deb_kk <- previous$chain
    deb_i <- previous$iter
    res <- previous$res
    restot <- previous$restot
    deb_varp2 <- previous$varp2
    deb_varp <- previous$varp
    cpt <- previous$cpt
    cpt_trace <- previous$cpt_trace
    resL <- previous$resL
    resLtot <- previous$resLtot
    sdg <- previous$sdg
    MaxL <- previous$MaxL
    t <- as.character(previous$trace)
    adaptive <- previous$adaptive
    adaptive.lag <- previous$adaptive.lag
    adaptive.fun <- previous$adaptive.fun
    WAIC.array <- previous$WAIC.array
  }
  
  if (!is.null(progress.bar.ini)) {
    progress.bar.ini(n.adapt+n.iter)
  }
  
  pt <- NULL
  if (t=="TRUE") {pt <- 1;tf <- TRUE}
  if (t=="FALSE") {pt <- 0;tf <- FALSE}
  if (is.null(pt)) {
    tf <- TRUE
    pt <- floor((n.adapt+n.iter)/trace)
  }
  
  
  for (kk in deb_kk:n.chains) {
    
    varp <- deb_varp
    varp2 <- deb_varp2
    
    if (is.null(previous)) {
      
      deb_varp2 <- matrix(rep(NA, (nbvar+1)*(n.adapt+n.iter+2)), ncol=nbvar+1)
      deb_varp <- matrix(rep(NA, (nbvar+1)*(n.adapt+n.iter+2)), ncol=nbvar+1)
      colnames(deb_varp)<-c(rownames(parameters), "Ln L")
      colnames(deb_varp2)<-c(rownames(parameters), "Ln L")
      
      
      varp[1, 1:nbvar] <- as.numeric(parameters[1:nbvar, 'Init'])
      
      param <- list(varp[1, 1:nbvar])
      names(param) <- parameters_name
      
      # ll <- c(datax, param)
      # names(ll) <- c("data", parameters_name)
      LnLx <- -do.call(likelihood, modifyList(datax, param))
      varp[1, "Ln L"] <- LnLx
      if (WAIC.out) {
        # print(attributes(LnLx)$WAIC)
        # print(dim(WAIC.array))
        # print(length(as.vector(attributes(LnLx)$WAIC)))
        WAIC.array[1, kk, 1:length(as.vector(attributes(LnLx)$WAIC))] <- as.vector(attributes(LnLx)$WAIC)
      }
      # likelihood(par=param$par, fixed.parameters = datax$fixed.parameters, res_1=datax$res_1, res_2=datax$res_2)
      cpt <- 1
      varp2[cpt, 1:nbvar] <- varp[1, 1:nbvar]
      varp2[cpt, "Ln L"] <- varp[1, "Ln L"]
      cpt <- 2
      
      if (trace) {
        cat(paste("Chain ", kk, ": [", 1, "] ",as.numeric(varp[1, nbvar+1]), "\n", sep=""))
        # } else {
        #   cat(paste("Chain ", kk, "\n", sep=""))
      }
      
      if (traceML) {
        if (as.numeric(varp[1, "Ln L"]) > previousML) {
          previousML <- varp[1, "Ln L"]
          cat(d(varp[1, 1:nbvar]))
          cat("\n")
        }
      }
      MaxL <- varp[1, ]
      
      # sdg=NULL
      
      
      # 18/1/2013
      # for(i in 1:nbvar) sdg<-c(sdg, as.numeric(parameters[i, 'SDProp']))
      # sdg <- c(sdg, as.numeric(parameters[1:nbvar, 'SDProp']))
      sdg <- as.numeric(parameters[1:nbvar, 'SDProp'])
      
      
      # previous <- NULL
    }
    
    
    if (is.data.frame(parameters)) {
      if (any(colnames(parameters) == "Prior3")) {
        Prior <- as.matrix(parameters[,c("Prior1", "Prior2", "Prior3")])
      } else {
        Prior <- as.matrix(parameters[,c("Prior1", "Prior2")])
        Prior <- cbind(Prior, Prior3=NA)
      }
      Limites <- as.matrix(parameters[,c("Min", "Max")])
    } else {
      if (any(colnames(parameters) == "Prior3")) {
        Prior <- matrix(as.numeric(parameters[,c("Prior1", "Prior2", "Prior3")]), ncol=3)
      } else {
        Prior <- matrix(as.numeric(parameters[,c("Prior1", "Prior2")]), ncol=2)
        Prior <- cbind(Prior, Prior3=NA)
      }
      Limites <- matrix(as.numeric(parameters[,c("Min", "Max")]), ncol=2)
    }
    
    dfun <- as.character(parameters[,"Density"])
    rfun <- paste0("r", substr(dfun, 2, nchar(dfun)))
    
    # Itérations
    for (i in deb_i:(n.adapt+n.iter)) {
      
      if (!is.null(progress.bar)) {
        progress.bar(i) # session ?
      }
      
      
      # est-ce que je sauve où j'en suis
      if (!is.null(intermediate))
        if (i %% intermediate==0) {
          itr <- list(chain=kk, iter=i, varp2=varp2, res=res, restot=restot, 
                      n.iter=n.iter,
                      parameters=parameters,
                      datax=datax,
                      likelihood=likelihood,
                      parameters_name=parameters_name,
                      n.chains=n.chains,
                      n.adapt=n.adapt,
                      thin=thin,
                      trace=trace,
                      traceML=traceML,
                      intermediate=intermediate,
                      filename=filename, 
                      varp=varp, 
                      cpt=cpt,
                      cpt_trace=cpt_trace, 
                      resL=resL, 
                      resLtot=resLtot, 
                      sdg=sdg, MaxL=MaxL, 
                      adaptive=adaptive,
                      adaptive.lag=adaptive.lag,
                      adaptive.fun=adaptive.fun, 
                      WAIC.array=WAIC.array)
          save(itr, file=filename)
        }
      
      
      ###### Pas clair si previous
      deb_i <- 2
      Lprevious <- LpreviousT <- varp[i-1, "Ln L"]
      if (WAIC.out) {
        previousWAIC <- WAIC.array[i-1, kk, ]
      }
      
      newvarp <- varp[i-1, 1:nbvar]
      
      for (j in 1:nbvar) {	
        # Nouveau paramètre
        propvarp <- newvarp
        if (sdg[j] >=0) {
          propvarp[j] <- propvarp[j] + rnorm(1, mean=0, sd=sdg[j])
        } else {
          if (is.na(Prior[j,3])) {
            propvarp[j] <- get(rfun[j])(1, Prior[j,1], Prior[j,2])
          } else {
            propvarp[j] <- get(rfun[j])(1, Prior[j,1], Prior[j,2], Prior[j,3])
          }
        }
        
        if (propvarp[j] <= Limites[j,2] && propvarp[j] >= Limites[j,1]) {
          
          param <- list(propvarp)
          names(param) <- parameters_name
          Lprevious2 <- - do.call(likelihood, modifyList(datax, param))
          if (sdg[j] >=0) {
            if (is.na(Prior[j,3])) {
              logratio <- get(dfun[j])(propvarp[j],Prior[j,1],Prior[j,2],log=TRUE) + Lprevious2 -
                (get(dfun[j])(newvarp[j],Prior[j,1],Prior[j,2],log=TRUE)+LpreviousT)
            } else {
              logratio <- get(dfun[j])(propvarp[j],Prior[j,1],Prior[j,2],Prior[j,3], log=TRUE) + Lprevious2 -
                (get(dfun[j])(newvarp[j],Prior[j,1],Prior[j,2],Prior[j,3],log=TRUE)+LpreviousT)
            }
            
            alpha <- min(c(1, exp(logratio)))
            if (warn) print(paste(as.character(c(logratio, exp(logratio), alpha))))
            # print(c(alpha, propvarp[j], newvarp[j]))
            # 15/2/2015 Pour éviter des erreurs
            if (!is.finite(alpha)) alpha <- 1
            if (runif(1, min=0, max=1) <= alpha) {
              newvarp <- propvarp
              LpreviousT <- Lprevious2
              if (WAIC.out) {
                previousWAIC <- attributes(Lprevious2)$WAIC
              }
            }
          } else {
            newvarp <- propvarp
            LpreviousT <- Lprevious2
            if (WAIC.out) {
              previousWAIC <- attributes(Lprevious2)$WAIC
            }
          }
        }
      }
      varp[i, 1:nbvar] <- newvarp
      varp[i, "Ln L"] <- LpreviousT
      if (WAIC.out) WAIC.array[i, kk, 1:length(previousWAIC)] <- previousWAIC
      
      if (MaxL["Ln L"] < varp[i, "Ln L"]) {
        MaxL <- varp[i, ]
      }
      
      
      # 6/10/2012: Je stocke tout	
      #	if ((i>n.adapt) && ((i%%thin)==0)) {
      varp2[cpt, 1:nbvar] <- newvarp
      varp2[cpt, "Ln L"] <- varp[i, "Ln L"]
      cpt <- cpt + 1
      #	}
      
      if (tf) {
        cpt_trace <- cpt_trace+1
        if (cpt_trace>=pt) {
          cat(paste("Chain ", kk, ": [", i, "] ", as.numeric(varp[i, "Ln L"]), "\n", sep=""))
          cpt_trace <- 0
        }
      }
      
      if (traceML) {
        if (as.numeric(varp[i, "Ln L"]) > previousML) {
          previousML <- varp[i, "Ln L"]
          cat(d(varp[i, 1:nbvar]))
          cat("\n")
        }
      }
      
      
      
      if (adaptive) {
        if ((i %% adaptive.lag) == 0) {
          
          #		    sdp <- 
          for (j in 1:nbvar) {
            if (sdg[j] >= 0) {
              dta <- varp[(i-adaptive.lag):i, j]
              txaccept <- sum(diff(dta)!=0)/(length(dta)-1)
              sdg[j] <- adaptive.fun(txaccept)*sdg[j]
            }
          }
          # print(sdg)
        }
      }
      
    }
    
    lp <- getFromNamespace("mcmc", ns="coda")(varp2[seq(from=(n.adapt+1), to=(cpt-1), by=thin), 1:nbvar, drop=FALSE], start=n.adapt+1, thin=thin)
    lp1 <- getFromNamespace("mcmc", ns="coda")(varp2[seq(from=(n.adapt+1), to=(cpt-1), by=1), 1:nbvar, drop=FALSE], start=n.adapt+1, thin=1)
    
    # Ca c'est uniquement pour les chaînes
    # Ce n'est pas ca qui fait perdre du temps
    
    # Je stocke les résultats avec thin
    res<-c(res, list(lp))
    restot <- c(restot, list(lp1))
    resL <- c(resL, list(varp2[seq(from=(n.adapt+1), to=(cpt-1), by=thin), "Ln L"]))
    resLtot <- c(resLtot, list(varp2[seq(from=(n.adapt+1), to=(cpt-1), by=1), "Ln L"]))
  }
  
  names(res) <- 1:n.chains
  res <- getFromNamespace("as.mcmc.list", ns="coda")(res)
  names(restot) <- 1:n.chains
  restot <- getFromNamespace("as.mcmc.list", ns="coda")(restot)
  
  
  if (trace) {
    mL <- +Inf
    vmL <- NULL
    for (cL in 1:length(resLtot)) {
      imL <- which.max(resLtot[[cL]])[1]
      if (mL > resLtot[[cL]][imL]) {
        mL <- resLtot[[cL]][imL]
        vmL <- restot[[cL]][imL, ]
      }
    }
    
    cat(paste0(mL, ": Best likelihood for: \n"))
    cat(d(vmL))
  }
  
  # 16/9/2019: Si un seul paramètre, renvoie quand même une matrix
  if (is.null(dim(res[[1]]))) {
    for (i in 1:length(res)) {
      res[[i]] <- matrix(data = res[[i]], ncol = 1, dimnames = list(c(NULL), rownames(parameters)))
    }
    for (i in 1:length(restot)) {
      restot[[i]] <- matrix(data = restot[[i]], ncol = 1, dimnames = list(c(NULL), rownames(parameters)))
    }
  }
  
  if (WAIC.out) {
    WAIC.array.thined <- WAIC.array[seq(from=n.adapt+1, to=n.adapt+n.iter, by=thin), , , drop=FALSE]
  } else {
    WAIC.array.thined <- NULL
  }
  
  out <- (list(resultMCMC=res, 
               resultMCMC.total=restot, 
               resultLnL=resL, 
               resultLnL.total=resLtot, 
               WAIC.total=WAIC.array, 
               WAIC=WAIC.array.thined,
               parametersMCMC=list(parameters=parameters, n.iter=n.iter, n.chains=n.chains, n.adapt=n.adapt, thin=thin, 
                                   SDProp.end=structure(sdg, names=rownames(parameters)), 
                                   control=datax)))
  out <- addS3Class(out, "mcmcComposite")
  return(out)
  
}

.radioButtons_withHTML <- function (inputId, label, choices, selected = NULL, 
								    inline = FALSE, width = NULL) 
{

  # http://stackoverflow.com/users/4474157/nice
  
  tags <- NULL
  HTML <- NULL
  div <- NULL
  validateCssUnit <- NULL
  generateOptions_withHTML <- function (inputId, choices, selected, inline, type = "checkbox") 
  {
    options <- mapply(choices, names(choices), FUN = function(value, 
                                                              name) {
      inputTag <- tags$input(type = type, name = inputId, value = value)
      if (value %in% selected) 
        inputTag$attribs$checked <- "checked"
      if (inline) {
        tags$label(class = paste0(type, "-inline"), inputTag, 
                   tags$span(HTML(name)))
      }
      else {
        tags$div(class = type, tags$label(inputTag, tags$span(HTML(name))))
      }
    }, SIMPLIFY = FALSE, USE.NAMES = FALSE)
    div(class = "shiny-options-group", options)
  }
  choices <- getFromNamespace("choicesWithNames", ns="shiny")(choices)
    # shiny:::choicesWithNames
  selected <- if (is.null(selected)) 
    choices[[1]]
  else {
    getFromNamespace("validateSelected", ns="shiny")(selected, choices, inputId)
    # shiny:::validateSelected(selected, choices, inputId)
  }
  if (length(selected) > 1) 
    stop("The 'selected' argument must be of length 1")
  options <- generateOptions_withHTML(inputId, choices, selected, inline, 
                                      type = "radio")
  divClass <- "form-group shiny-input-radiogroup shiny-input-container"
  if (inline) 
    divClass <- paste(divClass, "shiny-input-container-inline")
  tags$div(id = inputId, style = if (!is.null(width)) 
    paste0("width: ", validateCssUnit(width), ";"), class = divClass, 
    getFromNamespace("controlLabel", ns="shiny")(inputId, label), options)
}

#' list.packages lists the installed packages with their locations
#' @title List the installed packages with their locations
#' @author Marc Girondot
#' @return A list with the installed packages and their version.
#' @description List the installed packages with their locations and version.
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' list.packages()
#' }
#' @export


list.packages <- function() {
  ip <- as.data.frame(installed.packages(), stringsAsFactors = FALSE)
  lp <- levels(as.factor(ip[, "LibPath"]))
  grandL <- list()
  for (p in lp) {
    L <- subset(ip, subset=(ip$LibPath==p), select=c("Package", "Version"))
    rownames(L) <- L[, "Package"]
    grandL <- c(grandL, list(L))
  }
  names(grandL) <- lp
  return(grandL)
}

#' plot.cutter plot result of cutter
#' @title Plot results of cutter that best describe distribution
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The matrix of all the mcmc curves with invisible state.
#' @param x A result file generated by cutter
#' @param col.hist The color of histogram
#' @param col.DL The color of below of above samples
#' @param col.dist The color of distribution
#' @param col.unobserved The color of unobserved states
#' @param col.mcmc The color of mcmc outputs
#' @param legend If TRUE, a legend is shown
#' @param show.DL If TRUE, the limits of DL are shown
#' @param show.plot If FALSE, no plot is shown
#' @param set In case of mixture, will show and return only this set
#' @param ... Parameters for plot
#' @description Plot the estimates of cut distribution.
#' @family Distributions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' # _______________________________________________________________
#' # right censored distribution with gamma distribution
#' # _______________________________________________________________
#' # Detection limit
#' DL <- 100
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc>DL] <- +Inf
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, upper_detection_limit=DL, 
#'                            cut_method="censored")
#' result
#' plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # The same data seen as truncated data with gamma distribution
#' # _______________________________________________________________
#' obc <- obc[is.finite(obc)]
#' # search for the parameters the best fit these truncated data
#' result <- cutter(observations=obc, upper_detection_limit=DL, 
#'                            cut_method="truncated")
#' result
#' plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # left censored distribution with gamma distribution
#' # _______________________________________________________________
#' # Detection limit
#' DL <- 10
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc<DL] <- -Inf
#' # search for the parameters the best fit these truncated data
#' result <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored")
#' result
#' plot(result)
#' plot(result, xlim=c(0, 200), breaks=seq(from=0, to=200, by=10))
#' # _______________________________________________________________
#' # left and right censored distribution
#' # _______________________________________________________________
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # Detection limit
#' LDL <- 10
#' # remove the data below the detection limit
#' obc[obc<LDL] <- -Inf
#' # Detection limit
#' UDL <- 100
#' # remove the data below the detection limit
#' obc[obc>UDL] <- +Inf
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                            upper_detection_limit=UDL, 
#'                           cut_method="censored")
#' result
#' plot(result, xlim=c(0, 150), col.DL=c("black", "grey"), 
#'                              col.unobserved=c("green", "blue"), 
#'      breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # Example with two values for lower detection limits
#' # corresponding at two different methods of detection for example
#' # with gamma distribution
#' # _______________________________________________________________
#' obc <- rgamma(50, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL1 <- 10
#' # remove the data below the detection limit
#' obc[obc<LDL1] <- -Inf
#' obc2 <- rgamma(50, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL2 <- 20
#' # remove the data below the detection limit
#' obc2[obc2<LDL2] <- -Inf
#' obc <- c(obc, obc2)
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, 
#'                            lower_detection_limit=c(rep(LDL1, 50), rep(LDL2, 50)), 
#'                           cut_method="censored")
#' result
#' # It is difficult to choose the best set of colors
#' plot(result, xlim=c(0, 150), col.dist="red", 
#'      col.unobserved=c(rgb(red=1, green=0, blue=0, alpha=0.1), 
#'                       rgb(red=1, green=0, blue=0, alpha=0.2)), 
#'      col.DL=c(rgb(red=0, green=0, blue=1, alpha=0.5), 
#'                       rgb(red=0, green=0, blue=1, alpha=0.9)), 
#'      breaks=seq(from=0, to=200, by=10))
#' # ___________________________________________________________________
#' # left censored distribution comparison of normal, lognormal and gamma
#' # ___________________________________________________________________
#' # Detection limit
#' DL <- 10
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc<DL] <- -Inf
#' # search for the parameters the best fit these truncated data
#' result_gamma <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="gamma")
#' result_gamma
#' plot(result_gamma, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' result_lognormal <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="lognormal")
#' result_lognormal
#' plot(result_lognormal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' result_normal <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="normal")
#' result_normal
#' plot(result_normal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' compare_AICc(gamma=result_gamma, 
#'             lognormal=result_lognormal, 
#'             normal=result_normal)
#' # ___________________________________________________________________
#' # Test for similarity in gamma left censored distribution between two
#' # datasets
#' # ___________________________________________________________________
#' obc1 <- rgamma(100, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL <- 10
#' # remove the data below the detection limit
#' obc1[obc1<LDL] <- -Inf
#' obc2 <- rgamma(100, scale=10, shape=2)
#' # remove the data below the detection limit
#' obc2[obc2<LDL] <- -Inf
#' # search for the parameters the best fit these censored data
#' result1 <- cutter(observations=obc1, 
#'                   distribution="gamma", 
#'                   lower_detection_limit=LDL, 
#'                   cut_method="censored")
#' logLik(result1)
#' plot(result1, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#' result2 <- cutter(observations=obc2, 
#'                   distribution="gamma", 
#'                   lower_detection_limit=LDL, 
#'                   cut_method="censored")
#' logLik(result2)
#' plot(result2, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#' result_totl <- cutter(observations=c(obc1, obc2), 
#'                       distribution="gamma", 
#'                       lower_detection_limit=LDL, 
#'                       cut_method="censored")
#' logLik(result_totl)
#' plot(result_totl, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#'      
#' compare_AICc(Separate=list(result1, result2), 
#'             Common=result_totl, factor.value=1)
#' compare_BIC(Separate=list(result1, result2), 
#'             Common=result_totl, factor.value=1)           
#' }
#' @method plot cutter
#' @export



plot.cutter <- function(x, 
                        col.hist="grey", 
                        col.DL="blue", 
                        col.dist="black",
                        col.unobserved="green", 
                        col.mcmc=rgb(red=0.6, green = 0.0, blue = 0.0, alpha = 0.01), 
                        legend=TRUE, show.DL = TRUE, show.plot=TRUE, set=NULL, ...) {
  # x <- result
  # col.hist="grey"
  # col.DL="blue"
  # col.dist="black"
  # col.unobserved="green"
  # legend=TRUE
  # p3p <- list()
  # col.mcmc=rgb(red=0.6, green = 0.0, blue = 0.0, alpha = 0.005)
  
  p3p <- list(...) # p3p <- list()
  
  getparcutter <- getFromNamespace(".getparcutter", ns="HelpersMG")
  
  
  ddistr <- switch(x$distribution, 
                   gamma=dgamma, 
                   lognormal=dlnorm, 
                   normal=dnorm, 
                   weibull=dweibull, 
                   generalized.gamma=dggamma)
  
  qdistr <- switch(x$distribution, 
                   gamma=qgamma, 
                   lognormal=qlnorm, 
                   normal=qnorm, 
                   weibull=qweibull, 
                   generalized.gamma=qggamma)
  
  
  obc <- x$observations[, "Observations"]
  LDL <- unique(x$observations[, "LDL"])
  UDL <- unique(x$observations[, "UDL"])
  
  if (is.null(p3p$xlab))  p3p$xlab <- "Values"
  if (is.null(p3p$ylab))  p3p$ylab <- "Density"
  if (is.null(p3p$main))  p3p$main <- ""
  
  n.mixture <- x$n.mixture
  
  if (!is.null(x$par_mcmc)) {
    par <- x$par_mcmc[2, ]
  } else {
    par <- x$par
  }
  
  parX <- as.list(par)
  pparX <- getparcutter(par, set=NULL)
  parX_mixture <- list()
  for (p in 1:n.mixture)
    parX_mixture <- c(parX_mixture, list(getparcutter(parX, set=p)))
  
  
  if (is.null(p3p$xlim)) {
    nlength <- 100
    mx <- NULL
    for (p in 1:n.mixture) {
      mx <- max(c(mx, do.call(qdistr, args=c(list(p = 0.999, lower.tail = TRUE, log.p = FALSE), parX_mixture[[p]]))))
    }
    
    if (mx > max(x$observations[, "Observations"], na.rm=TRUE) * 5)
      mx <- max(x$observations[, "Observations"], na.rm=TRUE) * 5
    
  } else {
    mx <- p3p$xlim[2]
  }
  
  p3p$xlim <- c(0, mx)
  mi <- 0
  
  if (is.null(p3p$breaks)) {
    es <- suppressWarnings(do.call(hist, args=modifyList(p3p, list(col=col.hist, 
                                                                   x=obc[is.finite(obc)], 
                                                                   plot=FALSE, freq = FALSE))))
    breaks_begin <- es$breaks[1]
    breaks_end <- tail(es$breaks, n=1)
    delta_break <- es$breaks[2]-es$breaks[1]
    
    if (all(!is.na(LDL)) & (length(unique(LDL)) == 1)) {
      breaks <- unique(c(0, LDL, seq(from=LDL, to=breaks_end+delta_break, by=delta_break)))
    } else {
      breaks <- es$breaks
    }
    if (all(!is.na(UDL))) breaks <- unique(sort(c(UDL, breaks)))
    p3p$breaks <- breaks
  }
  
  npoints <- 10^floor(log10(mx)+1)
  if (npoints < 1000) {
    npoints <- 1000
  }
  x_axis <- seq(from=0, to=mx, length.out=npoints)[-1]
  if (all(!is.na(LDL))) x_axis <- sort(unique(c(LDL, x_axis)))
  if (all(!is.na(UDL))) x_axis <- sort(unique(c(UDL, x_axis)))
  npoints <- length(x_axis) + 1
  
  y_axis <- rep(0, npoints-1)
  for (p in 1:n.mixture)
    y_axis <- y_axis + do.call(ddistr, args=c(list(x=x_axis, log = FALSE), parX_mixture[[p]])) * pparX[p]
  
  if ((!is.null(x$mcmc)) & (!is.null(col.mcmc))) {
    pl_mcmc <- matrix(data = NA, ncol=length(x_axis), nrow = nrow(x$mcmc$resultMCMC[[1]]))
    
    # Là je fais tous les MCMC
    for (i in 1:nrow(x$mcmc$resultMCMC[[1]])) {
      par_mcmc <- x$mcmc$resultMCMC[[1]][i, ]
      pparX_mcmc <- getparcutter(par_mcmc, set=NULL)
      
      y <- rep(0, length(x_axis))
      if (is.null(set)) {
      for (p in 1:n.mixture)
        y <- y + do.call(ddistr, args=c(list(x=x_axis, log = FALSE), as.list(getparcutter(par_mcmc, set=p)))) * pparX_mcmc[p]
      } else {
        y <- y + do.call(ddistr, args=c(list(x=x_axis, log = FALSE), as.list(getparcutter(par_mcmc, set=set)))) * pparX_mcmc[set]
      }
      pl_mcmc[i, ] <- y
    }
    colnames(pl_mcmc) <- specify_decimal(x_axis, decimals = 4)
  } else {
    pl_mcmc <- NULL
  }
  
  
  if (is.null(p3p$ylim)) {
    es <- suppressWarnings(do.call(hist, args=modifyList(p3p, list(col=col.hist, 
                                                                   x=obc[is.finite(obc)], plot=FALSE, freq = FALSE))))
    y <- y_axis
    if (!is.null(pl_mcmc)) y <- c(y, as.vector(pl_mcmc))
    
    p3p$ylim <- c(0, max(c(y[is.finite(y)], es$density), na.rm=TRUE))
  }
  
  es <- suppressWarnings(do.call(hist, args=modifyList(p3p, list(col=col.hist, 
                                                x=obc[is.finite(obc)], plot=show.plot, freq = FALSE))))
  
  
  # Dans u j'ai le nombre total de limites
  u <- 0
  if (all(!is.na(UDL))) u <- length(UDL)
  if (all(!is.na(LDL))) u <- u + length(LDL)
  
  if ((u == 0) | (!show.DL)) {
    if (legend) {
      if (is.null(pl_mcmc)) {
        if (show.plot) legend("topright", legend = c("Observed values", "Fitted distribution"), 
               col=c(col.hist, col.dist), 
               pch=c(15, NA), lty=c(NA, 1))
      } else {
        c2 <- col.mcmc
        substr(c2, 8, 9) <- "88"
        if (show.plot) legend("topright", legend = c("Observed values", "Fitted distribution", "Posterior predictive distribution"), 
               col=c(col.hist, col.dist, c2), 
               pch=c(15, NA, NA), lty=c(NA, 1, 1), lwd=c(NA, 1, 10))
      }
    }
  } else {
    
    col.unobserved_X <- rep(col.unobserved, u)[1:u]
    col.DL_X <- rep(col.DL, u)[1:u]
    cpt_u <- 0
    
    if (all(!is.na(LDL))) {
      
      for (i in seq_along(LDL)) {
        posy <- i/(length(LDL)+1)
        x_axis_LDL <- x_axis[x_axis<=LDL[i]]
        # npoints <- 100
        # x_axis_LDL <- seq(from=0, to=LDL[i], length.out=npoints)
        y_LDL <- rep(0, length(x_axis_LDL))
        
        for (p in 1:n.mixture)
          y_LDL <- y_LDL + do.call(ddistr, args = modifyList(list(x=x_axis_LDL, log = FALSE), 
                                                             parX_mixture[[p]])) * pparX[p]
        
        # y_LDL <- ifelse(y_LDL>max(y), max(y), y_LDL)
        if (show.plot) polygon(c(x_axis_LDL, LDL[i], 0), 
                c(y_LDL, 0, 0), 
                col=col.unobserved_X[cpt_u+i], border=col.unobserved_X[cpt_u+i])
        
      }
      
      if (!is.null(x$LDL)) {
        for (i in seq_along(LDL)) {
          posy <- i/(length(LDL)+1)
          
          if (nrow(x$LDL) == 3) {
            
            if (show.plot) segments(x0=x$LDL[1, i], x1=x$LDL[3, i], y0=ScalePreviousPlot(x=0, y=posy)$y, 
                     y1=ScalePreviousPlot(x=0, y=posy)$y, 
                     col=col.DL_X[cpt_u+i])
            if (show.plot) points(x=x$LDL[2, i], y=ScalePreviousPlot(x=0, y=posy)$y, pch=19, col=col.DL_X[cpt_u+i])
            if (show.plot) segments(x0=x$LDL[1, i], x1=x$LDL[1, i], y0=ScalePreviousPlot(x=0, y=posy-0.03)$y, y1=ScalePreviousPlot(x=0, y=posy+0.03)$y, 
                     col=col.DL_X[cpt_u+i])
            if (show.plot) segments(x0=x$LDL[3, i], x1=x$LDL[3, i], y0=ScalePreviousPlot(x=0, y=posy-0.03)$y, y1=ScalePreviousPlot(x=0, y=posy+0.03)$y, 
                     col=col.DL_X[cpt_u+i])
          } else {
            if (show.plot) points(x=x$LDL[1, i], y=ScalePreviousPlot(x=0, y=posy)$y, pch=19, col=col.DL_X[cpt_u+i])
          }
        }
        cpt_u <- cpt_u + length(LDL)
      }
    }
    
    if (all(!is.na(UDL))) {
      for (i in seq_along(UDL)) {
        posy <- i/(length(UDL)+1)
        npoints <- 100
        x_axis_UDL <- seq(from=UDL[i], to=mx, length.out=npoints)
        
        y_UDL <- rep(0, length(x_axis_UDL))
        for (p in 1:n.mixture)
          y_UDL <- y_UDL + do.call(ddistr, args = modifyList(list(x=x_axis_UDL, log = FALSE), 
                                                             parX_mixture[[p]])) * pparX[p]
        
        # y_UDL <- ifelse(y_UDL > max(y), max(y), y_UDL)
        if (show.plot) polygon(c(x_axis_UDL, mx, UDL[i]), 
                c(y_UDL, 0, 0), 
                col=col.unobserved_X[cpt_u+i], border=col.unobserved_X[cpt_u+i])
      }
      
      if (!is.null(x$UDL)) {
        for (i in seq_along(UDL)) {
          posy <- i/(length(UDL)+1)
          
          if (nrow(x$UDL) == 3) {
            if (show.plot) segments(x0=x$UDL[1, i], x1=x$UDL[3, i], y0=ScalePreviousPlot(x=0, y=posy)$y, y1=ScalePreviousPlot(x=0, y=posy)$y, 
                     col=col.DL_X[cpt_u+i])
            if (show.plot) points(x=x$UDL[2, i], y=ScalePreviousPlot(x=0, y=posy)$y, pch=19, col=col.DL_X[cpt_u+i])
            if (show.plot) segments(x0=x$UDL[1, i], x1=x$UDL[1, i], y0=ScalePreviousPlot(x=0, y=posy-0.03)$y, y1=ScalePreviousPlot(x=0, y=posy+0.03)$y, 
                     col=col.DL_X[cpt_u+i])
            if (show.plot) segments(x0=x$UDL[3, i], x1=x$UDL[3, i], y0=ScalePreviousPlot(x=0, y=posy-0.03)$y, y1=ScalePreviousPlot(x=0, y=posy+0.03)$y, 
                     col=col.DL_X[cpt_u+i])
          } else {
            if (show.plot) points(x=x$UDL[1, i], y=ScalePreviousPlot(x=0, y=posy)$y, pch=19, col=col.DL_X[cpt_u+i])
          }
        }
      }
    }
    
    
    if (legend) {
      if (is.null(pl_mcmc)) {
        if (show.plot) legend("topright", legend = c("Observed values", "Fitted distribution", 
                                      paste(rep("Unobserved distribution", u), na.omit(c(LDL, UDL))), 
                                      paste(rep("Mean unobserved", u), na.omit(c(LDL, UDL)))), 
               col=c(col.hist, col.dist, col.unobserved_X, col.DL_X), 
               pch=c(15, NA, rep(17, u), rep(19, u)), lty=c(NA, 1, rep(NA, u) , rep(1, u)))
      } else {
        c2 <- col.mcmc
        substr(c2, 8, 9) <- "88"
        if (show.plot) legend("topright", legend = c("Observed values", "Fitted distribution", "Posterior predictive distribution", 
                                      paste(rep("Unobserved distribution", u), na.omit(c(LDL, UDL))), 
                                      paste(rep("Mean unobserved", u), na.omit(c(LDL, UDL)))), 
               col=c(col.hist, col.dist, c2, col.unobserved_X, col.DL_X), 
               pch=c(15, NA, NA, rep(17, u), rep(19, u)), lty=c(NA, 1, 1, rep(NA, u) , rep(1, u)), 
               lwd=c(NA, 1, 10, rep(NA, u) , rep(1, u)))
        
      }
    }
  }
  
  if (!is.null(pl_mcmc)) {
    for (i in 1:nrow(pl_mcmc)) {
      y <- pl_mcmc[i, ]
      if (show.plot) lines(x_axis[is.finite(y)], y[is.finite(y)], col=col.mcmc)
    }
  }
  
  if (show.plot) lines(x_axis[is.finite(y_axis)], y_axis[is.finite(y_axis)], col=col.dist)
  return(invisible(t(pl_mcmc)))
}

#' RM_get gets a value in results managment to an object
#' @title Get a value in a results managment to an object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Return a value in a results managment object
#' @param x The object in which to get a result in a results managment
#' @param RM The name of results managment stored
#' @param RMname The name of the results managment to be read
#' @param valuename The name of the value to be read
#' @description Return the value valuename of the results managment RMname.
#' @family Results Managment
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Let an object of class objclass being created
#' obj <- list(A=100, name="My object")
#' class(obj) <- "objclass"
#' # And now I create a RM to this object
#' obj <- RM_add(x=obj, RMname="NewAnalysis1")
#' RM_list(obj)
#' obj <- RM_add(x=obj, RMname="NewAnalysis2")
#' RM_list(obj)
#' obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V1", value=100)
#' RM_get(x=obj, RMname="NewAnalysis2", valuename="V1")
#' }
#' @export

RM_get <- function(x=stop("An object with results managment must be provided"), 
                   RM="RM", 
                   RMname=NULL, 
                   valuename=NULL) {
  xa <- attributes(x)
  # dans rx j'ai le RM
  if ((!is.null(valuename)) & (!is.null(RMname)) & (!is.null(RM))) {
    rx <- xa[[RM]][[RMname]][[valuename]]
  } else {
    if ((!is.null(RMname)) & (!is.null(RM))) {
      rx <- xa[[RM]][[RMname]]
    } else {
      if ((!is.null(RM))) {
        rx <- xa[[RM]]
      } else {
        rx <- NULL
      }
    }
  }
  return(rx)
}

#' logLik.compareAIC Return Log Likelihood of a fit
#' @title Return Log Likelihood generated by FormatCompareAIC
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The Log Likelihood value for the fitted model with data
#' @param object A result generated by FormatCompareAIC
#' @param ... Not used
#' @description Return Log Likelihood generated by FormatCompareAIC
#' @family AIC functions
#' @examples
#' \dontrun{
#' ED <- FormatCompareAIC(logLik=-140, nobs=100, df=3)
#' logLik(ED)
#' }
#' @method logLik compareAIC
#' @export


logLik.compareAIC <- function(object, ...) {
  l <- object$logLik
  attributes(l) <- list(nall=attributes(object)$nall, 
                                                  nobs=attributes(object)$nobs, 
                                                  df=attributes(object)$df)
    return(l)
}

#' LD50 estimates the parameters that best describe LD50
#' @title Estimate the parameters that best describe LD50
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with the LD50, Transitional Range of Doses and their SE
#' @param alive A vector with alive individuals at the end of experiment
#' @param dead A vector with dead individuals at the end of experiment
#' @param N A vector with total numbers of tested individuals
#' @param doses The doses
#' @param df A dataframe with at least two columns named alive, dead or N and doses columns
#' @param l The limit to define TRD (see Girondot, 1999)
#' @param parameters.initial Initial values for P, S or K search as a vector, ex. c(P=29, S=-0.3)
#' @param fixed.parameters Parameters that will not be changed during fit
#' @param SE Standard errors for parameters
#' @param equation Could be "logistic", "logit", "probit", Hill", "Richards", "Hulin", "flexit" or "Double-Richards"
#' @param replicates Number of replicates to estimate confidence intervals
#' @param range.CI The range of confidence interval for estimation, default=0.95
#' @param limit.low.TRD.minimum Minimum lower limit for TRD
#' @param limit.high.TRD.maximum Maximum higher limit for TRD
#' @param doses.plot Sequences of doses that will be used for plotting. If NULL, does not estimate them
#' @param print Do the results must be printed at screen? TRUE (default) or FALSE
#' @description Estimate the parameters that best describe LD50\cr
#' Logistic and logit models are the same but with different parametrization:\cr
#' logistic = 1/(1+exp((1/S)*(P-d)))\cr
#' logit = 1/(1+exp(P+d*S))\cr
#' See these publications for the description of equations:\cr
#' Girondot, M. 1999. Statistical description of temperature-dependent sex determination using maximum likelihood. Evolutionary Ecology Research, 1, 479-486.\cr
#' Godfrey, M.H., Delmas, V., Girondot, M., 2003. Assessment of patterns of temperature-dependent sex determination using maximum likelihood model selection. Ecoscience 10, 265-272.\cr
#' Hulin, V., Delmas, V., Girondot, M., Godfrey, M.H., Guillon, J.-M., 2009. Temperature-dependent sex determination and global change: are some species at greater risk? Oecologia 160, 493-506.\cr
#' The flexit equation is not still published :
#' \deqn{if dose < P then (1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}{if dose < P then (1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}
#' \deqn{if dose > P then 1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}{if dose > P then 1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}
#' with:\cr
#'      \deqn{S1 = S/((4/K1)*(2^(-K1))^(1/K1+1)*(2^K1-1))}{S1 = S/((4/K1)*(2^(-K1))^(1/K1+1)*(2^K1-1))}
#'      \deqn{S2 = S/((4/K2)*(2^(-K2))^(1/K2+1)*(2^K2-1))}{S2 = S/((4/K2)*(2^(-K2))^(1/K2+1)*(2^K2-1))}
#' @family LD50 functions
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
#' Alive=c(10, 12, 8, 6, 2, 1),
#' Dead=c(0, 1, 5, 6, 9, 15))
#' LD50_logistic <- LD50(data, equation="logistic")
#' predict(LD50_logistic, doses=c(140, 170))
#' plot(LD50_logistic, xlim=c(0, 300), at=seq(from=0, to=300, by=50))
#' LD50_probit <- LD50(data, equation="probit")
#' predict(LD50_probit, doses=c(140, 170))
#' plot(LD50_probit)
#' LD50_logit <- LD50(data, equation="logit")
#' predict(LD50_logit, doses=c(140, 170))
#' plot(LD50_logit)
#' LD50_hill <- LD50(data, equation="hill")
#' predict(LD50_hill, doses=c(140, 170))
#' plot(LD50_hill)
#' LD50_Richards <- LD50(data, equation="Richards")
#' predict(LD50_Richards, doses=c(140, 170))
#' plot(LD50_Richards)
#' LD50_Hulin <- LD50(data, equation="Hulin")
#' predict(LD50_Hulin, doses=c(140, 170))
#' plot(LD50_Hulin)
#' LD50_DoubleRichards <- LD50(data, equation="Double-Richards")
#' predict(LD50_DoubleRichards, doses=c(140, 170))
#' plot(LD50_DoubleRichards)
#' LD50_flexit <- LD50(data, equation="flexit")
#' predict(LD50_flexit, doses=c(140, 170))
#' plot(LD50_flexit)
#' }
#' @export


LD50 <- function(df=NULL, alive=NULL, dead=NULL, N=NULL, 
                 doses=NULL, 
                 l=0.05, parameters.initial=NULL, 
                 fixed.parameters=NULL, SE=NULL,
                 equation="logistic", replicates=1000, range.CI=0.95, 
                 limit.low.TRD.minimum=5, limit.high.TRD.maximum=1000, print=TRUE, 
                 doses.plot=seq(from=0, to=1000, by=0.1)) {
  # df=NULL; alive=NULL; dead=NULL; N=NULL; doses=NULL; l=0.05; parameters.initial=NULL; fixed.parameters=NULL; SE=NULL;equation="logistic"; replicates=1000; range.CI=0.95; limit.low.TRD.minimum=5; limit.high.TRD.maximum=1000; print=TRUE; doses.plot=seq(from=0, to=1000, by=0.1)
  # df <- data.frame(Doses=c(80, 120, 150, 150, 180, 200), Alive=c(10, 12, 8, 6, 2, 1), Dead=c(0, 1, 5, 6, 9, 15))
  
  range.CI.qnorm <- qnorm(1-((1-range.CI)/2))
  
  if (!is.null(df)) {
    if ((!inherits(df, "data.frame")) & (!inherits(df, "matrix"))) {
      stop("df parameter must be a data.frame or a matrix")
    }
    
    namesdf <- tolower(colnames(df))
    alive <- NULL
    dead <- NULL
    N <- NULL
    
    if (any(namesdf=="alive")) alive <- df[, which(namesdf=="alive")]
    if (any(namesdf=="dead")) dead <- df[, which(namesdf=="dead")]
    if (any(namesdf=="n")) N <- df[, which(namesdf=="n")]
    if (any(namesdf=="doses")) doses <- df[, which(namesdf=="doses")]
  }
  
  if (is.null(doses) ) {
    stop("Doses must be provided")
  }
  
  
  cpt1 <- ifelse(is.null(alive), 0, 1)+ifelse(is.null(dead), 0, 1)+ifelse(is.null(N), 0, 1)
  if (cpt1<2) {
    stop("At least two informations from alive, dead or N must be provided")
  }
  
  if (is.null(alive)) alive <- N-dead
  if (is.null(dead)) dead <- N-alive
  if (is.null(N)) N <- alive+dead
  
  if (length(doses)!=length(N)) {
    stop("A dose must be provided for each experiment")
  }
  
  equation <- tolower(equation)
  
  equation <- match.arg(equation, choices = c("gsd", "hulin", "logistic", "logit", "probit", "hill", "flexit", "richards", "double-richards"))
  
  if (is.null(parameters.initial)) parameters.initial <- c(P=NA, S=NA, K=0, K1=1, K2=0)
  
  if (equation=="double-richards") parameters.initial["K2"] <- 1
  if (equation=="hulin") parameters.initial["K1"] <- 1/max(doses)
  
  
  par <- parameters.initial
  if (is.na(par["P"])) {
    if ((equation!="probit") & (equation!="logit")) {
      par["P"] <- doses[which.min(abs((alive/(alive+dead))-0.5))]
    } else {
      par["P"] <- 0
    }
  }
  
  if (is.na(par["S"])) {
    if ((equation!="probit")) {
    pente <- lm(alive/(alive+dead) ~ doses)
    par["S"] <- pente$coefficients["doses"]
    if (equation!="flexit") par["S"] <- 1/(4*par["S"])
    } else {
      par["S"] <- 0
    }
  }
  
  LD50_fit <- getFromNamespace(".LD50_fit", ns="HelpersMG")
  modelLD50 <- getFromNamespace(".modelLD50", ns="HelpersMG")
  
  if (equation!="richards") par <- par[which(names(par)!="K")]
  if (equation!="hulin" & equation!="double-richards" & equation!="flexit") par <- par[which(names(par)!="K1")]
  if (equation!="hulin" & equation!="double-richards" & equation!="flexit") par <- par[which(names(par)!="K2")]
  
  # Je commence toujours toujours par un glm
  if (equation=="probit") {
    inter <- glm(cbind(dead, alive) ~ doses, family=binomial(link="probit"))
    par["P"] <- unname(inter$coefficients["(Intercept)"])
    par["S"] <- unname(inter$coefficients["doses"])
  } else {
    inter <- glm(cbind(dead, alive) ~ doses, family=binomial(link="logit"))
    if (equation=="logit") {
      par["P"] <- unname(inter$coefficients["(Intercept)"])
      par["S"] <- unname(inter$coefficients["doses"])
    } else {
      a <- inter$coefficients["(Intercept)"]
      b <- inter$coefficients["doses"]
      par["P"] <- unname(-a/b)
      if (equation=="flexit") {
        par["S"] <- 1/(4*unname(-1/b))
      } else {
        par["S"] <- unname(-1/b)
      }
    }
  }
  
  repeat {
    result  <- optim(par, LD50_fit, fixed.parameters=fixed.parameters, alive=alive, 
                     N=N, doses=doses, equation=equation, method="BFGS", hessian=TRUE, 
                     control = list(maxit=1000))
    
    if (result$convergence==0) break
    par <- result$par
    if (print) print("Convergence is not achieved. Optimization continues !")
  }
  
  par <- c(result$par, fixed.parameters)
  
  if (!is.null(SE)) {
    res <- SE
  } else {
    mathessian <- result$hessian
    
    inversemathessian <- try(solve(mathessian), silent=TRUE)
    
    if (inherits(inversemathessian, "try-error")) {
      res <- rep(NA, length(par))
      names(res) <- names(par)
    } else {
      res <- abs(diag(inversemathessian))
    }
  }
  
  result$SE <- res
  result$AIC <- 2*result$value+2*length(result$par)
  
  
  result$range.CI <- range.CI
  result$l <- l
  
  l.l.TRD.c <- NULL
  l.h.TRD.c <- NULL
  TRD.c <- NULL
  #   plist <- NULL
  
  rep <- replicates-1
  df_par <- data.frame(P=unname(c(par["P"], rnorm(rep, par["P"], res["P"]))), 
                       S=unname(c(par["S"], rnorm(rep, par["S"], res["S"]))),
                       K=unname(ifelse(is.na(par["K"]), rep(NA, replicates), c(par["K"], rnorm(rep, par["K"], ifelse(is.na(res["K"]), 0, res["K"]))))),
                       K1=unname(ifelse(is.na(par["K1"]), rep(NA, replicates), c(par["K1"], rnorm(rep, par["K1"], ifelse(is.na(res["K1"]), 0, res["K1"]))))),
                       K2=unname(ifelse(is.na(par["K2"]), rep(NA, replicates), c(par["K2"], rnorm(rep, par["K2"], ifelse(is.na(res["K2"]), 0, res["K2"]))))))
  
  # sens==TRUE si descend
  sens <- (modelLD50(par, limit.low.TRD.minimum, equation)>modelLD50(par, limit.high.TRD.maximum, equation))
  
  out_LD50 <- apply(df_par, 1, function(par) {
    # marche en doses mais par en IP - corrige 9/9/2014
    limit.low.TRD <- c(limit.low.TRD.minimum, limit.high.TRD.maximum)
    limit.high.TRD <- c(limit.low.TRD.minimum, limit.high.TRD.maximum)
    P.TRD <- c(limit.low.TRD.minimum, limit.high.TRD.maximum)
    if (sens == (modelLD50(par, limit.low.TRD.minimum, equation)>modelLD50(par, limit.high.TRD.maximum, equation))) {
      
      
      
      repeat {
        doses.1 <- seq(from=limit.low.TRD[1], to=limit.low.TRD[2], length=3)
        p1 <- modelLD50(par, doses.1, equation)
        doses.2 <- seq(from=limit.high.TRD[1], to=limit.high.TRD[2], length=3)
        p2 <- modelLD50(par, doses.2, equation)
        doses.3 <- seq(from=P.TRD[1], to=P.TRD[2], length=3)
        p3 <- modelLD50(par, doses.3, equation)
        
        if (sens) {
          if (p1[2]<(1-l)) {
            limit.low.TRD <- doses.1[1:2]
          } else {
            limit.low.TRD <- doses.1[2:3]
          }
          if (p2[2]<l) {
            limit.high.TRD <- doses.2[1:2]
          } else {
            limit.high.TRD <- doses.2[2:3]
          }
          if (p3[2]<0.5) {
            P.TRD <- doses.3[1:2]
          } else {
            P.TRD <- doses.3[2:3]
          }
          
        } else {
          if (p1[2]<l) {
            limit.low.TRD <- doses.1[1:2]
          } else {
            limit.low.TRD <- doses.1[2:3]
          }
          if (p2[2]<(1-l)) {
            limit.high.TRD <- doses.2[1:2]
          } else {
            limit.high.TRD <- doses.2[2:3]
          }
          if (p3[2]>0.5) {
            P.TRD <- doses.3[1:2]
          } else {
            P.TRD <- doses.3[2:3]
          }
          
        }
        
        if (max(c(P.TRD[2]-P.TRD[1], limit.high.TRD[2]-limit.high.TRD[1], 
                  limit.low.TRD[2]-limit.low.TRD[1]))<1E-4) break
        
      }
      return(data.frame(l.l.TRD.c=mean(limit.low.TRD), l.h.TRD.c=mean(limit.high.TRD), 
                        TRD.c=mean(limit.high.TRD-limit.low.TRD), P.TRD=mean(P.TRD)))
      
    } else {
      return(data.frame(l.l.TRD.c=NA, l.h.TRD.c=NA, 
                        TRD.c=NA, P.TRD=NA))
      
    }
  })
  
  out_LD502 <- t(sapply(out_LD50, c))
  
  if (!is.null(doses.plot)) {
    out_LD50_plot <- apply(df_par, 1, function(par) {
      p <- modelLD50(par, doses.plot, equation)
      return(list(sr=p))
    }
    )
    
    
    out_LD502_plot <- sapply(out_LD50_plot, c, simplify = TRUE)
    # dans un df chaque colonne est une temperature
    out_LD503_plot <- t(sapply(out_LD502_plot, c, simplify = TRUE))
    #      dim(out_tsd3_plot)
    outquant <- apply(out_LD503_plot, 2, quantile, probs = c(0.025, 0.975))
    outquant <- rbind(outquant, mean=out_LD503_plot[1,], doses=doses.plot)
    result$CI <- outquant
    
  }
  
  
  result$LD50 <- as.numeric(out_LD502[1, "P.TRD"])
  result$SE_LD50 <- sd(as.numeric(out_LD502[, "P.TRD"]), na.rm = TRUE)
  result$TRD <- as.numeric(out_LD502[1, "TRD.c"])
  result$TRD_limits <- as.numeric(c(out_LD502[1, "l.l.TRD.c"], out_LD502[1, "l.h.TRD.c"]))
  result$SE_TRD <- sd(as.numeric(out_LD502[, "TRD.c"]), na.rm = TRUE)
  result$SE_TRD_limits <- c(sd(as.numeric(out_LD502[, "l.l.TRD.c"]), na.rm = TRUE), sd(as.numeric(out_LD502[, "l.h.TRD.c"]), na.rm = TRUE))
  limit.low.TRD <- result$TRD_limits[1]-range.CI.qnorm*result$SE_TRD_limits[1]
  limit.high.TRD <- result$TRD_limits[2]+range.CI.qnorm*result$SE_TRD_limits[2]
  
  
  saturated <- 0
  for (i in 1:length(N)) saturated <- saturated- dbinom(alive[i], N[i], alive[i]/N[i], log=TRUE)
  LRT <- 2*(result$value-saturated)
  result$GOF <- 1-pchisq(LRT, length(N)-length(par))
  
  result$alive <- alive
  result$dead <- dead
  result$N <- N
  result$doses <- doses
  result$equation <- equation
  result$l <- l
  result$fixed.parameters <- fixed.parameters
  
  if (print) {
    print(paste("The goodness of fit test is", sprintf("%.5f",result$GOF)))
    print(paste("The LD50 is", sprintf("%.3f",result$LD50), "SE", sprintf("%.3f",result$SE_LD50)))
    print(paste("The transitional range of doses is", sprintf("%.3f",result$TRD), "SE", sprintf("%.3f",result$SE_TRD)))
    print(paste("The lower limit of transitional range of doses is", sprintf("%.3f",result$TRD_limits[1]), "SE", sprintf("%.3f",result$SE_TRD_limits[1])))
    print(paste("The higher limit of transitional range of doses is", sprintf("%.3f",result$TRD_limits[2]), "SE", sprintf("%.3f",result$SE_TRD_limits[2])))
  }
  
  result <- addS3Class(result, "LD50")
  
  return(invisible(result))
}

#' plot.mcmcComposite plots the result of a MCMC search
#' @title Plot the result of a mcmcComposite object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return None
#' @param x A mcmcComposite object
#' @param chains The chains to use
#' @param parameters Name of parameters or "all"
#' @param transform Function to be used to transform the variable
#' @param legend If FALSE, the legend is not shown; see description
#' @param show.prior Should the prior be shown?
#' @param show.posterior.density Should the posterior density be shown?
#' @param scale.prior If TRUE, the prior is scaled at the same size as posterior
#' @param bty Design of box for Markov Chain plot
#' @param col.prior Color for prior curve 
#' @param lty.prior Type of line for prior curve 
#' @param lwd.prior Width of line for prior curve
#' @param col.posterior Color for posterior histogram 
#' @param lty.posterior Type of line for posterior histogram
#' @param lwd.posterior Width of line for posterior histogram
#' @param ylab.posterior y-label for posterior
#' @param ylab.prior y-label for prior
#' @param show.yaxis.prior Should the y-axis for prior be shown
#' @param las las parameter (orientation of y-axis graduation)
#' @param ... Graphical parameters to be sent to hist() or plot()
#' @param what can be Posterior, MarkovChain or LnL
#' @family mcmcComposite functions
#' @description Plot the results within a mcmcComposite object.\cr
#' If scale.prior is TRUE, another scale is shown at right.\cr
#' legend can take these values: \cr
#' FALSE, TRUE, topleft, topright, bottomleft, bottomright, c(x=, y=)
#' @examples 
#' \dontrun{
#' library(HelpersMG)
#' require(coda)
#' x <- rnorm(30, 10, 2)
#' dnormx <- function(data, x) {
#'  data <- unlist(data)
#'  return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
#' }
#' parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
#'                               Prior1=c(10, 0.5), Prior2=c(2, 0.5), 
#'                               SDProp=c(1, 1), 
#'                               Min=c(-3, 0), Max=c(100, 10), 
#'                               Init=c(10, 2), stringsAsFactors = FALSE, 
#'                               row.names=c('mean', 'sd'))
#'                               
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=x, 
#'                       adaptive = TRUE,
#'                       likelihood=dnormx, n.chains=4, 
#'                       n.adapt=100, thin=1, trace=1)
#' plot(mcmc_run, xlim=c(0, 20), parameters="mean")
#' plot(mcmc_run, xlim=c(0, 10), parameters="sd")
#' plot(x=mcmc_run, what="MarkovChain", ylim=c(0, 15), parameters="mean", 
#'      col.posterior = colorRampPalette(c("blue", "grey"), alpha = 0.001)
#'                      (mcmc_run$parametersMCMC$n.chains))
#' plot(mcmc_run, what="MarkovChain", ylim=c(0, 10), parameters="sd", 
#'      col.posterior = colorRampPalette(c("blue", "grey"), alpha = 0.001)
#'                      (mcmc_run$parametersMCMC$n.chains))
#' plot(mcmc_run, what="LnL", 
#'      col.posterior = colorRampPalette(c("blue", "grey"), alpha = 0.001)
#'                      (mcmc_run$parametersMCMC$n.chains))
#' mcmcforcoda <- as.mcmc(mcmc_run)
#' heidel.diag(mcmcforcoda)
#' raftery.diag(mcmcforcoda)
#' autocorr.diag(mcmcforcoda)
#' acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
#' acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
#' batchSE(mcmcforcoda, batchSize=100)
#' # The batch standard error procedure is usually thought to 
#' # be not as accurate as the time series methods used in summary
#' summary(mcmcforcoda)$statistics[,"Time-series SE"]
#' summary(mcmc_run)
#' as.parameters(mcmc_run)
#' lastp <- as.parameters(mcmc_run, index="last")
#' parameters_mcmc[,"Init"] <- lastp
#' # The n.adapt set to 1 is used to not record the first set of parameters
#' # then it is not duplicated (as it is also the last one for 
#' # the object mcmc_run)
#' mcmc_run2 <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=x, 
#' adaptive = TRUE,
#' likelihood=dnormx, n.chains=1, n.adapt=1, thin=1, trace=1)
#' mcmc_run3 <- merge(mcmc_run, mcmc_run2)
#' ####### no adaptation, n.adapt must be 0
#' parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
#' mcmc_run3 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
#' adaptive = TRUE,
#' likelihood=dnormx, n.chains=1, n.adapt=0, thin=1, trace=1)
#' 
#' #########################
#' ## Example with transform
#' #########################
#' 
#' x.1<-rnorm(6000, 2.4, 0.6)
#' x.2<-rlnorm(10000, 1.3,0.1)
#' 
#' X<-c(x.1, x.2)
#' hist(X,100,freq=FALSE, ylim=c(0,1.5))
#' Lnormlnorm <- function(par, val) {
#'   p <- invlogit(par["p"])
#'   return(-sum(log(p*dnorm(val, par["m1"], abs(par["s1"]), log = FALSE)+
#'                     (1-p)*dlnorm(val, par["m2"], abs(par["s2"]), log = FALSE))))
#' }
#' # Mean 1
#' m1=2.3; s1=0.5
#' # Mean 2
#' m2=1.3; s2=0.1
#' # proportion of category 1 - logit transform
#' p=0
#' 
#' par<-c(m1=m1, s1=s1, m2=m2, s2=s2, p=p)
#' 
#' result2<-optim(par, Lnormlnorm, method="BFGS", val=X, 
#'               hessian=FALSE, control=list(trace=1))
#'               
#' lines(seq(from=0, to=5, length=100), 
#' dnorm(seq(from=0, to=5, length=100), 
#'       result2$par["m1"], abs(result2$par["s1"])), col="red")
#' 
#' lines(seq(from=0, to=5, length=100), 
#'       dlnorm(seq(from=0, to=5, length=100), 
#'              result2$par["m2"], abs(result2$par["s2"])), col="green")
#' 
#' p <- invlogit(result2$par["p"])
#' 
#' paste("Proportion of Gaussian data",  p)
#' 
#' lines(seq(from=0, to=5, length=100), 
#'       p*dnorm(seq(from=0, to=5, length=100), 
#'               result2$par["m1"], result2$par["s1"])+
#'         (1-p)*dlnorm(seq(from=0, to=5, length=100), 
#'                      result2$par["m2"], result2$par["s2"]), col="blue")               
#'
#' parameters_mcmc <- data.frame(Density=c('dunif', 'dunif', 'dunif', 'dunif', 'dunif'), 
#'                                         Prior1=c(0, 0.001, 0, 0.001, -3), 
#'                                         Prior2=c(10, 10, 10, 10, 3), 
#'                                         SDProp=c(1, 1, 1, 1, 1), 
#'                                         Min=c(0, 0.001, 0, 0.001, -3), 
#'                                         Max=c(10, 10, 10, 10, 3), 
#'                                         Init=result2$par, stringsAsFactors = FALSE, 
#'                                         row.names=c('m1', 's1', 'm2', 's2', 'p'))
#'                                         
#' mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, val=X, 
#'                       parameters_name = "par",
#'                       adaptive = TRUE,
#'                       likelihood=Lnormlnorm, n.chains=1, 
#'                       n.adapt=100, thin=1, trace=100)
#' plot(mcmc_run, parameters="m1", breaks=seq(from=0, to =10, by=0.1), 
#'      legend=c(x=6, y=3.10))
#' plot(mcmc_run, parameters="p", transform=invlogit, xlim=c(0,1), 
#'      breaks=seq(from=0, to=1, by=0.01), legend=c(x=0.6, y=10))
#' plot(mcmc_run, parameters="p", xlim=c(-3,3), 
#'      breaks=seq(from=-3, to =3, by=0.05), legend=c(x=1, y= 3.10))
#' plot(mcmc_run, parameters="p", what="MarkovChain")
#' plot(mcmc_run, parameters="p", transform=invlogit, what="MarkovChain", 
#'      ylim=c(0, 1))
#' plot(mcmc_run, what="lnl", col.posterior = "black")
#'      
#' parameters_mcmc <- data.frame(Density=c('dunif', 'dunif', 'dunif', 'dunif', 'dnorm'), 
#'                                         Prior1=c(0, 0.001, 0, 0.001, 0.5), 
#'                                         Prior2=c(10, 10, 10, 10, 1), 
#'                                         SDProp=c(1, 1, 1, 1, 1), 
#'                                         Min=c(0, 0.001, 0, 0.001, -3), 
#'                                         Max=c(10, 10, 10, 10, 3), 
#'                                         Init=result2$par, stringsAsFactors = FALSE, 
#'                                         row.names=c('m1', 's1', 'm2', 's2', 'p'))
#'                                         
#' mcmc_run_pnorm <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, val=X, 
#'                       parameters_name = "par",
#'                       adaptive = TRUE,
#'                       likelihood=Lnormlnorm, n.chains=1, 
#'                       n.adapt=100, thin=1, trace=100)
#' plot(mcmc_run_pnorm, parameters="m1", breaks=seq(from=0, to =10, by=0.1), 
#'      legend=c(x=6, y=0.10))
#' plot(mcmc_run_pnorm, parameters="p", transform=invlogit, xlim=c(0,1), 
#'      breaks=seq(from=0, to=1, by=0.01), legend=c(x=0.6, y=0.10))
#' plot(x=mcmc_run_pnorm, parameters="p", xlim=c(-3,3), 
#'      breaks=seq(from=-3, to =3, by=0.05), legend=c(x=1, y= 0.10))
#'      
#'      
#' # Note that it is more logic to use beta distribution for p as a  
#' # proportion. However p value must be checked to be used in optim
#' # The use of logit transform can be a problem because it can stuck 
#' # the p value to 1 or 0 during fit.
#' 
#' Lnormlnorm <- function(par, val) {
#'   p <- par["p"]
#'   return(-sum(log(p*dnorm(val, par["m1"], abs(par["s1"]), log = FALSE)+
#'                     (1-p)*dlnorm(val, par["m2"], abs(par["s2"]), log = FALSE))))
#' }
#' 
#' # Example of beta distribution
#' 
#' # Mean is alpha/(alpha+beta)
#' # Variance is (alpha*beta)/((alpha+beta)^2*(alpha+beta+1))
#' alpha = 5
#' beta = 9
#' plot(x = seq(0.0001, 1, by = .0001), 
#'     y = dbeta(seq(0.0001, 1, by = .0001), alpha, beta),
#'     type = "l", ylab="Density", xlab="p", bty="n")
#' points(x=alpha/(alpha+beta), y=0, pch=4)
#' segments(x0=alpha/(alpha+beta)-sqrt((alpha*beta)/((alpha+beta)^2*(alpha+beta+1))), 
#'         x1=alpha/(alpha+beta)+sqrt((alpha*beta)/((alpha+beta)^2*(alpha+beta+1))),
#'         y0=0, y1=0)
#'
#' # Use of optim with L-BFGS-B to limit p between 0 and 1 and s > 0
#' 
#' # Mean 1
#' m1=2.3; s1=0.5
#' # Mean 2
#' m2=1.3; s2=0.1
#' # proportion of category 1 - logit transform
#' p=0.5
#' 
#' par <- c(m1=m1, s1=s1, m2=m2, s2=s2, p=p)
#' 
#' result2 <- optim(par, Lnormlnorm, method="L-BFGS-B", val=X, 
#'               lower = c(-Inf, 0, -Inf, 0, 0), 
#'               upper = c(Inf, Inf, Inf, Inf, 1),
#'               hessian=FALSE, control=list(trace=1))
#' 
#' parameters_mcmc <- data.frame(Density=c('dunif', 'dunif', 'dunif', 'dunif', 'dbeta'), 
#'                                         Prior1=c(0, 0.001, 0, 0.001, 5), 
#'                                         Prior2=c(10, 10, 10, 10, 9), 
#'                                         SDProp=c(1, 1, 1, 1, 1), 
#'                                         Min=c(0, 0.001, 0, 0.001, 0), 
#'                                         Max=c(10, 10, 10, 10, 1), 
#'                                         Init=c('m1' = 2.4, 
#'                                                's1' = 0.6, 
#'                                                'm2' = 1.3, 
#'                                                's2' = 0.1, 
#'                                                'p' = 0.5), stringsAsFactors = FALSE, 
#'                                         row.names=c('m1', 's1', 'm2', 's2', 'p'))
#'                                         
#' mcmc_run_pbeta <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, val=X, 
#'                       parameters_name = "par",
#'                       adaptive = TRUE,
#'                       likelihood=Lnormlnorm, n.chains=1, 
#'                       n.adapt=100, thin=1, trace=100)
#' plot(mcmc_run_pbeta, parameters="m1", breaks=seq(from=0, to =10, by=0.1), 
#'      legend=c(x=6, y=2.10))
#' plot(mcmc_run_pbeta, parameters="p", xlim=c(0,1), 
#'      breaks=seq(from=0, to=1, by=0.01), legend=c(x=0.6, y=6))
#' 
#' 
#' }
#' @method plot mcmcComposite
#' @export

plot.mcmcComposite <- function(x, 
                               ... , 
                               chains="all"                                                      , 
                               parameters=1                                                      , 
                               transform=NULL                                                    , 
                               scale.prior=TRUE                                                  , 
                               legend="topright"                                                 , 
                               ylab.posterior="Posterior density"                                ,
                               las = 1                                                           ,
                               bty = "n"                                                         ,
                               show.prior=TRUE                                                   , 
                               show.posterior.density=TRUE                                       ,
                               col.prior = "red"                                                 , 
                               lty.prior = 1                                                     , 
                               lwd.prior = 1                                                     , 
                               what = "Posterior"                                                ,
                               col.posterior = colorRampPalette(c("blue", "grey"), alpha=0.001)  , 
                               lty.posterior = 1                                                 , 
                               lwd.posterior = 1                                                 ,
                               show.yaxis.prior=TRUE                                              ,
                               ylab.prior="Prior density"                                        ) {
  
  # x<-NULL;transform=NULL;chains="all";parameters=1;scale.prior=TRUE;legend=TRUE;ylab.posterior="Posterior density";las=1;show.prior=TRUE;col.prior="red";lty.prior=1;lwd.prior=1;col.posterior="white";lty.posterior=1;lwd.posterior=1;ylab.prior="Prior density"; what="posterior"
  # col.posterior <- colorRampPalette(c("blue", "grey"))(x$parametersMCMC$n.chains)
  # print(col.posterior)
  
  if (all(chains == "all")) chains <- 1:x$parametersMCMC$n.chains
  
  what <- match.arg(tolower(what), choices=c("posterior", "markovchain", "lnl"))
  chains <- chains[chains <= x$parametersMCMC$n.chains]
  
  if (is.null(transform)) {
    transformx <- function(x) {x}
  } else {
    transformx <- transform
  }
  
  pre.mar <- par("mar")
  
  # chains=1; parameters=1; scale.prior=FALSE; legend=TRUE
  
  Parameters_L <- x[["parametersMCMC"]]
  # Parameters avec une majuscule, c'est le tableau pMCMC
  # avec une minuscule, c'est le paramètres à afficher !
  Parameters <- Parameters_L[["parameters"]]
  n.iter <- Parameters_L[["n.iter"]]
  n.chains <- Parameters_L[["n.chains"]]
  n.adapt <- Parameters_L[["n.adapt"]]
  thin <- Parameters_L[["thin"]]
  
  possible <- rownames(Parameters)
  NbTS <- length(possible)
  
  if (parameters[[1]]=="all") {
    parameters <- possible
  }
  
  tpt <- list(...) # tpt <- list()
  
  if (what == "lnl") {
    
    if (is.function(col.posterior)) col.posterior <- col.posterior(length(chains))
    
    L <- modifyList(modifyList(list(xlab="Iterations", 
                                    ylab="Ln L", 
                                    las=las, main="", 
                                    col=col.posterior[1],
                                    lty=lty.posterior[1], 
                                    lwd=lwd.posterior[1]), modifyList(list(x=1:length(x$resultLnL[[1]]), y=x$resultLnL[[1]], 
                                                                           bty=bty, type="l"), tpt)), list(type="n"))
    do.call(plot, L)
    
    col.posterior <- rep(col.posterior, length(chains))[1:length(chains)]
    lty.posterior <- rep(lty.posterior, length(chains))[1:length(chains)]
    lwd.posterior <- rep(lwd.posterior, length(chains))[1:length(chains)]
    
    for (chain in chains) {
      vals <- x[["resultLnL"]][[chain]]
      lines(x=1:length(vals), y=vals, col=col.posterior[chain], lty=lty.posterior[chain], lwd=lwd.posterior[chain])
    }
    
    
  }
  
  nitercorrige <- floor(n.iter/thin)
  if (is.numeric(parameters)) parameters <- possible[parameters]
  
  
  if (what == "posterior") {
    if (show.prior) {
      if (scale.prior) {
        pre.mar.2 <- pre.mar
        if (pre.mar.2[4] < 5.1)  pre.mar.2[4] <- 5.1
        if (pre.mar.2[2] < 5.1)  pre.mar.2[2] <- 5.1
        par(mar=pre.mar.2)
      }
    }
    
    if (is.function(col.posterior)) col.posterior <- col.posterior(1)
    
    for (variable in parameters) {
      
      vals <- NULL
      for (chain in chains)
        if (NbTS == 1) vals <- c(vals, x[["resultMCMC"]][[chain]]) else vals <- c(vals, x[["resultMCMC"]][[chain]][ ,variable])
        # c'est quoi ça ??? 6/10/2012
        # vals <- vals[(length(vals)-nitercorrige):length(vals)]
        
        rg <- range(vals)
        vals <- transformx(vals)
        
        
        if (Parameters[variable, "Density"]=="dunif") {
          # 22/8/2014
          # je suis en uniforme. Il faut que je limite les breaks
          # 9/3/2015 Si l'un est négatif ou nul, je dois faire un shift d'origine
          shift <- as.numeric(Parameters[variable, "Prior1"])
          mx <- as.numeric(Parameters[variable, "Prior2"])-shift
          mn <- as.numeric(Parameters[variable, "Prior1"])-shift
          mxlog <- 10^(floor(log10(mx))-1)
          # C'est quoi ???
          xl <- as.numeric(c(mn, mx)+shift)+c(-mxlog, mxlog)
          br1 <- mn
          br2 <- mx
          interval <- (mx-mn)/20
          decalage <- br1 %% interval
          
          br <- seq(from=decalage+shift, to=mx+shift, by=interval)
        } else {
          shift <- as.numeric(Parameters[variable, "Min"])
          mx <- as.numeric(Parameters[variable, "Max"])-shift
          mn <- as.numeric(Parameters[variable, "Min"])-shift
          mxlog <- 10^(floor(log10(mx))-1)
          # C'est quoi ???
          xl <- as.numeric(c(mn, mx)+shift)+c(-mxlog, mxlog)
          br1 <- mn
          br2 <- mx
          interval <- (mx-mn)/20
          decalage <- br1 %% interval
          
          br <- seq(from=decalage+shift, to=mx+shift, by=interval)
          
        }
        
        # tpt <- list(las=1, xlim=c(0,30), breaks=c(0, 1.00095, 2.0009, 3.00085, 4.0008, 5.00075, 6.0007, 7.00065, 8.0006, 9.00055, 10.0005, 11.00045, 12.0004, 13.00035, 14.0003, 15.00025, 16.0002, 17.00015, 18.0001, 19.00005, 20))
        
        L <- modifyList(list(ylab="", xlab=variable, 
                             las=las, main="", freq=FALSE, 
                             xlim=xl, breaks=br, 
                             col=col.posterior[1], 
                             lty=lty.posterior[1], 
                             lwd=lwd.posterior[1]), modifyList(list(x=vals), tpt)) 
        
        do.call(hist, L) 
        mtext(ylab.posterior, side=2, line=3)
        
        
        
        if (show.prior) {
          
          scl <- ScalePreviousPlot()
          sequence <- seq(from=scl$xlim[1], to=scl$xlim[2], length=200)
          
          p1 <- as.numeric(Parameters[variable, "Prior1"])
          p2 <- as.numeric(Parameters[variable, "Prior2"])
          y <- get(as.character(Parameters[variable, "Density"]))(sequence, p1, p2)
          yl <- c(0, max(y[is.finite(y)]))
          
          sequence <- transformx(sequence)
          
          
          if (Parameters[variable, "Density"] != "dunif") {
            
            
            if (scale.prior) {
              lines(x=sequence, y=(y/max(y[is.finite(y)]))*scl$ylim["end"], 
                    col=col.prior, lty=lty.prior, lwd=lwd.prior)
              if (show.yaxis.prior) {
                axis(side = 4, las=las)
                mtext(text=ylab.prior, side=4, line = 4)
              }
            } else {
              lines(x=sequence, y=y, 
                    col=col.prior, lty=lty.prior, lwd=lwd.prior)
            }
          } else {
            if (scale.prior) {
              # par(new=TRUE)
              # plot(c(scl$xlim["begin"], scl$xlim["end"]), c(0, max(y)), type="n", axes=FALSE, bty="n",
              #      xlab="", ylab="", main="")
              
              segments(x0=scl$xlim[1], x1=transformx(p1), y0=0, y1=0, col=col.prior, lty=lty.prior, lwd=lwd.prior)
              segments(x0=transformx(p1), x1=transformx(p1), y0=0, y1=scl$ylim[2], col=col.prior, lty=lty.prior, lwd=lwd.prior)
              segments(x0=transformx(p1), x1=transformx(p2), y0=scl$ylim[2], y1=scl$ylim[2], col=col.prior, lty=lty.prior, lwd=lwd.prior)
              segments(x0=transformx(p2), x1=transformx(p2), y0=scl$ylim[2], y1=0, col=col.prior, lty=lty.prior, lwd=lwd.prior)
              segments(x0=transformx(p2), x1=scl$xlim[2], y0=0, y1=0, col=col.prior, lty=lty.prior, lwd=lwd.prior)
              if (show.yaxis.prior) {
                axis(side = 4, las=las)
                mtext(text=ylab.prior, side=4, line = 4)
              }
              
            } else {
              segments(x0=scl$xlim[1], x1=transformx(p1), y0=0, y1=0, col=col.prior, lty=lty.prior, lwd=lwd.prior)
              segments(x0=transformx(p1), x1=transformx(p1), y0=0, y1=yl[2], col=col.prior, lty=lty.prior, lwd=lwd.prior)
              segments(x0=transformx(p1), x1=transformx(p2), y0=yl[2], y1=yl[2], col=col.prior, lty=lty.prior, lwd=lwd.prior)
              segments(x0=transformx(p2), x1=transformx(p2), y0=yl[2], y1=0, col=col.prior, lty=lty.prior, lwd=lwd.prior)
              segments(x0=transformx(p2), x1=scl$xlim[2], y0=0, y1=0, col=col.prior, lty=lty.prior, lwd=lwd.prior)
            }
          }
        }
    }
    
    
    if (legend[1] != FALSE) {
      if (length(legend) == 2) {
        if(show.prior) {
          legend(x=legend["x"], y=legend["y"], legend=c("Prior", "Posterior"), lty=c(lty.prior, lty.posterior), 
                 col=c(col.prior, col.posterior), bty="n", pch=c(NA, 0))
        } else {
          legend(x=legend["x"], y=legend["y"], legend=c("Posterior"), lty=c(lty.posterior), 
                 col=c(col.posterior), bty="n", pch=c(0))
        }
      } else {
        if (isTRUE(legend)) legend <- "topright"
        if(show.prior) {
          legend(x=legend, legend=c("Prior", "Posterior"), lty=c(lty.prior, lty.posterior), 
                 col=c(col.prior, col.posterior), bty="n", pch=c(NA, 0))
        } else {
          legend(x=legend, legend=c("Posterior"), lty=c(lty.posterior), 
                 col=c(col.posterior), bty="n", pch=c(0))
        }
      }
    }
  }
  
  if (what == "markovchain") {
    
    if (is.function(col.posterior)) col.posterior <- col.posterior(length(chains))
    
    for (variable in parameters) {
      
      # 22/8/2014
      # je suis en uniforme. Il faut que je limite les breaks
      # 9/3/2015 Si l'un est négatif ou nul, je dois faire un shift d'origine
      shift <- as.numeric(Parameters[variable, "Min"])
      mx <- as.numeric(Parameters[variable, "Max"])-shift
      mn <- as.numeric(Parameters[variable, "Min"])-shift
      mxlog <- 10^(floor(log10(mx))-1)
      # C'est quoi ???
      xl <- as.numeric(c(mn, mx)+shift)+c(-mxlog, mxlog)
      xl <- transformx(xl)
      if (NbTS == 1) vals <- x$resultMCMC[[1]] else vals <- x$resultMCMC[[1]][ ,variable]
      maxx <- length(vals)
      if (show.prior) maxx <- maxx + 0.1*maxx
      vals <- transformx(vals)
      
      L <- modifyList(modifyList(list(xlab="Iterations", 
                                      ylab=variable, 
                                      las=las, main="", 
                                      ylim=xl, 
                                      col=col.posterior[1],
                                      lty=lty.posterior[1], 
                                      lwd=lwd.posterior[1]), modifyList(list(x=1:length(vals), y=rep(0, length(vals)), 
                                                                             xlim=c(1, maxx), bty=bty), tpt)), list(type="n"))
      L <- modifyList(L, tpt)
      
      do.call(plot, L)
      
      col.posterior <- rep(col.posterior, length(chains))[1:length(chains)]
      lty.posterior <- rep(lty.posterior, length(chains))[1:length(chains)]
      lwd.posterior <- rep(lwd.posterior, length(chains))[1:length(chains)]
      for (chain in chains) {
        if (NbTS == 1) vals <- x[["resultMCMC"]][[chain]] else vals <- x[["resultMCMC"]][[chain]][ ,variable]
        vals <- transformx(vals)
        lines(x=1:length(vals), y=vals, col=col.posterior[chain], lty=lty.posterior[chain], lwd=lwd.posterior[chain])
      }
      
      if (show.posterior.density) {
        segments(x0=length(vals), x1=length(vals), y0=ScalePreviousPlot()$ylim["begin"], 
                 y1=ScalePreviousPlot()$ylim["end"], col="black")
        for (chain in chains) {
          if (NbTS == 1) valsd <- x[["resultMCMC"]][[chain]] else valsd <- x[["resultMCMC"]][[chain]][ ,variable]
          valsd <- transformx(valsd)
        y <- density(valsd)
        lines(x=length(vals) + (y$y/max(y$y) * length(vals) *0.1) , y=y$x, 
              col=col.posterior[chain], lty=lty.posterior[chain], lwd=lwd.posterior[chain])
        }
      }
      
      if (show.prior) {
        scl <- ScalePreviousPlot()
        
        # sequence <- seq(from=scl$ylim[1], to=scl$ylim[2], length=200)
        
        sequence <- seq(from=as.numeric(Parameters[variable, "Min"]), to=as.numeric(Parameters[variable, "Max"]), length=200)
        
        p1 <- as.numeric(Parameters[variable, "Prior1"])
        p2 <- as.numeric(Parameters[variable, "Prior2"])
        y <- get(as.character(Parameters[variable, "Density"]))(sequence, p1, p2)
        
        # sequence <- transformx(sequence)
        lines(x=length(vals) + (y/max(y) * length(vals) *0.1) , y=transformx(sequence), 
              col=col.prior, lty=lty.prior, lwd=lwd.prior)
      }
      
    }
    
    if (legend[1] != FALSE) {
      if (length(legend) == 2) {
        if(show.prior) {
          legend(x=legend["x"], y=legend["y"], legend=c("Prior", "Posterior"), lty=c(lty.prior, lty.posterior), 
                 col=c(col.prior, col.posterior), bty="n", pch=c(NA, 0))
        } else {
          legend(x=legend["x"], y=legend["y"], legend=c("Posterior"), lty=c(lty.posterior), 
                 col=c(col.posterior), bty="n", pch=c(0))
        }
      } else {
        if (isTRUE(legend)) legend <- "topright"
        if(show.prior) {
          legend(x=legend, legend=c("Prior", "Posterior"), lty=c(lty.prior, lty.posterior), 
                 col=c(col.prior, col.posterior), bty="n", pch=c(NA, 0))
        } else {
          legend(x=legend, legend=c("Posterior"), lty=c(lty.posterior), 
                 col=c(col.posterior), bty="n", pch=c(0))
        }
      }
    }
    
  }
  
  par(mar=pre.mar)
}

#' universalmclapply runs the function FUN on X using parallel computing
#' @title Run the function FUN on X using parallel computing
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The results of the function FUN applied to X
#' @param X A vector (atomic or list) or an expressions vector. Other objects (including classed objects) will be coerced by as.list.
#' @param FUN The function to be applied to each element of X
#' @param ... Optional arguments to FUN
#' @param mc.preschedule if set to TRUE then the computation is first divided to (at most) as many jobs are there are cores and then the jobs are started, each job possibly covering more than one value. If set to FALSE then one job is forked for each value of X. The former is better for short computations or large number of values in X, the latter is better for jobs that have high variance of completion time and not too many values of X compared to mc.cores.
#' @param mc.cores The number of cores to use, i.e. at most how many child processes will be run simultaneously. 
#' @param clusterExport List of clusterExport parameters as list
#' @param clusterEvalQ List of clusterEvalQ parameters as list
#' @param forking If TRUE will use forking
#' @param progressbar If pbapply package is installed, show a progressbar
#' @description Return the results of the function FUN applied to X. It uses forking in unix system and not in windows system.\cr
#' By default, it will send all the content of environment.
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' x <- 1:1000
#' funx <- function(y) {
#'   mint <- rep(NA, length(y))
#'   for (i in seq_along(y)) {
#'     k <- rnorm(runif(n = 1, 50, 50), mean=10, sd=2)
#'     mint[i] <- mean(k)
#'   }
#'   mint
#' }
#' # Note that parallel computing is not always the best solution !
#' (tp <- system.time({
#'    m <- lapply(X=x, FUN=funx)
#' }))
#' (tp <- system.time({
#'    m <- universalmclapply(X=x, FUN=funx, forking=FALSE)
#' }))
#' (tp <- system.time({
#'    m <- universalmclapply(X=x, FUN=funx, forking=TRUE)
#' }))
#' 
#' ### An example using clusterExport
#' # Here no error is generated because environment was exported
#' # However forking is not possible in windows and non parallel code is ran
#' pp <- runif(100)
#' x <- 1:100
#' funx1 <- function(y) {pp[y]*10}
#' u <- universalmclapply(x, FUN=funx1, forking=TRUE)
#' 
#' # Here an error is generated because environment was not exported when parLapplyLB is used
#' pp <- runif(100)
#' x <- 1:100
#' u <- universalmclapply(x, FUN=funx1, forking=FALSE)
#' u <- universalmclapply(x, FUN=funx1, forking=FALSE, 
#'                        clusterExport=list())
#' 
#' # here no error is generated because the variable pp is exported
#' pp <- runif(100)
#' x <- 1:100
#' u <- universalmclapply(x, FUN=funx1, forking=FALSE, 
#'                        clusterExport=list(varlist=c("pp"), envir=environment()))
#'
#' # here no error is generated because all the environment is exported
#' pp <- runif(100)
#' x <- 1:100
#' u <- universalmclapply(x, FUN=funx1, forking=FALSE, 
#'                        clusterExport=list(varlist=c(ls()), envir=environment()))
#' 
#' ### An example using clusterEvalQ
#' asc("a") # asc() is a function from packages HelpersMG
#' funx2 <- function(y) {asc("a")*10}
#' # In unix, the loaded packages are visible from all cores
#' x <- 1:100
#' u <- universalmclapply(x, FUN=funx2, forking=TRUE)
#' # In windows, the loaded packages are not visible from all cores
#' x <- 1:100
#' u <- universalmclapply(x, FUN=funx2, forking=FALSE)
#' # In windows, the loaded packages are not visible from all cores
#' x <- 1:100
#' u <- universalmclapply(x, FUN=funx2, forking=FALSE, 
#' clusterEvalQ=list(expr=expression(library(HelpersMG)))
#' )
#' 
#' ### If package pbapply is available, progress bar can be shown
#' m <- universalmclapply(X=x, FUN=funx, forking=FALSE, progressbar=TRUE)
#' m <- universalmclapply(X=x, FUN=funx, forking=TRUE, progressbar=TRUE)
#' 
#' ## You can manage the number of cores used using:
#' options(mc.cores=1)
#' }
#' @export


#

universalmclapply <- function(X, FUN, ..., 
                              mc.cores=getOption("mc.cores", parallel::detectCores()), 
                              mc.preschedule = TRUE, 
                              clusterExport=list(), 
                              clusterEvalQ=list(), 
                              forking=ifelse(.Platform$OS.type=="windows", 
                                             FALSE, TRUE), 
                              progressbar=FALSE) {
  
  if (FALSE) {
    mc.cores=getOption("mc.cores", parallel::detectCores())
    mc.preschedule = TRUE
    clusterExport=list(varlist=c(ls()), envir=environment())
    clusterEvalQ=list()
    forking=ifelse(.Platform$OS.type=="windows", FALSE, TRUE)
    progressbar=FALSE
  }
  m <- NULL
  
  if (forking) {
    if (progressbar & ("pbapply" %in% rownames(installed.packages()))) {
      m <- do.call(pbapply::pblapply, modifyList(list(...), list(X=X, FUN=FUN, cl=mc.cores)))
    } else {
      m <- do.call(parallel::mclapply, modifyList(list(...), list(X=X, FUN=FUN, 
                                                                  mc.cores=mc.cores, 
                                                                  mc.preschedule = mc.preschedule)))
    }
  } else {
    
    cl <- parallel::makeCluster(mc.cores)
    if (!identical(list(), clusterExport)) {
      do.call(parallel::clusterExport, args = modifyList(list(cl=cl), clusterExport))
    }
    if (!identical(list(), clusterEvalQ)) {
      do.call(parallel::clusterEvalQ, args=modifyList(list(cl=cl), clusterEvalQ))
    }
    if (progressbar & ("pbapply" %in% rownames(installed.packages()))) {
      tt <- try(expr = {
        m <- do.call(pbapply::pblapply, args=modifyList(list(...), list(X=X, FUN=FUN, cl=cl)))
      }, silent = FALSE)
    } else {
     
      tt <- try(expr = {
        m <- do.call(parallel::parLapplyLB, args=modifyList(list(...), list(X=X, fun=FUN, cl=cl)))
      }, silent = FALSE)
    }
    parallel::stopCluster(cl)
  }
  return(m)
}

#' ELPDweight calculates the probability that each model is best for prediction
#' @title Return the probability of models compared with loo being the best for prediction
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A vector with the probabilities
#' @param loos A named list of loos
#' @param nreplicates Number of replicates.
#' @description Calculate the probability of models compared with loo being the best for prediction. 
#' It uses simulation using the pointwise LOO log-predictive densities. This preserves the correlation 
#' between models, because all models are evaluated on the same observations. This is the approach 
#' recommended by Aki Vehtari et al. (2017) when uncertainty in model comparison matters.
#' @references Vehtari, A., Gelman, A., & Gabry, J. (2017). Practical Bayesian model evaluation using leave-one-out cross-validation and WAIC. Statistics and Computing, 27(5), 1413-1432. https://doi.org/10.1007/s11222-016-9696-4
#' @family AIC
#' @examples
#' \dontrun{
#' model1_loo <- loo::loo(model1)
#' model2_loo <- loo::loo(model2)
#' ELPDweight(list(model1=model1_loo, model2=model2_loo))
#' }
#' @export

ELPDweight <- function (loos = stop("A named list of loo data must be provided."), 
                                         nreplicates = 10000) 
{
  elpd_mat <- sapply(loos, function(x) x$pointwise[,"elpd_loo"])
  
  N <- nrow(elpd_mat)
  K <- ncol(elpd_mat)
  
  best <- numeric(nreplicates)
  
  for(s in 1:nreplicates){
    
    # Bayesian bootstrap weights
    w <- rexp(N)
    w <- w/sum(w)
    
    # weighted ELPD
    elpd_sim <- colSums(elpd_mat * w)
    
    best[s] <- which.max(elpd_sim)
  }
  
  prob <- table(best)/nreplicates
  prob_ec <- rep(0, K)
  names(prob_ec) <- as.character(1:K)
  prob_ec[names(prob)] <- as.numeric(prob)
  if (!any(is.null(names(loos)))) names(prob_ec) <- names(loos)
  return(prob_ec)
}

#' format_ncdf is used extract information from ncdf file
#' @title Return an array with ncdf data
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with two element: data is an array and time is the POSIX.lt time
#' @param ncdf An object read from package ncdf4 or a file name of ncdf file
#' @param label.latitude Label of latitude
#' @param label.longitude	Label of longitude
#' @param label.time	Label of time
#' @param varid	Name of variable to extract
#' @param longitude1 Longitude for first corner
#' @param latitude1 latitude for first corner
#' @param longitude2 Longitude for second corner
#' @param latitude2 latitude for second corner
#' @param package If ncdf is a file, give the package to use to open the file
#' @param bathy If TRUE, return a bathy object
#' @description Return a list with two elements: data is an array and time is the POSIX.lt time.\cr
#' Or if label.time is NULL or if bathy is TRUE, a bathy object.\cr
#' If varid is NULL, it shows the available variable and dimensions of the file.\cr
#' Bathymetry data can be download here: \cr
#' https://www.gebco.net/data_and_products/gridded_bathymetry_data/#global
#' @family ncdf
#' @examples
#' \dontrun{
#' url <- "https://downloads.psl.noaa.gov/Datasets/noaa.oisst.v2.highres/"
#' url <- paste0(url, "sst.day.mean.2012.v2.nc")
#' dest <- paste(Sys.getenv("HOME"), "/sst.day.mean.2012.v2.nc", sep="")
#' download.file(url, dest)
#' format_ncdf(dest)
#' }
#' @export


format_ncdf <- function(ncdf                         , 
                        label.latitude="latitude"    , 
                        label.longitude="longitude"  ,
                        label.time="time"            , 
                        varid=NULL                   , 
                        longitude1=NA                , 
                        latitude1=NA                 ,  
                        longitude2=NA                , 
                        latitude2=NA                 , 
                        package = "ncdf4"            , 
                        bathy = TRUE                 ) {
  
  package <- match.arg(arg=package, choices=c("ncdf4", "RNetCDF"))
  
  if (
    (
      (
        inherits(ncdf, "ncdf4") 
      ) | 
      ( 
        (
          inherits(ncdf, "character")
        ) & (
          package == "ncdf4"
        ) 
      ) 
    ) & 
    (!requireNamespace("ncdf4", quietly = TRUE) ) ) {
    
    stop("ncdf4 package is necessary for this function")
  }
  if (
    (
      (
        inherits(ncdf, "NetCDF") 
      ) | 
      ( 
        (
          inherits(ncdf, "character")
        ) & (
          package == "RNetCDF"
        ) 
      ) 
    ) & (!requireNamespace("RNetCDF", quietly = TRUE) ) ) {
    
    stop("RNetCDF package is necessary for this function")
  }
  
  # library("ncdf4")
  if (inherits(ncdf, "character")) {
    if (package == "ncdf4") {
      ncdf <- getFromNamespace("nc_open", ns="ncdf4")(ncdf)
    } else {
      ncdf <- getFromNamespace("open.nc", ns="RNetCDF")(ncdf)
    }
  }
  
  if (inherits(ncdf, "ncdf4")) {
    vr <- names(ncdf$var)
    dm <- names(ncdf$dim)
  } else {
    # Je suppose que je suis en RNetCDF
    info <- getFromNamespace("file.inq.nc", ns="RNetCDF")(ncdf)
    vr <- NULL
    for (i in (info$ndims):(info$nvars-1)) {
      vr <- c(vr, getFromNamespace("var.inq.nc", ns="RNetCDF")(ncdf, variable = i)$name)
    }
    dm <- NULL
    for (i in 0:(info$ndims-1)) {
      dm <- c(dm, getFromNamespace("dim.inq.nc", ns="RNetCDF")(ncdf, dimension = i)$name)
    }
  }
  
  message(paste0("The available variables are: ", paste(vr, collapse = ", ")))
  message(paste0("The available dimensions are: ", paste(dm, collapse = ", ")))
  
  if (is.null(varid)) {
    return(invisible())
  }
  
  if (!is.null(label.latitude)) 
    if (all(dm != label.latitude)) stop(paste0("Check the latitude label: ", paste(dm, collapse = ", ")))
  if (!is.null(label.longitude)) 
    if (all(dm != label.longitude)) stop(paste0("Check the longitude label: ", paste(dm, collapse = ", ")))
  if (!is.null(label.time)) 
    if (all(dm != label.time)) stop(paste0("Check the time label: ", paste(dm, collapse = ", ")))
  
  if (all(c(!is.na(longitude1), !is.na(latitude1), 
            !is.na(longitude1), !is.na(latitude1)))) {
    ind1 <- ind_long_lat(ncdf=ncdf, 
                         long = longitude1, 
                         lat = latitude1, 
                         label.longitude = label.longitude, 
                         label.latitude = label.latitude)
    ind2 <- ind_long_lat(ncdf=ncdf, 
                         long = longitude2, 
                         lat = latitude2, 
                         label.longitude = label.longitude, 
                         label.latitude = label.latitude)
    start_lg <- min(ind1["indice.long"], ind2["indice.long"])
    count_lg <- abs(ind2["indice.long"] - ind1["indice.long"]) + 1
    start_lt <- min(ind1["indice.lat"], ind2["indice.lat"])
    count_lt <- abs(ind2["indice.lat"] - ind1["indice.lat"]) + 1
  } else {
    start_lg <- 1
    start_lt <- 1  
    if (inherits(ncdf, "ncdf4")) {
      count_lg <- ncdf$dim[[label.longitude]]$len
      count_lt <- ncdf$dim[[label.latitude]]$len
    } else {
      count_lg <- getFromNamespace("dim.inq.nc", ns="RNetCDF")(ncdf, dimension  = label.longitude)$length
      count_lt <- getFromNamespace("dim.inq.nc", ns="RNetCDF")(ncdf, dimension  = label.latitude)$length
    }
  }
  
  
  if ((!is.null(label.time)) | (!bathy)) {
    
    
    if (inherits(ncdf, "ncdf4")) {
      
      start <- NULL
      count <- NULL
      for (dm_ec in dm) {
        if (dm_ec == label.longitude) {
          start <- c(start, start_lg)
          count <- c(count, count_lg)
        } else if (dm_ec == label.latitude) {
          start <- c(start, start_lt)
          count <- c(count, count_lt)
        } else if (dm_ec == label.time) {
          start <- c(start, 1)
          count <- c(count, ncdf$dim[[label.time]]$len)
        } else {
          start <- c(start, 1)
          count <- c(count, 1)
        }
      }
      
      
      carte3D <- getFromNamespace("ncvar_get", ns="ncdf4")(ncdf, varid=varid,
                                                           start=start,
                                                           count=count)
      
      c3d <- array(data=carte3D[], dim=count[c(which(dm == label.longitude), 
                                               which(dm == label.latitude), 
                                               which(dm == label.time))])
      
      
      if (isTRUE(requireNamespace("RNetCDF", quietly = TRUE)) ) {
        date.char <- getFromNamespace("utcal.nc", ns="RNetCDF")(ncdf$dim[[label.time]]$units, 
                                                                ncdf$dim[[label.time]]$vals, 
                                                                type="s")
        date.POSIXlt <- strptime(date.char, format="%Y-%m-%d %H:%M:%S", tz="UTC")
      } else {
        warning("The date and time information required the package RNetCDF being installed")
        date.POSIXlt <- NULL
        date.char <- ncdf$dim[[label.time]]$vals
      }
      dnames <- list(ncdf$dim[[label.longitude]]$vals[start_lg:(start_lg+count_lg-1)], 
                     ncdf$dim[[label.latitude]]$vals[start_lt:(start_lt+count_lt-1)], 
                     date.char)
      
      names(dnames) <- c(label.longitude, label.latitude, label.time)
      
      if (ncdf$dim[[label.time]]$len > 1) {
        if (date.POSIXlt[1] > date.POSIXlt[2]) {
          # Je dois inverser les noms
          c3d <- c3d[, , dim(c3d)[3]:1, drop = FALSE]
          dnames[[3]] <- rev(dnames[[3]])
        }
      }
      
      if (ncdf$dim[[label.latitude]]$len > 1) {
        if (as.numeric(ncdf$dim[[label.latitude]]$vals[1]) > ncdf$dim[[label.latitude]]$vals[2]) {
          # Je dois inverser les noms
          c3d <- c3d[, dim(c3d)[2]:1, , drop = FALSE]
          dnames[[2]] <- rev(dnames[[2]])
        }
      }
      
      if (ncdf$dim[[label.longitude]]$len > 1) {
        if (as.numeric(ncdf$dim[[label.longitude]]$vals[1]) > ncdf$dim[[label.longitude]]$vals[2]) {
          # Je dois inverser les noms
          c3d <- c3d[dim(c3d)[1]:1, , , drop = FALSE]
          dnames[[1]] <- rev(dnames[[1]])
        }
      }
      
    } else {
      
      start <- NULL
      count <- NULL
      for (dm_ec in dm) {
        if (dm_ec == label.longitude) {
          start <- c(start, start_lg)
          count <- c(count, count_lg)
        } else if (dm_ec == label.latitude) {
          start <- c(start, start_lt)
          count <- c(count, count_lt)
        } else if (dm_ec == label.time) {
          start <- c(start, 1)
          count <- c(count, getFromNamespace("dim.inq.nc", ns="RNetCDF")(ncdf, dimension  = label.time)$length)
        } else {
          start <- c(start, 1)
          count <- c(count, 1)
        }
      }
      
      
      
      carte3D <- getFromNamespace("var.get.nc", ns="RNetCDF")(ncfile=ncdf, variable=varid, 
                                                              start=start, 
                                                              count=count)
      
      c3d <- array(data=carte3D[], dim=count[c(which(dm == label.longitude), 
                                               which(dm == label.latitude), 
                                               which(dm == label.time))])
      
      ulat <- getFromNamespace("var.get.nc", ns="RNetCDF")(ncfile=ncdf, variable = label.latitude, start = start_lt, count = count_lt) 
      ulon <- getFromNamespace("var.get.nc", ns="RNetCDF")(ncfile=ncdf, variable = label.longitude, start = start_lg, count = count_lg) 
      utime <- getFromNamespace("var.get.nc", ns="RNetCDF")(ncfile=ncdf, variable = label.time, start = 1, count = getFromNamespace("dim.inq.nc", ns="RNetCDF")(ncdf, dimension  = label.time)$length)
      
      date.char <- getFromNamespace("utcal.nc", ns="RNetCDF")(unitstring=getFromNamespace("att.get.nc", ns="RNetCDF")(ncdf, variable=label.time, attribute="units"), 
                                                              utime, type="s")
      date.POSIXlt <- strptime(date.char, format="%Y-%m-%d %H:%M:%S", tz="UTC")
      
      dnames <- list(ulon, 
                     ulat, 
                     date.char)
      names(dnames) <- c(label.longitude, label.latitude, label.time)
    }
    
    
    
    dimnames(c3d) <- dnames
    
    return(list(data=c3d, time=date.POSIXlt))
    
  } else {
    
    
    
    if (inherits(ncdf, "ncdf4")) {
      
      carte2D <- getFromNamespace("ncvar_get", ns="ncdf4")(ncdf, varid=varid, 
                                                           start=c(start_lg, start_lt), 
                                                           count=c(count_lg, 
                                                                   count_lt))
      c2d <- array(data=carte2D[], dim=c(count_lg, 
                                         count_lt))
      
      dnames <- list(ncdf$dim[[label.longitude]]$vals[start_lg:(start_lg+count_lg-1)], 
                     ncdf$dim[[label.latitude]]$vals[start_lt:(start_lt+count_lt-1)] )
      names(dnames) <- names(ncdf$dim)
      
      if (ncdf$dim[[label.latitude]]$len > 1) {
        if (as.numeric(ncdf$dim[[label.latitude]]$vals[1]) > ncdf$dim[[label.latitude]]$vals[2]) {
          # Je dois inverser les noms
          c2d <- c2d[, dim(c2d)[2]:1, drop = FALSE]
          dnames[[2]] <- rev(dnames[[2]])
        }
      }
      
      if (ncdf$dim[[label.longitude]]$len > 1) {
        if (as.numeric(ncdf$dim[[label.longitude]]$vals[1]) > ncdf$dim[[label.longitude]]$vals[2]) {
          # Je dois inverser les noms
          c2d <- c2d[dim(c2d)[1]:1, , drop = FALSE]
          dnames[[1]] <- rev(dnames[[1]])
        }
      }
      
    } else {
      
      carte2D <- getFromNamespace("var.get.nc", ns="RNetCDF")(ncfile=ncdf, variable=varid, 
                                                              start=c(start_lg, start_lt), 
                                                              count=c(count_lg, 
                                                                      count_lt))
      
      c2d <- array(data=carte2D[], dim=c(count_lg, 
                                         count_lt))
      
      ulon <- getFromNamespace("var.get.nc", ns="RNetCDF")(ncfile=ncdf, variable = label.longitude, start = start_lg, count = count_lg)
      ulat <- getFromNamespace("var.get.nc", ns="RNetCDF")(ncfile=ncdf, variable = label.latitude, start = start_lt, count = count_lt)
      
      dnames <- list(ulon, ulat)
      names(dnames) <- c(label.longitude, label.latitude)
    }
    
    dimnames(c2d) <- dnames
    
    if (bathy) c2d <- addS3Class(c2d, "bathy")
    
    return(c2d)
  }
}

#' DIx returns an index of quantitative asymmetry and complexity
#' @title Return an index of quantitative asymmetry and complexity named Developmental Instability Index (DIx)
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A numeric value
#' @param l1 Set of measures at one side of an organism
#' @param l2 Set of measures at the other side of an organism
#' @param details If TRUE, will show the details of computing
#' @param version Can be 1 or 2; see description
#' @description Return an index of quantitative asymmetry and complexity.\cr
#' Higher is the value, higher is the complexity (number of objects) and diversity (difference between them).\cr
#' The indice is based on the product of the average angular distance of 
#' Edwards (1971) for all permutations of measures for both sides with the 
#' geometric mean of the inverse of Shannon entropy H for both sides.
#' Let p1 and p2 two vectors of relative measures of objects with sum(p1) = 1 and sum(p2)=1 
#' and n1 being the number of objects in p1 and n2 being the number of objects in p2.\cr
#' Edwards distance for all permutations of p1 and p2 objects are computed and the average value E is calculated.\cr
#' The maximun possible Shannon index for identical n1 is max1 = sum((1/n1) * log(1/n1)).\cr
#' Shannon index is v1 = sum(p1 * log(p1)).\cr
#' If version == 2, the complementary of Shannon index for these n1 objects is used: c1 = 2 * max1 - v1\cr
#' If version == 1, the Shannon index is used directly.\cr
#' The geometry mean between both sides defined the measure of diversity within each side: S=sqrt(c1 * c2)\cr
#' The Developmental Instability Index is then S * E
#' @references 
#' Edwards, A.W.F., 1971. Distances between populations on the basis of gene frequencies. Biometrics 27, 873–881.\cr
#' Shannon C.E. 1948 A mathematical theory of communication. Bell System Technical Journal 27(3), 379-423.\cr
#' @examples
#' \dontrun{
#' l1 <- c(0.1, 0.1, 0.05, 0.2, 0.3, 0.25)
#' l2 <- c(0.2, 0.3, 0.5)
#' DIx(l1, l2)
#' 
#' l1 <- c(0.1, 0.1, 0.05, 0.2, 0.3, 0.25)
#' l2 <- c(0.1, 0.1, 0.05, 0.2, 0.3, 0.25)
#' DIx(l1, l2)
#' 
#' l1 <- c(0.2, 0.3, 0.5)
#' l2 <- c(0.2, 0.3, 0.5)
#' DIx(l1, l2)
#' 
#' l1 <- c(0.2, 0.2, 0.2, 0.2, 0.2)
#' l2 <- c(0.2, 0.3, 0.5)
#' DIx(l1, l2)
#' 
#' l1 <- c(0.2, 0.2, 0.2, 0.2, 0.2)
#' l2 <- c(0.3333, 0.3333, 0.3333)
#' DIx(l1, l2)
#' 
#' l1 <- c(0.2, 0.2, 0.2, 0.2, 0.2)
#' l2 <- c(0.2, 0.2, 0.2, 0.2, 0.2)
#' DIx(l1, l2)
#' 
#' l1 <- c(0.3333, 0.3333, 0.3333)
#' l2 <- c(0.3333, 0.3333, 0.3333)
#' DIx(l1, l2)
#' }
#' @export

DIx <- function(l1, l2, details=FALSE, version = 1) {
  l1 <- c(l1, rep(0, max(0, length(l2)-length(l1))))
  l2 <- c(l2, rep(0, max(0, length(l1)-length(l2))))
  
  l1 <- l1/sum(l1)
  l2 <- l2/sum(l2)
  
  pp <- getFromNamespace(".permutations", ns="HelpersMG")(v=l1, r=length(l1), n=length(l1), 
                                                          set=FALSE)
  p <- (mean(apply(pp, 1, function(x) {sqrt(1-(sum(x*l2)))})))
  vn1 <- sum(l1 != 0)
  vn2 <- sum(l2 != 0)
  if (version == 2) {
    vmax1 <- -sum(rep(1/vn1, vn1) * log(rep(1/vn1, vn1)))
    vmax2 <- -sum(rep(1/vn2, vn2) * log(rep(1/vn2, vn2)))
    
    p1 <- 2*vmax1+sum(l1[l1 != 0] * log(l1[l1 != 0]))
    p2 <- 2*vmax2+sum(l2[l2 != 0] * log(l2[l2 != 0]))
  } else {
    p1 <- sum(l1[l1 != 0] * log(l1[l1 != 0]))
    p2 <- sum(l2[l2 != 0] * log(l2[l2 != 0]))
  }
  if (details) {
    return(c(Average.Edwards=p, Geometric.mean.Shannon=sqrt(p1*p2), DIx=p*sqrt(p1*p2)))
  } else {
    return(p*sqrt(p1*p2))
  }
}

#' index.periodic estimate indices in periodic timeseries based on anchored minimum and maximum
#' @title Estimate indices in periodic timeseries based on anchored minimum and maximum
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A data.frame with a column time and a column index
#' @param minmax A data.frame returned by minmax.periodic
#' @param time	The time at which produced the estimate
#' @param replicates Number of replicates to estimate SD
#' @param progressbar Does a progression bar must be shown
#' @family Periodic patterns of indices
#' @description Estimate indices in periodic timeseries based on anchored minimum and maximum.\cr
#' The data.frame minmax can be generated manually. It should have three columns (time, index, SD), 
#' with all the successive minimum and maximum indices.\cr
#' It can be used with sun.info() to get the time of minimum and maximum air temperature or
#' with getTide() to reconstruct the sea level.
#' @examples
#' \dontrun{
#' # Generate a timeserie of time
#' time.obs <- NULL
#' for (i in 0:9) time.obs <- c(time.obs, c(0, 6, 12, 18)+i*24)
#' # For these time, generate a timeseries of temperatures
#' temp.obs <- rep(NA, length(time.obs))
#' temp.obs[3+(0:9)*4] <- rnorm(10, 25, 3)
#' temp.obs[1+(0:9)*4] <- rnorm(10, 10, 3)
#' for (i in 1:(length(time.obs)-1)) 
#'   if (is.na(temp.obs[i])) 
#'   temp.obs[i] <- mean(c(temp.obs[i-1], temp.obs[i+1]))
#'   if (is.na(temp.obs[length(time.obs)])) 
#'   temp.obs[length(time.obs)] <- temp.obs[length(time.obs)-1]/2
#' observed <- data.frame(time=time.obs, temperature=temp.obs)
#' # Search for the minimum and maximum values
#' r <- minmax.periodic(time.minmax.daily=c(Min=2, Max=15), 
#' observed=observed, period=24, colname.index="temperature")
#' 
#' # Estimate all the temperatures for these values
#' t <- index.periodic(minmax=r)
#' 
#' plot_errbar(x=t[,"time"], y=t[,"index"],
#' errbar.y=ifelse(is.na(t[,"sd"]), 0, 2*t[,"sd"]),
#' type="l", las=1, bty="n", errbar.y.polygon = TRUE, 
#' xlab="hours", ylab="Temperatures", ylim=c(0, 35), 
#' errbar.y.polygon.list = list(col="grey"))
#' 
#' plot_add(x=t[,"time"], y=t[,"index"], type="l")
#' 
#' plot_add(observed$time, observed$temperature, pch=19, cex=0.5)
#' }
#' @export

index.periodic <- function(minmax, time=NULL, 
                                 replicates=100, progressbar=FALSE) {
  if (!inherits(minmax, "data.frame")) {
    warning("minmax parameter must be a data.frame with at least the three columns: 'time', 'index' and 'SD'")
    return()
  }
  if (sum(match(colnames(minmax), c("index", "time", "SD")), na.rm=TRUE) !=6) {
    warning("minmax parameter must be a data.frame with at least the three columns: 'time', 'index' and 'SD'")
    return()
  }
  if (is.null(time)) {
    time <- seq(from=minmax$time[1], to=utils::tail(minmax$time, n=1), by=1)
  }
  if (progressbar) pb<-txtProgressBar(min=1, max=length(time), style=3)
  
  dt <- data.frame(time=time, index=rep(NA, length(time)), sd=rep(NA, length(time)))
  for (i in 1:length(time)) {
    
    if (progressbar) setTxtProgressBar(pb, i)
    
    
   g <- NULL
    tec <- time[i]
    dif <- minmax[,"time"]-tec
    change.sign <- which(sign(dif[1:(length(dif)-1)])*sign(dif[2:length(dif)])==-1 | 
    	sign(dif[1:(length(dif)-1)])*sign(dif[2:length(dif)])==0)[1]
    
    
    if (!identical(change.sign, integer(0)) & !is.na(change.sign)) {
  x0 <- minmax[change.sign, "time"]
  x1 <- minmax[change.sign+1, 'time']
  y0 <- minmax[change.sign, 'index']
  y1 <- minmax[change.sign+1, 'index']
  a <- pi/(x1-x0)
  b <- -x0*a
  p <- (y0-y1)/2
  q <- y0-p

  x <- tec
  y <- cos(x*a+b)*p+q
  dt[i, "index"] <- y
  
  
    y0 <- rnorm(replicates, mean=minmax[change.sign, 'index'], 
                sd=ifelse(is.na(minmax[change.sign, 'SD']), 0, minmax[change.sign, 'SD']))
  y1 <- rnorm(replicates, mean=minmax[change.sign+1, 'index'], 
              sd=ifelse(is.na(minmax[change.sign+1, 'SD']), 0, minmax[change.sign+1, 'SD']))

  for (j in 1:replicates) {
  
  a <- pi/(x1-x0)
  b <- -x0*a
  p <- (y0[j]-y1[j])/2
  q <- y0[j]-p

  x <- tec
  y <- cos(x*a+b)*p+q

	g <- c(g, y)
	}
	
	dt[i, "sd"] <- sd(g)

    
    }
  }
  return(dt)
  
}

#' symbol.Male plot a male symbol in the plotting region
#' @title Plot a male symbol in the plotting region
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @param centerx The x position of the center of the circle
#' @param centery The y position of the center of the circle
#' @param rayonx The size of the rayon in the scale of the x axis
#' @param lwd The width of the line of the symbol
#' @param col The color of the symbol
#' @description Plot a male symbol in the plotting region.
#' @family Symbol
#' @examples
#' \dontrun{
#' plot(x=1:2, y=c(10,20), type="n", bty="n", xlab="", ylab="")
#' 
#' rayonx <- 0.01
#' centerx <- 1.2
#' centery <- 15
#' 
#' symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx)
#' symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx)
#' 
#' rayonx <- 0.03
#' centerx <- 1.2
#' centery <- 18
#' 
#' symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx, lwd=3)
#' symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx, lwd=3, col="red")
#' 
#' rayonx <- 0.05
#' centerx <- 1.4
#' centery <- 13
#' 
#' symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx, lwd=4, col="blue")
#' symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx, lwd=4, col="red")
#' }
#' @export


symbol.Male <- function(centerx, centery, rayonx, lwd=2, col="black") {
xr <- ScalePreviousPlot()$xlim["range"]
yr <- ScalePreviousPlot()$ylim["range"]
ratio <- par("pin")[1]/par("pin")[2]
rayony <- rayonx*(yr/xr)*ratio

angle <- seq(from=0, to=2*pi, length.out = 20)
x <- centerx+rayonx*cos(angle)
y <- centery+rayony*sin(angle)
segments(x0=x, y0=y, x1=x[c(20, 1:19)], y1=y[c(20, 1:19)], col=col, lwd=lwd)

x0 = centerx+rayonx*sqrt(2)/2
y0 = centery+rayony*sqrt(2)/2
x1 = centerx+3*rayonx*sqrt(2)/2
y1 = centery+3*rayony*sqrt(2)/2
segments(x0=x0, y0=y0, x1=x1, y1=y1, col=col, lwd=lwd)

x11 <- centerx+2*rayonx*cos(2*pi*60/360)
y11 <- centery+2*rayony*sin(2*pi*60/360)

x12 <- centerx+2.5*rayonx*cos(2*pi*45/360)
y12 <- centery+2.5*rayony*sin(2*pi*45/360)

x13 <- centerx+2*rayonx*cos(2*pi*30/360)
y13 <- centery+2*rayony*sin(2*pi*30/360)


x <- c(x1, x11, x12, x13, x1)
y <- c(y1, y11, y12, y13, y1)
polygon(x, y, col=col, lwd=lwd, border=col)
}

#' rnbinom_new returns random numbers for the negative binomial distribution
#' @title Random numbers for the negative binomial distribution. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Random numbers for the negative binomial distribution
#' @param n number of observations.
#' @param size target for number of successful trials, or dispersion parameter (the shape parameter of the gamma mixing distribution). Must be strictly positive, need not be integer.
#' @param prob probability of success in each trial. 0 < prob <= 1.
#' @param mu alternative parametrization via mean.
#' @param sd alternative parametrization via standard deviation.
#' @param var alternative parametrization via variance.
#' @description See \code{rnbinom}.
#' @family Distributions
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' set.seed(1)
#' x <- rnbinom_new(n=1000, prob=6.25/(5+6.25), size=6.25)
#' mean(x)
#' sd(x)
#' set.seed(1)
#' x <- rnbinom_new(n=1000, mu=5, sd=3)
#' mean(x)
#' sd(x)
#' set.seed(1)
#' x <- rnbinom_new(n=1000, mu=5, var=3^2)
#' mean(x)
#' sd(x)
#' set.seed(1)
#' x <- rnbinom_new(n=1000, mu=5, size=6.25)
#' mean(x)
#' sd(x)
#' set.seed(1)
#' x <- rnbinom_new(n=1000, size=6.25, var=3^2)
#' mean(x)
#' sd(x)
#' set.seed(1)
#' x <- rnbinom_new(n=1000, prob=6.25/(5+6.25), var=3^2)
#' mean(x)
#' sd(x)
#' # Example of wrong parametrization
#' set.seed(1)
#' x <- rnbinom_new(n=1000, sd=3, var=3^2)
#' set.seed(1)
#' x <- rnbinom_new(n=1000, mu=10, var=3^2)
#' }
#' @export

rnbinom_new <- function(n, size=NULL, prob=NULL, mu=NULL, sd=NULL, var=NULL) {
  
  # Les combinatoires possibles sont
  # size=NULL, prob=NULL # OK
  # size=NULL, mu=NULL # OK
  # size=NULL, sd=NULL # ok
  # size=NULL, var=NULL ¨# ok
  # prob=NULL, mu=NULL # OK
  # prob=NULL, sd=NULL # ok
  # prob=NULL, var=NULL # ok
  # mu=NULL, sd=NULL # OK
  # mu=NULL, var=NULL # OK
  # sd=NULL, var=NULL # Impossible
  
  if (!is.null(sd) & !is.null(var)) if (var != sd^2) {
    warning("sd and var are incompatible.")
    return(rep(NA, n))
  }
  # # prob = size/(size+mu)
  # if (is.null(size) & !is.null(prob) & !is.null(mu)) size <- (prob * mu) / (1- prob)
  if (!is.null(sd)) var <- sd^2
  # prob = size/(size+mu)
  if (is.null(mu) & !is.null(prob) & !is.null(size)) mu <- (size/prob)-size
  # # prob = size/(size+mu)
  # if (is.null(prob) & !is.null(size) & !is.null(mu)) prob <- size/(size+mu)
   # var = mu + mu^2 / size
  if (is.null(size) & !is.null(prob) & !is.null(var)) {
    # size <- mu^2 / (var - mu)
    # mu <- ((size/prob)-size)
    # size <- (prob * mu) / (1- prob)
    # size <- mu^2 / (var - mu)
    # size * (var - mu) <- mu^2
    # size * var - size - mu - mu^2 <- 0
    
    # size <- mu^2 / (var - mu)
    # size <- ((size/prob)-size)^2 / (var - ((size/prob)-size))
    # size <- ((size/prob)-(prob*size)/prob)^2 / (var - (size/prob)+size)
    # size <- ((size/prob)^2-2*((prob*size)/prob)*(size/prob)+((prob*size)/prob)^2) / ((var*prob - size + size*prob)/prob)
    # size <- ((size^2/prob^2)-2*((prob*size^2)/prob^2)+((prob^2*size^2)/prob^2)) / ((var*prob - size + size*prob)/prob)
    # size <- ((size^2/prob^2)-2*((prob*size^2)/prob^2)+((prob^2*size^2)/prob^2)) * (prob/(var*prob - size + size*prob))
    # size <- (((size^2)-2*((prob*size^2))+((prob^2*size^2)))/prob^2) * (prob/(var*prob - size + size*prob))
    # size <- (((size^2)-2*((prob*size^2))+((prob^2*size^2)))/prob) * (1/(var*prob - size + size*prob))
    # size <- (((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)/1) * (1/(var*prob - size + size*prob))
    
    # size <- (((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)/(var*prob - size + size*prob))
    
    # size - (((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)/(var*prob - size + size*prob)) <- 0
    # (size*(var*prob - size + size*prob))/(var*prob - size + size*prob) - (((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)/(var*prob - size + size*prob)) <- 0
    # ((size*var*prob - size^2 + size^2*prob) - ((size^2-2*(prob*size^2)+(prob^2*size^2))/prob))/(var*prob - size + size*prob) <- 0
    # ((size*var*prob - size^2 + size^2*prob) - ((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)) <- 0
    
    #  ((size*var*prob^2 - size^2*prob + size^2*prob^2)/prob - ((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)) <- 0
    #  ((size*var*prob^2 - size^2*prob + size^2*prob^2) - ((size^2-2*(prob*size^2)+(prob^2*size^2))))/prob <- 0
    #  size*var*prob^2 - size^2*prob + size^2*prob^2 - size^2+2*(prob*size^2)-prob^2*size^2 <- 0
    #  var*prob^2 - size*prob + size*prob^2 - size+2*prob*size-prob^2*size <- 0
    #  var*prob^2  <- size*prob - size*prob^2 + size -2*prob*size +prob^2*size
    
    #  var*prob^2  <- size*(prob - prob^2 + 1 -2*prob +prob^2)
    size <- (var*prob^2) / (1-prob)
    mu <- (size/prob)-size
  }
  
  if (is.null(size) & !is.null(mu) & !is.null(var)) size <- mu^2 / (var - mu)
  
  if (is.null(mu) & !is.null(size) & !is.null(var)) {
    # size <- mu^2 / (var - mu)
    # size * var -size * mu <- mu^2
    # mu^2 + size * mu - size * var <- 0
    A <- 1
    B <- size
    C <- - size * var
    delta <- B^2-4*A*C
    if (delta < 0) {
      warning("mu cannot be negative nor mu > var")
      return(rep(NA, n))
    }
    m1 <- (-B+sqrt(delta))/(2*A)
    m2 <- (-B-sqrt(delta))/(2*A)
    if (m1 > 0) {
      mu <- m1
    } else {
      mu <- m2
    }
  }
  
  if (is.null(size) | is.null(mu)) {
    warning("Not enough information were provided.")
    return(rep(NA, n))
  }

  if (size < 0) {
    warning("size cannot be negative nor mu > var")
    return(rep(NA, n))
  } else {
    return(rnbinom(n=n, size=size, mu=mu))
  }
}

#' contingencyTable.compare compares contingency table using Akaike weight.
#' @title Contingency table comparison using Akaike weight
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The probability that a single proportion model is sufficient to explain the data
#' @param table A matrix or a data.frame with series in rows and number of each category in column
#' @param criterion Which criterion is used for model selection
#' @param probs Series of probabilities used for conformity comparison
#' @description This function is used as a replacement of chisq.test() to not use p-value. 
#' @references Girondot, M., Guillon, J.-M., 2018. The w-value: An alternative to t- and X2 tests. Journal of Biostatistics & Biometrics 1, 1-4.
#' @family w-value functions
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' 
#' # Symmetry of Lepidochelys olivacea scutes
#' table <- t(data.frame(SriLanka=c(200, 157), AfricaAtl=c(19, 12), 
#'                       Guyana=c(8, 6), Suriname=c(162, 88), 
#'                       MexicoPac1984=c(42, 34), MexicoPac2014Dead=c(8, 9),
#'                       MexicoPac2014Alive=c(13, 12), 
#'                       row.names =c("Symmetric", "Asymmetric")))
#' table
#' contingencyTable.compare(table)
#' 
#' table <- t(data.frame(SriLanka=c(200, 157), AfricaAtl=c(19, 12), Guyana=c(8, 6),
#'                       Suriname=c(162, 88), MexicoPac1984=c(42, 34), 
#'                       MexicoPac2014Dead=c(8, 9),
#'                       MexicoPac2014Alive=c(13, 12), Lepidochelys.kempii=c(99, 1), 
#'                       row.names =c("Symmetric", "Asymmetric")))
#' table
#' contingencyTable.compare(table)
#' 
#' # Conformity to a model
#' table <- matrix(c(33, 12, 25, 75), ncol = 2, byrow = TRUE)
#' probs <- c(0.5, 0.5)
#' contingencyTable.compare(table, probs=probs)
#' 
#' # Conformity to a model
#' table <- matrix(c(33, 12), ncol = 2, byrow = TRUE)
#' probs <- c(0.5, 0.5)
#' contingencyTable.compare(table, probs=probs)
#' 
#' # Conformity to a model
#' table <- matrix(c(33, 12, 8, 25, 75, 9), ncol = 3, byrow = TRUE)
#' probs <- c(0.8, 0.1, 0.1)
#' contingencyTable.compare(table, probs=probs)
#' 
#' # Comparison of chisq.test() and this function
#' table <- matrix(c(NA, NA, 25, 75), ncol = 2, byrow = TRUE)
#' 
#' pv <- NULL
#' aw <- NULL
#' par(new=FALSE)
#' n <- 100
#' 
#' for (GroupA in 0:n) {
#'   table[1, 1] <- GroupA
#'   table[1, 2] <- n-GroupA
#'   pv <- c(pv, chisq.test(table)$p.value)
#'   aw <- c(aw, contingencyTable.compare(table, criterion="BIC")[1])
#' }
#' 
#' x <- 0:n
#' y <- pv
#' y2 <- aw
#' plot(x=x, y=y, type="l", bty="n", las=1, xlab="Number of type P in Group B", ylab="Probability", 
#'      main="", lwd=2)
#' lines(x=x, y=y2, type="l", col="red", lwd=2)
#' 
#' # w-value
#' (l1 <- x[which(aw>0.05)[1]])
#' (l2 <- rev(x)[which(rev(aw)>0.05)[1]])
#' 
#' aw[l1]
#' pv[l1]
#' 
#' aw[l2+2]
#' pv[l2+2]
#' 
#' # p-value
#' l1 <- which(pv>0.05)[1]
#' l2 <- max(which(pv>0.05))
#' 
#' aw[l1]
#' pv[l1]
#' 
#' aw[l2]
#' pv[l2]
#' 
#' y[which(y2>0.05)[1]]
#' y[which(rev(y2)>0.05)[1]]
#' 
#' par(xpd=TRUE)
#' text(x=25, y=1.15, labels="Group A: 25 type P / 100", pos=1)
#' 
#' segments(x0=25, y0=0, x1=25, y1=1, lty=3)
#' 
#' # plot(1, 1)
#' 
#' v1 <- c(expression(italic("p")*"-value"), expression("after "*chi^2*"-test"))
#' v2 <- c(expression(italic("w")*"-value for A"), expression("and B identical models"))
#' legend("topright", legend=c(v1, v2), 
#'        y.intersp = 1, 
#'        col=c("black", "black", "red", "red"), bty="n", lty=c(1, 0, 1, 0))
#' 
#' segments(x0=0, x1=n, y0=0.05, y1=0.05, lty=2)
#' text(x=101, y=0.05, labels = "0.05", pos=4)
#' }
#' @export


contingencyTable.compare <- function(table, criterion=c("AIC", "AICc", "BIC"), probs=NULL) {
  # table <- matrix(c(10, 12, 19, 21, 7, 8), ncol = 2)
  
  
  nseries <- nrow(table)
  c <- ncol(table)
  k <- c-1
  n <- nseries*k
  
  AIC_s <- 0
  AIC <- 0
  BIC_s <- 0
  BIC <- 0
  AICc_s <- 0
  AICc <- 0
  for (nec in 1:nseries){
    l <- table[nec, ]
    if (is.null(probs)) {
      prob <- l/sum(l)
    } else {
      prob <- probs
    }
    logL <- dmultinom(l, size = sum(l), prob = prob, log = TRUE)
    BIC_s <- BIC_s -2*logL
    AIC_s <- AIC_s -2*logL
    AICc_s <- AICc_s -2*logL
    if (is.null(probs)) {
      prob <- colSums(table)/sum(colSums(table))
    } else {
      prob <- probs
    }
    logL <- dmultinom(l, size = sum(l), prob = prob, log = TRUE)
    AIC <- AIC -2*logL
    BIC <- BIC -2*logL
    AICc <- AICc -2*logL
  }
  
  k_common <- ifelse(is.null(probs), k, 0)
  k_s <- n
    
  AIC <- AIC + 2*k_common
  AIC_s <- AIC_s + 2*k_s
  
  BIC <- BIC + k_common*log(n)
  BIC_s <- BIC_s + k_s*log(n)
  
  AICc <- AICc + 2*k_common + (2*k_common*(k_common+1))/(n-k_common-1)
  AICc_s <- AICc_s + 2*k_s + (2*k_s*(k_s+1))/(n-k_s-1)
  
  crit <- NULL
  AICmin <- min(c(AIC, AIC_s))
  AICcmin <- min(c(AICc, AICc_s))
  BICmin <- min(c(BIC, BIC_s))
  
  if (any(criterion=="AIC")) crit <- c("AICw(identical)"=exp(-0.5*(AIC-AICmin))/(exp(-0.5*(AIC-AICmin))+exp(-0.5*(AIC_s-AICmin))), 
                                       "AICw(different)"=1-(exp(-0.5*(AIC-AICmin))/(exp(-0.5*(AIC-AICmin))+exp(-0.5*(AIC_s-AICmin))))
  )
  if (any(criterion=="BIC")) crit <- c(crit, "BICw(identical)"=exp(-0.5*(BIC-BICmin))/(exp(-0.5*(BIC-BICmin))+exp(-0.5*(BIC_s-BICmin))), 
                                       "BICw(different)"=1-(exp(-0.5*(BIC-BICmin))/(exp(-0.5*(BIC-BICmin))+exp(-0.5*(BIC_s-BICmin))))                  
  )
  
  if (any(criterion=="AICc")) {
   if ((2*k_s*(k_s+1))/(n-k_s-1)<0) {
      warning("The AICc cannot be estimated.")
      crit <- c(crit, "AICcw(identical)"=NA, "AICcw(different)"=NA)
    } else {
      crit <- c(crit, "AICcw(identical)"=exp(-0.5*(AICc-AICcmin))/(exp(-0.5*(AICc-AICcmin))+exp(-0.5*(AICc_s-AICcmin))), 
                                        "AICcw(different)"=1-(exp(-0.5*(AICc-AICcmin))/(exp(-0.5*(AICc-AICcmin))+exp(-0.5*(AICc_s-AICcmin))))
                )
    }
  }
    
  
  return(crit)
}

#' IC_clean_data checks and corrects the dataframe to be used with IC_threshold_matrix
#' @title Clean the dataframe before to be used with IC_threshold_matrix
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A dataframe
#' @param data The data.frame to be cleaned
#' @param use an optional character string giving a method for computing covariances in the presence of missing values. This must be (an abbreviation of) one of the strings "everything", "all.obs", "complete.obs", "na.or.complete", or "pairwise.complete.obs".
#' @param method a character string indicating which correlation coefficient (or covariance) is to be computed. One of "pearson" (default), "kendall", or "spearman": can be abbreviated.
#' @param variable.retain a vector with the name of columns to keep
#' @param test.partial.correlation should the partial correlations be tested ?
#' @param progress Show a progress bar
#' @param debug if TRUE, information about progression of cleaning are shown
#' @description This function must be used if missing values are present in the dataset.\cr
#' It ensures that all correlations and partial correlations can be calculated. 
#' The columns of the dataframe are removed one per one until all can be calculated without error. 
#' It is possible to say that one or more columns must be retained because they are of particular importance in the analysis. 
#' The use and method parameters are used by cor() function. The function uses by default a parallel computing in Unix or MacOSX systems. 
#' If progress is TRUE and the package pbmcapply is present, a progress bar is displayed. If debug is TRUE, some informations are shown during the process.
#' \code{https://fr.wikipedia.org/wiki/Iconographie_des_corrélations}
#' @family Iconography of correlations
#' @references Lesty, M., 1999. Une nouvelle approche dans le choix des régresseurs de la régression multiple en présence d’interactions et de colinéarités. Revue de Modulad 22, 41-77.
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # based on https://fr.wikipedia.org/wiki/Iconographie_des_corrélations
#' es <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
#'                      Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
#'                      Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
#'                      Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
#'                      Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
#'                      row.names = c(NA, -8L), class = "data.frame")
#' es
#' 
#' df_clean <- IC_clean_data(es, debug = TRUE)
#' cor_matrix <- IC_threshold_matrix(data=df_clean, threshold = NULL, progress=FALSE)
#' cor_threshold <- IC_threshold_matrix(data=df_clean, threshold = 0.3)
#' plot(cor_threshold, show.legend.strength=FALSE, show.legend.direction = FALSE)
#' cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
#' plot(cor_threshold_Note, show.legend.strength=FALSE, show.legend.direction = FALSE)
#' 
#' cor_threshold <- IC_threshold_matrix(data=df_clean, threshold = 0.6)
#' plot(cor_threshold, 
#' layout=matrix(data=c(53, 53, 55, 55, 
#'                      55, 53, 55, 53), ncol=2, byrow=FALSE), 
#' show.legend.direction = FALSE,
#' show.legend.strength = FALSE, xlim=c(-2, 2), ylim=c(-2, 2))
#' }
#' @export


IC_clean_data <- function(data=stop("A dataframe object is required")               , 
                          use = c("pairwise.complete.obs", "everything", 
                                        "all.obs", "complete.obs", "na.or.complete"), 
                          method=c("pearson", "kendall", "spearman")                , 
                          variable.retain=NULL                                      ,
                          test.partial.correlation=TRUE                             , 
                          progress=TRUE                                             , 
                          debug=FALSE                                               ) {
     
  # data=NULL
  # use="pairwise.complete.obs"
  # method="pearson"
  # variable.retain=NULL
  # test.partial.correlation=TRUE
  # progress=TRUE
  # debug=FALSE
  
  method <- method[1]
  use <- use[1]
  
  if (!(is.element("ppcor", installed.packages()[,1]))) {
    ginv <- function (X, tol = sqrt(.Machine$double.eps)) 
    {
      if (length(dim(X)) > 2L || !(is.numeric(X) || is.complex(X))) 
        stop("'X' must be a numeric or complex matrix")
      if (!is.matrix(X)) 
        X <- as.matrix(X)
      Xsvd <- svd(X)
      if (is.complex(X)) 
        Xsvd$u <- Conj(Xsvd$u)
      Positive <- Xsvd$d > max(tol * Xsvd$d[1L], 0)
      if (all(Positive)) 
        Xsvd$v %*% (1/Xsvd$d * t(Xsvd$u))
      else if (!any(Positive)) 
        array(0, dim(X)[2L:1L])
      else Xsvd$v[, Positive, drop = FALSE] %*% ((1/Xsvd$d[Positive]) * 
                                                   t(Xsvd$u[, Positive, drop = FALSE]))
    }
    
    pcor <- function (x, method = c("pearson", "kendall", "spearman")) 
    {
      method <- match.arg(method)
      if (is.data.frame(x)) 
        x <- as.matrix(x)
      if (!is.matrix(x)) 
        stop("supply a matrix-like 'x'")
      if (!(is.numeric(x) || is.logical(x))) 
        stop("'x' must be numeric")
      stopifnot(is.atomic(x))
      n <- dim(x)[1]
      gp <- dim(x)[2] - 2
      cvx <- cov(x, method = method)
      if (det(cvx) < .Machine$double.eps) {
        warning("The inverse of variance-covariance matrix is calculated using Moore-Penrose generalized matrix invers due to its determinant of zero.")
        icvx <- ginv(cvx)
      }
      else icvx <- solve(cvx)
      pcor <- -cov2cor(icvx)
      diag(pcor) <- 1
      if (method == "kendall") {
        statistic <- pcor/sqrt(2 * (2 * (n - gp) + 5)/(9 * (n - 
                                                              gp) * (n - 1 - gp)))
        p.value <- 2 * pnorm(-abs(statistic))
      }
      else {
        statistic <- pcor * sqrt((n - 2 - gp)/(1 - pcor^2))
        p.value <- 2 * pt(-abs(statistic), (n - 2 - gp))
      }
      diag(statistic) <- 0
      diag(p.value) <- 0
      list(estimate = pcor, p.value = p.value, statistic = statistic, 
           n = n, gp = gp, method = method)
    }
  } else {
    pcor <- getFromNamespace("pcor", ns="ppcor")
  }


  zc_y <- data
  # J'enlève les colonnes qui n'ont que des NA
   if (debug) cat(paste0("The data have ", ncol(zc_y), " variables\n"))
   if (debug) cat("I remove the columns with only NA\n")
  if (debug) {cat(colnames(zc_y)[apply(X = zc_y, MARGIN = 2, FUN = function(x) all(is.na(x)))]); cat("\n")}
  zc_y <- zc_y[, !apply(X = zc_y, MARGIN = 2, FUN = function(x) all(is.na(x)))]
   if (debug) cat(paste0("The data have ", ncol(zc_y), " variables\n"))
  
  # J'enlève les colonnes qui ne présente pas de variabilité
  if (debug) cat("I remove the columns with no variability\n")
  if (debug) {cat(colnames(zc_y)[apply(X = zc_y, MARGIN = 2, FUN = function(x) all(na.omit(x)==na.omit(x)[1]))]); cat("\n")}
  zc_y <- zc_y[, !apply(X = zc_y, MARGIN = 2, FUN = function(x) all(na.omit(x)==na.omit(x)[1]))]
   if (debug) cat(paste0("The data have ", ncol(zc_y), " variables\n"))
  # J'enlève les colonnes qui ne sont pas numériques
  if (debug) cat("I remove the non-numeric columns\n")
  # Là c'est plutôt un inherits
  zc_y <- zc_y[, (sapply(zc_y, class) == "numeric") | (sapply(zc_y, class) == "integer")]
   if (debug) cat(paste0("The data have ", ncol(zc_y), " variables\n"))
  # J'enlève les colonnes qui empêchent de calculer les corrélations
  if (debug) cat("I remove the columns producing an error for correlations estimations\n")

  repeat {
    c <- suppressWarnings(expr=cor(zc_y, use=use, method=method))
    k <- apply(c, MARGIN=1, FUN=function(x) sum(ifelse(is.na(x), 1, 0)))
    # names(k) <- colnames(zc_y)
    # km1 <- max(k)
    if (!is.null(variable.retain)) k[variable.retain] <- 0
    km <- max(k)
    if (km == 0) break
    if (debug) cat(paste0("Remove: ", colnames(zc_y)[which.max(k)], "\n"))
    zc_y <- zc_y[, -which.max(k)]
  }
  
  if (debug) cat(paste0("The data have ", ncol(zc_y), " variables\n"))
  
  if (test.partial.correlation) {
    if (debug) cat("I remove the columns producing an error for partial correlations estimations\n")
  # J'enlève les colonnes qui empêchent les calculs des corrélations partielles
  
    lapp <- lapply
  if (is.element("parallel", installed.packages()[,1]))  lapp <- getFromNamespace("mclapply", ns="parallel")
  if (progress & (is.element("pbmcapply", installed.packages()[,1])))  lapp <- getFromNamespace("pbmclapply", ns="pbmcapply")
  # if (debug) lapp <- lapply
  
  options(mc.cores = getFromNamespace("detectCores", ns="parallel")())
  
  cname <- colnames(zc_y)
  lc <- length(cname)
  llc <- 1:lc
  # df <- expand.grid(e=1:(lc-2), f=2:(lc-1), g=3:lc)
  # df <- df[(df[, 1]<df[, 2]) & (df[, 2]<df[, 3]), ]
  
  # lc <- 5
  
  df <- expand.grid(e=1:(lc-2), f=2:(lc-1), g=3:lc)
  df <- df[(df[, 1]<df[, 2]) & (df[, 2]<df[, 3]), ]
  
  # pc <- array(data=Inf, dim=c(lc, lc, lc), dimnames = list(cname, cname, cname))
  

  z <- lapp(1:nrow(df), FUN = function(x) {
    e <- df[x, 1]
    f <- df[x, 2]
    g <- df[x, 3]
    
    dfg <- na.omit(zc_y[, c(e, f, g)])
    pc <- try(suppressWarnings(pcor(dfg, method=method)), 
              silent=TRUE)
    outg <- (inherits(pc, "try-error"))
    if (!outg) outg <- any(!is.finite(pc$estimate))
    # if (outg==0) {
    #   outg <- c(outg, pc$estimate[1, 2], pc$estimate[1, 3], pc$estimate[2, 3])
    # } else {
    #   outg <- c(outg, NA, NA, NA)
    # }
      return(!outg)
    }
  )
  
  
  # Si TRUE, c'est bon
  # FALSE c'est une erreur
  zul <- unlist(z)
  
  asupprimer <- NULL
    repeat {

    if (all(zul)) break
    
    zdf <- as.data.frame(table(as.vector(as.matrix(df[!zul, ]))), stringsAsFactors = FALSE)
    
    if (!is.null(variable.retain)) {
      zdf <- cbind(zdf, name=colnames(zc_y)[as.numeric(zdf$Var1)])
      zdf[zdf$name == variable.retain, "Freq"] <- 0
    }
    as <- as.numeric(zdf$Var1[which.max(zdf$Freq)])
    
    if (debug) cat(paste0("Remove: ", colnames(zc_y)[as], "\n"))
    
    zul[which(df[,1]==as)] <- TRUE
    zul[which(df[,2]==as)] <- TRUE
    zul[which(df[,3]==as)] <- TRUE

    asupprimer <- c(asupprimer, as)
    }
  
  if (!is.null(asupprimer)) zc_y <- zc_y[, -asupprimer]
  if (debug) cat(paste0("The data have ", ncol(zc_y), " variables\n"))
  }
  
return(zc_y)
}

#' r2norm returns random numbers for Gaussian distributions different at left and right
#' @title Random generation for Gaussian distributions different at left and right 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return r2norm returns random numbers
#' @param n number of observations.
#' @param mean vector of means
#' @param sd_low vector of standard deviations below the mean.
#' @param sd_high vector of standard deviations above the mean.
#' @description Random generation for Gaussian distributions different at left and right
#' @family Distributions
#' @examples
#' \dontrun{
#' n <- r2norm(1000, mean=25, sd_low=2, sd_high=10)
#' 
#' hist(n)
#' }
#' @export

# Random number when the distribution is known by mediane and 0.025 and 0.975 quantiles
# It will be supposed that each half distribution is Gaussian.
# It should be done better but it is not easy

r2norm <- function(n, mean = 0, sd_low = 1, sd_high = 1) {
  rn <- rnorm(n, mean = 0, sd = 1)
  return(ifelse(rn<0, rn*sd_low+mean, rn*sd_high+mean))
}

#' @title Posterior Predictive Distribution from mcmcComposite object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A data.frame with quantiles
#' @param x A mcmcComposite obtained as a result of \code{MHalgoGen()} function
#' @param chain The number of the chain in which to get parameters or "all"
#' @param FUN The function to apply the parameters. It must be text.
#' @param probs The probability to get quantiles
#' @param xlim The values to apply in fun if required
#' @param nameparxlim The name of the parameter for xlim
#' @param namepar The name of parameters from mcmc object to be used in fun
#' @description Extract quantile distributions from mcmcComposite object. If FUN is used, it keeps the 
#' correlations between parameters.
#' @family mcmcComposite functions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' require(coda)
#' x <- rnorm(30, 10, 2)
#' dnormx <- function(data, x) {
#'  data <- unlist(data)
#'  return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
#' }
#' 
#' parameters_mcmc <- setPriors1(Name = "mean", 
#'                               Density = "dnorm", Prior1 = 10, Prior2 = 2, 
#'                               SDProp = 1, Min = -3, Max = 100, Init = 10)
#' parameters_mcmc <- parameters_mcmc + setPriors1(Name = "sd", 
#'                               Density = "dlnorm", Prior1 = 0.5, Prior2 = 0.5, 
#'                               SDProp = 1, Min = 0, Max = 10, Init = 2)
#'                               
#' mcmc_run <- MHalgoGen(n.iter=10000, parameters=parameters_mcmc, data=x, 
#'                      likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
#' 
#' # Return the 2.5% and 97.5% quantiles of the mean parameter
#' k <- PPD(x=mcmc_run, namepar="mean")
#' k <- PPD(x=mcmc_run, namepar=NULL)
#' k <- PPD(x=mcmc_run, namepar=c("mean", "sd"))
#' 
#' # Here I return the distribution of coefficient of variation as a posterior predictive distribution
#' k <- PPD(x=mcmc_run            , 
#'                  FUN="sd/mean" )
#' 
#' # I can generate an output based on a x variable 
#' k <- PPD(x=mcmc_run                   , 
#'                   xlim=c(1, 2)        ,
#'                   nameparxlim="mult"  ,
#'                   FUN="mult*mean"     )
#' }
#' @export


PPD <- function(x                         , 
                chain="all"               , 
                FUN=NULL                  , 
                probs=c(0.025, 0.975)     , 
                xlim=NULL                 , 
                nameparxlim=NULL          , 
                namepar=NULL              ) {
  
  # chain=1; fun=function(x) {x}; probs=c(0.025, 0.975); xlim=NULL; nameparxlim=NULL; namepar=NULL
  if (chain[1] == "all") chain <- seq_along(x$resultMCMC)
  
  # if (total) p <- x$resultMCMC.total[[chain[1]]] else p <- x$resultMCMC[[chain[1]]]
  
  p <- NULL
  
  if (length(chain) >= 1) {
    for (i in chain) {
      p <- rbind(p, x$resultMCMC[[i]])
    }
  }
  
  if (is.null(p)) stop("No data are available.")
  
  if (is.null(namepar)) namepar <- colnames(p)
  
	# p <- x$resultMCMC[[chain]]
  if (is.null(FUN)) {
    df <- matrix(NA, ncol=length(namepar), nrow = nrow(p))
    colnames(df) <- namepar
  } else {
    if (is.null(xlim)) {
      xlimpos <- 1
    } else {
      xlimpos <- length(xlim)
    }
    df <- matrix(NA, ncol=xlimpos, nrow = nrow(p))
    colnames(df) <- as.character(xlim)
  }
	
  if (!is.null(FUN)) {
    FUN <- eval(parse(text = paste0("function(...) {dots <- list(...); return(with(dots, ", FUN, "))}")))
  }
  
	for (l in 1:nrow(p)) {
	  if (is.null(FUN)) {
	    funx <- function(...) return(as.numeric(list(...)))
	    par <- p[l, ]
	    if (!is.null(namepar)) par <- par[namepar]
	    parx <- as.list(par)
	    h <- do.call(funx, args=parx)
	    df[l, ] <- h
	  } else {
	    par <- p[l, ]
	    if (!is.null(namepar)) par <- par[namepar]
	    
	    for (i in 1:xlimpos) {
	      if (!is.null(xlim)) {
	      m <- xlim[i]
	      names(m) <- nameparxlim
	      parx <- as.list(c(m, par))
	      } else {
	        parx <- as.list(par)
	      }
	      h <- do.call(FUN, args=parx)
	      df[l, i] <- h
	    }
	  }
	}

	q <- apply(df, MARGIN = 2, FUN = function(x) quantile(x, probs))
	colnames(q) <- colnames(df)
	return(q)

}

#' RM_delete deletes a results managment or a result within a results managment from an object
#' @title Delete a results managment or a result within a results managment from an object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The original object with the deleted results managment
#' @param x The object to delete a results managment
#' @param RM The name of results managment stored
#' @param RMname The name of the result that will be deleted or its rank
#' @param valuename The name of the result that will be deleted
#' @description Return the original object with the deleted results managment or result.
#' @family Results Managment
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Let an object of class objclass being created
#' obj <- list(A=100, name="My object")
#' class(obj) <- "objclass"
#' # And now I create a RM to this object
#' obj <- RM_add(x=obj, RMname="NewAnalysis1")
#' obj <- RM_add(x=obj, RMname="NewAnalysis2")
#' RM_list(obj)
#' obj <- RM_delete(x=obj, RMname="NewAnalysis1")
#' RM_list(obj)
#' obj <- RM_delete(x=obj, RMname=1)
#' RM_list(obj)
#' obj <- RM_add(x=obj, RMname="NewAnalysis1", valuename="V1", value=100)
#' RM_list(obj)
#' RM_get(x=obj, RMname="NewAnalysis1", valuename="V1")
#' obj <- RM_add(x=obj, RMname="NewAnalysis1", valuename="V2", value=200)
#' RM_get(x=obj, RMname="NewAnalysis1", valuename="V2")
#' obj <- RM_delete(x=obj, RMname="NewAnalysis1", valuename="V1")
#' RM_get(x=obj, RMname="NewAnalysis1", valuename="V1")
#' RM_get(x=obj, RMname="NewAnalysis1", valuename="V2")
#' }
#' @export

RM_delete <- function(x=stop("An object with results managment must be provided"), 
                      RM="RM", 
                      RMname=stop("A name must be provided"), 
                      valuename=NULL) {
  xa <- attributes(x)
  rx <- xa[[RM]]
  if (is.null(rx)) {
    # Il n'y avait pas d'object RM
    return(x)
  }
  
  # Si le name est un numéro, je prends la liste et le nom
  if (is.numeric(RMname)) {
    RMname <- names(rx)[RMname]
  }
  
  # il y en avait un
  irx <- which(names(rx)==RMname)
  # Si le nom n'existe pas
  if (identical(irx, integer(0))) {
    return(x)
  }
  
  if (is.null(valuename)) {
    #  Je retire tout le RNname
    rx <- rx[-irx]
  } else {
    if (!identical(integer(0), which(names(rx[[irx]]) == valuename))) {
      rx[[irx]] <- rx[[irx]][-which(names(rx[[irx]]) == valuename)]
    }
  }
  xa[[RM]] <- rx
  attributes(x) <- xa
  
  return(x)
  
}

#' print.cutter plot result of cutter
#' @title Print results of cutter that best describe distribution
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @param x A result file generated by cutter
#' @param silent If TRUE does not show the outpout
#' @param ... Not used
#' @description Print the estimates of cut distribution.
#' @family Distributions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' # _______________________________________________________________
#' # right censored distribution with gamma distribution
#' # _______________________________________________________________
#' # Detection limit
#' DL <- 100
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc>DL] <- +Inf
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, upper_detection_limit=DL, 
#'                            cut_method="censored")
#' result
#' plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # The same data seen as truncated data with gamma distribution
#' # _______________________________________________________________
#' obc <- obc[is.finite(obc)]
#' # search for the parameters the best fit these truncated data
#' result <- cutter(observations=obc, upper_detection_limit=DL, 
#'                            cut_method="truncated")
#' result
#' plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # left censored distribution with gamma distribution
#' # _______________________________________________________________
#' # Detection limit
#' DL <- 10
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc<DL] <- -Inf
#' # search for the parameters the best fit these truncated data
#' result <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored")
#' result
#' plot(result, xlim=c(0, 200), breaks=seq(from=0, to=200, by=10))
#' # _______________________________________________________________
#' # left and right censored distribution
#' # _______________________________________________________________
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # Detection limit
#' LDL <- 10
#' # remove the data below the detection limit
#' obc[obc<LDL] <- -Inf
#' # Detection limit
#' UDL <- 100
#' # remove the data below the detection limit
#' obc[obc>UDL] <- +Inf
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                            upper_detection_limit=UDL, 
#'                           cut_method="censored")
#' result
#' plot(result, xlim=c(0, 150), col.DL=c("black", "grey"), 
#'                              col.unobserved=c("green", "blue"), 
#'      breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # Example with two values for lower detection limits
#' # corresponding at two different methods of detection for example
#' # with gamma distribution
#' # _______________________________________________________________
#' obc <- rgamma(50, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL1 <- 10
#' # remove the data below the detection limit
#' obc[obc<LDL1] <- -Inf
#' obc2 <- rgamma(50, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL2 <- 20
#' # remove the data below the detection limit
#' obc2[obc2<LDL2] <- -Inf
#' obc <- c(obc, obc2)
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, 
#'                            lower_detection_limit=c(rep(LDL1, 50), rep(LDL2, 50)), 
#'                           cut_method="censored")
#' result
#' # It is difficult to choose the best set of colors
#' plot(result, xlim=c(0, 150), col.dist="red", 
#'      col.unobserved=c(rgb(red=1, green=0, blue=0, alpha=0.1), 
#'                       rgb(red=1, green=0, blue=0, alpha=0.2)), 
#'      col.DL=c(rgb(red=0, green=0, blue=1, alpha=0.5), 
#'                       rgb(red=0, green=0, blue=1, alpha=0.9)), 
#'      breaks=seq(from=0, to=200, by=10))
#' # ___________________________________________________________________
#' # left censored distribution comparison of normal, lognormal and gamma
#' # ___________________________________________________________________
#' # Detection limit
#' DL <- 10
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc<DL] <- -Inf
#' # search for the parameters the best fit these truncated data
#' result_gamma <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="gamma")
#' result_gamma
#' plot(result_gamma, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' result_lognormal <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="lognormal")
#' result_lognormal
#' plot(result_lognormal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' result_normal <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="normal")
#' result_normal
#' plot(result_normal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' compare_AICc(gamma=result_gamma, 
#'             lognormal=result_lognormal, 
#'             normal=result_normal)
#' # ___________________________________________________________________
#' # Test for similarity in gamma left censored distribution between two
#' # datasets
#' # ___________________________________________________________________
#' obc1 <- rgamma(100, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL <- 10
#' # remove the data below the detection limit
#' obc1[obc1<LDL] <- -Inf
#' obc2 <- rgamma(100, scale=10, shape=2)
#' # remove the data below the detection limit
#' obc2[obc2<LDL] <- -Inf
#' # search for the parameters the best fit these censored data
#' result1 <- cutter(observations=obc1, 
#'                   distribution="gamma", 
#'                   lower_detection_limit=LDL, 
#'                   cut_method="censored")
#' logLik(result1)
#' plot(result1, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#' result2 <- cutter(observations=obc2, 
#'                   distribution="gamma", 
#'                   lower_detection_limit=LDL, 
#'                   cut_method="censored")
#' logLik(result2)
#' plot(result2, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#' result_totl <- cutter(observations=c(obc1, obc2), 
#'                       distribution="gamma", 
#'                       lower_detection_limit=LDL, 
#'                       cut_method="censored")
#' logLik(result_totl)
#' plot(result_totl, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#'      
#' compare_AICc(Separate=list(result1, result2), 
#'             Common=result_totl, factor.value=1)
#' compare_BIC(Separate=list(result1, result2), 
#'             Common=result_totl, factor.value=1)           
#' }
#' @method print cutter
#' @export



print.cutter <- function(x, silent=FALSE, ...) {
  
  # p3p <- list(...)
  
  mc <- "censored and truncated"
  if (all(x$observation[, "Cut"] == "censored")) mc <- "censored"
  if (all(x$observation[, "Cut"] == "truncated")) mc <- "truncated"
  
  textoutput <- paste(x$distribution, mc, "distribution using Bayesian MCMC:")
  textoutput <- paste0(textoutput, "\n", paste0(rep("-", nchar(textoutput)), collapse = ""))
  
  textoutput <- paste0(textoutput, "\nNumber of data: ", x$n.quantified+x$n.LDL+x$n.UDL)
  textoutput <- paste0(textoutput, "\nNumber of LDL data: ", x$n.LDL, " (", specify_decimal(x$n.LDL/(x$n.quantified+x$n.LDL+x$n.UDL)), ")")
  textoutput <- paste0(textoutput, "\nNumber of UDL data: ", x$n.UDL, " (", specify_decimal(x$n.UDL/(x$n.quantified+x$n.LDL+x$n.UDL)), ")")
  if (!is.null(x$prob.LDL)) {
    textoutput <- paste0(textoutput, "\nExpected proportion of LDL data: ")
    textoutput <- paste0(textoutput, "\n", paste0(capture.output(print(x$prob.LDL)), collapse = "\n"))
  }
  if (!is.null(x$prob.UDL))  {
    textoutput <- paste0(textoutput, "\nExpected proportion of UDL data: ")
    textoutput <- paste0(textoutput, "\n", paste0(capture.output(print(x$prob.UDL)), collapse = "\n"))
  }
  
  if (!is.null(x$par_mcmc)) {
    for (i in 1:ncol(x$par_mcmc)) {
      textoutput <- paste0(textoutput, "\n", paste(colnames(x$par_mcmc)[i], "=", specify_decimal(x$par_mcmc[2, i]), "; Credible Interval 95%= [", specify_decimal(x$par_mcmc[1, i]), ",", specify_decimal(unname(x$par_mcmc[3, i])), "]", sep=""))
    }
  } else {
    for (i in 1:length(x$par)) {
      textoutput <- paste0(textoutput, "\n", paste(names(x$par)[i], "=", specify_decimal(x$par[i]), sep=""))
    }
  }
  
  
  textoutput <- paste0(textoutput, "\n", paste("-Ln L=", specify_decimal(x$value)))
  textoutput <- paste0(textoutput, "\n", paste("AIC=", specify_decimal(x$AIC)))
  textoutput <- paste0(textoutput, "\n", paste("AICc=", specify_decimal(x$AICc)))
  textoutput <- paste0(textoutput, "\n", paste("BIC=", specify_decimal(x$BIC)))
  
  mean <- x$total.distribution["mean"]
  sd<- x$total.distribution["sd"]
  
  textoutput <- paste0(textoutput, "\n", paste0("The distribution has a mean of ", specify_decimal(mean), 
                                                " and a standard deviation of ", specify_decimal(sd), "."))
  
  if (!is.null(x$LDL)) {
    for (LDL_c in 1:ncol(x$LDL)) {
      if (nrow(x$LDL)==3) {
      textoutput <- paste0(textoutput, "\n", paste0("The median value for samples below the detection limit ", specify_decimal(colnames(x$LDL)[LDL_c])," is ", specify_decimal(x$LDL[2, LDL_c])))
      textoutput <- paste0(textoutput, "\n", paste0("   The 95% credible interval is ", specify_decimal(x$LDL[1, LDL_c]), " to ", specify_decimal(x$LDL[3, LDL_c])))
      } else {
        textoutput <- paste0(textoutput, "\n", paste0("The mean value for samples below the detection limit ", specify_decimal(colnames(x$LDL)[LDL_c])," is ", specify_decimal(x$LDL["mean", LDL_c])))
      }
    }
  }
  
  if (!is.null(x$UDL)) {
    for (UDL_c in 1:ncol(x$UDL)) {
      if (nrow(x$UDL)==3) {
      textoutput <- paste0(textoutput, "\n", paste0("The median value for samples above the detection limit ", specify_decimal(colnames(x$UDL)[UDL_c])," is ", specify_decimal(x$UDL[2, UDL_c])))
      textoutput <- paste0(textoutput, "\n", paste0("   The 95% credible interval is ", specify_decimal(x$UDL[1, UDL_c]), " to ", specify_decimal(x$UDL[3, UDL_c])))
      } else {
        textoutput <- paste0(textoutput, "\n", paste0("The mean value for samples above the detection limit ", specify_decimal(colnames(x$UDL)[UDL_c])," is ", specify_decimal(x$UDL["mean", UDL_c])))
      }
    }
  }
  
  if (!is.null(x$GoF)) {
    
    textoutput <- paste0(textoutput, "\n", paste("The probability that the data could have been obtained using the fitted distribution is", specify_decimal(x$GoF), "taking into account the distribution of likelihoods of posterior predictive distribution compared to the likelihood of observed data in the fitted model."))
    
  }
  textoutput <- paste0(textoutput, "\n")
  
  if (!silent) cat(textoutput)
  return(invisible(textoutput))
}

#' minmax.periodic search for minimum and maximum indices (temperatures or levels) in periodic timeseries
#' @title Search for minimum and maximum indices in periodic timeseries
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A data.frame with a column time, a column index and a column SD
#' @param time.minmax.daily A named vector with Min and Max being the time in the day with minimum and maximum indices (temperature or level)
#' @param time.minmax A named vector daily with time in the day at which minimum and maximum indices are observed
#' @param observed	A dataframe with at least two columns: time and temperatures. A third column SD can indicate the know error in index
#' @param period The unit of day period (24 for hours, 24*60 for minutes)
#' @param progressbar Tell if a progression bar must be shown
#' @param colname.time The name of the column for time in observed
#' @param colname.index The name of the column for indices in observed
#' @param colname.SD The name of the column for SD in observed
#' @param plot If TRUE, show a plot with the different estimates
#' @family Periodic patterns of indices
#' @description Search for minimum and maximum for periodic timeseries when only intermediate values are known.\cr
#' For each couple of value with an increasing or decreasing segment of 
#' the sinusoid function, it is possible to estimate a minimum and maximum 
#' values using analytical algebra.\cr
#' Then the average and standard deviations of all minima and maxima are evaluated.\cr
#' It should be noted that any extremum can be estimated at least twice, one by
#' increasing segment and one by decreasing segment. Both are used here to produce SD.\cr
#' \code{time.minmax.daily} should be used when the time at which maximum and minimum indices are regular and
#' \code{time.minmax} permits to define this time day by day.
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Generate a timeserie of time
#' time.obs <- NULL
#' for (i in 0:9) time.obs <- c(time.obs, c(0, 6, 12, 18)+i*24)
#' # For these time, generate a timeseries of temperatures
#' temp.obs <- rep(NA, length(time.obs))
#' temp.obs[3+(0:9)*4] <- rnorm(10, 25, 3)
#' temp.obs[1+(0:9)*4] <- rnorm(10, 10, 3)
#' for (i in 1:(length(time.obs)-1)) 
#'   if (is.na(temp.obs[i])) 
#'   temp.obs[i] <- mean(c(temp.obs[i-1], temp.obs[i+1]))
#'   if (is.na(temp.obs[length(time.obs)])) 
#'   temp.obs[length(time.obs)] <- temp.obs[length(time.obs)-1]/2
#' observed <- data.frame(time=time.obs, temperature=temp.obs)
#' # Search for the minimum and maximum values
#' r <- minmax.periodic(time.minmax.daily=c(Min=2, Max=15), 
#' observed=observed, period=24, colname.index="temperature")
#' 
#' # Estimate all the temperatures for these values
#' t <- index.periodic(minmax=r)
#' 
#' plot_errbar(x=t[,"time"], y=t[,"index"],
#' errbar.y=ifelse(is.na(t[,"sd"]), 0, 2*t[,"sd"]),
#' type="l", las=1, bty="n", errbar.y.polygon = TRUE, 
#' xlab="hours", ylab="Temperatures", ylim=c(0, 35), 
#' errbar.y.polygon.list = list(col="grey"))
#' 
#' plot_add(x=t[,"time"], y=t[,"index"], type="l")
#' 
#' plot_add(observed$time, observed$temperature, pch=19, cex=0.5)
#' }
#' @export

minmax.periodic <- function(time.minmax.daily=NULL, time.minmax=NULL, progressbar=FALSE, 
                            observed=stop("data.frame with observed indices"), 
                            period=24, 
                            colname.time="time", colname.index="index", colname.SD="SD", 
                            plot=FALSE) {

if (is.null(time.minmax) & is.null(time.minmax.daily)) {
  stop("Or time.minmax or time.minmax.daily must be provided")
}
if (!is.null(time.minmax) & !is.null(time.minmax.daily)) {
  stop("time.minmax and time.minmax.daily cannot be both provided")
}


observed <- observed[order(observed[,colname.time]), ]

temp.obs <- observed[,colname.index]
time.obs <- observed[,colname.time]
sd.obs <- if(any(colnames(observed)==colname.SD)) observed[,colname.SD] else NULL

# SI j'ai un sd sur les observations, je prends + ou - 1.96 SD
if (!is.null(sd.obs)) {
  temp.obs <- c(temp.obs-1.96*sd.obs, temp.obs+1.96*sd.obs)
  time.obs <- c(time.obs, time.obs)
}


# J'ai juste le time.minmax.daily, je dois reconstruire le time.minmax
if (is.null(time.minmax)) {
  # nombre de périodes
  maxday <- max(time.obs)/period+2
  time.minmax <- NULL
  for (i in 1:maxday) time.minmax <- c(time.minmax, 
                                     Min=unname(period*(i-1)+time.minmax.daily["Min"]), 
                                     Max=unname(period*(i-1)+time.minmax.daily["Max"]))
}

# Je ne garde que ceux dont j'ai besoin
if (any(time.minmax>max(time.obs))) time.minmax <- time.minmax[1:which(time.minmax>max(time.obs))[1]]

if (is.null(names(time.minmax)) | any(names(time.minmax)=="")) {
  stop("time.minmax must be named vector using names Min or Max")
}

# je classe time.minmax
time.minmax <- sort(time.minmax)

res <- data.frame(time=numeric(), temperature=numeric(), extrema=character(), stringsAsFactors=FALSE)

if (progressbar) pb<-txtProgressBar(min=1, max=(length(time.minmax)-1), style=3)

if (plot) {
  plot(x=time.obs, y=temp.obs, pch=4, bty="n", xlab="Time", ylab="Index")
  for (i in seq_along(time.minmax)) segments(x0=time.minmax[i], x1=time.minmax[i], 
                                            y0=ScalePreviousPlot()$ylim[1], 
                                            y1=ScalePreviousPlot()$ylim[2], lty=2)
}

for (i in 1:(length(time.minmax)-1)) {
  if (progressbar) setTxtProgressBar(pb, i)
  
  # Je cherche la valeur y0 et y1 a x0 et x1
  x0 <- time.minmax[i]
  x1 <- time.minmax[i+1]
  sen0 <- names(time.minmax)[i]
  sen1 <- names(time.minmax)[i+1]
  
  # Si je n'ai pas une observation pour exactement ces temps
  # Je ne comprends pas pourquoi je fais ce test
  if (all(time.obs!=x0) | all(time.obs!=x1)) {
    #  print(paste(1, i,length(res)))
    # Je prends les temps avec des observations entre
    lim <- (time.obs>=x0 & time.obs<=x1)
    wlim <- which(lim)
    if (length(wlim)>=2) {
      # Toutes ls combinaisons possibles
      couple <- t(combn(wlim, 2))
      for (j in 1:nrow(couple)) {
        x <- time.obs[couple[j, 1]]
        xp <- time.obs[couple[j, 2]]
        if (x!=xp) {
        y <- temp.obs[couple[j, 1]]
        yp <- temp.obs[couple[j, 2]]
        C <- cos(x*pi/(x1-x0)-x0*pi/(x1-x0))
        Cp <- cos(xp*pi/(x1-x0)-x0*pi/(x1-x0))
        L <- 1/2*(1-C)
        K <- 1/2*(Cp+1) 
        y0 <- (yp/K -y/(2*K*L) +(Cp*y)/(2*K*L)) / 
          (1 -C/(4*K*L) -1/(4*K*L) +(Cp*C)/(4*K*L) +(Cp)/(4*K*L))
        y1 <- y/L -C*y0/(2*L) -y0/(2*L)
        
        if (plot) {
          a <- pi/(x1-x0)
          b <- -x0*a
          p <- (y0-y1)/2
          q <- y0-p
          
          x <- seq(from=x0, to=x1, length=15)
          y <- cos(x*a+b)*p+q
          
          plot_add(x, y, type="l")
        }
        
        if (all(time.obs!=x0)) res <- rbind(res, data.frame(time=x0, temperature=y0, extrema=sen0))
        if (all(time.obs!=x1)) res <- rbind(res, data.frame(time=x1, temperature=y1, extrema=sen1))
        }
      }
    }
  }
  #    print(paste(2, i,length(res)))
}

##### fin du premier tour
##### je fais le deuxième

mn <- aggregate(temperature ~ time, subset(res, res$extrema=="Min"), mean)
mx <- aggregate(temperature ~ time, subset(res, res$extrema=="Max"), mean)
mn2 <- aggregate(temperature ~ time, subset(res, res$extrema=="Min"), sd)
mx2 <- aggregate(temperature ~ time, subset(res, res$extrema=="Max"), sd)

mx <- cbind(mx, mx2[,2])
mx <- cbind(mx, extrema=rep("Max", nrow(mx)), stringsAsFactors=FALSE)
mn <- cbind(mn, mn2[,2])
mn <- cbind(mn, extrema=rep("Min", nrow(mn)), stringsAsFactors=FALSE)

names(mx) <- c("time", "index", "SD", "extrema")
names(mn) <- c("time", "index", "SD", "extrema")

mxmn <- rbind(mx, mn)

o <- order(mxmn$time)
mxmn2 <- mxmn[o,]

rownames(mxmn2) <- 1:dim(mxmn2)[1]

return(mxmn2)

}

#' qvlmer is Quasi Variances for lmer Model Coefficients
#' @title Quasi Variances for lmer Model Coefficients
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list of class qv.
#' @param object A object obtained using lmer from package lme4
#' @param factorname Either NULL, or a character vector of length 1
#' @param coef.indices Either NULL, or a numeric vector of length at least 3
#' @param dispersion An optional scalar multiplier for the covariance matrix, to cope with overdispersion for example
#' @param ... Other arguments to pass to qvcalc.default
#' @description Computes a set of quasi variances (and corresponding quasi standard errors) 
#' for estimated model coefficients relating to the levels of a categorical (i.e., factor) 
#' explanatory variable. For details of the method see Firth (2000), Firth (2003) or Firth 
#' and de Menezes (2004). Quasi variances generalize and improve the accuracy of “floating 
#' absolute risk” (Easton et al., 1991). This device for economical model summary was first 
#' suggested by Ridout (1989).\cr
#' Modified from qvcalc.lm() of packages qvcalc by David Firth, d.firth@@warwick.ac.uk
#' @references Easton, D. F, Peto, J. and Babiker, A. G. A. G. (1991) Floating absolute risk: an alternative to relative risk in survival and case-control analysis avoiding an arbitrary reference group. Statistics in Medicine 10, 1025–1035.
#' @references Firth, D. (2000) Quasi-variances in Xlisp-Stat and on the web. Journal of Statistical Software 5.4, 1–13. At http://www.jstatsoft.org
#' @references Firth, D. (2003) Overcoming the reference category problem in the presentation of statistical models. Sociological Methodology 33, 1–18.
#' @references Firth, D. and de Mezezes, R. X. (2004) Quasi-variances. Biometrika 91, 65–80.
#' @references McCullagh, P. and Nelder, J. A. (1989) Generalized Linear Models. London: Chapman and Hall.
#' @references Menezes, R. X. de (1999) More useful standard errors for group and factor effects in generalized linear models. D.Phil. Thesis, Department of Statistics, University of Oxford.
#' @references Ridout, M.S. (1989). Summarizing the results of fitting generalized linear models to data from designed experiments. In: Statistical Modelling: Proceedings of GLIM89 and the 4th International Workshop on Statistical Modelling held in Trento, Italy, July 17–21, 1989 (A. Decarli et al., eds.), pp 262–269. New York: Springer.
#' @examples
#' \dontrun{
#' x <- rnorm(100)
#' y <- rnorm(100)
#' G <- as.factor(sample(c("A", "B", "C", "D"), 100, replace = TRUE))
#' R <- as.factor(rep(1:25, 4))
#' library(lme4)
#' m <- lmer(y ~ x + G + (1 | R))
#' qvlmer(m, factorname="G")
#' }
#' @export

qvlmer <- function (object, factorname = NULL, coef.indices = NULL, dispersion = NULL, 
          ...) {
  
  fixef <- getFromNamespace("fixef", ns="lme4")
  
  coef.indices.saved <- coef.indices
  if (is.null(factorname) && is.null(coef.indices)) {
    stop("arguments 'factorname' and 'coef.indices' are both NULL")
  }
  
  qvcd <- function (object, factorname = NULL, coef.indices = NULL, labels = NULL, 
            dispersion = NULL, estimates = NULL, modelcall = NULL, ...) 
  {
    covmat <- object
    if (!is.null(labels)) 
      rownames(covmat) <- colnames(covmat) <- labels
    n <- dim(covmat)[1]
    if (n <= 2) 
      stop("qvcalc works only for factors with 3 or more levels")
    simple.contrasts <- function(n, levelnames = 1:n) {
      result <- list()
      for (i in 1:(n - 1)) {
        for (j in (i + 1):n) {
          result[[paste(levelnames[i], levelnames[j], sep = ",")]] <- c(i, 
                                                                        j)
        }
      }
      result
    }
    qvdesign <- function(n) {
      nrows <- choose(n, 2)
      m <- matrix(0, nrows, n)
      indices <- simple.contrasts(n)
      for (i in 1:nrows) {
        m[i, indices[[i]][1]] <- 1
        m[i, indices[[i]][2]] <- 1
      }
      m
    }
    level <- qvdesign(n)
    contrast.variance <- function(contrast, covmat) {
      if (!(is.matrix(covmat) && (dim(covmat)[1] == dim(covmat)[2]))) 
        stop("covmat must be a square matrix")
      n <- dim(covmat)[1]
      if (length(contrast) == n && sum(contrast) == 0) 
        return(as.vector(contrast %*% covmat %*% contrast))
      if (length(contrast) == 2 && all(contrast %in% 1:n)) {
        i <- contrast[1]
        j <- contrast[2]
        return(covmat[i, i] + covmat[j, j] - 2 * covmat[i, 
                                                        j])
      } else stop("invalid contrast")
    }
    simple.contrast.variances <- function(n, covmat) {
      if (!is.null(rownames(covmat))) 
        levelnames <- rownames(covmat) else levelnames <- 1:n
      sapply(simple.contrasts(n, levelnames), function(contrast) {
        contrast.variance(contrast, covmat)
      })
    }
    response <- simple.contrast.variances(n, covmat)
    if (any(response <= 0)) {
      stop("not all contrasts have positive variance")
    } else response <- log(response)
    expLinear <- structure(list(family = "expLinear", link = "exp", 
                                linkfun = function(mu) exp(mu), linkinv = function(eta) log(eta), 
                                variance = function(mu) rep(1, length(mu)), dev.resids = function(y, 
                                                                                                  mu, wt) wt * ((y - mu)^2), aic = function(y, n, mu, 
                                                                                                                                            wt, dev) sum(wt) * (log(dev/sum(wt) * 2 * pi) + 1) + 
                                  2, mu.eta = function(eta) 1/eta, initialize = expression({
                                    n <- rep(1, nobs)
                                    mustart <- y
                                  }), validmu = function(mu) TRUE), class = "family")
    model <- glm(response ~ 0 + level, family = expLinear)
    qv <- coef(model)
    NAs <- rep(NA, length(qv))
    if (!is.null(rownames(covmat))) names(qv) <- rownames(covmat)
    frame <- data.frame(estimate = NAs, SE = sqrt(diag(covmat)), 
                        quasiSE = sqrt(qv), quasiVar = qv, row.names = names(qv))
    if (!is.null(estimates)) frame$estimate <- estimates
    relerrs <- sqrt(exp(-residuals(model))) - 1
    names(relerrs) <- names(response)
    return(structure(list(covmat = covmat, qvframe = frame, dispersion = dispersion, 
                          relerrs = relerrs, factorname = factorname, coef.indices = coef.indices, 
                          modelcall = modelcall), class = "qv"))
  }
  
  
  if (is.null(coef.indices)) {
    term.index <- which(attr(terms(object), "term.labels") == 
                          factorname)
    modelmat <- model.matrix(object)
    
    has.coef <- colnames(modelmat) %in% names(fixef(object))
    
    coef.indices <- which(attr(modelmat, "assign")[has.coef] == 
                            term.index)
    xlevels <- list()
    
    for (r in 1:ncol(object@frame)) 
      if ((is.factor(object@frame[, r])) & (colnames(object@frame)[r] %in% names(attr(object@pp$X, "contrasts")))) {
        l <- list(levels(object@frame[, r]))
        names(l) <- colnames(object@frame)[r]
        xlevels <- c(xlevels, l)
      }
    
    if (length(xlevels[[factorname]]) == length(coef.indices)) {
      contmat <- diag(length(coef.indices))
    } else {
      # contmat <- diag(length(xlevels[[factorname]]))
      # rownames(contmat) <- colnames(contmat) <- xlevels[[factorname]]
      contmat <- eval(call(attr(object@pp$X,"contrasts")[[factorname]], xlevels[[factorname]]))
    }
    
    estimates <- contmat %*% fixef(object)[coef.indices]
    covmat <- vcov(object, dispersion = dispersion)
    covmat <- covmat[coef.indices, coef.indices, drop = FALSE]
    covmat <- contmat %*% covmat %*% t(contmat)
  } else {
    k <- length(coef.indices)
    refPos <- numeric(0)
    if (0 %in% coef.indices) {
      refPos <- which(coef.indices == 0)
      coef.indices <- coef.indices[-refPos]
    }
    covmat <- vcov(object, dispersion = dispersion)
    covmat <- covmat[coef.indices, coef.indices, drop = FALSE]
    estimates <- coef(object)[coef.indices]
    if (length(refPos) == 1) {
      if (length(estimates) != k) 
        estimates <- c(0, estimates)
      covmat <- cbind(0, rbind(0, covmat))
      names(estimates)[1] <- rownames(covmat)[1] <- colnames(covmat)[1] <- "(reference)"
      if (refPos != 1) {
        if (refPos == k) {
          perm <- c(2:k, 1)
        } else {
          perm <- c(2:refPos, 1, (refPos + 1):k)
        }
        estimates <- estimates[perm]
        covmat <- covmat[perm, perm, drop = FALSE]
      }
    }
  }
  
  # print(covmat)
  # print(dim(covmat))
  
  return(qvcd(object=as.matrix(covmat), factorname = factorname, coef.indices = coef.indices.saved, 
                        labels = NULL, dispersion = dispersion, estimates = estimates, 
                        modelcall = object@call, ...))
}

#' IC_threshold_matrix calculates correlation matrix thresholed by partial correlation
#' @title Calculate correlation matrix
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list
#' @param data A dataframe or an IconoCorel object from a previous run of IC_threshold_matrix
#' @param threshold threshold for partial and full correlations
#' @param use an optional character string giving a method for computing covariances in the presence of missing values. This must be (an abbreviation of) one of the strings "everything", "all.obs", "complete.obs", "na.or.complete", or "pairwise.complete.obs".
#' @param method a character string indicating which correlation coefficient (or covariance) is to be computed. One of "pearson" (default), "kendall", or "spearman": can be abbreviated.
#' @param model a character string indicating if linear model uses all variables at a time (AAT) or one at a time (OAT).
#' @param significance.level if FALSE, does not use significance level; or use this significance level.
#' @param correction.multiple.comparisons "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", or "none".
#' @param progress show a progress bar
#' @param debug display information about progression of computing
#' @description This function calculates the matrix of correlations thresholded using partial correlation.\cr
#' If the threshold is not given, the object that is produced can be used later for thresholding.\cr
#' For model OAT: The link between A and B is “remarkable” if and only if the total correlation between them is higher than a given threshold and if the partial correlation between A and B in respect to any other variable C is also higher in absolute values than this threshold and with the same sign as the total correlation.
#' For model AAT: A correlation is retained if it is higher than the threshold and the partial correlation is lower than the threshold. In this case, no missing value is accepted.\cr
#' The use and method parameters are used by cor() function. The function uses by default a parallel computing in Unix or MacOSX systems. 
#' If progress is TRUE and the package pbmcapply is present, a progress bar is displayed. If debug is TRUE, some informations are shown during the process but parallel computing is not used.\cr
#' \code{https://fr.wikipedia.org/wiki/Iconographie_des_corrélations}
#' @family Iconography of correlations
#' @references Lesty, M., 1999. Une nouvelle approche dans le choix des régresseurs de la régression multiple en présence d’interactions et de colinéarités. Revue de Modulad 22, 41-77.
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' es <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
#'                  Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
#'                  Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
#'                  Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
#'                  Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
#'                  row.names = c(NA, -8L), class = "data.frame")
#' 
#' es
#' 
#' df_clean <- IC_clean_data(es, debug = TRUE)
#' cor_matrix <- IC_threshold_matrix(data=df_clean, threshold = NULL, progress=FALSE)
#' cor_threshold <- IC_threshold_matrix(data=df_clean, threshold = 0.3)
#' plot(cor_threshold, show.legend.strength=FALSE, show.legend.direction = FALSE)
#' cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
#' plot(cor_threshold_Note)
#' 
#' cor_threshold <- IC_threshold_matrix(data=df_clean, threshold = 0.8, progress=FALSE)
#' gr <- plot(cor_threshold, plot=FALSE)
#' ly <- getFromNamespace("layout_nicely", ns="igraph")(gr)
#' plot(cor_threshold, 
#' layout=matrix(data=c(53, 53, 55, 55, 
#'                      55, 53, 55, 53), ncol=2, byrow=FALSE), 
#' show.legend.direction = FALSE,
#' show.legend.strength = FALSE, xlim=c(-2, 2), ylim=c(-2, 2))
#' 
#' # Using significance level
#' 
#' cor_threshold <- IC_threshold_matrix(data=df_clean, 
#'                                      significance.level=0.05, debug=TRUE)
#' plot(cor_threshold, show.legend.strength=FALSE, show.legend.direction = FALSE)
#' cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
#' plot(cor_threshold_Note)
#' 
#' # Using the model All at a time
#' 
#' cor_threshold_AAT <- IC_threshold_matrix(data=df_clean, threshold = 0.3, model="AAT")
#' par(mar=c(1,1,1,1))
#' set.seed(4)
#' plot(cor_threshold_AAT, show.legend.strength="bottomleft")
#' 
#' 
#' 
#' ############
#' dta <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
#'                      Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
#'                      Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
#'                      Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
#'                      Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
#'                      row.names = c(NA, -8L), class = "data.frame")
#'
#' dta0 <- dta[, 2:ncol(dta)]
#' ic0 <- IC_threshold_matrix(data = dta0)
#' cor_threshold <- IC_threshold_matrix(data=ic0, threshold = 0.3)
#' par(mar=c(1,1,1,1))
#' set.seed(4)
#' library("igraph")
#' 
#' plot(cor_threshold, vertex.color="red", show.legend.strength = FALSE)
#' plot(IC_correlation_simplify(matrix=cor_threshold), 
#'      show.legend.strength = FALSE, show.legend.direction = FALSE)
#' 
#' }
#' @export


IC_threshold_matrix <- function(data=stop("A dataframe or an IconoCorel object is required")     , 
                                threshold=NULL                                                   , 
                                use = c("pairwise.complete.obs", "everything", 
                                        "all.obs", "complete.obs", "na.or.complete")             , 
                                method=c("pearson", "kendall", "spearman")                       ,
                                model=c("OAT", "ATT")                                            ,
                                significance.level=FALSE                                         ,
                                correction.multiple.comparisons="fdr"                            ,
                                progress=TRUE                                                    , 
                                debug=FALSE                                                      ) {
  
  
  
  
  if ((is.null(threshold)) & (!identical(significance.level, FALSE))) {
    if (debug) message("Threshold is fixed to 0 because significance.level is set up.")
    threshold <- 0
  }
  
  if (inherits(data, "IconoCorel")) {
    cor_mat <- data$correlation
    p_cor_mat <- data$correlation.pvalue
    cor_mat_threshold <- data$thresholded_correlation 
    cor_seuil_binary <- data$thresholded_correlation_binary 
    cor_seuil <- data$threshold_matrix
    
  } else {
    
    if (!(is.element("ppcor", installed.packages()[,1]))) {
      
      ginv <- function (X, tol = sqrt(.Machine$double.eps)) 
      {
        if (length(dim(X)) > 2L || !(is.numeric(X) || is.complex(X))) 
          stop("'X' must be a numeric or complex matrix")
        if (!is.matrix(X)) 
          X <- as.matrix(X)
        Xsvd <- svd(X)
        if (is.complex(X)) 
          Xsvd$u <- Conj(Xsvd$u)
        Positive <- Xsvd$d > max(tol * Xsvd$d[1L], 0)
        if (all(Positive)) 
          Xsvd$v %*% (1/Xsvd$d * t(Xsvd$u))
        else if (!any(Positive)) 
          array(0, dim(X)[2L:1L])
        else Xsvd$v[, Positive, drop = FALSE] %*% ((1/Xsvd$d[Positive]) * 
                                                     t(Xsvd$u[, Positive, drop = FALSE]))
      }
      
      pcor <- function (x, method = c("pearson", "kendall", "spearman")) 
      {
        method <- match.arg(method)
        if (is.data.frame(x)) 
          x <- as.matrix(x)
        if (!is.matrix(x)) 
          stop("supply a matrix-like 'x'")
        if (!(is.numeric(x) || is.logical(x))) 
          stop("'x' must be numeric")
        stopifnot(is.atomic(x))
        n <- dim(x)[1]
        gp <- dim(x)[2] - 2
        cvx <- cov(x, method = method)
        if (det(cvx) < .Machine$double.eps) {
          warning("The inverse of variance-covariance matrix is calculated using Moore-Penrose generalized matrix invers due to its determinant of zero.")
          icvx <- ginv(cvx)
        }
        else icvx <- solve(cvx)
        pcor <- -cov2cor(icvx)
        diag(pcor) <- 1
        if (method == "kendall") {
          statistic <- pcor/sqrt(2 * (2 * (n - gp) + 5)/(9 * (n - 
                                                                gp) * (n - 1 - gp)))
          p.value <- 2 * pnorm(-abs(statistic))
        }
        else {
          statistic <- pcor * sqrt((n - 2 - gp)/(1 - pcor^2))
          p.value <- 2 * pt(-abs(statistic), (n - 2 - gp))
        }
        diag(statistic) <- 0
        diag(p.value) <- 0
        list(estimate = pcor, p.value = p.value, statistic = statistic, 
             n = n, gp = gp, method = method)
      }
    } else {
      pcor <- getFromNamespace("pcor", ns="ppcor")
    }
    
    model <- match.arg(model, choices = c("OAT", "ATT"))
    method <- match.arg(method, choices = c("pearson", "kendall", "spearman"))
    use <- match.arg(use, choices = c("pairwise.complete.obs", "everything", 
                                      "all.obs", "complete.obs", "na.or.complete"))
    # Là j'ai la matrice des corrélations
    cor_mat <- cor(data, method=method, use=use)
    
    if (use == "pairwise.complete.obs") {
    
    n <- nrow(data)
    daf <- expand.grid(e=1:ncol(data), f=1:ncol(data))
    n <- sapply(1:nrow(daf), FUN=function(x) {
      sum(as.numeric((!is.na(data[, daf[x, 1]])) & (!is.na(data[, daf[x, 2]]))))
    })
    
    n_cor_mat <- cor_mat
    n_cor_mat[] <- NA
    for (x in 1:nrow(daf)) n_cor_mat[daf[x, 1], daf[x, 2]] <- n[x]
    
    statistic <- cor_mat * sqrt((n_cor_mat - 2)/(1 - cor_mat^2))
    p_cor_mat <- 2 * pt(-abs(statistic), (n_cor_mat - 2))
    
    daf <- expand.grid(e=1:(ncol(data)-1), f=2:ncol(data))
    daf <- daf[(daf[, 1]<daf[, 2]), ]
    
    ppp <- NULL
    for (x in 1:nrow(daf)) ppp <- c(ppp, p_cor_mat[daf[x, 1], daf[x, 2]])
    
    ppp <- p.adjust(ppp, method = correction.multiple.comparisons, 
             n=length(ppp))
    
    
    for (x in 1:nrow(daf)) {
      p_cor_mat[daf[x, 1], daf[x, 2]] <- ppp[x]
      p_cor_mat[daf[x, 2], daf[x, 1]] <- ppp[x]
    }
    } else {
      p_cor_mat <- NULL
    }
   
    
    if (model == "OAT") {
      
      cname <- colnames(data)
      lc <- length(cname)
      llc <- 1:lc
      
      lapp <- lapply
      if (is.element("parallel", installed.packages()[,1])) lapp <- getFromNamespace("mclapply", ns="parallel")
      if (progress & (is.element("pbmcapply", installed.packages()[,1]))) lapp <- getFromNamespace("pbmclapply", ns="pbmcapply")
      if (debug) lapp <- lapply
      
      options(mc.cores = getFromNamespace("detectCores", ns="parallel")())
      
      daf <- expand.grid(e=1:(lc-2), f=2:(lc-1), g=3:lc)
      daf <- daf[(daf[, 1]<daf[, 2]) & (daf[, 2]<daf[, 3]), ]
      
      
      
      zu <- lapp(1:nrow(daf), FUN = function(x) {
        
        e <- daf[x, 1]
        f <- daf[x, 2]
        g <- daf[x, 3]
        
        dfg <- na.omit(data[, c(e, f, g)])
        pc <- try(suppressWarnings(pcor(dfg, method=method)), 
                  silent=TRUE)
        outg <- (inherits(pc, "try-error"))
        if (!outg) outg <- any(!is.finite(pc$estimate))
        if (!outg) {
          return(c(1, pc$estimate[1, 2], pc$estimate[1, 3], pc$estimate[2, 3]))
        } else {
          return(0)
        }
      }
      )
      
      cor_partial <- array(data=NA, dim=c(lc, lc, lc), dimnames = list(cname, cname, cname))
      for(x in which(sapply(zu, FUN = function(z) z[1]==1))) {
        e <- daf[x, 1]
        f <- daf[x, 2]
        g <- daf[x, 3]
        cor_partial[e, f, g] <- cor_partial[f, e, g] <- zu[[x]][2]
        cor_partial[e, g, f] <- cor_partial[g, e, f] <- zu[[x]][3]
        cor_partial[f, g, e] <- cor_partial[g, f, e] <- zu[[x]][4]
      }
      
      
      # The link between A and B is “remarkable” if and only if the total correlation 
      # between them is higher than a given threshold and if the partial correlation 
      # between A and B in respect to any other variable C is also higher in absolute 
      # values than this threshold and with the same sign as the total correlation.
      cor_partial_2D <- cor_mat
      daf <- expand.grid(e=1:(lc-1), f=2:lc)
      daf <- daf[(daf[, 1]<daf[, 2]), ]
      for (x in 1:nrow(daf)) {
        cor_point <- cor_mat[daf[x, 1], daf[x, 2]]
        pcor_x <- na.omit(cor_partial[daf[x, 1], daf[x, 2], ])
        if (sign(cor_point) > 0) {
          pos <- which.min(pcor_x)
          
          cor_partial_2D[daf[x, 1], daf[x, 2]] <- pcor_x[pos]
          cor_partial_2D[daf[x, 2], daf[x, 1]] <- pcor_x[pos]
          
        } else {
          pos <- which.max(pcor_x)
          
          cor_partial_2D[daf[x, 1], daf[x, 2]] <- pcor_x[pos]
          cor_partial_2D[daf[x, 2], daf[x, 1]] <- pcor_x[pos]
          
        }
        
      }
      cor_seuil <- cor_partial_2D
      diag(cor_seuil) <- 0
    } else {
      
      cor_seuil <- pcor(data, method=method)$estimate
      diag(cor_seuil) <- 0
    }
    
  }
  
  if (!is.null(threshold)) {
    # Pour éviter les redondances, le lien AB est tracé si et seulement si la 
    # corrélation totale r(A,B) est supérieure au seuil en valeur absolue, 
    # et si les corrélations partielles r(A,B), par rapport à une variable Z, 
    # sont supérieures au seuil, en valeur absolue, et de même signe que la 
    # corrélation totale, pour tout Z parmi les variables disponibles, y 
    # compris les « instants ».
    # cor_seuil_binary <- (abs(cor_seuil)>threshold) & (abs(cor_mat)>threshold) & (sign(cor_mat) == sign(cor_seuil))
    cor_seuil_binary <- (abs(cor_seuil)>threshold) & (abs(cor_mat)>threshold)
    cor_mat_threshold <- cor_mat * as.numeric(cor_seuil_binary)
    if (!identical(significance.level, FALSE)) {
      if (is.null(p_cor_mat)) {
        message("Threshold by significance level is not available for this 'use' parameter.")
      } else {
      cor_seuil_binary_plevel <- (p_cor_mat <= significance.level)
      cor_seuil_binary <- cor_seuil_binary & cor_seuil_binary_plevel
      cor_mat_threshold <- cor_mat * as.numeric(cor_seuil_binary)
      }
    }
  } else {
    cor_seuil_binary <- NULL
    cor_mat_threshold <- NULL
  }
  
  
  
  out <- list(correlation=cor_mat, 
              correlation.pvalue=p_cor_mat, 
              thresholded_correlation=cor_mat_threshold, 
              thresholded_correlation_binary=cor_seuil_binary, 
              threshold_matrix=cor_seuil, 
              model=model, 
              use=use, 
              method=method, 
              threshold=threshold)
  out <- addS3Class(out, "IconoCorel")
  
  return(out)
}

#' cutter returns the fitted distribution without cut
#' @title Distribution of the fitted distribution without cut. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The parameters of distribution of values below or above the detection limit.
#' @param observations The observations; see description
#' @param observations.colname If observations is a data.frame, the name of column with observations
#' @param lower_detection_limit.colname If observations is a data.frame, the name of column with lower detection limit
#' @param upper_detection_limit.colname If observations is a data.frame, the name of column with upper detection limit
#' @param cut_method.colname If observations is a data.frame, the name of column with cut method, being "censored" or "truncated"
#' @param par Initial values for parameters of distribution 
#' @param lower_detection_limit Value for lower detection limit 
#' @param upper_detection_limit Value for upper detection limit
#' @param cut_method Value for cut method, being "censored" or "truncated"
#' @param distribution Can be gamma, normal, weibull, lognormal, or generalized.gamma
#' @param n.iter Number of iteration for Bayesian MCMC and to estimate the goodness-of-fit
#' @param n.adapt Number of burn-in iterations Bayesian MCMC
#' @param n.mixture Number of distributions
#' @param priors A dataframe with priors. 
#' @param adaptive Should the adaptive methodologie for SDprop be used
#' @param debug If TRUE, show some information
#' @param progress.bar If TRUE, show a progress bar for MCMC
#' @param session The session of a shiny process
#' @description If observations is a data.frame, it can have 4 columns:\cr
#' A column for the measurements;\cr
#' A column for the lower detection limit;\cr
#' A column for the upper detection limit;\cr
#' A column for the truncated of censored nature of the data.\cr
#' The names of the different columns are in the observations.colname, lower_detection_limit.colname, 
#' upper_detection_limit.colname and cut_method.colname.\cr
#' If lower_detection_limit.colname is NULL or if the column does not exist, 
#' the data are supposed to not be left-cut 
#' and if upper_detection_limit.colname is NULL or if the column does not exist, 
#' the data are supposed to not be right-cut.\cr
#' If observations is a vector, then the parameters lower_detection_limit and/or upper_detection_limit 
#' must be given. Then cut_method must be also provided.\cr
#' In abservations, -Inf must be used to indicate a value below the lower detection limit and +Inf must be used 
#' for a value above the upper detection limit.\cr
#' Be careful: NA is used to represent a missing data and not a value below of above the detection limit.\cr
#' If lower_detection_limit, upper_detection_limit or cut_method are only one value, they are supposed to 
#' be used for all the observations.\cr
#' Definitions for censored or truncated distribution vary, and the two terms are sometimes used interchangeably. Let the following data set:\cr
#' 1 1.25 2 4 5\cr
#' \cr
#' Censoring: some observations will be censored, meaning that we only know that they are below (or above) some bound. This can for instance occur if we measure the concentration of a chemical in a water sample. If the concentration is too low, the laboratory equipment cannot detect the presence of the chemical. It may still be present though, so we only know that the concentration is below the laboratory's detection limit.\cr
#' \cr
#' If the detection limit is 1.5, so that observations that fall below this limit is censored, our example data set would become:\cr
#' <1.5 <1.5 2 4 5; that is, we don't know the actual values of the first two observations, but only that they are smaller than 1.5.\cr
#' \cr
#' Truncation: the process generating the data is such that it only is possible to observe outcomes above (or below) the truncation limit. This can for instance occur if measurements are taken using a detector which only is activated if the signals it detects are above a certain limit. There may be lots of weak incoming signals, but we can never tell using this detector.\cr
#' \cr
#' If the truncation limit is 1.5, our example data set would become: 2 4 5; and we would not know that there in fact were two signals which were not recorded.\cr
#' If n.iter is NULL, no Bayesian MCMC is performed but credible interval will not be available.
#' @family Distributions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' # _______________________________________________________________
#' # right censored distribution with gamma distribution
#' # _______________________________________________________________
#' # Detection limit
#' DL <- 100
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc>DL] <- +Inf
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, upper_detection_limit=DL, 
#'                            cut_method="censored")
#' result
#' plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10), col.mcmc=NULL)
#' plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # The same data seen as truncated data with gamma distribution
#' # _______________________________________________________________
#' obc <- obc[is.finite(obc)]
#' # search for the parameters the best fit these truncated data
#' result <- cutter(observations=obc, upper_detection_limit=DL, 
#'                            cut_method="truncated")
#' result
#' plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # left censored distribution with gamma distribution
#' # _______________________________________________________________
#' # Detection limit
#' DL <- 10
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc<DL] <- -Inf
#' # search for the parameters the best fit these truncated data
#' result <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored")
#' result
#' plot(result, xlim=c(0, 200), breaks=seq(from=0, to=300, by=10))
#' # _______________________________________________________________
#' # left censored distribution with mixture of gamma distribution
#' # _______________________________________________________________
#' #' # Detection limit
#' library(HelpersMG)
#' # Generate 200 random data from a gamma distribution
#' set.seed(1234)
#' obc <- c(rgamma(100, scale=10, shape=5), rgamma(100, scale=20, shape=10))
#' LDL <- 20
#' l <- seq(from=0, to=LDL, length.out=1001)
#' p <- pgamma(l, scale=10, shape=5)*0.5+pgamma(l, scale=20, shape=10)
#' deltal <- l[2]-l[1]
#' expected_LDL <- sum((l[-1]-deltal/2)*(p[-1]-p[-length(p)]))/sum((p[-1]-p[-length(p)]))
#' # remove the data below the detection limit
#' obc[obc<LDL] <- -Inf
#' 
#' UDL <- 300
#' l <- seq(from=UDL, to=1000, length.out=1001)
#' p <- pgamma(l, scale=10, shape=5)*0.5+pgamma(l, scale=20, shape=10)
#' deltal <- l[2]-l[1]
#' expected_UDL <- sum((l[-1]-deltal/2)*(p[-1]-p[-length(p)]))/sum((p[-1]-p[-length(p)]))
#' obc[obc>UDL] <- +Inf
#' 
#' # search for the parameters the best fit these truncated data
#' result1_gamma <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="gamma", 
#'                           cut_method="censored", n.iter=5000, debug=0)
#' result1_normal <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="normal", 
#'                           cut_method="censored", n.iter=5000)
#' result1_lognormal <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="lognormal", 
#'                           cut_method="censored", n.iter=5000)
#' result1_Weibull <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL,  
#'                           distribution="Weibull", 
#'                           cut_method="censored", n.iter=5000)
#' result1_generalized.gamma <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="generalized.gamma", 
#'                           cut_method="censored", n.iter=5000)
#' result2_gamma <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="gamma", 
#'                           n.mixture=2, 
#'                           cut_method="censored", n.iter=5000)
#' result2_normal <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="normal", 
#'                           n.mixture=2, 
#'                           cut_method="censored", n.iter=5000)
#' result2_lognormal <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="lognormal", 
#'                           n.mixture=2, 
#'                           cut_method="censored", n.iter=5000)
#' result2_Weibull <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="Weibull", 
#'                           n.mixture=2, 
#'                           cut_method="censored", n.iter=5000)
#' result2_generalized.gamma <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                           upper_detection_limit = UDL, 
#'                           distribution="generalized.gamma", 
#'                           n.mixture=2, 
#'                           cut_method="censored", n.iter=5000)
#'                           
#' compare_AIC(nomixture.gamma=result1_gamma, 
#'              nomixture.normal=result1_normal, 
#'              nomixture.lognormal=result1_lognormal, 
#'              nomixture.Weibull=result1_Weibull, 
#'              nomixture.generalized.gamma=result1_generalized.gamma, 
#'              mixture.gamma=result2_gamma, 
#'              mixture.normal=result2_normal, 
#'              mixture.lognormal=result2_lognormal, 
#'              mixture.Weibull=result2_Weibull, 
#'              mixture.generalized.gamma=result2_generalized.gamma)
#'              
#' plot(result2_gamma, xlim=c(0, 600), breaks=seq(from=0, to=600, by=10))
#' plot(result2_generalized.gamma, xlim=c(0, 600), breaks=seq(from=0, to=600, by=10))
#' 
#' # _______________________________________________________________
#' # left and right censored distribution
#' # _______________________________________________________________
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # Detection limit
#' LDL <- 10
#' # remove the data below the detection limit
#' obc[obc<LDL] <- -Inf
#' # Detection limit
#' UDL <- 100
#' # remove the data below the detection limit
#' obc[obc>UDL] <- +Inf
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, lower_detection_limit=LDL, 
#'                            upper_detection_limit=UDL, 
#'                           cut_method="censored")
#' result
#' plot(result, xlim=c(0, 150), col.DL=c("black", "grey"), 
#'                              col.unobserved=c("green", "blue"), 
#'      breaks=seq(from=0, to=150, by=10))
#' # _______________________________________________________________
#' # Example with two values for lower detection limits
#' # corresponding at two different methods of detection for example
#' # with gamma distribution
#' # _______________________________________________________________
#' obc <- rgamma(50, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL1 <- 10
#' # remove the data below the detection limit
#' obc[obc<LDL1] <- -Inf
#' obc2 <- rgamma(50, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL2 <- 20
#' # remove the data below the detection limit
#' obc2[obc2<LDL2] <- -Inf
#' obc <- c(obc, obc2)
#' # search for the parameters the best fit these censored data
#' result <- cutter(observations=obc, 
#'                            lower_detection_limit=c(rep(LDL1, 50), rep(LDL2, 50)), 
#'                           cut_method="censored")
#' result
#' # It is difficult to choose the best set of colors
#' plot(result, xlim=c(0, 150), col.dist="red", 
#'      col.unobserved=c(rgb(red=1, green=0, blue=0, alpha=0.1), 
#'                       rgb(red=1, green=0, blue=0, alpha=0.2)), 
#'      col.DL=c(rgb(red=0, green=0, blue=1, alpha=0.5), 
#'                       rgb(red=0, green=0, blue=1, alpha=0.9)), 
#'      breaks=seq(from=0, to=200, by=10))
#'      
#' # ________________________________________________________________________
#' # left censored distribution comparison of normal, lognormal, 
#' # weibull, generalized gamma, and gamma without Bayesian MCMC
#' # Comparison with Akaike Information Criterion
#' # ________________________________________________________________________
#' # Detection limit
#' DL <- 10
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc<DL] <- -Inf
#' 
#' result_gamma <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="gamma", 
#'                           n.iter=NULL)
#' plot(result_gamma)
#' 
#' result_lognormal <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="lognormal", 
#'                           n.iter=NULL)
#' plot(result_lognormal)
#' 
#' result_weibull <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="weibull", 
#'                           n.iter=NULL)
#' plot(result_weibull)
#' 
#' result_normal <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="normal", 
#'                           n.iter=NULL)
#' plot(result_normal)
#' 
#' result_generalized.gamma <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="generalized.gamma", 
#'                           n.iter=NULL)
#' plot(result_generalized.gamma)
#' 
#' compare_AIC(gamma=result_gamma, 
#'             lognormal=result_lognormal, 
#'             normal=result_normal, 
#'             Weibull=result_weibull, 
#'             Generalized.gamma=result_generalized.gamma)
#' 
#' # ______________________________________________________________________________
#' # left censored distribution comparison of normal, lognormal, 
#' # weibull, generalized gamma, and gamma
#' # ______________________________________________________________________________
#' # Detection limit
#' DL <- 10
#' # Generate 100 random data from a gamma distribution
#' obc <- rgamma(100, scale=20, shape=2)
#' # remove the data below the detection limit
#' obc[obc<DL] <- -Inf
#' # search for the parameters the best fit these truncated data
#' result_gamma <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="gamma")
#' result_gamma
#' plot(result_gamma, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' result_lognormal <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="lognormal")
#' result_lognormal
#' plot(result_lognormal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' result_weibull <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="weibull")
#' result_weibull
#' plot(result_weibull, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' result_normal <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="normal")
#' result_normal
#' plot(result_normal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' result_generalized.gamma <- cutter(observations=obc, lower_detection_limit=DL, 
#'                           cut_method="censored", distribution="generalized.gamma")
#' result_generalized.gamma
#' plot(result_generalized.gamma, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))
#' 
#' # ___________________________________________________________________
#' # Test for similarity in gamma left censored distribution between two
#' # datasets
#' # ___________________________________________________________________
#' obc1 <- rgamma(100, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL <- 10
#' # remove the data below the detection limit
#' obc1[obc1<LDL] <- -Inf
#' obc2 <- rgamma(100, scale=10, shape=2)
#' # remove the data below the detection limit
#' obc2[obc2<LDL] <- -Inf
#' # search for the parameters the best fit these censored data
#' result1 <- cutter(observations=obc1, 
#'                   distribution="gamma", 
#'                   lower_detection_limit=LDL, 
#'                   cut_method="censored", n.iter=NULL)
#' logLik(result1)
#' plot(result1, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#' result2 <- cutter(observations=obc2, 
#'                   distribution="gamma", 
#'                   lower_detection_limit=LDL, 
#'                   cut_method="censored", n.iter=NULL)
#' logLik(result2)
#' plot(result2, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#' result_totl <- cutter(observations=c(obc1, obc2), 
#'                       distribution="gamma", 
#'                       lower_detection_limit=LDL, 
#'                       cut_method="censored", n.iter=NULL)
#' logLik(result_totl)
#' plot(result_totl, xlim=c(0, 200), 
#'      breaks=seq(from=0, to=200, by=10))
#'      
#' compare_AIC(Separate=list(result1, result2), 
#'             Common=result_totl, factor.value=1)
#' compare_BIC(Separate=list(result1, result2), 
#'             Common=result_totl, factor.value=1)           
#' }
#' @export

cutter <- function(observations=stop("Observations must be provided"), 
                   observations.colname="Observations", 
                   lower_detection_limit.colname="LDL", 
                   upper_detection_limit.colname="UDL", 
                   cut_method.colname="Cut", 
                   par=NULL, 
                   lower_detection_limit=NULL, 
                   upper_detection_limit=NULL, 
                   cut_method="censored", 
                   distribution="gamma", 
                   n.mixture=1, 
                   n.iter=5000, n.adapt=100, debug=FALSE, progress.bar=TRUE, 
                   priors=NULL, 
                   adaptive=TRUE, 
                   session=NULL) {
  
  # observations=NULL 
  # observations.colname="Observations" 
  # lower_detection_limit.colname="LDL" 
  # upper_detection_limit.colname="UDL" 
  # cut_method.colname="Cut" 
  # par=NULL 
  # lower_detection_limit=NULL 
  # upper_detection_limit=NULL 
  # cut_method="censored" 
  # n.iter=5000
  # n.mixture=1
  # distribution="gamma"
  # debug <- FALSE
  # progress.bar=TRUE
  # n.iter=5000
  # n.adapt=100
  # debug=FALSE
  # progress.bar=TRUE
  # session=NULL
  
  # observations=obc
  # lower_detection_limit=LDL 
  # upper_detection_limit = UDL 
  # distribution="generalized.gamma" 
  # n.mixture=2 
  # cut_method="truncated"
  # n.iter=5000
  # debug = 1
  
  # observations = Ce
  # lower_detection_limit = 0.001
  # cut_method = "censored"
  # n.iter = 5000
  # n.mixture = 2
  # distribution = "gamma"
  
  fitn <- function(par, observations=NULL, distribution="gamma", 
                   n.mixture=NULL, debug=FALSE, limits.lower=NULL, 
                   limits.upper=NULL, log = TRUE) {
    - dcutter(par=par, observations=observations, distribution=distribution, 
              n.mixture=n.mixture, debug=debug, limits.lower=limits.lower, 
              limits.upper=limits.upper, log = log)
  }
  
  getparcutter <- getFromNamespace(".getparcutter", ns="HelpersMG")
  
  distribution <- tolower(distribution)
  distribution <- match.arg(distribution, choices = c("gamma", "normal", "lognormal", "weibull", "generalized.gamma"))
  
  
  if (is.data.frame(observations)) {
    if (all(colnames(observations) != observations.colname)) {
      stop("No observations are found in data.frame")
    }
    if (all(colnames(observations) != lower_detection_limit.colname)) {
      observations <- cbind(observations, LDL=NA)
      colnames(observations)[ncol(observations)] <- lower_detection_limit.colname
    }
    if (all(colnames(observations) != upper_detection_limit.colname)) {
      observations <- cbind(observations, UDL=NA)
      colnames(observations)[ncol(observations)] <- upper_detection_limit.colname
    }
    if (all(colnames(observations) != cut_method.colname)) {
      observations <- cbind(observations, Cut="censored")
      colnames(observations)[ncol(observations)] <- cut_method.colname
    }
    observations <- observations[, c(observations.colname, lower_detection_limit.colname, 
                                     upper_detection_limit.colname, cut_method.colname)]
    colnames(observations) <- c("Observations", "LDL", "UDL", "Cut")
  } else {
    observations <- data.frame(Observations=observations, 
                               LDL=NA, UDL=NA, Cut=NA)
    if (!is.null(lower_detection_limit)) observations$LDL <- lower_detection_limit
    if (!is.null(upper_detection_limit)) observations$UDL <- upper_detection_limit
    if (!is.null(cut_method)) observations$Cut <- cut_method
  }
  
  observations$Cut <- tolower(observations$Cut)
  
  if (any((observations$Cut == "truncated") & (is.infinite(observations$Observations)))) {
    stop("When data are truncated, all values must be finite")
  }
  
  obc <- observations$Observations[is.finite(observations$Observations)]
  qdistr <- switch(distribution, 
                   gamma=qgamma, 
                   lognormal=qlnorm, 
                   normal=qnorm, 
                   weibull=qweibull, 
                   generalized.gamma=qggamma)
  
  pdistr <- switch(EXPR = distribution, 
                   gamma=pgamma, 
                   lognormal=plnorm, 
                   normal=pnorm, 
                   weibull=pweibull, 
                   generalized.gamma=pggamma)
  
  ddistr <- switch(EXPR = distribution, 
                   gamma=dgamma, 
                   lognormal=dlnorm, 
                   normal=dnorm, 
                   weibull=dweibull, 
                   generalized.gamma=dggamma)
  
  if (is.null(par)) {
    
    mean <- mean(obc)
    var <- var(obc)
    if (distribution == "gamma") {
      s <- var/mean
      a <-  mean/s
      par <- c(shape=a, scale=s)
    }
    if (distribution == "lognormal") {
      mean <- mean(log(obc))
      var <- var(log(obc))
      par <- c(meanlog=mean, sdlog=sqrt(var))
    }
    if (distribution == "normal") {
      par <- c(mean=mean, sd=sqrt(var))
    }
    if (distribution == "weibull") {
      par <- c(shape=1, scale=10)
    }
    if (distribution == "generalized.gamma") {
      s <- var/mean
      a <-  mean/s
      par <- c(theta=s,  kappa=a, delta=1)
    }
    previouspar <- par
    names(par) <- paste0(names(par), "1")
    
    if (n.mixture > 1) {
      
      for (p in 2:n.mixture) {
        parint <- previouspar*0.95
        names(parint) <- paste0(names(parint), as.character(p))
        par <- c(par, parint)
      }
      parint <- rep(0.5, (n.mixture-1))
      names(parint) <- paste0("p", as.character(1:(n.mixture-1)))
      par <- c(par, parint)
    }
  }
  
  
  lower <- NULL
  upper <- NULL
  for (i in 1:length(par)) {
    parint <- par[i]
    nparint <- names(par[i])
    if (grepl("^p[0-9]", nparint)) {
      lowerint <- 1e-4
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- 100
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^shape[0-9]", nparint)) {
      lowerint <- parint/10
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- parint*10
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^scale[0-9]", nparint)) {
      lowerint <- parint/10
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- parint*10
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^delta[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^meanlog[0-9]", nparint)) {
      lowerint <- -abs(parint*10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^sdlog[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^mean[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^sd[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^theta[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^kappa[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
  }
  
  # fitn(par, observations=observations, distribution=distribution, n.mixture=n.mixture, debug=2)
  
  # first analysis by ML
  result2 <- optim(par, fitn, observations=observations, distribution=distribution, 
                   method="L-BFGS-B", 
                   limits.lower=lower, limits.upper=upper, 
                   lower=lower, 
                   upper=upper, 
                   hessian=FALSE, n.mixture=n.mixture, debug=debug)
  
  par <- result2$par
  
  if (!is.null(n.iter)) {
    
    par[par == lower] <- par[par == lower] * 2
    par[par == upper] <- par[par == upper] / 2
    
    lower <- NULL
    upper <- NULL
    for (i in 1:length(par)) {
      parint <- par[i]
      nparint <- names(par[i])
      if (grepl("^p[0-9]", nparint)) {
        lowerint <- 1e-4
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- 100
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^shape[0-9]", nparint)) {
        lowerint <- parint/10
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- parint*10
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^scale[0-9]", nparint)) {
        lowerint <- parint/10
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- parint*10
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^delta[0-9]", nparint)) {
        lowerint <- abs(parint/10)
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- abs(parint*10)
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^meanlog[0-9]", nparint)) {
        lowerint <- -abs(parint*10)
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- abs(parint*10)
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^sdlog[0-9]", nparint)) {
        lowerint <- abs(parint/10)
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- abs(parint*10)
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^mean[0-9]", nparint)) {
        lowerint <- abs(parint/10)
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- abs(parint*10)
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^sd[0-9]", nparint)) {
        lowerint <- abs(parint/10)
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- abs(parint*10)
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^theta[0-9]", nparint)) {
        lowerint <- abs(parint/10)
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- abs(parint*10)
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
      if (grepl("^kappa[0-9]", nparint)) {
        lowerint <- abs(parint/10)
        names(lowerint) <- nparint
        lower <- c(lower, lowerint)
        upperint <- abs(parint*10)
        names(upperint) <- nparint
        upper <- c(upper, upperint)
      }
    }
    
    # priors <- data.frame(Density=rep('dnorm', length(par)), 
    #                     Prior1=par, 
    #                     Prior2=abs(par/4), 
    #                     SDProp=abs(log(abs(par))), 
    #                     Min=lower, 
    #                     Max=upper, 
    #                     Init=par, 
    #                     stringsAsFactors = FALSE, 
    #                     row.names=names(par))
    
    if (is.null(priors))
    priors <- data.frame(Density=rep('dunif', length(par)), 
                        Prior1=lower, 
                        Prior2=upper, 
                        SDProp=abs(log(abs(par))), 
                        Min=lower, 
                        Max=upper, 
                        Init=par, 
                        stringsAsFactors = FALSE, 
                        row.names=names(par))
    
    if (!is.null(session)) {
      # Je suis en shiny
      mcmc_run <- MHalgoGen(n.iter=n.iter, parameters=priors, observations=observations, 
                            distribution=distribution, 
                            parameters_name="par", adaptive = adaptive, 
                            n.mixture=n.mixture, debug=debug, 
                            likelihood=fitn, n.chains=1, n.adapt=n.adapt, thin=1, trace=FALSE, 
                            progress.bar.ini=NULL, 
                            session=session, 
                            progress.bar=function(iter, session) {
                              getFromNamespace("updateProgressBar", ns="shinyWidgets")(
                                session = session,
                                id = "pb2",
                                value = iter, total = 10000,
                                title = "MCMC algorithm"
                              )
                            })
      
      
    } else {
      if (progress.bar) {
        
        pb <- txtProgressBar(min=0, max=n.iter*2, style=3)
        mcmc_run <- MHalgoGen(n.iter=n.iter, parameters=priors, observations=observations, 
                              distribution=distribution, 
                              parameters_name="par", adaptive = adaptive, 
                              n.mixture=n.mixture, debug=debug, 
                              likelihood=fitn, n.chains=1, n.adapt=n.adapt, thin=1, trace=FALSE, 
                              session=NULL, 
                              progress.bar.ini=NULL, 
                              progress.bar=function(iter, session=NULL) {setTxtProgressBar(get("pb", envir = parent.env(environment())), iter)})
        
      } else {
        mcmc_run <- MHalgoGen(n.iter=n.iter, parameters=priors, observations=observations, 
                              distribution=distribution, 
                              parameters_name="par", adaptive = adaptive, 
                              n.mixture=n.mixture, debug=debug, 
                              likelihood=fitn, n.chains=1, n.adapt=n.adapt, thin=1, trace=FALSE)
      }
    }
    par_mcmc <- PPD(mcmc_run, namepar=names(par)[1], probs = c(0.025, 0.5, 0.975))
    
    for (i in seq_along(par)[-1])
      par_mcmc <- cbind(par_mcmc, 
                        PPD(mcmc_run, namepar=names(par)[i], probs = c(0.025, 0.5, 0.975)))
    
    colnames(par_mcmc) <- names(par)
    
    result2 <- c(result2, list(mcmc=mcmc_run), list(par_mcmc=par_mcmc))
    
    parX <- suppressMessages(as.parameters(mcmc_run))
  } else {
    # result2 <- c(result2, list(mcmc=NA), list(par_mcmc=NA))
    parX <- par
  }
  
  par <- parX
  
  lower <- NULL
  upper <- NULL
  for (i in 1:length(par)) {
    parint <- par[i]
    nparint <- names(par[i])
    if (grepl("^p[0-9]", nparint)) {
      lowerint <- 1e-4
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- 100
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^shape[0-9]", nparint)) {
      lowerint <- parint/10
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- parint*10
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^scale[0-9]", nparint)) {
      lowerint <- parint/10
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- parint*10
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^delta[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^meanlog[0-9]", nparint)) {
      lowerint <- -abs(parint*10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^sdlog[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^mean[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^sd[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^theta[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
    if (grepl("^kappa[0-9]", nparint)) {
      lowerint <- abs(parint/10)
      names(lowerint) <- nparint
      lower <- c(lower, lowerint)
      upperint <- abs(parint*10)
      names(upperint) <- nparint
      upper <- c(upper, upperint)
    }
  }
  
  result3 <- optim(par, fitn, observations=observations, distribution=distribution, 
                   method="L-BFGS-B", 
                   limits.lower=lower, limits.upper=upper, 
                   lower=lower, 
                   upper=upper, n.mixture=n.mixture, 
                   hessian=TRUE, debug=debug)
  
  result2 <- modifyList(result2, result3)
  
  parX <- result2$par
  
  
  # Inverse the hessian matrix to get SE for each parameters
  mathessian <- result2$hessian
  inversemathessian <- try(solve(mathessian), silent = TRUE)
  if (inherits(inversemathessian, "try-error")) {
    res <- rep(NA, length(lower))
    names(res) <- colnames(mathessian)
  } else {
    if (any(diag(inversemathessian) < 0)) {
      res <- rep(NA, length(lower))
      names(res) <- colnames(mathessian)
    } else {
      res <- sqrt(diag(inversemathessian))
    }
  }
  result2 <- c(result2, list(distribution=distribution))
  result2 <- c(result2, list(observations=observations))
  result2 <- c(result2, list(n.mixture=n.mixture))
  result2 <- c(result2, list(n.LDL=sum(observations[, "Observations"] == c(-Inf))))
  result2 <- c(result2, list(n.UDL=sum(observations[, "Observations"] == c(Inf))))
  result2 <- c(result2, list(n.quantified=sum(is.finite(observations[, "Observations"]))))
  
  result2 <- c(result2, list(SE=res), list(AIC=2*result2$value+2*length(par)), 
               list(AICc=2*result2$value+4+(2*length(par)*(length(par)+1))/(nrow(observations)-length(par)-1)), 
               list(BIC=2*result2$value+log(nrow(observations))*length(par)))
  
  
  nlength <- 100
  
  mx <- max(obc[is.finite(obc)], na.rm = FALSE)
  for (p in 1:n.mixture) {
    mx_ec <- do.call(qdistr, args=c(list(p = 0.99999, lower.tail = TRUE, log.p = FALSE), as.list(getparcutter(parX, set=p))))
    if (is.finite(mx_ec)) {
      mx <- max(c(mx, mx_ec))
    }
  }
  
  seqx <- seq(from=0, to=mx, length.out=nlength)
  pparX <- getparcutter(parX, set=NULL)
  seqpg <- rep(0, length(nlength))
  for (p in 1:n.mixture) {
    seqpg <- seqpg + do.call(pdistr, args = modifyList(list(q=seqx, lower.tail = TRUE,
                                                            log.p = FALSE), as.list(getparcutter(parX, set=p))))*pparX[p]
  }
  
  meanx <- sum((seqpg[2:nlength]-seqpg[1:(nlength-1)])*(seqx[1:(nlength-1)]+(seqx[2]-seqx[1])/2))/(sum((seqpg[2:nlength]-seqpg[1:(nlength-1)])))
  sdx <- sqrt(sum(((seqx[1:(nlength-1)]+(seqx[2]-seqx[1])/2) - meanx)^2*(seqpg[2:nlength]-seqpg[1:(nlength-1)])))
  
  result2 <- c(result2, list(total.distribution=c(mean=unname(meanx), sd=unname(sdx))))
  
  attributes(result2) <- modifyList(attributes(result2), 
                                    list(nall=nrow(result2$observations), 
                                         df=length(result2$par), 
                                         nobs=nrow(result2$observations)))
  
  if (!is.null(n.iter)) {
    LDL_x <- unique(observations[, "LDL"])
    
    
    if (all(!is.na(LDL_x))) {
      prob_mcmc_LDL_tot <- NULL
      LDL_totp <- NULL
      
      for (LDL in LDL_x) {
        mcmc_xp <- NULL
        prob_mcmc_LDL <- NULL
        
        for (i in 1:n.iter) {
          nlength <- 100
          seqx <- seq(from=0, to=LDL, length.out=nlength)
          seqpg <- rep(0, length(seqx))
          pparX_mcmc <- getparcutter(mcmc_run$resultMCMC[[1]][i, ], set=NULL)
          for (p in 1:n.mixture)
            seqpg <- seqpg + do.call(pdistr, args = modifyList(list(q=seqx, lower.tail = TRUE,
                                                                    log.p = FALSE), as.list(getparcutter(mcmc_run$resultMCMC[[1]][i, ], set=p)))) * pparX_mcmc[p]
          
          mcmc_xp <- c(mcmc_xp, sum((seqpg[2:nlength]-seqpg[1:(nlength-1)])*(seqx[1:(nlength-1)]+(seqx[2]-seqx[1])/2))/(sum((seqpg[2:nlength]-seqpg[1:(nlength-1)]))))
          prob_mcmc_LDL <- c(prob_mcmc_LDL, do.call(pdistr, args = modifyList(list(q=LDL, lower.tail = TRUE,
                                                                                   log.p = FALSE), as.list(getparcutter(mcmc_run$resultMCMC[[1]][i, ], set=p)))) * pparX_mcmc[p])
        }
        LDL_totp <- c(LDL_totp, quantile(mcmc_xp, probs=c(0.025, 0.5, 0.975), na.rm=TRUE))
        prob_mcmc_LDL_tot <- c(prob_mcmc_LDL_tot, quantile(prob_mcmc_LDL, probs=c(0.025, 0.5, 0.975), na.rm=TRUE))
      }
      LDL_totp <- matrix(LDL_totp, nrow=3, byrow=FALSE)
      colnames(LDL_totp) <- as.character(LDL_x)
      rownames(LDL_totp) <- c("2.5%", "50%", "97.5%")
      result2 <- c(result2, list(LDL=LDL_totp))
      
      prob_mcmc_LDL_tot <- matrix(prob_mcmc_LDL_tot, nrow=3, byrow=FALSE)
      colnames(prob_mcmc_LDL_tot) <- as.character(LDL_x)
      rownames(prob_mcmc_LDL_tot) <- c("2.5%", "50%", "97.5%")
      result2 <- c(result2, list(prob.LDL=prob_mcmc_LDL_tot))
    }
    
    
    
    UDL_x <- unique(observations[, "UDL"])
    
    if (all(!is.na(UDL_x))) {
      prob_mcmc_UDL_tot <- NULL
      UDL_totp <- NULL
      
      for (UDL in UDL_x) {
        # mcmc_x <- NULL
        mcmc_xp <- NULL
        prob_mcmc_UDL <- NULL
        for (i in 1:n.iter) {
          # Alternative plus correcte
          nlength <- 100
          seqx <- seq(from=UDL, to=mx, length.out=nlength)
          seqpg <- rep(0, length(seqx))
          pparX_mcmc <- getparcutter(mcmc_run$resultMCMC[[1]][i, ], set=NULL)
          for (p in 1:n.mixture)
            seqpg <- seqpg + do.call(pdistr, args = modifyList(list(q=seqx, lower.tail = TRUE,
                                                                    log.p = FALSE), as.list(getparcutter(mcmc_run$resultMCMC[[1]][i, ], set=p)))) * pparX_mcmc[p]
          
          mcmc_xp <- c(mcmc_xp, sum((seqpg[2:nlength]-seqpg[1:(nlength-1)])*(seqx[1:(nlength-1)]+(seqx[2]-seqx[1])/2))/(sum((seqpg[2:nlength]-seqpg[1:(nlength-1)]))))
          
          prob_mcmc_UDL <- c(prob_mcmc_UDL, do.call(pdistr, args = modifyList(list(q=UDL, lower.tail = FALSE,
                                                                                   log.p = FALSE), as.list(getparcutter(mcmc_run$resultMCMC[[1]][i, ], set=p)))) * pparX_mcmc[p])
        }
        UDL_totp <- c(UDL_totp, quantile(mcmc_xp, probs=c(0.025, 0.5, 0.975), na.rm=TRUE))
        prob_mcmc_UDL_tot <- c(prob_mcmc_UDL_tot, quantile(prob_mcmc_UDL, probs=c(0.025, 0.5, 0.975), na.rm=TRUE))
        
      }
      UDL_totp <- matrix(UDL_totp, nrow=3, byrow=FALSE)
      colnames(UDL_totp) <- as.character(UDL_x)
      rownames(UDL_totp) <- c("2.5%", "50%", "97.5%")
      result2 <- c(result2, list(UDL=UDL_totp))
      
      prob_mcmc_UDL_tot <- matrix(prob_mcmc_UDL_tot, nrow=3, byrow=FALSE)
      colnames(prob_mcmc_UDL_tot) <- as.character(UDL_x)
      rownames(prob_mcmc_UDL_tot) <- c("2.5%", "50%", "97.5%")
      result2 <- c(result2, list(prob.UDL=prob_mcmc_UDL_tot))
    }
    
    
    
    L_MCMC <- do.call(fitn, args = list(observations=observations, 
                                        distribution=distribution, 
                                        par=result2$par_mcmc[2, ]))
    
    # Goodness of fit using likelihood of posterior predictive distribution
    
    L_PPD <- NULL
    obc <- observations
    for (i in 1:n.iter) {
      
      if (!is.null(session)) {
        getFromNamespace(x="updateProgressBar", ns="shinyWidgets")(
          session = session,
          id = "pb2",
          value = n.iter+i, total = 10000,
          title = "Posterior predictive distribution")
        
      } else {
        if (progress.bar) setTxtProgressBar(pb, n.iter+i)
      }
      # print(i)
      r <- rcutter(cutter=result2, 
                   observed_detection_limit=TRUE, 
                   random_method = "MCMC", 
                   index_mcmc = i)
      if (all(!is.na(r)) & any(is.finite(r))) {
        obc[!is.na(obc[, "Observations"]), "Observations"] <- r
        
        L <- do.call(fitn, args = list(observations=obc, 
                                       distribution=distribution, 
                                       par=result2$par_mcmc[2, ]))
        
        
        L_PPD <- c(L_PPD, L)
      }
    }
    
    result2 <- c(result2, list(GoF=sum(L_PPD>L_MCMC)/length(L_PPD)))
    result2 <- c(result2, list(n.GoF=length(L_PPD)))
    
  } else {
    # result2 <- c(result2, list(UDL=NA, LDL=NA))
    # result2 <- c(result2, list(GoF=NA))
    
    LDL_x <- unique(observations[, "LDL"])
    
    if (all(!is.na(LDL_x))) {
      LDL_totp <- NULL
      prob_mcmc_LDL_tot <- NULL
      for (LDL in LDL_x) {
        nlength <- 100
        seqx <- seq(from=0, to=LDL, length.out=nlength)
        seqpg <- rep(0, length(seqx))
        for (p in 1:n.mixture)
          seqpg <- seqpg + do.call(pdistr, args = modifyList(list(q=seqx, lower.tail = TRUE,
                                                                  log.p = FALSE), as.list(getparcutter(parX, set=p)))) * pparX[p]
        
        xp <- sum((seqpg[2:nlength]-seqpg[1:(nlength-1)])*(seqx[1:(nlength-1)]+(seqx[2]-seqx[1])/2))/(sum((seqpg[2:nlength]-seqpg[1:(nlength-1)])))
        LDL_totp <- c(LDL_totp, "mean"=xp)
        prob_mcmc_LDL_tot <- c(prob_mcmc_LDL_tot, do.call(pdistr, args = modifyList(list(q=LDL, lower.tail = TRUE,
                                                                                         log.p = FALSE), as.list(getparcutter(parX, set=p)))) * pparX[p])
        
      }
      LDL_totp <- matrix(LDL_totp, nrow=1, byrow=FALSE)
      colnames(LDL_totp) <- as.character(LDL_x)
      rownames(LDL_totp) <- c("mean")
      result2 <- c(result2, list(LDL=LDL_totp))
      
      prob_mcmc_LDL_tot <- matrix(prob_mcmc_LDL_tot, nrow=1, byrow=FALSE)
      colnames(prob_mcmc_LDL_tot) <- as.character(LDL_x)
      rownames(prob_mcmc_LDL_tot) <- c("mean")
      result2 <- c(result2, list(prob.LDL=prob_mcmc_LDL_tot))
      
    } 
    
    
    
    UDL_x <- unique(observations[, "UDL"])
    
    if (all(!is.na(UDL_x))) {
      UDL_totp <- NULL
      prob_mcmc_UDL_tot <- NULL
      
      for (UDL in UDL_x) {
        # Alternative plus correcte
        nlength <- 100
        seqx <- seq(from=UDL, to=mx, length.out=nlength)
        
        seqpg <- rep(0, length(seqx))
        for (p in 1:n.mixture)
          seqpg <- seqpg + do.call(pdistr, args = modifyList(list(q=seqx, lower.tail = TRUE,
                                                                  log.p = FALSE), as.list(getparcutter(parX, set=p)))) * pparX[p]
        
        xp <- sum((seqpg[2:nlength]-seqpg[1:(nlength-1)])*(seqx[1:(nlength-1)]+(seqx[2]-seqx[1])/2))/(sum((seqpg[2:nlength]-seqpg[1:(nlength-1)])))
        UDL_totp <- c(UDL_totp, "mean"=xp)
        
        prob_mcmc_UDL_tot <- c(prob_mcmc_UDL_tot, do.call(pdistr, args = modifyList(list(q=UDL, lower.tail = FALSE,
                                                                                         log.p = FALSE), as.list(getparcutter(parX, set=p)))) * pparX[p])
        
      }
      UDL_totp <- matrix(UDL_totp, nrow=1, byrow=FALSE)
      colnames(UDL_totp) <- as.character(UDL_x)
      rownames(UDL_totp) <- c("mean")
      result2 <- c(result2, list(UDL=UDL_totp))
      
      prob_mcmc_UDL_tot <- matrix(prob_mcmc_UDL_tot, nrow=1, byrow=FALSE)
      colnames(prob_mcmc_UDL_tot) <- as.character(UDL_x)
      rownames(prob_mcmc_UDL_tot) <- c("mean")
      result2 <- c(result2, list(prob.UDL=prob_mcmc_UDL_tot))
    }
    
  }
  
  
  if (progress.bar & is.null(session)) cat("\n")
  
  result2 <- addS3Class(result2, class=c("cutter", "CompareAIC"))
  return(result2)
}

.getparcutter <- function(par, set=1) {
  # Retourne le set de paramètres set
  # Ou si c'est NULL, le set des p classé
  par_ec <- unlist(par)
  if (is.null(set)) {
    parX <- par_ec[grepl("^p[0-9]*", names(par_ec))]
    parX <- parX[order(as.numeric(gsub(".+([0-9]+)", "\\1", names(parX))))]
    parX <- c(parX, structure(1, names=paste0("p", as.character(length(parX)+1))))
    parX <- parX/sum(parX)
  } else {
    parX <- par_ec[grepl(as.character(set), names(par_ec)) & names(par_ec) != paste0("p", as.character(set))]
    names(parX) <- gsub("[0-9]+", "", names(parX))
  }
  if (is.list(par)) parX <- as.list(parX)
  return(parX)
}

#' Return the flexit value
#' @title Return the flexit
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A vector with the probabilities
#' @param x The values at which the flexit model must be calculated
#' @param par The vector with P, S, K1, and K2 values
#' @param P P value
#' @param S S value
#' @param K1 K1 value
#' @param K2 K2 value
#' @param Min Min value for scaled flexit model
#' @param Max Max value for scaled flexit model
#' @param zero Value to replace zero
#' @param error0 Value to return if an error is observed toward 0
#' @param error1 Value to return if an error is observed toward 1
#' @description Return a vector with the probabilities.
#' The flexit equation is published in:\cr
#' Abreu-Grobois, F.A., Morales-Mérida, B.A., Hart, C.E., Guillon, J.-M., Godfrey, M.H., 
#' Navarro, E. & Girondot, M. (2020) Recent advances on the estimation of the thermal 
#' reaction norm for sex ratios. PeerJ, 8, e8451.\cr
#' If dose < P then \eqn{(1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}{(1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}\cr
#' If dose > P then \eqn{1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}{1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}\cr
#' with:\cr
#'      \deqn{S1 = (2^(K1 - 1) * S * K1)/(2^K1 - 1)}{S1 = (2^(K1 - 1) * S * K1)/(2^K1 - 1)}
#'      \deqn{S2 = (2^(K2 - 1) * S * K2)/(2^K2 - 1)}{S2 = (2^(K2 - 1) * S * K2)/(2^K2 - 1)}
#' \cr
#' If \eqn{2^K1}{2^K1} is too large to be estimated, the approximation \eqn{S1 = S*K1/2}{S1 = S*K1/2} is used.\cr
#' Demonstration:\cr
#' \deqn{S1 = (2^(K1 - 1) * S * K1)/(2^K1 - 1)}{S1 = (2^(K1 - 1) * S * K1)/(2^K1 - 1)}
#' \deqn{S1 = exp(log((2^(K1 - 1) * S * K1)/(2^K1 - 1)))}{S1 = exp(log((2^(K1 - 1) * S * K1)/(2^K1 - 1)))}
#' \deqn{S1 = exp(log(2^(K1 - 1)) + log(S * K1) - log(2^K1 - 1))}{S1 = exp(log(2^(K1 - 1)) + log(S * K1) - log(2^K1 - 1))}
#' When \eqn{K1}{K1} is very large, \eqn{2^K1 - 1 = 2^K1}{2^K1 - 1 = 2^K1} then
#' \deqn{S1 = exp((K1 - 1) * log(2) + log(S * K1) - K1 * log(2))}{S1 = exp((K1 - 1) * log(2) + log(S * K1) - K1 * log(2))}
#' \deqn{S1 = exp((K1 * log(2) - log(2) + log(S * K1) - K1 * log(2))}{S1 = exp((K1 * log(2) - log(2) + log(S * K1) - K1 * log(2))}
#' \deqn{S1 = exp(log(S * K1)- log(2))}{S1 = exp(log(S * K1)- log(2))}
#' \deqn{S1 = S * K1 / 2}{S1 = S * K1 / 2}
#' If \eqn{2^K2}{2^K2} is too large to be estimated, the approximation \eqn{S2 = S*K2/2}{S2 = S*K2/2} is used.\cr
#' If \eqn{(1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}{(1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)} is not finite, 
#' the following approximation is used:\cr
#' \deqn{exp((-1/K1)*(K1*log(2)+(4*S1*(P-x))))}{exp((-1/K1)*(K1*log(2)+(4*S1*(P-x))))}
#' If \eqn{1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}{1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)} is not finite, 
#' the following approximation is used:\cr
#' \deqn{1 - exp((-1/K2)*(K2*log(2)+(4*S2*(x - P))))}{1 - exp((-1/K2)*(K2*log(2)+(4*S2*(x - P))))}
#' @family logit
#' @examples
#' \dontrun{
#' n <- flexit(x=1:100, par=c(P=50, S=0.001, K1=0.01, K2=0.02))
#' n <- flexit(x=1:100, P=50, S=0.001, K1=0.01, K2=0.02)
#' 
#' 1/(1+exp(0.01*4*(50-1:100)))
#' flexit(1:100, P=50, S=0.01, K1=1, K2=1)
#' }
#' @export

flexit <- function(x, par=NULL, P=NULL, S=NULL, K1=NULL, K2=NULL, Min=0, Max=1, zero=1E-9, error0 = 0, error1 = 1) {

  # par=NULL; P=NULL; S=NULL; K1=NULL; K2=NULL; Min=0; Max=1; zero=1E-9; error0 = 0; error1 = 1
  
#             par <- c('P' = 0.66325432700364861, 
#               'S' = 6.68963311308306, 
#               'Min' = 0.16643548495805249, 
#               'Max' = 1, 
#               'K1' = 1, 
#               'K2' = 1)
#             x <- seq(from=0.005, to=0.995, by=0.01)
#             zero=1E-9; error0 = 0; error1 = 1
  
  # Peut-être  encore des problèmes de exp(K1 ou K2)
  if (!is.null(par)) {
    if (is.na(par["K1"])) par <- c(par, K1=1)
    if (is.na(par["K2"])) par <- c(par, K2=1)
    if (is.na(par["Min"])) par <- c(par, Min=0)
    if (is.na(par["Max"])) par <- c(par, Max=1)
    K1 <- ifelse(par["K1"] == 0, zero, par["K1"])
    K2 <- ifelse(par["K2"] == 0, zero, par["K2"])
    Min <- par["Min"]
    Max <- par["Max"]
    S <- par["S"]
    P <- par["P"]
  }
  
  lg <- max(c(length(x), length(K1), length(K2), length(P), length(S), length(Min), length(Max)))
  
  if (is.null(Min)) {
    Min <- rep(zero, lg)
  } else {
    if (length(Min) != lg) Min <- rep(Min, lg)[1:lg]
  }
  if (is.null(Max)) {
    Max <- rep(1-zero, lg)
  } else {
    if (length(Max) != lg) Max <- rep(Max, lg)[1:lg]
  }
  if (is.null(K1)) {
    K1 <- rep(1, lg)
  } else {
    if (length(K1) != lg) K1 <- rep(K1, lg)[1:lg]
  }
  if (is.null(K2)) {
    K2 <- rep(1, lg)
  } else {
    if (length(K2) != lg) K2 <- rep(K2, lg)[1:lg]
  }
  K1 <- ifelse(K1 == 0, zero, K1)
  K2 <- ifelse(K2 == 0, zero, K2)
  
  # Je les mets tous à la même taille
  if (length(P) != lg) P <- rep(P, lg)[1:lg]
  if (length(S) != lg) S <- rep(S, lg)[1:lg]
  if (length(x) != lg) x <- rep(x, lg)[1:lg]
  
  S1 <- ifelse(is.infinite(2^(K1)), K1*S/2, (2^(K1 - 1)*K1*S)/(2^(K1) - 1))
  
  # if (is.infinite(2^(K1))) {
  #   S1 <- K1*S/2
  # } else {
  #   S1 <- (2^(K1 - 1)*K1*S)/(2^(K1) - 1)
  # }
  
  Test1 <- (1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))
  Test1_p <- ifelse(!is.infinite(Test1), 
                    Test1^(-1/K1), 
                    exp((-1/K1)*(K1*log(2)+(4*S1*(P-x)))))
  
  S2 <- ifelse(is.infinite(2^(K2)), K2*S/2, (2^(K2 - 1)*K2*S)/(2^(K2) - 1))
  
  # if (is.infinite(2^(K2))) {
  #   S2 <- K2*S/2
  # } else {
  #   S2 <- (2^(K2 - 1)*K2*S)/(2^(K2) - 1)
  # }
  
  Test2 <- (1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))
  Test2_p <- ifelse(!is.infinite(Test2), 
                    1 - Test2^(-1/K2), 
                    1 - exp((-1/K2)*(K2*log(2)+(4*S2*(x - P)))))
  
  
  p <- ifelse(x < P,  ifelse(Test1 < 0,
                                 ifelse(S1 < 0, error1, error0),
                             Test1_p)
              ,
              ifelse(Test2 < 0,
                     ifelse(S2 < 0, error0, error1),
                     Test2_p)
  )
  
  p <- (Max - Min) * p + Min
  
  return(unname(p))
}

#' wget download a file from internet and save it locally
#' @title Download a file from internet and save it locally
#' @author Marc Girondot
#' @return Nothing
#' @param url The url where to download file
#' @param ... The parameters send to download.file()
#' @description Download a file from internet and save it locally. This function is a wrapper for
#' download.files() that keep the name identical and can get several files at once. It was written to simplify 
#' downloading of file. It doest not use the true wget function (https://www.gnu.org/software/wget/) which is 
#' much more complex but also powerful.
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' # Save locally the files send in the parameter url
#' wget(c("https://cran.r-project.org/web/packages/HelpersMG/HelpersMG.pdf", 
#'          "https://cran.r-project.org/web/packages/embryogrowth/embryogrowth.pdf"))
#' }
#' @export


wget <- function(url=stop("At least one internet adress is required"), ...) {
  for (i in 1:length(url))
    do.call(download.file, modifyList(list(url=url[i], destfile=basename(url[i])), list(...)))
  }

#' @title Extract parameters from mcmcComposite object
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A vector with parameters at maximum likelihood or index position
#' @param x A mcmcComposite obtained as a result of \code{MHalgoGen()} function
#' @param total If TRUE, does not use the thinned results.
#' @param index At which iteration the parameters must be taken, see description.
#' @param probs Quantiles to be returned, see description.
#' @param chain The chain in which to get parameters; "all" is for all chains.
#' @param silent If TRUE, does not print any information.
#' @description Take a mcmcComposite object and create a vector object with parameter value at specified iteration.\cr
#' If \code{index="best"}, the function will return the parameters for the highest likelihood. It also indicates at which iteration the maximum lihelihood has been observed.\cr
#' If \code{index="last"}, the function will return the parameters for the last likelihood.\cr
#' If \code{index="median"}, the function will return the median value of the parameter.\cr
#' if \code{index="quantile"}, the function will return the \code{probs} defined by quantiles parameter.\cr
#' If \code{index="mode"}, the function will return the mode value of the parameter based on Asselin de Beauville (1978) method.\cr
#' \code{index} can also be a numeric value. It uses all the chains being concatanated.\cr
#' This function uses the complete iterations available if total is TRUE. The adaptation part is never used.\cr
#' \code{index} can also be a function applied to the iterations.
#' @references Asselin de Beauville J.-P. (1978). Estimation non paramétrique de la densité et du mode, 
#' exemple de la distribution Gamma. Revue de Statistique Appliquée, 26(3):47-70.
#' @family mcmcComposite functions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' require(coda)
#' x <- rnorm(30, 10, 2)
#' dnormx <- function(data, x) {
#'  data <- unlist(data)
#'  return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
#' }
#' parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
#'                               Prior1=c(10, 0.5), 
#'                               Prior2=c(2, 0.5), 
#'                               SDProp=c(1, 1), 
#'                               Min=c(-3, 0), 
#'                               Max=c(100, 10), 
#'                               Init=c(10, 2), 
#'                               stringsAsFactors = FALSE, 
#'                               row.names=c('mean', 'sd'))
#' mcmc_run <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
#'                       likelihood=dnormx, n.chains=1, n.adapt=100, 
#'                       thin=1, trace=1)
#' plot(mcmc_run, xlim=c(0, 20))
#' plot(mcmc_run, xlim=c(0, 10), parameters="sd")
#' mcmcforcoda <- as.mcmc(mcmc_run)
#' #' heidel.diag(mcmcforcoda)
#' raftery.diag(mcmcforcoda)
#' autocorr.diag(mcmcforcoda)
#' acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
#' acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
#' batchSE(mcmcforcoda, batchSize=100)
#' 
#' # The batch standard error procedure is usually thought to 
#' # be not as accurate as the time series methods used in summary
#' summary(mcmcforcoda)$statistics[,"Time-series SE"]
#' summary(mcmc_run)
#' as.parameters(mcmc_run)
#' lastp <- as.parameters(mcmc_run, index="last")
#' parameters_mcmc[,"Init"] <- lastp
#' 
#' # The n.adapt set to 1 is used to not record the first set of parameters
#' # then it is not duplicated (as it is also the last one for 
#' # the object mcmc_run)
#' mcmc_run2 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
#'                        likelihood=dnormx, n.chains=1, n.adapt=1, 
#'                        thin=1, trace=1)
#' mcmc_run3 <- merge(mcmc_run, mcmc_run2)
#' 
#' ####### no adaptation, n.adapt must be 0
#' parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
#' mcmc_run3 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
#'                        likelihood=dnormx, n.chains=1, n.adapt=0, 
#'                        thin=1, trace=1)
#' # With index being median, it returns the median value for each parameter
#' as.parameters(mcmc_run3, index="median")
#' as.parameters(mcmc_run3, index="mode")
#' as.parameters(mcmc_run3, index="best")
#' as.parameters(mcmc_run3, index="quantile", probs=0.025)
#' as.parameters(mcmc_run3, index="quantile", probs=0.975)
#' as.parameters(mcmc_run3, index="quantile", probs=c(0.025, 0.975))
#' d(as.parameters(mcmc_run3, index = function(x) mean(x)))
#' }
#' @export


as.parameters <- function(x=stop("A result obtained after a MCMC analysis must be given.") , 
                          total = FALSE                                                    ,
                          index="median"                                                     , 
                          chain="all"                                                      , 
                          probs=c(0.025, 0.5, 0.975)                                       , 
                          silent=FALSE                                                     ) {
  
  if (chain[1] == "all") if (total) chain <- seq_along(x$resultMCMC.total) else chain <- seq_along(x$resultMCMC)
  
  # if (total) p <- x$resultMCMC.total[[chain[1]]] else p <- x$resultMCMC[[chain[1]]]
  
  p <- NULL
  
  if (length(chain) >= 1) {
    for (i in chain) {
      if (total) p <- rbind(p, x$resultMCMC.total[[i]]) else p <- rbind(p, x$resultMCMC[[i]])
    }
  }
  
  if (is.null(p)) stop("No data are available.")
  
  if (is.function(index)) {
    pml <- apply(p, MARGIN=2, FUN = index)
  } else {
    
    if (index[1] == "median") {
      pml <- apply(p, MARGIN=2, FUN = median)
      # names(pml) <- colnames(p)
    } else {
      if (index[1] == "quantile") {
        pml <- apply(p, MARGIN=2, FUN = quantile, probs=probs)
        # names(pml) <- colnames(p)
      } else {
        
        if (index[1] == "mode") {
          asselin <- function (x, bw = NULL, ...) {
            if (is.null(bw)) 
              bw <- 1
            nx <- length(x)
            kmax <- floor(ifelse(nx < 30, 10, 15) * log(nx))
            y <- sort(x)
            ok1 <- FALSE
            while (!ok1) {
              ny <- length(y)
              if (ny == 1) 
                return(y)
              qy <- stats::quantile(y, probs = c(0.1, 0.25, 0.5, 0.75, 
                                                 0.9), names = FALSE, ...)
              delta <- min(qy[5] - qy[4], qy[2] - qy[1])
              a <- qy[1] - 3 * delta
              b <- qy[5] + 3 * delta
              yab <- y[y >= a & y <= b]
              k <- kmax
              ok2 <- FALSE
              while (!ok2) {
                b <- seq(from = min(yab), to = max(yab), length = k + 
                           1)
                n <- c(tabulate(findInterval(yab, b[-(k + 1)])), 
                       0)
                N <- sum(n)
                v <- as.numeric(n >= N/k)
                w <- which.max(v)
                v2 <- v[w:(k + 1)]
                w2 <- which.min(v2) + w - 1
                v3 <- v[w2:(k + 1)]
                nc <- sum(n[w:(w2 - 1)])
                if (any(v3 == 1) && nc == 1) {
                  if (k > 3) {
                    k <- k - 1
                  }
                  else if (k == 3) {
                    if (n[3] > 1) {
                      w <- 3
                      w2 <- 4
                    }
                    ok2 <- TRUE
                  }
                  else {
                    stop("k < 3", call. = FALSE)
                  }
                }
                else if (any(v3 == 1) && nc > 1) {
                  if (k > 3) {
                    k <- k - 1
                  }
                  else if (k == 3) {
                    if (n[3] > 1) {
                      p1 <- (1/n[1]) * prod(diff(yab[yab >= b[w] & 
                                                       yab <= b[w2]]))
                      p2 <- (1/n[3]) * prod(diff(yab[yab >= b[3] & 
                                                       yab <= b[4]]))
                      if (p1 > p2) {
                        w <- 3
                        w2 <- 4
                      }
                    }
                    ok2 <- TRUE
                  }
                  else {
                    stop("k < 3", call. = FALSE)
                  }
                }
                else if (!any(v3 == 1)) {
                  ok2 <- TRUE
                }
              }
              nc <- sum(n[w:(w2 - 1)])
              d <- abs((qy[4] + qy[2] - 2 * qy[3])/(qy[4] - qy[2]))
              nc2 <- ny * (1 - d)
              y <- yab[yab >= b[w] & yab <= b[w2]]
              if (nc == ny) {
                ok1 <- TRUE
              }
              else if (nc <= ifelse(nx < 30, nx/3, bw * nc2)) {
                ok1 <- TRUE
              }
              else {
                ok1 <- FALSE
              }
            }
            stats::median(y)
          }
          
          pml <- apply(p, MARGIN=2, FUN = asselin)
          # names(pml) <- colnames(p)
          
        } else {
          
          
          # if (total) L <- x$resultLnL.total[[chain[1]]] else L <- x$resultLnL[[chain[1]]]
          L <- NULL
          
          if (length(chain) >= 1) {
            for (i in chain[]) {
              if (total) L <- c(L, x$resultLnL.total[[i]]) else L <- c(L, x$resultLnL[[i]])
            }
          }
          
          
          pos <- NULL
          
          if (any(is.numeric(index))) {
            pos <- index
          } else {
            
            if (index[1] == "all") {
              pos <- seq_along(L)
            }
            
            if (index[1] =="best") {
              pos <- which.max(L)
              
              if (!silent) message(paste("The best likelihood has been observed at iteration", pos, ": Ln L=", specify_decimal(max(L), decimals = 2)))
            }
            
            if (index[1] == "last") {
              pos <- length(L)
            }
          }
          
          if (is.null(pos)) {
            stop("index is not recognized")
          }
          
          pml <- p[pos,]
          # names(pml) <- names(p[pos,])
        }
      }
    }
  }
  return(pml)
  
}

#' fitdistrquantiles returns the parameters of beta, normal or gamma distribution
#' @title Parameters of beta, normal or gamma distribution based on quantiles. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Parameters of beta, normal or gamma distribution based on quantiles.
#' @param quantiles Vector of quantiles.
#' @param probs Numeric vector of probabilities with values in \[0,1\].
#' @param scaled Used scaled least-square.
#' @param distribution Distribution to be fitted: beta, normal, or gamma.
#' @description Return the parameters of beta or gamm that fits the best the
#' quantiles. The vector of probabilities can be obtained from names of 
#' quantiles.
#' @examples
#' \dontrun{
#' rd <- rbeta(100000, shape1 = 0.7, shape2 = 6.2, ncp=0)
#' (q <- quantile(rd, probs=c(0.025, 0.5, 0.975)))
#' 
#' (best <- fitdistrquantiles(quantiles = q, probs = c(0.025, 0.5, 0.975), 
#'                            scaled=FALSE, distribution = "beta"))
#' rd10000 <- rbeta(10000, shape1 = best["shape1"], shape2 = best["shape2"], ncp=best["ncp"])
#' quantile(rd10000, probs=c(0.025, 0.5, 0.975))
#' 
#' # Here the probabilities are obtained from names of quantiles
#' (best <- fitdistrquantiles(quantiles = q, scaled=FALSE, distribution = "beta"))
#' rd10000 <- rbeta(10000, shape1 = best["shape1"], shape2 = best["shape2"], ncp=best["ncp"])
#' quantile(rd10000, probs=c(0.025, 0.5, 0.975))
#' 
#' # If only two quantiles are provided, ncp cannot be fitted
#' (q2 <- quantile(rd, probs=c(0.025, 0.975)))
#' (best <- fitdistrquantiles(quantiles = q2, scaled=FALSE, distribution = "beta"))
#' rd10000 <- rbeta(10000, shape1 = best["shape1"], shape2 = best["shape2"])
#' quantile(rd10000, probs=c(0.025, 0.975))
#' x <- seq(from=0.00, to=1, by=0.001)
#' plot(x=x, y=pbeta(x, shape1 = best["shape1"], shape2 = best["shape2"]), 
#'      las=1, bty="n", type="l", ylim=c(0, 1))
#' segments(x0=q2[1], x1=q2[1], y0=0, y1=1, lty=2)
#' segments(x0=q2[2], x1=q2[2], y0=0, y1=1, lty=2)
#' 
#' (best <- fitdistrquantiles(quantiles = q, probs = c(0.025, 0.5, 0.975), 
#'                            scaled=FALSE, distribution = "gamma"))
#' rd10000 <- rgamma(10000, shape = best["shape"], scale = best["scale"])
#' quantile(rd10000, probs=c(0.025, 0.5, 0.975))
#' 
#' (best <- fitdistrquantiles(quantiles = c(10, 20, 30), probs = c(0.025, 0.5, 0.975), 
#'                            scaled=FALSE, distribution = "normal"))
#' rd10000 <- rnorm(10000, mean = best["mean"], sd = best["sd"])
#' quantile(rd10000, probs=c(0.025, 0.5, 0.975))
#' }
#' @export

fitdistrquantiles <- function(quantiles=stop("At least two quantiles must be provided"), 
                              probs=NULL, 
                              scaled=FALSE, distribution="beta") {
  distribution <- match.arg(distribution, choices = c("beta", "gamma", "normal"))
  
  optimfitdistrquantiles <- function(x, quantiles, probs, scaled=FALSE, distribution="beta") {
    if (distribution == "beta") {
      shape1 <- x["shape1"]
      shape2 <- x["shape2"]
      ncp <- x["ncp"]
      if (is.na(ncp)) {
        out_distr <- suppressWarnings(qbeta(p=probs, shape1 = shape1, shape2 = shape2))
        
      } else {
        out_distr <- suppressWarnings(qbeta(p=probs, shape1 = shape1, shape2 = shape2, ncp=ncp))
      }
    }
    if (distribution =="gamma") {
      rate <- x["rate"]
      shape <- x["shape"]
      scale <- x["scale"]
      # if (is.na(rate)) {
      out_distr <- suppressWarnings(qgamma(p=probs, shape=shape, scale=scale))
      # } else {
      #   out_distr <- qgamma(p=probs, rate=rate, shape=shape)
      # }
    }
    if (distribution =="normal") {
      mean <- x["mean"]
      sd <- x["sd"]
      out_distr <- suppressWarnings(qnorm(p=probs, mean=mean, sd=sd))
    }
    
    if (scaled) {
      c <- sum(((out_distr-quantiles)/out_distr)^2)
    } else {
      c <- sum((out_distr-quantiles)^2)
    }
    c <- ifelse(is.na(c), +Inf, c)
    return(c)
  }
  
  if (is.null(probs) & is.null(names(quantiles))) {
    stop("Or probs or named quantiles must be provided.")
  }
  if (is.null(probs)) {
    probs <- as.numeric(gsub("%", "", names(quantiles)))/ifelse(grepl("%", names(quantiles)), 100, 1)
  }
  quantile_median <- unname(quantiles[which.min(abs(probs-0.5))][1])
  if (distribution == "beta") {
    # The mean is shape1/(shape1+shape2) and the variance is shape1*shape2/((shape1+shape2)^2 (shape1+shape2+1)). 
    if (length(quantiles)>2) {
      par <- c(shape1=0.5, shape2=(0.5-quantile_median*0.5)/quantile_median, ncp=0)
    } else {
      par <- c(shape1=0.5, shape2=(0.5-quantile_median*0.5)/quantile_median)
    }
    lower <- rep(1e-7, length(par))
  }
  if (distribution == "gamma") {
    # The mean and variance are E(X) = shape*scale and Var(X) = shape*scale^2.
    par <- c(shape=1, scale=1/quantile_median)
    lower <- rep(1e-7, length(par))
  }
  if (distribution == "normal") {
    par <- c(mean=quantile_median, sd=1)
    lower <- c(-Inf, 1e-7)
  }
  
  
  best <- optim(par=par, 
                lower=lower, upper=rep(Inf, length(par)), 
                method="L-BFGS-B", 
                fn=optimfitdistrquantiles, 
                quantiles=quantiles, probs=probs, scaled=scaled, distribution=distribution)
  return(best$par)
  
}

#' Show the name of a point
#' @title Show the name of a point
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Name of the point
#' @param points A list with x, y and names elements
#' @param x The x coordinates
#' @param y The y coordinates.
#' @param names The names of the points
#' @param col Color of the legend.
#' @param silent TRUE or FALSE
#' @family plot and barplot functions
#' @seealso \code{plot_errorbar}
#' @description Click on a point in plot region and it will tell you what is the point.
#' @examples
#' \dontrun{
#' k <- plot_errbar(1:100, rnorm(100, 1, 2), 
#'		xlab="axe x", ylab="axe y", bty="n", xlim=c(1,100), 
#' 		errbar.x=2, errbar.y=rnorm(100, 1, 0.1))
#' show_name(k)
#' k <- plot_errbar(1:10, rnorm(10, 1, 2), 
#'		xlab="axe x", ylab="axe y", bty="n", xlim=c(1,10), 
#' 		errbar.x=2, errbar.y=rnorm(10, 1, 0.1), 
#' 		names=LETTERS[1:10])
#' show_name(k)
#' k <- plot_errbar(1:10, rnorm(10, 1, 2), 
#'		xlab="axe x", ylab="axe y", bty="n", xlim=c(1,10), 
#' 		errbar.x=2, errbar.y=rnorm(10, 1, 0.1))
#' show_name(k, names=LETTERS[1:10])
#' 		}
#' @export


show_name <- function(points=NULL, x=NULL, y=NULL, names=NULL, col="red", 
                      silent=FALSE) {
  
  if (!silent) message("Click on a point in the plot")
  if (!is.null(points)) {
    if (is.null(x)) x <- points[["x"]]
    if (is.null(y)) y <- points[["y"]]
    if (is.null(names)) names <- points[["names"]]
  }
  
  k <- locator(n=1)
  kpos <- which.min(abs(x - k$x) + 
                      abs(y - k$y))
  
  dx <- ScalePreviousPlot()$x["range"]/30
  dy <- ScalePreviousPlot()$y["range"]/30
  
  if (x[kpos] < ScalePreviousPlot()$x["center"]) {
    if (y[kpos] < ScalePreviousPlot()$y["center"]) {
      par(xpd=TRUE)
      text(x=x[kpos]+dx, 
           y=y[kpos]+dy, 
           labels = names[kpos], 
           col=col, 
           pos=4)
      segments(x0=x[kpos], x1=x[kpos]+dx, 
               y0=y[kpos], y1=y[kpos]+dy, col=col)
    } else {
      par(xpd=TRUE)
      text(x=x[kpos]+dx, 
           y=y[kpos]-dy, 
           labels = names[kpos], 
           col=col, 
           pos=4)
      segments(x0=x[kpos], x1=x[kpos]+dx, 
               y0=y[kpos], y1=y[kpos]-dy, col=col)
    }
  } else {
    if (y[kpos] < ScalePreviousPlot()$y["center"]) {
      par(xpd=TRUE)
      text(x=x[kpos]-dx, 
           y=y[kpos]+dy, 
           labels = names[kpos], 
           col=col, 
           pos=2)
      segments(x0=x[kpos], x1=x[kpos]-dx, 
               y0=y[kpos], y1=y[kpos]+dy, col=col)
    } else {
      par(xpd=TRUE)
      text(x=x[kpos]-dx, 
           y=y[kpos]-dy, 
           labels = names[kpos], 
           col=col, 
           pos=2)
      segments(x0=x[kpos], x1=x[kpos]-dx, 
               y0=y[kpos], y1=y[kpos]-dy, col=col)
    }
  }
  return(invisible(names[kpos]))
  
}

#' ChangeCoordinate returns a value in a changed coordinate
#' @title Return a value in a changed coordinate
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A value in the new system
#' @param x value to convert
#' @param initial Set of two values in the original system
#' @param transformed Set of the two values in the converted system
#' @description Return a value in a changed coordinate system.
#' @examples
#' ChangeCoordinate(x=c(10, 20), initial=c(1, 100), transformed=c(0, 1))
#' @export


ChangeCoordinate <- function(x=stop("At least one value to convert must be provided"), 
	initial=stop("Set of two values must be provided as references"),
	transformed=stop("Set of two transformed values must be provided")) {
  # x1.new = a*x1.ref+b => b = x1.new-a*x1.ref
  # x2.new = a*x2.ref+b => x2.new = a*x2.ref+ x1.new-a*x1.ref
  # x2.new-x1.new = a*(x2.ref-x1.ref)
  # a = (x2.new-x1.new) / (x2.ref-x1.ref)
  # b = x1.new-a*x1.ref
  
  x1.ref <- initial[1]
  x2.ref <- initial[2]
  x1.new <- transformed[1]
  x2.new <- transformed[2]
  
  
  a <- (x2.new-x1.new) / (x2.ref-x1.ref)
  b <- x1.new-a*x1.ref
  return(a*x+b)
}

#' ellipse plots an ellipse
#' @title Plot an ellipse
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @param center.x Center of the ellipse on x axis
#' @param center.y Center of the ellipse on y axis
#' @param radius.x Radius along the x axis
#' @param radius.y Radius along the y axis
#' @param radius.x.lower Radius along the x axis, at left of center
#' @param radius.x.upper Radius along the x axis, at right of center
#' @param radius.y.lower Radius along the y axis, at bottom of center
#' @param radius.y.upper Radius along the y axis, at top of center
#' @param alpha Rotation in radians
#' @param binconf.x A data.frame or a matrix with two columns, x and n or with three columns, PointEst, Lower, and Upper
#' @param binconf.y A data.frame or a matrix with two columns, x and n or with three columns, PointEst, Lower, and Upper
#' @param control.binconf A list with options for binomial confidence
#' @param length Number of points to draw the ellipse
#' @param ... Graphical parameters
#' @description Plot a ellipse defined by the center and the radius. The options for 
#' binomial confidence parameters are:\cr
#' - conf.level\cr
#' - method  must be one of these "wald", "wilson", "wilsoncc", "agresti-coull", "jeffreys", "modified wilson", "modified jeffreys", "clopper-pearson", "arcsine", "logit", "witting", "pratt", "midp", "lik" and "blaker". Defaults to "wilson". Abbreviation of method is accepted. See details.\cr
#' Default is wilsoncc (Wilson with continuity correction)
#' col parameter can be a list of colors. See examples
#' @examples
#' plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
#'      xlab="Variable x", ylab="variable y")
#'  
#' ellipse(center.x = c(0.2, 0.3, 0.25), center.y = c(0.7, 0.6, 0.55), 
#'         radius.x = c(0.1, 0.1, 0.1), radius.y = c(0.15, 0.2, 0.4), 
#'         border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1))
#' 
#' ellipse(center.x = 0.5, center.y = 0.5, 
#'         radius.x.lower = 0.1, radius.x.upper = 0.3, 
#'         radius.y = 0.2, 
#'         border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1))
#' 
#' ellipse(center.x = 0.6, center.y = 0.3, 
#'         radius.x.lower = 0.3, radius.x.upper = 0.3, 
#'         radius.y.lower = 0.2, radius.y.upper = 0.4, 
#'         border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1))
#' 
#' plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", bty="n", asp=1, 
#'      xlab="Variable x", ylab="variable y", axes=FALSE)
#' axis(1, at=c(0, 0.25, 0.5, 0.75, 1))
#' axis(2, at=c(0, 0.25, 0.5, 0.75, 1), las=1)
#' 
#' ellipse(center.x = 0.5, center.y = 0.5, radius.x = 0.2, radius.y = 0.4, 
#'        border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1))
#' ellipse(center.x = 0.5, center.y = 0.5, radius.x = 0.2, radius.y = 0.4, 
#'         border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1), alpha = pi/4)
#' 
#' plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
#'      xlab="Variable x", ylab="variable y")
#' 
#' for (k in 0:8)
#'   ellipse(center.x=0.5, center.y=0.5, radius.x=0.1, radius.y=0.4, 
#'           alpha=seq(from=0, to=pi/4, length=9)[k], 
#'           border=rainbow(9)[k])
#'
#' # Exemple with confidence of proportions
#' males <- c(10, 25, 3, 4)
#' N <- c(12, 52, 17, 10)
#' 
#' males2 <- c(12, 20, 3, 6)
#' N2 <- c(15, 50, 20, 12)
#' 
#' plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
#'      xlab="Variable x", ylab="variable y")
#'
#' ellipse(binconf.x = data.frame(x=males, n=N), binconf.y = data.frame(x=males2, n=N2),  
#'         border=NA, col=rgb(red = 0.1, green = 0.5, blue = 0.1, alpha = 0.1))
#'         
#' plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
#'      xlab="Variable x", ylab="variable y")
#'      
#' ellipse(binconf.x = data.frame(x=males, n=N), 
#'         binconf.y = data.frame(PointEst=c(0.1, 0.2, 0.3, 0.5), 
#'                                Lower=c(0.02, 0.12, 0.25, 0.30), 
#'                                Upper=c(0.18, 0.29, 0.35, 0.67)), 
#'         border=NA, col=rgb(red = 0.1, green = 0.5, blue = 0.1, alpha = 0.1))
#'         
#' # Examples with a gradient
#' plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
#'      xlab="Variable x", ylab="variable y")
#' ellipse(center.x = 0.6, center.y = 0.3, 
#'         radius.x.lower = 0.3, radius.x.upper = 0.3, 
#'         radius.y.lower = 0.2, radius.y.upper = 0.4, 
#'         border=NA, col=grey.colors(100, alpha = 0.1))
#'         
#' plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
#'      xlab="Variable x", ylab="variable y")
#' ellipse(binconf.x = data.frame(x=males, n=N), binconf.y = data.frame(x=males2, n=N2),  
#'         border=NA, col=grey.colors(100, alpha = 0.1))
#' 
#' @export


ellipse <- function(center.x = 0, center.y = 0, 
                    radius.x = 1, radius.y = 1, 
                    radius.x.lower=NULL, radius.x.upper=NULL, 
                    radius.y.lower=NULL, radius.y.upper=NULL, 
                    alpha = 0, 
                    binconf.x=NULL, 
                    binconf.y=NULL, 
                    control.binconf=list(conf.level = 0.95, method = "wilsoncc"), 
                    length=100, ...) {
  
  p3p <- list(...)
  
  if (!is.null(binconf.x)) {
    if ((inherits(binconf.x, "binconf")) | (ncol(binconf.x) == 3)) {
      bc.x <- binconf.x
    } else {
      control.binconf.x <- modifyList(control.binconf, c(list(x=binconf.x[, "x"]), 
                                        list(n=binconf.x[, "n"])))
      bc.x <- do.call(getFromNamespace(".BinomialConfidence", ns="HelpersMG"), control.binconf.x) 
    }
    
    center.x = bc.x[, "PointEst"]
    radius.x.lower = bc.x[, "PointEst"]-bc.x[, "Lower"]
    radius.x.upper = bc.x[, "Upper"]-bc.x[, "PointEst"] 
  }
  
  if (!is.null(binconf.y)) {
    if ((inherits(binconf.y, "binconf")) | (ncol(binconf.y) == 3))  {
      bc.y <- binconf.y
    } else {
      control.binconf.y <- modifyList(control.binconf, c(list(x=binconf.y[, "x"]), 
                                        list(n=binconf.y[, "n"])))
      bc.y <- do.call(getFromNamespace(".BinomialConfidence", ns="HelpersMG"), control.binconf.y) 
    }
    
    center.y = bc.y[, "PointEst"]
    radius.y.lower = bc.y[, "PointEst"]-bc.y[, "Lower"] 
    radius.y.upper = bc.y[, "Upper"]-bc.y[, "PointEst"] 
  }
  
  if (is.null(radius.x.lower))  radius.x.lower <- radius.x
  if (is.null(radius.y.lower))  radius.y.lower <- radius.y
  if (is.null(radius.x.upper))  radius.x.upper <- radius.x
  if (is.null(radius.y.upper))  radius.y.upper <- radius.y
  
  ncol <- 1
  if (!is.null(p3p$col)) {
    ncol <- length(p3p$col)
    p3p$col <- rev(p3p$col)
  }
  
  
  for (k in seq_along(center.x)) {
    
    cptcol <- 1
    for (nc in seq(from=1, to=1/ncol, length.out = ncol)) {
      
      theta <- seq(0, pi / 2, length=length/4)
      x <- center.x[k] + radius.x.upper[k] * nc * cos(theta) * cos(alpha) - radius.y.upper[k] * nc * sin(theta) * sin(alpha)
      y <- center.y[k] + radius.x.upper[k] * nc * cos(theta) * sin(alpha) + radius.y.upper[k] * nc * sin(theta) * cos(alpha)
      
      theta <- seq(pi / 2, pi, length=length/4)
      x <- c(x, center.x[k] + radius.x.lower[k] * nc * cos(theta) * cos(alpha) - radius.y.upper[k] * nc * sin(theta) * sin(alpha))
      y <- c(y, center.y[k] + radius.x.lower[k] * nc * cos(theta) * sin(alpha) + radius.y.upper[k] * nc * sin(theta) * cos(alpha))
      
      theta <- seq(pi, 3/2*pi, length=length/4)
      x <- c(x, center.x[k] + radius.x.lower[k] * nc * cos(theta) * cos(alpha) - radius.y.lower[k] * nc * sin(theta) * sin(alpha))
      y <- c(y, center.y[k] + radius.x.lower[k] * nc * cos(theta) * sin(alpha) + radius.y.lower[k] * nc * sin(theta) * cos(alpha))
      
      theta <- seq(3/2*pi, 2 * pi, length=length/4)
      x <- c(x, center.x[k] + radius.x.upper[k] * nc * cos(theta) * cos(alpha) - radius.y.lower[k] * nc * sin(theta) * sin(alpha))
      y <- c(y, center.y[k] + radius.x.upper[k] * nc * cos(theta) * sin(alpha) + radius.y.lower[k] * nc * sin(theta) * cos(alpha))
      
      if ((ncol == 1)) {
        do.call(polygon, modifyList(p3p, list(x=x, y=y)))
      } else {
        p3p_ec <- modifyList(p3p, list(col=p3p$col[cptcol]))
        do.call(polygon, modifyList(p3p_ec, list(x=x, y=y)))
      }
      cptcol <- cptcol + 1
    }
    
  }
}

.LD50_fit <- function(par, fixed.parameters=NULL, alive, N, doses, equation) {

  par <- c(par, fixed.parameters)
  p <- .modelLD50(par, doses, equation)
  p <- ifelse(p==0, 1E-9, p)
  p <- ifelse(p==1, 1-1E-9, p)

    if (any(is.infinite(p)) | any(is.na(p))) {return(Inf)} else {
   return(-sum(dbinom(alive, N, p, log = TRUE)))
  }
  
}

.modelLD50 <- function(par, doses, equation="logistic") {
#  if (inherits(par, "data.frame")) par <- na.omit(t(par))[,1]
#  names(parx) <- colnames(par)
#  par <- parx
  # embryogrowth:::.modelTSD(par, doses, equation)
  if (equation=="logistic")	p <- 1/(1+exp((1/par["S"])*(par["P"]-doses)))
  if (equation=="logit")	p <- 1/(1+exp(par["P"]+doses*par["S"]))
  if (equation=="probit")	p <- pnorm(par["P"]+doses*par["S"])
  if (equation=="flexit") {
    # Peut-être  encore des problèmes de exp(K1 ou K2)
    p <- flexit(x = doses, par=par)
  }
  if (equation=="richards") p <- ifelse(par["K"]>3 & sign(par["P"]-doses) == sign(par["S"]), 
                                        0.5*exp((doses-par["P"])/(par["S"]*exp(par["K"]))), 
                                        (1+(2^exp(par["K"])-1)*exp((1/par["S"])*(par["P"]-doses)))^(-1/exp(par["K"])))
  if (equation=="double-richards") p <- ifelse(doses<par["P"], 
                                               ifelse(par["K1"]>3 & sign(par["P"]-doses)==sign(par["S"]), 
                                                      0.5*exp((doses-par["P"])/(par["S"]*exp(par["K1"]))), 
                                                      (1+(2^exp(par["K1"])-1)*exp((1/par["S"])*(par["P"]-doses)))^(-1/exp(par["K1"]))) ,
                                               ifelse(par["K2"]>3 & sign(par["P"]-doses)==sign(par["S"]), 
                                                      0.5*exp((doses-par["P"])/(par["S"]*exp(par["K2"]))), 
                                                      (1+(2^exp(par["K2"])-1)*exp((1/par["S"])*(par["P"]-doses)))^(-1/exp(par["K2"])))
  )
  if (equation=="hill") if (par["P"]<=0) {p <- rep(Inf, length(doses))} else {p <- 1/(1+exp((1/par["S"])*(log(par["P"])-log(doses))))}
  if (equation=="hulin") {
    elin <- par["K1"]*doses+par["K2"]
    p <- ifelse(elin>6, 0, (1+(2^exp(elin)-1)*exp((1/par["S"])*(par["P"]-doses)))^(-1/exp(elin)))}
  return(p)
  
}

.permutations <- function (n, r, v = 1:n, set = TRUE, repeats.allowed = FALSE) 
  
  # code from the package gtools
{
  if (mode(n) != "numeric" || length(n) != 1 || n < 1 || (n%%1) != 
      0) 
    stop("bad value of n")
  if (mode(r) != "numeric" || length(r) != 1 || r < 1 || (r%%1) != 
      0) 
    stop("bad value of r")
  if (!is.atomic(v) || length(v) < n) 
    stop("v is either non-atomic or too short")
  if ((r > n) & repeats.allowed == FALSE) 
    stop("r > n and repeats.allowed=FALSE")
  if (set) {
    v <- unique(sort(v))
    if (length(v) < n) 
      stop("too few different elements")
  }
  v0 <- vector(mode(v), 0)
  if (repeats.allowed) 
    sub <- function(n, r, v) {
      if (r == 1) 
        matrix(v, n, 1)
      else if (n == 1) 
        matrix(v, 1, r)
      else {
        inner <- Recall(n, r - 1, v)
        cbind(rep(v, rep(nrow(inner), n)), matrix(t(inner), 
                                                  ncol = ncol(inner), nrow = nrow(inner) * n, 
                                                  byrow = TRUE))
      }
    }
  else sub <- function(n, r, v) {
    if (r == 1) 
      matrix(v, n, 1)
    else if (n == 1) 
      matrix(v, 1, r)
    else {
      X <- NULL
      for (i in 1:n) X <- rbind(X, cbind(v[i], Recall(n - 
                                                        1, r - 1, v[-i])))
      X
    }
  }
  sub(n, r, v[1:n])
}

#' LD50_MHmcmc_p generates set of parameters to be used with LD50_MHmcmc()
#' @title Generates set of parameters to be used with LD50_MHmcmc()
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A matrix with the parameters
#' @param result An object obtained after a LD50 fit
#' @param accept If TRUE, the script does not wait user information
#' @description Interactive script used to generate set of parameters to be 
#' used with LD50_MHmcmc().
#' @family LD50 functions
#' @examples 
#' \dontrun{
#' library("HelpersMG")
#' data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
#' Alive=c(10, 12, 8, 6, 2, 1),
#' Dead=c(0, 1, 5, 6, 9, 15))
#' LD50_logistic <- LD50(data, equation="logistic")
#' pmcmc <- LD50_MHmcmc_p(LD50_logistic, accept=TRUE)
#' }
#' @export


LD50_MHmcmc_p<-function(result=stop("An output from LD50() must be provided"), 
                       accept=FALSE) {

# d'abord je sors les paramtres  utiliser

par <- result$par

# "P"

P <- c("dnorm", par["P"], par["P"]/10, 2, par["P"]/2, par["P"]*2, ifelse(is.na(par["P"]), 29.5, par["P"]))
S <- c("dnorm", par["S"], abs(par["S"])/2, 0.5, -abs(par["S"])*2, abs(par["S"])*2, 
       ifelse(is.na(par["S"]), 0.01, par["S"]))
K <- c("dnorm", par["K"], abs(par["K"])/2, 0.5, -abs(par["K"])*2, abs(par["K"])*2, 
       ifelse(is.na(par["K"]), 0.01, par["K"]))
K1 <- c("dnorm", par["K"], abs(par["K1"])/2, 0.5, -abs(par["K1"])*2, abs(par["K1"])*2, 
        ifelse(is.na(par["K1"]), 0.01, par["K1"]))
K2 <- c("dnorm", par["K2"], abs(par["K2"])/2, 0.5, -abs(par["K2"])*2, abs(par["K2"])*2, 
        ifelse(is.na(par["K2"]), 0.01, par["K2"]))


priors <- list(P=P, S=S, K=K, K1=K1, K2=K2)

prencours <- NULL

for (i in 1:length(par)) {
	prencours <- c(prencours, priors[[names(par)[i]]])
}

parametersMCMC <- matrix(prencours, ncol=7, byrow=T)
colnames(parametersMCMC) <- c("Density", "Prior1", "Prior2", "SDProp", "Min", "Max", "Init")
rownames(parametersMCMC)<-names(par)

parametersMCMC <- as.data.frame(parametersMCMC, stringsAsFactors = FALSE)

for (i in 2:7)
  parametersMCMC[,i] <- as.numeric(parametersMCMC[,i])

parameters <- parametersMCMC

if (accept) {

	return(parameters)

} else {

	repeat {

cat("Proposition:\n")
print(parameters)
cat("Name of the parameter to change or Enter to quit:\n")
f<-scan(nmax=1, quiet=TRUE, what=character())

if (length(f)==0) f <- "q"

if (f=="q") {
	return(parameters)
	
} else {

	variable <- which(f==names(par))
	if (length(variable)==0) {
	cat("The parameter does not exist:\n")
	} else {
	print(variable)
	cat(paste("Change for the parameter ",names(par)[variable],":\n",sep=""))

	cat(paste("Distribution of the prior (Enter for default ",parameters[variable, "Density"], "):", sep=""))
	density<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(density)!=0) { parameters[variable, "Density"] <- density } else { density <- parameters[variable, "Density"] }
	
	if (density == "dunif") {
	
	cat(paste("Distribution of the prior, Minimum (Enter for default ", parameters[variable, "Prior1"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Prior1"] <- f
	cat(paste("Distribution of the prior, Maximum (Enter for default ", parameters[variable, "Prior2"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Prior2"] <- f
	
	} else {
	
	if (density == "dnorm") {
	
	cat(paste("Distribution of the prior, Mean (Enter for default ", parameters[variable, "Prior1"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Prior1"] <- f
	cat(paste("Distribution of the prior, Standard deviation (Enter for default ",parameters[variable, "Prior2"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Prior2"] <- f
	
	} else {

	cat(paste("Distribution of the prior, value 1 (Enter for default ",parameters[variable, "Prior1"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Prior1"] <- f
	cat(paste("Distribution of the prior, value 2 (Enter for default ",parameters[variable, "Prior2"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Prior2"] <- f

	}
	}
	
	
	cat(paste("SD of new proposition (Enter for default ",parameters[variable, "SDProp"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "SDProp"] <- f
	cat(paste("Minimum for the parameter (default ",parameters[variable, "Min"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Min"] <- f
	cat(paste("Maximum for the parameter (Enter for default ",parameters[variable, "Max"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Max"] <- f
	cat(paste("Initial value (Enter for default ",parameters[variable, "Init"], "):", sep=""))
	f<-scan(nmax=1, quiet=TRUE, what=character())
	if (length(f)!=0) parameters[variable, "Init"] <- f
	}

}
}

}

}

#' read_folder reads all files present in a folder
#' @title Read files present in a folder and creates a list with the content of these files
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Return a list with the data in the files of the folder (directory for windows users)
#' @param folder Where to search for files; can be or a file path or a folder path
#' @param wildcard Define which files are to be read (examples: "*.*", "*.xls", "essai*.txt"). It can be also a vector with all filenames.
#' @param file list of files 
#' @param read Function used to read file. Ex: read.delim or read.xls from gdata package
#' @param ... Parameters send to the read function
#' @description To create a list, the syntax is:\cr
#' datalist <- read_folder(folder=".", read=read.delim, header=FALSE)\cr
#' It returns an error if the folder does not exist.\cr
#' The names of the elements of the list are the filenames.\cr
#' The parameter file can be used to predefine a list of file. If file is NULL, all the files of the folder/directory are used.
#' @examples 
#' \dontrun{
#' library(HelpersMG)
#' # Read all the .csv files from the current folder/directory
#' contentaslist <- read_folder(folder=".", wildcard="*.csv", read=read.csv2)
#' # Read all the files from the current folder/directory
#' contentaslist <- read_folder(folder=".", wildcard="*.*", read=read.csv2)
#' # Read two files from the current folder/directory
#' files <- c("filename1.csv", "filename2.csv")
#' contentaslist <- read_folder(folder=".", wildcard=files, read=read.csv2)
#' # To concvert the list into a single dataframe:
#' mydf <- do.call("rbind", contentaslist)
#' }
#' @export


read_folder <- function(folder=try(file.choose(), silent=TRUE), 
                        file=NULL,
                        wildcard="*.*", read=read.delim, ...) {
  # wd <- getwd()
	if (!inherits(folder, "try-error")) {
    fi <- file.info(folder)
    if (is.na(fi$isdir)) {
      stop("Folder/directory does not exist")
    }
    if (!fi$isdir) {
      folder <- dirname(folder)
    }
    lf <- Sys.glob(file.path(folder, wildcard))
  	tp <- list(...)
	
    	if (length(lf)!=0) {
    	  # si cela plante au milieu, le dossier reste changé; ce n'est pas top
	      # previous<-getwd()
	      # setwd(folder)
	      ladd <- lapply(lf, function(lfx) do.call(read, modifyList(list(file=lfx), tp)))
      	names(ladd) <- lf
      	# setwd(previous)
      	return(ladd)
	    } else {
	      warning("No selected files in folder/directory")
	      return(invisible(list()))
	    }
	}
  # setwd(dir = wd)

}

#' specify_decimals format a number with specified number of decimals
#' @title Return a number as character with specified number of decimals
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A character
#' @param x The numbers to be formated
#' @param decimals Number of decimals to print
#' @param decimal.point Character to be used as decimal point
#' @description Return a number as character with specified number of decimals. 
#' If a is a matrix, it will return a matrix of the same size and the same attributes.
#' @examples
#' specify_decimal(x=pi, decimal.point=".")
#' specify_decimal(x=pi, decimals=4, decimal.point=".")
#' specify_decimal(x=c(pi, exp(1)), decimals=3, decimal.point=",")
#' specify_decimal(x=c(pi, exp(1)), decimal.point=",")
#' specify_decimal(x=c(pi*10, pi, pi/10, pi/100, pi/1000))
#' specify_decimal(x=c(pi=pi), decimal.point=".")
#' specify_decimal(x=matrix(pi*1:4, ncol=2), decimal.point=".")
#' m <- matrix(pi*1:4, ncol=2)
#' rownames(m) <- c("A", "B")
#' colnames(m) <- c("C", "D")
#' specify_decimal(x=m, decimal.point=".")
#' @export

specify_decimal <- function(x, 
                            decimals=NULL, 
                            decimal.point=".") {
  x_ini <- x
  cxa <- attributes(x_ini)
  if (is.character(x)) x <- as.numeric(x)
  if (is.null(decimals)) {
    na <- is.na(x)
    x[is.na(x)] <- 1
    og <- log10(abs(x))
    og <- ifelse(og == -Inf, 1, og)
    decimals <- ifelse(og > 0, 3, abs(floor(og))+3)
  } else {
    na <- rep(FALSE, length(x))
  }
  decimals <- rep(decimals, length(x))[1:length(x)]
  cx <- sapply(seq_along(x), FUN=function(i) formatC(x[i], digits = decimals[i], format = "f"))
  
  if (any(na)) cx[na] <- "NA"
  cx <- gsub("\\.", decimal.point, cx)
  x_ini <- as.character(x_ini)

  x_ini[] <- cx
  attributes(x_ini) <- cxa
  return(x_ini)
}

#' RM_list returns the list of results managment of an object
#' @title Return the list of results managment of an object.
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with the names of results stored in an object
#' @param x The object to add a results managment
#' @param RM The name of results managment stored
#' @param silent Should the results be shown ?
#' @param max.level If TRUE, will return all list element of the objects
#' @description RM_list returns the list of results managment of an object.
#' @family Results Managment
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Let an object of class objclass being created
#' obj <- list(A=100, name="My object")
#' class(obj) <- "objclass"
#' # And now I create a RM to this object
#' obj <- RM_add(x=obj, RMname="NewAnalysis1")
#' RM_list(obj)
#' obj <- RM_add(x=obj, RMname="NewAnalysis2")
#' RM_list(obj)
#' obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V1", value=100)
#' RM_get(x=obj, RMname="NewAnalysis2", valuename="V1")
#' obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V1", value=200)
#' RM_get(x=obj, RMname="NewAnalysis2", valuename="V1")
#' obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V2", value=300)
#' RM_get(x=obj, RMname="NewAnalysis2", valuename="V2")
#' RM_list(obj)
#' rmlist <- RM_list(obj, max.level=TRUE)
#' rmlist
#' }
#' @export

RM_list <- function (x=stop("An object with results managment must be provided"), 
                     RM = "RM", silent=FALSE, max.level = FALSE) {
  x <- attributes(x)[[RM]]
  rx <- list()
  for (i in names(x)) {
    ix <- list(name = i, timestamp = x[[i]]$timestamp)
    if ((length(x[[i]]) > 2) & max.level) {
      jx <- as.list(names(x[[i]])[3:length(x[[i]])])
      names(jx) <- names(x[[i]])[3:length(x[[i]])]
      ix <- c(ix, jx)
    }
    rx <- c(rx, list(ix))
    if (!silent) cat(i, " at ", x[[i]]$timestamp, "\n")
  }
  names(rx) <- names(x)
  return(invisible(rx))
}

#' convert.tz Convert one Date-Time from one timezone to another
#' @title Convert one Date-Time from one timezone to another
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A POSIXlt or POSIXct date converted
#' @param x The date-time in POSIXlt or POSIXct format
#' @param tz The timezone
#' @description Convert one Date-Time from one timezone to another.\cr
#' Available timezones can be shown using OlsonNames().
#' @seealso Function \code{with_tz()} from \code{lubridate} package does the same. I keep it here only for compatibility with old scripts.
#' @examples
#' d <- as.POSIXlt("2010-01-01 17:34:20", tz="UTC")
#' convert.tz(d, tz="America/Guatemala")
#' @export


convert.tz <- function(x, tz=Sys.timezone()) {
d <- as.POSIXct(format(as.POSIXct(x), tz=tz, usetz=TRUE), tz=tz)
if (inherits(c, "POSIXlt")) d <- as.POSIXlt(d)
return(d)
}

#' barplot_errbar plot a barplot with error bar on y
#' @title Plot a barplot graph with error bar on y
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A numeric vector (or matrix, when beside = TRUE), say mp, giving the coordinates of all the bar midpoints drawn, useful for adding to the graph.\cr
#' If beside is true, use colMeans(mp) for the midpoints of each group of bars, see example.
#' @param ... Parameters for barplot() such as main= or ylim=
#' @param errbar.y The length of error bars for y. Recycled if necessary.
#' @param errbar.y.plus The length of positive error bars for y. Recycled if necessary.
#' @param errbar.y.minus The length of negative error bars for y. Recycled if necessary.
#' @param y.plus The absolut position of the positive error bar for y. Recycled if necessary.
#' @param y.minus The absolut position of the nagative error bar for y. Recycled if necessary.
#' @param errbar.tick Size of small ticks at the end of error bars defined as a proportion of total width or height graph size.
#' @param errbar.lwd Error bar line width, see par("lwd")
#' @param errbar.lty Error bar line type, see par("lwd")
#' @param errbar.col Error bar line color, see par("col")
#' @param add If true, add the graph to the previous one.
#' @family plot and barplot functions
#' @description To plot data, just use it as a normal barplot but add the errbar.y 
#' values or errbar.y.minus, errbar.y.plus if bars for y axis are 
#' asymetric. Use y.plus and y.minus to set absolut limits for
#' error bars. Note that y.plus and y.minus have priority over errbar.y, 
#' errbar.y.minus and errbar.y.plus.
#' @seealso \code{plot_errorbar}
#' @examples
#' \dontrun{
#' barplot_errbar(rnorm(10, 10, 3), 
#'		xlab="axe x", ylab="axe y", bty="n", 
#' 		errbar.y.plus=rnorm(10, 1, 0.1), col=rainbow(10), 
#' 		names.arg=paste("Group",1:10), cex.names=0.6)
#' y <- rnorm(10, 10, 3)
#' barplot_errbar(y, 
#'                	xlab="axe x", ylab="axe y", bty="n", 
#'             		y.plus=y+2)
#' }
#' @export


barplot_errbar <- function(..., 
                        errbar.y=NULL, 
                        errbar.y.plus=NULL, errbar.y.minus=NULL,
                        y.plus=NULL, y.minus=NULL,
                        errbar.tick=1/50, 
                        errbar.lwd=par("lwd"), 
                        errbar.lty=par("lty"), 
                        errbar.col=par("fg"), 
                        add=FALSE) 
  {
# errbar.y=NULL; errbar.y.plus=NULL; errbar.y.minus=NULL; y.plus=NULL; y.minus=NULL; errbar.tick=1/50; errbar.lwd=par("lwd"); errbar.lty=par("lty"); errbar.col=par("fg"); add=FALSE 

  par.plot <- list(...)
  if (add) {
  	s <- ScalePreviousPlot()
  	par(new=TRUE)
  	par.plot <- modifyList(par.plot, list(xlim=s$xlim, ylim=s$ylim, xlab="", ylab="", main="", axes=FALSE))
  }


  essai <- do.call(barplot, par.plot)
  x <- as.numeric(essai) 
  y <- par.plot[[1]]
  
  if (!is.null(y.plus)) errbar.y.plus <- y.plus-y
  if (!is.null(y.minus)) errbar.y.minus <- y-y.minus
  
  if (is.null(errbar.y.minus) & !is.null(errbar.y)) {
  	errbar.y.minus <- errbar.y
  }
  if (is.null(errbar.y.plus) & !is.null(errbar.y)) {
  	errbar.y.plus <- errbar.y
  }
    
  sizebar <- (par("usr")[2]-par("usr")[1])*errbar.tick
  
  par(xpd=TRUE)
  
  if (!is.null(errbar.y.minus)) {
    segments(x, y-errbar.y.minus, x, y, 
             col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
    segments(x-sizebar, y-errbar.y.minus, x+sizebar, y-errbar.y.minus, 
             col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
  }
  if (!is.null(errbar.y.plus)) {
    segments(x, y+errbar.y.plus, x, y, 
             col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
    segments(x-sizebar, y+errbar.y.plus, x+sizebar, y+errbar.y.plus, 
             col=errbar.col, lty=errbar.lty, lwd=errbar.lwd)
  }

  # 2020-11-08
  par(xpd=FALSE)
  
  return(invisible(essai))
}

#' ExtractAIC.glm returns AIC, AICc or BIC from a glm object
#' @title Return AIC, AICc or BIC from a glm object
#' @author Modified from stats:::extract.AIC.glm
#' @return A numeric named vector of length 2, with first and second elements giving\cr
#' edf	the ‘equivalent degrees of freedom’ for the fitted model fit.\cr
#' x	the Information Criterion for fit.
#' @param fit fitted model, the result of a fitter glm.
#' @param scale unused for glm.
#' @param k numeric specifying the ‘weight’ of the equivalent degrees of freedom (=: edf) part in the AIC formula.
#' @param ... further arguments (currently unused because addterm.glm and dropterm.glm using this function do not transmit them).
#' @description For glm fits the family's aic() function is used to compute the AIC.\cr
#' The choice between different criteria is done by setting a global option AIC. It can be checked using show.option=TRUE.
#' Indeed, it is not possible to use the ... parameter due to a bug in some functions of MASS package. If you want to use this function as a replacement for setpAIC(), do extractAIC.glm <- ExtractAIC.glm before.
#' @family AIC
#' @examples
#' \dontrun{
#' extractAIC.glm <- ExtractAIC.glm
#' n <- 100
#' x <- rnorm(n, 20, 2)
#' A <- rnorm(n, 20, 5)
#' g <- glm(x ~ A)
#' extractAIC(g, show.option=TRUE)
#' options(AIC="AIC")
#' extractAIC(g)
#' options(AIC="BIC")
#' extractAIC(g)
#' options(AIC="AICc")
#' extractAIC(g)
#' }
#' @export

ExtractAIC.glm <- function (fit, scale = 0, k = 2, ...) {
  p3p <- tryCatch(list(...), error=function(e) list())
  # print(str(p3p))
  type <- getOption("AIC")
  if (is.null(type)) type <- "AIC (default)"
  if (!identical(p3p, list())) {
    if (p3p[["show.option"]]) print(type)
  }
  n <- length(fit$residuals)
  edf <- n - fit$df.residual
  aic <- fit$aic
  if (((type == "AIC (default)")) | (type == "AIC")) return(c(edf, AIC=aic + (k - 2) * edf))
  if (type == "BIC") return(c(edf, BIC=aic + (log(n) - 2) * edf))
  if (type == "AICc") return(c(edf, AICc=aic + (2*edf*(edf+1))/(n-edf-1)))
  if (type == "QAIC") return(c(edf, QAIC=NA))
  if (type == "AICc") return(c(edf, QAICc=NA))
  if (type == "TIC") return(c(edf, TIC=NA))
  return(c(edf, NA))
}

#' symbol.Female plot a female symbol in the plotting region
#' @title Plot a female symbol in the plotting region
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @param centerx The x position of the center of the circle
#' @param centery The y position of the center of the circle
#' @param rayonx The size of the rayon in the scale of the x axis
#' @param lwd The width of the line of the symbol
#' @param col The color of the symbol
#' @description Plot a female symbol in the plotting region.
#' @family Symbol
#' @examples
#' \dontrun{
#' plot(x=1:2, y=c(10,20), type="n", bty="n", xlab="", ylab="")
#' 
#' rayonx <- 0.01
#' centerx <- 1.2
#' centery <- 15
#' 
#' symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx)
#' symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx)
#' 
#' rayonx <- 0.03
#' centerx <- 1.2
#' centery <- 18
#' 
#' symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx, lwd=3)
#' symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx, lwd=3, col="red")
#' 
#' rayonx <- 0.05
#' centerx <- 1.4
#' centery <- 13
#' 
#' symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx, lwd=4, col="blue")
#' symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx, lwd=4, col="red")
#' }
#' @export


symbol.Female <- function(centerx, centery, rayonx, lwd=2, col="black") {
  xr <- ScalePreviousPlot()$xlim["range"]
  yr <- ScalePreviousPlot()$ylim["range"]
  ratio <- par("pin")[1]/par("pin")[2]
  rayony <- rayonx*(yr/xr)*ratio
  
  angle <- seq(from=0, to=2*pi, length.out = 20)
  x <- centerx+rayonx*cos(angle)
  y <- centery+rayony*sin(angle)
  segments(x0=x, y0=y, x1=x[c(20, 1:19)], y1=y[c(20, 1:19)], col=col, lwd=lwd)

  segments(x0=centerx, y0=centery-rayony, x1=centerx, y1=centery-2.5*rayony, col=col, lwd=lwd)
  segments(x0=centerx-rayonx, y0=centery-rayony-(2.5*rayony-rayony)/2, 
           x1=centerx+rayonx, y1=centery-rayony-(2.5*rayony-rayony)/2, col=col, lwd=lwd)
  
}

#' LD50_MHmcmc runs the Metropolis-Hastings algorithm for LD50 (Bayesian MCMC)
#' @title Metropolis-Hastings algorithm for LD50
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with resultMCMC being mcmc.list object, resultLnL being likelihoods and parametersMCMC being the parameters used
#' @param n.iter Number of iterations for each step
#' @param parametersMCMC A set of parameters used as initial point for searching with information on priors
#' @param result An object obtained after a SearchR fit
#' @param n.chains Number of replicates
#' @param n.adapt Number of iterations before to store outputs
#' @param thin Number of iterations between each stored output
#' @param trace True or False, shows progress
#' @param batchSize Number of observations to include in each batch fo SE estimation
#' @param adaptive Should an adaptive process for SDProp be used
#' @param adaptive.lag  Lag to analyze the SDProp value in an adaptive content
#' @param adaptive.fun Function used to change the SDProp
#' @param filename If intermediate is not NULL, save intermediate result in this file
#' @param intermediate Period for saving intermediate result, NULL for no save
#' @param previous Previous result to be continued. Can be the filename in which intermediate results are saved.
#' @description Run the Metropolis-Hastings algorithm for tsd.\cr
#' Deeply modified from a MCMC script by Olivier Martin (INRA, Paris-Grignon).\cr
#' The number of iterations is n.iter+n.adapt+1 because the initial likelihood is also displayed.\cr
#' I recommend that thin=1 because the method to estimate SE uses resampling.\cr
#' If initial point is maximum likelihood, n.adapt = 0 is a good solution.\cr
#' To get the SE from result_mcmc <- tsd_MHmcmc(result=try), use:\cr
#' result_mcmc$BatchSE or result_mcmc$TimeSeriesSE\cr
#' The batch standard error procedure is usually thought to be not as accurate as the time series methods.\cr
#' Based on Jones, Haran, Caffo and Neath (2005), the batch size should be equal to sqrt(n.iter).\cr
#' Jones, G.L., Haran, M., Caffo, B.S. and Neath, R. (2006) Fixed Width Output Analysis for Markov chain Monte Carlo , Journal of the American Statistical Association, 101:1537-1547.\cr
#' coda package is necessary for this function.\cr
#' The parameters intermediate and filename are used to save intermediate results every 'intermediate' iterations (for example 1000). Results are saved in a file of name filename.\cr
#' The parameter previous is used to indicate the list that has been save using the parameters intermediate and filename. It permits to continue a mcmc search.\cr
#' These options are used to prevent the consequences of computer crash or if the run is very very long and processes at time limited.\cr
#' @family LD50 functions
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
#' Alive=c(10, 12, 8, 6, 2, 1),
#' Dead=c(0, 1, 5, 6, 9, 15))
#' LD50_logistic <- LD50(data, equation="logistic")
#' pMCMC <- LD50_MHmcmc_p(LD50_logistic, accept=TRUE)
#' # Take care, it can be very long
#' result_mcmc_LD50 <- LD50_MHmcmc(result=LD50_logistic, 
#' 		parametersMCMC=pMCMC, n.iter=10000, n.chains = 1,  
#' 		n.adapt = 0, thin=1, trace=1000, adaptive=TRUE, )
#' # summary() permits to get rapidly the standard errors for parameters
#' summary(result_mcmc_LD50)
#' plot(x=result_mcmc_LD50, parameters="S", scale.prior=TRUE, las=1)
#' plot(result_mcmc_LD50, parameters="S", scale.prior=TRUE, las=1, xlim=c(-20, 20))
#' plot(result_mcmc_LD50, parameters="P", scale.prior=TRUE, las=1)
#' 1-rejectionRate(as.mcmc(result_mcmc_LD50))
#' raftery.diag(as.mcmc(result_mcmc_LD50))
#' heidel.diag(as.mcmc(result_mcmc_LD50))
#' 
#' #### Example with Uniforms priors
#' 
#' pMCMC <- structure(list(Density = c("dunif", "dunif"), 
#' Prior1 = c(77.6216005852911, -31.0438095277258), 
#' Prior2 = c(310.486402341165, 31.0438095277258), 
#' SDProp = c(2, 0.5), 
#' Min = c(77.6216005852911, -31.0438095277258), 
#' Max = c(310.486402341165, 31.0438095277258), 
#' Init = c(155.243201170582, -15.5219047638629)), 
#' row.names = c("P", "S"), class = "data.frame")
#' result_mcmc_LD50 <- LD50_MHmcmc(result=LD50_logistic, 
#' 		parametersMCMC=pMCMC, n.iter=10000, n.chains = 1,  
#' 		n.adapt = 0, thin=1, trace=1000, adaptive=TRUE, )
#' # summary() permits to get rapidly the standard errors for parameters
#' summary(result_mcmc_LD50)
#' plot(x=result_mcmc_LD50, parameters="S", scale.prior=TRUE, las=1)
#' plot(result_mcmc_LD50, parameters="S", scale.prior=TRUE, las=1, xlim=c(-40, 40))
#' plot(result_mcmc_LD50, parameters="P", scale.prior=TRUE, las=1)
#' 1-rejectionRate(as.mcmc(result_mcmc_LD50))
#' raftery.diag(as.mcmc(result_mcmc_LD50))
#' heidel.diag(as.mcmc(result_mcmc_LD50))
#' 
#' }
#' @export

LD50_MHmcmc <- function(result=stop("A result of LD50() fit must be provided"), n.iter=10000, 
                        parametersMCMC=NULL, n.chains = 1, n.adapt = 0, 
                        thin=1, trace=FALSE, batchSize=sqrt(n.iter), 
                        adaptive=FALSE, adaptive.lag=500, 
                        adaptive.fun=function(x) {ifelse(x>0.234, 1.3, 0.7)},
                        intermediate=NULL, filename="intermediate.Rdata", previous=NULL) {
  
  # result=eo_logistic; parametersMCMC=NULL; 
  # n.iter=10000; n.chains = 1;  n.adapt = 0; thin=1; trace=TRUE; batchSize=sqrt(n.iter);intermediate=NULL; filename="intermediate.Rdata"; previous=NULL; adaptive=FALSE; adaptive.lag=500; adaptive.fun=function(x) {ifelse(x>0.234, 1.3, 0.7)}
  
  if (is.character(previous)) {
    itr <- NULL
    load(previous)
    previous <- itr
    rm(itr)
    print("Continue previous mcmc run")
  } else {
    print(parametersMCMC)
  }
  
  
  # 29/1/2014; Ajout de result$weight
  # 30/1/2015 Ajout de fixedparameters
  out <- MHalgoGen(n.iter=n.iter, parameters=parametersMCMC, 
                   parameters_name = "par", 
                   n.chains = n.chains, n.adapt = n.adapt, thin=thin, trace=trace, 
                   alive=result$alive, N=result$N, doses=result$doses, 
                   equation=result$equation, fixed.parameters=result$fixed.parameters, 
                   likelihood= getFromNamespace(".LD50_fit", ns="HelpersMG"), 
                   adaptive=adaptive, adaptive.lag=adaptive.lag, adaptive.fun=adaptive.fun,
                   intermediate=intermediate, filename=filename, previous=previous)
  
  if (batchSize>=n.iter/2) {
    print("batchSize cannot be larger than half the number of iterations.")
    rese <- rep(NA, dim(parametersMCMC)[1])
    names(rese) <- rownames(parametersMCMC)
    out <- c(out, SE=list(rese))
  } else {
    out <- c(out, BatchSE=list(coda::batchSE(out$resultMCMC, batchSize=batchSize)))
  }
  
  # pas clair si c'est utile
  out <- addS3Class(out, "mcmcComposite")
  
  fin <- try(summary(out), silent=TRUE)
  
  if (inherits(fin, "try-error")) {
    lp <- rep(NA, nrow(out$parametersMCMC$parameters))
    names(lp) <- rownames(out$parametersMCMC$parameters)
    out <- c(out, TimeSeriesSE=list(lp))
    out <- c(out, SD=list(lp))
  } else {
    out <- c(out, TimeSeriesSE=list(fin$statistics[,4]))
    out <- c(out, SD=list(fin$statistics[,"SD"]))
  }
  
  out <- addS3Class(out, "mcmcComposite")
  
  return(out)
                        }

#' newcompassRose Display a compass rose
#' @title Display a compass rose
#' @author modified from Jim Lemon; See [sp::compassRose()]
#' @return none
#' @param x The position of the center of the compass rose in user units.
#' @param y The position of the center of the compass rose in user units.
#' @param rot Rotation for the compass rose in degrees. See Details.
#' @param cex The character expansion to use in the display.
#' @param col The color of text
#' @param col.arrows.light The color of lighter lines
#' @param col.arrows.dark The color of darker lines
#' @description Displays a basic compass rose, usually to orient a map.\cr
#' newcompassRose displays a conventional compass rose at the position requested.\cr 
#' The size of the compass rose is determined by the character expansion, 
#' as the central "rose" is calculated relative to the character size.\cr
#' Rotation is in degrees counterclockwise.
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' require("maps")
#' map("world", "China")
#' newcompassRose(x=110, y=35, col.arrows.light="grey")
#' }
#' @export


newcompassRose <- function (x, y, rot = 0, cex = 1, col="black", 
	col.arrows.light="white", col.arrows.dark="black") 
{
  oldcex <- par(cex = cex)
  mheight <- strheight("M")
  xylim <- par("usr")
  plotdim <- par("pin")
  xmult <- (xylim[2] - xylim[1])/(xylim[4] - xylim[3]) * plotdim[2]/plotdim[1]
  point.angles <- seq(0, 2 * pi, by = pi/4) + pi * rot/180
  crspans <- rep(c(mheight * 3, mheight/2), length.out = 9)
  xpoints <- cos(point.angles) * crspans * xmult + x
  ypoints <- sin(point.angles) * crspans + y
  for (point in 1:8) {
    pcol <- ifelse(point%%2, col.arrows.dark, col.arrows.light)
    polygon(c(xpoints[c(point, point + 1)], x), c(ypoints[c(point, 
                                                            point + 1)], y), col = pcol)
  }
  txtxpoints <- cos(point.angles[c(1, 3, 5, 7)]) * 1.2 * crspans[1] * 
    xmult + x
  txtypoints <- sin(point.angles[c(1, 3, 5, 7)]) * 1.2 * crspans[1] + 
    y
  text(txtxpoints, txtypoints, c("E", "N", "W", "S"), col=col)
  par(oldcex)
}

#' logLik.cutter return log likelihood of a cutter fitted model
#' @title Return log likelihood of a cutter fitted model
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Nothing
#' @param object A result file generated by cutter
#' @param ... Not used
#' @description Return log likelihood of a cutter fitted model.
#' @family Distributions
#' @examples
#' \dontrun{
#' # ___________________________________________________________________
#' # Test for similarity in gamma left censored distribution between two
#' # datasets
#' # ___________________________________________________________________
#' obc1 <- rgamma(100, scale=20, shape=2)
#' # Detection limit for sample 1 to 50
#' LDL <- 10
#' # remove the data below the detection limit
#' obc1[obc1<LDL] <- -Inf
#' obc2 <- rgamma(100, scale=10, shape=2)
#' # remove the data below the detection limit
#' obc2[obc2<LDL] <- -Inf
#' # search for the parameters the best fit these censored data
#' result1 <- cutter(observations=obc1, 
#'                            lower_detection_limit=LDL, 
#'                           cut_method="censored")
#' logLik(result1)
#' result2 <- cutter(observations=obc2, 
#'                            lower_detection_limit=LDL, 
#'                           cut_method="censored")
#' logLik(result2)
#' result_totl <- cutter(observations=c(obc1, obc2), 
#'                            lower_detection_limit=LDL, 
#'                           cut_method="censored")
#' logLik(result_totl)
#' compare_AICc(Separate=list(result1, result2), 
#'             Common=result_totl, factor.value=1)
#' compare_BIC(Separate=list(result1, result2), 
#'             Common=result_totl, factor.value=1) 
#' }
#' @method logLik cutter
#' @export


logLik.cutter <- function(object, ...) {

  x <- object$value

  attributes(x) <- list(nall=NULL, 
                        df=length(object$par), 
                        nobs=nrow(object$observations))
  return(x)
}

#' dcutter returns the density of the cutter function
#' @title Distribution of the fitted distribution without cut. 
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return The density of the cutter function according to observations.
#' @param par Values for parameters of distribution 
#' @param observations The observations; see description.
#' @param distribution Can be gamma, normal, weibull, lognormal, or generalized.gamma.
#' @param n.mixture Number of distributions
#' @param debug If TRUE, show some information. If 2, show more information.
#' @param limits.lower Value for lower detection limit 
#' @param limits.upper Value for upper detection limit
#' @param log If TRUE, return the log likelihood
#' @description If observations must be a data.frame with 4 columns:\cr
#' observations: A column for the measurements;\cr
#' LDL: A column for the lower detection limit;\cr
#' UDL: A column for the upper detection limit;\cr
#' Cut: A column for the truncated of censored nature of the data.\cr
#' @family Distributions
#' @examples
#' \dontrun{
#' library(HelpersMG)
#' par <- c('shape1' = 0.42265849507444225, 
#'          'scale1' = 14.139457094879594, 
#'          'shape2' = 1.667131542489706, 
#'          'scale2' = 0.10763344388223803, 
#'          'p1' = 0.12283307526788023)
#' obs <- data.frame(Observations=c(0.755, 1.013, 2.098, 6.265, 4.708, 0.078, 2.169, 0.403, 1.251, 
#'                                  0.008, 1.419, 1.078, 2.744, 81.534, 1.426, 13.486, 7.813, 0.165, 
#'                                  0.118, 0.864, 0.369, 7.159, 2.605, 1.579, 1.646, 0.484, 4.492, 
#'                                  0.139, 0.28, 0.154, 0.106, 0.104, 4.185, 0.735, 0.149, 0.183, 
#'                                  0.062, 8.246, 0.165, 0.121, 0.109, 0.092, 0.162, 0.108, 0.139, 
#'                                  0.141, 0.124, 0.124, 0.151, 0.141, 0.364, 0.295, 0.09, 0.135, 
#'                                  0.154, 0.218, 0.167, -Inf, 0.203, 0.228, 0.107, 0.162, 0.194, 
#'                                  0.322, 0.351, 0.17, 0.236, 0.176, 0.107, 0.12, 0.095, 0.27, 0.194, 
#'                                  0.125, 0.123, 0.085, 0.164, 0.106, 0.079, 0.162), 
#'                  LDL=0.001, UDL=NA, Cut="censored")
#' dcutter(par=par, observations=obs, distribution="gamma", 
#'         n.mixture=NULL, debug=FALSE, limits.lower=NULL, 
#'         limits.upper=NULL,log=FALSE)
#' dcutter(par=par, observations=obs, distribution="gamma", 
#'         n.mixture=NULL, debug=FALSE, limits.lower=NULL, 
#'         limits.upper=NULL, log=TRUE)
#'         
#' }
#' @export

dcutter <- function(par, observations=NULL, distribution="gamma", 
                    n.mixture=NULL, debug=FALSE, limits.lower=NULL, 
                    limits.upper=NULL, log = TRUE) {
  
  # par=NULL; observations=NULL; distribution="gamma"; n.mixture=NULL; debug=FALSE
  # limits.lower=NULL; limits.upper=NULL; log = TRUE
  
  # par=result$par
  # observations=result$observations
  # distribution=result$distribution
  # n.mixture=result$n.mixture
  # debug=2
  # limits.lower=NULL; limits.upper=NULL; log = TRUE
  
  
  getparcutter <- getFromNamespace(".getparcutter", ns="HelpersMG")
  debug <- as.numeric(debug)
  
  if (debug == 2) print(par)
  
  if (!is.null(limits.lower)) {
    if (any(limits.lower > par)) {
      if (debug >= 1) {
        np <- names(par[limits.lower > par])
        for (i in np) {
          cat(paste0("Parameter ", i," is lower than lower limit during search", "\n"))
          cat(paste0("The parameter was ", as.character(par[i]), " and "))
          cat(paste0("the lower limit was ", as.character(limits.lower[i]), 
                     " (difference was ", specify_decimal(limits.lower[i]-par[i]), ")\n"))
        }
      }
      return(-1E6)
    }
  }
  
  
  if (!is.null(limits.upper)) {
    if (any(limits.upper < par)) {
      if (debug >= 1) {
        np <- names(par[limits.upper < par])
        for (i in np) {
          cat(paste0("Parameter ", i," is higher than upper limit during search", "\n"))
          cat(paste0("The parameter was ", as.character(par[i]), " and "))
          cat(paste0("the upper limit was ", as.character(limits.upper[i]), 
                     " (difference was ", specify_decimal(-limits.upper[i]+par[i]), ")\n"))
        }
      }
      return(-1E6)
    }
  }
  
  # par <- c('shape1' = 1, 'scale1' = 3, 'shape2' = 4, 'scale2' = 6, 'p1' = 0.1)
  # par <- c('shape' = 1, 'scale' = 3)
  # observations <- data.frame(Observations=1:10, Cut=rep("censored", 10), LDL=rep(0.01, 10), UDL=rep(NA, 10))
  # distribution="gamma"
  # debug=FALSE
  
  # dcutter(par=result2$par, observations=result2$observations, distribution=result2$distribution, n.mixture=result2$n.mixture, debug=FALSE)
  
  
  
  # gsub(".+([0-9]).+", "\\1", names(par))
  
  if (is.null(n.mixture)) {
    np <- unique(as.numeric(gsub("[a-zA-Z]+([0-9]*)$", "\\1", names(par))))
    if (all(is.na(np))) np <- 0 else np <- max(np, na.rm=TRUE)
    
    if (np == 0) {
      names(par) <- paste0(names(par), "1")
      np <- 1
    }
  } else {
    np <- n.mixture
  }
  
  LDL <- observations[, "LDL"]
  UDL <- observations[, "UDL"]
  Cut <- observations[, "Cut"]
  obs <- observations[, "Observations"]
  
  obs_finite <- is.finite(obs)
  LDLX <- LDL[obs_finite]
  UDLX <- UDL[obs_finite]
  CutX <- Cut[obs_finite]
  obsX <- obs[obs_finite]
  
  parX <- as.list(par)
  pparX <- getparcutter(par, set=NULL)
  
  ddistr <- switch(EXPR = distribution, gamma=dgamma, lognormal=dlnorm, 
                   normal=dnorm, weibull=dweibull, 
                   generalized.gamma=dggamma)
  
  pdistr <- switch(EXPR = distribution, gamma=pgamma, lognormal=plnorm, 
                   normal=pnorm, weibull=pweibull, 
                   generalized.gamma=pggamma)
  
  
  # L <- Inf
  
  if (np == 1) {
    
    parX_mixture <- getparcutter(parX, set=1)
    
    L <- sum(do.call("ddistr", args = modifyList(list(x=obsX, log = TRUE), parX_mixture)))
    L <- L - sum(log(1 - ifelse(!is.na(LDLX), do.call("pdistr", args = modifyList(list(q=LDLX, lower.tail = TRUE, log.p = FALSE), parX_mixture)), 0) 
                   -ifelse(!is.na(UDLX), do.call("pdistr", args = modifyList(list(q=UDLX, lower.tail = FALSE, log.p = FALSE), parX_mixture)), 0)))
    
    if (debug >= 1) {
      cat(paste0("The log likelihood of quantifiable data is ", specify_decimal(L), "\n"))
    }
    
    
    if (any(observations$Cut == "censored")) {
      cs <- subset(observations, subset = (Cut == "censored"))
      fby1 <- factor(cs$LDL, exclude = "")
      fby2 <- factor(cs$UDL, exclude = "")
      
      d1 <- aggregate(x=cs, by=list(fby1, fby2), FUN=function(x) sum(is.finite(x)))
      dplus <- aggregate(x=cs, by=list(fby1, fby2), FUN=function(x) sum(x == +Inf))
      dmoins <- aggregate(x=cs, by=list(fby1, fby2), FUN=function(x) sum(x == -Inf))
      
      NA_character <- structure(1L, levels = NA_character_, class = "factor")
      
      for (i in 1:nrow(d1)) {
        if ((!identical(d1[i, "Group.1"], NA_character)) | 
            (!identical(d1[i, "Group.2"], NA_character))) {
          # J'en ai au moins un qui a une valeur
          if ((!identical(d1[i, "Group.1"], NA_character)) & 
              (identical(d1[i, "Group.2"], NA_character))) {
            # J'ai une valeur sur LDL; donc je suis en left censored
            right <- d1[i, "Observations"]
            
            pright <- do.call(pdistr, modifyList(list(q=as.numeric(as.character(d1[i, "Group.1"])), lower.tail = FALSE, log.p = FALSE), parX_mixture))
            
            left <- dmoins[i, "Observations"]
            pleft <- 1-pright
            if (pleft <= 0) pleft <- 1E-6
            if (pleft >= 1) pleft <- 1-1E-6
            Lint <- dbinom(x=left, size=left+right, prob=pleft, log=TRUE)
            # if (!is.finite(Lint)) {
            #   message(paste0("L from binomial is infinite: ", d(par)))
            #   Lint <- 0
            # }
          } else {
            if ((identical(d1[i, "Group.1"], NA_character)) & 
                (!identical(d1[i, "Group.2"], NA_character))) {
              # J'ai une valeur sur UDL; donc je suis en right censored
              left <- d1[i, "Observations"]
              right <- dplus[i, "Observations"]
              pleft <- do.call(pdistr, modifyList(list(q=as.numeric(as.character(d1[i, "Group.2"])), lower.tail = TRUE, log.p = FALSE), parX_mixture))
              if (pleft <= 0) pleft <- 1E-6
              if (pleft >= 1) pleft <- 1-1E-6
              Lint <- dbinom(x=left, size=left+right, prob=pleft, log=TRUE)
              # if (!is.finite(Lint)) {
              #   message(paste0("L from binomial is infinite: ", d(par)))
              #   Lint <- 0
              # }
            } else {
              # if ((!identical(d1[i, "Group.1"], NA_character)) & 
              #     (!identical(d1[i, "Group.2"], NA_character))) {
              # J'ai une valeur sur LDL et UDL; donc je suis en left & right censored
              left <- dmoins[i, "Observations"]
              center <- d1[i, "Observations"]
              right <- dplus[i, "Observations"]
              
              pright <- do.call(pdistr, modifyList(list(q=as.numeric(as.character(d1[i, "Group.2"])), lower.tail = FALSE, log.p = FALSE), parX_mixture))
              pleft <- do.call(pdistr, modifyList(list(q=as.numeric(as.character(d1[i, "Group.1"])), lower.tail = TRUE, log.p = FALSE), parX_mixture))
              if (pright <= 0) pright <- 1E-6
              if (pright >= 1) pright <- 1-1E-6
              if (pleft <= 0) pleft <- 1E-6
              if (pleft >= 1) pleft <- 1-1E-6
              pcenter <- 1-pleft-pright
              if (pcenter <= 0) pcenter <- 1E-6
              if (pcenter >= 1) pcenter <- 1-1E-6
              Lint <- dmultinom(x=c(left, center, right), size = left+center+right, prob=c(pleft, pcenter, pright), log = TRUE)
            }
          }
          
          
          if (debug >= 1) {
            cat(paste0("The log likelihood of unquantifiable data LDL=", d1[i, "Group.1"]," and UDL=", d1[i, "Group.2"]," is ", specify_decimal(Lint), "\n"))
          }
          
          L <- L + Lint
          
        }
        
      }
      # fin du censored pour np = 1
    }
    # fin du np == 1
  } else {
    # Je fais chaque individu
    
    parX_mixture <- list()
    for (p in 1:np)
      parX_mixture <- c(parX_mixture, list(getparcutter(parX, set=p)))
    
    if (length(obsX) > 0) {
    L <- sapply(1:length(obsX), FUN = function(n) {
      L_set <- sapply(1:np, FUN = function(p) {
        pr <- do.call(ddistr, args = modifyList(list(x=obsX[n], log = FALSE), parX_mixture[[p]]))
        pLDL <- 0
        if (!is.na(LDLX[n])) pLDL <- do.call(pdistr, args = modifyList(list(q=LDLX[n], lower.tail = TRUE, log.p = FALSE), parX_mixture[[p]]))
        pUDL <- 0
        if (!is.na(UDLX[n])) pUDL <- do.call(pdistr, args = modifyList(list(q=UDLX[n], lower.tail = FALSE, log.p = FALSE), parX_mixture[[p]]))
        
        prcond <- pr / (1 - pLDL - pUDL)
        return(pparX[p] * prcond)
      })
      return(log(sum(L_set)))
    })
    # Dans L j'ai log L
    L <- sum(L)
    } else {
      L <- 0
    }
    
    if (debug >= 1) {
      cat(paste0("The log likelihood of quantifiable data is ", specify_decimal(L), "\n"))
    }
    
    # Là je dois traiter le censored
    
    if (any(observations$Cut == "censored")) {
      cs <- subset(observations, subset = (Cut == "censored"))
      fby1 <- factor(cs$LDL, exclude = "")
      fby2 <- factor(cs$UDL, exclude = "")
      
      d1 <- aggregate(x=cs, by=list(fby1, fby2), FUN=function(x) sum(is.finite(x)))
      dplus <- aggregate(x=cs, by=list(fby1, fby2), FUN=function(x) sum(x == +Inf))
      dmoins <- aggregate(x=cs, by=list(fby1, fby2), FUN=function(x) sum(x == -Inf))
      
      NA_character <- structure(1L, levels = NA_character_, class = "factor")
      
      for (i in 1:nrow(d1)) {
        if ((!identical(d1[i, "Group.1"], NA_character)) | 
            (!identical(d1[i, "Group.2"], NA_character))) {
          # J'en ai au moins un qui a une valeur
          if ((!identical(d1[i, "Group.1"], NA_character)) & 
              (identical(d1[i, "Group.2"], NA_character))) {
            # J'ai une valeur sur LDL; donc je suis en left censored
            right <- d1[i, "Observations"]
            pright <- NULL
            for (p in 1:np) 
              pright <- c(pright, do.call("pdistr", modifyList(list(q=as.numeric(as.character(d1[i, "Group.1"])), lower.tail = FALSE, log.p = FALSE), parX_mixture[[p]] )))
            pright <- sum(pright * pparX)
            
            left <- dmoins[i, "Observations"]
            pleft <- 1-pright
            if (pleft <= 0) pleft <- 1E-6
            if (pleft >= 1) pleft <- 1-1E-6
            Lint <- dbinom(x=left, size=left+right, prob=pleft, log=TRUE)
            
          } else {
            if ((identical(d1[i, "Group.1"], NA_character)) & 
                (!identical(d1[i, "Group.2"], NA_character))) {
              # J'ai une valeur sur UDL; donc je suis en right censored
              left <- d1[i, "Observations"]
              right <- dplus[i, "Observations"]
              
              pleft <- NULL
              for (p in 1:np) 
                pleft <- c(pleft, do.call("pdistr", modifyList(list(q=as.numeric(as.character(d1[i, "Group.2"])), lower.tail = TRUE, log.p = FALSE), parX_mixture[[p]] )))
              pleft <- sum(pleft * pparX)
              
              
              # pright <- 1-pleft
              if (pleft <= 0) pleft <- 1E-6
              if (pleft >= 1) pleft <- 1-1E-6
              Lint <- dbinom(x=left, size=left+right, prob=pleft, log=TRUE)
              
            } else {
              left <- dmoins[i, "Observations"]
              center <- d1[i, "Observations"]
              right <- dplus[i, "Observations"]
              
              pright <- NULL
              for (p in 1:np) 
                pright <- c(pright, do.call("pdistr", modifyList(list(q=as.numeric(as.character(d1[i, "Group.2"])), lower.tail = FALSE, log.p = FALSE), parX_mixture[[p]] )))
              pright <- sum(pright * pparX)
              if (pright <= 0) pright <- 1E-6
              if (pright >= 1) pright <- 1-1E-6
              
              pleft <- NULL
              for (p in 1:np) 
                pleft <- c(pleft, do.call("pdistr", modifyList(list(q=as.numeric(as.character(d1[i, "Group.1"])), lower.tail = TRUE, log.p = FALSE), parX_mixture[[p]] )))
              pleft <- sum(pleft * pparX)
              
              if (pleft <= 0) pleft <- 1E-6
              if (pleft >= 1) pleft <- 1-1E-6
              pcenter <- 1-pleft-pright
              if (pcenter <= 0) pcenter <- 1E-6
              if (pcenter >= 1) pcenter <- 1-1E-6
              Lint <- dmultinom(x=c(left, center, right), size = left+center+right, prob=c(pleft, pcenter, pright), log = TRUE)
              
            }
          }
          
          
          
          if (debug >= 1) {
            cat(paste0("The log likelihood of unquantifiable data LDL=", d1[i, "Group.1"]," and UDL=", d1[i, "Group.2"]," is ", specify_decimal(Lint), "\n"))
          }
          
          L <- L + Lint
        }
      }
      
    }
  }
  
  
  if (as.numeric(debug) == 2) print(paste0("log Likelihood is ", specify_decimal(L)))
  if (is.na(L)) L <- -1E6
  if (is.infinite(L)) L <- -1E6
  # L est log L
  if (!log) L <- exp(L)
  
  return(L)
}

.getTableAddressing <- function (variableNames, truth){		
  
  # Get the adjacency list addressing template. 
  # 
  # Useful if you want to store the networks in their condensed upper-diagonal form while still
  # having the benefit of convenient addressing and/or if you are using a simulated dataset in
  # which you know the truth and want to store all the values in a single data.frame.
  # 
  # Internal function used to get the addressing template for a data.frame to contain the adjacency 
  # list representation of a matrix.
  # @param variableNames the names of all genes to include in the adjacency list
  # @param truth The true adjacency matrix. Often will not be available, but is useful for 
  # debugging and testing.
  # @return A data.frame representing the adjacency list of the matrix provided.
  # @description This function was part of the package ENA. This package is no more available
  # and it cannot be installed from archive because some dependencies are no more
  # available.
  # @export
  # @author Jeffrey D. Allen \email{Jeffrey.Allen@@UTSouthwestern.edu}
  # @examples
  # # getTableAddressing example
  # 
  
	joint <- matrix(0, nrow=length(variableNames), ncol=length(variableNames))
	rownames(joint) <- variableNames
	colnames(joint) <- variableNames
	
	#extract the row, col indices for the upper-triangular portion of the matrix
	address <- cbind(row(joint)[upper.tri(joint)], col(joint)[upper.tri(joint)])
	address <- matrix(row.names(joint)[address], ncol=2)
	
	upper <- upper.tri(joint);
	
	if (missing(truth)){
		aggregate <- data.frame(address)
		colnames(aggregate) <- c("Source", "Dest")
	}
	else{
		aggregate <- data.frame(address, truth[upper])
		colnames(aggregate) <- c("Source", "Dest", "Truth")
	}
	return(aggregate);
}

#' dnbinom_new returns density for the negative binomial distribution
#' @title Density for the negative binomial distribution - Alternative parametrization.
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return Density for the negative binomial distribution - Alternative parametrization.
#' @param x vector of (non-negative integer) quantiles.
#' @param size target for number of successful trials, or dispersion parameter (the shape parameter of the gamma mixing distribution). Must be strictly positive, need not be integer.
#' @param prob probability of success in each trial. 0 < prob <= 1.
#' @param mu alternative parametrization via mean.
#' @param sd alternative parametrization via standard deviation.
#' @param var alternative parametrization via variance.
#' @param log logical; if TRUE, probabilities p are given as log(p).
#' @family Distributions
#' @description Density for the negative binomial distribution with parameters mu, sd, var, size or prob. See \code{dnbinom}.
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' set.seed(1)
#' x <- rnbinom_new(n=100, mu=2, sd=3)
#' LnL <- NULL
#' df <- data.frame(mu=seq(from=0.1, to=8, by=0.1), "-LnL"=NA)
#' for (mu in df[, "mu"])
#' LnL <- c(LnL, -sum(dnbinom_new(x=x, mu=mu, sd=3, log=TRUE)))
#' df[, "-LnL"] <- LnL
#' ggplot(data = df, aes(x = .data[["mu"]], y = .data[["-LnL"]])) + geom_line()
#' # Examples of wrong parametrization
#' dnbinom_new(x=x, mu=c(1, 2), sd=3, log=TRUE)
#' }
#' @export

dnbinom_new <- function(x, size=NULL, prob=NULL, mu=NULL, sd=NULL, var=NULL, log=FALSE) {
  # size=NULL; prob=NULL; mu=NULL; sd=NULL; var=NULL; log=FALSE
  
  if (!is.null(size)) if (length(size) > 1) {
    warning("size can be only of size 1. Only the first element is used.")
    size <- size[1]
  }
  if (!is.null(prob))  if (length(prob) > 1) {
    warning("prob can be only of size 1. Only the first element is used.")
    prob <- prob[1]
  }
  if (!is.null(mu))  if (length(mu) > 1) {
    warning("mu can be only of size 1. Only the first element is used.")
    mu <- mu[1]
  }
  if (!is.null(sd))  if (length(sd) > 1) {
    warning("sd can be only of size 1. Only the first element is used.")
    sd <- sd[1]
  }
  if (!is.null(var))  if (length(var) > 1) {
    warning("var can be only of size 1. Only the first element is used.")
    var <- var[1]
  }
  # Les combinatoires possibles sont
  # size=NULL, prob=NULL # OK
  # size=NULL, mu=NULL # OK
  # size=NULL, sd=NULL # ok
  # size=NULL, var=NULL ¨# ok
  # prob=NULL, mu=NULL # OK
  # prob=NULL, sd=NULL # ok
  # prob=NULL, var=NULL # ok
  # mu=NULL, sd=NULL # OK
  # mu=NULL, var=NULL # OK
  # sd=NULL, var=NULL # Impossible
  
  if (!is.null(sd) & !is.null(var)) if (var != sd^2) {
    warning("sd and var are incompatible.")
    return(rep(NA, length(x)))
  }
  # # prob = size/(size+mu)
  # if (is.null(size) & !is.null(prob) & !is.null(mu)) size <- (prob * mu) / (1- prob)
  if (!is.null(sd)) var <- sd^2
  # prob = size/(size+mu)
  if (is.null(mu) & !is.null(prob) & !is.null(size)) mu <- (size/prob)-size
  # # prob = size/(size+mu)
  # if (is.null(prob) & !is.null(size) & !is.null(mu)) prob <- size/(size+mu)
   # var = mu + mu^2 / size
  if (is.null(size) & !is.null(prob) & !is.null(var)) {
    # size <- mu^2 / (var - mu)
    # mu <- ((size/prob)-size)
    # size <- (prob * mu) / (1- prob)
    # size <- mu^2 / (var - mu)
    # size * (var - mu) <- mu^2
    # size * var - size - mu - mu^2 <- 0
    
    # size <- mu^2 / (var - mu)
    # size <- ((size/prob)-size)^2 / (var - ((size/prob)-size))
    # size <- ((size/prob)-(prob*size)/prob)^2 / (var - (size/prob)+size)
    # size <- ((size/prob)^2-2*((prob*size)/prob)*(size/prob)+((prob*size)/prob)^2) / ((var*prob - size + size*prob)/prob)
    # size <- ((size^2/prob^2)-2*((prob*size^2)/prob^2)+((prob^2*size^2)/prob^2)) / ((var*prob - size + size*prob)/prob)
    # size <- ((size^2/prob^2)-2*((prob*size^2)/prob^2)+((prob^2*size^2)/prob^2)) * (prob/(var*prob - size + size*prob))
    # size <- (((size^2)-2*((prob*size^2))+((prob^2*size^2)))/prob^2) * (prob/(var*prob - size + size*prob))
    # size <- (((size^2)-2*((prob*size^2))+((prob^2*size^2)))/prob) * (1/(var*prob - size + size*prob))
    # size <- (((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)/1) * (1/(var*prob - size + size*prob))
    
    # size <- (((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)/(var*prob - size + size*prob))
    
    # size - (((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)/(var*prob - size + size*prob)) <- 0
    # (size*(var*prob - size + size*prob))/(var*prob - size + size*prob) - (((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)/(var*prob - size + size*prob)) <- 0
    # ((size*var*prob - size^2 + size^2*prob) - ((size^2-2*(prob*size^2)+(prob^2*size^2))/prob))/(var*prob - size + size*prob) <- 0
    # ((size*var*prob - size^2 + size^2*prob) - ((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)) <- 0
    
    #  ((size*var*prob^2 - size^2*prob + size^2*prob^2)/prob - ((size^2-2*(prob*size^2)+(prob^2*size^2))/prob)) <- 0
    #  ((size*var*prob^2 - size^2*prob + size^2*prob^2) - ((size^2-2*(prob*size^2)+(prob^2*size^2))))/prob <- 0
    #  size*var*prob^2 - size^2*prob + size^2*prob^2 - size^2+2*(prob*size^2)-prob^2*size^2 <- 0
    #  var*prob^2 - size*prob + size*prob^2 - size+2*prob*size-prob^2*size <- 0
    #  var*prob^2  <- size*prob - size*prob^2 + size -2*prob*size +prob^2*size
    
    #  var*prob^2  <- size*(prob - prob^2 + 1 -2*prob +prob^2)
    size <- (var*prob^2) / (1-prob)
    mu <- (size/prob)-size
  }
  
  if (is.null(size) & !is.null(mu) & !is.null(var)) size <- mu^2 / (var - mu)
  
  if (is.null(mu) & !is.null(size) & !is.null(var)) {
    # size <- mu^2 / (var - mu)
    # size * var -size * mu <- mu^2
    # mu^2 + size * mu - size * var <- 0
    A <- 1
    B <- size
    C <- - size * var
    delta <- B^2-4*A*C
    if (delta < 0) {
      warning("mu cannot be negative nor mu > var")
      return(rep(NA, length(x)))
    }
    m1 <- (-B+sqrt(delta))/(2*A)
    m2 <- (-B-sqrt(delta))/(2*A)
    if (m1 > 0) {
      mu <- m1
    } else {
      mu <- m2
    }
  }
  
  if (is.null(size) | is.null(mu)) {
    warning("Not enough information were provided.")
    return(rep(NA, length(x)))
  }

  if (size < 0) {
    warning("size cannot be negative nor mu > var")
    out <- rep(NA, length(x))
  } else {
    if ((mu == 0) & any(x != 0)) {
      warning("When mu is 0, x cannot be different from 0.")
    }
    out <- dnbinom(x=x, size=size, mu=mu, log=log)
    
  }
  return(out)
}

#' compare_AICc compares the AICc of several outputs obtained with the same data.
#' @title Compares the AICc of several outputs
#' @author Marc Girondot \email{marc.girondot@@gmail.com}
#' @return A list with DeltaAICc and Akaike weight for the models.
#' @param ... Successive results to be compared as lists.
#' @param factor.value The $value of the list object is multiplied by factor.value to calculate BIC.
#' @param silent If TRUE, nothing is displayed.
#' @param FUN Function used to show values
#' @description This function is used to compare the AICc of several outputs obtained with the same data but with different set of parameters.\cr
#' Each object must have associated \code{logLik()} method with df and nobs attributes.\cr
#' AICc for object x will be calculated as \code{2*factor.value*logLik(x)+(2*attributes(logLik(x))$df*(attributes(logLik(x))$df+1)/(attributes(logLik(x))$nobs-attributes(logLik(x))$df-1)}.\cr
#' @family AIC
#' @examples
#' \dontrun{
#' library("HelpersMG")
#' # Here two different models are fitted
#' x <- 1:30
#' y <- rnorm(30, 10, 2)+log(x)
#' plot(x, y)
#' d <- data.frame(x=x, y=y)
#' m1 <- lm(y ~ x, data=d)
#' m2 <- lm(y ~ log(x), data=d)
#' compare_BIC(linear=m1, log=m2, factor.value=-1)
#' # Here test if two datasets can be modeled with a single model
#' x2 <- 1:30
#' y2 <- rnorm(30, 15, 2)+log(x2)
#' plot(x, y, ylim=c(5, 25))
#' plot_add(x2, y2, col="red")
#' d2 <- data.frame(x=x2, y=y2)
#' m1_2 <- lm(y ~ x, data=d2)
#' x_grouped <- c(x, x2)
#' y_grouped <- c(y, y2)
#' d_grouped <- data.frame(x=x_grouped, y=y_grouped)
#' m1_grouped <- lm(y ~ x, data=d_grouped)
#' compare_AICc(separate=list(m1, m1_2), grouped=m1_grouped, factor.value=-1)
#' # Or simply
#' compare_AICc(m1=list(AICc=100), m2=list(AICc=102))
#' }
#' @export


compare_AICc <- function(..., factor.value=-1, silent=FALSE, 
                         FUN=function(x) specify_decimal(x, decimals=2)) {

  # factor.value=-1
  # silent=FALSE 
  # FUN=function(x) specify_decimal(x, decimals=2)
  # result <- list()
  
  result <- list(...)
  
  if (is.list(result) & length(result)==1) result <- unlist(result, recursive=FALSE)
  
  if (!is.null(result)) {
    if (!is.list(result) || (is.null(names(result))) || (any(names(result)==""))) {
      stop("The results must be included within a list with names; see example.")
    } else {
      out<-NULL
      
      l <- length(result)
      
      if (l<2) {
        stop("A least two results must be provided.")
      } else {
        aicc <- NULL
        name <- names(result)
        for (i in 1:l) {
          
          encours <- result[i]
          # Je vais être un peu plus strict que pour l'AIC
          # Il faut que logLik(encours[[1]]) existe avec les attributs corrects
          # nall, nobs (nb d'observations) et df (nb de parametres)
          
          if (!is.null(encours[[1]]$AICc)) {
            aicc <- c(aicc, encours[[1]]$AICc)
          } else {
            
            aaec <- try(logLik(encours[[1]]), silent=TRUE)
           t <- (inherits(aaec, "try-error"))
          if (t) encours <- encours[[1]]
          
          sumL <- 0
          sumdf <- 0
          sumnobs <- 0

          for (j in 1:length(encours)) {
            encours2 <- encours[[j]]
            aaec <- try(logLik(encours2), silent=TRUE)
            t <- (inherits(aaec, "try-error"))
            if (t) {
              stop(paste("Object", name[i], "has not the required format"))
            }
            sumL <- sumL + logLik(encours2)
            sumdf <- sumdf + attributes(logLik(encours2))$df
            sumnobs <- sumnobs + attributes(logLik(encours2))$nobs
          }
          aicc <- c(aicc, 2*factor.value*sumL+2*sumdf+(2*sumdf*(sumdf+1))/(sumnobs-sumdf-1))
          }
          
        }
        
        bestaicc <- min(aicc)
        ser<-which.min(aicc)
        deltaaicc<-aicc-bestaicc
        aw<-exp(-0.5*deltaaicc)
        saw=sum(aw)
        aw<-aw/saw
        
        out<-data.frame(cbind(AICc=FUN(aicc), DeltaAICc=FUN(deltaaicc), Akaike_weight=FUN(aw)), row.names=name)
        if (!silent) print(paste("The lowest AICc (",sprintf("%.3f", bestaicc) ,") is for series ", name[ser], " with Akaike weight=", sprintf("%.3f", aw[ser]), sep=""))
        
        return(out)
      }
    }
  }
}

#' cArrows draws curved lines with arrowhead
#' @title Draw curved lines with arrowhead
#' @author Modified from iGraph
#' @return A list wit lab.x and lab.y being the position where to draw label
#' @param x1 coordinates of points from which to draw.
#' @param y1 coordinates of points from which to draw.
#' @param x2 coordinates of points to which to draw.
#' @param y2 coordinates of points to which to draw.
#' @param code integer code (1, 2, or 3), determining kind of arrows to be drawn.
#' @param size size of the arrowhead.
#' @param width width of the arrowhead.
#' @param open shape of the arrowhead.
#' @param sh.adj Shift the beginning of the line.
#' @param sh.lwd width of the line.
#' @param sh.col color of the line.
#' @param sh.lty type of line.
#' @param h.col color of the arrowhead.
#' @param h.col.bo color of the arrowhead border.
#' @param h.lwd width of the arrowhead.
#' @param h.lty type of line for the arrowhead.
#' @param curved 0 is a straigth line, positive of negative value make the line curved.
#' @param beautiful.arrow if open is false, make the arrowhead more beautiful.
#' @description Draw a curved line with arrowhead.
#' @examples
#' plot(c(1, 10), c(1, 10), type="n", bty="n")
#' cArrows(x1=2, y1=2, x2=6, y2=6, curved=1)
#' cArrows(x1=2, y1=2, x2=6, y2=6, curved=0)
#' cArrows(x1=2, y1=2, x2=6, y2=6, curved=1, sh.adj=1)
#' cArrows(x1=2, y1=2, x2=6, y2=6, curved=-1, open=FALSE)
#' cArrows(x1=9, y1=2, x2=6, y2=6, curved=-1, open=FALSE, sh.col="red")
#' cArrows(x1=9, y1=9, x2=6, y2=6, curved=-1, open=FALSE, h.col="red")
#' cArrows(x1=2, y1=9, x2=6, y2=6, curved=1, open=FALSE, h.col="red", h.col.bo="red")
#' @export


cArrows <- function (x1, y1, x2, y2, code = 2, size = 1, width = 1.2/4/cin, 
                     open = TRUE, sh.adj = 0.1, sh.lwd = 1, sh.col = par("fg"), 
                     sh.lty = 1, h.col = sh.col, h.col.bo = sh.col, h.lwd = sh.lwd, 
                     h.lty = sh.lty, curved = FALSE, beautiful.arrow=2/3) 
{
  
  # plot(c(1, 10), c(1, 10))  
  #  x1=1; y1=1; x2=5; y2=5; code=2; size=1; width=1.2/4/0.2; open=TRUE; sh.adj = 0.1; sh.lwd = 1; sh.col = par("fg"); sh.lty = 1; h.col = sh.col; h.col.bo = sh.col; h.lwd = sh.lwd; h.lty = sh.lty; curved = FALSE
  
  
  cin <- size * par("cin")[2]
  width <- width * (1.2/4/cin)
  uin <- 1/xyinch()
  x <- sqrt(seq(0, cin^2, length = floor(35 * cin) + 2))
  delta <- sqrt(h.lwd) * par("cin")[2] * 0.005
  x.arr <- c(-rev(x), -x)
  wx2 <- width * x^2
  y.arr <- c(-rev(wx2 + delta), wx2 + delta)
  # Pourquoi rajouter NA ?
  deg.arr <- c(atan2(y.arr, x.arr), NA)
  r.arr <- c(sqrt(x.arr^2 + y.arr^2), NA)
  bx1 <- x1
  bx2 <- x2
  by1 <- y1
  by2 <- y2
  lx <- length(x1)
  r.seg <- rep(cin * sh.adj, lx)
  theta1 <- atan2((y1 - y2) * uin[2], (x1 - x2) * uin[1])
  th.seg1 <- theta1 + rep(atan2(0, -cin), lx)
  theta2 <- atan2((y2 - y1) * uin[2], (x2 - x1) * uin[1])
  th.seg2 <- theta2 + rep(atan2(0, -cin), lx)
  x1d <- y1d <- x2d <- y2d <- 0
  if (code %in% c(1, 3)) {
    x2d <- r.seg * cos(th.seg2)/uin[1]
    y2d <- r.seg * sin(th.seg2)/uin[2]
  }
  if (code %in% c(2, 3)) {
    x1d <- r.seg * cos(th.seg1)/uin[1]
    y1d <- r.seg * sin(th.seg1)/uin[2]
  }
  if (is.logical(curved) && all(!curved) || is.numeric(curved) && 
      all(!curved)) {
    segments(x1 + x1d, y1 + y1d, x2 + x2d, y2 + y2d, lwd = sh.lwd, 
             col = sh.col, lty = sh.lty)
    phi <- atan2(y1 - y2, x1 - x2)
    r <- sqrt((x1 - x2)^2 + (y1 - y2)^2)
    lc.x <- x2 + 2/3 * r * cos(phi)
    lc.y <- y2 + 2/3 * r * sin(phi)
  }  else {
    if (is.numeric(curved)) {
      lambda <- curved
    } else {
      lambda <- as.logical(curved) * 0.5
    }
    lambda <- rep(lambda, length.out = length(x1))
    c.x1 <- x1 + x1d
    c.y1 <- y1 + y1d
    c.x2 <- x2 + x2d
    c.y2 <- y2 + y2d
    midx <- (x1 + x2)/2
    midy <- (y1 + y2)/2
    spx <- midx - lambda * 1/2 * (c.y2 - c.y1)
    spy <- midy + lambda * 1/2 * (c.x2 - c.x1)
    sh.col <- rep(sh.col, length = length(c.x1))
    sh.lty <- rep(sh.lty, length = length(c.x1))
    sh.lwd <- rep(sh.lwd, length = length(c.x1))
    lc.x <- lc.y <- numeric(length(c.x1))
    for (i in seq_len(length(c.x1))) {
      if (lambda[i] == 0) {
        segments(c.x1[i], c.y1[i], c.x2[i], c.y2[i], 
                 lwd = sh.lwd[i], col = sh.col[i], lty = sh.lty[i])
        phi <- atan2(y1[i] - y2[i], x1[i] - x2[i])
        r <- sqrt((x1[i] - x2[i])^2 + (y1[i] - y2[i])^2)
        lc.x[i] <- x2[i] + 2/3 * r * cos(phi)
        lc.y[i] <- y2[i] + 2/3 * r * sin(phi)
      }  else {
        spl <- xspline(x = c(c.x1[i], spx[i], c.x2[i]), 
                       y = c(c.y1[i], spy[i], c.y2[i]), shape = 1, 
                       draw = FALSE)
        lines(spl, lwd = sh.lwd[i], col = sh.col[i], 
              lty = sh.lty[i])
        if (code %in% c(2, 3)) {
          x1[i] <- spl$x[3 * length(spl$x)/4]
          y1[i] <- spl$y[3 * length(spl$y)/4]
        }
        if (code %in% c(1, 3)) {
          x2[i] <- spl$x[length(spl$x)/4]
          y2[i] <- spl$y[length(spl$y)/4]
        }
        lc.x[i] <- spl$x[2/3 * length(spl$x)]
        lc.y[i] <- spl$y[2/3 * length(spl$y)]
      }
    }
  }
  if (code %in% c(2, 3)) {
    theta <- atan2((by2 - y1) * uin[2], (bx2 - x1) * uin[1])
    Rep <- rep(length(deg.arr), lx)
    p.x2 <- rep(bx2, Rep)
    p.y2 <- rep(by2, Rep)
    ttheta <- rep(theta, Rep) + rep(deg.arr, lx)
    r.arr <- rep(r.arr, lx)
    x <- (p.x2 + r.arr * cos(ttheta)/uin[1])
    y <- (p.y2 + r.arr * sin(ttheta)/uin[2])
    
    if (open) {
      lines(x[-17], y[-17],
            lwd = h.lwd, col = h.col.bo, lty = h.lty)
    } else { 
      
      # Début
      # x[1]
      # y[1]
      # Fin
      # x[length(x)-1]
      # y[length(x)-1]
      
      # Pointe
      xp <- x[1+(length(x)-1)/2]
      yp <- y[1+(length(x)-1)/2]
      # Point de convergence sur la ligne
      xl <- x[1]+(x[length(x)-1]-x[1])/2
      yl <- y[1]+(y[length(x)-1]-y[1])/2
      
      a <- (yl-yp) / (xl-xp)
      b <- yl - a*xl
      
      d <- sqrt((yl-yp)^2+(xl-xp)^2)
      dp <- beautiful.arrow*d
      
      A <- (1+a^2)
      B <- (-2*xp-2*a*yp+2*a*b)
      C <- (xp^2+yp^2-2*b*yp+b^2-dp^2)
      Delta <- B^2-4*A*C
      
      x01 <- (-B-sqrt(Delta))/(2*A)
      x02 <- (-B+sqrt(Delta))/(2*A)
      
      y01 <- a*x01+b
      y02 <- a*x02+b
      
      d01 <- mean(sqrt((x01-x[1])^2+(y01-y[1])^2), sqrt((x01-x[length(x)-1])^2+(y01-y[length(x)-1])^2))
      d02 <- mean(sqrt((x02-x[1])^2+(y02-y[1])^2), sqrt((x02-x[length(x)-1])^2+(y02-y[length(x)-1])^2))
      
      if (d01<d02) {
        x0 <- x01
        y0 <- y01
      } else {
        x0 <- x02
        y0 <- y02
      }
      
      polygon(c(x0, x[-17], x0), c(y0, y[-17], y0), 
                     col = h.col, lwd = h.lwd, 
                     border = h.col.bo, lty = h.lty)
    }
  }
  if (code %in% c(1, 3)) {
    x1 <- bx1
    y1 <- by1
    tmp <- x1
    x1 <- x2
    x2 <- tmp
    tmp <- y1
    y1 <- y2
    y2 <- tmp
    theta <- atan2((y2 - y1) * uin[2], (x2 - x1) * uin[1])
    lx <- length(x1)
    Rep <- rep(length(deg.arr), lx)
    p.x2 <- rep(x2, Rep)
    p.y2 <- rep(y2, Rep)
    ttheta <- rep(theta, Rep) + rep(deg.arr, lx)
    r.arr <- rep(r.arr, lx)
    x <- (p.x2 + r.arr * cos(ttheta)/uin[1])
    y <- (p.y2 + r.arr * sin(ttheta)/uin[2])
    
    if (open) {
      lines(x[-17], y[-17],
            lwd = h.lwd, col = h.col.bo, lty = h.lty)
    } else {
      # Début
      # x[1]
      # y[1]
      # Fin
      # x[length(x)-1]
      # y[length(x)-1]
      
      # Pointe
      xp <- x[1+(length(x)-1)/2]
      yp <- y[1+(length(x)-1)/2]
      # Point de convergence sur la ligne
      xl <- x[1]+(x[length(x)-1]-x[1])/2
      yl <- y[1]+(y[length(x)-1]-y[1])/2
      
      a <- (yl-yp) / (xl-xp)
      b <- yl - a*xl
      
      d <- sqrt((yl-yp)^2+(xl-xp)^2)
      dp <- beautiful.arrow*d
      
      A <- (1+a^2)
      B <- (-2*xp-2*a*yp+2*a*b)
      C <- (xp^2+yp^2-2*b*yp+b^2-dp^2)
      Delta <- B^2-4*A*C
      
      x01 <- (-B-sqrt(Delta))/(2*A)
      x02 <- (-B+sqrt(Delta))/(2*A)
      
      y01 <- a*x01+b
      y02 <- a*x02+b
      
      d01 <- mean(sqrt((x01-x[1])^2+(y01-y[1])^2), sqrt((x01-x[length(x)-1])^2+(y01-y[length(x)-1])^2))
      d02 <- mean(sqrt((x02-x[1])^2+(y02-y[1])^2), sqrt((x02-x[length(x)-1])^2+(y02-y[length(x)-1])^2))
      
      if (d01<d02) {
        x0 <- x01
        y0 <- y01
      } else {
        x0 <- x02
        y0 <- y02
      }
      
      polygon(c(x0, x[-17], x0), c(y0, y[-17], y0), 
              col = h.col, lwd = h.lwd, 
              border = h.col.bo, lty = h.lty)
    }
  }
  invisible(list(lab.x = lc.x, lab.y = lc.y))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/local.search.R
\name{local.search}
\alias{local.search}
\title{Return path of file searched for in local disk based on its file name}
\usage{
local.search(
  pattern,
  directory = "",
  folder = "$HOME",
  intern = TRUE,
  ignore.stdout = FALSE,
  ignore.stderr = TRUE
)
}
\arguments{
\item{pattern}{The name of file to be searched for. Can use wildcards *}

\item{directory}{The path of directory to be explored in for Windows}

\item{folder}{The path of folder to be explored in for Unix based systems}

\item{intern}{A logical (not NA) which indicates whether to capture the output of the command as an R character vector (see system()).}

\item{ignore.stdout}{a logical (not NA) indicating whether messages written to 'stdout' should be ignored  (see system()).}

\item{ignore.stderr}{a logical (not NA) indicating whether messages written to 'stderr' should be ignored  (see system()).}
}
\value{
A vector with paths
}
\description{
Return path of file searched for in local disk based on its file name.\cr
It has been tested only with Windows XP and MacOSX. In MacOSX, you must have created the locate database first. Use OnyX utilities for this purpose.
}
\details{
local.search() returns path of file serached in local disk based on its file name
}
\examples{
\dontrun{
RnwFiles <- local.search("*.Rnw")
nc.files <- local.search("*.nc", folder=paste0("'",getwd(),"'"))
}
}
\author{
Marc Girondot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/symmetricize.R
\name{symmetricize}
\alias{symmetricize}
\title{Make a matrix symmetric}
\usage{
symmetricize(
  matrix,
  method = c("max", "min", "avg", "ld", "ud"),
  adjacencyList = FALSE
)
}
\arguments{
\item{matrix}{The matrix to make symmetric}

\item{method}{The method to use to make the matrix symmetric. Default is to take the maximum.
\describe{
\item{"max"}{For each position, \eqn{m_{i,j}}, use the maxiumum of \eqn{(m_{i,j}, m_{j,i})}}
\item{"min"}{For each position, \eqn{m_{i,j}}, use the minimum of \eqn{(m_{i,j}, m_{j,i})}}
\item{"avg"}{For each position, \eqn{m_{i,j}}, use the mean: \eqn{(m_{i,j} + m_{j,i})/2}}
\item{"ld"}{Copy the lower triangular portion of the matrix to the upper triangular portion.}
\item{"ud"}{Copy the upper triangular portion of the matrix to the lower triangular portion.}
}}

\item{adjacencyList}{Logical. If false, returns the symmetric matrix (the same format as the input). If true, returns an adjacency list representing the upper triangular portion of the adjacency matrix with addressing based on the row.names of the matrix provided.}
}
\value{
The symmetric matrix
}
\description{
This function was part of the package ENA. This package is no more available
and it cannot be installed from archive because some dependencies are no more
available.\cr
}
\details{
Make the matrix symmetric by making all "mirrored" positions consistent. A variety of methods are provided to make the matrix symmetrical.
}
\examples{
#Create a sample 3x3 matrix
mat <- matrix(1:9, ncol=3)

#Copy the upper diagonal portion to the lower
symmetricize(mat, "ud")

#Take the average of each symmetric location
symmetricize(mat, "avg")

}
\author{
Jeffrey D. Allen \email{Jeffrey.Allen@UTSouthwestern.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.packages.R
\name{list.packages}
\alias{list.packages}
\title{List the installed packages with their locations}
\usage{
list.packages()
}
\value{
A list with the installed packages and their version.
}
\description{
List the installed packages with their locations and version.
}
\details{
list.packages lists the installed packages with their locations
}
\examples{
\dontrun{
library(HelpersMG)
list.packages()
}
}
\author{
Marc Girondot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ChangeCoordinate.R
\name{ChangeCoordinate}
\alias{ChangeCoordinate}
\title{Return a value in a changed coordinate}
\usage{
ChangeCoordinate(
  x = stop("At least one value to convert must be provided"),
  initial = stop("Set of two values must be provided as references"),
  transformed = stop("Set of two transformed values must be provided")
)
}
\arguments{
\item{x}{value to convert}

\item{initial}{Set of two values in the original system}

\item{transformed}{Set of the two values in the converted system}
}
\value{
A value in the new system
}
\description{
Return a value in a changed coordinate system.
}
\details{
ChangeCoordinate returns a value in a changed coordinate
}
\examples{
ChangeCoordinate(x=c(10, 20), initial=c(1, 100), transformed=c(0, 1))
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barplot_errbar.R
\name{barplot_errbar}
\alias{barplot_errbar}
\title{Plot a barplot graph with error bar on y}
\usage{
barplot_errbar(
  ...,
  errbar.y = NULL,
  errbar.y.plus = NULL,
  errbar.y.minus = NULL,
  y.plus = NULL,
  y.minus = NULL,
  errbar.tick = 1/50,
  errbar.lwd = par("lwd"),
  errbar.lty = par("lty"),
  errbar.col = par("fg"),
  add = FALSE
)
}
\arguments{
\item{...}{Parameters for barplot() such as main= or ylim=}

\item{errbar.y}{The length of error bars for y. Recycled if necessary.}

\item{errbar.y.plus}{The length of positive error bars for y. Recycled if necessary.}

\item{errbar.y.minus}{The length of negative error bars for y. Recycled if necessary.}

\item{y.plus}{The absolut position of the positive error bar for y. Recycled if necessary.}

\item{y.minus}{The absolut position of the nagative error bar for y. Recycled if necessary.}

\item{errbar.tick}{Size of small ticks at the end of error bars defined as a proportion of total width or height graph size.}

\item{errbar.lwd}{Error bar line width, see par("lwd")}

\item{errbar.lty}{Error bar line type, see par("lwd")}

\item{errbar.col}{Error bar line color, see par("col")}

\item{add}{If true, add the graph to the previous one.}
}
\value{
A numeric vector (or matrix, when beside = TRUE), say mp, giving the coordinates of all the bar midpoints drawn, useful for adding to the graph.\cr
If beside is true, use colMeans(mp) for the midpoints of each group of bars, see example.
}
\description{
To plot data, just use it as a normal barplot but add the errbar.y
values or errbar.y.minus, errbar.y.plus if bars for y axis are
asymetric. Use y.plus and y.minus to set absolut limits for
error bars. Note that y.plus and y.minus have priority over errbar.y,
errbar.y.minus and errbar.y.plus.
}
\details{
barplot_errbar plot a barplot with error bar on y
}
\examples{
\dontrun{
barplot_errbar(rnorm(10, 10, 3), 
	xlab="axe x", ylab="axe y", bty="n", 
		errbar.y.plus=rnorm(10, 1, 0.1), col=rainbow(10), 
		names.arg=paste("Group",1:10), cex.names=0.6)
y <- rnorm(10, 10, 3)
barplot_errbar(y, 
               	xlab="axe x", ylab="axe y", bty="n", 
            		y.plus=y+2)
}
}
\seealso{
\code{plot_errorbar}

Other plot and barplot functions:
\code{\link[=ScalePreviousPlot]{ScalePreviousPlot()}},
\code{\link[=plot_add]{plot_add()}},
\code{\link[=plot_errbar]{plot_errbar()}},
\code{\link[=show_name]{show_name()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{plot and barplot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RandomFromHessianOrMCMC.R
\name{RandomFromHessianOrMCMC}
\alias{RandomFromHessianOrMCMC}
\title{Random numbers based on Hessian matrix or MCMC}
\usage{
RandomFromHessianOrMCMC(
  se = NULL,
  Hessian = NULL,
  mcmc = NULL,
  chain = "all",
  regularThin = TRUE,
  MinMax = NULL,
  fitted.parameters = NULL,
  fixed.parameters = NULL,
  method = NULL,
  probs = c(0.025, 0.5, 0.975),
  replicates = 10000,
  fn = NULL,
  silent = FALSE,
  ParTofn = "par",
  ...
)
}
\arguments{
\item{se}{A named vector with SE of parameters}

\item{Hessian}{An Hessian matrix}

\item{mcmc}{A result from MHalgogen()}

\item{chain}{MCMC chain to be used or "all"}

\item{regularThin}{If TRUE, use regular thin for MCMC or use a number}

\item{MinMax}{A data.frame with at least two columns: Min and Max and rownames being the variable names}

\item{fitted.parameters}{The fitted parameters}

\item{fixed.parameters}{The fixed parameters}

\item{method}{Can be NULL, "SE", "Hessian", "MCMC", or "PseudoHessianFromMCMC"}

\item{probs}{Probability for quantiles}

\item{replicates}{Number of replicates to generate the randoms}

\item{fn}{The function to apply to each replicate}

\item{silent}{Should the function display some information}

\item{ParTofn}{Name of the parameter to send random values to fn}

\item{...}{Parameters send to fn function}
}
\value{
Returns a list with three data.frames named random, fn, and quantiles
}
\description{
If it is very long, use silent parameter to check if something goes wrong.\cr
If replicates is NULL or is 0, or if method is NULL, parameters are just copied into data.frame.\cr
If method is NULL, replicate.CI is set to 0.\cr
If method is hessian, it will generate replicate.CI random numbers using
Hessian matrix with covariance.\cr
If method is se, it will generate replicate.CI random numbers using
SE values then without covariance.\cr
If method is mcmc, it will generate replicate.CI random numbers using
random samples of the MCMC or the regularThin number if it is a number.\cr
}
\details{
RandomFromHessianOrMCMC returns random numbers based on Hessian matrix or MCMC
}
\examples{
\dontrun{
library(HelpersMG)
val <- rnorm(100, mean=20, sd=5)+(1:100)/10
# Return -ln L of values in val in Gaussian distribution with mean and sd in par
fitnorm <- function(par, data) {
  -sum(dnorm(data, par["mean"], abs(par["sd"]), log = TRUE))
}
# Initial values for search
p<-c(mean=20, sd=5)
# fit the model
result <- optim(par=p, fn=fitnorm, data=val, method="BFGS", hessian=TRUE)
# Using Hessian
df <- RandomFromHessianOrMCMC(Hessian=result$hessian, 
                              fitted.parameters=result$par, 
                              method="Hessian")$random
hist(df[, 1], main="mean")
hist(df[, 2], main="sd")
plot(df[, 1], df[, 2], xlab="mean", ylab="sd", las=1, bty="n")

# Using MCMC
parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(0.35, 0.2), 
Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
row.names=c('mean', 'sd'))
# Use of trace and traceML parameters
# trace=1 : Only one likelihood is printed
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
parameters_name = "par", 
likelihood=fitnorm, n.chains=1, n.adapt=100, thin=1, trace=1)
df <- RandomFromHessianOrMCMC(mcmc=mcmc_run, fitted.parameters=NULL, 
                              method="MCMC")$random
hist(df[, 1], main="mean")
hist(df[, 2], main="sd")
plot(df[, 1], df[, 2], xlab="mean", ylab="sd", las=1, bty="n")

# Return the two first elements of the MCMC
df <- RandomFromHessianOrMCMC(mcmc=mcmc_run, fitted.parameters=NULL, 
               method="MCMC", replicates = 2, regularThin = c(1, 2))$random

# Using a function fn
fitnorm <- function(par, data, x) { 
  y=par["a"]*(x)+par["b"]
  -sum(dnorm(data, y, abs(par["sd"]), log = TRUE))
}
p<-c(a=0.1, b=20, sd=5)
# fit the model
x <- 1:100
result <- optim(par=p, fn=fitnorm, data=val, x=x, method="BFGS", hessian=TRUE)
# Using Hessian
df <- RandomFromHessianOrMCMC(Hessian=result$hessian, fitted.parameters=result$par, 
                              method="Hessian", 
                              fn=function(par) (par["a"]*(x)+par["b"]))
plot(1:100, val)
lines(1:100, df$quantiles["50\%", ])
lines(1:100, df$quantiles["2.5\%", ], lty=2)
lines(1:100, df$quantiles["97.5\%", ], lty=2)
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/symbol.Female.R
\name{symbol.Female}
\alias{symbol.Female}
\title{Plot a female symbol in the plotting region}
\usage{
symbol.Female(centerx, centery, rayonx, lwd = 2, col = "black")
}
\arguments{
\item{centerx}{The x position of the center of the circle}

\item{centery}{The y position of the center of the circle}

\item{rayonx}{The size of the rayon in the scale of the x axis}

\item{lwd}{The width of the line of the symbol}

\item{col}{The color of the symbol}
}
\value{
Nothing
}
\description{
Plot a female symbol in the plotting region.
}
\details{
symbol.Female plot a female symbol in the plotting region
}
\examples{
\dontrun{
plot(x=1:2, y=c(10,20), type="n", bty="n", xlab="", ylab="")

rayonx <- 0.01
centerx <- 1.2
centery <- 15

symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx)
symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx)

rayonx <- 0.03
centerx <- 1.2
centery <- 18

symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx, lwd=3)
symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx, lwd=3, col="red")

rayonx <- 0.05
centerx <- 1.4
centery <- 13

symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx, lwd=4, col="blue")
symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx, lwd=4, col="red")
}
}
\seealso{
Other Symbol: 
\code{\link{symbol.Male}()}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Symbol}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.LD50.R
\name{predict.LD50}
\alias{predict.LD50}
\title{Estimate survival according to doses}
\usage{
\method{predict}{LD50}(
  object,
  doses = NULL,
  SE = NULL,
  range.CI = 0.95,
  replicates = 1000,
  progressbar = FALSE,
  ...
)
}
\arguments{
\item{object}{A result file generated by LD50}

\item{doses}{A vector of temperatures}

\item{SE}{The standard error for doses, optional}

\item{range.CI}{The range of confidence interval for estimation, default=0.95}

\item{replicates}{Number of replicates to estimate CI}

\item{progressbar}{Logical. Does a progression bar must be shown}

\item{...}{Not used}
}
\value{
A data.frame with informations about survival
}
\description{
Estimate survival according to doses.\cr
The returned data.frame has the following components:\cr
doses, SE, survival, CI.minus.sexratio, CI.plus.sexratio, range.CI\cr
}
\details{
predict.LD50 Estimate survival according to doses
}
\examples{
\dontrun{
#' data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
Alive=c(10, 12, 8, 6, 2, 1),
Dead=c(0, 1, 5, 6, 9, 15))
LD50_logistic <- LD50(data, equation="logistic")
predict(LD50_logistic, doses=c(140, 170))
plot(LD50_logistic
}
}
\seealso{
Other LD50 functions:
\code{\link[=LD50]{LD50()}},
\code{\link[=LD50_MHmcmc]{LD50_MHmcmc()}},
\code{\link[=LD50_MHmcmc_p]{LD50_MHmcmc_p()}},
\code{\link[=logLik.LD50]{logLik.LD50()}},
\code{\link[=plot.LD50]{plot.LD50()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{LD50 functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asc.R
\name{asc}
\alias{asc}
\title{Return the codes (in UTF-8) of a string}
\usage{
asc(x)
}
\arguments{
\item{x}{The string to be analyzed}
}
\value{
A vector with ITF-8 codes of a string
}
\description{
Return the codes (in UTF-8) of a string.
}
\details{
asc returns the codes (in UTF-8) of a string
}
\examples{
asc("abcd")
asc("ABCD")
}
\seealso{
Other Characters:
\code{\link[=char]{char()}},
\code{\link[=d]{d()}},
\code{\link[=tnirp]{tnirp()}}
}
\author{
Based on this blog: http://datadebrief.blogspot.com/2011/03/ascii-code-table-in-r.html
}
\concept{Characters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minmax.periodic.R
\name{minmax.periodic}
\alias{minmax.periodic}
\title{Search for minimum and maximum indices in periodic timeseries}
\usage{
minmax.periodic(
  time.minmax.daily = NULL,
  time.minmax = NULL,
  progressbar = FALSE,
  observed = stop("data.frame with observed indices"),
  period = 24,
  colname.time = "time",
  colname.index = "index",
  colname.SD = "SD",
  plot = FALSE
)
}
\arguments{
\item{time.minmax.daily}{A named vector with Min and Max being the time in the day with minimum and maximum indices (temperature or level)}

\item{time.minmax}{A named vector daily with time in the day at which minimum and maximum indices are observed}

\item{progressbar}{Tell if a progression bar must be shown}

\item{observed}{A dataframe with at least two columns: time and temperatures. A third column SD can indicate the know error in index}

\item{period}{The unit of day period (24 for hours, 24*60 for minutes)}

\item{colname.time}{The name of the column for time in observed}

\item{colname.index}{The name of the column for indices in observed}

\item{colname.SD}{The name of the column for SD in observed}

\item{plot}{If TRUE, show a plot with the different estimates}
}
\value{
A data.frame with a column time, a column index and a column SD
}
\description{
Search for minimum and maximum for periodic timeseries when only intermediate values are known.\cr
For each couple of value with an increasing or decreasing segment of
the sinusoid function, it is possible to estimate a minimum and maximum
values using analytical algebra.\cr
Then the average and standard deviations of all minima and maxima are evaluated.\cr
It should be noted that any extremum can be estimated at least twice, one by
increasing segment and one by decreasing segment. Both are used here to produce SD.\cr
\code{time.minmax.daily} should be used when the time at which maximum and minimum indices are regular and
\code{time.minmax} permits to define this time day by day.
}
\details{
minmax.periodic search for minimum and maximum indices (temperatures or levels) in periodic timeseries
}
\examples{
\dontrun{
library("HelpersMG")
# Generate a timeserie of time
time.obs <- NULL
for (i in 0:9) time.obs <- c(time.obs, c(0, 6, 12, 18)+i*24)
# For these time, generate a timeseries of temperatures
temp.obs <- rep(NA, length(time.obs))
temp.obs[3+(0:9)*4] <- rnorm(10, 25, 3)
temp.obs[1+(0:9)*4] <- rnorm(10, 10, 3)
for (i in 1:(length(time.obs)-1)) 
  if (is.na(temp.obs[i])) 
  temp.obs[i] <- mean(c(temp.obs[i-1], temp.obs[i+1]))
  if (is.na(temp.obs[length(time.obs)])) 
  temp.obs[length(time.obs)] <- temp.obs[length(time.obs)-1]/2
observed <- data.frame(time=time.obs, temperature=temp.obs)
# Search for the minimum and maximum values
r <- minmax.periodic(time.minmax.daily=c(Min=2, Max=15), 
observed=observed, period=24, colname.index="temperature")

# Estimate all the temperatures for these values
t <- index.periodic(minmax=r)

plot_errbar(x=t[,"time"], y=t[,"index"],
errbar.y=ifelse(is.na(t[,"sd"]), 0, 2*t[,"sd"]),
type="l", las=1, bty="n", errbar.y.polygon = TRUE, 
xlab="hours", ylab="Temperatures", ylim=c(0, 35), 
errbar.y.polygon.list = list(col="grey"))

plot_add(x=t[,"time"], y=t[,"index"], type="l")

plot_add(observed$time, observed$temperature, pch=19, cex=0.5)
}
}
\seealso{
Other Periodic patterns of indices:
\code{\link[=index.periodic]{index.periodic()}},
\code{\link[=moon.info]{moon.info()}},
\code{\link[=sun.info]{sun.info()}},
\code{\link[=tide.info]{tide.info()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Periodic patterns of indices}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_BIC.R
\name{compare_BIC}
\alias{compare_BIC}
\title{Compares the BIC of several outputs}
\usage{
compare_BIC(
  ...,
  factor.value = -1,
  silent = FALSE,
  FUN = function(x) specify_decimal(x, decimals = 2)
)
}
\arguments{
\item{...}{Successive results to be compared as lists.}

\item{factor.value}{The $value of the list object is multiplied by factor.value to calculate BIC.}

\item{silent}{If TRUE, nothing is displayed.}

\item{FUN}{Function used to show values}
}
\value{
A list with DeltaBIC and Akaike weight for the models.
}
\description{
This function is used to compare the BIC of several outputs obtained with the same data but with different set of parameters.\cr
Each object must have associated \code{logLik()} method with df and nobs attributes.\cr
BIC for object x will be calculated as \code{2*factor.value*sum(logLik(x))+sum(attributes(logLik(x))$df)*log(attributes(logLik(x))$nobs))}.\cr
When several data (i..n) are included, the global BIC is calculated as:\cr
\code{2*factor.value*sum(logLik(x)) for i..n+sum(attributes(logLik(x))$df) for i..n*log(attributes(logLik(x))$nobs for i..n))}
}
\details{
compare_BIC compares the BIC of several outputs obtained with the same data.
}
\examples{
\dontrun{
library("HelpersMG")
# Here two different models are fitted
x <- 1:30
y <- rnorm(30, 10, 2)+log(x)
plot(x, y)
d <- data.frame(x=x, y=y)
m1 <- lm(y ~ x, data=d)
m2 <- lm(y ~ log(x), data=d)
compare_BIC(linear=m1, log=m2, factor.value=-1)
# Here test if two datasets can be modeled with a single model
x2 <- 1:30
y2 <- rnorm(30, 15, 2)+log(x2)
plot(x, y, ylim=c(5, 25))
plot_add(x2, y2, col="red")
d2 <- data.frame(x=x2, y=y2)
m1_2 <- lm(y ~ x, data=d2)
x_grouped <- c(x, x2)
y_grouped <- c(y, y2)
d_grouped <- data.frame(x=x_grouped, y=y_grouped)
m1_grouped <- lm(y ~ x, data=d_grouped)
compare_BIC(separate=list(m1, m1_2), grouped=m1_grouped, factor.value=-1)
}
}
\seealso{
Other AIC:
\code{\link[=ELPDweight]{ELPDweight()}},
\code{\link[=ExtractAIC.glm]{ExtractAIC.glm()}},
\code{\link[=FormatCompareAIC]{FormatCompareAIC()}},
\code{\link[=compare_AIC]{compare_AIC()}},
\code{\link[=compare_AICc]{compare_AICc()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{AIC}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.LD50.R
\name{logLik.LD50}
\alias{logLik.LD50}
\title{Return Log Likelihood of a fit generated by LD50}
\usage{
\method{logLik}{LD50}(object, ...)
}
\arguments{
\item{object}{A result file generated by fitRMU}

\item{...}{Not used}
}
\value{
The Log Likelihood value for the fitted model with data
}
\description{
Return Log Likelihood of a fit generated by LD50
}
\details{
logLik.LD50 Return Log Likelihood of a fit for LD50
}
\examples{
\dontrun{
data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
Alive=c(10, 12, 8, 6, 2, 1),
Dead=c(0, 1, 5, 6, 9, 15))
LD50_logistic <- LD50(data, equation="logistic")
logLik(LD50_logistic)
AIC(LD50_logistic)
}
}
\seealso{
Other LD50 functions:
\code{\link[=LD50]{LD50()}},
\code{\link[=LD50_MHmcmc]{LD50_MHmcmc()}},
\code{\link[=LD50_MHmcmc_p]{LD50_MHmcmc_p()}},
\code{\link[=plot.LD50]{plot.LD50()}},
\code{\link[=predict.LD50]{predict.LD50()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{LD50 functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.cutter.R
\name{logLik.cutter}
\alias{logLik.cutter}
\title{Return log likelihood of a cutter fitted model}
\usage{
\method{logLik}{cutter}(object, ...)
}
\arguments{
\item{object}{A result file generated by cutter}

\item{...}{Not used}
}
\value{
Nothing
}
\description{
Return log likelihood of a cutter fitted model.
}
\details{
logLik.cutter return log likelihood of a cutter fitted model
}
\examples{
\dontrun{
# ___________________________________________________________________
# Test for similarity in gamma left censored distribution between two
# datasets
# ___________________________________________________________________
obc1 <- rgamma(100, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL <- 10
# remove the data below the detection limit
obc1[obc1<LDL] <- -Inf
obc2 <- rgamma(100, scale=10, shape=2)
# remove the data below the detection limit
obc2[obc2<LDL] <- -Inf
# search for the parameters the best fit these censored data
result1 <- cutter(observations=obc1, 
                           lower_detection_limit=LDL, 
                          cut_method="censored")
logLik(result1)
result2 <- cutter(observations=obc2, 
                           lower_detection_limit=LDL, 
                          cut_method="censored")
logLik(result2)
result_totl <- cutter(observations=c(obc1, obc2), 
                           lower_detection_limit=LDL, 
                          cut_method="censored")
logLik(result_totl)
compare_AICc(Separate=list(result1, result2), 
            Common=result_totl, factor.value=1)
compare_BIC(Separate=list(result1, result2), 
            Common=result_totl, factor.value=1) 
}
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FormatCompareAIC.R
\name{FormatCompareAIC}
\alias{FormatCompareAIC}
\title{Format data to be used with compare_AIC()}
\usage{
FormatCompareAIC(logLik, nobs, df)
}
\arguments{
\item{logLik}{The log likelihood}

\item{nobs}{Number of observations}

\item{df}{Number of parameters}
}
\value{
An object to be used with compare_AIC()
}
\description{
Format data to be used with compare_AIC(), compare_AICc() and compare_BIC().\cr
Note that logLik is supposed to not be -logLik.
}
\details{
FormatCompareAIC formats data to be used with compare_AIC()
}
\examples{
\dontrun{
ED <- FormatCompareAIC(logLik=-140, nobs=100, df=3)
L <- FormatCompareAIC(logLik=-145, nobs=100, df=4)
compare_AIC(L=L, ED=ED)
compare_AICc(L=L, ED=ED)
compare_BIC(L=L, ED=ED)
}
}
\seealso{
Other AIC:
\code{\link[=ELPDweight]{ELPDweight()}},
\code{\link[=ExtractAIC.glm]{ExtractAIC.glm()}},
\code{\link[=compare_AIC]{compare_AIC()}},
\code{\link[=compare_AICc]{compare_AICc()}},
\code{\link[=compare_BIC]{compare_BIC()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{AIC}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcutter.R
\name{dcutter}
\alias{dcutter}
\title{Distribution of the fitted distribution without cut.}
\usage{
dcutter(
  par,
  observations = NULL,
  distribution = "gamma",
  n.mixture = NULL,
  debug = FALSE,
  limits.lower = NULL,
  limits.upper = NULL,
  log = TRUE
)
}
\arguments{
\item{par}{Values for parameters of distribution}

\item{observations}{The observations; see description.}

\item{distribution}{Can be gamma, normal, weibull, lognormal, or generalized.gamma.}

\item{n.mixture}{Number of distributions}

\item{debug}{If TRUE, show some information. If 2, show more information.}

\item{limits.lower}{Value for lower detection limit}

\item{limits.upper}{Value for upper detection limit}

\item{log}{If TRUE, return the log likelihood}
}
\value{
The density of the cutter function according to observations.
}
\description{
If observations must be a data.frame with 4 columns:\cr
observations: A column for the measurements;\cr
LDL: A column for the lower detection limit;\cr
UDL: A column for the upper detection limit;\cr
Cut: A column for the truncated of censored nature of the data.\cr
}
\details{
dcutter returns the density of the cutter function
}
\examples{
\dontrun{
library(HelpersMG)
par <- c('shape1' = 0.42265849507444225, 
         'scale1' = 14.139457094879594, 
         'shape2' = 1.667131542489706, 
         'scale2' = 0.10763344388223803, 
         'p1' = 0.12283307526788023)
obs <- data.frame(Observations=c(0.755, 1.013, 2.098, 6.265, 4.708, 0.078, 2.169, 0.403, 1.251, 
                                 0.008, 1.419, 1.078, 2.744, 81.534, 1.426, 13.486, 7.813, 0.165, 
                                 0.118, 0.864, 0.369, 7.159, 2.605, 1.579, 1.646, 0.484, 4.492, 
                                 0.139, 0.28, 0.154, 0.106, 0.104, 4.185, 0.735, 0.149, 0.183, 
                                 0.062, 8.246, 0.165, 0.121, 0.109, 0.092, 0.162, 0.108, 0.139, 
                                 0.141, 0.124, 0.124, 0.151, 0.141, 0.364, 0.295, 0.09, 0.135, 
                                 0.154, 0.218, 0.167, -Inf, 0.203, 0.228, 0.107, 0.162, 0.194, 
                                 0.322, 0.351, 0.17, 0.236, 0.176, 0.107, 0.12, 0.095, 0.27, 0.194, 
                                 0.125, 0.123, 0.085, 0.164, 0.106, 0.079, 0.162), 
                 LDL=0.001, UDL=NA, Cut="censored")
dcutter(par=par, observations=obs, distribution="gamma", 
        n.mixture=NULL, debug=FALSE, limits.lower=NULL, 
        limits.upper=NULL,log=FALSE)
dcutter(par=par, observations=obs, distribution="gamma", 
        n.mixture=NULL, debug=FALSE, limits.lower=NULL, 
        limits.upper=NULL, log=TRUE)
        
}
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IC_correlation_simplify.R
\name{IC_correlation_simplify}
\alias{IC_correlation_simplify}
\title{Simplify the correlation matrix}
\usage{
IC_correlation_simplify(matrix, variable = NULL)
}
\arguments{
\item{matrix}{The correlation matrix to simplify}

\item{variable}{a vector with the name of columns to keep}
}
\value{
A list
}
\description{
This function can be used to simplify the network of correlations.\cr
If no vector of variables is given, the variables not linked to any other variable are removed.
If a vector of variables is given, only link to these variables are retained.
\code{https://fr.wikipedia.org/wiki/Iconographie_des_corrélations}
}
\details{
IC_correlation_simplify simplifies the correlation matrix
}
\examples{
\dontrun{
library("HelpersMG")
es <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
                 Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
                 Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
                 Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
                 Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
                 row.names = c(NA, -8L), class = "data.frame")

es

df <- IC_clean_data(es, debug = TRUE)
cor_matrix <- IC_threshold_matrix(data=df, threshold = NULL, progress=FALSE)
cor_threshold <- IC_threshold_matrix(data=df, threshold = 0.3)
par(mar=c(1,1,1,1))
set.seed(4)
plot(cor_threshold)
cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
plot(cor_threshold_Note)
}
}
\references{
Lesty, M., 1999. Une nouvelle approche dans le choix des régresseurs de la régression multiple en présence d’interactions et de colinéarités. Revue de Modulad 22, 41-77.
}
\seealso{
Other Iconography of correlations:
\code{\link[=IC_clean_data]{IC_clean_data()}},
\code{\link[=IC_threshold_matrix]{IC_threshold_matrix()}},
\code{\link[=plot.IconoCorel]{plot.IconoCorel()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Iconography of correlations}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.PriorsmcmcComposite.R
\name{plot.PriorsmcmcComposite}
\alias{plot.PriorsmcmcComposite}
\title{Plot a prior defined with setPriors function}
\usage{
\method{plot}{PriorsmcmcComposite}(x, parameter = 1, ...)
}
\arguments{
\item{x}{The priors to show}

\item{parameter}{The name or rank of prior to show}

\item{...}{Not used}
}
\value{
A ggplot object
}
\description{
Create a ggplot graph with prior.\cr
The function makes minimal effort to decorate the plot.
}
\details{
plot.PriorsmcmcComposite plot a prior
}
\examples{
\dontrun{
library("HelpersMG")
par <- c(a0=10, a1=2, b2=20, b1=-1)
rules <- rbind(data.frame(Name="^a", Min=0, Max="x*2"), 
              data.frame(Name="^b", Min=0, Max=100))
p <- setPriors(par=par, se=NULL, density="dgamma", rules=rules)
plot(p, parameter="a0")
q <- plot(p, parameter="b1")
q + geom_line(color = "red") + theme_bw() + 
theme(plot.margin=unit(c(2,1,1,1), 'cm'), 
      panel.border = element_blank(), 
      axis.line.x.bottom = element_line(colour = "black"), 
      axis.line.y.left = element_line(colour = "black")) + 
labs(title="Parameter: b1") + theme(plot.title = element_text(hjust = 0.5))
}
}
\seealso{
Other mcmcComposite functions:
\code{\link[=+.PriorsmcmcComposite]{+.PriorsmcmcComposite()}},
\code{\link[=MHalgoGen]{MHalgoGen()}},
\code{\link[=PPD]{PPD()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=as.parameters]{as.parameters()}},
\code{\link[=merge.mcmcComposite]{merge.mcmcComposite()}},
\code{\link[=plot.mcmcComposite]{plot.mcmcComposite()}},
\code{\link[=setPriors]{setPriors()}},
\code{\link[=setPriors1]{setPriors1()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IC_threshold_matrix.R
\name{IC_threshold_matrix}
\alias{IC_threshold_matrix}
\title{Calculate correlation matrix}
\usage{
IC_threshold_matrix(
  data = stop("A dataframe or an IconoCorel object is required"),
  threshold = NULL,
  use = c("pairwise.complete.obs", "everything", "all.obs", "complete.obs",
    "na.or.complete"),
  method = c("pearson", "kendall", "spearman"),
  model = c("OAT", "ATT"),
  significance.level = FALSE,
  correction.multiple.comparisons = "fdr",
  progress = TRUE,
  debug = FALSE
)
}
\arguments{
\item{data}{A dataframe or an IconoCorel object from a previous run of IC_threshold_matrix}

\item{threshold}{threshold for partial and full correlations}

\item{use}{an optional character string giving a method for computing covariances in the presence of missing values. This must be (an abbreviation of) one of the strings "everything", "all.obs", "complete.obs", "na.or.complete", or "pairwise.complete.obs".}

\item{method}{a character string indicating which correlation coefficient (or covariance) is to be computed. One of "pearson" (default), "kendall", or "spearman": can be abbreviated.}

\item{model}{a character string indicating if linear model uses all variables at a time (AAT) or one at a time (OAT).}

\item{significance.level}{if FALSE, does not use significance level; or use this significance level.}

\item{correction.multiple.comparisons}{"holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr", or "none".}

\item{progress}{show a progress bar}

\item{debug}{display information about progression of computing}
}
\value{
A list
}
\description{
This function calculates the matrix of correlations thresholded using partial correlation.\cr
If the threshold is not given, the object that is produced can be used later for thresholding.\cr
For model OAT: The link between A and B is “remarkable” if and only if the total correlation between them is higher than a given threshold and if the partial correlation between A and B in respect to any other variable C is also higher in absolute values than this threshold and with the same sign as the total correlation.
For model AAT: A correlation is retained if it is higher than the threshold and the partial correlation is lower than the threshold. In this case, no missing value is accepted.\cr
The use and method parameters are used by cor() function. The function uses by default a parallel computing in Unix or MacOSX systems.
If progress is TRUE and the package pbmcapply is present, a progress bar is displayed. If debug is TRUE, some informations are shown during the process but parallel computing is not used.\cr
\code{https://fr.wikipedia.org/wiki/Iconographie_des_corrélations}
}
\details{
IC_threshold_matrix calculates correlation matrix thresholed by partial correlation
}
\examples{
\dontrun{
library("HelpersMG")
es <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
                 Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
                 Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
                 Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
                 Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
                 row.names = c(NA, -8L), class = "data.frame")

es

df_clean <- IC_clean_data(es, debug = TRUE)
cor_matrix <- IC_threshold_matrix(data=df_clean, threshold = NULL, progress=FALSE)
cor_threshold <- IC_threshold_matrix(data=df_clean, threshold = 0.3)
plot(cor_threshold, show.legend.strength=FALSE, show.legend.direction = FALSE)
cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
plot(cor_threshold_Note)

cor_threshold <- IC_threshold_matrix(data=df_clean, threshold = 0.8, progress=FALSE)
gr <- plot(cor_threshold, plot=FALSE)
ly <- getFromNamespace("layout_nicely", ns="igraph")(gr)
plot(cor_threshold, 
layout=matrix(data=c(53, 53, 55, 55, 
                     55, 53, 55, 53), ncol=2, byrow=FALSE), 
show.legend.direction = FALSE,
show.legend.strength = FALSE, xlim=c(-2, 2), ylim=c(-2, 2))

# Using significance level

cor_threshold <- IC_threshold_matrix(data=df_clean, 
                                     significance.level=0.05, debug=TRUE)
plot(cor_threshold, show.legend.strength=FALSE, show.legend.direction = FALSE)
cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
plot(cor_threshold_Note)

# Using the model All at a time

cor_threshold_AAT <- IC_threshold_matrix(data=df_clean, threshold = 0.3, model="AAT")
par(mar=c(1,1,1,1))
set.seed(4)
plot(cor_threshold_AAT, show.legend.strength="bottomleft")



############
dta <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
                     Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
                     Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
                     Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
                     Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
                     row.names = c(NA, -8L), class = "data.frame")

dta0 <- dta[, 2:ncol(dta)]
ic0 <- IC_threshold_matrix(data = dta0)
cor_threshold <- IC_threshold_matrix(data=ic0, threshold = 0.3)
par(mar=c(1,1,1,1))
set.seed(4)
library("igraph")

plot(cor_threshold, vertex.color="red", show.legend.strength = FALSE)
plot(IC_correlation_simplify(matrix=cor_threshold), 
     show.legend.strength = FALSE, show.legend.direction = FALSE)

}
}
\references{
Lesty, M., 1999. Une nouvelle approche dans le choix des régresseurs de la régression multiple en présence d’interactions et de colinéarités. Revue de Modulad 22, 41-77.
}
\seealso{
Other Iconography of correlations:
\code{\link[=IC_clean_data]{IC_clean_data()}},
\code{\link[=IC_correlation_simplify]{IC_correlation_simplify()}},
\code{\link[=plot.IconoCorel]{plot.IconoCorel()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Iconography of correlations}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggamma.R
\name{dggamma}
\alias{dggamma}
\alias{pggamma}
\alias{qggamma}
\alias{rggamma}
\title{Generalized gamma distribution.}
\usage{
dggamma(x, theta, kappa, delta, log = FALSE)

pggamma(q, theta, kappa, delta, lower.tail = TRUE, log.p = FALSE)

qggamma(p, theta, kappa, delta, lower.tail = TRUE, log.p = FALSE)

rggamma(n, theta, kappa, delta)
}
\arguments{
\item{x, q}{vector of quantiles.}

\item{theta}{scale parameter.}

\item{kappa}{shape parameter.}

\item{delta}{shape parameter.}

\item{log, log.p}{logical; if TRUE, probabilities \emph{p} are given as \emph{log(p)}.}

\item{lower.tail}{logical; if TRUE (default), probabilities are \emph{P[X <= x]} otherwise, \emph{P[X > x]}.}

\item{p}{vector of probabilities.}

\item{n}{number of observations.}
}
\value{
\code{dggamma} gives the density, \code{pggamma} gives the distribution function,
\code{qggamma} gives the quantile function, and \code{rggamma} generates random deviates.
}
\description{
Generalized gamma distribution
}
\details{
\code{pggamma}, \code{qggamma}, \code{dggamma}, and \code{rggamma} are used to model the
generalized gamma distribution.

The code is modified from \url{https://rpubs.com/FJRubio/GG}.
}
\section{Functions}{
\itemize{
\item \code{dggamma()}: Density of the generalized gamma.

\item \code{pggamma()}: Distribution function of the generalized gamma.

\item \code{qggamma()}: Quantile of the generalized gamma.

\item \code{rggamma()}: Random of the generalized gamma.

}}
\section{More details here}{

The generalized gamma is described here \url{https://en.wikipedia.org/wiki/Generalized_gamma_distribution}.\cr
With \code{a} being \code{theta}, \code{b} being \code{kappa}, and \code{p} being \code{delta}.\cr
\code{theta}, \code{kappa} and \code{delta} must be all > 0.
}

\examples{
# To reproduce the wikipedia page graphic
x <- seq(from=0, to=8, by=0.1)
plot(x, dggamma(x, theta=2, kappa=0.5, delta=0.5), lty=1, col="blue", 
     type="l", lwd=2, xlab="x", ylab="PDF")
lines(x, dggamma(x, theta=1, kappa=1, delta=0.5), lty=1, col="green", lwd=2)
lines(x, dggamma(x, theta=2, kappa=1, delta=2), lty=1, col="red", lwd=2)
lines(x, dggamma(x, theta=5, kappa=1, delta=5), lty=1, col="yellow", lwd=2)
lines(x, dggamma(x, theta=7, kappa=1, delta=7), lty=1, col="grey", lwd=2)
legend("topright", legend=c("a=2, d=0.5, p=0.5", "a=1, d=1, p=0.5", 
                            "a=2, d=1, p=2", "a=5, d=1, p=5", "a=7, d=1, p=7"), 
                            col=c("blue", "green", "red", "yellow", "grey"), 
                            lty=1, lwd=2, bty="n")
par <- c(theta=2, kappa=0.5, delta=0.5)
# Mean, var and sd
mean.ggamma <- function(theta, kappa, delta) 
       return(theta*(gamma((kappa+1)/delta))/gamma(kappa/delta))
var.ggamma <- function(theta, kappa, delta) 
       return(theta^2* ( ( (gamma((kappa+2)/delta))/gamma(kappa/delta) ) - 
                ( (gamma((kappa+1)/delta))/gamma(kappa/delta) )^2 ) )
sd.ggamma <- function(theta, kappa, delta) 
       return(sqrt(theta^2* ( ( (gamma((kappa+2)/delta))/gamma(kappa/delta) ) - 
                ( (gamma((kappa+1)/delta))/gamma(kappa/delta) )^2 ) ))
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/show_name.R
\name{show_name}
\alias{show_name}
\title{Show the name of a point}
\usage{
show_name(
  points = NULL,
  x = NULL,
  y = NULL,
  names = NULL,
  col = "red",
  silent = FALSE
)
}
\arguments{
\item{points}{A list with x, y and names elements}

\item{x}{The x coordinates}

\item{y}{The y coordinates.}

\item{names}{The names of the points}

\item{col}{Color of the legend.}

\item{silent}{TRUE or FALSE}
}
\value{
Name of the point
}
\description{
Click on a point in plot region and it will tell you what is the point.
}
\details{
Show the name of a point
}
\examples{
\dontrun{
k <- plot_errbar(1:100, rnorm(100, 1, 2), 
	xlab="axe x", ylab="axe y", bty="n", xlim=c(1,100), 
		errbar.x=2, errbar.y=rnorm(100, 1, 0.1))
show_name(k)
k <- plot_errbar(1:10, rnorm(10, 1, 2), 
	xlab="axe x", ylab="axe y", bty="n", xlim=c(1,10), 
		errbar.x=2, errbar.y=rnorm(10, 1, 0.1), 
		names=LETTERS[1:10])
show_name(k)
k <- plot_errbar(1:10, rnorm(10, 1, 2), 
	xlab="axe x", ylab="axe y", bty="n", xlim=c(1,10), 
		errbar.x=2, errbar.y=rnorm(10, 1, 0.1))
show_name(k, names=LETTERS[1:10])
		}
}
\seealso{
\code{plot_errorbar}

Other plot and barplot functions:
\code{\link[=ScalePreviousPlot]{ScalePreviousPlot()}},
\code{\link[=barplot_errbar]{barplot_errbar()}},
\code{\link[=plot_add]{plot_add()}},
\code{\link[=plot_errbar]{plot_errbar()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{plot and barplot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.cutter.R
\name{print.cutter}
\alias{print.cutter}
\title{Print results of cutter that best describe distribution}
\usage{
\method{print}{cutter}(x, silent = FALSE, ...)
}
\arguments{
\item{x}{A result file generated by cutter}

\item{silent}{If TRUE does not show the outpout}

\item{...}{Not used}
}
\value{
Nothing
}
\description{
Print the estimates of cut distribution.
}
\details{
print.cutter plot result of cutter
}
\examples{
\dontrun{
library(HelpersMG)
# _______________________________________________________________
# right censored distribution with gamma distribution
# _______________________________________________________________
# Detection limit
DL <- 100
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc>DL] <- +Inf
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, upper_detection_limit=DL, 
                           cut_method="censored")
result
plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# The same data seen as truncated data with gamma distribution
# _______________________________________________________________
obc <- obc[is.finite(obc)]
# search for the parameters the best fit these truncated data
result <- cutter(observations=obc, upper_detection_limit=DL, 
                           cut_method="truncated")
result
plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# left censored distribution with gamma distribution
# _______________________________________________________________
# Detection limit
DL <- 10
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc<DL] <- -Inf
# search for the parameters the best fit these truncated data
result <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored")
result
plot(result, xlim=c(0, 200), breaks=seq(from=0, to=200, by=10))
# _______________________________________________________________
# left and right censored distribution
# _______________________________________________________________
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# Detection limit
LDL <- 10
# remove the data below the detection limit
obc[obc<LDL] <- -Inf
# Detection limit
UDL <- 100
# remove the data below the detection limit
obc[obc>UDL] <- +Inf
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, lower_detection_limit=LDL, 
                           upper_detection_limit=UDL, 
                          cut_method="censored")
result
plot(result, xlim=c(0, 150), col.DL=c("black", "grey"), 
                             col.unobserved=c("green", "blue"), 
     breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# Example with two values for lower detection limits
# corresponding at two different methods of detection for example
# with gamma distribution
# _______________________________________________________________
obc <- rgamma(50, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL1 <- 10
# remove the data below the detection limit
obc[obc<LDL1] <- -Inf
obc2 <- rgamma(50, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL2 <- 20
# remove the data below the detection limit
obc2[obc2<LDL2] <- -Inf
obc <- c(obc, obc2)
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, 
                           lower_detection_limit=c(rep(LDL1, 50), rep(LDL2, 50)), 
                          cut_method="censored")
result
# It is difficult to choose the best set of colors
plot(result, xlim=c(0, 150), col.dist="red", 
     col.unobserved=c(rgb(red=1, green=0, blue=0, alpha=0.1), 
                      rgb(red=1, green=0, blue=0, alpha=0.2)), 
     col.DL=c(rgb(red=0, green=0, blue=1, alpha=0.5), 
                      rgb(red=0, green=0, blue=1, alpha=0.9)), 
     breaks=seq(from=0, to=200, by=10))
# ___________________________________________________________________
# left censored distribution comparison of normal, lognormal and gamma
# ___________________________________________________________________
# Detection limit
DL <- 10
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc<DL] <- -Inf
# search for the parameters the best fit these truncated data
result_gamma <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="gamma")
result_gamma
plot(result_gamma, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

result_lognormal <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="lognormal")
result_lognormal
plot(result_lognormal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

result_normal <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="normal")
result_normal
plot(result_normal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

compare_AICc(gamma=result_gamma, 
            lognormal=result_lognormal, 
            normal=result_normal)
# ___________________________________________________________________
# Test for similarity in gamma left censored distribution between two
# datasets
# ___________________________________________________________________
obc1 <- rgamma(100, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL <- 10
# remove the data below the detection limit
obc1[obc1<LDL] <- -Inf
obc2 <- rgamma(100, scale=10, shape=2)
# remove the data below the detection limit
obc2[obc2<LDL] <- -Inf
# search for the parameters the best fit these censored data
result1 <- cutter(observations=obc1, 
                  distribution="gamma", 
                  lower_detection_limit=LDL, 
                  cut_method="censored")
logLik(result1)
plot(result1, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
result2 <- cutter(observations=obc2, 
                  distribution="gamma", 
                  lower_detection_limit=LDL, 
                  cut_method="censored")
logLik(result2)
plot(result2, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
result_totl <- cutter(observations=c(obc1, obc2), 
                      distribution="gamma", 
                      lower_detection_limit=LDL, 
                      cut_method="censored")
logLik(result_totl)
plot(result_totl, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
     
compare_AICc(Separate=list(result1, result2), 
            Common=result_totl, factor.value=1)
compare_BIC(Separate=list(result1, result2), 
            Common=result_totl, factor.value=1)           
}
}
\seealso{
Other Distributions: 
\code{\link{cutter}()},
\code{\link{dSnbinom}()},
\code{\link{dbeta_new}()},
\code{\link{dcutter}()},
\code{\link{dggamma}()},
\code{\link{logLik.cutter}()},
\code{\link{plot.cutter}()},
\code{\link{r2norm}()},
\code{\link{rcutter}()},
\code{\link{rmnorm}()},
\code{\link{rnbinom_new}()}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/newcompassRose.R
\name{newcompassRose}
\alias{newcompassRose}
\title{Display a compass rose}
\usage{
newcompassRose(
  x,
  y,
  rot = 0,
  cex = 1,
  col = "black",
  col.arrows.light = "white",
  col.arrows.dark = "black"
)
}
\arguments{
\item{x}{The position of the center of the compass rose in user units.}

\item{y}{The position of the center of the compass rose in user units.}

\item{rot}{Rotation for the compass rose in degrees. See Details.}

\item{cex}{The character expansion to use in the display.}

\item{col}{The color of text}

\item{col.arrows.light}{The color of lighter lines}

\item{col.arrows.dark}{The color of darker lines}
}
\value{
none
}
\description{
Displays a basic compass rose, usually to orient a map.\cr
newcompassRose displays a conventional compass rose at the position requested.\cr
The size of the compass rose is determined by the character expansion,
as the central "rose" is calculated relative to the character size.\cr
Rotation is in degrees counterclockwise.
}
\details{
newcompassRose Display a compass rose
}
\examples{
\dontrun{
library(HelpersMG)
require("maps")
map("world", "China")
newcompassRose(x=110, y=35, col.arrows.light="grey")
}
}
\author{
modified from Jim Lemon; See \code{\link[sp:compassRose]{sp::compassRose()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.mcmcComposite.R
\name{merge.mcmcComposite}
\alias{merge.mcmcComposite}
\title{Merge two mcmcComposite results}
\usage{
\method{merge}{mcmcComposite}(x, y, ...)
}
\arguments{
\item{x}{A mcmcComposite obtained as a result of \code{MHalgoGen()} function}

\item{y}{A mcmcComposite obtained as a result of \code{MHalgoGen()} function}

\item{...}{not used}
}
\value{
A mcmcComposite result
}
\description{
Merge two mcmcComposite results and produced a new one mcmcComposite object.\cr
Note that the initial value
for the second run must use the last value of the first one as shown in example.
}
\details{
merge.mcmcComposite Merge two mcmcComposite results
}
\examples{
\dontrun{
library(HelpersMG)
require(coda)
x <- rnorm(30, 10, 2)
dnormx <- function(data, x) {
 data <- unlist(data)
 return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
}
parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(1, 1), 
Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
row.names=c('mean', 'sd'))
mcmc_run <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
plot(mcmc_run, xlim=c(0, 20))
plot(mcmc_run, xlim=c(0, 10), parameters="sd")
mcmcforcoda <- as.mcmc(mcmc_run)
#' heidel.diag(mcmcforcoda)
raftery.diag(mcmcforcoda)
autocorr.diag(mcmcforcoda)
acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
batchSE(mcmcforcoda, batchSize=100)
# The batch standard error procedure is usually thought to 
# be not as accurate as the time series methods used in summary
summary(mcmcforcoda)$statistics[,"Time-series SE"]
summary(mcmc_run)
as.parameters(mcmc_run)
lastp <- as.parameters(mcmc_run, index="last")
parameters_mcmc[,"Init"] <- lastp
# The n.adapt set to 1 is used to not record the first set of parameters
# then it is not duplicated (as it is also the last one for 
# the object mcmc_run)
mcmc_run2 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
likelihood=dnormx, n.chains=1, n.adapt=1, thin=1, trace=1)
mcmc_run3 <- merge(mcmc_run, mcmc_run2)
####### no adaptation, n.adapt must be 0
parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
mcmc_run3 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
likelihood=dnormx, n.chains=1, n.adapt=0, thin=1, trace=1)
}
}
\seealso{
Other mcmcComposite functions:
\code{\link[=+.PriorsmcmcComposite]{+.PriorsmcmcComposite()}},
\code{\link[=MHalgoGen]{MHalgoGen()}},
\code{\link[=PPD]{PPD()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=as.parameters]{as.parameters()}},
\code{\link[=plot.PriorsmcmcComposite]{plot.PriorsmcmcComposite()}},
\code{\link[=plot.mcmcComposite]{plot.mcmcComposite()}},
\code{\link[=setPriors]{setPriors()}},
\code{\link[=setPriors1]{setPriors1()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/char.R
\name{char}
\alias{char}
\title{Return the characters defined by the codes}
\usage{
char(n)
}
\arguments{
\item{n}{The code to be used to return a character}
}
\value{
A string with characters defined by the codes
}
\description{
Return a string with characters defined by the codes.
}
\details{
char returns the characters defined by the codes
}
\examples{
char(65:75)
char(unlist(tapply(144:175, 144:175, function(x) {c(208, x)})))
}
\seealso{
Other Characters:
\code{\link[=asc]{asc()}},
\code{\link[=d]{d()}},
\code{\link[=tnirp]{tnirp()}}
}
\author{
Based on this blog: http://datadebrief.blogspot.com/2011/03/ascii-code-table-in-r.html
}
\concept{Characters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnbinom_new.R
\name{rnbinom_new}
\alias{rnbinom_new}
\title{Random numbers for the negative binomial distribution.}
\usage{
rnbinom_new(n, size = NULL, prob = NULL, mu = NULL, sd = NULL, var = NULL)
}
\arguments{
\item{n}{number of observations.}

\item{size}{target for number of successful trials, or dispersion parameter (the shape parameter of the gamma mixing distribution). Must be strictly positive, need not be integer.}

\item{prob}{probability of success in each trial. 0 < prob <= 1.}

\item{mu}{alternative parametrization via mean.}

\item{sd}{alternative parametrization via standard deviation.}

\item{var}{alternative parametrization via variance.}
}
\value{
Random numbers for the negative binomial distribution
}
\description{
See \code{rnbinom}.
}
\details{
rnbinom_new returns random numbers for the negative binomial distribution
}
\examples{
\dontrun{
library("HelpersMG")
set.seed(1)
x <- rnbinom_new(n=1000, prob=6.25/(5+6.25), size=6.25)
mean(x)
sd(x)
set.seed(1)
x <- rnbinom_new(n=1000, mu=5, sd=3)
mean(x)
sd(x)
set.seed(1)
x <- rnbinom_new(n=1000, mu=5, var=3^2)
mean(x)
sd(x)
set.seed(1)
x <- rnbinom_new(n=1000, mu=5, size=6.25)
mean(x)
sd(x)
set.seed(1)
x <- rnbinom_new(n=1000, size=6.25, var=3^2)
mean(x)
sd(x)
set.seed(1)
x <- rnbinom_new(n=1000, prob=6.25/(5+6.25), var=3^2)
mean(x)
sd(x)
# Example of wrong parametrization
set.seed(1)
x <- rnbinom_new(n=1000, sd=3, var=3^2)
set.seed(1)
x <- rnbinom_new(n=1000, mu=10, var=3^2)
}
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Ellipse.R
\name{ellipse}
\alias{ellipse}
\title{Plot an ellipse}
\usage{
ellipse(
  center.x = 0,
  center.y = 0,
  radius.x = 1,
  radius.y = 1,
  radius.x.lower = NULL,
  radius.x.upper = NULL,
  radius.y.lower = NULL,
  radius.y.upper = NULL,
  alpha = 0,
  binconf.x = NULL,
  binconf.y = NULL,
  control.binconf = list(conf.level = 0.95, method = "wilsoncc"),
  length = 100,
  ...
)
}
\arguments{
\item{center.x}{Center of the ellipse on x axis}

\item{center.y}{Center of the ellipse on y axis}

\item{radius.x}{Radius along the x axis}

\item{radius.y}{Radius along the y axis}

\item{radius.x.lower}{Radius along the x axis, at left of center}

\item{radius.x.upper}{Radius along the x axis, at right of center}

\item{radius.y.lower}{Radius along the y axis, at bottom of center}

\item{radius.y.upper}{Radius along the y axis, at top of center}

\item{alpha}{Rotation in radians}

\item{binconf.x}{A data.frame or a matrix with two columns, x and n or with three columns, PointEst, Lower, and Upper}

\item{binconf.y}{A data.frame or a matrix with two columns, x and n or with three columns, PointEst, Lower, and Upper}

\item{control.binconf}{A list with options for binomial confidence}

\item{length}{Number of points to draw the ellipse}

\item{...}{Graphical parameters}
}
\value{
Nothing
}
\description{
Plot a ellipse defined by the center and the radius. The options for
binomial confidence parameters are:\cr
\itemize{
\item conf.level\cr
\item method  must be one of these "wald", "wilson", "wilsoncc", "agresti-coull", "jeffreys", "modified wilson", "modified jeffreys", "clopper-pearson", "arcsine", "logit", "witting", "pratt", "midp", "lik" and "blaker". Defaults to "wilson". Abbreviation of method is accepted. See details.\cr
Default is wilsoncc (Wilson with continuity correction)
col parameter can be a list of colors. See examples
}
}
\details{
ellipse plots an ellipse
}
\examples{
plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
     xlab="Variable x", ylab="variable y")
 
ellipse(center.x = c(0.2, 0.3, 0.25), center.y = c(0.7, 0.6, 0.55), 
        radius.x = c(0.1, 0.1, 0.1), radius.y = c(0.15, 0.2, 0.4), 
        border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1))

ellipse(center.x = 0.5, center.y = 0.5, 
        radius.x.lower = 0.1, radius.x.upper = 0.3, 
        radius.y = 0.2, 
        border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1))

ellipse(center.x = 0.6, center.y = 0.3, 
        radius.x.lower = 0.3, radius.x.upper = 0.3, 
        radius.y.lower = 0.2, radius.y.upper = 0.4, 
        border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1))

plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", bty="n", asp=1, 
     xlab="Variable x", ylab="variable y", axes=FALSE)
axis(1, at=c(0, 0.25, 0.5, 0.75, 1))
axis(2, at=c(0, 0.25, 0.5, 0.75, 1), las=1)

ellipse(center.x = 0.5, center.y = 0.5, radius.x = 0.2, radius.y = 0.4, 
       border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1))
ellipse(center.x = 0.5, center.y = 0.5, radius.x = 0.2, radius.y = 0.4, 
        border=NA, col=rgb(red = 0.1, green = 0.1, blue = 0.1, alpha = 0.1), alpha = pi/4)

plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
     xlab="Variable x", ylab="variable y")

for (k in 0:8)
  ellipse(center.x=0.5, center.y=0.5, radius.x=0.1, radius.y=0.4, 
          alpha=seq(from=0, to=pi/4, length=9)[k], 
          border=rainbow(9)[k])

# Exemple with confidence of proportions
males <- c(10, 25, 3, 4)
N <- c(12, 52, 17, 10)

males2 <- c(12, 20, 3, 6)
N2 <- c(15, 50, 20, 12)

plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
     xlab="Variable x", ylab="variable y")

ellipse(binconf.x = data.frame(x=males, n=N), binconf.y = data.frame(x=males2, n=N2),  
        border=NA, col=rgb(red = 0.1, green = 0.5, blue = 0.1, alpha = 0.1))
        
plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
     xlab="Variable x", ylab="variable y")
     
ellipse(binconf.x = data.frame(x=males, n=N), 
        binconf.y = data.frame(PointEst=c(0.1, 0.2, 0.3, 0.5), 
                               Lower=c(0.02, 0.12, 0.25, 0.30), 
                               Upper=c(0.18, 0.29, 0.35, 0.67)), 
        border=NA, col=rgb(red = 0.1, green = 0.5, blue = 0.1, alpha = 0.1))
        
# Examples with a gradient
plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
     xlab="Variable x", ylab="variable y")
ellipse(center.x = 0.6, center.y = 0.3, 
        radius.x.lower = 0.3, radius.x.upper = 0.3, 
        radius.y.lower = 0.2, radius.y.upper = 0.4, 
        border=NA, col=grey.colors(100, alpha = 0.1))
        
plot(0:1, 0:1, xlim=c(0, 1), ylim=c(0,1), lty=2, type="l", las=1, bty="n", 
     xlab="Variable x", ylab="variable y")
ellipse(binconf.x = data.frame(x=males, n=N), binconf.y = data.frame(x=males2, n=N2),  
        border=NA, col=grey.colors(100, alpha = 0.1))

}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/series.compare.R
\name{series.compare}
\alias{series.compare}
\title{Data series comparison using Akaike weight}
\usage{
series.compare(..., criterion = c("BIC", "AIC", "AICc"), var.equal = TRUE)
}
\arguments{
\item{...}{Series of data (at least two or data are in a table with series in different rows)}

\item{criterion}{Which criterion is used for model selection. can be AIC, AICc or BIC}

\item{var.equal}{Should the variances of all series being equal? Default TRUE}
}
\value{
The probability that a single proportion model is sufficient to explain the data
}
\description{
This function is used as a replacement of t.test() to not use p-value.
}
\details{
series.compare compares series of data using Akaike weight.
}
\examples{
\dontrun{
library("HelpersMG")
A <- rnorm(100, 10, 2)
B <- rnorm(100, 11.1, 2)
series.compare(A, B, criterion = "BIC", var.equal=TRUE)
B <- B[1:10]
series.compare(A, B, criterion = "BIC", var.equal=TRUE)
A <- rnorm(100, 10, 2)
B <- rnorm(100, 10.1, 2)
C <- rnorm(100, 10.5, 2)
series.compare(A, B, C, criterion = "BIC", var.equal=TRUE)
B <- B[1:10]
series.compare(A, B, criterion = "BIC", var.equal=TRUE)
t.test(A, B, var.equal=TRUE)
# Example with a data.frame
series.compare(t(data.frame(A=c(10, 27, 19, 20, NA), B=c(10, 20, NA, NA, NA))))
# Test in the context of big data
A <- rnorm(10000, 10, 2)
B <- rnorm(10000, 10.1, 2)
series.compare(A, B, criterion = "BIC", var.equal=TRUE)
t.test(A, B, var.equal=TRUE)
###########################
w <- NULL
p <- NULL

for (i in 1:1000) {
  
  A <- rnorm(50000, 10, 2)
  B <- rnorm(50000, 10.01, 2)
  w <- c(w, unname(series.compare(A, B, criterion = "BIC", var.equal=TRUE)[1]))
  p <- c(p, t.test(A, B, var.equal=TRUE)$p.value)

}

layout(mat = 1:2)
par(mar=c(4, 4, 1, 1)+0.4)
hist(p, main="", xlim=c(0, 1), las=1, breaks = (0:20)/20, 
     freq=FALSE, xlab = expression(italic("p")*"-value"))
hist(w, main="", xlim=c(0, 1), las=1, breaks = (0:20)/20, 
    freq=FALSE, xlab = expression(italic("w")*"-value"))
###########################

x <- seq(from=8, to=13, by=0.1)

pv <- NULL
aw <- NULL
A <- rnorm(100, mean=10, sd=2)
B <- A-2

for (meanB in x) {
  pv <- c(pv, t.test(A, B, var.equal = FALSE)$p.value)
  aw <- c(aw, series.compare(A, B, criterion="BIC", var.equal = FALSE)[1])
  B <- B + 0.1
}

par(mar=c(4, 4, 2, 1)+0.4)
y <- pv
plot(x=x, y=y, type="l", lwd=2,
     bty="n", las=1, xlab="Mean B value (SD = 4)", ylab="Probability", ylim=c(0,1), 
     main="")
y2 <- aw
lines(x=x, y=y2, type="l", col="red", lwd=2)

l1 <- which(aw>0.05)[1]
l2 <- max(which(aw>0.05))

aw[l1]
pv[l1]

aw[l2]
pv[l2]

l1 <- which(pv>0.05)[1]
l2 <- max(which(pv>0.05))

aw[l1]
pv[l1]

aw[l2]
pv[l2]

par(xpd=TRUE)
segments(x0=10-1.96*2/10, x1=10+1.96*2/10, y0=1.1, y1=1.1, lwd=2)
segments(x0=10, x1=10, y0=1.15, y1=1.05, lwd=2)
par(xpd=TRUE)
text(x=10.5, y=1.1, labels = "Mean A = 10, SD = 2", pos=4)

v1 <- c(expression(italic("p")*"-value"), expression("based on "*italic("t")*"-test"))
v2 <- c(expression(italic("w")*"-value for A"), expression("and B identical models"))
legend("topright", legend=c(v1, v2), 
       y.intersp = 1, 
       col=c("black", "black", "red", "red"), bty="n", lty=c(1, 0, 1, 0))

segments(x0=min(x), x1=max(x), y0=0.05, y1=0.05, lty=2)
par(xpd = TRUE)
text(x=13.05, y=0.05, labels = "0.05", pos=4)
}
}
\references{
Girondot, M., Guillon, J.-M., 2018. The w-value: An alternative to t- and X2 tests. Journal of Biostatistics & Biometrics 1, 1-4.
}
\seealso{
Other w-value functions:
\code{\link[=compare]{compare()}},
\code{\link[=contingencyTable.compare]{contingencyTable.compare()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{w-value functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.IconoCorel.R
\name{plot.IconoCorel}
\alias{plot.IconoCorel}
\title{Clean the dataframe before to be used with IC_threshold_matrix}
\usage{
\method{plot}{IconoCorel}(
  x,
  ...,
  show.legend.direction = "bottomright",
  show.legend.strength = "topleft",
  title = "Correlation iconography",
  vertex.label.color = "black",
  vertex.label = NULL,
  vertex.color = "white",
  vertex.label.cex = 1,
  plot = TRUE
)
}
\arguments{
\item{x}{The correlation matrix to show}

\item{...}{other options of plot.igraph()}

\item{show.legend.direction}{the position of the legend of direction; FALSE to not show it}

\item{show.legend.strength}{the position of the legend with intensity of correlation; FALSE to not show it}

\item{title}{the title of the plot}

\item{vertex.label.color}{a vector with the colors of labels}

\item{vertex.label}{a vector with the labels}

\item{vertex.color}{a vector of colors}

\item{vertex.label.cex}{a vector of cex}

\item{plot}{if TRUE, the plot is shown}
}
\value{
A igraph object
}
\description{
This function plots the data as a network. It returns an invisible object that can be used with visIgraph from package visNetwork.
\code{https://fr.wikipedia.org/wiki/Iconographie_des_corrélations}
}
\details{
plot.IconoCorel checks and corrects the dataframe to be used with IC_threshold_matrix
}
\examples{
\dontrun{
library("HelpersMG")
es <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
                 Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
                 Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
                 Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
                 Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
                 row.names = c(NA, -8L), class = "data.frame")

es

df <- IC_clean_data(es, debug = TRUE)
cor_matrix <- IC_threshold_matrix(data=df, threshold = NULL, progress=FALSE)
cor_threshold <- IC_threshold_matrix(data=df, threshold = 0.3)
par(mar=c(1,1,1,1))
set.seed(4)
library("igraph")
library("visNetwork")
kk <- plot(cor_threshold, vertex.color="red")
# it can be shown also with the visNetwork package
visIgraph(kk)
cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
plot(cor_threshold_Note)

# You can record the position of elements and use them later
ly <- layout_nicely(kk)
plot(cor_threshold, vertex.color="red", layout=ly)
}
}
\references{
Lesty, M., 1999. Une nouvelle approche dans le choix des régresseurs de la régression multiple en présence d’interactions et de colinéarités. Revue de Modulad 22, 41-77.
}
\seealso{
Other Iconography of correlations:
\code{\link[=IC_clean_data]{IC_clean_data()}},
\code{\link[=IC_correlation_simplify]{IC_correlation_simplify()}},
\code{\link[=IC_threshold_matrix]{IC_threshold_matrix()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Iconography of correlations}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LD50_MHmcmc_p.R
\name{LD50_MHmcmc_p}
\alias{LD50_MHmcmc_p}
\title{Generates set of parameters to be used with LD50_MHmcmc()}
\usage{
LD50_MHmcmc_p(
  result = stop("An output from LD50() must be provided"),
  accept = FALSE
)
}
\arguments{
\item{result}{An object obtained after a LD50 fit}

\item{accept}{If TRUE, the script does not wait user information}
}
\value{
A matrix with the parameters
}
\description{
Interactive script used to generate set of parameters to be
used with LD50_MHmcmc().
}
\details{
LD50_MHmcmc_p generates set of parameters to be used with LD50_MHmcmc()
}
\examples{
\dontrun{
library("HelpersMG")
data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
Alive=c(10, 12, 8, 6, 2, 1),
Dead=c(0, 1, 5, 6, 9, 15))
LD50_logistic <- LD50(data, equation="logistic")
pmcmc <- LD50_MHmcmc_p(LD50_logistic, accept=TRUE)
}
}
\seealso{
Other LD50 functions:
\code{\link[=LD50]{LD50()}},
\code{\link[=LD50_MHmcmc]{LD50_MHmcmc()}},
\code{\link[=logLik.LD50]{logLik.LD50()}},
\code{\link[=plot.LD50]{plot.LD50()}},
\code{\link[=predict.LD50]{predict.LD50()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{LD50 functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/symbol.Male.R
\name{symbol.Male}
\alias{symbol.Male}
\title{Plot a male symbol in the plotting region}
\usage{
symbol.Male(centerx, centery, rayonx, lwd = 2, col = "black")
}
\arguments{
\item{centerx}{The x position of the center of the circle}

\item{centery}{The y position of the center of the circle}

\item{rayonx}{The size of the rayon in the scale of the x axis}

\item{lwd}{The width of the line of the symbol}

\item{col}{The color of the symbol}
}
\value{
Nothing
}
\description{
Plot a male symbol in the plotting region.
}
\details{
symbol.Male plot a male symbol in the plotting region
}
\examples{
\dontrun{
plot(x=1:2, y=c(10,20), type="n", bty="n", xlab="", ylab="")

rayonx <- 0.01
centerx <- 1.2
centery <- 15

symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx)
symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx)

rayonx <- 0.03
centerx <- 1.2
centery <- 18

symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx, lwd=3)
symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx, lwd=3, col="red")

rayonx <- 0.05
centerx <- 1.4
centery <- 13

symbol.Male(centerx=centerx, centery = centery, rayonx=rayonx, lwd=4, col="blue")
symbol.Female(centerx=centerx+0.5, centery = centery, rayonx=rayonx, lwd=4, col="red")
}
}
\seealso{
Other Symbol:
\code{\link[=symbol.Female]{symbol.Female()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Symbol}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/r2norm.R
\name{r2norm}
\alias{r2norm}
\title{Random generation for Gaussian distributions different at left and right}
\usage{
r2norm(n, mean = 0, sd_low = 1, sd_high = 1)
}
\arguments{
\item{n}{number of observations.}

\item{mean}{vector of means}

\item{sd_low}{vector of standard deviations below the mean.}

\item{sd_high}{vector of standard deviations above the mean.}
}
\value{
r2norm returns random numbers
}
\description{
Random generation for Gaussian distributions different at left and right
}
\details{
r2norm returns random numbers for Gaussian distributions different at left and right
}
\examples{
\dontrun{
n <- r2norm(1000, mean=25, sd_low=2, sd_high=10)

hist(n)
}
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.LD50.R
\name{plot.LD50}
\alias{plot.LD50}
\title{Plot results of LD50() that best describe LD50}
\usage{
\method{plot}{LD50}(
  x,
  ...,
  las.x = 1,
  las.y = 1,
  lab.PT = "LD50",
  at = NULL,
  lab.TRD = paste0("Transitional range of doses l=", l * 100, "\%"),
  col.TRD = "gray",
  col.TRD.CI = rgb(0.8, 0.8, 0.8, 0.5),
  col.PT.CI = rgb(0.8, 0.8, 0.8, 0.5),
  show.CI = TRUE
)
}
\arguments{
\item{x}{A result file generated by LD50()}

\item{...}{Parameters for plot()}

\item{las.x}{las parameter for x axis}

\item{las.y}{las parameter for y axis}

\item{lab.PT}{Label to describe pivotal dose}

\item{at}{Position of ticks in x-axis}

\item{lab.TRD}{Label to describe transitional range of dose}

\item{col.TRD}{The color of TRD}

\item{col.TRD.CI}{The color of CI of TRD based on range.CI}

\item{col.PT.CI}{The color of CI of PT based on range.CI}

\item{show.CI}{Do the CI for the curve should be shown}
}
\value{
Nothing
}
\description{
Plot the estimates that best describe lethality of doses.
}
\details{
plot.LD50 plot result of LD50() that best describe LD50
}
\examples{
\dontrun{
data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
Alive=c(10, 12, 8, 6, 2, 1),
Dead=c(0, 1, 5, 6, 9, 15))
LD50_logistic <- LD50(data, equation="logistic")
predict(LD50_logistic, doses=c(140, 170))
plot(LD50_logistic, xlim=c(0, 300))
}
}
\seealso{
Other LD50 functions:
\code{\link[=LD50]{LD50()}},
\code{\link[=LD50_MHmcmc]{LD50_MHmcmc()}},
\code{\link[=LD50_MHmcmc_p]{LD50_MHmcmc_p()}},
\code{\link[=logLik.LD50]{logLik.LD50()}},
\code{\link[=predict.LD50]{predict.LD50()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{LD50 functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/similar.R
\name{similar}
\alias{similar}
\title{Test if two vectors contains the same elements independently of their order}
\usage{
similar(x, y, test.names = FALSE)
}
\arguments{
\item{x}{A vector with numeric or character elements}

\item{y}{A vector with numeric or character elements}

\item{test.names}{Logical. If TRUE, the names of the vector elements must be also identical and unique}
}
\value{
A logical TRUE or FALSE
}
\description{
Return TRUE only if all elements of x are present and only once in y.\cr
}
\examples{
\dontrun{
A <- c("A", "B", "C", "D")
B <- c("A", "B", "C", "D")
similar(A, B)
similar(B, A)
A <- c(x="A", y="B", z="C", k="D")
B <- c(x="A", y="B", z="C", l="D")
similar(B, A)
similar(A, B, test.names=TRUE)
A <- c(x="A", y="B", z="C", k="D")
B <- c(x="A", z="C", k="D", y="B")
similar(B, A)
similar(A, B, test.names=TRUE)
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/specify_decimals.R
\name{specify_decimal}
\alias{specify_decimal}
\title{Return a number as character with specified number of decimals}
\usage{
specify_decimal(x, decimals = NULL, decimal.point = ".")
}
\arguments{
\item{x}{The numbers to be formated}

\item{decimals}{Number of decimals to print}

\item{decimal.point}{Character to be used as decimal point}
}
\value{
A character
}
\description{
Return a number as character with specified number of decimals.
If a is a matrix, it will return a matrix of the same size and the same attributes.
}
\details{
specify_decimals format a number with specified number of decimals
}
\examples{
specify_decimal(x=pi, decimal.point=".")
specify_decimal(x=pi, decimals=4, decimal.point=".")
specify_decimal(x=c(pi, exp(1)), decimals=3, decimal.point=",")
specify_decimal(x=c(pi, exp(1)), decimal.point=",")
specify_decimal(x=c(pi*10, pi, pi/10, pi/100, pi/1000))
specify_decimal(x=c(pi=pi), decimal.point=".")
specify_decimal(x=matrix(pi*1:4, ncol=2), decimal.point=".")
m <- matrix(pi*1:4, ncol=2)
rownames(m) <- c("A", "B")
colnames(m) <- c("C", "D")
specify_decimal(x=m, decimal.point=".")
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/from_min_max.R
\name{from_min_max}
\alias{from_min_max}
\title{Distribution from minimum and maximum}
\usage{
from_min_max(
  n = stop("n must be known."),
  fitted.parameters = stop("At least one parameter must be supplied."),
  observed.Minimum,
  observed.Maximum,
  observed.Median = NULL,
  observed.Mean = NULL,
  observed.Quantiles = NULL,
  priors = "dnorm",
  fixed.parameters = NULL,
  D = "norm**",
  n.iter = 10000,
  n.chains = 1,
  n.adapt = 100,
  thin = 30,
  adaptive = FALSE,
  trace = 100,
  replicates = 10000,
  silent = FALSE
)
}
\arguments{
\item{n}{Number of observations}

\item{fitted.parameters}{The initial value to fit}

\item{observed.Minimum}{The observed minimum}

\item{observed.Maximum}{The observed maximum}

\item{observed.Median}{The observed median (can be omitted)}

\item{observed.Mean}{The observed mean (can be omitted)}

\item{observed.Quantiles}{Observed quantiles with names being the values of quantiles with Q being first letter (can be omitted; see description)}

\item{priors}{The priors (see MHAlgogen()) or character "dnorm" or "dunif"}

\item{fixed.parameters}{The fixed parameters}

\item{D}{The distribution to fit as character. ex "norm" or "lnorm"}

\item{n.iter}{Number of iterations for each chain}

\item{n.chains}{Number of chains}

\item{n.adapt}{Number of iteration to stabilize likelihood}

\item{thin}{Interval for thinning likelihoods}

\item{adaptive}{Should an adaptive process for SDProp be used}

\item{trace}{Or FALSE or period to show progress}

\item{replicates}{Number of replicates to model D}

\item{silent}{If TRUE, do not print information}
}
\value{
from_min_max returns a list with output, ML, Bayesian results
}
\description{
Bayesian estimate of distribution when minimum, maximum, median or mean are known.\cr
If D="norm**" or "lnorm**", it will use the approximation of Gumbel based on D.\cr
If D="norm*" or "lnorm*", it will generate D distribution using replicates number of random numbers,
and estimate Gumbel parameters from the simulated D distribution. \cr
Otherwise it will estimate parameters of Gumbel distribution based on maximum likelihood.\cr
For D="pois", "beta" or "chisq" only second (D="pois*", "beta*" or "chisq*") and third solutions are available.\cr
The observed.Quantiles parameter must be a named value, for example observed.Quantiles = c(Q0.025=17, Q0.975=26)\cr
}
\details{
from_min_max returns standard deviation and/or mean from minimum and maximum
}
\examples{
\dontrun{
minobs <- 5
maxobs <- 25
# These two values are not mandatory
meanobs <- 15
medianobs <- 16
n <- 10
# To estimate only the sd of the distribution; mean is fixed
# Note that there is an obligation to have mean even if it
# is in fixed.parameters
# By defaut a normal distribution is fitted

out_sd <- from_min_max(n=n                                , 
                       observed.Minimum=minobs                    , 
                       observed.Maximum=maxobs                    , 
                       fitted.parameters=c(sd=5)                  , 
                       fixed.parameters=c(mean=meanobs)           ,
                       n.iter=10000                               ,
                       trace=TRUE                                 )
                            
plot(out_sd, what="MarkovChain", parameters="sd")
plot(out_sd, what="posterior", parameters="sd")
as.parameters(out_sd, index="quantile")

# To estimate both the sd and mean of the distribution

out_sd_mean_norm <- from_min_max(n=n, 
                            observed.Minimum=minobs                    , 
                            observed.Maximum=maxobs                    , 
                            fitted.parameters=c(mean=15, sd=5)         , 
                            fixed.parameters=NULL                      ,
                            n.iter=10000                               ,
                            D="norm**"                                 ,
                            trace=FALSE                                )
                            
plot(out_sd_mean_norm, what="MarkovChain", parameters="sd", ylim=c(0, 10))
plot(out_sd_mean_norm, what="MarkovChain", parameters="mean", ylim=c(10, 20))
plot(out_sd_mean_norm, what="posterior", parameters="mean",  
     breaks=seq(from=0, to=100, by=1), xlim=c(10, 20))
as.parameters(out_sd_mean_norm, index="quantile")

# Let see what's happened for a lognormal distribution

out_sd_mean_lnorm <- from_min_max(n=n, 
                            observed.Minimum=minobs                    , 
                            observed.Maximum=maxobs                    , 
                            fitted.parameters=c(meanlog=15, sdlog=5)   , 
                            fixed.parameters=NULL                      ,
                            n.iter=10000                               ,
                            D="lnorm**"                                ,
                            trace=FALSE                                )
                            
plot(out_sd_mean_lnorm, what="MarkovChain", parameters="sdlog", ylim=c(0, 10))
plot(out_sd_mean_lnorm, what="MarkovChain", parameters="meanlog", ylim=c(10, 20))
plot(out_sd_mean_lnorm, what="posterior", parameters="meanlog", xlim=c(0, 20), 
     breaks=seq(from=0, to=100, by=1))
as.parameters(out_sd_mean_lnorm, index="quantile")

# To be compared with the rule of thumb:
print(paste0("mean = ", as.character((maxobs + minobs) / 2))) # Mean Not so bad
print(paste0("sd = ", as.character((maxobs - minobs) / 4))) # SD Clearly biased

# Covariation of sd and mean is nearly NULL
cor(x=as.parameters(out_sd_mean_norm, index="all")[, "mean"], 
    y=as.parameters(out_sd_mean_norm, index="all")[, "sd"])^2
plot(x=as.parameters(out_sd_mean_norm, index="all")[, "mean"], 
     y=as.parameters(out_sd_mean_norm, index="all")[, "sd"], 
     xlab="mean", ylab="sd")
     
# Example when minimum, maximum and mean are known

out_sd_mean2 <- from_min_max(n=n                                   , 
                             observed.Minimum=minobs               , 
                             observed.Maximum=maxobs               , 
                             observed.Mean=meanobs                 ,
                             fitted.parameters=c(mean=15, sd=5)    , 
                             fixed.parameters=NULL                 ,
                             n.iter=10000                          ,
                             trace=FALSE                            )
                            
# Example when minimum, maximum, mean and median are known

out_sd_mean3 <- from_min_max(n=n                                   , 
                             observed.Minimum=minobs               , 
                             observed.Maximum=maxobs               , 
                             observed.Mean=meanobs                 ,
                             observed.Median=medianobs             ,
                             fitted.parameters=c(mean=15, sd=5)    , 
                             fixed.parameters=NULL                 ,
                             n.iter=10000                          ,
                             trace=FALSE                            )
                             
plot(out_sd_mean2, what="MarkovChain", parameters="sd")
plot(out_sd_mean2, what="MarkovChain", parameters="mean")
plot(out_sd_mean2, what="posterior", parameters="mean", xlim=c(0, 100), 
     breaks=seq(from=0, to=100, by=5))
as.parameters(out_sd_mean2, index="quantile")

# Example of GEV density function  
# Parametrisation from https://en.wikipedia.org/wiki/Generalized_extreme_value_distribution
dGEV <- getFromNamespace(".dGEV", ns="HelpersMG")
x <- seq(from=-4, to=4, by=0.1)
plot(x, y=dGEV(x=x, 
               location=0, scale=1, shape=-1/2, log=FALSE, sum=FALSE), 
     type="l", col="green", xlab="x", ylab="Density")
lines(x, y=dGEV(x=x, 
                location=0, scale=1, shape=0, log=FALSE, sum=FALSE), col="red")
lines(x, y=dGEV(x=x, 
                location=0, scale=1, shape=1/2, log=FALSE, sum=FALSE), col="blue")
legend("topleft", legend=c("shape=-1/2", "shape=0", "shape=1/2"), 
       lty=1, col=c("green", "red", "blue"))
       
# Note the different parametrisation about shape
dGEV <- getFromNamespace("dgevd", ns="EnvStats")
x <- seq(from=-4, to=4, by=0.1)
plot(x, y=dGEV(x=x, 
               location=0, scale=1, shape=-1/2), 
     type="l", col="green", xlab="x", ylab="Density")
lines(x, y=dGEV(x=x, 
                location=0, scale=1, shape=0), col="red")
lines(x, y=dGEV(x=x, 
                location=0, scale=1, shape=1/2), col="blue")
legend("topleft", legend=c("shape=-1/2", "shape=0", "shape=1/2"), 
       lty=1, col=c("green", "red", "blue"))

# Compute dn using the approximation from Wan et al. (2014)
   get_dn <- function(n) {
     if (n < 2) {
       stop("Sample size n must be at least 2.")
     }
     qnorm((n - 0.375) / (n + 0.25)) * 2
   }
   
   # Estimate standard deviation from min and max
   estimate_sd_from_range <- function(min_val, max_val, n) {
     dn <- get_dn(n)
     range <- max_val - min_val
     sd_estimate <- range / dn
     return(sd_estimate)
   }
   
   # Example usage:
   n <- 10
   min_val <- 5
   max_val <- 25
   
   dn_value <- get_dn(n)
   sd_estimate <- estimate_sd_from_range(min_val, max_val, n)
   
   cat("dn =", dn_value, "\n")
   cat("Estimated SD =", sd_estimate, "\n")
   
   # To generate data from publication
   
   library(parallel)
   library(embryogrowth)
   
   # Values for the prior of SD
   outSD <- subset(DatabaseTSD, subset = (((!is.na(IP.SD)) | 
                   (!is.na(IP.SE))) & (!is.na(Hatched))), 
                   select=c("Hatched", "IP.SE", "IP.SD", "IP.mean"))
   outSD$IP.SD <- ifelse(is.na(outSD$IP.SD), outSD$IP.SE*sqrt(outSD$Hatched), outSD$IP.SD)
   
   # Model estimation
   
   Example <- subset(DatabaseTSD, subset = (!is.na(IP.min)) & 
                       ((is.na(IP.SE)) & (is.na(IP.SD)) & (!is.na(Hatched)) & 
                          (is.na(IP.mean))), select=c("Species", "Incubation.temperature.set", 
                                                      "Hatched", "IP.min", "IP.max", 
                                                      "Reference"))
   
   out <- universalmclapply(X=1:nrow(Example), FUN=function(i) {
     n <- Example[i, "Hatched"]
     
     priors <- structure(list(Density = c("dunif", "dlnorm"), 
                              Prior1 = c(30, log(mean(outSD$IP.SD))), 
                              Prior2 = c(120, log(sd(outSD$IP.SD))), 
                              SDProp = c(1, 1), 
                              Min = c(30, 0.1), 
                              Max = c(120, 6), 
                              Init = c((Example[i, "IP.min"]+ Example[i, "IP.max"])/2, log(2))), 
                         row.names = c("mean", "sd"), 
                         class = c("PriorsmcmcComposite", "data.frame"))
     
     
     out_sd_mean_mcmc <- from_min_max(n=n, observed.Minimum=Example[i, "IP.min"], 
                                      observed.Maximum=Example[i, "IP.max"], 
                                      fitted.parameters=c(mean=(Example[i, "IP.min"]+ 
                                                             Example[i, "IP.max"])/2, 
                                                          sd=log(2)), 
                                      priors = priors, 
                                      D="norm**", 
                                      n.iter = 10000, n.adapt=15000, thin=30, 
                                      trace=100, adaptive = TRUE)
     
     # plot(out_sd_mean_mcmc, what = "MarkovChain", parameters = "sd")
     
     assign(paste0("out_sd_mean_mcmc_", as.character(i)), out_sd_mean_mcmc)
     save(list = paste0("out_sd_mean_mcmc_", as.character(i)), 
          file = file.path("dataOut", paste0("out_sd_mean_mcmc_", as.character(i), ".Rdata")))
     # rm(list=paste0("out_sd_mean_mcmc_", as.character(i)))
   }, progressbar = TRUE)
   
   # Generate table with all results
   
   Example <- subset(DatabaseTSD, subset = (!is.na(IP.min)) & 
   ((is.na(IP.SE)) & (is.na(IP.SD)) & (!is.na(Hatched)) & 
     (is.na(IP.mean))), select=c("Species", "Incubation.temperature.set", 
                                 "Hatched", "IP.min", "IP.max", 
                                 "Reference"))
   
   Example <- cbind(Example, dn=NA)
   Example <- cbind(Example, "SD(Hozo 2005)"=NA)
   Example <- cbind(Example, "SD(Wan 2014)"=NA)
   Example <- cbind(Example, "mean(Wan 2014)"=NA)
   Example <- cbind(Example, "median(SD)"=NA)
   Example <- cbind(Example, "2.5\%(SD)"=NA)
   Example <- cbind(Example, "97.5\%(SD)"=NA)
   Example <- cbind(Example, "25\%(SD)"=NA)
   Example <- cbind(Example, "75\%(SD)"=NA)
   Example <- cbind(Example, "median(mean)"=NA)
   Example <- cbind(Example, "2.5\%(mean)"=NA)
   Example <- cbind(Example, "97.5\%(mean)"=NA)
   Example <- cbind(Example, "25\%(mean)"=NA)
   Example <- cbind(Example, "75\%(mean)"=NA)
   Example <- cbind(Example, "z(mean)"=NA)
   Example <- cbind(Example, "z(SD)"=NA)
   
   
   library(coda)
   
   for (i in 1:nrow(Example)) {
     
     n <- Example[i, "Hatched"]
     if (n<= 15) {
       Example[i, "SD(Hozo 2005)"] <- (1/sqrt(12))*sqrt(((Example[i, "IP.max"] - 
        Example[i, "IP.min"]))^2+((Example[i, "IP.max"] - Example[i, "IP.min"]))^2/4)
     } else {
       if (n<=70) {
         Example[i, "SD(Hozo 2005)"] <- (Example[i, "IP.max"] - Example[i, "IP.min"])/4
       } else {
         Example[i, "SD(Hozo 2005)"] <- (Example[i, "IP.max"] - Example[i, "IP.min"])/6
       }
     }
     
     Example[i, "dn"] <- get_dn(n)
     Example[i, "SD(Wan 2014)"] <- estimate_sd_from_range(Example[i, "IP.min"], 
            Example[i, "IP.max"], n)
     Example[i, "mean(Wan 2014)"] <- (Example[i, "IP.min"]+ Example[i, "IP.max"])/2
     load(file = file.path("dataOut", paste0("out_sd_mean_mcmc_", as.character(i), ".Rdata")))
     out_sd_mean_mcmc <- get(paste0("out_sd_mean_mcmc_", as.character(i)))
     k <- as.parameters(out_sd_mean_mcmc, index="quantile", probs=c(0.025, 0.25, 0.5, 0.75, 0.975))
     outgk <- geweke.diag(as.mcmc(out_sd_mean_mcmc))
     rm(out_sd_mean_mcmc)
     rm(list=paste0("out_sd_mean_mcmc_", as.character(i)))
     Example[i, "median(SD)"] <- k["50\%", "sd"]
     Example[i, "2.5\%(SD)"] <- k["2.5\%", "sd"]
     Example[i, "97.5\%(SD)"] <- k["97.5\%", "sd"]
     Example[i, "25\%(SD)"] <- k["25\%", "sd"]
     Example[i, "75\%(SD)"] <- k["75\%", "sd"]
     Example[i, "median(mean)"] <- k["50\%", "mean"]
     Example[i, "2.5\%(mean)"] <- k["2.5\%", "mean"]
     Example[i, "97.5\%(mean)"] <- k["97.5\%", "mean"]
     Example[i, "25\%(mean)"] <- k["25\%", "mean"]
     Example[i, "75\%(mean)"] <- k["75\%", "mean"]
     Example[i, "z(mean)"] <- outgk$`1`$z["mean"]
     Example[i, "z(SD)"] <- outgk$`1`$z["sd"]
   }
   
   rownames(Example) <- as.character(1:nrow(Example))
  
  # Figure 1
  layout(mat = matrix(1:4, nrow=2))
  par(mar=c(4, 4, 0, 0))
  plot(out_sd_mean_mcmc_11, what = "MarkovChain", parameters = "mean", ylim=c(70, 76))
  text(x=ScalePreviousPlot(x=0.05, y=0.95)$x, y=ScalePreviousPlot(x=0.85, y=0.95)$y, 
     labels = "A: mean", cex=1.5, pos=4)
  plot(out_sd_mean_mcmc_8, what = "MarkovChain", parameters = "mean", ylim=c(50, 52))
  text(x=ScalePreviousPlot(x=0.05, y=0.95)$x, y=ScalePreviousPlot(x=0.85, y=0.95)$y, 
      labels = "C: mean", cex=1.5, pos=4)
  plot(out_sd_mean_mcmc_11, what = "MarkovChain", parameters = "sd")
  text(x=ScalePreviousPlot(x=0.05, y=0.95)$x, y=ScalePreviousPlot(x=0.85, y=0.95)$y, 
      labels = "B: sd", cex=1.5, pos=4)
  plot(out_sd_mean_mcmc_8, what = "MarkovChain", parameters = "sd")
  text(x=ScalePreviousPlot(x=0.05, y=0.95)$x, y=ScalePreviousPlot(x=0.85, y=0.95)$y, 
      labels = "D: sd", cex=1.5, pos=4)
  
  # Figure 2
  dtafigure2 <- matrix(NA, nrow=nrow(as.parameters(out_sd_mean_mcmc, index = "all")), 
                       ncol=nrow(Example))
  
  for (i in 1:nrow(Example)) {
    # i <- 1
    load(file=file.path("dataOut", paste0("out_sd_mean_mcmc_", as.character(i), ".Rdata")))
    out_sd_mean_mcmc <- get(paste0("out_sd_mean_mcmc_", as.character(i)))
    PPM <- rnorm(nrow(as.parameters(out_sd_mean_mcmc, index = "all")), 
       mean = as.parameters(out_sd_mean_mcmc, index = "all")[, "mean"], 
       sd=as.parameters(out_sd_mean_mcmc, index = "all")[, "sd"])
    dtafigure2[, i] <- PPM
  }
  
  layout(mat = 1)
  par(mar=c(3, 4, 0, 0))
  boxplot(dtafigure2, outline=FALSE, las=1, bty="n", xaxt="n", frame=FALSE, ylim=c(40, 90), 
    col=sapply(as.character(Example$Species), 
      FUN=function(i) switch(i, "Caretta caretta"="white", 
                                 "Chelonia mydas"="green", 
                                 "Dermochelys coriacea"="lightblue")), 
    ylab="Incubation duration in days")
  axis(1, at=1:30, labels = rep(NA, 30))
  Cc <- sum(as.character(Example$Species) == "Caretta caretta")
  CcCm <- sum(as.character(Example$Species) == "Caretta caretta" | 
              as.character(Example$Species) == "Chelonia mydas")
  segments(x0= Cc + 0.5, 
           x1= Cc + 0.5, 
           y0=40, y1=90, lty = 2)
  
  segments(x0= CcCm + 0.5, 
           x1= CcCm + 0.5, 
           y0=40, y1=90, lty = 2)
  
  par(xpd=TRUE)
  text(x=Cc/2, y=33, labels = expression(italic("Caretta caretta")), cex=0.9)
  
  text(x=Cc+(CcCm-Cc)/2, y=32, labels = expression(italic("Chelonia\n  mydas")), cex=0.9)
  text(x=CcCm+(30-CcCm)/2, y=32, labels = expression(italic("Dermochelys\n  coriacea")), cex=0.9)
  
  # With Lognormal
   # To generate data from publication
   
   library(parallel)
   library(embryogrowth)
   
   # Values for the prior of SD
   outSD <- subset(DatabaseTSD, subset = (((!is.na(IP.SD)) | 
                   (!is.na(IP.SE))) & (!is.na(Hatched))), 
                   select=c("Hatched", "IP.SE", "IP.SD", "IP.mean"))
   outSD$IP.SD <- ifelse(is.na(outSD$IP.SD), outSD$IP.SE*sqrt(outSD$Hatched), outSD$IP.SD)
   
   # Model estimation
   
   Example <- subset(DatabaseTSD, subset = (!is.na(IP.min)) & 
                       ((is.na(IP.SE)) & (is.na(IP.SD)) & (!is.na(Hatched)) & 
                          (is.na(IP.mean))), select=c("Species", "Incubation.temperature.set", 
                                                      "Hatched", "IP.min", "IP.max", 
                                                      "Reference"))
   
   out <- universalmclapply(X=1:nrow(Example), FUN=function(i) {
     n <- Example[i, "Hatched"]
     
     priors <- structure(list(Density = c("dunif", "dlnorm"), 
                              Prior1 = c(30, log(mean(outSD$IP.SD))), 
                              Prior2 = c(120, log(sd(outSD$IP.SD))), 
                              SDProp = c(1, 1), 
                              Min = c(30, 0.1), 
                              Max = c(120, 6), 
                              Init = c((Example[i, "IP.min"]+ Example[i, "IP.max"])/2, log(2))), 
                         row.names = c("meanlog", "sdlog"), 
                         class = c("PriorsmcmcComposite", "data.frame"))
     
     
     out_sd_mean_mcmc_LN <- from_min_max(n=n, observed.Minimum=Example[i, "IP.min"], 
                                      observed.Maximum=Example[i, "IP.max"], 
                                      fitted.parameters=c(mean=(Example[i, "IP.min"]+ 
                                                             Example[i, "IP.max"])/2, 
                                                          sd=log(2)), 
                                      priors = priors, 
                                      D="lnorm**", 
                                      n.iter = 10000, n.adapt=15000, thin=30, 
                                      trace=100, adaptive = TRUE)
     
     # plot(out_sd_mean_mcmc, what = "MarkovChain", parameters = "sd")
     
     assign(paste0("out_sd_mean_mcmc_LN_", as.character(i)), out_sd_mean_mcmc_LN)
     save(list = paste0("out_sd_mean_mcmc_LN_", as.character(i)), 
          file = file.path("dataOut", paste0("out_sd_mean_mcmc_LN_", as.character(i), ".Rdata")))
     # rm(list=paste0("out_sd_mean_mcmc_LN_", as.character(i)))
   }, progressbar = TRUE)
   
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_errbar.R
\name{plot_errbar}
\alias{plot_errbar}
\title{Plot a xy graph with error bar on x and/or y}
\usage{
plot_errbar(
  ...,
  errbar.x = NULL,
  errbar.y = NULL,
  errbar.x.plus = NULL,
  errbar.x.minus = NULL,
  errbar.y.plus = NULL,
  errbar.y.minus = NULL,
  x.plus = NULL,
  x.minus = NULL,
  y.plus = NULL,
  y.minus = NULL,
  errbar.tick = 1/50,
  errbar.lwd = par("lwd"),
  errbar.lty = par("lty"),
  errbar.col = par("fg"),
  errbar.y.polygon = FALSE,
  errbar.y.polygon.list = list(NULL),
  names = NULL,
  add = FALSE
)
}
\arguments{
\item{...}{Parameters for plot() such as main= or ylim=}

\item{errbar.x}{The length of error bars for x. Recycled if necessary.}

\item{errbar.y}{The length of error bars for y. Recycled if necessary.}

\item{errbar.x.plus}{The length of positive error bars for x. Recycled if necessary.}

\item{errbar.x.minus}{The length of negative error bars for x. Recycled if necessary.}

\item{errbar.y.plus}{The length of positive error bars for y. Recycled if necessary.}

\item{errbar.y.minus}{The length of negative error bars for y. Recycled if necessary.}

\item{x.plus}{The absolut position of the positive error bar for x. Recycled if necessary.}

\item{x.minus}{The absolut position of the negative error bar for x. Recycled if necessary.}

\item{y.plus}{The absolut position of the positive error bar for y. Recycled if necessary.}

\item{y.minus}{The absolut position of the nagative error bar for y. Recycled if necessary.}

\item{errbar.tick}{Size of small ticks at the end of error bars defined as a proportion of total width or height graph size.}

\item{errbar.lwd}{Error bar line width, see par("lwd")}

\item{errbar.lty}{Error bar line type, see par("lwd")}

\item{errbar.col}{Error bar line color, see par("col")}

\item{errbar.y.polygon}{If true, the errors are shown as a filed polygon.}

\item{errbar.y.polygon.list}{List of parameters to be used for polygon.}

\item{names}{The names of the points to be used with show_name().}

\item{add}{If true, add the graph to the previous one.}
}
\value{
A list with x, y and names for points
}
\description{
To plot data, just use it as a normal plot but add the errbar.x
and errbar.y values or errbar.x.minus, errbar.x.plus if bars for x axis are
asymetric and errbar.y.minus, errbar.y.plus if bars for y axis are
asymetric. Use x.plus, x.minus, y.plus and y.minus to set absolut limits for
error bars. Note that x.plus and x.minus have priority over errbar.x, errbar.x.minus and
errbar.x.plus and that y.plus and y.minus have priority over errbar.y, errbar.y.minus and
errbar.y.plus.\cr
The parameter errbar.y.polygon=TRUE permits to define error as an envolop for y axis.
}
\details{
plot_errbar plot a xy graph with error bar on x and/or y
}
\examples{
\dontrun{
plot_errbar(1:100, rnorm(100, 1, 2), 
	xlab="axe x", ylab="axe y", bty="n", xlim=c(1,100), 
		errbar.x=2, errbar.y=rnorm(100, 1, 0.1))
x <- 1:100
plot_errbar(x=1:100, rnorm(100, 1, 2), 
               	xlab="axe x", ylab="axe y", bty="n", xlim=c(1,100), 
            		x.minus=x-2, x.plus=x+2)
x <- 1:100
plot_errbar(x=1:100, rnorm(100, 1, 2), 
               	xlab="axe x", ylab="axe y", bty="n", 
               	pch=21, bg="white", 
            		x.minus=x-10, x.plus=x+10)
x <- (1:200)/10
y <- sin(x)
plot_errbar(x=x, y=y, xlab="axe x", ylab="axe y", bty="n", xlim=c(1,20), 
     y.minus=y-1, y.plus=y+1, ylim=c(-3, 3), type="l",  
		errbar.y.polygon=TRUE, 
		errbar.y.polygon.list=list(border=NA, col=rgb(0, 0, 0, 0.5)))
		}
}
\seealso{
\code{barplot_errorbar}

Other plot and barplot functions:
\code{\link[=ScalePreviousPlot]{ScalePreviousPlot()}},
\code{\link[=barplot_errbar]{barplot_errbar()}},
\code{\link[=plot_add]{plot_add()}},
\code{\link[=show_name]{show_name()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{plot and barplot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RM_duplicate.R
\name{RM_duplicate}
\alias{RM_duplicate}
\title{Duplicate a results managment within an object.}
\usage{
RM_duplicate(
  x = stop("An object with results managment must be provided"),
  RM = "RM",
  RMnamefrom = 1,
  RMnameto = 2
)
}
\arguments{
\item{x}{The object to duplicate a results managment}

\item{RM}{The name of results managment stored}

\item{RMnamefrom}{The name of the results managment to be duplicated}

\item{RMnameto}{The new name of the results managment}
}
\value{
The original object with a duplicated results managment.
}
\description{
RM_duplicate duplicates a results managment within an object.
}
\details{
RM_duplicate duplicates a results managment within an object
}
\examples{
\dontrun{
library("HelpersMG")
# Let an object of class objclass being created
obj <- list(A=100, name="My object")
class(obj) <- "objclass"
# And now I create a RM to this object
obj <- RM_add(x=obj, RMname="NewAnalysis1")
RM_list(obj)
obj <- RM_duplicate(x=obj, RMnamefrom="NewAnalysis1", RMnameto="NewAnalysis2")
RM_list(obj)
}
}
\seealso{
Other Results Managment:
\code{\link[=RM_add]{RM_add()}},
\code{\link[=RM_delete]{RM_delete()}},
\code{\link[=RM_get]{RM_get()}},
\code{\link[=RM_list]{RM_list()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Results Managment}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qvlmer.R
\name{qvlmer}
\alias{qvlmer}
\title{Quasi Variances for lmer Model Coefficients}
\usage{
qvlmer(object, factorname = NULL, coef.indices = NULL, dispersion = NULL, ...)
}
\arguments{
\item{object}{A object obtained using lmer from package lme4}

\item{factorname}{Either NULL, or a character vector of length 1}

\item{coef.indices}{Either NULL, or a numeric vector of length at least 3}

\item{dispersion}{An optional scalar multiplier for the covariance matrix, to cope with overdispersion for example}

\item{...}{Other arguments to pass to qvcalc.default}
}
\value{
A list of class qv.
}
\description{
Computes a set of quasi variances (and corresponding quasi standard errors)
for estimated model coefficients relating to the levels of a categorical (i.e., factor)
explanatory variable. For details of the method see Firth (2000), Firth (2003) or Firth
and de Menezes (2004). Quasi variances generalize and improve the accuracy of “floating
absolute risk” (Easton et al., 1991). This device for economical model summary was first
suggested by Ridout (1989).\cr
Modified from qvcalc.lm() of packages qvcalc by David Firth, d.firth@warwick.ac.uk
}
\details{
qvlmer is Quasi Variances for lmer Model Coefficients
}
\examples{
\dontrun{
x <- rnorm(100)
y <- rnorm(100)
G <- as.factor(sample(c("A", "B", "C", "D"), 100, replace = TRUE))
R <- as.factor(rep(1:25, 4))
library(lme4)
m <- lmer(y ~ x + G + (1 | R))
qvlmer(m, factorname="G")
}
}
\references{
Easton, D. F, Peto, J. and Babiker, A. G. A. G. (1991) Floating absolute risk: an alternative to relative risk in survival and case-control analysis avoiding an arbitrary reference group. Statistics in Medicine 10, 1025–1035.

Firth, D. (2000) Quasi-variances in Xlisp-Stat and on the web. Journal of Statistical Software 5.4, 1–13. At http://www.jstatsoft.org

Firth, D. (2003) Overcoming the reference category problem in the presentation of statistical models. Sociological Methodology 33, 1–18.

Firth, D. and de Mezezes, R. X. (2004) Quasi-variances. Biometrika 91, 65–80.

McCullagh, P. and Nelder, J. A. (1989) Generalized Linear Models. London: Chapman and Hall.

Menezes, R. X. de (1999) More useful standard errors for group and factor effects in generalized linear models. D.Phil. Thesis, Department of Statistics, University of Oxford.

Ridout, M.S. (1989). Summarizing the results of fitting generalized linear models to data from designed experiments. In: Statistical Modelling: Proceedings of GLIM89 and the 4th International Workshop on Statistical Modelling held in Trento, Italy, July 17–21, 1989 (A. Decarli et al., eds.), pp 262–269. New York: Springer.
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RM_delete.R
\name{RM_delete}
\alias{RM_delete}
\title{Delete a results managment or a result within a results managment from an object}
\usage{
RM_delete(
  x = stop("An object with results managment must be provided"),
  RM = "RM",
  RMname = stop("A name must be provided"),
  valuename = NULL
)
}
\arguments{
\item{x}{The object to delete a results managment}

\item{RM}{The name of results managment stored}

\item{RMname}{The name of the result that will be deleted or its rank}

\item{valuename}{The name of the result that will be deleted}
}
\value{
The original object with the deleted results managment
}
\description{
Return the original object with the deleted results managment or result.
}
\details{
RM_delete deletes a results managment or a result within a results managment from an object
}
\examples{
\dontrun{
library("HelpersMG")
# Let an object of class objclass being created
obj <- list(A=100, name="My object")
class(obj) <- "objclass"
# And now I create a RM to this object
obj <- RM_add(x=obj, RMname="NewAnalysis1")
obj <- RM_add(x=obj, RMname="NewAnalysis2")
RM_list(obj)
obj <- RM_delete(x=obj, RMname="NewAnalysis1")
RM_list(obj)
obj <- RM_delete(x=obj, RMname=1)
RM_list(obj)
obj <- RM_add(x=obj, RMname="NewAnalysis1", valuename="V1", value=100)
RM_list(obj)
RM_get(x=obj, RMname="NewAnalysis1", valuename="V1")
obj <- RM_add(x=obj, RMname="NewAnalysis1", valuename="V2", value=200)
RM_get(x=obj, RMname="NewAnalysis1", valuename="V2")
obj <- RM_delete(x=obj, RMname="NewAnalysis1", valuename="V1")
RM_get(x=obj, RMname="NewAnalysis1", valuename="V1")
RM_get(x=obj, RMname="NewAnalysis1", valuename="V2")
}
}
\seealso{
Other Results Managment:
\code{\link[=RM_add]{RM_add()}},
\code{\link[=RM_duplicate]{RM_duplicate()}},
\code{\link[=RM_get]{RM_get()}},
\code{\link[=RM_list]{RM_list()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Results Managment}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmnorm.R
\name{rmnorm}
\alias{rmnorm}
\title{Generate random numbers from the multivariate normal distribution}
\usage{
rmnorm(n = 1, mean = rep(0, d), varcov)
}
\arguments{
\item{n}{the number of random vectors to be generated.}

\item{mean}{a vector with means of length d.}

\item{varcov}{a variance-covariance matrix with dimentions d * d.}
}
\value{
For n > 1 rmnorm returns a matrix of n rows of random vectors, while for n = 1 rmnorm returns a named random vector.
}
\description{
rmnorm generate random numbers from a multivariate normal distribution.
}
\details{
rmnorm Generate random numbers from the multivariate normal distribution
}
\examples{
\dontrun{
#Variance-covariance matrix
varcov <- matrix(c(2.047737e-03, 3.540039e-05, 0.0075178920, 3.540039e-05,
6.122832e-07, 0.0001299661, 7.517892e-03, 1.299661e-04, 0.0276005740), ncol = 3)
#Set names
nam <- c("a", "b", "c")
dimnames(varcov) <- list(nam, nam)
#Check positive definiteness (all positive eigenvalues = positive definite)
eigen(varcov) $values
#Mean
mean <- c(1, 0.3, 0.5)
#Generate n = 1 random vector
rmnorm(n = 1, mean = mean, varcov = varcov)
#Generate n = 10 random vectors
rmnorm(n = 10, mean = mean, varcov = varcov)
#Generate n = 1 random vectors when varcov is non-positive definite
#Non-positive definite varcov matrix
varcov2 <- matrix(c(2.04e-03, 3.54e-05, 7.52e-03, 3.54e-05, 6.15e-07,
  1.30e-04, 7.52e-03, 1.30e-04, 2.76e-02), ncol = 3)
  dimnames(varcov2) <- dimnames(varcov)
eigen(varcov2)
#Random vector
rmnorm(n = 1, mean = mean, varcov = varcov2)
}
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Based on lmf package
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/universalmclapply.R
\name{universalmclapply}
\alias{universalmclapply}
\title{Run the function FUN on X using parallel computing}
\usage{
universalmclapply(
  X,
  FUN,
  ...,
  mc.cores = getOption("mc.cores", parallel::detectCores()),
  mc.preschedule = TRUE,
  clusterExport = list(),
  clusterEvalQ = list(),
  forking = ifelse(.Platform$OS.type == "windows", FALSE, TRUE),
  progressbar = FALSE
)
}
\arguments{
\item{X}{A vector (atomic or list) or an expressions vector. Other objects (including classed objects) will be coerced by as.list.}

\item{FUN}{The function to be applied to each element of X}

\item{...}{Optional arguments to FUN}

\item{mc.cores}{The number of cores to use, i.e. at most how many child processes will be run simultaneously.}

\item{mc.preschedule}{if set to TRUE then the computation is first divided to (at most) as many jobs are there are cores and then the jobs are started, each job possibly covering more than one value. If set to FALSE then one job is forked for each value of X. The former is better for short computations or large number of values in X, the latter is better for jobs that have high variance of completion time and not too many values of X compared to mc.cores.}

\item{clusterExport}{List of clusterExport parameters as list}

\item{clusterEvalQ}{List of clusterEvalQ parameters as list}

\item{forking}{If TRUE will use forking}

\item{progressbar}{If pbapply package is installed, show a progressbar}
}
\value{
The results of the function FUN applied to X
}
\description{
Return the results of the function FUN applied to X. It uses forking in unix system and not in windows system.\cr
By default, it will send all the content of environment.
}
\details{
universalmclapply runs the function FUN on X using parallel computing
}
\examples{
\dontrun{
library(HelpersMG)
x <- 1:1000
funx <- function(y) {
  mint <- rep(NA, length(y))
  for (i in seq_along(y)) {
    k <- rnorm(runif(n = 1, 50, 50), mean=10, sd=2)
    mint[i] <- mean(k)
  }
  mint
}
# Note that parallel computing is not always the best solution !
(tp <- system.time({
   m <- lapply(X=x, FUN=funx)
}))
(tp <- system.time({
   m <- universalmclapply(X=x, FUN=funx, forking=FALSE)
}))
(tp <- system.time({
   m <- universalmclapply(X=x, FUN=funx, forking=TRUE)
}))

### An example using clusterExport
# Here no error is generated because environment was exported
# However forking is not possible in windows and non parallel code is ran
pp <- runif(100)
x <- 1:100
funx1 <- function(y) {pp[y]*10}
u <- universalmclapply(x, FUN=funx1, forking=TRUE)

# Here an error is generated because environment was not exported when parLapplyLB is used
pp <- runif(100)
x <- 1:100
u <- universalmclapply(x, FUN=funx1, forking=FALSE)
u <- universalmclapply(x, FUN=funx1, forking=FALSE, 
                       clusterExport=list())

# here no error is generated because the variable pp is exported
pp <- runif(100)
x <- 1:100
u <- universalmclapply(x, FUN=funx1, forking=FALSE, 
                       clusterExport=list(varlist=c("pp"), envir=environment()))

# here no error is generated because all the environment is exported
pp <- runif(100)
x <- 1:100
u <- universalmclapply(x, FUN=funx1, forking=FALSE, 
                       clusterExport=list(varlist=c(ls()), envir=environment()))

### An example using clusterEvalQ
asc("a") # asc() is a function from packages HelpersMG
funx2 <- function(y) {asc("a")*10}
# In unix, the loaded packages are visible from all cores
x <- 1:100
u <- universalmclapply(x, FUN=funx2, forking=TRUE)
# In windows, the loaded packages are not visible from all cores
x <- 1:100
u <- universalmclapply(x, FUN=funx2, forking=FALSE)
# In windows, the loaded packages are not visible from all cores
x <- 1:100
u <- universalmclapply(x, FUN=funx2, forking=FALSE, 
clusterEvalQ=list(expr=expression(library(HelpersMG)))
)

### If package pbapply is available, progress bar can be shown
m <- universalmclapply(X=x, FUN=funx, forking=FALSE, progressbar=TRUE)
m <- universalmclapply(X=x, FUN=funx, forking=TRUE, progressbar=TRUE)

## You can manage the number of cores used using:
options(mc.cores=1)
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wget.R
\name{wget}
\alias{wget}
\title{Download a file from internet and save it locally}
\usage{
wget(url = stop("At least one internet adress is required"), ...)
}
\arguments{
\item{url}{The url where to download file}

\item{...}{The parameters send to download.file()}
}
\value{
Nothing
}
\description{
Download a file from internet and save it locally. This function is a wrapper for
download.files() that keep the name identical and can get several files at once. It was written to simplify
downloading of file. It doest not use the true wget function (https://www.gnu.org/software/wget/) which is
much more complex but also powerful.
}
\details{
wget download a file from internet and save it locally
}
\examples{
\dontrun{
library(HelpersMG)
# Save locally the files send in the parameter url
wget(c("https://cran.r-project.org/web/packages/HelpersMG/HelpersMG.pdf", 
         "https://cran.r-project.org/web/packages/embryogrowth/embryogrowth.pdf"))
}
}
\author{
Marc Girondot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ind_long_lat.R
\name{ind_long_lat}
\alias{ind_long_lat}
\title{Return or the index in ncdf object from lat/longitude or inverse}
\usage{
ind_long_lat(
  ncdf = stop("The ncdf data must be supplied"),
  long = NULL,
  lat = NULL,
  indice.long = NULL,
  indice.lat = NULL,
  label.longitude = "lon",
  label.latitude = "lat"
)
}
\arguments{
\item{ncdf}{An object read from package ncdf4, ncdf or RNetCDF}

\item{long}{Longitude in decimal format}

\item{lat}{Latitude in decimal format}

\item{indice.long}{Index of longitude}

\item{indice.lat}{Index of latitude}

\item{label.longitude}{Name of argument for longitude, default is lon}

\item{label.latitude}{Name of argument for latitude, default is lat}
}
\value{
Or the index in ncdf object from lat/longitude or inverse
}
\description{
Return or the index in ncdf object from lat/longitude or reverse.
}
\details{
ind_long_lat is used to manage ncdf information
}
\examples{
\dontrun{
url <- "https://downloads.psl.noaa.gov/Datasets/noaa.oisst.v2.highres/"
url <- paste0(url, "sst.day.mean.2012.v2.nc")
dest <- paste(Sys.getenv("HOME"), "/sst.day.mean.2012.v2.nc", sep="")
download.file(url, dest)
library("ncdf4")
dta2012 <- nc_open(dest)
indices <- ind_long_lat(ncdf=dta2012, lat=5.89, long=-20.56)
coordinates <- ind_long_lat(ncdf=dta2012, indice.lat=20, indice.long=30)
# library("RNetCDF")
# dta2012 <- open.nc(dest)
# indices <- ind_long_lat(ncdf=dta2012, lat=5.89, long=-20.56)
# coordinates <- ind_long_lat(ncdf=dta2012, indice.lat=20, indice.long=30)
# ncdf library is depreciated in CRAN
# library("ncdf")
# dta2012 <- open.ncdf(dest)
# indices <- ind_long_lat(ncdf=dta2012, lat=5.89, long=-20.56)
# coordinates <- ind_long_lat(ncdf=dta2012, indice.lat=20, indice.long=30)
}
}
\seealso{
Other ncdf:
\code{\link[=format_ncdf]{format_ncdf()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{ncdf}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/openwd.R
\name{openwd}
\alias{openwd}
\title{Open a finder window with current working directory in MacOS X and windows}
\usage{
openwd(directory = getwd())
}
\arguments{
\item{directory}{The directory you want to open}
}
\value{
A vector with the list of files.
}
\description{
This function opens a finder window with directory files
in MacOS X. It has not been fully tested in Windows. In linux, it just returns the
list of files in directory.\cr
By defaut, it uses the current working directory.
}
\details{
openwd will open a finder window with current working directory
}
\examples{
\dontrun{
openwd()
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contingencyTable.compare.R
\name{contingencyTable.compare}
\alias{contingencyTable.compare}
\title{Contingency table comparison using Akaike weight}
\usage{
contingencyTable.compare(
  table,
  criterion = c("AIC", "AICc", "BIC"),
  probs = NULL
)
}
\arguments{
\item{table}{A matrix or a data.frame with series in rows and number of each category in column}

\item{criterion}{Which criterion is used for model selection}

\item{probs}{Series of probabilities used for conformity comparison}
}
\value{
The probability that a single proportion model is sufficient to explain the data
}
\description{
This function is used as a replacement of chisq.test() to not use p-value.
}
\details{
contingencyTable.compare compares contingency table using Akaike weight.
}
\examples{
\dontrun{
library("HelpersMG")

# Symmetry of Lepidochelys olivacea scutes
table <- t(data.frame(SriLanka=c(200, 157), AfricaAtl=c(19, 12), 
                      Guyana=c(8, 6), Suriname=c(162, 88), 
                      MexicoPac1984=c(42, 34), MexicoPac2014Dead=c(8, 9),
                      MexicoPac2014Alive=c(13, 12), 
                      row.names =c("Symmetric", "Asymmetric")))
table
contingencyTable.compare(table)

table <- t(data.frame(SriLanka=c(200, 157), AfricaAtl=c(19, 12), Guyana=c(8, 6),
                      Suriname=c(162, 88), MexicoPac1984=c(42, 34), 
                      MexicoPac2014Dead=c(8, 9),
                      MexicoPac2014Alive=c(13, 12), Lepidochelys.kempii=c(99, 1), 
                      row.names =c("Symmetric", "Asymmetric")))
table
contingencyTable.compare(table)

# Conformity to a model
table <- matrix(c(33, 12, 25, 75), ncol = 2, byrow = TRUE)
probs <- c(0.5, 0.5)
contingencyTable.compare(table, probs=probs)

# Conformity to a model
table <- matrix(c(33, 12), ncol = 2, byrow = TRUE)
probs <- c(0.5, 0.5)
contingencyTable.compare(table, probs=probs)

# Conformity to a model
table <- matrix(c(33, 12, 8, 25, 75, 9), ncol = 3, byrow = TRUE)
probs <- c(0.8, 0.1, 0.1)
contingencyTable.compare(table, probs=probs)

# Comparison of chisq.test() and this function
table <- matrix(c(NA, NA, 25, 75), ncol = 2, byrow = TRUE)

pv <- NULL
aw <- NULL
par(new=FALSE)
n <- 100

for (GroupA in 0:n) {
  table[1, 1] <- GroupA
  table[1, 2] <- n-GroupA
  pv <- c(pv, chisq.test(table)$p.value)
  aw <- c(aw, contingencyTable.compare(table, criterion="BIC")[1])
}

x <- 0:n
y <- pv
y2 <- aw
plot(x=x, y=y, type="l", bty="n", las=1, xlab="Number of type P in Group B", ylab="Probability", 
     main="", lwd=2)
lines(x=x, y=y2, type="l", col="red", lwd=2)

# w-value
(l1 <- x[which(aw>0.05)[1]])
(l2 <- rev(x)[which(rev(aw)>0.05)[1]])

aw[l1]
pv[l1]

aw[l2+2]
pv[l2+2]

# p-value
l1 <- which(pv>0.05)[1]
l2 <- max(which(pv>0.05))

aw[l1]
pv[l1]

aw[l2]
pv[l2]

y[which(y2>0.05)[1]]
y[which(rev(y2)>0.05)[1]]

par(xpd=TRUE)
text(x=25, y=1.15, labels="Group A: 25 type P / 100", pos=1)

segments(x0=25, y0=0, x1=25, y1=1, lty=3)

# plot(1, 1)

v1 <- c(expression(italic("p")*"-value"), expression("after "*chi^2*"-test"))
v2 <- c(expression(italic("w")*"-value for A"), expression("and B identical models"))
legend("topright", legend=c(v1, v2), 
       y.intersp = 1, 
       col=c("black", "black", "red", "red"), bty="n", lty=c(1, 0, 1, 0))

segments(x0=0, x1=n, y0=0.05, y1=0.05, lty=2)
text(x=101, y=0.05, labels = "0.05", pos=4)
}
}
\references{
Girondot, M., Guillon, J.-M., 2018. The w-value: An alternative to t- and X2 tests. Journal of Biostatistics & Biometrics 1, 1-4.
}
\seealso{
Other w-value functions:
\code{\link[=compare]{compare()}},
\code{\link[=series.compare]{series.compare()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{w-value functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MovingWindow.R
\name{MovingWindow}
\alias{MovingWindow}
\title{Return a moving average of a vector.}
\usage{
MovingWindow(x, window, hole = "begin", fill = TRUE, FUN = mean)
}
\arguments{
\item{x}{The vector to analyze}

\item{window}{The window size}

\item{hole}{Should the returned vector have the same length than x}

\item{fill}{TRUE or FALSE, should the vector return NA}

\item{FUN}{Function to apply to the window}
}
\value{
A vector
}
\description{
Return a moving average of a vector.\cr
hole parameter can be "none", "bothL", "bothR", "both", "begin", or "end".
}
\details{
MovingWindow returns a moving average of a vector.
}
\examples{
MovingWindow(1:10, window = 4, fill = TRUE, hole="bothL")
MovingWindow(1:10, window = 4, fill = TRUE, hole="bothR")
MovingWindow(1:10, window = 4, fill = TRUE, hole="both")
MovingWindow(1:10, window = 4, fill = TRUE, hole="none")
MovingWindow(1:10, window = 4, fill = TRUE, hole="begin")
MovingWindow(1:10, window = 4, fill = TRUE, hole="end")
MovingWindow(1:10, window = 4, fill = TRUE, hole="end", FUN=sd)
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitdistrquantiles.R
\name{fitdistrquantiles}
\alias{fitdistrquantiles}
\title{Parameters of beta, normal or gamma distribution based on quantiles.}
\usage{
fitdistrquantiles(
  quantiles = stop("At least two quantiles must be provided"),
  probs = NULL,
  scaled = FALSE,
  distribution = "beta"
)
}
\arguments{
\item{quantiles}{Vector of quantiles.}

\item{probs}{Numeric vector of probabilities with values in [0,1].}

\item{scaled}{Used scaled least-square.}

\item{distribution}{Distribution to be fitted: beta, normal, or gamma.}
}
\value{
Parameters of beta, normal or gamma distribution based on quantiles.
}
\description{
Return the parameters of beta or gamm that fits the best the
quantiles. The vector of probabilities can be obtained from names of
quantiles.
}
\details{
fitdistrquantiles returns the parameters of beta, normal or gamma distribution
}
\examples{
\dontrun{
rd <- rbeta(100000, shape1 = 0.7, shape2 = 6.2, ncp=0)
(q <- quantile(rd, probs=c(0.025, 0.5, 0.975)))

(best <- fitdistrquantiles(quantiles = q, probs = c(0.025, 0.5, 0.975), 
                           scaled=FALSE, distribution = "beta"))
rd10000 <- rbeta(10000, shape1 = best["shape1"], shape2 = best["shape2"], ncp=best["ncp"])
quantile(rd10000, probs=c(0.025, 0.5, 0.975))

# Here the probabilities are obtained from names of quantiles
(best <- fitdistrquantiles(quantiles = q, scaled=FALSE, distribution = "beta"))
rd10000 <- rbeta(10000, shape1 = best["shape1"], shape2 = best["shape2"], ncp=best["ncp"])
quantile(rd10000, probs=c(0.025, 0.5, 0.975))

# If only two quantiles are provided, ncp cannot be fitted
(q2 <- quantile(rd, probs=c(0.025, 0.975)))
(best <- fitdistrquantiles(quantiles = q2, scaled=FALSE, distribution = "beta"))
rd10000 <- rbeta(10000, shape1 = best["shape1"], shape2 = best["shape2"])
quantile(rd10000, probs=c(0.025, 0.975))
x <- seq(from=0.00, to=1, by=0.001)
plot(x=x, y=pbeta(x, shape1 = best["shape1"], shape2 = best["shape2"]), 
     las=1, bty="n", type="l", ylim=c(0, 1))
segments(x0=q2[1], x1=q2[1], y0=0, y1=1, lty=2)
segments(x0=q2[2], x1=q2[2], y0=0, y1=1, lty=2)

(best <- fitdistrquantiles(quantiles = q, probs = c(0.025, 0.5, 0.975), 
                           scaled=FALSE, distribution = "gamma"))
rd10000 <- rgamma(10000, shape = best["shape"], scale = best["scale"])
quantile(rd10000, probs=c(0.025, 0.5, 0.975))

(best <- fitdistrquantiles(quantiles = c(10, 20, 30), probs = c(0.025, 0.5, 0.975), 
                           scaled=FALSE, distribution = "normal"))
rd10000 <- rnorm(10000, mean = best["mean"], sd = best["sd"])
quantile(rd10000, probs=c(0.025, 0.5, 0.975))
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cArrows.R
\name{cArrows}
\alias{cArrows}
\title{Draw curved lines with arrowhead}
\usage{
cArrows(
  x1,
  y1,
  x2,
  y2,
  code = 2,
  size = 1,
  width = 1.2/4/cin,
  open = TRUE,
  sh.adj = 0.1,
  sh.lwd = 1,
  sh.col = par("fg"),
  sh.lty = 1,
  h.col = sh.col,
  h.col.bo = sh.col,
  h.lwd = sh.lwd,
  h.lty = sh.lty,
  curved = FALSE,
  beautiful.arrow = 2/3
)
}
\arguments{
\item{x1}{coordinates of points from which to draw.}

\item{y1}{coordinates of points from which to draw.}

\item{x2}{coordinates of points to which to draw.}

\item{y2}{coordinates of points to which to draw.}

\item{code}{integer code (1, 2, or 3), determining kind of arrows to be drawn.}

\item{size}{size of the arrowhead.}

\item{width}{width of the arrowhead.}

\item{open}{shape of the arrowhead.}

\item{sh.adj}{Shift the beginning of the line.}

\item{sh.lwd}{width of the line.}

\item{sh.col}{color of the line.}

\item{sh.lty}{type of line.}

\item{h.col}{color of the arrowhead.}

\item{h.col.bo}{color of the arrowhead border.}

\item{h.lwd}{width of the arrowhead.}

\item{h.lty}{type of line for the arrowhead.}

\item{curved}{0 is a straigth line, positive of negative value make the line curved.}

\item{beautiful.arrow}{if open is false, make the arrowhead more beautiful.}
}
\value{
A list wit lab.x and lab.y being the position where to draw label
}
\description{
Draw a curved line with arrowhead.
}
\details{
cArrows draws curved lines with arrowhead
}
\examples{
plot(c(1, 10), c(1, 10), type="n", bty="n")
cArrows(x1=2, y1=2, x2=6, y2=6, curved=1)
cArrows(x1=2, y1=2, x2=6, y2=6, curved=0)
cArrows(x1=2, y1=2, x2=6, y2=6, curved=1, sh.adj=1)
cArrows(x1=2, y1=2, x2=6, y2=6, curved=-1, open=FALSE)
cArrows(x1=9, y1=2, x2=6, y2=6, curved=-1, open=FALSE, sh.col="red")
cArrows(x1=9, y1=9, x2=6, y2=6, curved=-1, open=FALSE, h.col="red")
cArrows(x1=2, y1=9, x2=6, y2=6, curved=1, open=FALSE, h.col="red", h.col.bo="red")
}
\author{
Modified from iGraph
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inside.R
\name{inside}
\alias{inside}
\title{Search a string within files of a folder}
\usage{
inside(
  text = stop("A text to be searched for is necessary"),
  path = ".",
  pattern = "*\\\\.R$",
  showallfilenames = FALSE,
  ...,
  fixed = TRUE,
  ignore.case = FALSE
)
}
\arguments{
\item{text}{Text to search in files}

\item{path}{Path of the folder to search in}

\item{pattern}{Pattern for file names to search in}

\item{showallfilenames}{logical. Show all the filenames search for in}

\item{...}{Options for readLines(), example warn = FALSE}

\item{fixed}{logical. If TRUE, pattern is a string to be matched as is. Overrides all conflicting arguments (see gsub)}

\item{ignore.case}{logical. if FALSE, the pattern matching for text is case sensitive and if TRUE, case is ignored during matching.}
}
\value{
Return an invisible vector with filenames in which the pattern occurs
}
\description{
Search for a string inside the files of a folder and return where the string is found.\cr
The pattern for files that must be included uses regex for filtering.\cr
}
\details{
inside Search a string within files of a folder
}
\examples{
\dontrun{
library(HelpersMG)
# Search for files in path with names based on pattern that have the string search inside.
inside("embryogrowth", path=".", pattern="*\\\\.R$")
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NagelkerkeScaledR2.R
\name{NagelkerkeScaledR2}
\alias{NagelkerkeScaledR2}
\title{Return the scaled R2 defined by Nagelkerke (1991)}
\usage{
NagelkerkeScaledR2(x, size, prediction, scaled = TRUE)
}
\arguments{
\item{x}{The number of observations}

\item{size}{Number of trials}

\item{prediction}{Prediction of x/size}

\item{scaled}{If TRUE, return the scaled R2}
}
\value{
The scaled R2 value
}
\description{
Return the scaled R2 of a binomial model based on:\cr
Nagelkerke NJD (1991) A note on a general definition of the coefficient
of determination. Biometrika 78:691-192.\cr
This definition of scaled R2 by Nagelkerke (1991) has the following properties:\cr
(i) It is consistent with classical R2, that is the general definition applied to e.g. linear regression yields the classical R2.\cr
(ii) It is consistent with maximum likelihood as an estimation method, i.e. the maximum likelihood estimates of the model parameters maximize R2.\cr
(iii) It is asymptotically independent of the sample size n.\cr
(iv) 1-R2 has the interpretation of the proportion of unexplained 'variation'.\cr
(v) It is dimensionless, i.e. it does not depend on the units used.\cr
The reported value is similar to the value estimated with nagelkerke() function from rcompanion package but not
from the NagelkerkeR2() function from fmsb package. I don't know why.
}
\details{
NagelkerkeScaledR2 returns the scaled R2 defined by Nagelkerke (1991)
}
\examples{
x <- c(10, 9, 6, 4, 3, 1, 0)
size <- c(10, 10, 10, 10, 10, 10, 10)
prediction <- c(0.9, 0.8, 0.7, 0.5, 0.4, 0.3, 0.2)
NagelkerkeScaledR2(x, size, prediction)

# Using the example in fmsb::NagelkerkeR2
res <- glm(cbind(ncases,ncontrols) ~ agegp+alcgp+tobgp, data=esoph, family=binomial())
NagelkerkeScaledR2(x=esoph$ncases, size = esoph$ncases+esoph$ncontrols, 
                   prediction = res$fitted.values)
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Statistics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/invlogit.R
\name{invlogit}
\alias{invlogit}
\title{Return the inverse logit}
\usage{
invlogit(n)
}
\arguments{
\item{n}{The value to inverse to get the probability}
}
\value{
A value
}
\description{
Return the inverse logit.
}
\details{
invlogit returns the inverse logit
}
\examples{
n <- logit(0.5)
invlogit(n)
}
\seealso{
Other logit:
\code{\link[=flexit]{flexit()}},
\code{\link[=logit]{logit()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{logit}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moon.info.R
\name{moon.info}
\alias{moon.info}
\title{Moon phase based on a date}
\usage{
moon.info(date = Sys.Date(), phase = FALSE)
}
\arguments{
\item{date}{A date in class Date. By default, it will use today date}

\item{phase}{If TRUE, a vector of characters with NM, FQ, FL LQ will be returned}
}
\value{
Return a value describing the moon phase:\cr
0 and 100 are full moon, 50 is new moon, 25 last quarter and 75 first quater
}
\description{
The script gives an index (base 100) that represents moon phase.\cr
If the return value (from 0 to 100) is between:\cr
0 and 1.6931595 or 98.3068405 and 100, it is full moon,\cr
23.3068405 and 26.6931595, last quarter,\cr
48.3068405 and 51.6931595, new moon,\cr
73.3068405 and 76.6931595, first quarter\cr
When phase is set to TRUE, a character representing the moon phase is returned.
}
\details{
moon.info calculates the moon phase based on a date.
}
\examples{
\dontrun{
library("HelpersMG")
moon.info(as.Date("2001-12-31"))
moon.info(as.Date("14/04/2010", "\%d/\%m/\%Y"))
moon.info(as.Date("22/06/07", "\%d/\%m/\%y"))
moon.info(seq(from=as.Date("2012-03-01"), 
	to=as.Date("2012-04-15"), by="days"))
moon.info(seq(from=as.Date("2012-03-01"), 
		to=as.Date("2012-04-15"), by="days"), phase=TRUE)
}
}
\seealso{
Other Periodic patterns of indices:
\code{\link[=index.periodic]{index.periodic()}},
\code{\link[=minmax.periodic]{minmax.periodic()}},
\code{\link[=sun.info]{sun.info()}},
\code{\link[=tide.info]{tide.info()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Periodic patterns of indices}
\keyword{Lunar}
\keyword{Lune}
\keyword{Moon}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addS3Class.R
\name{addS3Class}
\alias{addS3Class}
\title{Add a S3 class to an object.}
\usage{
addS3Class(x, class = NULL)
}
\arguments{
\item{x}{The object to add class.}

\item{class}{The class to add.}
}
\value{
The same object with the new class as first class
}
\description{
Add a S3 class as first class to an object.
}
\details{
addS3Class add a S3 class to an object
}
\examples{
print.CF <- function(x) {cat("print.CF ", x)}
result <- "Je suis donc je pense"
result <- addS3Class(result, class="CF")
class(result)
print(result)
result <- addS3Class(result, class=c("ECF", "OCF"))
class(result)
print(result)
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tide.info.R
\name{tide.info}
\alias{tide.info}
\title{Annual tide calendar for one particular location}
\usage{
tide.info(
  location = NULL,
  year = 2021,
  longitude = NULL,
  latitude = NULL,
  force.tide.height = TRUE
)
}
\arguments{
\item{location}{Textual information about location name}

\item{year}{Year to get the calendar}

\item{longitude}{Longitude to search for}

\item{latitude}{Latitude to search for}

\item{force.tide.height}{If FALSE, can return a current speed rather than tide height}
}
\value{
Return a data.frame with annual tide calendar.
}
\description{
Annual tide information.\cr
The columns are: Location, Longitude, Latitude, Phase, DateTime.local, DateTime.UTC, Tide.meter\cr
This function uses an API linking xtide software
(https://flaterco.com/xtide/) with tide.info() function.\cr
You must have a working internet connection for this function.
}
\details{
tide.info gets the annual tide calendar for one particular location.
}
\examples{
\dontrun{
library("HelpersMG")
Location <- "Les Hattes"
Year <- 2010
tide <- tide.info(Location, Year)
plot(tide[, "DateTime.local"], tide[, "Tide.meter"], 
     type="l", bty="n", las=1, 
     main=tide[1, "Location"], 
     xlab=as.character(Year), ylab="Tide level in meter")

Location <- "Hawaii"
Year <- 2010
tide <- tide.info(Location, Year)

Location <- "Hanamaulu Bay, Kauai Island, Hawaii"
Year <- 2010
tide <- tide.info(Location, Year)
plot(tide[, "DateTime.local"], tide[, "Tide.meter"], 
     type="l", bty="n", las=1, 
     main=tide[1, "Location"], 
     xlab=as.character(Year), ylab="Tide level in meter")
     
tide <- tide.info(year=2010, longitude=-32, latitude=-4)
library(maps)
map(database = "world", regions = "Brazil", asp=1, 
    xlim=c(-80, -30), ylim=c(-33, 5))
points(tide[1, "Longitude"], tide[1, "Latitude"], col="red", pch=19)
points(-32, -4, col="blue", pch=19)
axis(1)
axis(2, las=1)

# Show the locations with data    
library(maps)
map(xlim=c(-180, 180), ylim=c(-90, 90))
title("Locations with harmonics data")
axis(1, at=seq(from=-180, to=180, by=45))
axis(2, las=1, at=seq(from=-90, to=90, by=15))
points(getFromNamespace(x="tide_location", ns="HelpersMG")[, c("longitude")], 
       getFromNamespace(x="tide_location", ns="HelpersMG")[, c("latitude")], 
       pch=".", col="red", cex=2)
# Another example
tikei_lon  <- (-144.5465183)
tikei_lat <- -14.9505897
Year <- 2021
tikei_tide <- tide.info(year=Year, longitude=tikei_lon, latitude=tikei_lat)
plot(tikei_tide[, "DateTime.local"], tikei_tide[, "Tide.meter"], 
     type="l", bty="n", las=1, 
     main=tikei_tide[1, "Location"], 
     xlab=as.character(Year), ylab="Tide level in meter")
## Another one
tikei_lon <- (-75.56861111)
tikei_lat <- 39.50083333
Year <- 2012
tikei_tide <- tide.info(year=Year, longitude=tikei_lon, latitude=tikei_lat)

library(mapdata)
map('worldHires', xlim=c(-77, -74), ylim=c(37, 40))
points(x=tikei_lon, y=tikei_lat, pch=19, col="red", cex=1)
points(x=tikei_tide$Longitude[1], y=tikei_tide$Latitude[2], 
       pch=19, col="blue", cex=1)

par(mar=c(4, 4, 2, 2))
plot(tikei_tide$DateTime.local, tikei_tide$Tide.meter, type="l")
}
}
\seealso{
Other Periodic patterns of indices:
\code{\link[=index.periodic]{index.periodic()}},
\code{\link[=minmax.periodic]{minmax.periodic()}},
\code{\link[=moon.info]{moon.info()}},
\code{\link[=sun.info]{sun.info()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Periodic patterns of indices}
\keyword{Tide}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_add.R
\name{plot_add}
\alias{plot_add}
\title{Add a plot to a previous one}
\usage{
plot_add(...)
}
\arguments{
\item{...}{Parameters for plot()}
}
\value{
Nothing
}
\description{
To plot data, just add use it as a normal plot. It will plot
the new data without axes, or labels for axes.\cr
This function is complementary to matlines() and matpoints() from package graphics.
}
\details{
plot_add adds a plot to a previous one
}
\examples{
\dontrun{
plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="x", ylab="y")
plot_add(x=1:200, y=cos(1:200), type="l", bty="n", col="red")
}
}
\seealso{
Other plot and barplot functions:
\code{\link[=ScalePreviousPlot]{ScalePreviousPlot()}},
\code{\link[=barplot_errbar]{barplot_errbar()}},
\code{\link[=plot_errbar]{plot_errbar()}},
\code{\link[=show_name]{show_name()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{plot and barplot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SEfromHessian.R
\name{SEfromHessian}
\alias{SEfromHessian}
\title{Standard error of parameters based on Hessian matrix}
\usage{
SEfromHessian(a, hessian = FALSE, silent = FALSE)
}
\arguments{
\item{a}{An Hessian matrix}

\item{hessian}{If TRUE, return a list with the hessian and SE}

\item{silent}{If TRUE, report some problems}
}
\value{
SEfromHessian returns a vector with standard errors
}
\description{
Standard error of parameters based on Hessian matrix.\cr
The strategy is as follow:\cr
First it tries to inverse the Hessian matrix. If it fails, it uses the near
positive definite matrix of the Hessian.\cr
So now the inverse of the Hessian matrix can be computed.\cr
The diagonal of the inverse of the Hessian matrix is calculated. If all values are positive,
the SEs are the square root of the inverse of the Hessian.\cr
If not all values are positive, it will estimate the pseudo-variance
matrix based on Gill & King (2004). It necessitates a Cholesky matrix.\cr
If from some reason it fails (for example all SE are 0 in output), then the strategy of
Rebonato and Jackel (2000) will be used to generate the Cholesky matrix.
}
\details{
SEfromHessian returns standard error of parameters based on Hessian matrix
}
\examples{
\dontrun{
val=rnorm(100, mean=20, sd=5)
# Return -ln L of values in val in Gaussian distribution with mean and sd in par
fitnorm<-function(par, val) {
  -sum(dnorm(val, par["mean"], par["sd"], log = TRUE))
}
# Initial values for search
p<-c(mean=20, sd=5)
# fit the model
result <- optim(par=p, fn=fitnorm, val=val, method="BFGS", hessian=TRUE)
SE <- SEfromHessian(result$hessian)
library(MASS)
fitdistr(val, densfun = "normal")
}
}
\references{
Gill J. and G. King 2004. What to do when your Hessian is not invertible:
Alternatives to model respecification in nonlinear estimation. Sociological Methods
& Research 33: 54-87.

Rebonato and Jackel, “The most general methodology for creating a valid
correlation matrix for risk management and option pricing purposes”, Journal of Risk,
Vol 2, No 2, 2000.
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PPD.R
\name{PPD}
\alias{PPD}
\title{Posterior Predictive Distribution from mcmcComposite object}
\usage{
PPD(
  x,
  chain = "all",
  FUN = NULL,
  probs = c(0.025, 0.975),
  xlim = NULL,
  nameparxlim = NULL,
  namepar = NULL
)
}
\arguments{
\item{x}{A mcmcComposite obtained as a result of \code{MHalgoGen()} function}

\item{chain}{The number of the chain in which to get parameters or "all"}

\item{FUN}{The function to apply the parameters. It must be text.}

\item{probs}{The probability to get quantiles}

\item{xlim}{The values to apply in fun if required}

\item{nameparxlim}{The name of the parameter for xlim}

\item{namepar}{The name of parameters from mcmc object to be used in fun}
}
\value{
A data.frame with quantiles
}
\description{
Extract quantile distributions from mcmcComposite object. If FUN is used, it keeps the
correlations between parameters.
}
\examples{
\dontrun{
library(HelpersMG)
require(coda)
x <- rnorm(30, 10, 2)
dnormx <- function(data, x) {
 data <- unlist(data)
 return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
}

parameters_mcmc <- setPriors1(Name = "mean", 
                              Density = "dnorm", Prior1 = 10, Prior2 = 2, 
                              SDProp = 1, Min = -3, Max = 100, Init = 10)
parameters_mcmc <- parameters_mcmc + setPriors1(Name = "sd", 
                              Density = "dlnorm", Prior1 = 0.5, Prior2 = 0.5, 
                              SDProp = 1, Min = 0, Max = 10, Init = 2)
                              
mcmc_run <- MHalgoGen(n.iter=10000, parameters=parameters_mcmc, data=x, 
                     likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)

# Return the 2.5\% and 97.5\% quantiles of the mean parameter
k <- PPD(x=mcmc_run, namepar="mean")
k <- PPD(x=mcmc_run, namepar=NULL)
k <- PPD(x=mcmc_run, namepar=c("mean", "sd"))

# Here I return the distribution of coefficient of variation as a posterior predictive distribution
k <- PPD(x=mcmc_run            , 
                 FUN="sd/mean" )

# I can generate an output based on a x variable 
k <- PPD(x=mcmc_run                   , 
                  xlim=c(1, 2)        ,
                  nameparxlim="mult"  ,
                  FUN="mult*mean"     )
}
}
\seealso{
Other mcmcComposite functions:
\code{\link[=+.PriorsmcmcComposite]{+.PriorsmcmcComposite()}},
\code{\link[=MHalgoGen]{MHalgoGen()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=as.parameters]{as.parameters()}},
\code{\link[=merge.mcmcComposite]{merge.mcmcComposite()}},
\code{\link[=plot.PriorsmcmcComposite]{plot.PriorsmcmcComposite()}},
\code{\link[=plot.mcmcComposite]{plot.mcmcComposite()}},
\code{\link[=setPriors]{setPriors()}},
\code{\link[=setPriors1]{setPriors1()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.quantiles.R
\name{as.quantiles}
\alias{as.quantiles}
\title{Extract quantile distribution from mcmcComposite object}
\usage{
as.quantiles(
  x,
  chain = "all",
  fun = function(...) return(as.numeric(list(...))),
  probs = c(0.025, 0.975),
  xlim = NULL,
  nameparxlim = NULL,
  namepar = NULL
)
}
\arguments{
\item{x}{A mcmcComposite obtained as a result of \code{MHalgoGen()} function}

\item{chain}{The number of the chain in which to get parameters or "all"}

\item{fun}{The function to apply the parameters}

\item{probs}{The probability to get quantiles}

\item{xlim}{The values to apply in fun}

\item{nameparxlim}{The name of the parameter for xlim}

\item{namepar}{The name of parameters from mcmc object to be used in fun}
}
\value{
A data.frame with quantiles
}
\description{
Extract quantile distribution from mcmcComposite object
}
\examples{
\dontrun{
library(HelpersMG)
require(coda)
x <- rnorm(30, 10, 2)
dnormx <- function(data, x) {
 data <- unlist(data)
 return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
}
parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(1, 1), 
Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
row.names=c('mean', 'sd'))
mcmc_run <- MHalgoGen(n.iter=10000, parameters=parameters_mcmc, data=x, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
k <- as.quantiles(x=mcmc_run, namepar="mean")
k <- as.quantiles(x=mcmc_run, namepar="mean", 
                 xlim=c(1:5), nameparxlim="sd", 
                 fun=function(...) return(sum(as.numeric(list(...)))))
}
}
\seealso{
Other mcmcComposite functions: 
\code{\link{+.PriorsmcmcComposite}()},
\code{\link{MHalgoGen}()},
\code{\link{as.mcmc.mcmcComposite}()},
\code{\link{as.parameters}()},
\code{\link{merge.mcmcComposite}()},
\code{\link{plot.PriorsmcmcComposite}()},
\code{\link{plot.mcmcComposite}()},
\code{\link{setPriors}()},
\code{\link{setPriors1}()},
\code{\link{summary.mcmcComposite}()}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Snbinom.R
\name{dSnbinom}
\alias{dSnbinom}
\alias{pSnbinom}
\alias{qSnbinom}
\alias{rSnbinom}
\title{Distribution of the sum independent negative binomial random variables.}
\usage{
dSnbinom(
  x = stop("You must provide at least one x value"),
  size = NULL,
  prob = NULL,
  mu = NULL,
  log = FALSE,
  tol = NULL,
  method = "Furman",
  normalize = TRUE,
  max.iter = NULL,
  mean = NULL,
  sd = NULL,
  n.random = 1e+06,
  parallel = FALSE,
  verbose = FALSE
)

pSnbinom(
  q = stop("At least one quantile must be provided"),
  size = NULL,
  prob = NULL,
  mu = NULL,
  lower.tail = TRUE,
  log.p = FALSE,
  tol = NULL,
  method = "Furman",
  normalize = TRUE
)

qSnbinom(
  p = stop("At least one probability must be provided"),
  size = stop("size parameter is mandatory"),
  prob = NULL,
  mu = NULL,
  lower.tail = TRUE,
  log.p = FALSE,
  tol = NULL,
  method = "Furman"
)

rSnbinom(n = 1, size = NULL, prob = NULL, mu = NULL)
}
\arguments{
\item{x}{vector of (non-negative integer) quantiles.}

\item{size}{target for number of successful trials, or dispersion parameter (the shape parameter of the gamma mixing distribution). Must be strictly positive, need not be integer.}

\item{prob}{probability of success in each trial. 0 < prob <= 1.}

\item{mu}{alternative parametrization via mean.}

\item{log, log.p}{logical; if TRUE, probabilities \emph{p} are given as \emph{log(p)}.}

\item{tol}{Tolerance for recurrence for Furman (2007) method. If NULL, will use a saddlepoint estimation.}

\item{method}{Can be Furman (default), Vellaisamy&Upadhye or exact, approximate.normal, approximate.negativebinomial, approximate.RandomObservations, or saddlepoint.}

\item{normalize}{If TRUE (default) will normalize the saddlepoint approximation estimate.}

\item{max.iter}{Number of maximum iterations for Furman method. Can be NULL.}

\item{mean}{Mean of the distribution for approximate.normal method. If NULL, the theoretical mean will be used.}

\item{sd}{Standard deviation of the distribution for approximate.normal method. If NULL, the theoretical sd will be used.}

\item{n.random}{Number of random numbers used to estimate parameters of distribution for approximate.RandomObservations method.}

\item{parallel}{logical; if FALSE (default), parallel computing is not used for Vellaisamy&Upadhye methods.}

\item{verbose}{Give more information on the method.}

\item{q}{vector of quantiles.}

\item{lower.tail}{logical; if TRUE (default), probabilities are P[X <= x], otherwise, P[X > x].}

\item{p}{vector of probabilities.}

\item{n}{number of observations.}
}
\value{
\code{dSnbinom} gives the density, \code{pSnbinom} gives the distribution function,
\code{qSnbinom} gives the quantile function, and \code{rSnbinom} generates random deviates.
}
\description{
Distribution of the sum of random variable with negative binomial distributions.\cr
Technically the sum of random variable with negative binomial distributions is a convolution of negative
binomial random variables.\cr
\code{dSnbinom} returns the density for the sum of random variable with negative binomial distributions.\cr
\code{pSnbinom} returns the distribution function for the sum of random variable with negative binomial distributions.\cr
\code{qSnbinom} returns the quantile function for the sum of random variable with negative binomial distributions.\cr
\code{rSnbinom} returns random numbers for the sum of random variable with negative binomial distributions.\cr
If all prob values are the same, exact probabilities are estimated.\cr
Estimate using \code{Vellaisamy&Upadhye} method uses parallel computing
depending on value of \code{parallel}. The number of cores in usage can be
defined using \code{options(mc.cores = c)} with \code{c} being the number of cores to be used. By default it will
use all the available cores. Forking will be used in Unix system and no forking on Windows systems.\cr
When \code{Furman} method is in use, it will return the progress of Pr(S = x) during recursion
in an attribute if verbose is TRUE (see examples).
}
\details{
Distribution of the Sum of Independent Negative Binomial Random Variables.
}
\section{Functions}{
\itemize{
\item \code{dSnbinom()}: Density for the sum of random variable with negative binomial distributions.

\item \code{pSnbinom()}: Distribution function for the sum of random variable with negative binomial distributions.

\item \code{qSnbinom()}: Quantile function for the sum of random variable with negative binomial distributions.

\item \code{rSnbinom()}: Random numbers for the sum of random variable with negative binomial distributions.

}}
\examples{
\dontrun{
library(HelpersMG)
alpha <- c(1, 2, 5, 1, 2)
p <- c(0.1, 0.12, 0.13, 0.14, 0.14)
# By default, the Furman method with tol=1E-40 is used
dSnbinom(20, size=alpha, prob=p)
# Note the attribute is the dynamics of convergence of Pr(X=x)
attributes(dSnbinom(20, size=alpha, prob=p, verbose=TRUE))$Pk[, 1]

mutest <- c(0.01, 0.02, 0.03)
sizetest <- 2
x <- 20
# Exact probability
dSnbinom(x, size=sizetest, mu=mutest, method="vellaisamy&upadhye")
dSnbinom(x, size=sizetest, mu=mutest, method="vellaisamy&upadhye", log=TRUE)
# With Furman method and tol=1E-12, when probability 
# is very low, it will be biased
dSnbinom(x, size=sizetest, mu=mutest, method="Furman", tol=1E-12)
# The solution is to use a tolerance lower than the estimate
dSnbinom(x, size=sizetest, mu=mutest, method="Furman", tol=1E-45)
# Here the estimate used a first estimation by saddlepoint approximation
dSnbinom(x, size=sizetest, mu=mutest, method="Furman", tol=NULL)
# Or a huge number of iterations; but it is not the best solution
dSnbinom(x, size=sizetest, mu=mutest, method="Furman", 
         tol=1E-12, max.iter=10000)
# With the saddle point approximation method
dSnbinom(x, size=sizetest, mu=mutest, method="saddlepoint", log=FALSE)
dSnbinom(x, size=sizetest, mu=mutest, method="saddlepoint", log=TRUE)

# Another example
sizetest <- c(1, 1, 0.1)
mutest <- c(2, 1, 10)
x <- 5
(exact <- dSnbinom(x=x, size=sizetest, mu=mutest, method="Vellaisamy&Upadhye"))
(sp <- dSnbinom(x=x, size=sizetest, mu=mutest, method="saddlepoint"))
paste0("Saddlepoint approximation: Error of ", specify_decimal(100*abs(sp-exact)/exact, 2), "\%")
(furman <- dSnbinom(x=x, size=sizetest, mu=mutest, method="Furman"))
paste0("Inversion of mgf: Error of ", specify_decimal(100*abs(furman-exact)/exact, 2), "\%")
(na <- dSnbinom(x=x, size=sizetest, mu=mutest, method="approximate.normal")) 
paste0("Gaussian approximation: Error of ", specify_decimal(100*abs(na-exact)/exact, 2), "\%")
(nb <- dSnbinom(x=x, size=sizetest, mu=mutest, method="approximate.negativebinomial"))
paste0("NB approximation: Error of ", specify_decimal(100*abs(nb-exact)/exact, 2), "\%")

plot(0:20, dSnbinom(0:20, size=sizetest, mu=mutest, method="furman"), bty="n", type="h", 
     xlab="x", ylab="Density", ylim=c(0, 0.2), las=1)
points(x=0:20, y=dSnbinom(0:20, size=sizetest, mu=mutest, 
                           method="saddlepoint"), pch=1, col="blue")
points(x=0:20, y=dSnbinom(0:20, size=sizetest, mu=mutest, 
                           method="approximate.negativebinomial"), 
                           col="red")
points(x=0:20, y=dSnbinom(0:20, size=sizetest, mu=mutest, 
                           method="approximate.normal"), 
                           col="green")

# Test with a single distribution
dSnbinom(20, size=1, mu=20)
# when only one distribution is available, it is the same as dnbinom()
dnbinom(20, size=1, mu=20)

# If a parameter is supplied as only one value, it is supposed to be constant
dSnbinom(20, size=1, mu=c(14, 15, 10))
dSnbinom(20, size=c(1, 1, 1), mu=c(14, 15, 10))

# The functions are vectorized:
plot(0:200, dSnbinom(0:200, size=alpha, prob=p, method="furman"), bty="n", type="h", 
     xlab="x", ylab="Density")
points(0:200, dSnbinom(0:200, size=alpha, prob=p, method="saddlepoint"), 
     col="red", pch=3)
     
# Comparison with simulated distribution using rep replicates
alpha <- c(2.1, 2.05, 2)
mu <- c(10, 30, 20)
rep <- 100000
distEmpirique <- rSnbinom(rep, size=alpha, mu=mu)
tabledistEmpirique <- rep(0, 301)
names(tabledistEmpirique) <- as.character(0:300)
tabledistEmpirique[names(table(distEmpirique))] <- table(distEmpirique)/rep

plot(0:300, dSnbinom(0:300, size=alpha, mu=mu, method="furman"), type="h", bty="n", 
   xlab="x", ylab="Density", ylim=c(0,0.02))
plot_add(0:(length(tabledistEmpirique)-1), tabledistEmpirique, type="l", col="red")
legend(x=200, y=0.02, legend=c("Empirical", "Theoretical"), 
   text.col=c("red", "black"), bty="n")


# Example from Vellaisamy, P. & Upadhye, N.S. (2009) - Table 1
# Note that computing time for k = 7 using exact method is very long
k <- 2:7
x <- c(3, 5, 8, 10, 15)
table1_Vellaisamy <- matrix(NA, ncol=length(x), nrow=length(k))
rownames(table1_Vellaisamy) <- paste0("n = ", as.character(k))
colnames(table1_Vellaisamy) <- paste0("x = ", as.character(x))
table1_approximateObservations <- table1_Vellaisamy
table1_Furman3 <- table1_Vellaisamy
table1_Furman6 <- table1_Vellaisamy
table1_Furman9 <- table1_Vellaisamy
table1_Furman12 <- table1_Vellaisamy
table1_Furman40 <- table1_Vellaisamy
table1_Furman40 <- table1_Vellaisamy
table1_FurmanAuto <- table1_Vellaisamy
table1_FurmanAuto_iter <- table1_Vellaisamy
table1_Vellaisamy_parallel <- table1_Vellaisamy
table1_Approximate_Normal <- table1_Vellaisamy
table1_saddlepoint <- table1_Vellaisamy

st_Furman3 <- rep(NA, length(k))
st_Furman6 <- rep(NA, length(k))
st_Furman9 <- rep(NA, length(k))
st_Furman12 <- rep(NA, length(k))
st_Furman40 <- rep(NA, length(k))
st_FurmanAuto <- rep(NA, length(k))
st_approximateObservations <- rep(NA, length(k))
st_Vellaisamy <- rep(NA, length(k))
st_Vellaisamy_parallel <- rep(NA, length(k))
st_Approximate_Normal <- rep(NA, length(k))
st_saddlepoint <- rep(NA, length(k))

for (n in k) {
    print(n)
    alpha <- 1:n
    p <- (1:n)/10
    st_Vellaisamy[which(n == k)] <- 
        system.time({
        table1_Vellaisamy[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="Vellaisamy&Upadhye", log=FALSE, verbose=FALSE)
        })[1]
    st_Vellaisamy_parallel[which(n == k)] <- 
        system.time({
        table1_Vellaisamy_parallel[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     parallel=TRUE, 
                                     method="Vellaisamy&Upadhye", log=FALSE, verbose=FALSE)
        })[1]
    st_approximateObservations[which(n == k)] <- 
        system.time({
        table1_approximateObservations[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="approximate.RandomObservations", log=FALSE, 
                                     verbose=FALSE)
        })[1]
    st_Furman3[which(n == k)] <- 
        system.time({
            table1_Furman3[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="Furman", tol=1E-3, log=FALSE, 
                                     verbose=FALSE)
        })[1]
    st_Furman6[which(n == k)] <- 
        system.time({
            table1_Furman6[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="Furman", tol=1E-6, log=FALSE, 
                                     verbose=FALSE)
        })[1]
    st_Furman9[which(n == k)] <- 
        system.time({
            table1_Furman9[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="Furman", tol=1E-9, log=FALSE, 
                                     verbose=FALSE)
        })[1]
    st_Furman12[which(n == k)] <- 
        system.time({
            table1_Furman12[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="Furman", tol=1E-12, log=FALSE, 
                                     verbose=FALSE)
        })[1]
    st_Furman40[which(n == k)] <- 
        system.time({
            table1_Furman40[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="Furman", tol=1E-40, log=FALSE, 
                                     verbose=FALSE)
        })[1]

    st_FurmanAuto[which(n == k)] <- 
        system.time({
            table1_FurmanAuto[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="Furman", tol=NULL, log=FALSE, 
                                     verbose=FALSE)
        })[1]

    st_Approximate_Normal[which(n == k)] <- 
        system.time({
            table1_Approximate_Normal[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="approximate.normal", tol=1E-12, log=FALSE, 
                                     verbose=FALSE)
        })[1]
        st_saddlepoint[which(n == k)] <- 
        system.time({
            table1_saddlepoint[which(n == k), ] <- dSnbinom(x=x, prob=p, size=alpha, 
                                     method="saddlepoint", tol=1E-12, log=FALSE, 
                                     verbose=FALSE)
        })[1]

for (xc in x) {
 essai <- dSnbinom(x=xc, prob=p, size=alpha, method="Furman", tol=NULL, log=FALSE, verbose=TRUE)
 table1_FurmanAuto_iter[which(n == k), which(xc == x)] <- nrow(attributes(essai)[[1]])
}
}

cbind(table1_Vellaisamy, st_Vellaisamy)
cbind(table1_Vellaisamy_parallel, st_Vellaisamy_parallel)
cbind(table1_Furman3, st_Furman3)
cbind(table1_Furman6, st_Furman6)
cbind(table1_Furman9, st_Furman9)
cbind(table1_Furman12, st_Furman12)
cbind(table1_Furman40, st_Furman40)
cbind(table1_FurmanAuto, st_FurmanAuto)
cbind(table1_approximateObservations, st_approximateObservations)
cbind(table1_Approximate_Normal, st_Approximate_Normal)
cbind(table1_saddlepoint, st_saddlepoint)


# Test of different methods
n <- 9
x <- 17
alpha <- 1:n
p <- (1:n)/10

# Parallel computing is not always performant
# Here it is very performant
system.time({print(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
         verbose=TRUE, parallel=TRUE))})
system.time({print(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
         verbose=TRUE, parallel=FALSE))})
         
# Test of different methods
n <- 7
x <- 8
alpha <- 1:n
p <- (1:n)/10

# Parallel computing is not always performant
# Here it is approximately the same time of execution
system.time({print(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
         verbose=TRUE, parallel=TRUE))})
system.time({print(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
         verbose=TRUE, parallel=FALSE))})
         
# Test of different methods
n <- 7
x <- 15
alpha <- 1:n
p <- (1:n)/10

# Parallel computing is sometimes very performant
# Here parallel computing is 7 times faster (with a 8 cores computer) 
#             for vellaisamy&upadhye method
system.time(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
         verbose=TRUE, parallel=TRUE))
system.time(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
         verbose=TRUE, parallel=FALSE))
         
# Test of different methods
n <- 2
x <- 3
alpha <- 1:n
p <- (1:n)/10

# Parallel computing is sometimes very performant
# Here parallel computing is 7 times faster (with a 8 cores computer) 
#             for vellaisamy&upadhye method
system.time(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
         verbose=TRUE, parallel=TRUE))
system.time(dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, 
         verbose=TRUE, parallel=FALSE))
         
# Test for different tolerant values
n <- 7
x <- 8
alpha <- 1:n
p <- (1:n)/10
dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, verbose=TRUE)
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-3, verbose=TRUE))
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-6, verbose=TRUE))
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-9, verbose=TRUE))
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, verbose=TRUE))
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Saddlepoint", log=FALSE, verbose=TRUE))
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="approximate.RandomObservations", 
         log=FALSE, verbose=TRUE))

# Test for criteria of convergence
Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
                   log=FALSE, verbose=TRUE)
Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
                 verbose=TRUE)
Pr_exact;as.numeric(Pr_Furman)
plot(1:length(attributes(Pr_Furman)$Pk), 
     log10(abs(attributes(Pr_Furman)$Pk-Pr_exact)), type="l", xlab="Iterations", 
     ylab="Abs log10", bty="n")
lines(1:(length(attributes(Pr_Furman)$Pk)-1), 
      log10(abs(diff(attributes(Pr_Furman)$Pk))), col="red")
legend("bottomleft", legend=c("Log10 Convergence to true value", "Log10 Rate of change"), 
       col=c("black", "red"), 
       lty=1)
       
n <- 7
x <- 6
alpha <- 1:n
p <- (1:n)/10
Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
                   log=FALSE, verbose=TRUE)
Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
                 verbose=TRUE)
Pr_saddlepoint <- dSnbinom(x=x, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=TRUE)                  
                 
# pdf("figure.pdf", width=7, height=7, pointsize=14)                 
ylab <- as.expression(bquote(.("P(S=6) x 10")^"6"))
layout(1:2)
par(mar=c(3, 4, 1, 1))
plot(1:length(attributes(Pr_Furman)$Pk), 
     attributes(Pr_Furman)$Pk*1E6, type="l", xlab="", 
     ylab="", bty="n", las=1, xlim=c(0, 80))
mtext(text=ylab, side=2, line=2.5)
segments(x0=25, x1=80, y0=Pr_exact*1E6, y1=Pr_exact*1E6, lty=3)
par(xpd=TRUE)
text(x=0, y=6, labels="Exact probability", pos=4)
txt <- "        Approximate probability\n    based on Furman (2007)\nrecursive iterations"
text(x=20, y=2, labels=txt, pos=4)
text(x=75, y=5, labels="A", cex=2)
par(mar=c(4, 4, 1, 1))
ylab <- as.expression(bquote("log"["10"]*""*"(P"["k+1"]*""*" - P"["k"]*""*")"))
plot(1:(length(attributes(Pr_Furman)$Pk)-1), 
      log10(diff(attributes(Pr_Furman)$Pk)), col="black", xlim=c(0, 80), type ="l", 
      bty="n", las=1, xlab="Iterations", ylab="")
mtext(text=ylab, side=2, line=2.5)
 peak <- (1:(length(attributes(Pr_Furman)$Pk)-1))[which.max(
            log10(abs(diff(attributes(Pr_Furman)$Pk))))]
segments(x0=peak, x1=peak, y0=-12, y1=-5, lty=2)
text(x=0, y=-6, labels="Positive trend", pos=4)
text(x=30, y=-6, labels="Negative trend", pos=4)
segments(x0=0, x1=43, y0=-12, y1=-12, lty=4)
segments(x0=62, x1=80, y0=-12, y1=-12, lty=4)
text(x=45, y=-12, labels="Tolerance", pos=4)
text(x=75, y=-7, labels="B", cex=2)
# dev.off()

# pdf("figure 2.pdf", width=7, height=7, pointsize=14)                 
ylab <- as.expression(bquote(.("P(S=6) x 10")^"6"))
layout(1:2)
par(mar=c(3, 4, 1, 1))
plot(1:length(attributes(Pr_Furman)$Pk), 
     attributes(Pr_Furman)$Pk*1E6, type="l", xlab="", 
     ylab="", bty="n", las=1, xlim=c(0, 80))
mtext(text=ylab, side=2, line=2.5)
segments(x0=25, x1=80, y0=Pr_exact*1E6, y1=Pr_exact*1E6, lty=3)
par(xpd=TRUE)
text(x=0, y=6, labels="Exact probability", pos=4)
txt <- "        Approximate probability\n    based on Furman (2007)\nrecursive iterations"
text(x=20, y=2, labels=txt, pos=4)
text(x=75, y=5, labels="A", cex=2)
par(mar=c(4, 4, 1, 1))
ylab <- as.expression(bquote("(P"["k+1"]*""*" - P"["k"]*""*") x 10"^"7"))
plot(1:(length(attributes(Pr_Furman)$Pk)-1), 
      diff(attributes(Pr_Furman)$Pk)*1E7, 
      col="black", xlim=c(0, 80), type ="l", 
      bty="n", las=1, xlab="Iterations", ylab="")
mtext(text=ylab, side=2, line=2.5)
 peak <- (1:(length(attributes(Pr_Furman)$Pk)-1))[which.max(diff(attributes(Pr_Furman)$Pk))]
segments(x0=peak, x1=peak, y0=0, y1=3.5, lty=2)
text(x=-2, y=3.5, labels="Positive trend", pos=4)
text(x=30, y=3.5, labels="Negative trend", pos=4)
segments(x0=0, x1=22, y0=1E-12, y1=1E-12, lty=4)
segments(x0=40, x1=80, y0=1E-12, y1=1E-12, lty=4)
text(x=22, y=1E-12+0.2, labels="Tolerance", pos=4)
text(x=75, y=3, labels="B", cex=2)
# dev.off()

# Test of different methods
n <- 2
x <- 15
alpha <- 1:n
p <- (1:n)/10
dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", log=FALSE, verbose=TRUE)
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-3, verbose=TRUE))
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-6, verbose=TRUE))
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-9, verbose=TRUE))
as.numeric(dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, verbose=TRUE))
dSnbinom(x=x, prob=p, size=alpha, method="approximate.RandomObservations", 
         log=FALSE, verbose=TRUE)


n <- 50
x <- 300
alpha <- (1:n)/100
p <- (1:n)/1000
# Produce an error
Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
                   log=FALSE, verbose=TRUE)
Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
                 verbose=FALSE)
Pr_ApproximateNormal <- dSnbinom(x=x, prob=p, size=alpha, method="approximate.normal", 
                        log=FALSE, 
                        verbose=TRUE)
Pr_ApproximateRandom <- dSnbinom(x=x, prob=p, size=alpha, method="approximate.RandomObservations", 
                        log=FALSE, n.random=1E6, 
                        verbose=TRUE)
                        
n <- 500
x <- 3000
alpha <- (1:n)/100
p <- (1:n)/1000
# Produce an error
Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
                   log=FALSE, verbose=TRUE)
Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
                 verbose=FALSE)
Pr_ApproximateNormal <- dSnbinom(x=x, prob=p, size=alpha, method="approximate.normal", 
                        log=FALSE, 
                        verbose=TRUE)
Pr_ApproximateNegativeBinomial <- dSnbinom(x=x, prob=p, size=alpha, 
                        method="approximate.negativebinomial", 
                        log=FALSE, 
                        verbose=TRUE)
Pr_ApproximateRandom <- dSnbinom(x=x, prob=p, size=alpha, 
                        method="approximate.RandomObservations", 
                        log=FALSE, n.random=1E6, 
                        verbose=TRUE)
Pr_ApproximateSaddlepoint <- dSnbinom(x=x, prob=p, size=alpha, 
                        method="saddlepoint", 
                        log=FALSE, 
                        verbose=TRUE)
             
             
             
layout(matrix(1:4, ncol=2, byrow=TRUE))
par(mar=c(3, 4.5, 1, 1))
alpha <- seq(from=10, to=100, length.out=3)
p <- seq(from=0.5, to=0.9, length.out=3)

p_nb <- dSnbinom(0:100, prob=p, size=alpha, method="vellaisamy&upadhye", verbose=TRUE)
p_Furman <- dSnbinom(0:100, prob=p, size=alpha, method="Furman", verbose=FALSE)
p_normal <- dSnbinom(0:100, prob=p, size=alpha, method="approximate.normal", verbose=TRUE)
p_aNB <- dSnbinom(0:100, prob=p, size=alpha, method="approximate.negativebinomial", verbose=TRUE)
p_SA <- dSnbinom(0:100, prob=p, size=alpha, method="saddlepoint", verbose=TRUE)

lab_PSnx <- bquote(italic("P(S"* "" [n] * "=x)"))

plot(1, 1, las=1, bty="n", col="grey", xlab="", 
       xlim=c(10, 70), ylim=c(0, 0.05), 
       ylab=lab_PSnx, type="n")
par(xpd=FALSE)
segments(x0=(0:100), x1=(0:100), 
         y0=0, y1=as.numeric(p_nb), col="black")
         
plot(x=p_nb, y=p_normal, pch=4, cex=0.5, las=1, bty="n", 
     xlab=bquote(italic("P" * "" [exact] * "(S"* "" [n] * "=x)")), 
     ylab=bquote(italic("P" * "" [approximate] * "(S"* "" [n] * "=x)")),  
     xlim=c(0, 0.05), ylim=c(0, 0.05))
points(x=p_nb, y=p_aNB, pch=5, cex=0.5)
points(x=p_nb, y=p_SA, pch=6, cex=0.5)
points(x=p_Furman, y=p_SA, pch=19, cex=0.5)

n <- 2
x <- 15
alpha <- 1:n
p <- (1:n)/10

p_nb <- dSnbinom(0:80, prob=p, size=alpha, method="vellaisamy&upadhye", verbose=TRUE)
p_Furman <- dSnbinom(0:80, prob=p, size=alpha, method="Furman", verbose=FALSE)
p_normal <- dSnbinom(0:80, prob=p, size=alpha, method="approximate.normal", verbose=TRUE)
p_aNB <- dSnbinom(0:80, prob=p, size=alpha, method="approximate.negativebinomial", verbose=TRUE)
p_SA <- dSnbinom(0:80, prob=p, size=alpha, method="saddlepoint", verbose=TRUE)


par(mar=c(4, 4.5, 1, 1))
plot(1, 1, las=1, bty="n", col="grey", xlab="x", 
       xlim=c(0, 60), ylim=c(0, 0.05), 
       ylab=lab_PSnx, type="n")
par(xpd=FALSE)
segments(x0=(0:80), x1=(0:80), 
         y0=0, y1=as.numeric(p_nb), col="black")
         
plot(x=p_nb, y=p_normal, pch=4, cex=0.5, las=1, bty="n", 
     xlab=bquote(italic("P" * "" [exact] * "(S"* "" [n] * "=x)")), 
     ylab=bquote(italic("P" * "" [approximate] * "(S"* "" [n] * "=x)")), 
     xlim=c(0, 0.05), ylim=c(0, 0.05))
points(x=p_nb, y=p_aNB, pch=5, cex=0.5)
points(x=p_nb, y=p_SA, pch=6, cex=0.5)
points(x=p_Furman, y=p_SA, pch=19, cex=0.5)

# pdf("figure 1.pdf", width=7, height=7, pointsize=14)    

layout(1:2)
par(mar=c(3, 4, 1, 1))
alpha <- seq(from=10, to=100, length.out=3)
p <- seq(from=0.5, to=0.9, length.out=3)

p_nb <- dSnbinom(0:100, prob=p, size=alpha, method="vellaisamy&upadhye", verbose=TRUE)
p_Furman <- dSnbinom(0:100, prob=p, size=alpha, method="Furman", verbose=FALSE)
p_normal <- dSnbinom(0:100, prob=p, size=alpha, method="approximate.normal", verbose=TRUE)
p_aNB <- dSnbinom(0:100, prob=p, size=alpha, method="approximate.negativebinomial", verbose=TRUE)
p_SA <- dSnbinom(0:100, prob=p, size=alpha, method="saddlepoint", verbose=TRUE)

lab_PSnx <- bquote(italic("P(S"* "" [n] * "=x)"))

plot(1, 1, las=1, bty="n", col="grey", xlab="", 
       xlim=c(10, 70), ylim=c(0, 0.09), 
       ylab="", type="n", yaxt="n")
axis(2, at=seq(from=0, to=0.05, by=0.01), las=1)
mtext(lab_PSnx, side = 2, adj=0.3, line=3)
par(xpd=FALSE)
segments(x0=(0:100), x1=(0:100), 
         y0=0, y1=as.numeric(p_nb), col="black")
errr <- (abs((100*(p_Furman-p_nb)/p_nb)))/1000+0.055
errr <- ifelse(is.infinite(errr), NA, errr)
lines(x=(0:100), y=errr, lty=5, col="red", lwd=2)
errr <- (abs((100*(p_normal-p_nb)/p_nb)))/1000+0.055
lines(x=(0:100), y=errr, lty=2, col="blue", lwd=2)
errr <- (abs((100*(p_aNB-p_nb)/p_nb)))/1000+0.055
lines(x=(0:100), y=errr, lty=3, col="purple", lwd=2)
errr <- (abs((100*(p_SA-p_nb)/p_nb)))/1000+0.055
lines(x=(0:100), y=errr, lty=4, col="green", lwd=2)
axis(2, at=seq(from=0, to=40, by=10)/1000+0.055, las=1, 
     labels=as.character(seq(from=0, to=40, by=10)))
mtext("|\% error|", side = 2, adj=0.9, line=3)
par(xpd=TRUE)
legend(x=30, y=0.1, legend=c("Inversion of mgf", "Saddlepoint", "Normal", "Negative binomial"), 
       lty=c(5, 4, 2, 3), bty="n", cex=0.8, col=c("red", "green", "blue", "purple"), lwd=2)
legend(x=10, y=0.05, legend=c("Exact"), lty=c(1), bty="n", cex=0.8)
par(xpd=TRUE)
text(x=ScalePreviousPlot(x = 0.95, y = 0.1)$x, 
     y=ScalePreviousPlot(x = 0.95, y = 0.1)$y, labels="A", cex=2)
     
     
# When normal approximation will fail
n <- 2
x <- 15
alpha <- 1:n
p <- (1:n)/10

p_nb <- dSnbinom(0:80, prob=p, size=alpha, method="vellaisamy&upadhye", verbose=TRUE)
p_Furman <- dSnbinom(0:80, prob=p, size=alpha, method="Furman", verbose=FALSE)
p_normal <- dSnbinom(0:80, prob=p, size=alpha, method="approximate.normal", verbose=TRUE)
p_aNB <- dSnbinom(0:80, prob=p, size=alpha, method="approximate.negativebinomial", verbose=TRUE)
p_SA <- dSnbinom(0:80, prob=p, size=alpha, method="saddlepoint", verbose=TRUE)

par(mar=c(4, 4, 1, 1))
plot(1, 1, las=1, bty="n", col="grey", xlab="x", 
       xlim=c(0, 60), ylim=c(0, 0.09), 
       ylab="", type="n", yaxt="n")
axis(2, at=seq(from=0, to=0.05, by=0.01), las=1)
mtext(lab_PSnx, side = 2, adj=0.3, line=3)
par(xpd=FALSE)
segments(x0=(0:80), x1=(0:80), 
         y0=0, y1=as.numeric(p_nb), col="black")
errr <- (abs((100*(p_Furman-p_nb)/p_nb)))/1000+0.055
errr <- ifelse(is.infinite(errr), NA, errr)
lines(x=(0:80), y=errr, lty=5, col="red", lwd=2)
errr <- (abs((100*(p_normal-p_nb)/p_nb)))/1000+0.055
lines(x=(0:80), y=errr, lty=2, col="blue", lwd=2)
errr <- (abs((100*(p_aNB-p_nb)/p_nb)))/1000+0.055
lines(x=(0:80), y=errr, lty=3, col="purple", lwd=2)
errr <- (abs((100*(p_SA-p_nb)/p_nb)))/1000+0.055
lines(x=(0:80), y=errr, lty=4, col="green", lwd=2)
axis(2, at=seq(from=0, to=40, by=10)/1000+0.055, las=1, 
     labels=as.character(seq(from=0, to=40, by=10)))
mtext("|\% error|", side = 2, adj=0.9, line=3)
legend(x=30, y=0.055, 
       legend=c("Exact", "Inversion of mgf", "Saddlepoint", "Normal", "Negative binomial"), 
       lty=c(1, 5, 4, 2, 3), bty="n", cex=0.8, col=c("black", "red", "green", "blue", "purple"), 
       lwd=c(1, 2, 2, 2, 2))
par(xpd=TRUE)
text(x=ScalePreviousPlot(x = 0.95, y = 0.1)$x, 
     y=ScalePreviousPlot(x = 0.95, y = 0.1)$y, labels="B", cex=2)
     

# dev.off()


# Test for criteria of convergence
Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
                   log=FALSE, verbose=TRUE)
Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, 
                 verbose=TRUE)
Pr_exact;as.numeric(Pr_Furman)
plot(1:length(attributes(Pr_Furman)$Pk), 
     log10(abs(attributes(Pr_Furman)$Pk-Pr_exact)), type="l", xlab="Iterations", 
     ylab="Abs log10", bty="n")
lines(1:(length(attributes(Pr_Furman)$Pk)-1), 
      log10(abs(diff(attributes(Pr_Furman)$Pk))), col="red")
legend("bottomleft", legend=c("Log10 Convergence to true value", "Log10 Rate of change"), 
       col=c("black", "red"), 
       lty=1)


# Test of different methods
alpha <- c(2.05, 2)
mu <- c(10, 30)
test <- rSnbinom(n=100000, size=alpha, mu=mu)
plot(0:200, table(test)[as.character(0:200)]/sum(table(test), na.rm=TRUE), 
     bty="n", type="h", xlab="x", ylab="Density")
lines(x=0:200, dSnbinom(0:200, size=alpha, mu=mu, log=FALSE, method="Furman"), col="blue")
lines(x=0:200, y=dSnbinom(0:200, size=alpha, mu=mu, log=FALSE, 
                          method="vellaisamy&upadhye"), col="red")
lines(x=0:200, y=dSnbinom(0:200, size=alpha, mu=mu, log=FALSE, 
                          method="approximate.randomobservations"), col="green")

# Test for criteria of convergence for x = 50
x <- 50
# Test for criteria of convergence
Pr_exact <- dSnbinom(x=x, prob=p, size=alpha, method="vellaisamy&upadhye", 
                   log=FALSE, verbose=TRUE)
Pr_Furman <- dSnbinom(x=x, prob=p, size=alpha, method="Furman", log=FALSE, tol=1E-12, 
                 verbose=TRUE)
Pr_exact;as.numeric(Pr_Furman)
plot(1:length(attributes(Pr_Furman)$Pk), 
     log10(abs(attributes(Pr_Furman)$Pk-Pr_exact)), type="l", xlab="Iterations", 
     ylab="Abs log10", bty="n")
lines(1:(length(attributes(Pr_Furman)$Pk)-1), 
      log10(abs(diff(attributes(Pr_Furman)$Pk))), col="red")
legend("bottomleft", legend=c("Log10 Convergence to true value", "Log10 Rate of change"), 
       col=c("black", "red"), 
       lty=1)

# Another example more complicated
set.seed(2)
mutest <- c(56, 6.75, 1)
ktest <- c(50, 50, 50)
nr <- 100000
test <- rSnbinom(nr, size=ktest, mu=mutest)
system.time({pr_vellaisamy <- dSnbinom(x=0:150, size=ktest, mu=mutest, 
                          method = "vellaisamy&upadhye", verbose=FALSE, parallel=FALSE)})
# Parallel computing is not efficient
system.time({pr_vellaisamy <- dSnbinom(x=0:150, size=ktest, mu=mutest, 
                          method = "vellaisamy&upadhye", verbose=FALSE, parallel=TRUE)})
system.time({pr_furman <- dSnbinom(x=0:150, size=ktest, mu=mutest, prob=NULL, 
                      method = "furman", verbose=FALSE, log=FALSE)})
pr_approximateObservations <- dSnbinom(0:150, size=ktest, mu=mutest, 
                                       method = "approximate.randomobservations")

plot(table(test), xlab="N", ylab="Density", las=1, bty="n", ylim=c(0, 4000), xlim=c(0, 150))
lines(0:150, pr_vellaisamy*nr, col="red")
lines(0:150, pr_furman*nr, col="blue")
lines(0:150, pr_approximateObservations*nr, col="green")

dSnbinom(x=42, size=ktest, mu=mutest, prob=NULL, 
         method = "vellaisamy&upadhye", verbose=TRUE)
as.numeric(dSnbinom(x=42, size=ktest, mu=mutest, prob=NULL, 
           method = "Furman", verbose=TRUE))
dSnbinom(x=42, size=ktest, mu=mutest, prob=NULL, 
         method = "approximate.randomobservations", verbose=TRUE)

x <- 100
# Test for criteria of convergence
Pr_exact <- dSnbinom(x=x, size=ktest, mu=mutest, method="vellaisamy&upadhye", 
                   log=FALSE, verbose=TRUE)
Pr_Furman <- dSnbinom(x=x, size=ktest, mu=mutest, method="Furman", log=FALSE, 
                 verbose=TRUE)
Pr_exact;as.numeric(Pr_Furman)
plot(1:length(attributes(Pr_Furman)$Pk), 
     log10(abs(attributes(Pr_Furman)$Pk-Pr_exact)), type="l", xlab="Iterations", 
     ylab="Abs log10", bty="n", ylim=c(-100, 0))
lines(1:(length(attributes(Pr_Furman)$Pk)-1), 
      log10(abs(diff(attributes(Pr_Furman)$Pk))), col="red")
legend("bottomright", legend=c("Log10 Convergence to true value", "Log10 Rate of change"), 
       col=c("black", "red"), 
       lty=1)
       
     
# example to fit a distribution
data <- rnbinom(1000, size=1, mu=10)
hist(data)
ag <- rep(1:100, 10)
r <- aggregate(data, by=list(ag), FUN=sum)
hist(r[,2])

parx <- c(size=1, mu=10)

dSnbinomx <- function(x, par) {
  -sum(dSnbinom(x=x[,2], mu=rep(par["mu"], 10), size=par["size"], log=TRUE))
}

fit_mu_size <- optim(par = parx, fn=dSnbinomx, x=r, method="BFGS", control=c(trace=TRUE))
fit_mu_size$par

alpha <- c(2.1, 2.05, 2)
mu <- c(10, 30, 20)
p <- pSnbinom(q=10, size=alpha, mu=mu, lower.tail = TRUE)

alpha <- c(2.1, 2.05, 2)
mu <- c(10, 30, 20)
q <- qSnbinom(p=0.1, size=alpha, mu=mu, lower.tail = TRUE)

alpha <- c(2.1, 2.05, 2)
mu <- c(10, 30, 20)
rep <- 100000
distEmpirique <- rSnbinom(n=rep, size=alpha, mu=mu)
tabledistEmpirique <- rep(0, 301)
names(tabledistEmpirique) <- as.character(0:300)
tabledistEmpirique[names(table(distEmpirique))] <- table(distEmpirique)/rep

plot(0:300, dSnbinom(0:300, size=alpha, mu=mu), type="h", bty="n", 
   xlab="x", ylab="Density", ylim=c(0,0.02))
plot_add(0:300, tabledistEmpirique, type="l", col="red")
legend(x=200, y=0.02, legend=c("Empirical", "Theoretical"), 
   text.col=c("red", "black"), bty="n")
   
   
# Test if saddlepoint approximation must be normalized
# Yes, it must be
n <- 7
alpha <- 1:n
p <- (1:n)/10
dSnbinom(x=10, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=TRUE)
dSnbinom(x=10, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=TRUE, normalize=FALSE)
                 
# Test for saddlepoint when x=0
n <- 7
alpha <- 1:n
p <- (1:n)/10
dSnbinom(x=0, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=TRUE)
dSnbinom(x=1, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=TRUE)
dSnbinom(x=c(0, 1), prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=TRUE)
dSnbinom(x=c(0, 1), prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=FALSE)
                 
# Test when prob are all the same
p <- rep(0.2, 7)
n <- 7
alpha <- 1:n
dSnbinom(x=0:10, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=TRUE)
dSnbinom(x=0:10, prob=p, size=alpha, method="furman", log=FALSE,  
                 verbose=TRUE)
dSnbinom(x=0:10, prob=p, size=alpha, method="exact", log=FALSE,  
                 verbose=TRUE)
                 
# Test when n=1
p <- 0.2
n <- 1
alpha <- 1:n
dSnbinom(x=0:10, prob=p, size=alpha, method="saddlepoint", log=FALSE,  
                 verbose=TRUE)
dSnbinom(x=0:10, prob=p, size=alpha, method="furman", log=FALSE,  
                 verbose=TRUE)
dSnbinom(x=0:10, prob=p, size=alpha, method="exact", log=FALSE,  
                 verbose=TRUE)

# Poisson distribution
dpois(20, lambda =sum(c(14, 10)))
dnbinom(20, size=+Inf, mu=sum(c(14, 10)))
dSnbinom(20, size=+Inf, mu=c(14, 10))
}
}
\references{
Furman, E., 2007. On the convolution of the negative binomial random variables. Statistics & Probability Letters 77, 169-172.

Vellaisamy, P. & Upadhye, N.S. 2009. On the sums of compound negative binomial and gamma random variables. Journal of Applied Probability, 46, 272-283.

Girondot M, Barry J. 2023. Computation of the distribution of the sum of independent negative binomial random variables. Mathematical and Computational Applications 2023, 28, 63, doi:10.3390/mca28030063
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com} and Jon Barry \email{jon.barry@cefas.gov.uk}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.parameters.R
\name{as.parameters}
\alias{as.parameters}
\title{Extract parameters from mcmcComposite object}
\usage{
as.parameters(
  x = stop("A result obtained after a MCMC analysis must be given."),
  total = FALSE,
  index = "median",
  chain = "all",
  probs = c(0.025, 0.5, 0.975),
  silent = FALSE
)
}
\arguments{
\item{x}{A mcmcComposite obtained as a result of \code{MHalgoGen()} function}

\item{total}{If TRUE, does not use the thinned results.}

\item{index}{At which iteration the parameters must be taken, see description.}

\item{chain}{The chain in which to get parameters; "all" is for all chains.}

\item{probs}{Quantiles to be returned, see description.}

\item{silent}{If TRUE, does not print any information.}
}
\value{
A vector with parameters at maximum likelihood or index position
}
\description{
Take a mcmcComposite object and create a vector object with parameter value at specified iteration.\cr
If \code{index="best"}, the function will return the parameters for the highest likelihood. It also indicates at which iteration the maximum lihelihood has been observed.\cr
If \code{index="last"}, the function will return the parameters for the last likelihood.\cr
If \code{index="median"}, the function will return the median value of the parameter.\cr
if \code{index="quantile"}, the function will return the \code{probs} defined by quantiles parameter.\cr
If \code{index="mode"}, the function will return the mode value of the parameter based on Asselin de Beauville (1978) method.\cr
\code{index} can also be a numeric value. It uses all the chains being concatanated.\cr
This function uses the complete iterations available if total is TRUE. The adaptation part is never used.\cr
\code{index} can also be a function applied to the iterations.
}
\examples{
\dontrun{
library(HelpersMG)
require(coda)
x <- rnorm(30, 10, 2)
dnormx <- function(data, x) {
 data <- unlist(data)
 return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
}
parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
                              Prior1=c(10, 0.5), 
                              Prior2=c(2, 0.5), 
                              SDProp=c(1, 1), 
                              Min=c(-3, 0), 
                              Max=c(100, 10), 
                              Init=c(10, 2), 
                              stringsAsFactors = FALSE, 
                              row.names=c('mean', 'sd'))
mcmc_run <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
                      likelihood=dnormx, n.chains=1, n.adapt=100, 
                      thin=1, trace=1)
plot(mcmc_run, xlim=c(0, 20))
plot(mcmc_run, xlim=c(0, 10), parameters="sd")
mcmcforcoda <- as.mcmc(mcmc_run)
#' heidel.diag(mcmcforcoda)
raftery.diag(mcmcforcoda)
autocorr.diag(mcmcforcoda)
acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
batchSE(mcmcforcoda, batchSize=100)

# The batch standard error procedure is usually thought to 
# be not as accurate as the time series methods used in summary
summary(mcmcforcoda)$statistics[,"Time-series SE"]
summary(mcmc_run)
as.parameters(mcmc_run)
lastp <- as.parameters(mcmc_run, index="last")
parameters_mcmc[,"Init"] <- lastp

# The n.adapt set to 1 is used to not record the first set of parameters
# then it is not duplicated (as it is also the last one for 
# the object mcmc_run)
mcmc_run2 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
                       likelihood=dnormx, n.chains=1, n.adapt=1, 
                       thin=1, trace=1)
mcmc_run3 <- merge(mcmc_run, mcmc_run2)

####### no adaptation, n.adapt must be 0
parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
mcmc_run3 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
                       likelihood=dnormx, n.chains=1, n.adapt=0, 
                       thin=1, trace=1)
# With index being median, it returns the median value for each parameter
as.parameters(mcmc_run3, index="median")
as.parameters(mcmc_run3, index="mode")
as.parameters(mcmc_run3, index="best")
as.parameters(mcmc_run3, index="quantile", probs=0.025)
as.parameters(mcmc_run3, index="quantile", probs=0.975)
as.parameters(mcmc_run3, index="quantile", probs=c(0.025, 0.975))
d(as.parameters(mcmc_run3, index = function(x) mean(x)))
}
}
\references{
Asselin de Beauville J.-P. (1978). Estimation non paramétrique de la densité et du mode,
exemple de la distribution Gamma. Revue de Statistique Appliquée, 26(3):47-70.
}
\seealso{
Other mcmcComposite functions:
\code{\link[=+.PriorsmcmcComposite]{+.PriorsmcmcComposite()}},
\code{\link[=MHalgoGen]{MHalgoGen()}},
\code{\link[=PPD]{PPD()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=merge.mcmcComposite]{merge.mcmcComposite()}},
\code{\link[=plot.PriorsmcmcComposite]{plot.PriorsmcmcComposite()}},
\code{\link[=plot.mcmcComposite]{plot.mcmcComposite()}},
\code{\link[=setPriors]{setPriors()}},
\code{\link[=setPriors1]{setPriors1()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ELPDweight.R
\name{ELPDweight}
\alias{ELPDweight}
\title{Return the probability of models compared with loo being the best for prediction}
\usage{
ELPDweight(
  loos = stop("A named list of loo data must be provided."),
  nreplicates = 10000
)
}
\arguments{
\item{loos}{A named list of loos}

\item{nreplicates}{Number of replicates.}
}
\value{
A vector with the probabilities
}
\description{
Calculate the probability of models compared with loo being the best for prediction.
It uses simulation using the pointwise LOO log-predictive densities. This preserves the correlation
between models, because all models are evaluated on the same observations. This is the approach
recommended by Aki Vehtari et al. (2017) when uncertainty in model comparison matters.
}
\details{
ELPDweight calculates the probability that each model is best for prediction
}
\examples{
\dontrun{
model1_loo <- loo::loo(model1)
model2_loo <- loo::loo(model2)
ELPDweight(list(model1=model1_loo, model2=model2_loo))
}
}
\references{
Vehtari, A., Gelman, A., & Gabry, J. (2017). Practical Bayesian model evaluation using leave-one-out cross-validation and WAIC. Statistics and Computing, 27(5), 1413-1432. https://doi.org/10.1007/s11222-016-9696-4
}
\seealso{
Other AIC:
\code{\link[=ExtractAIC.glm]{ExtractAIC.glm()}},
\code{\link[=FormatCompareAIC]{FormatCompareAIC()}},
\code{\link[=compare_AIC]{compare_AIC()}},
\code{\link[=compare_AICc]{compare_AICc()}},
\code{\link[=compare_BIC]{compare_BIC()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{AIC}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RM_list.R
\name{RM_list}
\alias{RM_list}
\title{Return the list of results managment of an object.}
\usage{
RM_list(
  x = stop("An object with results managment must be provided"),
  RM = "RM",
  silent = FALSE,
  max.level = FALSE
)
}
\arguments{
\item{x}{The object to add a results managment}

\item{RM}{The name of results managment stored}

\item{silent}{Should the results be shown ?}

\item{max.level}{If TRUE, will return all list element of the objects}
}
\value{
A list with the names of results stored in an object
}
\description{
RM_list returns the list of results managment of an object.
}
\details{
RM_list returns the list of results managment of an object
}
\examples{
\dontrun{
library("HelpersMG")
# Let an object of class objclass being created
obj <- list(A=100, name="My object")
class(obj) <- "objclass"
# And now I create a RM to this object
obj <- RM_add(x=obj, RMname="NewAnalysis1")
RM_list(obj)
obj <- RM_add(x=obj, RMname="NewAnalysis2")
RM_list(obj)
obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V1", value=100)
RM_get(x=obj, RMname="NewAnalysis2", valuename="V1")
obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V1", value=200)
RM_get(x=obj, RMname="NewAnalysis2", valuename="V1")
obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V2", value=300)
RM_get(x=obj, RMname="NewAnalysis2", valuename="V2")
RM_list(obj)
rmlist <- RM_list(obj, max.level=TRUE)
rmlist
}
}
\seealso{
Other Results Managment:
\code{\link[=RM_add]{RM_add()}},
\code{\link[=RM_delete]{RM_delete()}},
\code{\link[=RM_duplicate]{RM_duplicate()}},
\code{\link[=RM_get]{RM_get()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Results Managment}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DIx.R
\name{DIx}
\alias{DIx}
\title{Return an index of quantitative asymmetry and complexity named Developmental Instability Index (DIx)}
\usage{
DIx(l1, l2, details = FALSE, version = 1)
}
\arguments{
\item{l1}{Set of measures at one side of an organism}

\item{l2}{Set of measures at the other side of an organism}

\item{details}{If TRUE, will show the details of computing}

\item{version}{Can be 1 or 2; see description}
}
\value{
A numeric value
}
\description{
Return an index of quantitative asymmetry and complexity.\cr
Higher is the value, higher is the complexity (number of objects) and diversity (difference between them).\cr
The indice is based on the product of the average angular distance of
Edwards (1971) for all permutations of measures for both sides with the
geometric mean of the inverse of Shannon entropy H for both sides.
Let p1 and p2 two vectors of relative measures of objects with sum(p1) = 1 and sum(p2)=1
and n1 being the number of objects in p1 and n2 being the number of objects in p2.\cr
Edwards distance for all permutations of p1 and p2 objects are computed and the average value E is calculated.\cr
The maximun possible Shannon index for identical n1 is max1 = sum((1/n1) * log(1/n1)).\cr
Shannon index is v1 = sum(p1 * log(p1)).\cr
If version == 2, the complementary of Shannon index for these n1 objects is used: c1 = 2 * max1 - v1\cr
If version == 1, the Shannon index is used directly.\cr
The geometry mean between both sides defined the measure of diversity within each side: S=sqrt(c1 * c2)\cr
The Developmental Instability Index is then S * E
}
\details{
DIx returns an index of quantitative asymmetry and complexity
}
\examples{
\dontrun{
l1 <- c(0.1, 0.1, 0.05, 0.2, 0.3, 0.25)
l2 <- c(0.2, 0.3, 0.5)
DIx(l1, l2)

l1 <- c(0.1, 0.1, 0.05, 0.2, 0.3, 0.25)
l2 <- c(0.1, 0.1, 0.05, 0.2, 0.3, 0.25)
DIx(l1, l2)

l1 <- c(0.2, 0.3, 0.5)
l2 <- c(0.2, 0.3, 0.5)
DIx(l1, l2)

l1 <- c(0.2, 0.2, 0.2, 0.2, 0.2)
l2 <- c(0.2, 0.3, 0.5)
DIx(l1, l2)

l1 <- c(0.2, 0.2, 0.2, 0.2, 0.2)
l2 <- c(0.3333, 0.3333, 0.3333)
DIx(l1, l2)

l1 <- c(0.2, 0.2, 0.2, 0.2, 0.2)
l2 <- c(0.2, 0.2, 0.2, 0.2, 0.2)
DIx(l1, l2)

l1 <- c(0.3333, 0.3333, 0.3333)
l2 <- c(0.3333, 0.3333, 0.3333)
DIx(l1, l2)
}
}
\references{
Edwards, A.W.F., 1971. Distances between populations on the basis of gene frequencies. Biometrics 27, 873–881.\cr
Shannon C.E. 1948 A mathematical theory of communication. Bell System Technical Journal 27(3), 379-423.\cr
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RM_get.R
\name{RM_get}
\alias{RM_get}
\title{Get a value in a results managment to an object}
\usage{
RM_get(
  x = stop("An object with results managment must be provided"),
  RM = "RM",
  RMname = NULL,
  valuename = NULL
)
}
\arguments{
\item{x}{The object in which to get a result in a results managment}

\item{RM}{The name of results managment stored}

\item{RMname}{The name of the results managment to be read}

\item{valuename}{The name of the value to be read}
}
\value{
Return a value in a results managment object
}
\description{
Return the value valuename of the results managment RMname.
}
\details{
RM_get gets a value in results managment to an object
}
\examples{
\dontrun{
library("HelpersMG")
# Let an object of class objclass being created
obj <- list(A=100, name="My object")
class(obj) <- "objclass"
# And now I create a RM to this object
obj <- RM_add(x=obj, RMname="NewAnalysis1")
RM_list(obj)
obj <- RM_add(x=obj, RMname="NewAnalysis2")
RM_list(obj)
obj <- RM_add(x=obj, RMname="NewAnalysis2", valuename="V1", value=100)
RM_get(x=obj, RMname="NewAnalysis2", valuename="V1")
}
}
\seealso{
Other Results Managment:
\code{\link[=RM_add]{RM_add()}},
\code{\link[=RM_delete]{RM_delete()}},
\code{\link[=RM_duplicate]{RM_duplicate()}},
\code{\link[=RM_list]{RM_list()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Results Managment}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modifyVector.R
\name{modifyVector}
\alias{modifyVector}
\title{Modifies Elements of a Vector}
\usage{
modifyVector(x, val, add = TRUE)
}
\arguments{
\item{x}{A named vector.}

\item{val}{A named vector with components to replace corresponding components in x.}

\item{add}{If FALSE, only existing elements of x are returned.}
}
\value{
A modified version of x, with the elements of val replacing the elements of x
}
\description{
Modifies a vector by changing a subset of elements to match a second vector.
}
\details{
modifyVector modifies elements of a vector
}
\examples{
library("HelpersMG")
e <- c(M=10, L=20, J=30)
modifyVector(e, c(U=10, M=30))
modifyVector(e, c(U=10, M=30), add=FALSE)
}
\author{
Marc Girondot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duplicated_packages.R
\name{duplicated_packages}
\alias{duplicated_packages}
\title{List the duplicated packages with their locations}
\usage{
duplicated_packages()
}
\value{
A data.frame with 4 elements for each duplicated packages:\cr
\itemize{
\item versions: the version of the packages\cr
\item libraries: the locations\cr
}
}
\description{
A data.frame with the duplicated packages and their locations and version.\cr
The columns Lib1 and Version1 should have the oldest version of the packages.
}
\details{
duplicated_packages lists the duplicated packages with their locations
}
\examples{
\dontrun{
library(HelpersMG)
duplicated_packages()
# To remove the oldest versions of the installed packages, use
li <- duplicated_packages()
if (nrow(li) != 0)
    for (i in 1:nrow(li))
        remove.packages(rownames(li)[i], lib=li[i, "Lib1"])
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/d.R
\name{d}
\alias{d}
\title{Write an ASCII Representation of a vector object}
\usage{
d(
  x,
  file = "",
  control = c("keepNA", "keepInteger", "showAttributes"),
  collapse = ", \\n  "
)
}
\arguments{
\item{x}{A named vector object}

\item{file}{either a character string naming a file or a connection. "" indicates output to the console.}

\item{control}{character vector indicating deparsing options. See .deparseOpts for their description.}

\item{collapse}{Characters used to separate values.}
}
\value{
A string
}
\description{
Writes an ASCII text representation of an R object.\cr
It can be used as a replacement of dput() for named vectors.\cr
The controls "keepNA", "keepInteger" and "showAttributes" are utilized for named vectors.
}
\details{
d Write an ASCII Representation of a vector object
}
\examples{
d(c(A=10, B=20))
dput(c(A=10, B=20))
}
\seealso{
Other Characters:
\code{\link[=asc]{asc()}},
\code{\link[=char]{char()}},
\code{\link[=tnirp]{tnirp()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Characters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.mcmcComposite.R
\name{plot.mcmcComposite}
\alias{plot.mcmcComposite}
\title{Plot the result of a mcmcComposite object}
\usage{
\method{plot}{mcmcComposite}(
  x,
  ...,
  chains = "all",
  parameters = 1,
  transform = NULL,
  scale.prior = TRUE,
  legend = "topright",
  ylab.posterior = "Posterior density",
  las = 1,
  bty = "n",
  show.prior = TRUE,
  show.posterior.density = TRUE,
  col.prior = "red",
  lty.prior = 1,
  lwd.prior = 1,
  what = "Posterior",
  col.posterior = colorRampPalette(c("blue", "grey"), alpha = 0.001),
  lty.posterior = 1,
  lwd.posterior = 1,
  show.yaxis.prior = TRUE,
  ylab.prior = "Prior density"
)
}
\arguments{
\item{x}{A mcmcComposite object}

\item{...}{Graphical parameters to be sent to hist() or plot()}

\item{chains}{The chains to use}

\item{parameters}{Name of parameters or "all"}

\item{transform}{Function to be used to transform the variable}

\item{scale.prior}{If TRUE, the prior is scaled at the same size as posterior}

\item{legend}{If FALSE, the legend is not shown; see description}

\item{ylab.posterior}{y-label for posterior}

\item{las}{las parameter (orientation of y-axis graduation)}

\item{bty}{Design of box for Markov Chain plot}

\item{show.prior}{Should the prior be shown?}

\item{show.posterior.density}{Should the posterior density be shown?}

\item{col.prior}{Color for prior curve}

\item{lty.prior}{Type of line for prior curve}

\item{lwd.prior}{Width of line for prior curve}

\item{what}{can be Posterior, MarkovChain or LnL}

\item{col.posterior}{Color for posterior histogram}

\item{lty.posterior}{Type of line for posterior histogram}

\item{lwd.posterior}{Width of line for posterior histogram}

\item{show.yaxis.prior}{Should the y-axis for prior be shown}

\item{ylab.prior}{y-label for prior}
}
\value{
None
}
\description{
Plot the results within a mcmcComposite object.\cr
If scale.prior is TRUE, another scale is shown at right.\cr
legend can take these values: \cr
FALSE, TRUE, topleft, topright, bottomleft, bottomright, c(x=, y=)
}
\details{
plot.mcmcComposite plots the result of a MCMC search
}
\examples{
\dontrun{
library(HelpersMG)
require(coda)
x <- rnorm(30, 10, 2)
dnormx <- function(data, x) {
 data <- unlist(data)
 return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
}
parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
                              Prior1=c(10, 0.5), Prior2=c(2, 0.5), 
                              SDProp=c(1, 1), 
                              Min=c(-3, 0), Max=c(100, 10), 
                              Init=c(10, 2), stringsAsFactors = FALSE, 
                              row.names=c('mean', 'sd'))
                              
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=x, 
                      adaptive = TRUE,
                      likelihood=dnormx, n.chains=4, 
                      n.adapt=100, thin=1, trace=1)
plot(mcmc_run, xlim=c(0, 20), parameters="mean")
plot(mcmc_run, xlim=c(0, 10), parameters="sd")
plot(x=mcmc_run, what="MarkovChain", ylim=c(0, 15), parameters="mean", 
     col.posterior = colorRampPalette(c("blue", "grey"), alpha = 0.001)
                     (mcmc_run$parametersMCMC$n.chains))
plot(mcmc_run, what="MarkovChain", ylim=c(0, 10), parameters="sd", 
     col.posterior = colorRampPalette(c("blue", "grey"), alpha = 0.001)
                     (mcmc_run$parametersMCMC$n.chains))
plot(mcmc_run, what="LnL", 
     col.posterior = colorRampPalette(c("blue", "grey"), alpha = 0.001)
                     (mcmc_run$parametersMCMC$n.chains))
mcmcforcoda <- as.mcmc(mcmc_run)
heidel.diag(mcmcforcoda)
raftery.diag(mcmcforcoda)
autocorr.diag(mcmcforcoda)
acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
batchSE(mcmcforcoda, batchSize=100)
# The batch standard error procedure is usually thought to 
# be not as accurate as the time series methods used in summary
summary(mcmcforcoda)$statistics[,"Time-series SE"]
summary(mcmc_run)
as.parameters(mcmc_run)
lastp <- as.parameters(mcmc_run, index="last")
parameters_mcmc[,"Init"] <- lastp
# The n.adapt set to 1 is used to not record the first set of parameters
# then it is not duplicated (as it is also the last one for 
# the object mcmc_run)
mcmc_run2 <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=x, 
adaptive = TRUE,
likelihood=dnormx, n.chains=1, n.adapt=1, thin=1, trace=1)
mcmc_run3 <- merge(mcmc_run, mcmc_run2)
####### no adaptation, n.adapt must be 0
parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
mcmc_run3 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, data=x, 
adaptive = TRUE,
likelihood=dnormx, n.chains=1, n.adapt=0, thin=1, trace=1)

#########################
## Example with transform
#########################

x.1<-rnorm(6000, 2.4, 0.6)
x.2<-rlnorm(10000, 1.3,0.1)

X<-c(x.1, x.2)
hist(X,100,freq=FALSE, ylim=c(0,1.5))
Lnormlnorm <- function(par, val) {
  p <- invlogit(par["p"])
  return(-sum(log(p*dnorm(val, par["m1"], abs(par["s1"]), log = FALSE)+
                    (1-p)*dlnorm(val, par["m2"], abs(par["s2"]), log = FALSE))))
}
# Mean 1
m1=2.3; s1=0.5
# Mean 2
m2=1.3; s2=0.1
# proportion of category 1 - logit transform
p=0

par<-c(m1=m1, s1=s1, m2=m2, s2=s2, p=p)

result2<-optim(par, Lnormlnorm, method="BFGS", val=X, 
              hessian=FALSE, control=list(trace=1))
              
lines(seq(from=0, to=5, length=100), 
dnorm(seq(from=0, to=5, length=100), 
      result2$par["m1"], abs(result2$par["s1"])), col="red")

lines(seq(from=0, to=5, length=100), 
      dlnorm(seq(from=0, to=5, length=100), 
             result2$par["m2"], abs(result2$par["s2"])), col="green")

p <- invlogit(result2$par["p"])

paste("Proportion of Gaussian data",  p)

lines(seq(from=0, to=5, length=100), 
      p*dnorm(seq(from=0, to=5, length=100), 
              result2$par["m1"], result2$par["s1"])+
        (1-p)*dlnorm(seq(from=0, to=5, length=100), 
                     result2$par["m2"], result2$par["s2"]), col="blue")               

parameters_mcmc <- data.frame(Density=c('dunif', 'dunif', 'dunif', 'dunif', 'dunif'), 
                                        Prior1=c(0, 0.001, 0, 0.001, -3), 
                                        Prior2=c(10, 10, 10, 10, 3), 
                                        SDProp=c(1, 1, 1, 1, 1), 
                                        Min=c(0, 0.001, 0, 0.001, -3), 
                                        Max=c(10, 10, 10, 10, 3), 
                                        Init=result2$par, stringsAsFactors = FALSE, 
                                        row.names=c('m1', 's1', 'm2', 's2', 'p'))
                                        
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, val=X, 
                      parameters_name = "par",
                      adaptive = TRUE,
                      likelihood=Lnormlnorm, n.chains=1, 
                      n.adapt=100, thin=1, trace=100)
plot(mcmc_run, parameters="m1", breaks=seq(from=0, to =10, by=0.1), 
     legend=c(x=6, y=3.10))
plot(mcmc_run, parameters="p", transform=invlogit, xlim=c(0,1), 
     breaks=seq(from=0, to=1, by=0.01), legend=c(x=0.6, y=10))
plot(mcmc_run, parameters="p", xlim=c(-3,3), 
     breaks=seq(from=-3, to =3, by=0.05), legend=c(x=1, y= 3.10))
plot(mcmc_run, parameters="p", what="MarkovChain")
plot(mcmc_run, parameters="p", transform=invlogit, what="MarkovChain", 
     ylim=c(0, 1))
plot(mcmc_run, what="lnl", col.posterior = "black")
     
parameters_mcmc <- data.frame(Density=c('dunif', 'dunif', 'dunif', 'dunif', 'dnorm'), 
                                        Prior1=c(0, 0.001, 0, 0.001, 0.5), 
                                        Prior2=c(10, 10, 10, 10, 1), 
                                        SDProp=c(1, 1, 1, 1, 1), 
                                        Min=c(0, 0.001, 0, 0.001, -3), 
                                        Max=c(10, 10, 10, 10, 3), 
                                        Init=result2$par, stringsAsFactors = FALSE, 
                                        row.names=c('m1', 's1', 'm2', 's2', 'p'))
                                        
mcmc_run_pnorm <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, val=X, 
                      parameters_name = "par",
                      adaptive = TRUE,
                      likelihood=Lnormlnorm, n.chains=1, 
                      n.adapt=100, thin=1, trace=100)
plot(mcmc_run_pnorm, parameters="m1", breaks=seq(from=0, to =10, by=0.1), 
     legend=c(x=6, y=0.10))
plot(mcmc_run_pnorm, parameters="p", transform=invlogit, xlim=c(0,1), 
     breaks=seq(from=0, to=1, by=0.01), legend=c(x=0.6, y=0.10))
plot(x=mcmc_run_pnorm, parameters="p", xlim=c(-3,3), 
     breaks=seq(from=-3, to =3, by=0.05), legend=c(x=1, y= 0.10))
     
     
# Note that it is more logic to use beta distribution for p as a  
# proportion. However p value must be checked to be used in optim
# The use of logit transform can be a problem because it can stuck 
# the p value to 1 or 0 during fit.

Lnormlnorm <- function(par, val) {
  p <- par["p"]
  return(-sum(log(p*dnorm(val, par["m1"], abs(par["s1"]), log = FALSE)+
                    (1-p)*dlnorm(val, par["m2"], abs(par["s2"]), log = FALSE))))
}

# Example of beta distribution

# Mean is alpha/(alpha+beta)
# Variance is (alpha*beta)/((alpha+beta)^2*(alpha+beta+1))
alpha = 5
beta = 9
plot(x = seq(0.0001, 1, by = .0001), 
    y = dbeta(seq(0.0001, 1, by = .0001), alpha, beta),
    type = "l", ylab="Density", xlab="p", bty="n")
points(x=alpha/(alpha+beta), y=0, pch=4)
segments(x0=alpha/(alpha+beta)-sqrt((alpha*beta)/((alpha+beta)^2*(alpha+beta+1))), 
        x1=alpha/(alpha+beta)+sqrt((alpha*beta)/((alpha+beta)^2*(alpha+beta+1))),
        y0=0, y1=0)

# Use of optim with L-BFGS-B to limit p between 0 and 1 and s > 0

# Mean 1
m1=2.3; s1=0.5
# Mean 2
m2=1.3; s2=0.1
# proportion of category 1 - logit transform
p=0.5

par <- c(m1=m1, s1=s1, m2=m2, s2=s2, p=p)

result2 <- optim(par, Lnormlnorm, method="L-BFGS-B", val=X, 
              lower = c(-Inf, 0, -Inf, 0, 0), 
              upper = c(Inf, Inf, Inf, Inf, 1),
              hessian=FALSE, control=list(trace=1))

parameters_mcmc <- data.frame(Density=c('dunif', 'dunif', 'dunif', 'dunif', 'dbeta'), 
                                        Prior1=c(0, 0.001, 0, 0.001, 5), 
                                        Prior2=c(10, 10, 10, 10, 9), 
                                        SDProp=c(1, 1, 1, 1, 1), 
                                        Min=c(0, 0.001, 0, 0.001, 0), 
                                        Max=c(10, 10, 10, 10, 1), 
                                        Init=c('m1' = 2.4, 
                                               's1' = 0.6, 
                                               'm2' = 1.3, 
                                               's2' = 0.1, 
                                               'p' = 0.5), stringsAsFactors = FALSE, 
                                        row.names=c('m1', 's1', 'm2', 's2', 'p'))
                                        
mcmc_run_pbeta <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, val=X, 
                      parameters_name = "par",
                      adaptive = TRUE,
                      likelihood=Lnormlnorm, n.chains=1, 
                      n.adapt=100, thin=1, trace=100)
plot(mcmc_run_pbeta, parameters="m1", breaks=seq(from=0, to =10, by=0.1), 
     legend=c(x=6, y=2.10))
plot(mcmc_run_pbeta, parameters="p", xlim=c(0,1), 
     breaks=seq(from=0, to=1, by=0.01), legend=c(x=0.6, y=6))


}
}
\seealso{
Other mcmcComposite functions:
\code{\link[=+.PriorsmcmcComposite]{+.PriorsmcmcComposite()}},
\code{\link[=MHalgoGen]{MHalgoGen()}},
\code{\link[=PPD]{PPD()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=as.parameters]{as.parameters()}},
\code{\link[=merge.mcmcComposite]{merge.mcmcComposite()}},
\code{\link[=plot.PriorsmcmcComposite]{plot.PriorsmcmcComposite()}},
\code{\link[=setPriors]{setPriors()}},
\code{\link[=setPriors1]{setPriors1()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{compare}
\alias{compare}
\title{Run a shiny application for basic functions of comparison}
\usage{
compare()
}
\value{
Nothing
}
\description{
Run a shiny application for basic functions of comparison.
}
\details{
compare runs a shiny application for basic functions of comparison
}
\examples{
\dontrun{
library(HelpersMG)
compare()
}
}
\references{
Girondot, M., Guillon, J.-M., 2018. The w-value: An alternative to t- and X2 tests. Journal of Biostatistics & Biometrics 1, 1-3.
}
\seealso{
Other w-value functions:
\code{\link[=contingencyTable.compare]{contingencyTable.compare()}},
\code{\link[=series.compare]{series.compare()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{w-value functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutter.R
\name{cutter}
\alias{cutter}
\title{Distribution of the fitted distribution without cut.}
\usage{
cutter(
  observations = stop("Observations must be provided"),
  observations.colname = "Observations",
  lower_detection_limit.colname = "LDL",
  upper_detection_limit.colname = "UDL",
  cut_method.colname = "Cut",
  par = NULL,
  lower_detection_limit = NULL,
  upper_detection_limit = NULL,
  cut_method = "censored",
  distribution = "gamma",
  n.mixture = 1,
  n.iter = 5000,
  n.adapt = 100,
  debug = FALSE,
  progress.bar = TRUE,
  priors = NULL,
  adaptive = TRUE,
  session = NULL
)
}
\arguments{
\item{observations}{The observations; see description}

\item{observations.colname}{If observations is a data.frame, the name of column with observations}

\item{lower_detection_limit.colname}{If observations is a data.frame, the name of column with lower detection limit}

\item{upper_detection_limit.colname}{If observations is a data.frame, the name of column with upper detection limit}

\item{cut_method.colname}{If observations is a data.frame, the name of column with cut method, being "censored" or "truncated"}

\item{par}{Initial values for parameters of distribution}

\item{lower_detection_limit}{Value for lower detection limit}

\item{upper_detection_limit}{Value for upper detection limit}

\item{cut_method}{Value for cut method, being "censored" or "truncated"}

\item{distribution}{Can be gamma, normal, weibull, lognormal, or generalized.gamma}

\item{n.mixture}{Number of distributions}

\item{n.iter}{Number of iteration for Bayesian MCMC and to estimate the goodness-of-fit}

\item{n.adapt}{Number of burn-in iterations Bayesian MCMC}

\item{debug}{If TRUE, show some information}

\item{progress.bar}{If TRUE, show a progress bar for MCMC}

\item{priors}{A dataframe with priors.}

\item{adaptive}{Should the adaptive methodologie for SDprop be used}

\item{session}{The session of a shiny process}
}
\value{
The parameters of distribution of values below or above the detection limit.
}
\description{
If observations is a data.frame, it can have 4 columns:\cr
A column for the measurements;\cr
A column for the lower detection limit;\cr
A column for the upper detection limit;\cr
A column for the truncated of censored nature of the data.\cr
The names of the different columns are in the observations.colname, lower_detection_limit.colname,
upper_detection_limit.colname and cut_method.colname.\cr
If lower_detection_limit.colname is NULL or if the column does not exist,
the data are supposed to not be left-cut
and if upper_detection_limit.colname is NULL or if the column does not exist,
the data are supposed to not be right-cut.\cr
If observations is a vector, then the parameters lower_detection_limit and/or upper_detection_limit
must be given. Then cut_method must be also provided.\cr
In abservations, -Inf must be used to indicate a value below the lower detection limit and +Inf must be used
for a value above the upper detection limit.\cr
Be careful: NA is used to represent a missing data and not a value below of above the detection limit.\cr
If lower_detection_limit, upper_detection_limit or cut_method are only one value, they are supposed to
be used for all the observations.\cr
Definitions for censored or truncated distribution vary, and the two terms are sometimes used interchangeably. Let the following data set:\cr
1 1.25 2 4 5\cr
\cr
Censoring: some observations will be censored, meaning that we only know that they are below (or above) some bound. This can for instance occur if we measure the concentration of a chemical in a water sample. If the concentration is too low, the laboratory equipment cannot detect the presence of the chemical. It may still be present though, so we only know that the concentration is below the laboratory's detection limit.\cr
\cr
If the detection limit is 1.5, so that observations that fall below this limit is censored, our example data set would become:\cr
<1.5 <1.5 2 4 5; that is, we don't know the actual values of the first two observations, but only that they are smaller than 1.5.\cr
\cr
Truncation: the process generating the data is such that it only is possible to observe outcomes above (or below) the truncation limit. This can for instance occur if measurements are taken using a detector which only is activated if the signals it detects are above a certain limit. There may be lots of weak incoming signals, but we can never tell using this detector.\cr
\cr
If the truncation limit is 1.5, our example data set would become: 2 4 5; and we would not know that there in fact were two signals which were not recorded.\cr
If n.iter is NULL, no Bayesian MCMC is performed but credible interval will not be available.
}
\details{
cutter returns the fitted distribution without cut
}
\examples{
\dontrun{
library(HelpersMG)
# _______________________________________________________________
# right censored distribution with gamma distribution
# _______________________________________________________________
# Detection limit
DL <- 100
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc>DL] <- +Inf
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, upper_detection_limit=DL, 
                           cut_method="censored")
result
plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10), col.mcmc=NULL)
plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# The same data seen as truncated data with gamma distribution
# _______________________________________________________________
obc <- obc[is.finite(obc)]
# search for the parameters the best fit these truncated data
result <- cutter(observations=obc, upper_detection_limit=DL, 
                           cut_method="truncated")
result
plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# left censored distribution with gamma distribution
# _______________________________________________________________
# Detection limit
DL <- 10
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc<DL] <- -Inf
# search for the parameters the best fit these truncated data
result <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored")
result
plot(result, xlim=c(0, 200), breaks=seq(from=0, to=300, by=10))
# _______________________________________________________________
# left censored distribution with mixture of gamma distribution
# _______________________________________________________________
#' # Detection limit
library(HelpersMG)
# Generate 200 random data from a gamma distribution
set.seed(1234)
obc <- c(rgamma(100, scale=10, shape=5), rgamma(100, scale=20, shape=10))
LDL <- 20
l <- seq(from=0, to=LDL, length.out=1001)
p <- pgamma(l, scale=10, shape=5)*0.5+pgamma(l, scale=20, shape=10)
deltal <- l[2]-l[1]
expected_LDL <- sum((l[-1]-deltal/2)*(p[-1]-p[-length(p)]))/sum((p[-1]-p[-length(p)]))
# remove the data below the detection limit
obc[obc<LDL] <- -Inf

UDL <- 300
l <- seq(from=UDL, to=1000, length.out=1001)
p <- pgamma(l, scale=10, shape=5)*0.5+pgamma(l, scale=20, shape=10)
deltal <- l[2]-l[1]
expected_UDL <- sum((l[-1]-deltal/2)*(p[-1]-p[-length(p)]))/sum((p[-1]-p[-length(p)]))
obc[obc>UDL] <- +Inf

# search for the parameters the best fit these truncated data
result1_gamma <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="gamma", 
                          cut_method="censored", n.iter=5000, debug=0)
result1_normal <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="normal", 
                          cut_method="censored", n.iter=5000)
result1_lognormal <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="lognormal", 
                          cut_method="censored", n.iter=5000)
result1_Weibull <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL,  
                          distribution="Weibull", 
                          cut_method="censored", n.iter=5000)
result1_generalized.gamma <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="generalized.gamma", 
                          cut_method="censored", n.iter=5000)
result2_gamma <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="gamma", 
                          n.mixture=2, 
                          cut_method="censored", n.iter=5000)
result2_normal <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="normal", 
                          n.mixture=2, 
                          cut_method="censored", n.iter=5000)
result2_lognormal <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="lognormal", 
                          n.mixture=2, 
                          cut_method="censored", n.iter=5000)
result2_Weibull <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="Weibull", 
                          n.mixture=2, 
                          cut_method="censored", n.iter=5000)
result2_generalized.gamma <- cutter(observations=obc, lower_detection_limit=LDL, 
                          upper_detection_limit = UDL, 
                          distribution="generalized.gamma", 
                          n.mixture=2, 
                          cut_method="censored", n.iter=5000)
                          
compare_AIC(nomixture.gamma=result1_gamma, 
             nomixture.normal=result1_normal, 
             nomixture.lognormal=result1_lognormal, 
             nomixture.Weibull=result1_Weibull, 
             nomixture.generalized.gamma=result1_generalized.gamma, 
             mixture.gamma=result2_gamma, 
             mixture.normal=result2_normal, 
             mixture.lognormal=result2_lognormal, 
             mixture.Weibull=result2_Weibull, 
             mixture.generalized.gamma=result2_generalized.gamma)
             
plot(result2_gamma, xlim=c(0, 600), breaks=seq(from=0, to=600, by=10))
plot(result2_generalized.gamma, xlim=c(0, 600), breaks=seq(from=0, to=600, by=10))

# _______________________________________________________________
# left and right censored distribution
# _______________________________________________________________
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# Detection limit
LDL <- 10
# remove the data below the detection limit
obc[obc<LDL] <- -Inf
# Detection limit
UDL <- 100
# remove the data below the detection limit
obc[obc>UDL] <- +Inf
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, lower_detection_limit=LDL, 
                           upper_detection_limit=UDL, 
                          cut_method="censored")
result
plot(result, xlim=c(0, 150), col.DL=c("black", "grey"), 
                             col.unobserved=c("green", "blue"), 
     breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# Example with two values for lower detection limits
# corresponding at two different methods of detection for example
# with gamma distribution
# _______________________________________________________________
obc <- rgamma(50, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL1 <- 10
# remove the data below the detection limit
obc[obc<LDL1] <- -Inf
obc2 <- rgamma(50, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL2 <- 20
# remove the data below the detection limit
obc2[obc2<LDL2] <- -Inf
obc <- c(obc, obc2)
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, 
                           lower_detection_limit=c(rep(LDL1, 50), rep(LDL2, 50)), 
                          cut_method="censored")
result
# It is difficult to choose the best set of colors
plot(result, xlim=c(0, 150), col.dist="red", 
     col.unobserved=c(rgb(red=1, green=0, blue=0, alpha=0.1), 
                      rgb(red=1, green=0, blue=0, alpha=0.2)), 
     col.DL=c(rgb(red=0, green=0, blue=1, alpha=0.5), 
                      rgb(red=0, green=0, blue=1, alpha=0.9)), 
     breaks=seq(from=0, to=200, by=10))
     
# ________________________________________________________________________
# left censored distribution comparison of normal, lognormal, 
# weibull, generalized gamma, and gamma without Bayesian MCMC
# Comparison with Akaike Information Criterion
# ________________________________________________________________________
# Detection limit
DL <- 10
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc<DL] <- -Inf

result_gamma <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="gamma", 
                          n.iter=NULL)
plot(result_gamma)

result_lognormal <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="lognormal", 
                          n.iter=NULL)
plot(result_lognormal)

result_weibull <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="weibull", 
                          n.iter=NULL)
plot(result_weibull)

result_normal <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="normal", 
                          n.iter=NULL)
plot(result_normal)

result_generalized.gamma <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="generalized.gamma", 
                          n.iter=NULL)
plot(result_generalized.gamma)

compare_AIC(gamma=result_gamma, 
            lognormal=result_lognormal, 
            normal=result_normal, 
            Weibull=result_weibull, 
            Generalized.gamma=result_generalized.gamma)

# ______________________________________________________________________________
# left censored distribution comparison of normal, lognormal, 
# weibull, generalized gamma, and gamma
# ______________________________________________________________________________
# Detection limit
DL <- 10
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc<DL] <- -Inf
# search for the parameters the best fit these truncated data
result_gamma <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="gamma")
result_gamma
plot(result_gamma, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

result_lognormal <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="lognormal")
result_lognormal
plot(result_lognormal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

result_weibull <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="weibull")
result_weibull
plot(result_weibull, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

result_normal <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="normal")
result_normal
plot(result_normal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

result_generalized.gamma <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="generalized.gamma")
result_generalized.gamma
plot(result_generalized.gamma, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

# ___________________________________________________________________
# Test for similarity in gamma left censored distribution between two
# datasets
# ___________________________________________________________________
obc1 <- rgamma(100, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL <- 10
# remove the data below the detection limit
obc1[obc1<LDL] <- -Inf
obc2 <- rgamma(100, scale=10, shape=2)
# remove the data below the detection limit
obc2[obc2<LDL] <- -Inf
# search for the parameters the best fit these censored data
result1 <- cutter(observations=obc1, 
                  distribution="gamma", 
                  lower_detection_limit=LDL, 
                  cut_method="censored", n.iter=NULL)
logLik(result1)
plot(result1, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
result2 <- cutter(observations=obc2, 
                  distribution="gamma", 
                  lower_detection_limit=LDL, 
                  cut_method="censored", n.iter=NULL)
logLik(result2)
plot(result2, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
result_totl <- cutter(observations=c(obc1, obc2), 
                      distribution="gamma", 
                      lower_detection_limit=LDL, 
                      cut_method="censored", n.iter=NULL)
logLik(result_totl)
plot(result_totl, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
     
compare_AIC(Separate=list(result1, result2), 
            Common=result_totl, factor.value=1)
compare_BIC(Separate=list(result1, result2), 
            Common=result_totl, factor.value=1)           
}
}
\seealso{
Other Distributions:
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/newmap.scale.R
\name{newmap.scale}
\alias{newmap.scale}
\title{Add Scale to Existing Unprojected Map}
\usage{
newmap.scale(
  x,
  y,
  relwidth = 0.15,
  metric = TRUE,
  ratio = TRUE,
  col.line = "black",
  ...
)
}
\arguments{
\item{x}{Location of left end of distance gauge.}

\item{y}{Location of left end of distance gauge.}

\item{relwidth}{Proportion of width of display to be used for the scale. The default is 0.15.}

\item{metric}{If TRUE, the distance gauge will be in km, otherwise miles.}

\item{ratio}{If FALSE, the scale ratio of the map is not displayed.}

\item{col.line}{The color of lines for the gauge.}

\item{...}{Further plotting parameters may be specified as for the command text().}
}
\value{
The exact calculated scale is returned.
}
\description{
Adds a scale to an existing map, both as a ratio and a distance gauge.
If x or y are not specified, this will be taken to be near the lower left corner of the map.
}
\details{
newmap.scale Add Scale to Existing Unprojected Map
}
\examples{
\dontrun{
library("maps")
library("HelpersMG")
map("world", "China")
newmap.scale(col.line = "red", col="blue")
}
}
\author{
See \code{\link[maps:map.scale]{maps::map.scale()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dbeta_new.R
\name{dbeta_new}
\alias{dbeta_new}
\title{Density for the Beta distributions.}
\usage{
dbeta_new(
  x,
  mu = NULL,
  v = NULL,
  shape1,
  shape2,
  ncp = 0,
  log = FALSE,
  silent = FALSE
)
}
\arguments{
\item{x}{vector of quantiles.}

\item{mu}{mean of the Beta distribution.}

\item{v}{variance of the Beta distribution.}

\item{shape1}{non-negative parameters of the Beta distribution.}

\item{shape2}{non-negative parameters of the Beta distribution.}

\item{ncp}{non-centrality parameter.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}

\item{silent}{If FALSE, show the shape1 and shape 2 values.}
}
\value{
dbeta_new gives the density for the Beta distributions
}
\description{
Density for the Beta distribution with parameters mu and v or shape1 and
shape2 (and optional non-centrality parameter ncp).\cr
The returned object has three attributes:\cr
shape1, shape2, and ncp\cr
Note that if x has other attributes, they are preserved.
}
\details{
dbeta_new returns the density for the Beta distributions

The Beta distribution with parameters shape1 = a and shape2 = b has density \cr
gamma(a+b)/(gamma(a)gamma(b))x^(a-1)(1-x)^(b-1)\cr
for a > 0, b > 0 and 0 <= x <= 1 where the boundary values at x=0 or x=1 are defined as by
continuity (as limits).\cr
The mean is a/(a+b) and the variance is ab/((a+b)^2 (a+b+1)). These moments and all
distributional properties can be defined as limits.
}
\examples{
pi <- rbeta(100, shape1=0.48, shape2=0.12)
hist(pi, freq=FALSE, breaks=seq(from=0, to=1, by=0.1), ylim=c(0, 8), las=1)
library("HelpersMG")
mx <- ScalePreviousPlot()$ylim["end"]/
      max(dbeta_new(seq(from=0.01, to=0.99, by=0.01), mu = 0.8, v=0.1))
curve(dbeta_new(x, mu = 0.8, v=0.1)*mx, add=TRUE, col="red")
out <- dbeta_new(0.1, mu = 0.8, v=0.1)
out
attributes(out)$shape1; attributes(out)$shape2; attributes(out)$ncp
dbeta(0.1, shape1=attributes(out)$shape1, shape2=attributes(out)$shape2, 
      ncp=attributes(out)$ncp)

# It can be used to generate random numbers using mu and v
out <- dbeta_new(0.1, mu = 0.8, v=0.1, silent=TRUE)
pi <- rbeta(100, shape1=attributes(out)$shape1, shape2=attributes(out)$shape2, 
      ncp=attributes(out)$ncp)
hist(pi, freq=FALSE, breaks=seq(from=0, to=1, by=0.1), ylim=c(0, 8), las=1)
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_ncdf.R
\name{format_ncdf}
\alias{format_ncdf}
\title{Return an array with ncdf data}
\usage{
format_ncdf(
  ncdf,
  label.latitude = "latitude",
  label.longitude = "longitude",
  label.time = "time",
  varid = NULL,
  longitude1 = NA,
  latitude1 = NA,
  longitude2 = NA,
  latitude2 = NA,
  package = "ncdf4",
  bathy = TRUE
)
}
\arguments{
\item{ncdf}{An object read from package ncdf4 or a file name of ncdf file}

\item{label.latitude}{Label of latitude}

\item{label.longitude}{Label of longitude}

\item{label.time}{Label of time}

\item{varid}{Name of variable to extract}

\item{longitude1}{Longitude for first corner}

\item{latitude1}{latitude for first corner}

\item{longitude2}{Longitude for second corner}

\item{latitude2}{latitude for second corner}

\item{package}{If ncdf is a file, give the package to use to open the file}

\item{bathy}{If TRUE, return a bathy object}
}
\value{
A list with two element: data is an array and time is the POSIX.lt time
}
\description{
Return a list with two elements: data is an array and time is the POSIX.lt time.\cr
Or if label.time is NULL or if bathy is TRUE, a bathy object.\cr
If varid is NULL, it shows the available variable and dimensions of the file.\cr
Bathymetry data can be download here: \cr
https://www.gebco.net/data_and_products/gridded_bathymetry_data/#global
}
\details{
format_ncdf is used extract information from ncdf file
}
\examples{
\dontrun{
url <- "https://downloads.psl.noaa.gov/Datasets/noaa.oisst.v2.highres/"
url <- paste0(url, "sst.day.mean.2012.v2.nc")
dest <- paste(Sys.getenv("HOME"), "/sst.day.mean.2012.v2.nc", sep="")
download.file(url, dest)
format_ncdf(dest)
}
}
\seealso{
Other ncdf:
\code{\link[=ind_long_lat]{ind_long_lat()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{ncdf}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcutter.R
\name{rcutter}
\alias{rcutter}
\title{Random values of unobserved values of cut distribution.}
\usage{
rcutter(
  cutter = stop("A result of cutter() must be provided"),
  n = 1,
  lower_detection_limit = NULL,
  upper_detection_limit = NULL,
  method_cut = c("censored", "truncated"),
  observed_detection_limit = FALSE,
  random_method = c("medianMCMC", "MCMC", "ML"),
  index_mcmc = NULL
)
}
\arguments{
\item{cutter}{The fitted model obtained with cutter()}

\item{n}{number of random numbers}

\item{lower_detection_limit}{The lower detection limit}

\item{upper_detection_limit}{The upper detection limit}

\item{method_cut}{What method is used to cut the distribution: "censored", "truncated"?}

\item{observed_detection_limit}{If TRUE, will use the pattern of detection limit as in observations}

\item{random_method}{How to get parameters; it can be "ML", "medianMCMC", or "MCMC"}

\item{index_mcmc}{For MCMC random_method, the index of data to be used.}
}
\value{
A vector with the random numbers.
}
\description{
Return n random numbers.\cr
It can be used to get the posterior predictive distribution; see example.\cr
If random_method is "ML", the parameter values obtained using maximum likelihood are used.\cr
If random_method is "medianMCMC", the parameter values obtained using median of posterior distribution are used.\cr
If random_method is "MCMC", the parameter values are one sample of the MCMC posterior distribution.\cr
if observed_detection_limit is set to TRUE, the number of random number is equal to the number of observations; n is not used.\cr
rcutter is the abbreviation for random-cutter.
}
\details{
rcutter returns random values based on fitted distribution with cut.
}
\examples{
\dontrun{
library(HelpersMG)
# _______________________________________________________________
# right censored distribution with gamma distribution
# _______________________________________________________________
# Detection limit
DL <- 100
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc>DL] <- +Inf
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, upper_detection_limit=DL, 
                           cut_method="censored")
result
# Posterior predictive distribution
r <- rcutter(cutter=result, upper_detection_limit=DL, n=100)
hist(r)
# _______________________________________________________________
# left censored distribution with gamma distribution
# _______________________________________________________________
# Detection limit
DL <- 10
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc<DL] <- -Inf
# search for the parameters the best fit these truncated data
result <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored")
result
plot(result, breaks=seq(from=0, to=200, by=10))
r <- rcutter(cutter=result, n=100)
hist(r, breaks=seq(from=0, to=200, by=10))
r <- rcutter(cutter=result, lower_detection_limit=DL, n=100)
hist(r, breaks=seq(from=0, to=250, by=10))
# With censored method, some values are replaced with +Inf or -Inf
any(is.infinite(r))
r <- rcutter(cutter=result, upper_detection_limit=DL, n=100, 
             method_cut="truncated")
# With truncated method, the values below LDL or upper UDL are not present
any(is.infinite(r))
hist(r, breaks=seq(from=0, to=10, by=0.25))
r <- rcutter(cutter=result, observed_detection_limit=TRUE)
hist(r, breaks=seq(from=0, to=300, by=10))
}
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.cutter.R
\name{plot.cutter}
\alias{plot.cutter}
\title{Plot results of cutter that best describe distribution}
\usage{
\method{plot}{cutter}(
  x,
  col.hist = "grey",
  col.DL = "blue",
  col.dist = "black",
  col.unobserved = "green",
  col.mcmc = rgb(red = 0.6, green = 0, blue = 0, alpha = 0.01),
  legend = TRUE,
  show.DL = TRUE,
  show.plot = TRUE,
  set = NULL,
  ...
)
}
\arguments{
\item{x}{A result file generated by cutter}

\item{col.hist}{The color of histogram}

\item{col.DL}{The color of below of above samples}

\item{col.dist}{The color of distribution}

\item{col.unobserved}{The color of unobserved states}

\item{col.mcmc}{The color of mcmc outputs}

\item{legend}{If TRUE, a legend is shown}

\item{show.DL}{If TRUE, the limits of DL are shown}

\item{show.plot}{If FALSE, no plot is shown}

\item{set}{In case of mixture, will show and return only this set}

\item{...}{Parameters for plot}
}
\value{
The matrix of all the mcmc curves with invisible state.
}
\description{
Plot the estimates of cut distribution.
}
\details{
plot.cutter plot result of cutter
}
\examples{
\dontrun{
library(HelpersMG)
# _______________________________________________________________
# right censored distribution with gamma distribution
# _______________________________________________________________
# Detection limit
DL <- 100
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc>DL] <- +Inf
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, upper_detection_limit=DL, 
                           cut_method="censored")
result
plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# The same data seen as truncated data with gamma distribution
# _______________________________________________________________
obc <- obc[is.finite(obc)]
# search for the parameters the best fit these truncated data
result <- cutter(observations=obc, upper_detection_limit=DL, 
                           cut_method="truncated")
result
plot(result, xlim=c(0, 150), breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# left censored distribution with gamma distribution
# _______________________________________________________________
# Detection limit
DL <- 10
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc<DL] <- -Inf
# search for the parameters the best fit these truncated data
result <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored")
result
plot(result)
plot(result, xlim=c(0, 200), breaks=seq(from=0, to=200, by=10))
# _______________________________________________________________
# left and right censored distribution
# _______________________________________________________________
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# Detection limit
LDL <- 10
# remove the data below the detection limit
obc[obc<LDL] <- -Inf
# Detection limit
UDL <- 100
# remove the data below the detection limit
obc[obc>UDL] <- +Inf
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, lower_detection_limit=LDL, 
                           upper_detection_limit=UDL, 
                          cut_method="censored")
result
plot(result, xlim=c(0, 150), col.DL=c("black", "grey"), 
                             col.unobserved=c("green", "blue"), 
     breaks=seq(from=0, to=150, by=10))
# _______________________________________________________________
# Example with two values for lower detection limits
# corresponding at two different methods of detection for example
# with gamma distribution
# _______________________________________________________________
obc <- rgamma(50, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL1 <- 10
# remove the data below the detection limit
obc[obc<LDL1] <- -Inf
obc2 <- rgamma(50, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL2 <- 20
# remove the data below the detection limit
obc2[obc2<LDL2] <- -Inf
obc <- c(obc, obc2)
# search for the parameters the best fit these censored data
result <- cutter(observations=obc, 
                           lower_detection_limit=c(rep(LDL1, 50), rep(LDL2, 50)), 
                          cut_method="censored")
result
# It is difficult to choose the best set of colors
plot(result, xlim=c(0, 150), col.dist="red", 
     col.unobserved=c(rgb(red=1, green=0, blue=0, alpha=0.1), 
                      rgb(red=1, green=0, blue=0, alpha=0.2)), 
     col.DL=c(rgb(red=0, green=0, blue=1, alpha=0.5), 
                      rgb(red=0, green=0, blue=1, alpha=0.9)), 
     breaks=seq(from=0, to=200, by=10))
# ___________________________________________________________________
# left censored distribution comparison of normal, lognormal and gamma
# ___________________________________________________________________
# Detection limit
DL <- 10
# Generate 100 random data from a gamma distribution
obc <- rgamma(100, scale=20, shape=2)
# remove the data below the detection limit
obc[obc<DL] <- -Inf
# search for the parameters the best fit these truncated data
result_gamma <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="gamma")
result_gamma
plot(result_gamma, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

result_lognormal <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="lognormal")
result_lognormal
plot(result_lognormal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

result_normal <- cutter(observations=obc, lower_detection_limit=DL, 
                          cut_method="censored", distribution="normal")
result_normal
plot(result_normal, xlim=c(0, 250), breaks=seq(from=0, to=250, by=10))

compare_AICc(gamma=result_gamma, 
            lognormal=result_lognormal, 
            normal=result_normal)
# ___________________________________________________________________
# Test for similarity in gamma left censored distribution between two
# datasets
# ___________________________________________________________________
obc1 <- rgamma(100, scale=20, shape=2)
# Detection limit for sample 1 to 50
LDL <- 10
# remove the data below the detection limit
obc1[obc1<LDL] <- -Inf
obc2 <- rgamma(100, scale=10, shape=2)
# remove the data below the detection limit
obc2[obc2<LDL] <- -Inf
# search for the parameters the best fit these censored data
result1 <- cutter(observations=obc1, 
                  distribution="gamma", 
                  lower_detection_limit=LDL, 
                  cut_method="censored")
logLik(result1)
plot(result1, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
result2 <- cutter(observations=obc2, 
                  distribution="gamma", 
                  lower_detection_limit=LDL, 
                  cut_method="censored")
logLik(result2)
plot(result2, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
result_totl <- cutter(observations=c(obc1, obc2), 
                      distribution="gamma", 
                      lower_detection_limit=LDL, 
                      cut_method="censored")
logLik(result_totl)
plot(result_totl, xlim=c(0, 200), 
     breaks=seq(from=0, to=200, by=10))
     
compare_AICc(Separate=list(result1, result2), 
            Common=result_totl, factor.value=1)
compare_BIC(Separate=list(result1, result2), 
            Common=result_totl, factor.value=1)           
}
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=dnbinom_new]{dnbinom_new()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_folder.R
\name{read_folder}
\alias{read_folder}
\title{Read files present in a folder and creates a list with the content of these files}
\usage{
read_folder(
  folder = try(file.choose(), silent = TRUE),
  file = NULL,
  wildcard = "*.*",
  read = read.delim,
  ...
)
}
\arguments{
\item{folder}{Where to search for files; can be or a file path or a folder path}

\item{file}{list of files}

\item{wildcard}{Define which files are to be read (examples: "\emph{.}", "\emph{.xls", "essai}.txt"). It can be also a vector with all filenames.}

\item{read}{Function used to read file. Ex: read.delim or read.xls from gdata package}

\item{...}{Parameters send to the read function}
}
\value{
Return a list with the data in the files of the folder (directory for windows users)
}
\description{
To create a list, the syntax is:\cr
datalist <- read_folder(folder=".", read=read.delim, header=FALSE)\cr
It returns an error if the folder does not exist.\cr
The names of the elements of the list are the filenames.\cr
The parameter file can be used to predefine a list of file. If file is NULL, all the files of the folder/directory are used.
}
\details{
read_folder reads all files present in a folder
}
\examples{
\dontrun{
library(HelpersMG)
# Read all the .csv files from the current folder/directory
contentaslist <- read_folder(folder=".", wildcard="*.csv", read=read.csv2)
# Read all the files from the current folder/directory
contentaslist <- read_folder(folder=".", wildcard="*.*", read=read.csv2)
# Read two files from the current folder/directory
files <- c("filename1.csv", "filename2.csv")
contentaslist <- read_folder(folder=".", wildcard=files, read=read.csv2)
# To concvert the list into a single dataframe:
mydf <- do.call("rbind", contentaslist)
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modeled.hist.R
\name{modeled.hist}
\alias{modeled.hist}
\title{Return the theoretical value for the histogram bar}
\usage{
modeled.hist(breaks, FUN, ..., sum = 1)
}
\arguments{
\item{breaks}{Vector with the breaks; it can be obtained directly from hist()}

\item{FUN}{Function to be used to integrate the density, ex. pnorm}

\item{...}{Parameters to be used by FUN}

\item{sum}{Total numbers in the histogram; 1 for emperical frequencies}
}
\value{
A list with x (the center of the bar) and y components
}
\description{
Return the theoretical value for the histogram bar based on
a model of distribution.
}
\details{
modeled.hist returns the theoretical value for the histogram bar based on a model of distribution.
}
\examples{
\dontrun{
n <- rnorm(100, mean=10, sd=2)
breaks <- 0:20
hist(n, breaks=breaks)

s <- modeled.hist(breaks=breaks, FUN=pnorm, mean=10, sd=2, sum=100)

points(s$x, s$y, pch=19)
lines(s$x, s$y)

n <- rlnorm(100, meanlog=2, sdlog=0.4)
b <- hist(n, ylim=c(0, 70))

s <- modeled.hist(breaks=b$breaks, FUN=plnorm, meanlog=2, sdlog=0.4, sum=100)

points(s$x, s$y, pch=19)
lines(s$x, s$y)
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index.periodic.R
\name{index.periodic}
\alias{index.periodic}
\title{Estimate indices in periodic timeseries based on anchored minimum and maximum}
\usage{
index.periodic(minmax, time = NULL, replicates = 100, progressbar = FALSE)
}
\arguments{
\item{minmax}{A data.frame returned by minmax.periodic}

\item{time}{The time at which produced the estimate}

\item{replicates}{Number of replicates to estimate SD}

\item{progressbar}{Does a progression bar must be shown}
}
\value{
A data.frame with a column time and a column index
}
\description{
Estimate indices in periodic timeseries based on anchored minimum and maximum.\cr
The data.frame minmax can be generated manually. It should have three columns (time, index, SD),
with all the successive minimum and maximum indices.\cr
It can be used with sun.info() to get the time of minimum and maximum air temperature or
with getTide() to reconstruct the sea level.
}
\details{
index.periodic estimate indices in periodic timeseries based on anchored minimum and maximum
}
\examples{
\dontrun{
# Generate a timeserie of time
time.obs <- NULL
for (i in 0:9) time.obs <- c(time.obs, c(0, 6, 12, 18)+i*24)
# For these time, generate a timeseries of temperatures
temp.obs <- rep(NA, length(time.obs))
temp.obs[3+(0:9)*4] <- rnorm(10, 25, 3)
temp.obs[1+(0:9)*4] <- rnorm(10, 10, 3)
for (i in 1:(length(time.obs)-1)) 
  if (is.na(temp.obs[i])) 
  temp.obs[i] <- mean(c(temp.obs[i-1], temp.obs[i+1]))
  if (is.na(temp.obs[length(time.obs)])) 
  temp.obs[length(time.obs)] <- temp.obs[length(time.obs)-1]/2
observed <- data.frame(time=time.obs, temperature=temp.obs)
# Search for the minimum and maximum values
r <- minmax.periodic(time.minmax.daily=c(Min=2, Max=15), 
observed=observed, period=24, colname.index="temperature")

# Estimate all the temperatures for these values
t <- index.periodic(minmax=r)

plot_errbar(x=t[,"time"], y=t[,"index"],
errbar.y=ifelse(is.na(t[,"sd"]), 0, 2*t[,"sd"]),
type="l", las=1, bty="n", errbar.y.polygon = TRUE, 
xlab="hours", ylab="Temperatures", ylim=c(0, 35), 
errbar.y.polygon.list = list(col="grey"))

plot_add(x=t[,"time"], y=t[,"index"], type="l")

plot_add(observed$time, observed$temperature, pch=19, cex=0.5)
}
}
\seealso{
Other Periodic patterns of indices:
\code{\link[=minmax.periodic]{minmax.periodic()}},
\code{\link[=moon.info]{moon.info()}},
\code{\link[=sun.info]{sun.info()}},
\code{\link[=tide.info]{tide.info()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Periodic patterns of indices}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MHalgoGen.R
\name{MHalgoGen}
\alias{MHalgoGen}
\title{Monte-Carlo Markov-chain with Metropolis-Hastings algorithm}
\usage{
MHalgoGen(
  likelihood = stop("A likelihood function must be supplied"),
  parameters = stop("Priors must be supplied"),
  ...,
  parameters_name = "x",
  n.iter = 10000,
  n.chains = 1,
  n.adapt = 100,
  thin = 30,
  trace = FALSE,
  traceML = FALSE,
  progress.bar.ini = NULL,
  progress.bar = NULL,
  adaptive = FALSE,
  adaptive.lag = 500,
  adaptive.fun = function(x) {
     ifelse(x > 0.234, 1.3, 0.7)
 },
  intermediate = NULL,
  filename = "intermediate.Rdata",
  previous = NULL,
  session = NULL,
  warn = FALSE,
  WAIC.out = FALSE,
  n.datapoints = NULL
)
}
\arguments{
\item{likelihood}{The function that returns -ln likelihood using data and parameters}

\item{parameters}{A data.frame with priors; see description and examples}

\item{...}{Parameters to be transmitted to likelihood function}

\item{parameters_name}{The name of the parameters in the likelihood function, default is "x".}

\item{n.iter}{Number of iterations for each chain}

\item{n.chains}{Number of chains}

\item{n.adapt}{Number of iteration to stabilize likelihood}

\item{thin}{Interval for thinning likelihoods}

\item{trace}{Or FALSE or period to show progress}

\item{traceML}{TRUE or FALSE to show ML}

\item{progress.bar.ini}{The command to initialize progress bar}

\item{progress.bar}{The command to run the progress bar}

\item{adaptive}{Should an adaptive process for SDProp be used}

\item{adaptive.lag}{Lag to analyze the SDProp value in an adaptive context}

\item{adaptive.fun}{Function used to change the SDProp}

\item{intermediate}{Or NULL of period to save intermediate result}

\item{filename}{Name of file in which intermediate results are saved}

\item{previous}{The content of the file in which intermediate results are saved}

\item{session}{The shiny session}

\item{warn}{If TRUE, show information for debug.}

\item{WAIC.out}{If TRUE matrix or array are stored to be used with loo or waic.}

\item{n.datapoints}{Number of datapoints when WAIC.out is TRUE.}
}
\value{
A mcmcComposite object with all characteristics of the model and mcmc run
}
\description{
The parameters must be stored in a data.frame with named rows for each parameter with the following columns:\cr
\itemize{
\item Density. The density function name, example \code{dnorm}, \code{dlnorm}, \code{dunif}, \code{dbeta}
\item Prior1. The first parameter to send to the \code{Density} function
\item Prior2. The second parameter to send to the \code{Density} function
\item Prior3. The third parameter to send to the \code{Density} function
\item SDProp. The standard error from new proposition value of this parameter
\item Min. The minimum value for this parameter
\item Max. The maximum value for this parameter
\item Init. The initial value for this parameter
}
This script has been deeply modified from a MCMC script provided by Olivier Martin (INRA, Paris-Grignon).\cr
The likelihood function must use a parameter named parameters_name for the named parameters.\cr
For adaptive mcmc, see:\cr
Rosenthal, J. S. 2011. Optimal Proposal Distributions and Adaptive MCMC. Pages 93-112 in S. Brooks, A. Gelman,
G. Jones, and X.-L. Meng, editors. MCMC Handbook. Chapman and Hall/CRC.\cr
}
\details{
MHalgoGen is a function to use mcmc with Metropolis-Hastings algorithm
}
\examples{
\dontrun{
library(HelpersMG)
require(coda)
val <- rnorm(30, 10, 2)
dnormx <- function(data, x) {
 data <- unlist(data)
 return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
}
parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(0.35, 0.2), 
Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
row.names=c('mean', 'sd'))
# Or simpler
parameters_mcmc <- setPriors1(Name="mean", Density="dnorm", Parameters=c(mean=10, sd=2), 
                            SDProp=0.35, Min=-3, Max=100, Init=10)
parameters_mcmc <- parameters_mcmc + setPriors1(Name="sd", Density="dlnorm", 
                            Parameters=c(meanlog=0.5, sdlog=0.5), 
                            SDProp=0.2, Min=0, Max=10, Init=2)
# Use of trace and traceML parameters
# trace=1 : Only one likelihood is printed
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
# trace=10 : 10 likelihoods are printed
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=10)
# trace=TRUE : all likelihoods are printed
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=TRUE)
# traceML=TRUE : values when likelihood is better are shown
mcmc_run <- MHalgoGen(n.iter=100, parameters=parameters_mcmc, data=val, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=TRUE, traceML=TRUE)
mcmc_run <- MHalgoGen(n.iter=100, parameters=parameters_mcmc, data=val, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=FALSE, traceML=TRUE)
# trace=FALSE : No likelihood is printed
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=FALSE)

plot(mcmc_run, xlim=c(0, 20))
plot(mcmc_run, xlim=c(0, 10), parameters="sd")
library(graphics)
library(fields)
# show a scatter plot of the result
x <- mcmc_run$resultMCMC[[1]][, 1]
y <- mcmc_run$resultMCMC[[1]][, 2]
marpre <- par(mar=c(4, 4, 2, 6)+0.4)
smoothScatter(x, y)
# show a scale
n <- matrix(0, ncol=128, nrow=128)
xrange <- range(x)
yrange <- range(y)
for (i in 1:length(x)) {
  posx <- 1+floor(127*(x[i]-xrange[1])/(xrange[2]-xrange[1]))
  posy <- 1+floor(127*(y[i]-yrange[1])/(yrange[2]-yrange[1]))
  n[posx, posy] <- n[posx, posy]+1
}
image.plot(legend.only=TRUE, zlim= c(0, max(n)), nlevel=128, 
 col=colorRampPalette(c("white", blues9))(128))
# Compare with a heatmap
x <- seq(from=8, to=12, by=0.2)
y <- seq(from=1, to=4, by=0.2)
df <- expand.grid(mean=x, sd=y)
df <- cbind(df, L=rep(0, length(nrow(df))))
for (i in 1:nrow(df)) df[i, "L"] <- -sum(dnorm(val, df[i, 1], df[i, 2], log = TRUE))
hm <- matrix(df[, "L"], nrow=length(x))
par(mar = marpre)
image.plot(x=x, y=y, z=hm, las=1)
# Diagnostic function from coda library
mcmcforcoda <- as.mcmc(mcmc_run)
#' heidel.diag(mcmcforcoda)
raftery.diag(mcmcforcoda)
autocorr.diag(mcmcforcoda)
acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
batchSE(mcmcforcoda, batchSize=100)
# The batch standard error procedure is usually thought to 
# be not as accurate as the time series methods used in summary
summary(mcmcforcoda)$statistics[,"Time-series SE"]
summary(mcmc_run)
as.parameters(mcmc_run)
lastp <- as.parameters(mcmc_run, index="last")
parameters_mcmc[,"Init"] <- lastp
# The n.adapt set to 1 is used to not record the first set of parameters
# then it is not duplicated (as it is also the last one for 
# the object mcmc_run)
mcmc_run2 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, x=x, data=val, 
likelihood=dnormx, n.chains=1, n.adapt=1, thin=1, trace=1)
mcmc_run3 <- merge(mcmc_run, mcmc_run2)
####### no adaptation, n.adapt must be 0
parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
mcmc_run3 <- MHalgoGen(n.iter=1000, parameters=parameters_mcmc, x=x, data=val, 
likelihood=dnormx, n.chains=1, n.adapt=0, thin=1, trace=1)
# Here is how to use adaptive mcmc
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, adaptive = FALSE, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
1-rejectionRate(as.mcmc(mcmc_run))
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, adaptive = TRUE,  
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
1-rejectionRate(as.mcmc(mcmc_run))
# To see the dynamics :
var <- "mean"
par(mar=c(4, 4, 1, 1)+0.4)
plot(1:nrow(mcmc_run$resultMCMC[[1]]), mcmc_run$resultMCMC[[1]][, var], type="l", 
       xlab="Iterations", ylab=var, bty="n", las=1)
plot(mcmc_run, what="Posterior", parameters="mean")
plot(mcmc_run, what="MarkovChain", parameters="mean", ylim=c(0, 20), legend="topleft")
plot(mcmc_run, what="MarkovChain", parameters="mean", legend="topleft")
# Exemple with a progress bar

val <- rnorm(30, 10, 2)
dnormx <- function(data, x) {
 data <- unlist(data)
 return(-sum(dnorm(data, mean=x['mean'], sd=x['sd'], log=TRUE)))
}
parameters_mcmc <- setPriors1(Name="mean", Density="dnorm", Parameters=c(mean=10, sd=2), 
                            SDProp=0.35, Min=-3, Max=100, Init=10)
parameters_mcmc <- parameters_mcmc + setPriors1(Name="sd", Density="dlnorm", 
                            Parameters=c(meanlog=0.5, sdlog=0.5), 
                            SDProp=0.2, Min=0, Max=10, Init=2)
# Set up the progress bar
mcmc_run <- MHalgoGen(n.iter=50000, parameters=parameters_mcmc, data=val, 
                      likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=FALSE, 
                       progress.bar.ini=function(n.iter) {
                                 assign("pb", txtProgressBar(min=0, max=n.iter, style=3), 
                                        env = parent.frame())}, 
        progress.bar=function(iter) {setTxtProgressBar(get("pb", envir = parent.frame()), iter)})
}
}
\seealso{
Other mcmcComposite functions:
\code{\link[=+.PriorsmcmcComposite]{+.PriorsmcmcComposite()}},
\code{\link[=PPD]{PPD()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=as.parameters]{as.parameters()}},
\code{\link[=merge.mcmcComposite]{merge.mcmcComposite()}},
\code{\link[=plot.PriorsmcmcComposite]{plot.PriorsmcmcComposite()}},
\code{\link[=plot.mcmcComposite]{plot.mcmcComposite()}},
\code{\link[=setPriors]{setPriors()}},
\code{\link[=setPriors1]{setPriors1()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dnbinom_new.R
\name{dnbinom_new}
\alias{dnbinom_new}
\title{Density for the negative binomial distribution - Alternative parametrization.}
\usage{
dnbinom_new(
  x,
  size = NULL,
  prob = NULL,
  mu = NULL,
  sd = NULL,
  var = NULL,
  log = FALSE
)
}
\arguments{
\item{x}{vector of (non-negative integer) quantiles.}

\item{size}{target for number of successful trials, or dispersion parameter (the shape parameter of the gamma mixing distribution). Must be strictly positive, need not be integer.}

\item{prob}{probability of success in each trial. 0 < prob <= 1.}

\item{mu}{alternative parametrization via mean.}

\item{sd}{alternative parametrization via standard deviation.}

\item{var}{alternative parametrization via variance.}

\item{log}{logical; if TRUE, probabilities p are given as log(p).}
}
\value{
Density for the negative binomial distribution - Alternative parametrization.
}
\description{
Density for the negative binomial distribution with parameters mu, sd, var, size or prob. See \code{dnbinom}.
}
\details{
dnbinom_new returns density for the negative binomial distribution
}
\examples{
\dontrun{
library("HelpersMG")
set.seed(1)
x <- rnbinom_new(n=100, mu=2, sd=3)
LnL <- NULL
df <- data.frame(mu=seq(from=0.1, to=8, by=0.1), "-LnL"=NA)
for (mu in df[, "mu"])
LnL <- c(LnL, -sum(dnbinom_new(x=x, mu=mu, sd=3, log=TRUE)))
df[, "-LnL"] <- LnL
ggplot(data = df, aes(x = .data[["mu"]], y = .data[["-LnL"]])) + geom_line()
# Examples of wrong parametrization
dnbinom_new(x=x, mu=c(1, 2), sd=3, log=TRUE)
}
}
\seealso{
Other Distributions:
\code{\link[=cutter]{cutter()}},
\code{\link[=dSnbinom]{dSnbinom()}},
\code{\link[=dbeta_new]{dbeta_new()}},
\code{\link[=dcutter]{dcutter()}},
\code{\link[=dggamma]{dggamma()}},
\code{\link[=logLik.cutter]{logLik.cutter()}},
\code{\link[=plot.cutter]{plot.cutter()}},
\code{\link[=print.cutter]{print.cutter()}},
\code{\link[=r2norm]{r2norm()}},
\code{\link[=rcutter]{rcutter()}},
\code{\link[=rmnorm]{rmnorm()}},
\code{\link[=rnbinom_new]{rnbinom_new()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iCutter.R
\name{iCutter}
\alias{iCutter}
\title{Run a shiny application to fit bone section}
\usage{
iCutter()
}
\value{
Nothing
}
\description{
Run a shiny application to fit bone section
}
\details{
BP runs a shiny application to fit bone section
}
\examples{
\dontrun{
# Not run:
library(HelpersMG)
iCutter()
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sun.info.R
\name{sun.info}
\alias{sun.info}
\title{Estimate the time of sunrise and sunset according to longitude, latitude and date}
\usage{
sun.info(date, latitude, longitude)
}
\arguments{
\item{date}{A vector with the time at which sun fates are needed}

\item{latitude}{The latitude at which estimate the sun fates}

\item{longitude}{The longitude at which estimate the sun fates}
}
\value{
A data.frame with information about daily sun
}
\description{
Estimate the sun fates according to latitude and date.\cr
Can be compared with the function \code{sunrise.set()} of package \code{StreamMetabolism}.
}
\details{
sun.info estimate the time of sunrise and sunset according to longitude, latitude and date
}
\examples{
\dontrun{
# Generate a timeserie of time
date <- seq(from=as.Date("2000-01-01"), to=as.Date("2000-12-31"), by="1 day")
plot(date, sun.info(date, latitude=23, longitude=0)$day.length, bty="n", 
 las=1, type="l", xlab="Ordinal days", ylab="Day length in hours")
plot(date, sun.info(date, latitude=23, longitude=0)$sunrise, bty="n", 
 las=1, type="l", xlab="Ordinal days", ylab="Sun rise in hours")
}
}
\references{
Teets, D.A. 2003. Predicting sunrise and sunset times. The College Mathematics Journal 34(4):317-321.
}
\seealso{
Other Periodic patterns of indices:
\code{\link[=index.periodic]{index.periodic()}},
\code{\link[=minmax.periodic]{minmax.periodic()}},
\code{\link[=moon.info]{moon.info()}},
\code{\link[=tide.info]{tide.info()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Periodic patterns of indices}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HelpersMG-package.R
\name{HelpersMG-package}
\alias{HelpersMG-package}
\title{Tools for Environmental Analyses, Ecotoxicology and Various R Functions}
\description{
Contains miscellaneous functions useful for managing \cr
'NetCDF' files (see http://en.wikipedia.org/wiki/NetCDF), \cr
get tide levels on any point of the globe, \cr
get moon phase and time for sun rise and fall, \cr
analyse and reconstruct daily time series of temperature \cr
with irregular sinusoidal pattern, \cr
show scales and wind rose in plot with change of color of text, \cr
Metropolis-Hastings algorithm for Bayesian MCMC analysis, \cr
plot graphs or boxplot with error bars, \cr
search files in disk by there names or their content, \cr
read the contents of all files from a folder at one time, \cr
calculate IC50 for ecotoxicological studies, \cr
calculate the probability mass function of the sum of negative binomial \cr
distributions, calculate distribution of unobserved values in censored or
truncated distributions.\cr
The latest version of this package can always be installed using:\cr
install.packages("http://marc.girondot.free.fr/CRAN/HelpersMG.tar.gz",
repos=NULL, type="source")\cr
\if{html}{\figure{TB.png}{options: alt="HelpersMG logo"}}
\if{latex}{\figure{TB.png}}
}
\details{
Helpers functions for several packages

\tabular{ll}{
Package: \tab HelpersMG\cr
Type: \tab Package\cr
Version: \tab 2026.8.24 build 1817\cr
Date: \tab 2026-08-24\cr
License: \tab GPL (>= 2)\cr
LazyLoad: \tab yes\cr
}
}
\examples{
\dontrun{
library(HelpersMG)
print('----------------------------------')
print('Examples for mcmcComposite objects')
print('----------------------------------')
require(coda)
x <- rnorm(30, 10, 2)
dnormx <- function(x, par) return(-sum(dnorm(x, mean=par['mean'], sd=par['sd'], log=TRUE)))
parameters_mcmc <- data.frame(Density=c('dnorm', 'dlnorm'), 
Prior1=c(10, 0.5), Prior2=c(2, 0.5), SDProp=c(0.35, 0.2), 
Min=c(-3, 0), Max=c(100, 10), Init=c(10, 2), stringsAsFactors = FALSE, 
row.names=c('mean', 'sd'))
mcmc_run <- MHalgoGen(n.iter=100000, parameters=parameters_mcmc, data=x, 
likelihood=dnormx, n.chains=1, n.adapt=100, thin=1, trace=1)
plot(mcmc_run, xlim=c(0, 20))
plot(mcmc_run, xlim=c(0, 10), parameters="sd")
mcmcforcoda <- as.mcmc(mcmc_run)
# Optimal rejection rate should be 0.234
rejectionRate(mcmcforcoda)
heidel.diag(mcmcforcoda)
raftery.diag(mcmcforcoda)
autocorr.diag(mcmcforcoda)
acf(mcmcforcoda[[1]][,"mean"], lag.max=20, bty="n", las=1)
acf(mcmcforcoda[[1]][,"sd"], lag.max=20, bty="n", las=1)
batchSE(mcmcforcoda, batchSize=100)
# The batch standard error procedure is usually thought to 
# be not as accurate as the time series methods used in summary
summary(mcmcforcoda)$statistics[,"Time-series SE"]
summary(mcmc_run)
as.parameters(mcmc_run)
lastp <- as.parameters(mcmc_run, index="last")
parameters_mcmc[,"Init"] <- lastp
# The n.adapt set to 1 is used to not record the first set of parameters
# then it is not duplicated (as it is also the last one for 
# the object mcmc_run)
mcmc_run2 <- MHalgoGen(n.iter=10000, parameters=parameters_mcmc, data=x, 
likelihood=dnormx, n.chains=1, n.adapt=1, thin=1, trace=1)
mcmc_run3 <- merge(mcmc_run, mcmc_run2)
####### no adaptation, n.adapt must be 0
parameters_mcmc[,"Init"] <- c(mean(x), sd(x))
mcmc_run3 <- MHalgoGen(n.iter=10000, parameters=parameters_mcmc, data=x, 
likelihood=dnormx, n.chains=1, n.adapt=0, thin=1, trace=1)
print('------------------------------------------')
print('Examples for Daily patterns of temperature')
print('------------------------------------------')
# Generate a timeserie of time
time.obs <- NULL
for (i in 0:9) time.obs <- c(time.obs, c(0, 6, 12, 18)+i*24)
# For these time, generate a timeseries of temperatures
temp.obs <- rep(NA, length(time.obs))
temp.obs[3+(0:9)*4] <- rnorm(10, 25, 3)
temp.obs[1+(0:9)*4] <- rnorm(10, 10, 3)
for (i in 1:(length(time.obs)-1)) 
  if (is.na(temp.obs[i])) 
  temp.obs[i] <- mean(c(temp.obs[i-1], temp.obs[i+1]))
  if (is.na(temp.obs[length(time.obs)])) 
  temp.obs[length(time.obs)] <- temp.obs[length(time.obs)-1]/2
observed <- data.frame(time=time.obs, temperature=temp.obs)
# Search for the minimum and maximum values
r <- minmax.periodic(time.minmax.daily=c(Min=2, Max=15), 
observed=observed, period=24)

# Estimate all the temperatures for these values
t <- temperature.periodic(minmax=r)

plot_errbar(x=t[,"time"], y=t[,"temperature"],
errbar.y=ifelse(is.na(t[,"sd"]), 0, 2*t[,"sd"]),
type="l", las=1, bty="n", errbar.y.polygon = TRUE, 
xlab="hours", ylab="Temperatures", ylim=c(0, 35), 
errbar.y.polygon.list = list(col="grey"))

plot_add(x=t[,"time"], y=t[,"temperature"], type="l")

# How many times this package has been download
library(cranlogs)
HelpersMG <- cran_downloads("HelpersMG", from = "2015-04-07", 
                            to = Sys.Date() - 1) 
sum(HelpersMG$count)
plot(HelpersMG$date, HelpersMG$count, type="l", bty="n")
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flexit.R
\name{flexit}
\alias{flexit}
\title{Return the flexit}
\usage{
flexit(
  x,
  par = NULL,
  P = NULL,
  S = NULL,
  K1 = NULL,
  K2 = NULL,
  Min = 0,
  Max = 1,
  zero = 1e-09,
  error0 = 0,
  error1 = 1
)
}
\arguments{
\item{x}{The values at which the flexit model must be calculated}

\item{par}{The vector with P, S, K1, and K2 values}

\item{P}{P value}

\item{S}{S value}

\item{K1}{K1 value}

\item{K2}{K2 value}

\item{Min}{Min value for scaled flexit model}

\item{Max}{Max value for scaled flexit model}

\item{zero}{Value to replace zero}

\item{error0}{Value to return if an error is observed toward 0}

\item{error1}{Value to return if an error is observed toward 1}
}
\value{
A vector with the probabilities
}
\description{
Return a vector with the probabilities.
The flexit equation is published in:\cr
Abreu-Grobois, F.A., Morales-Mérida, B.A., Hart, C.E., Guillon, J.-M., Godfrey, M.H.,
Navarro, E. & Girondot, M. (2020) Recent advances on the estimation of the thermal
reaction norm for sex ratios. PeerJ, 8, e8451.\cr
If dose < P then \eqn{(1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}{(1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}\cr
If dose > P then \eqn{1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}{1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}\cr
with:\cr
\deqn{S1 = (2^(K1 - 1) * S * K1)/(2^K1 - 1)}{S1 = (2^(K1 - 1) * S * K1)/(2^K1 - 1)}
\deqn{S2 = (2^(K2 - 1) * S * K2)/(2^K2 - 1)}{S2 = (2^(K2 - 1) * S * K2)/(2^K2 - 1)}
\cr
If \eqn{2^K1}{2^K1} is too large to be estimated, the approximation \eqn{S1 = S*K1/2}{S1 = S*K1/2} is used.\cr
Demonstration:\cr
\deqn{S1 = (2^(K1 - 1) * S * K1)/(2^K1 - 1)}{S1 = (2^(K1 - 1) * S * K1)/(2^K1 - 1)}
\deqn{S1 = exp(log((2^(K1 - 1) * S * K1)/(2^K1 - 1)))}{S1 = exp(log((2^(K1 - 1) * S * K1)/(2^K1 - 1)))}
\deqn{S1 = exp(log(2^(K1 - 1)) + log(S * K1) - log(2^K1 - 1))}{S1 = exp(log(2^(K1 - 1)) + log(S * K1) - log(2^K1 - 1))}
When \eqn{K1}{K1} is very large, \eqn{2^K1 - 1 = 2^K1}{2^K1 - 1 = 2^K1} then
\deqn{S1 = exp((K1 - 1) * log(2) + log(S * K1) - K1 * log(2))}{S1 = exp((K1 - 1) * log(2) + log(S * K1) - K1 * log(2))}
\deqn{S1 = exp((K1 * log(2) - log(2) + log(S * K1) - K1 * log(2))}{S1 = exp((K1 * log(2) - log(2) + log(S * K1) - K1 * log(2))}
\deqn{S1 = exp(log(S * K1)- log(2))}{S1 = exp(log(S * K1)- log(2))}
\deqn{S1 = S * K1 / 2}{S1 = S * K1 / 2}
If \eqn{2^K2}{2^K2} is too large to be estimated, the approximation \eqn{S2 = S*K2/2}{S2 = S*K2/2} is used.\cr
If \eqn{(1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}{(1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)} is not finite,
the following approximation is used:\cr
\deqn{exp((-1/K1)*(K1*log(2)+(4*S1*(P-x))))}{exp((-1/K1)*(K1*log(2)+(4*S1*(P-x))))}
If \eqn{1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}{1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)} is not finite,
the following approximation is used:\cr
\deqn{1 - exp((-1/K2)*(K2*log(2)+(4*S2*(x - P))))}{1 - exp((-1/K2)*(K2*log(2)+(4*S2*(x - P))))}
}
\details{
Return the flexit value
}
\examples{
\dontrun{
n <- flexit(x=1:100, par=c(P=50, S=0.001, K1=0.01, K2=0.02))
n <- flexit(x=1:100, P=50, S=0.001, K1=0.01, K2=0.02)

1/(1+exp(0.01*4*(50-1:100)))
flexit(1:100, P=50, S=0.01, K1=1, K2=1)
}
}
\seealso{
Other logit:
\code{\link[=invlogit]{invlogit()}},
\code{\link[=logit]{logit()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{logit}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AddPriors.R
\name{+.PriorsmcmcComposite}
\alias{+.PriorsmcmcComposite}
\title{Set priors for MHalgoGen()}
\usage{
\method{+}{PriorsmcmcComposite}(prior1, prior2)
}
\arguments{
\item{prior1}{The first prior.}

\item{prior2}{The second prior.}
}
\value{
Return a data.frame with the formatted priors
}
\description{
Construct priors for MHalgoGen()\cr
A check is done to verify that parameter names are not duplicated.
}
\details{
+.PriorsmcmcComposite is a general function to set priors for MHalgoGen()
}
\examples{
\dontrun{
library(HelpersMG)
Priors <- setPriors1(Name="a0", Prior1=0, Prior2=1, Density="dunif", 
                     SDProp=0.1, Min=0, Max=1, Init=0.5)
Priors <- Priors + setPriors1(Name="a1", Prior1=0.5, Prior2=0.1, Density="dnorm", 
                     SDProp=0.05, Min=0, Max=1, Init=0.5)
# Using Parameters
Priors <- setPriors1(Name="a0", Parameters=c(min=0, max=1), Density="dunif", 
                     SDProp=0.1, Min=0, Max=1, Init=0.5)
Priors <- Priors + setPriors1(Name="a1", Parameters=c(mean=0.5, sd=0.1), Density="dnorm", 
                     SDProp=0.05, Min=0, Max=1, Init=0.5)
}
}
\seealso{
Other mcmcComposite functions:
\code{\link[=MHalgoGen]{MHalgoGen()}},
\code{\link[=PPD]{PPD()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=as.parameters]{as.parameters()}},
\code{\link[=merge.mcmcComposite]{merge.mcmcComposite()}},
\code{\link[=plot.PriorsmcmcComposite]{plot.PriorsmcmcComposite()}},
\code{\link[=plot.mcmcComposite]{plot.mcmcComposite()}},
\code{\link[=setPriors]{setPriors()}},
\code{\link[=setPriors1]{setPriors1()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LD50_MHmcmc.R
\name{LD50_MHmcmc}
\alias{LD50_MHmcmc}
\title{Metropolis-Hastings algorithm for LD50}
\usage{
LD50_MHmcmc(
  result = stop("A result of LD50() fit must be provided"),
  n.iter = 10000,
  parametersMCMC = NULL,
  n.chains = 1,
  n.adapt = 0,
  thin = 1,
  trace = FALSE,
  batchSize = sqrt(n.iter),
  adaptive = FALSE,
  adaptive.lag = 500,
  adaptive.fun = function(x) {
     ifelse(x > 0.234, 1.3, 0.7)
 },
  intermediate = NULL,
  filename = "intermediate.Rdata",
  previous = NULL
)
}
\arguments{
\item{result}{An object obtained after a SearchR fit}

\item{n.iter}{Number of iterations for each step}

\item{parametersMCMC}{A set of parameters used as initial point for searching with information on priors}

\item{n.chains}{Number of replicates}

\item{n.adapt}{Number of iterations before to store outputs}

\item{thin}{Number of iterations between each stored output}

\item{trace}{True or False, shows progress}

\item{batchSize}{Number of observations to include in each batch fo SE estimation}

\item{adaptive}{Should an adaptive process for SDProp be used}

\item{adaptive.lag}{Lag to analyze the SDProp value in an adaptive content}

\item{adaptive.fun}{Function used to change the SDProp}

\item{intermediate}{Period for saving intermediate result, NULL for no save}

\item{filename}{If intermediate is not NULL, save intermediate result in this file}

\item{previous}{Previous result to be continued. Can be the filename in which intermediate results are saved.}
}
\value{
A list with resultMCMC being mcmc.list object, resultLnL being likelihoods and parametersMCMC being the parameters used
}
\description{
Run the Metropolis-Hastings algorithm for tsd.\cr
Deeply modified from a MCMC script by Olivier Martin (INRA, Paris-Grignon).\cr
The number of iterations is n.iter+n.adapt+1 because the initial likelihood is also displayed.\cr
I recommend that thin=1 because the method to estimate SE uses resampling.\cr
If initial point is maximum likelihood, n.adapt = 0 is a good solution.\cr
To get the SE from result_mcmc <- tsd_MHmcmc(result=try), use:\cr
result_mcmc$BatchSE or result_mcmc$TimeSeriesSE\cr
The batch standard error procedure is usually thought to be not as accurate as the time series methods.\cr
Based on Jones, Haran, Caffo and Neath (2005), the batch size should be equal to sqrt(n.iter).\cr
Jones, G.L., Haran, M., Caffo, B.S. and Neath, R. (2006) Fixed Width Output Analysis for Markov chain Monte Carlo , Journal of the American Statistical Association, 101:1537-1547.\cr
coda package is necessary for this function.\cr
The parameters intermediate and filename are used to save intermediate results every 'intermediate' iterations (for example 1000). Results are saved in a file of name filename.\cr
The parameter previous is used to indicate the list that has been save using the parameters intermediate and filename. It permits to continue a mcmc search.\cr
These options are used to prevent the consequences of computer crash or if the run is very very long and processes at time limited.\cr
}
\details{
LD50_MHmcmc runs the Metropolis-Hastings algorithm for LD50 (Bayesian MCMC)
}
\examples{
\dontrun{
library("HelpersMG")
data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
Alive=c(10, 12, 8, 6, 2, 1),
Dead=c(0, 1, 5, 6, 9, 15))
LD50_logistic <- LD50(data, equation="logistic")
pMCMC <- LD50_MHmcmc_p(LD50_logistic, accept=TRUE)
# Take care, it can be very long
result_mcmc_LD50 <- LD50_MHmcmc(result=LD50_logistic, 
		parametersMCMC=pMCMC, n.iter=10000, n.chains = 1,  
		n.adapt = 0, thin=1, trace=1000, adaptive=TRUE, )
# summary() permits to get rapidly the standard errors for parameters
summary(result_mcmc_LD50)
plot(x=result_mcmc_LD50, parameters="S", scale.prior=TRUE, las=1)
plot(result_mcmc_LD50, parameters="S", scale.prior=TRUE, las=1, xlim=c(-20, 20))
plot(result_mcmc_LD50, parameters="P", scale.prior=TRUE, las=1)
1-rejectionRate(as.mcmc(result_mcmc_LD50))
raftery.diag(as.mcmc(result_mcmc_LD50))
heidel.diag(as.mcmc(result_mcmc_LD50))

#### Example with Uniforms priors

pMCMC <- structure(list(Density = c("dunif", "dunif"), 
Prior1 = c(77.6216005852911, -31.0438095277258), 
Prior2 = c(310.486402341165, 31.0438095277258), 
SDProp = c(2, 0.5), 
Min = c(77.6216005852911, -31.0438095277258), 
Max = c(310.486402341165, 31.0438095277258), 
Init = c(155.243201170582, -15.5219047638629)), 
row.names = c("P", "S"), class = "data.frame")
result_mcmc_LD50 <- LD50_MHmcmc(result=LD50_logistic, 
		parametersMCMC=pMCMC, n.iter=10000, n.chains = 1,  
		n.adapt = 0, thin=1, trace=1000, adaptive=TRUE, )
# summary() permits to get rapidly the standard errors for parameters
summary(result_mcmc_LD50)
plot(x=result_mcmc_LD50, parameters="S", scale.prior=TRUE, las=1)
plot(result_mcmc_LD50, parameters="S", scale.prior=TRUE, las=1, xlim=c(-40, 40))
plot(result_mcmc_LD50, parameters="P", scale.prior=TRUE, las=1)
1-rejectionRate(as.mcmc(result_mcmc_LD50))
raftery.diag(as.mcmc(result_mcmc_LD50))
heidel.diag(as.mcmc(result_mcmc_LD50))

}
}
\seealso{
Other LD50 functions:
\code{\link[=LD50]{LD50()}},
\code{\link[=LD50_MHmcmc_p]{LD50_MHmcmc_p()}},
\code{\link[=logLik.LD50]{logLik.LD50()}},
\code{\link[=plot.LD50]{plot.LD50()}},
\code{\link[=predict.LD50]{predict.LD50()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{LD50 functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractAIC.R
\name{ExtractAIC.glm}
\alias{ExtractAIC.glm}
\title{Return AIC, AICc or BIC from a glm object}
\usage{
ExtractAIC.glm(fit, scale = 0, k = 2, ...)
}
\arguments{
\item{fit}{fitted model, the result of a fitter glm.}

\item{scale}{unused for glm.}

\item{k}{numeric specifying the ‘weight’ of the equivalent degrees of freedom (=: edf) part in the AIC formula.}

\item{...}{further arguments (currently unused because addterm.glm and dropterm.glm using this function do not transmit them).}
}
\value{
A numeric named vector of length 2, with first and second elements giving\cr
edf	the ‘equivalent degrees of freedom’ for the fitted model fit.\cr
x	the Information Criterion for fit.
}
\description{
For glm fits the family's aic() function is used to compute the AIC.\cr
The choice between different criteria is done by setting a global option AIC. It can be checked using show.option=TRUE.
Indeed, it is not possible to use the ... parameter due to a bug in some functions of MASS package. If you want to use this function as a replacement for setpAIC(), do extractAIC.glm <- ExtractAIC.glm before.
}
\details{
ExtractAIC.glm returns AIC, AICc or BIC from a glm object
}
\examples{
\dontrun{
extractAIC.glm <- ExtractAIC.glm
n <- 100
x <- rnorm(n, 20, 2)
A <- rnorm(n, 20, 5)
g <- glm(x ~ A)
extractAIC(g, show.option=TRUE)
options(AIC="AIC")
extractAIC(g)
options(AIC="BIC")
extractAIC(g)
options(AIC="AICc")
extractAIC(g)
}
}
\seealso{
Other AIC:
\code{\link[=ELPDweight]{ELPDweight()}},
\code{\link[=FormatCompareAIC]{FormatCompareAIC()}},
\code{\link[=compare_AIC]{compare_AIC()}},
\code{\link[=compare_AICc]{compare_AICc()}},
\code{\link[=compare_BIC]{compare_BIC()}}
}
\author{
Modified from stats:::extract.AIC.glm
}
\concept{AIC}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setPriors1.R
\name{setPriors1}
\alias{setPriors1}
\title{Set priors for MHalgoGen()}
\usage{
setPriors1(
  Name = stop("A name of parameter is required."),
  Prior1 = NULL,
  Prior2 = NULL,
  Prior3 = NULL,
  Parameters = NULL,
  Density = stop("A density is required."),
  SDProp = stop("A SD of proposal is required."),
  Min = -Inf,
  Max = +Inf,
  Init = stop("An initial value is required.")
)
}
\arguments{
\item{Name}{Name of the parameter.}

\item{Prior1}{The value for the first parameter of the prior distribution.}

\item{Prior2}{The value for the second parameter of the prior distribution.}

\item{Prior3}{The value for the third parameter of the prior distribution.}

\item{Parameters}{A vector with parameters of the prior distribution}

\item{Density}{The density function of the prior (ex. "dunif", "dnorm", "dbeta").}

\item{SDProp}{The SD of the propositions for the Markov chain.}

\item{Min}{The minimum value to truncate the prior distribution.}

\item{Max}{The maximum value to truncate the prior distribution.}

\item{Init}{The value at the first iteration.}
}
\value{
Return a data.frame with the formatted prior
}
\description{
Set priors for MHalgoGen()\cr
A check is done to verify that the Init value is well between the Min and Max range.\cr
The order of Prior1, 2, and 3 must be similar to the order in the Density function.
For this reason, it is safer to use Parameters.\cr
As a rule of thumb, SDProp can be 1/10th of the range of posterior. For example if
the posterior is thought to be from 0.1 to 0.2, you can use SDProp being (0.2-0.1)/10=0.01.
}
\details{
setPriors1 is a general function to set priors for MHalgoGen()
}
\examples{
\dontrun{
library(HelpersMG)
Priors <- setPriors1(Name="a0", Prior1=0, Prior2=1, Density="dunif", 
                     SDProp=0.1, Min=0, Max=1, Init=0.5)
Priors <- Priors + setPriors1(Name="a1", Prior1=0.5, Prior2=0.1, Density="dnorm", 
                     SDProp=0.05, Min=0, Max=1, Init=0.5)
# Using Parameters
Priors <- setPriors1(Name="a0", Parameters=c(min=0, max=1), Density="dunif", 
                     SDProp=0.1, Min=0, Max=1, Init=0.5)
Priors <- Priors + setPriors1(Name="a1", Parameters=c(mean=0.5, sd=0.1), Density="dnorm", 
                     SDProp=0.05, Min=0, Max=1, Init=0.5)
}
}
\seealso{
Other mcmcComposite functions:
\code{\link[=+.PriorsmcmcComposite]{+.PriorsmcmcComposite()}},
\code{\link[=MHalgoGen]{MHalgoGen()}},
\code{\link[=PPD]{PPD()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=as.parameters]{as.parameters()}},
\code{\link[=merge.mcmcComposite]{merge.mcmcComposite()}},
\code{\link[=plot.PriorsmcmcComposite]{plot.PriorsmcmcComposite()}},
\code{\link[=plot.mcmcComposite]{plot.mcmcComposite()}},
\code{\link[=setPriors]{setPriors()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.tz.R
\name{convert.tz}
\alias{convert.tz}
\title{Convert one Date-Time from one timezone to another}
\usage{
convert.tz(x, tz = Sys.timezone())
}
\arguments{
\item{x}{The date-time in POSIXlt or POSIXct format}

\item{tz}{The timezone}
}
\value{
A POSIXlt or POSIXct date converted
}
\description{
Convert one Date-Time from one timezone to another.\cr
Available timezones can be shown using OlsonNames().
}
\details{
convert.tz Convert one Date-Time from one timezone to another
}
\examples{
d <- as.POSIXlt("2010-01-01 17:34:20", tz="UTC")
convert.tz(d, tz="America/Guatemala")
}
\seealso{
Function \code{with_tz()} from \code{lubridate} package does the same. I keep it here only for compatibility with old scripts.
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setPriors.R
\name{setPriors}
\alias{setPriors}
\title{Set priors for MHalgoGen()}
\usage{
setPriors(
  par = stop("A vector with init values is necessary."),
  se = NULL,
  density = "dunif",
  rules = NULL,
  silent = FALSE
)
}
\arguments{
\item{par}{Named vector with init value of parameters}

\item{se}{Named vector with standard error of parameters}

\item{density}{Named vector with density or single value}

\item{rules}{List of rules to define priors}

\item{silent}{If TRUE, do not show warning.}
}
\value{
Return a data.frame with priors
}
\description{
Set priors for MHalgoGen()\cr
}
\details{
setPriors is a general function to set priors for MHalgoGen()
}
\examples{
\dontrun{
library(HelpersMG)
rules <- rbind(data.frame(Name="^a", Min=0, Max="x*2"), 
              data.frame(Name="^b", Min=0, Max=100))
par <- c(a0=10, a1=2, b2=20)
(p <- setPriors(par=par, se=NULL, density="dgamma", rules=rules))
(p <- setPriors(par=par, se=NULL, density="dnorm", rules=rules))
(p <- setPriors(par=par, se=NULL, density="dunif", rules=rules))
par <- c(a0=10, a1=2, b2=20, b1=-1)
(p <- setPriors(par=par, se=NULL, density="dgamma", rules=rules))
}
}
\seealso{
Other mcmcComposite functions:
\code{\link[=+.PriorsmcmcComposite]{+.PriorsmcmcComposite()}},
\code{\link[=MHalgoGen]{MHalgoGen()}},
\code{\link[=PPD]{PPD()}},
\code{\link[=as.mcmc.mcmcComposite]{as.mcmc.mcmcComposite()}},
\code{\link[=as.parameters]{as.parameters()}},
\code{\link[=merge.mcmcComposite]{merge.mcmcComposite()}},
\code{\link[=plot.PriorsmcmcComposite]{plot.PriorsmcmcComposite()}},
\code{\link[=plot.mcmcComposite]{plot.mcmcComposite()}},
\code{\link[=setPriors1]{setPriors1()}},
\code{\link[=summary.mcmcComposite]{summary.mcmcComposite()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{mcmcComposite functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ScalePreviousPlot.R
\name{ScalePreviousPlot}
\alias{ScalePreviousPlot}
\title{Return the scale of the previous plot}
\usage{
ScalePreviousPlot(x = NULL, y = NULL)
}
\arguments{
\item{x}{The position in x as relative position}

\item{y}{The position in y as relative position}
}
\value{
A list with xlim and ylim
}
\description{
Return a list with the limits of the previous plot, the center, the range, and the position of label on this axe.
}
\details{
ScalePreviousPlot returns the scale of the previous plot
}
\examples{
\dontrun{
par(xaxs="i", yaxs="i")
plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="x", ylab="y")
xlim= ScalePreviousPlot()$xlim[1:2]
ylim= ScalePreviousPlot()$ylim[1:2]
par(xaxs="r", yaxs="i")
plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="x", ylab="y")
xlim= ScalePreviousPlot()$xlim[1:2]
ylim= ScalePreviousPlot()$ylim[1:2]
# Here is an example of the use of the label output
plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="", ylab="")
text(x=ScalePreviousPlot()$xlim["label"], y=ScalePreviousPlot()$ylim["center"], 
  xpd=TRUE, "Legend for Y axes", pos=3, srt=90)
text(x=ScalePreviousPlot()$xlim["center"], y=ScalePreviousPlot()$ylim["label"], 
  xpd=TRUE, "Legend for X axes", pos=1)
Example to plot legend always in the same place
layout(1:2)
plot(x=1:100, y=sin(1:100), type="l", bty="n", xlim=c(1,200), xlab="", ylab="")
text(x=ScalePreviousPlot(x=0.95, y=0.05)$x, 
     y=ScalePreviousPlot(x=0.95, y=0.05)$y, 
     labels="A", cex=2)
plot(x=0:1, y=0:1, type="p", bty="n")
text(x=ScalePreviousPlot(x=0.95, y=0.05)$x, 
     y=ScalePreviousPlot(x=0.95, y=0.05)$y, 
     labels="B", cex=2)
}
}
\seealso{
Other plot and barplot functions:
\code{\link[=barplot_errbar]{barplot_errbar()}},
\code{\link[=plot_add]{plot_add()}},
\code{\link[=plot_errbar]{plot_errbar()}},
\code{\link[=show_name]{show_name()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{plot and barplot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LD50.R
\name{LD50}
\alias{LD50}
\title{Estimate the parameters that best describe LD50}
\usage{
LD50(
  df = NULL,
  alive = NULL,
  dead = NULL,
  N = NULL,
  doses = NULL,
  l = 0.05,
  parameters.initial = NULL,
  fixed.parameters = NULL,
  SE = NULL,
  equation = "logistic",
  replicates = 1000,
  range.CI = 0.95,
  limit.low.TRD.minimum = 5,
  limit.high.TRD.maximum = 1000,
  print = TRUE,
  doses.plot = seq(from = 0, to = 1000, by = 0.1)
)
}
\arguments{
\item{df}{A dataframe with at least two columns named alive, dead or N and doses columns}

\item{alive}{A vector with alive individuals at the end of experiment}

\item{dead}{A vector with dead individuals at the end of experiment}

\item{N}{A vector with total numbers of tested individuals}

\item{doses}{The doses}

\item{l}{The limit to define TRD (see Girondot, 1999)}

\item{parameters.initial}{Initial values for P, S or K search as a vector, ex. c(P=29, S=-0.3)}

\item{fixed.parameters}{Parameters that will not be changed during fit}

\item{SE}{Standard errors for parameters}

\item{equation}{Could be "logistic", "logit", "probit", Hill", "Richards", "Hulin", "flexit" or "Double-Richards"}

\item{replicates}{Number of replicates to estimate confidence intervals}

\item{range.CI}{The range of confidence interval for estimation, default=0.95}

\item{limit.low.TRD.minimum}{Minimum lower limit for TRD}

\item{limit.high.TRD.maximum}{Maximum higher limit for TRD}

\item{print}{Do the results must be printed at screen? TRUE (default) or FALSE}

\item{doses.plot}{Sequences of doses that will be used for plotting. If NULL, does not estimate them}
}
\value{
A list with the LD50, Transitional Range of Doses and their SE
}
\description{
Estimate the parameters that best describe LD50\cr
Logistic and logit models are the same but with different parametrization:\cr
logistic = 1/(1+exp((1/S)\emph{(P-d)))\cr
logit = 1/(1+exp(P+d}S))\cr
See these publications for the description of equations:\cr
Girondot, M. 1999. Statistical description of temperature-dependent sex determination using maximum likelihood. Evolutionary Ecology Research, 1, 479-486.\cr
Godfrey, M.H., Delmas, V., Girondot, M., 2003. Assessment of patterns of temperature-dependent sex determination using maximum likelihood model selection. Ecoscience 10, 265-272.\cr
Hulin, V., Delmas, V., Girondot, M., Godfrey, M.H., Guillon, J.-M., 2009. Temperature-dependent sex determination and global change: are some species at greater risk? Oecologia 160, 493-506.\cr
The flexit equation is not still published :
\deqn{if dose < P then (1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}{if dose < P then (1 + (2^K1 - 1) *  exp(4 * S1 * (P - x)))^(-1/K1)}
\deqn{if dose > P then 1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}{if dose > P then 1-((1 + (2^K2 - 1) * exp(4 * S2 * (x - P)))^(-1/K2)}
with:\cr
\deqn{S1 = S/((4/K1)*(2^(-K1))^(1/K1+1)*(2^K1-1))}{S1 = S/((4/K1)*(2^(-K1))^(1/K1+1)*(2^K1-1))}
\deqn{S2 = S/((4/K2)*(2^(-K2))^(1/K2+1)*(2^K2-1))}{S2 = S/((4/K2)*(2^(-K2))^(1/K2+1)*(2^K2-1))}
}
\details{
LD50 estimates the parameters that best describe LD50
}
\examples{
\dontrun{
library("HelpersMG")
data <- data.frame(Doses=c(80, 120, 150, 150, 180, 200),
Alive=c(10, 12, 8, 6, 2, 1),
Dead=c(0, 1, 5, 6, 9, 15))
LD50_logistic <- LD50(data, equation="logistic")
predict(LD50_logistic, doses=c(140, 170))
plot(LD50_logistic, xlim=c(0, 300), at=seq(from=0, to=300, by=50))
LD50_probit <- LD50(data, equation="probit")
predict(LD50_probit, doses=c(140, 170))
plot(LD50_probit)
LD50_logit <- LD50(data, equation="logit")
predict(LD50_logit, doses=c(140, 170))
plot(LD50_logit)
LD50_hill <- LD50(data, equation="hill")
predict(LD50_hill, doses=c(140, 170))
plot(LD50_hill)
LD50_Richards <- LD50(data, equation="Richards")
predict(LD50_Richards, doses=c(140, 170))
plot(LD50_Richards)
LD50_Hulin <- LD50(data, equation="Hulin")
predict(LD50_Hulin, doses=c(140, 170))
plot(LD50_Hulin)
LD50_DoubleRichards <- LD50(data, equation="Double-Richards")
predict(LD50_DoubleRichards, doses=c(140, 170))
plot(LD50_DoubleRichards)
LD50_flexit <- LD50(data, equation="flexit")
predict(LD50_flexit, doses=c(140, 170))
plot(LD50_flexit)
}
}
\seealso{
Other LD50 functions:
\code{\link[=LD50_MHmcmc]{LD50_MHmcmc()}},
\code{\link[=LD50_MHmcmc_p]{LD50_MHmcmc_p()}},
\code{\link[=logLik.LD50]{logLik.LD50()}},
\code{\link[=plot.LD50]{plot.LD50()}},
\code{\link[=predict.LD50]{predict.LD50()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{LD50 functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RectangleRegression.R
\name{RectangleRegression}
\alias{RectangleRegression}
\title{Return parameters of rectangle regression}
\usage{
RectangleRegression(
  x1,
  x2,
  replicate = 1000,
  x1new = seq(from = min(x1), to = max(x1), length.out = 100)
)
}
\arguments{
\item{x1}{The first series of data}

\item{x2}{The second series of data}

\item{replicate}{Number of replicates for bootstrap}

\item{x1new}{Values for x1 to generate x2}
}
\value{
A list with parameters of rectangle regression
}
\description{
Fit a line using least rectangle method.
}
\details{
RectangleRegression performs rectangle regression
}
\examples{
x1 <- runif(100, min=10, max=20)
x2 <- runif(100, min=10, max=20)+x1

rectreg <- RectangleRegression(x1, x2)

plot(x=x1, y=x2, bty="n", las=1, xlim=c(10, 20), ylim=c(20, 40))
abline(a=rectreg$par["Intercept"], b=rectreg$par["Slope"], lwd=2)
par(xpd=FALSE)
lines(rectreg$x2new["x1new", ], rectreg$x2new["50\%", ])
lines(rectreg$x2new["x1new", ], rectreg$x2new["2.5\%", ], lty=2)
lines(rectreg$x2new["x1new", ], rectreg$x2new["97.5\%", ], lty=2)

abline(a=rectreg$Intercept[1], b=rectreg$Slope[3], col="red")
abline(a=rectreg$Intercept[3], b=rectreg$Slope[1], col="red")

}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Rectangle Regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.compareAIC.R
\name{logLik.compareAIC}
\alias{logLik.compareAIC}
\title{Return Log Likelihood generated by FormatCompareAIC}
\usage{
\method{logLik}{compareAIC}(object, ...)
}
\arguments{
\item{object}{A result generated by FormatCompareAIC}

\item{...}{Not used}
}
\value{
The Log Likelihood value for the fitted model with data
}
\description{
Return Log Likelihood generated by FormatCompareAIC
}
\details{
logLik.compareAIC Return Log Likelihood of a fit
}
\examples{
\dontrun{
ED <- FormatCompareAIC(logLik=-140, nobs=100, df=3)
logLik(ED)
}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{AIC functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IC_clean_data.R
\name{IC_clean_data}
\alias{IC_clean_data}
\title{Clean the dataframe before to be used with IC_threshold_matrix}
\usage{
IC_clean_data(
  data = stop("A dataframe object is required"),
  use = c("pairwise.complete.obs", "everything", "all.obs", "complete.obs",
    "na.or.complete"),
  method = c("pearson", "kendall", "spearman"),
  variable.retain = NULL,
  test.partial.correlation = TRUE,
  progress = TRUE,
  debug = FALSE
)
}
\arguments{
\item{data}{The data.frame to be cleaned}

\item{use}{an optional character string giving a method for computing covariances in the presence of missing values. This must be (an abbreviation of) one of the strings "everything", "all.obs", "complete.obs", "na.or.complete", or "pairwise.complete.obs".}

\item{method}{a character string indicating which correlation coefficient (or covariance) is to be computed. One of "pearson" (default), "kendall", or "spearman": can be abbreviated.}

\item{variable.retain}{a vector with the name of columns to keep}

\item{test.partial.correlation}{should the partial correlations be tested ?}

\item{progress}{Show a progress bar}

\item{debug}{if TRUE, information about progression of cleaning are shown}
}
\value{
A dataframe
}
\description{
This function must be used if missing values are present in the dataset.\cr
It ensures that all correlations and partial correlations can be calculated.
The columns of the dataframe are removed one per one until all can be calculated without error.
It is possible to say that one or more columns must be retained because they are of particular importance in the analysis.
The use and method parameters are used by cor() function. The function uses by default a parallel computing in Unix or MacOSX systems.
If progress is TRUE and the package pbmcapply is present, a progress bar is displayed. If debug is TRUE, some informations are shown during the process.
\code{https://fr.wikipedia.org/wiki/Iconographie_des_corrélations}
}
\details{
IC_clean_data checks and corrects the dataframe to be used with IC_threshold_matrix
}
\examples{
\dontrun{
library("HelpersMG")
# based on https://fr.wikipedia.org/wiki/Iconographie_des_corrélations
es <- structure(list(Student = c("e1", "e2", "e3", "e4", "e5", "e6", "e7", "e8"), 
                     Mass = c(52, 59, 55, 58, 66, 62, 63, 69), 
                     Age = c(12, 12.5, 13, 14.5, 15.5, 16, 17, 18), 
                     Assiduity = c(12, 9, 15, 5, 11, 15, 12, 9), 
                     Note = c(5, 5, 9, 5, 13.5, 18, 18, 18)), 
                     row.names = c(NA, -8L), class = "data.frame")
es

df_clean <- IC_clean_data(es, debug = TRUE)
cor_matrix <- IC_threshold_matrix(data=df_clean, threshold = NULL, progress=FALSE)
cor_threshold <- IC_threshold_matrix(data=df_clean, threshold = 0.3)
plot(cor_threshold, show.legend.strength=FALSE, show.legend.direction = FALSE)
cor_threshold_Note <- IC_correlation_simplify(matrix=cor_threshold, variable="Note")
plot(cor_threshold_Note, show.legend.strength=FALSE, show.legend.direction = FALSE)

cor_threshold <- IC_threshold_matrix(data=df_clean, threshold = 0.6)
plot(cor_threshold, 
layout=matrix(data=c(53, 53, 55, 55, 
                     55, 53, 55, 53), ncol=2, byrow=FALSE), 
show.legend.direction = FALSE,
show.legend.strength = FALSE, xlim=c(-2, 2), ylim=c(-2, 2))
}
}
\references{
Lesty, M., 1999. Une nouvelle approche dans le choix des régresseurs de la régression multiple en présence d’interactions et de colinéarités. Revue de Modulad 22, 41-77.
}
\seealso{
Other Iconography of correlations:
\code{\link[=IC_correlation_simplify]{IC_correlation_simplify()}},
\code{\link[=IC_threshold_matrix]{IC_threshold_matrix()}},
\code{\link[=plot.IconoCorel]{plot.IconoCorel()}}
}
\author{
Marc Girondot \email{marc.girondot@gmail.com}
}
\concept{Iconography of correlations}
